/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibvorbisTYPES32_H_
#define __wrappedlibvorbisTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void (*vFp_t)(void*);
typedef int32_t (*iFp_t)(void*);
typedef void (*vFpp_t)(void*, void*);
typedef int32_t (*iFpi_t)(void*, int32_t);
typedef int32_t (*iFpp_t)(void*, void*);
typedef intptr_t (*lFpp_t)(void*, void*);
typedef void* (*pFpi_t)(void*, int32_t);
typedef void (*vFppp_t)(void*, void*, void*);
typedef int32_t (*iFppp_t)(void*, void*, void*);
typedef void* (*pFppi_t)(void*, void*, int32_t);
typedef int32_t (*iFppppp_t)(void*, void*, void*, void*, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(vorbis_comment_clear, vFp_t) \
	GO(vorbis_comment_init, vFp_t) \
	GO(vorbis_dsp_clear, vFp_t) \
	GO(vorbis_info_clear, vFp_t) \
	GO(vorbis_info_init, vFp_t) \
	GO(vorbis_bitrate_addblock, iFp_t) \
	GO(vorbis_block_clear, iFp_t) \
	GO(vorbis_synthesis_halfrate_p, iFp_t) \
	GO(vorbis_synthesis_idheader, iFp_t) \
	GO(vorbis_synthesis_restart, iFp_t) \
	GO(vorbis_comment_add, vFpp_t) \
	GO(vorbis_analysis_wrote, iFpi_t) \
	GO(vorbis_info_blocksize, iFpi_t) \
	GO(vorbis_synthesis_halfrate, iFpi_t) \
	GO(vorbis_synthesis_read, iFpi_t) \
	GO(vorbis_analysis, iFpp_t) \
	GO(vorbis_analysis_blockout, iFpp_t) \
	GO(vorbis_analysis_init, iFpp_t) \
	GO(vorbis_bitrate_flushpacket, iFpp_t) \
	GO(vorbis_block_init, iFpp_t) \
	GO(vorbis_comment_query_count, iFpp_t) \
	GO(vorbis_commentheader_out, iFpp_t) \
	GO(vorbis_synthesis, iFpp_t) \
	GO(vorbis_synthesis_blockin, iFpp_t) \
	GO(vorbis_synthesis_init, iFpp_t) \
	GO(vorbis_synthesis_lapout, iFpp_t) \
	GO(vorbis_synthesis_pcmout, iFpp_t) \
	GO(vorbis_synthesis_trackonly, iFpp_t) \
	GO(vorbis_packet_blocksize, lFpp_t) \
	GO(vorbis_analysis_buffer, pFpi_t) \
	GO(vorbis_window, pFpi_t) \
	GO(vorbis_comment_add_tag, vFppp_t) \
	GO(vorbis_synthesis_headerin, iFppp_t) \
	GO(vorbis_comment_query, pFppi_t) \
	GO(vorbis_analysis_headerout, iFppppp_t)

#endif // __wrappedlibvorbisTYPES32_H_
