#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(gss_accept_sec_context, 
//GO(gss_acquire_cred, 
//GO(gss_acquire_cred_with_password, 
//GO(gss_add_buffer_set_member, 
//GO(gss_add_cred, 
//GO(gss_add_cred_with_password, 
//GO(gss_add_oid_set_member, 
//GO(_gssapi_wrap_size_cfx, 
//GO(gss_authorize_localname, 
//GO(gss_canonicalize_name, 
//DATA(__gss_c_attr_local_login_user, 
//DATA(__gss_c_attr_stream_sizes_oid_desc, 
//DATA(__gss_c_cred_certificate_oid_desc, 
//DATA(__gss_c_cred_password_oid_desc, 
//DATA(__gss_c_ma_auth_init_anon_oid_desc, 
//DATA(__gss_c_ma_auth_init_init_oid_desc, 
//DATA(__gss_c_ma_auth_init_oid_desc, 
//DATA(__gss_c_ma_auth_targ_anon_oid_desc, 
//DATA(__gss_c_ma_auth_targ_init_oid_desc, 
//DATA(__gss_c_ma_auth_targ_oid_desc, 
//DATA(__gss_c_ma_cbindings_oid_desc, 
//DATA(__gss_c_ma_compress_oid_desc, 
//DATA(__gss_c_ma_conf_prot_oid_desc, 
//DATA(__gss_c_ma_ctx_trans_oid_desc, 
//DATA(__gss_c_ma_deleg_cred_oid_desc, 
//DATA(__gss_c_ma_deprecated_oid_desc, 
//DATA(__gss_c_ma_integ_prot_oid_desc, 
//DATA(__gss_c_ma_itok_framed_oid_desc, 
//DATA(__gss_c_ma_mech_composite_oid_desc, 
//DATA(__gss_c_ma_mech_concrete_oid_desc, 
//DATA(__gss_c_ma_mech_description_oid_desc, 
//DATA(__gss_c_ma_mech_glue_oid_desc, 
//DATA(__gss_c_ma_mech_name_oid_desc, 
//DATA(__gss_c_ma_mech_nego_oid_desc, 
//DATA(__gss_c_ma_mech_pseudo_oid_desc, 
//DATA(__gss_c_ma_mic_oid_desc, 
//DATA(__gss_c_ma_not_dflt_mech_oid_desc, 
//DATA(__gss_c_ma_not_mech_oid_desc, 
//DATA(__gss_c_ma_oos_det_oid_desc, 
//DATA(__gss_c_ma_pfs_oid_desc, 
//DATA(__gss_c_ma_prot_ready_oid_desc, 
//DATA(__gss_c_ma_replay_det_oid_desc, 
//DATA(__gss_c_ma_sasl_mech_name_oid_desc, 
//DATA(__gss_c_ma_wrap_oid_desc, 
//DATA(__gss_c_nt_anonymous_oid_desc, 
//DATA(__gss_c_nt_dn_oid_desc, 
//DATA(__gss_c_nt_export_name_oid_desc, 
//DATA(__gss_c_nt_hostbased_service_oid_desc, 
//DATA(__gss_c_nt_hostbased_service_x_oid_desc, 
//DATA(__gss_c_ntlm_avguest_oid_desc, 
//DATA(__gss_c_ntlm_force_v1_oid_desc, 
//DATA(__gss_c_ntlm_session_key_oid_desc, 
//DATA(__gss_c_ntlm_v1_oid_desc, 
//DATA(__gss_c_ntlm_v2_oid_desc, 
//DATA(__gss_c_nt_machine_uid_name_oid_desc, 
//DATA(__gss_c_nt_ntlm_oid_desc, 
//DATA(__gss_c_nt_string_uid_name_oid_desc, 
//DATA(__gss_c_nt_user_name_oid_desc, 
//GO(gss_compare_name, 
//GO(gss_context_query_attributes, 
//GO(gss_context_time, 
//DATA(__gss_c_peer_has_updated_spnego_oid_desc, 
//GO(gss_create_empty_buffer_set, 
//GO(gss_create_empty_oid_set, 
//GO(gss_decapsulate_token, 
//GO(gss_delete_name_attribute, 
//GO(gss_delete_sec_context, 
//GO(gss_display_mech_attr, 
//GO(gss_display_name, 
//GO(gss_display_name_ext, 
//GO(gss_display_status, 
//GO(gss_duplicate_name, 
//GO(gss_duplicate_oid, 
//GO(gss_encapsulate_token, 
//GO(gss_export_cred, 
//GO(gss_export_name, 
//GO(gss_export_name_composite, 
//GO(gss_export_sec_context, 
//GO(gss_get_mic, 
//GO(gss_get_name_attribute, 
//GO(gss_import_cred, 
//GO(gss_import_name, 
//GO(gss_import_sec_context, 
//GO(gss_indicate_mechs, 
//GO(gss_indicate_mechs_by_attrs, 
//GO(gss_init_sec_context, 
//GO(gss_inquire_attrs_for_mech, 
//GO(gss_inquire_context, 
//GO(gss_inquire_cred, 
//GO(gss_inquire_cred_by_mech, 
//GO(gss_inquire_cred_by_oid, 
//GO(gss_inquire_mech_for_saslname, 
//GO(gss_inquire_mechs_for_name, 
//GO(gss_inquire_name, 
//GO(gss_inquire_names_for_mech, 
//GO(gss_inquire_saslname_for_mech, 
//GO(gss_inquire_sec_context_by_oid, 
//GO(gss_krb5_ccache_name, 
//DATA(__gss_krb5_ccache_name_x_oid_desc, 
//GO(_gsskrb5cfx_wrap_length_cfx, 
//DATA(__gss_krb5_compat_des3_mic_x_oid_desc, 
//GO(gss_krb5_copy_ccache, 
//DATA(__gss_krb5_copy_ccache_x_oid_desc, 
//DATA(__gss_krb5_cred_no_ci_flags_x_oid_desc, 
//DATA(__gss_krb5_export_lucid_context_v1_x_oid_desc, 
//DATA(__gss_krb5_export_lucid_context_x_oid_desc, 
//GO(gss_krb5_export_lucid_sec_context, 
//GO(gsskrb5_extract_authtime_from_sec_context, 
//GO(gsskrb5_extract_authz_data_from_sec_context, 
//DATA(__gss_krb5_extract_authz_data_from_sec_context_x_oid_desc, 
//GO(gsskrb5_extract_service_keyblock, 
//GO(gss_krb5_free_lucid_sec_context, 
//DATA(__gss_krb5_get_acceptor_subkey_x_oid_desc, 
//DATA(__gss_krb5_get_authtime_x_oid_desc, 
//GO(gsskrb5_get_initiator_subkey, 
//DATA(__gss_krb5_get_initiator_subkey_x_oid_desc, 
//DATA(__gss_krb5_get_service_keyblock_x_oid_desc, 
//GO(gsskrb5_get_subkey, 
//DATA(__gss_krb5_get_subkey_x_oid_desc, 
//GO(gsskrb5_get_time_offset, 
//DATA(__gss_krb5_get_time_offset_x_oid_desc, 
//GO(gss_krb5_get_tkt_flags, 
//DATA(__gss_krb5_get_tkt_flags_x_oid_desc, 
//GO(gss_krb5_import_cred, 
//DATA(__gss_krb5_import_cred_x_oid_desc, 
//DATA(__gss_krb5_mechanism_oid_desc, 
//DATA(__gss_krb5_nt_principal_name_oid_desc, 
//DATA(__gss_krb5_nt_principal_name_referral_oid_desc, 
//DATA(__gss_krb5_plugin_register_x_oid_desc, 
//GO(gsskrb5_register_acceptor_identity, 
//DATA(__gss_krb5_register_acceptor_identity_x_oid_desc, 
//DATA(__gss_krb5_send_to_kdc_x_oid_desc, 
//GO(gss_krb5_set_allowable_enctypes, 
//DATA(__gss_krb5_set_allowable_enctypes_x_oid_desc, 
//GO(gsskrb5_set_default_realm, 
//DATA(__gss_krb5_set_default_realm_x_oid_desc, 
//GO(gsskrb5_set_dns_canonicalize, 
//DATA(__gss_krb5_set_dns_canonicalize_x_oid_desc, 
//GO(gsskrb5_set_send_to_kdc, 
//GO(gsskrb5_set_time_offset, 
//DATA(__gss_krb5_set_time_offset_x_oid_desc, 
//GO(gss_localname, 
//GO(gss_mg_collect_error, 
//GO(gss_mo_get, 
//GO(gss_mo_list, 
//GO(gss_mo_name, 
//GO(gss_mo_set, 
//GO(gss_name_to_oid, 
//DATA(__gss_ntlm_get_session_key_x_oid_desc, 
//DATA(__gss_ntlm_mechanism_oid_desc, 
//GO(gss_oid_equal, 
//GO(gss_oid_to_name, 
//GO(gss_oid_to_str, 
//GO(gss_pname_to_uid, 
//GO(gss_process_context_token, 
//GO(gss_pseudo_random, 
//GO(gss_release_buffer, 
//GO(gss_release_buffer_set, 
//GO(gss_release_cred, 
//GO(gss_release_iov_buffer, 
//GO(gss_release_name, 
//GO(gss_release_oid, 
//GO(gss_release_oid_set, 
//DATA(__gss_sasl_digest_md5_mechanism_oid_desc, 
//GO(gss_seal, 
//GO(gss_set_cred_option, 
//GO(gss_set_name_attribute, 
//GO(gss_set_sec_context_option, 
//GO(gss_sign, 
//DATA(__gss_spnego_mechanism_oid_desc, 
//GO(gss_store_cred, 
//GO(gss_test_oid_set_member, 
//GO(gss_unseal, 
//GO(gss_unwrap, 
//GO(gss_unwrap_aead, 
//GO(gss_unwrap_iov, 
//GO(gss_userok, 
//GO(gss_verify, 
//GO(gss_verify_mic, 
//GO(gss_wrap, 
//GO(gss_wrap_aead, 
//GO(gss_wrap_iov, 
//GO(gss_wrap_iov_length, 
GO(gss_wrap_size_limit, uFppiuup)
//GO(krb5_gss_register_acceptor_identity, 
