#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

// UDate is double
// UBool is int8_t

GO(ucal_add_76, vFpiip)
GO(ucal_clear_76, vFp)
GO(ucal_clearField_76, vFpi)
GO(ucal_clone_76, pFpp)
GO(ucal_close_76, vFp)
GO(ucal_countAvailable_76, iFv)
GO(ucal_equivalentTo_76, cFpp)
GO(ucal_get_76, iFpip)
GO(ucal_getAttribute_76, iFpi)
GO(ucal_getAvailable_76, pFi)
GO(ucal_getCanonicalTimeZoneID_76, iFpipipp)
GO(ucal_getDayOfWeekType_76, iFpip)
GO(ucal_getDefaultTimeZone_76, iFpip)
GO(ucal_getDSTSavings_76, iFpp)
GO(ucal_getFieldDifference_76, iFpdip)
GO(ucal_getGregorianChange_76, dFpp)
GO(ucal_getHostTimeZone_76, iFpip)
// GO(ucal_getIanaTimeZoneID_76, iFpipip) // enable if needed, not in widespread use yet
GO(ucal_getKeywordValuesForLocale_76, pFppCp)
GO(ucal_getLimit_76, iFpiip)
GO(ucal_getLocaleByType_76, pFpip)
GO(ucal_getMillis_76, dFpp)
GO(ucal_getNow_76, dFv)
GO(ucal_getTimeZoneDisplayName_76, iFpippip)
GO(ucal_getTimeZoneID_76, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_76, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_76, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_76, cFpipp)
GO(ucal_getType_76, pFpp)
GO(ucal_getTZDataVersion_76, pFp)
GO(ucal_getWeekendTransition_76, iFpip)
GO(ucal_getWindowsTimeZoneID_76, iFpipip)
GO(ucal_inDaylightTime_76, cFpp)
GO(ucal_isSet_76, cFpi)
GO(ucal_isWeekend_76, cFpdp)
GO(ucal_open_76, pFpipip)
GO(ucal_openCountryTimeZones_76, pFpp)
GO(ucal_openTimeZoneIDEnumeration_76, pFippp)
GO(ucal_openTimeZones_76, pFp)
GO(ucal_roll_76, vFpiip)
GO(ucal_set_76, vFpii)
GO(ucal_setAttribute_76, vFpii)
GO(ucal_setDate_76, vFpiiip)
GO(ucal_setDateTime_76, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_76, vFpp)
GO(ucal_setGregorianChange_76, vFpdp)
GO(ucal_setMillis_76, vFpdp)
GO(ucal_setTimeZone_76, vFppip)

GO(ucol_clone_76, pFpp)
GO(ucol_close_76, vFp)
GO(ucol_closeElements_76, vFp)
GO(ucol_setMaxVariable_76, vFpip)
GO(ucol_getOffset_76, iFp)
GO(ucol_getRules_76, pFpp)
GO(ucol_getSortKey_76, iFppipi)
GO(ucol_getStrength_76, iFp)
GO(ucol_getVersion_76, vFpp)
GO(ucol_next_76, iFpp)
GO(ucol_previous_76, iFpp)
GO(ucol_open_76, pFpp)
GO(ucol_openElements_76, pFppip)
GO(ucol_openRules_76, pFpiiipp)
GO(ucol_safeClone_76, pFpppp)
GO(ucol_setAttribute_76, vFpiip)
GO(ucol_setVariableTop_76, uFppip)
GO(ucol_strcoll_76, iFppipi)

GO(udat_close_76, vFp)
GO(udat_countSymbols_76, iFpi)
GO(udat_getSymbols_76, iFpiipip)
GO(udat_format_76, iFpdpipp)
GO(udat_open_76, pFiippipip)
GO(udat_setCalendar_76, vFpp)
GO(udat_toPattern_76, iFpCpip)

GO(unum_close_76, vFp)
GO(unum_getAttribute_76, iFpi)
GO(unum_getSymbol_76, iFpipip)
GO(unum_open_76, pFipippp)
GO(unum_toPattern_76, iFpCpip)

GO(udatpg_close_76, vFp)
GO(udatpg_getBestPattern_76, iFppipip)
GO(udatpg_open_76, pFpp)

GO(ulocdata_getCLDRVersion_76, vFpp)
GO(ulocdata_getMeasurementSystem_76, iFpp)

GO(usearch_close_76, vFp)
GO(usearch_first_76, iFpp)
GO(usearch_getBreakIterator_76, pFp)
GO(usearch_getMatchedLength_76, iFp)
GO(usearch_last_76, iFpp)
GO(usearch_openFromCollator_76, pFpipippp)
GO(usearch_setPattern_76, vFppip)
GO(usearch_setText_76, vFppip)
