#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(docbCreateFileParserCtxt, 
//GO(docbCreatePushParserCtxt, 
//GO(__docbDefaultSAXHandler, 
GO(docbDefaultSAXHandlerInit, vFv)
//GO(docbEncodeEntities, 
//GO(docbFreeParserCtxt, 
//GO(docbParseChunk, 
//GO(docbParseDoc, 
//GO(docbParseDocument, 
//GO(docbParseFile, 
//GO(docbSAXParseDoc, 
//GO(docbSAXParseFile, 
//GO(_fini, 
GO(htmlAttrAllowed, uFppi)
GO(htmlAutoCloseTag, iFppp)
GO(htmlCreateFileParserCtxt, pFpp)
//GOM(htmlCreateMemoryParserCtxt, pFEpi)
//GOM(htmlCreatePushParserCtxt, pFEpppipi)
//GOM(htmlCtxtReadDoc, pFEppppi)
//GOM(htmlCtxtReadFd, pFEpippi)
//GOM(htmlCtxtReadFile, pFEpppi)
//GOM(htmlCtxtReadIO, pFEppppppi)
//GOM(htmlCtxtReadMemory, pFEppippi)
//GOM(htmlCtxtReset, vFEp)
//GOM(htmlCtxtUseOptions, iFEpi)
//GOM(__htmlDefaultSAXHandler, pFEv)
GO(htmlDefaultSAXHandlerInit, vFv)
GO(htmlDocContentDumpFormatOutput, vFpppi)
GO(htmlDocContentDumpOutput, vFppp)
GO(htmlDocDump, iFSp)
GO(htmlDocDumpMemory, vFppp)
GO(htmlDocDumpMemoryFormat, vFpppi)
GO(htmlElementAllowedHere, iFpp)
GO(htmlElementStatusHere, uFpp)
GO(htmlEncodeEntities, iFppppi)
GO(htmlEntityLookup, pFp)
GO(htmlEntityValueLookup, pFu)
//GOM(htmlFreeParserCtxt, vFEp)
GO(htmlGetMetaEncoding, pFp)
GO(htmlHandleOmittedElem, iFi)
GO(htmlInitAutoClose, vFv)
GO(htmlIsAutoClosed, iFpp)
GO(htmlIsBooleanAttr, iFp)
GO(htmlIsScriptAttribute, iFp)
GO(htmlNewDoc, pFpp)
GO(htmlNewDocNoDtD, pFpp)
//GOM(htmlNewParserCtxt, pFEv)
GO(htmlNodeDump, iFppp)
GO(htmlNodeDumpFile, vFSpp)
GO(htmlNodeDumpFileFormat, iFSpppi)
//GOM(htmlNodeDumpFormatOutput, vFEppppi)
//GOM(htmlNodeDumpOutput, vFEpppp)
GO(htmlNodeStatus, uFpi)
//GOM(htmlParseCharRef, iFEp)
//GOM(htmlParseChunk, iFEppii)
//GO(__htmlParseContent, 
GO(htmlParseDoc, pFpp)
//GOM(htmlParseDocument, iFEp)
//GOM(htmlParseElement, vFEp)
//GOM(htmlParseEntityRef, pFEpp)
GO(htmlParseFile, pFpp)
GO(htmlReadDoc, pFpppi)
GO(htmlReadFd, pFippi)
GO(htmlReadFile, pFppi)
//GOM(htmlReadIO, pFEpppppi)
GO(htmlReadMemory, pFpippi)
GO(htmlSaveFile, iFpp)
GO(htmlSaveFileEnc, iFppp)
GO(htmlSaveFileFormat, iFpppi)
//GOM(htmlSAXParseDoc, pFEpppp)
//GOM(htmlSAXParseFile, pFEpppp)
GO(htmlSetMetaEncoding, iFpp)
GO(htmlTagLookup, pFp)
//GO(_init, 
//GOM(initGenericErrorDefaultFunc, vFEp)
GO(inputPop, pFp)
GO(inputPush, iFpp)
GO(isolat1ToUTF8, iFpppp)
GO(namePop, pFp)
GO(namePush, iFpp)
GO(nodePop, pFp)
GO(nodePush, iFpp)
GO(__oldXMLWDcompatibility, pFv)
GO(UTF8ToHtml, iFpppp)
GO(UTF8Toisolat1, iFpppp)
GO(valuePop, pFp)
GOM(valuePush, iFEpp)
//GO(xlinkGetDefaultDetect, 
//GO(xlinkGetDefaultHandler, 
//GO(xlinkIsLink, 
//GO(xlinkSetDefaultDetect, 
//GO(xlinkSetDefaultHandler, 
GO(xmlACatalogAdd, iFpppp)
GO(xmlACatalogDump, vFpS)
GO(xmlACatalogRemove, iFpp)
GO(xmlACatalogResolve, pFppp)
GO(xmlACatalogResolvePublic, pFpp)
GO(xmlACatalogResolveSystem, pFpp)
GO(xmlACatalogResolveURI, pFpp)
GO(xmlAddAttributeDecl, pFpppppuupp)
GO(xmlAddChild, pFpp)
GO(xmlAddChildList, pFpp)
GO(xmlAddDocEntity, pFppippp)
GO(xmlAddDtdEntity, pFppippp)
GO(xmlAddElementDecl, pFpppup)
GO(xmlAddEncodingAlias, iFpp)
GO(xmlAddID, pFpppp)
GO(xmlAddNextSibling, pFpp)
GO(xmlAddNotationDecl, pFppppp)
GO(xmlAddPrevSibling, pFpp)
GO(xmlAddRef, pFpppp)
GO(xmlAddSibling, pFpp)
GO(xmlAllocOutputBuffer, pFp)
GO(xmlAllocOutputBufferInternal, pFp)
GO(xmlAllocParserInputBuffer, pFi)
GO(xmlAttrSerializeTxtContent, vFpppp)
GO(xmlAutomataCompile, pFp)
GO(xmlAutomataGetInitState, pFp)
GO(xmlAutomataIsDeterminist, iFp)
GO(xmlAutomataNewAllTrans, pFpppi)
GO(xmlAutomataNewCountedTrans, pFpppi)
GO(xmlAutomataNewCounter, iFpii)
GO(xmlAutomataNewCounterTrans, pFpppi)
GO(xmlAutomataNewCountTrans, pFppppiip)
GO(xmlAutomataNewCountTrans2, pFpppppiip)
GO(xmlAutomataNewEpsilon, pFppp)
GO(xmlAutomataNewNegTrans, pFpppppp)
GO(xmlAutomataNewOnceTrans, pFppppiip)
GO(xmlAutomataNewOnceTrans2, pFpppppiip)
GO(xmlAutomataNewState, pFp)
GO(xmlAutomataNewTransition, pFppppp)
GO(xmlAutomataNewTransition2, pFpppppp)
GO(xmlAutomataSetFinalState, iFpp)
//GO(xmlAutomataSetFlags, 
GO(xmlBufContent, pFp)
GO(xmlBufEnd, pFp)
GO(xmlBufGetNodeContent, iFpp)
GO(xmlBufNodeDump, LFpppii)
GO(xmlBufShrink, LFpL)
GO(xmlBufUse, LFp)
GO(xmlBufferAdd, iFppi)
GO(xmlBufferAddHead, iFppi)
GO(__xmlBufferAllocScheme, pFv)
GO(xmlBufferCat, iFpp)
GO(xmlBufferCCat, iFpp)
GO(xmlBufferContent, pFp)
GO(xmlBufferCreate, pFv)
GO(xmlBufferCreateSize, pFL)
GO(xmlBufferCreateStatic, pFpL)
GO(xmlBufferDump, iFSp)
GO(xmlBufferEmpty, vFp)
GO(xmlBufferFree, vFp)
GO(xmlBufferGrow, iFpu)
GO(xmlBufferLength, iFp)
GO(xmlBufferResize, iFpu)
GO(xmlBufferSetAllocationScheme, vFpu)
GO(xmlBufferShrink, iFpu)
GO(xmlBufferWriteChar, vFpp)
GO(xmlBufferWriteCHAR, vFpp)
GO(xmlBufferWriteQuotedString, vFpp)
GO(xmlBuildQName, pFpppi)
GO(xmlBuildRelativeURI, pFpp)
GO(xmlBuildURI, pFpp)
GO(xmlByteConsumed, lFp)
GO(xmlC14NDocDumpMemory, iFppipip)
GO(xmlC14NDocSave, iFppipipi)
//GOM(xmlC14NDocSaveTo, iFEppipip)
//GOM(xmlC14NExecute, iFEpppipip)
//GO(xmlCanonicPath, 
GO(xmlCatalogAdd, iFppp)
GO(xmlCatalogAddLocal, pFpp)
GO(xmlCatalogCleanup, vFv)
GO(xmlCatalogConvert, iFv)
GO(xmlCatalogDump, vFS)
GO(xmlCatalogFreeLocal, vFp)
GO(xmlCatalogGetDefaults, uFv)
GO(xmlCatalogGetPublic, pFp)
GO(xmlCatalogGetSystem, pFp)
GO(xmlCatalogIsEmpty, iFp)
GO(xmlCatalogLocalResolve, pFppp)
GO(xmlCatalogLocalResolveURI, pFpp)
GO(xmlCatalogRemove, iFp)
GO(xmlCatalogResolve, pFpp)
GO(xmlCatalogResolvePublic, pFp)
GO(xmlCatalogResolveSystem, pFp)
GO(xmlCatalogResolveURI, pFp)
GO(xmlCatalogSetDebug, iFi)
GO(xmlCatalogSetDefaultPrefer, uFu)
GO(xmlCatalogSetDefaults, vFu)
GO(xmlCharEncCloseFunc, iFp)
GO(xmlCharEncFirstLine, iFppp)
//GO(xmlCharEncFirstLineInt, 
GO(xmlCharEncInFunc, iFppp)
GO(xmlCharEncOutFunc, iFppp)
GO(xmlCharInRange, iFup)
GO(xmlCharStrdup, pFp)
GO(xmlCharStrndup, pFpi)
GO(xmlCheckFilename, iFp)
GO(xmlCheckHTTPInput, pFpp)
GO(xmlCheckLanguageID, iFp)
GO(xmlCheckUTF8, iFp)
GO(xmlCheckVersion, vFi)
GO(xmlChildElementCount, LFp)
GO(xmlCleanupCharEncodingHandlers, vFv)
GO(xmlCleanupEncodingAliases, vFv)
GO(xmlCleanupGlobals, vFv)
GO(xmlCleanupInputCallbacks, vFv)
GO(xmlCleanupMemory, vFv)
GO(xmlCleanupOutputCallbacks, vFv)
GO(xmlCleanupParser, vFv)
GO(xmlCleanupPredefinedEntities, vFv)
GO(xmlCleanupThreads, vFv)
GO(xmlClearNodeInfoSeq, vFp)
GO(xmlClearParserCtxt, vFp)
GO(xmlConvertSGMLCatalog, iFp)
GO(xmlCopyAttributeTable, pFp)
GO(xmlCopyChar, iFipi)
GO(xmlCopyCharMultiByte, iFpi)
GO(xmlCopyDoc, pFpi)
GO(xmlCopyDocElementContent, pFpp)
GO(xmlCopyDtd, pFp)
GO(xmlCopyElementContent, pFp)
GO(xmlCopyElementTable, pFp)
GO(xmlCopyEntitiesTable, pFp)
GO(xmlCopyEnumeration, pFp)
GO(xmlCopyError, iFpp)
GO(xmlCopyNamespace, pFp)
GO(xmlCopyNamespaceList, pFp)
GO(xmlCopyNode, pFpi)
GO(xmlCopyNodeList, pFp)
GO(xmlCopyNotationTable, pFp)
GO(xmlCopyProp, pFpp)
GO(xmlCopyPropList, pFpp)
GO(xmlCreateDocParserCtxt, pFp)
GO(xmlCreateEntitiesTable, pFv)
GO(xmlCreateEntityParserCtxt, pFppp)
GO(xmlCreateEnumeration, pFp)
GO(xmlCreateFileParserCtxt, pFp)
GO(xmlCreateIntSubset, pFpppp)
GOM(xmlCreateIOParserCtxt, pFEpppppi)
GO(xmlCreateMemoryParserCtxt, pFpi)
GOM(xmlCreatePushParserCtxt, pFEpppip)
//GO(xmlCreateURI, 
GO(xmlCreateURLParserCtxt, pFpi)
GO(xmlCtxtGetLastError, pFp)
GO(xmlCtxtReadDoc, pFppppi)
GO(xmlCtxtReadFd, pFpippi)
GO(xmlCtxtReadFile, pFpppi)
GOM(xmlCtxtReadIO, pFEppppppi)
GO(xmlCtxtReadMemory, pFppippi)
GO(xmlCtxtReset, vFp)
GO(xmlCtxtResetLastError, vFp)
GO(xmlCtxtResetPush, iFppipp)
GO(xmlCtxtUseOptions, iFpi)
GO(xmlCurrentChar, iFpp)
GO(xmlDecodeEntities, pFpiiCCC)
GO(__xmlDefaultBufferSize, pFv)
//GOM(__xmlDefaultSAXHandler, pFEv)
GO(xmlDefaultSAXHandlerInit, vFv)
//GOM(__xmlDefaultSAXLocator, pFEv)
GO(xmlDelEncodingAlias, iFp)
//GOM(xmlDeregisterNodeDefault, pFEp)
//GOM(__xmlDeregisterNodeDefaultValue, pFEv)
GO(xmlDetectCharEncoding, iFpi)
GO(xmlDictCleanup, vFv)
GO(xmlDictCreate, pFv)
GO(xmlDictCreateSub, pFp)
GO(xmlDictExists, pFppi)
GO(xmlDictFree, vFp)
GO(xmlDictGetUsage, LFp)
GO(xmlDictLookup, pFppi)
GO(xmlDictOwns, iFpp)
GO(xmlDictQLookup, pFppp)
GO(xmlDictReference, iFp)
GO(xmlDictSetLimit, LFpL)
GO(xmlDictSize, iFp)
GO(xmlDocCopyNode, pFppi)
GO(xmlDocCopyNodeList, pFpp)
GO(xmlDocDump, iFSp)
GO(xmlDocDumpFormatMemory, vFpppi)
GO(xmlDocDumpFormatMemoryEnc, vFppppi)
GO(xmlDocDumpMemory, vFppp)
GO(xmlDocDumpMemoryEnc, vFpppp)
GO(xmlDocFormatDump, iFSpi)
GO(xmlDocGetRootElement, pFp)
GO(xmlDocSetRootElement, pFpp)
GO(xmlDOMWrapAdoptNode, iFpppppi)
GO(xmlDOMWrapCloneNode, iFppppppii)
GO(xmlDOMWrapFreeCtxt, vFp)
GO(xmlDOMWrapNewCtxt, pFv)
GO(xmlDOMWrapReconcileNamespaces, iFppi)
GO(xmlDOMWrapRemoveNode, iFpppi)
GO(__xmlDoValidityCheckingDefaultValue, pFv)
GO(xmlDumpAttributeDecl, vFpp)
GO(xmlDumpAttributeTable, vFpp)
GO(xmlDumpElementDecl, vFpp)
GO(xmlDumpElementTable, vFpp)
GO(xmlDumpEntitiesTable, vFpp)
GO(xmlDumpEntityDecl, vFpp)
GO(xmlDumpNotationDecl, vFpp)
GO(xmlDumpNotationTable, vFpp)
GO(xmlElemDump, vFSpp)
GO(xmlEncodeEntities, pFpp)
GO(xmlEncodeEntitiesReentrant, pFpp)
GO(xmlEncodeSpecialChars, pFpp)
//GO(__xmlErrEncoding, 
GO(xmlErrMemory, vFpp)
//GO(xmlExpCtxtNbCons, 
//GO(xmlExpCtxtNbNodes, 
//GO(xmlExpDump, 
//GO(xmlExpExpDerive, 
//GO(xmlExpFree, 
//GO(xmlExpFreeCtxt, 
//GO(xmlExpGetLanguage, 
//GO(xmlExpGetStart, 
//GO(xmlExpIsNillable, 
//GO(xmlExpMaxToken, 
//GO(xmlExpNewAtom, 
//GO(xmlExpNewCtxt, 
//GO(xmlExpNewOr, 
//GO(xmlExpNewRange, 
//GO(xmlExpNewSeq, 
//GO(xmlExpParse, 
//GO(xmlExpRef, 
//GO(xmlExpStringDerive, 
//GO(xmlExpSubsume, 
GO(xmlFileClose, iFp)
GO(xmlFileMatch, iFp)
GO(xmlFileOpen, pFp)
GO(xmlFileRead, iFppi)
GO(xmlFindCharEncodingHandler, pFp)
GO(xmlFirstElementChild, pFp)
GO(xmlFreeAttributeTable, vFp)
GO(xmlFreeAutomata, vFp)
GO(xmlFreeCatalog, vFp)
GO(xmlFreeDoc, vFp)
GO(xmlFreeDocElementContent, vFpp)
GO(xmlFreeDtd, vFp)
GO(xmlFreeElementContent, vFp)
GO(xmlFreeElementTable, vFp)
GO(xmlFreeEntitiesTable, vFp)
GO(xmlFreeEnumeration, vFp)
GO(xmlFreeIDTable, vFp)
GO(xmlFreeInputStream, vFp)
GO(xmlFreeMutex, vFp)
GO(xmlFreeNode, vFp)
GO(xmlFreeNodeList, vFp)
GO(xmlFreeNotationTable, vFp)
GO(xmlFreeNs, vFp)
GO(xmlFreeNsList, vFp)
GO(xmlFreeParserCtxt, vFp)
GO(xmlFreeParserInputBuffer, vFp)
GO(xmlFreePattern, vFp)
GO(xmlFreePatternList, vFp)
GO(xmlFreeProp, vFp)
GO(xmlFreePropList, vFp)
GO(xmlFreeRefTable, vFp)
GO(xmlFreeRMutex, vFp)
GO(xmlFreeStreamCtxt, vFp)
GO(xmlFreeTextReader, vFp)
GO(xmlFreeTextWriter, vFp)
//GO(xmlFreeURI, 
GO(xmlFreeValidCtxt, vFp)
//GOM(xmlGcMemGet, iFEppppp)
//GOM(xmlGcMemSetup, iFEppppp)
GOM(__xmlGenericError, pFEv)
GO(__xmlGenericErrorContext, pFv)
//GO(xmlGenericErrorDefaultFunc, 
GO(xmlGetBufferAllocationScheme, uFv)
GO(xmlGetCharEncodingHandler, pFi)
GO(xmlGetCharEncodingName, pFi)
GO(xmlGetCompressMode, iFv)
GO(xmlGetDocCompressMode, iFp)
GO(xmlGetDocEntity, pFpp)
GO(xmlGetDtdAttrDesc, pFppp)
GO(xmlGetDtdElementDesc, pFpp)
GO(xmlGetDtdEntity, pFpp)
GO(xmlGetDtdNotationDesc, pFpp)
GO(xmlGetDtdQAttrDesc, pFpppp)
GO(xmlGetDtdQElementDesc, pFppp)
GO(xmlGetEncodingAlias, pFp)
GOM(xmlGetExternalEntityLoader, pFEv)
GO(xmlGetFeature, iFppp)
GO(xmlGetFeaturesList, iFpp)
//GOM(xmlGetGlobalState, pFEv)
GO(xmlGetID, pFpp)
GO(xmlGetIntSubset, pFp)
GO(xmlGetLastChild, pFp)
GO(xmlGetLastError, pFv)
GO(xmlGetLineNo, lFp)
GO(xmlGetNodePath, pFp)
GO(xmlGetNoNsProp, pFpp)
GO(xmlGetNsList, pFpp)
GO(xmlGetNsProp, pFppp)
GO(xmlGetParameterEntity, pFpp)
GO(xmlGetPredefinedEntity, pFp)
GO(xmlGetProp, pFpp)
GO(xmlGetRefs, pFpp)
GO(xmlGetThreadId, iFv)
GO(xmlGetUTF8Char, iFpp)
GO(__xmlGetWarningsDefaultValue, pFv)
//GO(__xmlGlobalInitMutexDestroy, 
//GO(__xmlGlobalInitMutexLock, 
//GO(__xmlGlobalInitMutexUnlock, 
GO(xmlHandleEntity, vFpp)
GO(xmlHashAddEntry, iFppp)
GO(xmlHashAddEntry2, iFpppp)
GO(xmlHashAddEntry3, iFppppp)
GOM(xmlHashCopy, pFEpp)
GO(xmlHashCreate, pFi)
GO(xmlHashCreateDict, pFip)
GOM(xmlHashFree, vFEpp)
GO(xmlHashLookup, pFpp)
GO(xmlHashLookup2, pFppp)
GO(xmlHashLookup3, pFpppp)
GO(xmlHashQLookup, pFppp)
GO(xmlHashQLookup2, pFppppp)
GO(xmlHashQLookup3, pFppppppp)
GOM(xmlHashRemoveEntry, iFEppp)
GOM(xmlHashRemoveEntry2, iFEpppp)
GOM(xmlHashRemoveEntry3, iFEppppp)
GOM(xmlHashScan, vFEppp)
GOM(xmlHashScan3, vFEpppppp)
GOM(xmlHashScanFull, vFEppp)
GOM(xmlHashScanFull3, vFEpppppp)
GO(xmlHashSize, iFp)
GOM(xmlHashUpdateEntry, iFEpppp)
GOM(xmlHashUpdateEntry2, iFEppppp)
GOM(xmlHashUpdateEntry3, iFEpppppp)
GO(xmlHasFeature, iFu)
GO(xmlHasNsProp, pFppp)
GO(xmlHasProp, pFpp)
GO(__xmlIndentTreeOutput, pFv)
GO(xmlInitCharEncodingHandlers, vFv)
GO(xmlInitializeDict, iFv)
GO(xmlInitGlobals, vFv)
GO(xmlInitializeCatalog, vFv)
GO(xmlInitializeGlobalState, vFp)
GO(xmlInitMemory, iFv)
GO(xmlInitializePredefinedEntities, vFv)
GO(xmlInitNodeInfoSeq, vFp)
GO(xmlInitParser, vFv)
GO(xmlInitParserCtxt, iFp)
GO(xmlInitThreads, vFv)
//GO(__xmlIOErr, 
GO(xmlIOFTPClose, iFp)
GO(xmlIOFTPMatch, iFp)
GO(xmlIOFTPOpen, pFp)
GO(xmlIOFTPRead, iFppi)
GO(xmlIOHTTPClose, iFp)
GO(xmlIOHTTPMatch, iFp)
GO(xmlIOHTTPOpen, pFp)
GO(xmlIOHTTPOpenW, pFpi)
GO(xmlIOHTTPRead, iFppi)
GO(xmlIOParseDTD, pFppi)
GO(xmlIsBaseChar, iFu)
GO(xmlIsBlank, iFu)
GO(xmlIsBlankNode, iFp)
GO(xmlIsChar, iFu)
GO(xmlIsCombining, iFu)
GO(xmlIsDigit, iFu)
GO(xmlIsExtender, iFu)
GO(xmlIsID, iFppp)
GO(xmlIsIdeographic, iFu)
GO(xmlIsLetter, iFi)
GO(xmlIsMainThread, iFv)
GO(xmlIsMixedElement, iFpp)
GO(xmlIsPubidChar, iFu)
GO(xmlIsRef, iFppp)
GO(xmlIsXHTML, iFpp)
GO(xmlKeepBlanksDefault, iFi)
GO(__xmlKeepBlanksDefaultValue, pFv)
GO(xmlLastElementChild, pFp)
GO(__xmlLastError, pFv)
GO(xmlLineNumbersDefault, iFi)
GO(__xmlLineNumbersDefaultValue, pFv)
GO(xmlLinkGetData, pFp)
GO(xmlListAppend, iFpp)
GO(xmlListClear, vFp)
GO(xmlListCopy, iFpp)
//GOM(xmlListCreate, pFEpp)
GO(xmlListDelete, vFp)
GO(xmlListDup, pFp)
GO(xmlListEmpty, iFp)
GO(xmlListEnd, pFp)
GO(xmlListFront, pFp)
GO(xmlListInsert, iFpp)
GO(xmlListMerge, vFpp)
GO(xmlListPopBack, vFp)
GO(xmlListPopFront, vFp)
GO(xmlListPushBack, iFpp)
GO(xmlListPushFront, iFpp)
GO(xmlListRemoveAll, iFpp)
GO(xmlListRemoveFirst, iFpp)
GO(xmlListRemoveLast, iFpp)
GO(xmlListReverse, vFp)
GO(xmlListReverseSearch, pFpp)
//GOM(xmlListReverseWalk, vFEppp)
GO(xmlListSearch, pFpp)
GO(xmlListSize, iFp)
GO(xmlListSort, vFp)
//GOM(xmlListWalk, vFEppp)
GO(xmlLoadACatalog, pFp)
GO(xmlLoadCatalog, iFp)
GO(xmlLoadCatalogs, vFp)
//GO(__xmlLoaderErr, 
GO(__xmlLoadExtDtdDefaultValue, pFv)
GO(xmlLoadExternalEntity, pFppp)
GO(xmlLoadSGMLSuperCatalog, pFp)
GO(xmlLockLibrary, vFv)
GO(xmlMallocAtomicLoc, pFLpi)
//GO(xmlMallocBreakpoint, 
GO(xmlMallocLoc, pFLpi)
GO(xmlMemBlocks, iFv)
GO(xmlMemDisplay, vFS)
GO(xmlMemDisplayLast, vFSl)
GO(xmlMemFree, vFp)
GO(xmlMemGet, iFpppp)
GO(xmlMemMalloc, pFL)
GO(xmlMemoryDump, vFv)
GO(xmlMemoryStrdup, pFp)
GO(xmlMemRealloc, pFpL)
GOM(xmlMemSetup, iFEpppp)
GO(xmlMemShow, vFSi)
GO(xmlMemStrdupLoc, pFppi)
GO(xmlMemUsed, iFv)
//GO(xmlModuleClose, 
//GO(xmlModuleFree, 
//GO(xmlModuleOpen, 
//GO(xmlModuleSymbol, 
GO(xmlMutexLock, vFp)
GO(xmlMutexUnlock, vFp)
GO(xmlNamespaceParseNCName, pFp)
GO(xmlNamespaceParseNSDef, pFp)
GO(xmlNamespaceParseQName, pFpp)
//GO(xmlNanoFTPCheckResponse, 
//GO(xmlNanoFTPCleanup, 
//GO(xmlNanoFTPClose, 
//GO(xmlNanoFTPCloseConnection, 
//GO(xmlNanoFTPConnect, 
//GO(xmlNanoFTPConnectTo, 
//GO(xmlNanoFTPCwd, 
//GO(xmlNanoFTPDele, 
//GO(xmlNanoFTPFreeCtxt, 
//GO(xmlNanoFTPGet, 
//GO(xmlNanoFTPGetConnection, 
//GO(xmlNanoFTPGetResponse, 
//GO(xmlNanoFTPGetSocket, 
//GO(xmlNanoFTPInit, 
//GO(xmlNanoFTPList, 
//GO(xmlNanoFTPNewCtxt, 
//GO(xmlNanoFTPOpen, 
//GO(xmlNanoFTPProxy, 
//GO(xmlNanoFTPQuit, 
//GO(xmlNanoFTPRead, 
//GO(xmlNanoFTPScanProxy, 
//GO(xmlNanoFTPUpdateURL, 
GO(xmlNanoHTTPAuthHeader, pFp)
GO(xmlNanoHTTPCleanup, vFv)
GO(xmlNanoHTTPClose, vFp)
GO(xmlNanoHTTPContentLength, iFp)
GO(xmlNanoHTTPEncoding, pFp)
GO(xmlNanoHTTPFetch, iFppp)
GO(xmlNanoHTTPInit, vFv)
GO(xmlNanoHTTPMethod, pFpppppi)
GO(xmlNanoHTTPMethodRedir, pFppppppi)
GO(xmlNanoHTTPMimeType, pFp)
GO(xmlNanoHTTPOpen, pFpp)
GO(xmlNanoHTTPOpenRedir, pFppp)
GO(xmlNanoHTTPRead, iFppi)
GO(xmlNanoHTTPRedir, pFp)
GO(xmlNanoHTTPReturnCode, iFp)
GO(xmlNanoHTTPSave, iFpp)
GO(xmlNanoHTTPScanProxy, vFp)
GO(xmlNewAutomata, pFv)
GO(xmlNewCatalog, pFi)
GO(xmlNewCDataBlock, pFppi)
GOM(xmlNewCharEncodingHandler, pFEppp)
GO(xmlNewCharRef, pFpp)
GO(xmlNewChild, pFpppp)
GO(xmlNewComment, pFp)
GO(xmlNewDoc, pFp)
GO(xmlNewDocComment, pFpp)
GO(xmlNewDocElementContent, pFppu)
GO(xmlNewDocFragment, pFp)
GO(xmlNewDocNode, pFpppp)
GO(xmlNewDocNodeEatName, pFpppp)
GO(xmlNewDocPI, pFppp)
GO(xmlNewDocProp, pFppp)
GO(xmlNewDocRawNode, pFpppp)
GO(xmlNewDocText, pFpp)
GO(xmlNewDocTextLen, pFppi)
GO(xmlNewDtd, pFpppp)
GO(xmlNewElementContent, pFpu)
GO(xmlNewEntity, pFppippp)
GO(xmlNewEntityInputStream, pFpp)
GO(xmlNewGlobalNs, pFppp)
GO(xmlNewInputFromFile, pFpp)
GO(xmlNewInputStream, pFp)
GO(xmlNewIOInputStream, pFppi)
GO(xmlNewMutex, pFv)
GO(xmlNewNode, pFpp)
GO(xmlNewNodeEatName, pFpp)
GO(xmlNewNs, pFppp)
GO(xmlNewNsProp, pFpppp)
GO(xmlNewNsPropEatName, pFpppp)
GO(xmlNewParserCtxt, pFv)
GO(xmlNewPI, pFpp)
GO(xmlNewProp, pFppp)
GO(xmlNewReference, pFpp)
GO(xmlNewRMutex, pFv)
GO(xmlNewStringInputStream, pFpp)
GO(xmlNewText, pFp)
GO(xmlNewTextChild, pFpppp)
GO(xmlNewTextLen, pFpi)
//GOM(xmlNewTextReader, pFEpp)
GO(xmlNewTextReaderFilename, pFp)
//GOM(xmlNewTextWriter, pFEp)
GO(xmlNewTextWriterDoc, pFpi)
GO(xmlNewTextWriterFilename, pFpi)
GO(xmlNewTextWriterMemory, pFpi)
//GOM(xmlNewTextWriterPushParser, pFEpi)
GO(xmlNewTextWriterTree, pFppi)
GO(xmlNewValidCtxt, pFv)
GO(xmlNextChar, vFp)
GO(xmlNextElementSibling, pFp)
GO(xmlNodeAddContent, vFpp)
GO(xmlNodeAddContentLen, vFppi)
GO(xmlNodeBufGetContent, iFpp)
GO(xmlNodeDump, iFpppii)
GO(xmlNodeDumpOutput, vFpppiip)
GO(xmlNodeGetBase, pFpp)
GO(xmlNodeGetContent, pFp)
GO(xmlNodeGetLang, pFp)
GO(xmlNodeGetSpacePreserve, iFp)
GO(xmlNodeIsText, iFp)
GO(xmlNodeListGetRawString, pFppi)
GO(xmlNodeListGetString, pFppi)
GO(xmlNodeSetBase, vFpp)
GO(xmlNodeSetContent, vFpp)
GO(xmlNodeSetContentLen, vFppi)
GO(xmlNodeSetLang, vFpp)
GO(xmlNodeSetName, vFpp)
GO(xmlNodeSetSpacePreserve, vFpi)
GO(xmlNoNetExternalEntityLoader, pFppp)
//GO(xmlNormalizeURIPath, 
GO(xmlNormalizeWindowsPath, pFp)
//GO(xmlNsListDumpOutput, 
GO(xmlOutputBufferClose, iFp)
GO(xmlOutputBufferCreateBuffer, pFpp)
GO(xmlOutputBufferCreateFd, pFip)
GO(xmlOutputBufferCreateFile, pFSp)
GO(xmlOutputBufferCreateFilename, pFppi)
//GO(__xmlOutputBufferCreateFilename, 
//GOM(xmlOutputBufferCreateFilenameDefault, BFEB)
//GO(__xmlOutputBufferCreateFilenameValue, 
GOM(xmlOutputBufferCreateIO, pFEpppp)
GO(xmlOutputBufferFlush, iFp)
GO(xmlOutputBufferGetContent, pFp)
GO(xmlOutputBufferWrite, iFpip)
GO(xmlOutputBufferWriteEscape, iFppp)
GO(xmlOutputBufferWriteString, iFpp)
GO(xmlParseAttribute, pFpp)
GO(xmlParseAttributeListDecl, vFp)
GO(xmlParseAttributeType, iFpp)
GO(xmlParseAttValue, pFp)
GO(xmlParseBalancedChunkMemory, iFpppipp)
GO(xmlParseBalancedChunkMemoryRecover, iFpppippi)
GO(xmlParseCatalogFile, pFp)
GO(xmlParseCDSect, vFp)
GO(xmlParseCharData, vFpi)
GO(xmlParseCharEncoding, iFp)
GO(xmlParseCharRef, iFp)
GO(xmlParseChunk, iFppii)
GO(xmlParseComment, vFp)
GO(xmlParseContent, vFp)
GO(xmlParseCtxtExternalEntity, iFpppp)
GO(xmlParseDefaultDecl, iFpp)
GO(xmlParseDoc, pFp)
GO(xmlParseDocTypeDecl, vFp)
GOM(xmlParseDocument, iFEp)
GO(xmlParseDTD, pFpp)
GO(xmlParseElement, vFp)
GO(xmlParseElementChildrenContentDecl, pFpi)
GO(xmlParseElementContentDecl, iFppp)
GO(xmlParseElementDecl, iFp)
GO(xmlParseElementMixedContentDecl, pFpi)
GO(xmlParseEncName, pFp)
GO(xmlParseEncodingDecl, pFp)
GO(xmlParseEndTag, vFp)
GO(xmlParseEntity, pFp)
GO(xmlParseEntityDecl, vFp)
GO(xmlParseEntityRef, pFp)
GO(xmlParseEntityValue, pFpp)
GO(xmlParseEnumeratedType, iFp)
GO(xmlParseEnumerationType, pFp)
GO(xmlParseExternalEntity, iFpppippp)
GO(xmlParseExternalID, pFppi)
GO(xmlParseExternalSubset, vFppp)
//GOM(xmlParseExtParsedEnt, iFEp)
GO(xmlParseFile, pFp)
GO(xmlParseInNodeContext, uFppiip)
GO(xmlParseMarkupDecl, vFp)
GO(xmlParseMemory, pFpi)
GO(xmlParseMisc, vFp)
GO(xmlParseName, pFp)
GO(xmlParseNmtoken, pFp)
GO(xmlParseNotationDecl, vFp)
GO(xmlParseNotationType, pFp)
GO(xmlParsePEReference, vFp)
GO(xmlParsePI, vFp)
GO(xmlParsePITarget, pFp)
GO(xmlParsePubidLiteral, pFp)
GO(xmlParserAddNodeInfo, vFpp)
GO(__xmlParserDebugEntities, pFv)
GO(xmlParseQuotedString, pFp)
GO(xmlParseReference, vFp)
//GOM(xmlParserError, vFEppV)
GO(xmlParserFindNodeInfo, pFpp)
GO(xmlParserFindNodeInfoIndex, LFpp)
GO(xmlParserGetDirectory, pFp)
GO(xmlParserHandlePEReference, vFp)
GO(xmlParserHandleReference, vFp)
GO(xmlParserInputBufferCreateFd, pFii)
GO(xmlParserInputBufferCreateFile, pFSi)
GO(xmlParserInputBufferCreateFilename, pFpi)
//GOM(__xmlParserInputBufferCreateFilename, pFEpi)
//GOM(xmlParserInputBufferCreateFilenameDefault, pFEp)
//GOM(__xmlParserInputBufferCreateFilenameValue, pFEv)
GOM(xmlParserInputBufferCreateIO, pFEpppi)
GO(xmlParserInputBufferCreateMem, pFpii)
GO(xmlParserInputBufferCreateStatic, pFpii)
GO(xmlParserInputBufferGrow, iFpi)
GO(xmlParserInputBufferPush, iFpip)
GO(xmlParserInputBufferRead, iFpi)
GO(xmlParserInputGrow, iFpi)
GO(xmlParserInputRead, iFpi)
GO(xmlParserInputShrink, vFp)
//GOM(xmlParserPrintFileContext, vFEp)
//GOM(xmlParserPrintFileInfo, vFEp)
//GOM(xmlParserValidityError, vFEppV)
//GOM(xmlParserValidityWarning, vFEppV)
GO(__xmlParserVersion, pFv)
//GOM(xmlParserWarning, vFEppV)
GO(xmlParseSDDecl, iFp)
GO(xmlParseStartTag, pFp)
GO(xmlParseSystemLiteral, pFp)
GO(xmlParseTextDecl, vFp)
//GO(xmlParseURI, 
//GO(xmlParseURIRaw, 
//GO(xmlParseURIReference, 
GO(xmlParseVersionInfo, pFp)
GO(xmlParseVersionNum, pFp)
GO(xmlParseXMLDecl, vFp)
//GO(xmlPathToURI, 
GO(xmlPatterncompile, pFppip)
GO(xmlPatternFromRoot, iFp)
GO(xmlPatternGetStreamCtxt, pFp)
GO(xmlPatternMatch, iFpp)
GO(xmlPatternMaxDepth, iFp)
GO(xmlPatternMinDepth, iFp)
GO(xmlPatternStreamable, iFp)
GO(xmlPedanticParserDefault, iFi)
GO(__xmlPedanticParserDefaultValue, pFv)
GO(xmlPopInput, CFp)
GO(xmlPopInputCallbacks, iFv)
GO(xmlPreviousElementSibling, pFp)
//GO(xmlPrintURI, 
GO(xmlPushInput, iFpi)
//GO(__xmlRaiseError, 
GO(xmlReadDoc, pFpppi)
GO(xmlReaderForDoc, pFpppi)
GO(xmlReaderForFd, pFippi)
GO(xmlReaderForFile, pFppi)
GOM(xmlReaderForIO, pFEpppppi)
GO(xmlReaderForMemory, pFpippi)
GO(xmlReaderNewDoc, iFppppi)
GO(xmlReaderNewFd, iFpippi)
GO(xmlReaderNewFile, iFpppi)
//GOM(xmlReaderNewIO, iFEppppppi)
GO(xmlReaderNewMemory, iFppippi)
GO(xmlReaderNewWalker, iFpp)
GO(xmlReaderWalker, pFp)
GO(xmlReadFd, pFippi)
GO(xmlReadFile, pFppi)
//GOM(xmlReadIO, pFEpppppi)
GO(xmlReadMemory, pFpippi)
GO(xmlReallocLoc, pFpLpi)
GO(xmlReconciliateNs, iFpp)
GO(xmlRecoverDoc, pFp)
GO(xmlRecoverFile, pFp)
GO(xmlRecoverMemory, pFpi)
GO(xmlRegExecErrInfo, iFpppppp)
GO(xmlRegExecNextValues, iFppppp)
GO(xmlRegExecPushString, iFppp)
GO(xmlRegExecPushString2, iFpppp)
GO(xmlRegexpCompile, pFp)
GO(xmlRegexpExec, iFpp)
GO(xmlRegexpIsDeterminist, iFp)
GO(xmlRegexpPrint, vFSp)
GO(xmlRegFreeExecCtxt, vFp)
GO(xmlRegFreeRegexp, vFp)
//GOM(xmlRegisterCharEncodingHandler, vFEp)
GO(xmlRegisterDefaultInputCallbacks, vFv)
GO(xmlRegisterDefaultOutputCallbacks, vFv)
GO(xmlRegisterHTTPPostCallbacks, vFv)
GOM(xmlRegisterInputCallbacks, iFEpppp)
//GOM(xmlRegisterNodeDefault, pFEp)
//GOM(__xmlRegisterNodeDefaultValue, pFEv)
GOM(xmlRegisterOutputCallbacks, iFEpppp)
//GOM(xmlRegNewExecCtxt, pFEppp)
GO(xmlRelaxNGCleanupTypes, vFv)
GO(xmlRelaxNGDump, vFSp)
GO(xmlRelaxNGDumpTree, vFSp)
GO(xmlRelaxNGFree, vFp)
GO(xmlRelaxNGFreeParserCtxt, vFp)
GO(xmlRelaxNGFreeValidCtxt, vFp)
//GOM(xmlRelaxNGGetParserErrors, iFEpppp)
//GOM(xmlRelaxNGGetValidErrors, iFEpppp)
GO(xmlRelaxNGInitTypes, iFv)
GO(xmlRelaxNGNewDocParserCtxt, pFp)
GO(xmlRelaxNGNewMemParserCtxt, pFpi)
GO(xmlRelaxNGNewParserCtxt, pFp)
GO(xmlRelaxNGNewValidCtxt, pFp)
GO(xmlRelaxNGParse, pFp)
//GOM(xmlRelaxNGSetParserErrors, vFEpppp)
//GOM(xmlRelaxNGSetParserStructuredErrors, vFEppp)
//GOM(xmlRelaxNGSetValidErrors, vFEpppp)
//GOM(xmlRelaxNGSetValidStructuredErrors, vFEppp)
GO(xmlRelaxNGValidateDoc, iFpp)
GO(xmlRelaxNGValidateFullElement, iFppp)
GO(xmlRelaxNGValidatePopElement, iFppp)
GO(xmlRelaxNGValidatePushCData, iFppi)
GO(xmlRelaxNGValidatePushElement, iFppp)
GO(xmlRelaxParserSetFlag, iFpi)
GO(xmlRemoveID, iFpp)
GO(xmlRemoveProp, iFp)
GO(xmlRemoveRef, iFpp)
GO(xmlReplaceNode, pFpp)
GO(xmlResetError, vFp)
GO(xmlResetLastError, vFv)
GO(xmlRMutexLock, vFp)
GO(xmlRMutexUnlock, vFp)
GO(xmlSaveClose, iFp)
GO(xmlSaveDoc, lFpp)
GO(xmlSaveFile, iFpp)
GO(xmlSaveFileEnc, iFppp)
GO(xmlSaveFileTo, iFppp)
GO(xmlSaveFlush, iFp)
GO(xmlSaveFormatFile, iFppi)
GO(xmlSaveFormatFileEnc, iFpppi)
GO(xmlSaveFormatFileTo, iFpppi)
GO(__xmlSaveNoEmptyTags, pFv)
//GOM(xmlSaveSetAttrEscape, iFEpB)
//GOM(xmlSaveSetEscape, iFEpB)
GO(xmlSaveToBuffer, pFppi)
GO(xmlSaveToFd, pFipi)
GO(xmlSaveToFilename, pFppi)
GOM(xmlSaveToIO, pFEppppi)
GO(xmlSaveTree, lFpp)
//GO(xmlSaveUri, 
GO(xmlSAX2AttributeDecl, vFpppiipp)
GO(xmlSAX2CDataBlock, vFppi)
GO(xmlSAX2Characters, vFppi)
GO(xmlSAX2Comment, vFpp)
GO(xmlSAX2ElementDecl, vFppip)
GO(xmlSAX2EndDocument, vFp)
GO(xmlSAX2EndElement, vFpp)
GO(xmlSAX2EndElementNs, vFpppp)
GO(xmlSAX2EntityDecl, vFppippp)
GO(xmlSAX2ExternalSubset, vFpppp)
GO(xmlSAX2GetColumnNumber, iFp)
GO(xmlSAX2GetEntity, pFpp)
GO(xmlSAX2GetLineNumber, iFp)
GO(xmlSAX2GetParameterEntity, pFpp)
GO(xmlSAX2GetPublicId, pFp)
GO(xmlSAX2GetSystemId, pFp)
GO(xmlSAX2HasExternalSubset, iFp)
GO(xmlSAX2HasInternalSubset, iFp)
GO(xmlSAX2IgnorableWhitespace, vFppi)
GO(xmlSAX2InitDefaultSAXHandler, vFpi)
GO(xmlSAX2InitDocbDefaultSAXHandler, vFp)
GO(xmlSAX2InitHtmlDefaultSAXHandler, vFp)
GO(xmlSAX2InternalSubset, vFpppp)
GO(xmlSAX2IsStandalone, iFp)
GO(xmlSAX2NotationDecl, vFpppp)
GO(xmlSAX2ProcessingInstruction, vFppp)
GO(xmlSAX2Reference, vFpp)
GO(xmlSAX2ResolveEntity, pFppp)
GO(xmlSAX2SetDocumentLocator, vFpp)
GO(xmlSAX2StartDocument, vFp)
GO(xmlSAX2StartElement, vFppp)
GO(xmlSAX2StartElementNs, vFppppipiip)
GO(xmlSAX2UnparsedEntityDecl, vFppppp)
GO(xmlSAXDefaultVersion, iFi)
GO(xmlSAXParseDoc, pFppi)
GO(xmlSAXParseDTD, pFppp)
GO(xmlSAXParseEntity, pFpp)
GO(xmlSAXParseFile, pFppi)
GO(xmlSAXParseFileWithData, pFppip)
GO(xmlSAXParseMemory, pFppii)
GO(xmlSAXParseMemoryWithData, pFppiip)
GO(xmlSAXUserParseFile, iFppp)
GO(xmlSAXUserParseMemory, iFpppi)
GO(xmlSAXVersion, iFpi)
GO(xmlScanName, pFp)
GO(xmlSchemaCheckFacet, iFpppp)
GO(xmlSchemaCleanupTypes, vFv)
GO(xmlSchemaCollapseString, pFp)
GO(xmlSchemaCompareValues, iFpp)
GO(xmlSchemaCompareValuesWhtsp, iFpupu)
GO(xmlSchemaCopyValue, pFp)
GO(xmlSchemaDump, vFSp)
GO(xmlSchemaFree, vFp)
GO(xmlSchemaFreeFacet, vFp)
GO(xmlSchemaFreeParserCtxt, vFp)
GO(xmlSchemaFreeType, vFp)
GO(xmlSchemaFreeValidCtxt, vFp)
GO(xmlSchemaFreeValue, vFp)
GO(xmlSchemaFreeWildcard, vFp)
GO(xmlSchemaGetBuiltInListSimpleTypeItemType, pFp)
GO(xmlSchemaGetBuiltInType, pFu)
GO(xmlSchemaGetCanonValue, iFpp)
GO(xmlSchemaGetCanonValueWhtsp, iFppu)
GO(xmlSchemaGetFacetValueAsULong, LFp)
//GOM(xmlSchemaGetParserErrors, iFEpppp)
GO(xmlSchemaGetPredefinedType, pFpp)
//GOM(xmlSchemaGetValidErrors, iFEpppp)
GO(xmlSchemaGetValType, uFp)
GO(xmlSchemaInitTypes, vFv)
GO(xmlSchemaIsBuiltInTypeFacet, iFpi)
GO(xmlSchemaIsValid, iFp)
GO(xmlSchemaNewDocParserCtxt, pFp)
GO(xmlSchemaNewFacet, pFv)
GO(xmlSchemaNewMemParserCtxt, pFpi)
GO(xmlSchemaNewNOTATIONValue, pFpp)
GO(xmlSchemaNewParserCtxt, pFp)
GO(xmlSchemaNewQNameValue, pFpp)
GO(xmlSchemaNewStringValue, pFup)
GO(xmlSchemaNewValidCtxt, pFp)
GO(xmlSchemaParse, pFp)
GO(xmlSchemaSAXPlug, pFppp)
GO(xmlSchemaSAXUnplug, iFp)
GOM(xmlSchemaSetParserErrors, vFEpppp)
GOM(xmlSchemaSetParserStructuredErrors, vFEppp)
GOM(xmlSchemaSetValidErrors, vFEpppp)
GO(xmlSchemaSetValidOptions, iFpi)
GOM(xmlSchemaSetValidStructuredErrors, vFEppp)
//GO(xmlSchematronFree, 
//GO(xmlSchematronFreeParserCtxt, 
//GO(xmlSchematronFreeValidCtxt, 
//GO(xmlSchematronNewDocParserCtxt, 
//GO(xmlSchematronNewMemParserCtxt, 
//GO(xmlSchematronNewParserCtxt, 
//GO(xmlSchematronNewValidCtxt, 
//GO(xmlSchematronParse, 
//GO(xmlSchematronSetValidStructuredErrors, 
//GO(xmlSchematronValidateDoc, 
GO(xmlSchemaValidateDoc, iFpp)
GO(xmlSchemaValidateFacet, iFpppp)
GO(xmlSchemaValidateFacetWhtsp, iFpuuppu)
GO(xmlSchemaValidateFile, iFppi)
GO(xmlSchemaValidateLengthFacet, iFppppp)
GO(xmlSchemaValidateLengthFacetWhtsp, iFpupppu)
GO(xmlSchemaValidateListSimpleTypeFacet, iFppLp)
GO(xmlSchemaValidateOneElement, iFpp)
GO(xmlSchemaValidatePredefinedType, iFppp)
GO(xmlSchemaValidateSetFilename, vFpp)
//GOM(xmlSchemaValidateSetLocator, vFEppp)
GOM(xmlSchemaValidateStream, iFppipp)
GO(xmlSchemaValidCtxtGetOptions, iFp)
GO(xmlSchemaValidCtxtGetParserCtxt, pFp)
GO(xmlSchemaValPredefTypeNode, iFpppp)
GO(xmlSchemaValPredefTypeNodeNoNorm, iFpppp)
GO(xmlSchemaValueAppend, iFpp)
GO(xmlSchemaValueGetAsBoolean, iFp)
GO(xmlSchemaValueGetAsString, pFp)
GO(xmlSchemaValueGetNext, pFp)
GO(xmlSchemaWhiteSpaceReplace, pFp)
GO(xmlSearchNs, pFppp)
GO(xmlSearchNsByHref, pFppp)
GO(xmlSetBufferAllocationScheme, vFu)
GO(xmlSetCompressMode, vFi)
GO(xmlSetDocCompressMode, vFpi)
GOM(xmlSetExternalEntityLoader, vFEp)
GO(xmlSetFeature, iFppp)
GOM(xmlSetGenericErrorFunc, vFEpp)
GO(xmlSetListDoc, vFpp)
GO(xmlSetNs, vFpp)
GO(xmlSetNsProp, pFpppp)
GO(xmlSetProp, pFppp)
GOM(xmlSetStructuredErrorFunc, vFEpp)
GO(xmlSetTreeDoc, vFpp)
GO(xmlSetupParserForBuffer, vFppp)
//GO(__xmlSimpleError, 
GO(xmlSkipBlankChars, iFp)
GO(xmlSnprintfElementContent, vFpipi)
GO(xmlSplitQName, pFppp)
GO(xmlSplitQName2, pFpp)
GO(xmlSplitQName3, pFpp)
GO(xmlSprintfElementContent, vFppi)
GO(xmlStopParser, vFp)
GO(xmlStrcasecmp, iFpp)
GO(xmlStrcasestr, pFpp)
GO(xmlStrcat, pFpp)
GO(xmlStrchr, pFpC)
GO(xmlStrcmp, iFpp)
GO(xmlStrdup, pFp)
GO(xmlStreamPop, iFp)
GO(xmlStreamPush, iFppp)
GO(xmlStreamPushAttr, iFppp)
GO(xmlStreamPushNode, iFpppi)
GO(xmlStreamWantsAnyNode, iFp)
GO(xmlStrEqual, iFpp)
GO(xmlStringCurrentChar, iFppp)
GO(xmlStringDecodeEntities, pFppiCCC)
GO(xmlStringGetNodeList, pFpp)
GO(xmlStringLenDecodeEntities, pFppiiCCC)
GO(xmlStringLenGetNodeList, pFppi)
GO(xmlStrlen, iFp)
GO(xmlStrncasecmp, iFppi)
GO(xmlStrncat, pFppi)
GO(xmlStrncatNew, pFppi)
GO(xmlStrncmp, iFppi)
GO(xmlStrndup, pFpi)
GO2(xmlStrPrintf, iFpipV, xmlStrVPrintf)
GO(xmlStrQEqual, iFppp)
GO(xmlStrstr, pFpp)
GO(xmlStrsub, pFpii)
GOM(__xmlStructuredError, pFEv)
GO(__xmlStructuredErrorContext, pFv)
GO(xmlStrVPrintf, iFpipp)
GO(xmlSubstituteEntitiesDefault, iFi)
GO(__xmlSubstituteEntitiesDefaultValue, pFv)
GO(xmlSwitchEncoding, iFpi)
GO(xmlSwitchInputEncoding, iFppp)
GO(xmlSwitchToEncoding, iFpp)
GO(xmlTextConcat, iFppi)
GO(xmlTextMerge, pFpp)
GO(xmlTextReaderAttributeCount, iFp)
GO(xmlTextReaderBaseUri, pFp)
GO(xmlTextReaderByteConsumed, lFp)
GO(xmlTextReaderClose, iFp)
GO(xmlTextReaderConstBaseUri, pFp)
GO(xmlTextReaderConstEncoding, pFp)
GO(xmlTextReaderConstLocalName, pFp)
GO(xmlTextReaderConstName, pFp)
GO(xmlTextReaderConstNamespaceUri, pFp)
GO(xmlTextReaderConstPrefix, pFp)
GO(xmlTextReaderConstString, pFpp)
GO(xmlTextReaderConstValue, pFp)
GO(xmlTextReaderConstXmlLang, pFp)
GO(xmlTextReaderConstXmlVersion, pFp)
GO(xmlTextReaderCurrentDoc, pFp)
GO(xmlTextReaderCurrentNode, pFp)
GO(xmlTextReaderDepth, iFp)
GO(xmlTextReaderExpand, pFp)
GO(xmlTextReaderGetAttribute, pFpp)
GO(xmlTextReaderGetAttributeNo, pFpi)
GO(xmlTextReaderGetAttributeNs, pFppp)
//GOM(xmlTextReaderGetErrorHandler, vFEppp)
GO(xmlTextReaderGetParserColumnNumber, iFp)
GO(xmlTextReaderGetParserLineNumber, iFp)
GO(xmlTextReaderGetParserProp, iFpi)
//GOM(xmlTextReaderGetRemainder, pFEp)
GO(xmlTextReaderHasAttributes, iFp)
GO(xmlTextReaderHasValue, iFp)
GO(xmlTextReaderIsDefault, iFp)
GO(xmlTextReaderIsEmptyElement, iFp)
GO(xmlTextReaderIsNamespaceDecl, iFp)
GO(xmlTextReaderIsValid, iFp)
GO(xmlTextReaderLocalName, pFp)
GO(xmlTextReaderLocatorBaseURI, pFp)
GO(xmlTextReaderLocatorLineNumber, iFp)
GO(xmlTextReaderLookupNamespace, pFpp)
GO(xmlTextReaderMoveToAttribute, iFpp)
GO(xmlTextReaderMoveToAttributeNo, iFpi)
GO(xmlTextReaderMoveToAttributeNs, iFppp)
GO(xmlTextReaderMoveToElement, iFp)
GO(xmlTextReaderMoveToFirstAttribute, iFp)
GO(xmlTextReaderMoveToNextAttribute, iFp)
GO(xmlTextReaderName, pFp)
GO(xmlTextReaderNamespaceUri, pFp)
GO(xmlTextReaderNext, iFp)
GO(xmlTextReaderNextSibling, iFp)
GO(xmlTextReaderNodeType, iFp)
GO(xmlTextReaderNormalization, iFp)
GO(xmlTextReaderPrefix, pFp)
GO(xmlTextReaderPreserve, pFp)
GO(xmlTextReaderPreservePattern, iFppp)
GO(xmlTextReaderQuoteChar, iFp)
GO(xmlTextReaderRead, iFp)
GO(xmlTextReaderReadAttributeValue, iFp)
GO(xmlTextReaderReadInnerXml, pFp)
GO(xmlTextReaderReadOuterXml, pFp)
GO(xmlTextReaderReadState, iFp)
GO(xmlTextReaderReadString, pFp)
GO(xmlTextReaderRelaxNGSetSchema, iFpp)
GO(xmlTextReaderRelaxNGValidate, iFpp)
GO(xmlTextReaderSchemaValidate, iFpp)
GO(xmlTextReaderSchemaValidateCtxt, iFppi)
GOM(xmlTextReaderSetErrorHandler, vFEppp)
GO(xmlTextReaderSetParserProp, iFpii)
GO(xmlTextReaderSetSchema, iFpp)
//GOM(xmlTextReaderSetStructuredErrorHandler, vFEppp)
//GOM(xmlTextReaderSetup, iFEppppi)
GO(xmlTextReaderStandalone, iFp)
GO(xmlTextReaderValue, pFp)
GO(xmlTextReaderXmlLang, pFp)
GO(xmlTextWriterEndAttribute, iFp)
GO(xmlTextWriterEndCDATA, iFp)
GO(xmlTextWriterEndComment, iFp)
GO(xmlTextWriterEndDocument, iFp)
GO(xmlTextWriterEndDTD, iFp)
GO(xmlTextWriterEndDTDAttlist, iFp)
GO(xmlTextWriterEndDTDElement, iFp)
GO(xmlTextWriterEndDTDEntity, iFp)
GO(xmlTextWriterEndElement, iFp)
GO(xmlTextWriterEndPI, iFp)
GO(xmlTextWriterFlush, iFp)
GO(xmlTextWriterFullEndElement, iFp)
GO(xmlTextWriterSetIndent, iFpi)
GO(xmlTextWriterSetIndentString, iFpp)
GO(xmlTextWriterStartAttribute, iFpp)
GO(xmlTextWriterStartAttributeNS, iFpppp)
GO(xmlTextWriterStartCDATA, iFp)
GO(xmlTextWriterStartComment, iFp)
GO(xmlTextWriterStartDocument, iFpppp)
GO(xmlTextWriterStartDTD, iFpppp)
GO(xmlTextWriterStartDTDAttlist, iFpp)
GO(xmlTextWriterStartDTDElement, iFpp)
GO(xmlTextWriterStartDTDEntity, iFpip)
GO(xmlTextWriterStartElement, iFpp)
GO(xmlTextWriterStartElementNS, iFpppp)
GO(xmlTextWriterStartPI, iFpp)
GO(xmlTextWriterWriteAttribute, iFppp)
GO(xmlTextWriterWriteAttributeNS, iFppppp)
GO(xmlTextWriterWriteBase64, iFppii)
GO(xmlTextWriterWriteBinHex, iFppii)
GO(xmlTextWriterWriteCDATA, iFpp)
GO(xmlTextWriterWriteComment, iFpp)
GO(xmlTextWriterWriteDTD, iFppppp)
GO(xmlTextWriterWriteDTDAttlist, iFppp)
GO(xmlTextWriterWriteDTDElement, iFppp)
GO(xmlTextWriterWriteDTDEntity, iFpippppp)
GO(xmlTextWriterWriteDTDExternalEntity, iFpipppp)
GO(xmlTextWriterWriteDTDExternalEntityContents, iFpppp)
GO(xmlTextWriterWriteDTDInternalEntity, iFpipp)
GO(xmlTextWriterWriteDTDNotation, iFpppp)
GO(xmlTextWriterWriteElement, iFppp)
GO(xmlTextWriterWriteElementNS, iFppppp)
GOM(xmlTextWriterWriteFormatAttribute, iFEpppV)
//GO(xmlTextWriterWriteFormatAttributeNS, 
//GO(xmlTextWriterWriteFormatCDATA, 
//GO(xmlTextWriterWriteFormatComment, 
//GO(xmlTextWriterWriteFormatDTD, 
//GO(xmlTextWriterWriteFormatDTDAttlist, 
//GO(xmlTextWriterWriteFormatDTDElement, 
//GO(xmlTextWriterWriteFormatDTDInternalEntity, 
//GO(xmlTextWriterWriteFormatElement, 
//GO(xmlTextWriterWriteFormatElementNS, 
//GO(xmlTextWriterWriteFormatPI, 
//GO(xmlTextWriterWriteFormatRaw, 
//GO(xmlTextWriterWriteFormatString, 
GO(xmlTextWriterWritePI, iFppp)
GO(xmlTextWriterWriteRaw, iFpp)
GO(xmlTextWriterWriteRawLen, iFppi)
GO(xmlTextWriterWriteString, iFpp)
GOM(xmlTextWriterWriteVFormatAttribute, iFEpppA)
//GO(xmlTextWriterWriteVFormatAttributeNS, 
//GO(xmlTextWriterWriteVFormatCDATA, 
//GO(xmlTextWriterWriteVFormatComment, 
//GO(xmlTextWriterWriteVFormatDTD, 
//GO(xmlTextWriterWriteVFormatDTDAttlist, 
//GO(xmlTextWriterWriteVFormatDTDElement, 
//GO(xmlTextWriterWriteVFormatDTDInternalEntity, 
//GO(xmlTextWriterWriteVFormatElement, 
//GO(xmlTextWriterWriteVFormatElementNS, 
//GO(xmlTextWriterWriteVFormatPI, 
//GO(xmlTextWriterWriteVFormatRaw, 
//GO(xmlTextWriterWriteVFormatString, 
GO(xmlThrDefBufferAllocScheme, uFu)
GO(xmlThrDefDefaultBufferSize, iFi)
//GOM(xmlThrDefDeregisterNodeDefault, pFEp)
GO(xmlThrDefDoValidityCheckingDefaultValue, iFi)
GO(xmlThrDefGetWarningsDefaultValue, iFi)
GO(xmlThrDefIndentTreeOutput, iFi)
GO(xmlThrDefKeepBlanksDefaultValue, iFi)
GO(xmlThrDefLineNumbersDefaultValue, iFi)
GO(xmlThrDefLoadExtDtdDefaultValue, iFi)
//GOM(xmlThrDefOutputBufferCreateFilenameDefault, pFEp)
GO(xmlThrDefParserDebugEntities, iFi)
//GOM(xmlThrDefParserInputBufferCreateFilenameDefault, pFEp)
GO(xmlThrDefPedanticParserDefaultValue, iFi)
//GOM(xmlThrDefRegisterNodeDefault, pFEp)
GO(xmlThrDefSaveNoEmptyTags, iFi)
//GOM(xmlThrDefSetGenericErrorFunc, vFEpp)
//GOM(xmlThrDefSetStructuredErrorFunc, vFEpp)
GO(xmlThrDefSubstituteEntitiesDefaultValue, iFi)
GO(xmlThrDefTreeIndentString, pFp)
GO(__xmlTreeIndentString, pFv)
GO(xmlUCSIsAegeanNumbers, iFi)
GO(xmlUCSIsAlphabeticPresentationForms, iFi)
GO(xmlUCSIsArabic, iFi)
GO(xmlUCSIsArabicPresentationFormsA, iFi)
GO(xmlUCSIsArabicPresentationFormsB, iFi)
GO(xmlUCSIsArmenian, iFi)
GO(xmlUCSIsArrows, iFi)
GO(xmlUCSIsBasicLatin, iFi)
GO(xmlUCSIsBengali, iFi)
GO(xmlUCSIsBlock, iFip)
GO(xmlUCSIsBlockElements, iFi)
GO(xmlUCSIsBopomofo, iFi)
GO(xmlUCSIsBopomofoExtended, iFi)
GO(xmlUCSIsBoxDrawing, iFi)
GO(xmlUCSIsBraillePatterns, iFi)
GO(xmlUCSIsBuhid, iFi)
GO(xmlUCSIsByzantineMusicalSymbols, iFi)
GO(xmlUCSIsCat, iFip)
GO(xmlUCSIsCatC, iFi)
GO(xmlUCSIsCatCc, iFi)
GO(xmlUCSIsCatCf, iFi)
GO(xmlUCSIsCatCo, iFi)
GO(xmlUCSIsCatCs, iFi)
GO(xmlUCSIsCatL, iFi)
GO(xmlUCSIsCatLl, iFi)
GO(xmlUCSIsCatLm, iFi)
GO(xmlUCSIsCatLo, iFi)
GO(xmlUCSIsCatLt, iFi)
GO(xmlUCSIsCatLu, iFi)
GO(xmlUCSIsCatM, iFi)
GO(xmlUCSIsCatMc, iFi)
GO(xmlUCSIsCatMe, iFi)
GO(xmlUCSIsCatMn, iFi)
GO(xmlUCSIsCatN, iFi)
GO(xmlUCSIsCatNd, iFi)
GO(xmlUCSIsCatNl, iFi)
GO(xmlUCSIsCatNo, iFi)
GO(xmlUCSIsCatP, iFi)
GO(xmlUCSIsCatPc, iFi)
GO(xmlUCSIsCatPd, iFi)
GO(xmlUCSIsCatPe, iFi)
GO(xmlUCSIsCatPf, iFi)
GO(xmlUCSIsCatPi, iFi)
GO(xmlUCSIsCatPo, iFi)
GO(xmlUCSIsCatPs, iFi)
GO(xmlUCSIsCatS, iFi)
GO(xmlUCSIsCatSc, iFi)
GO(xmlUCSIsCatSk, iFi)
GO(xmlUCSIsCatSm, iFi)
GO(xmlUCSIsCatSo, iFi)
GO(xmlUCSIsCatZ, iFi)
GO(xmlUCSIsCatZl, iFi)
GO(xmlUCSIsCatZp, iFi)
GO(xmlUCSIsCatZs, iFi)
GO(xmlUCSIsCherokee, iFi)
GO(xmlUCSIsCJKCompatibility, iFi)
GO(xmlUCSIsCJKCompatibilityForms, iFi)
GO(xmlUCSIsCJKCompatibilityIdeographs, iFi)
GO(xmlUCSIsCJKCompatibilityIdeographsSupplement, iFi)
GO(xmlUCSIsCJKRadicalsSupplement, iFi)
GO(xmlUCSIsCJKSymbolsandPunctuation, iFi)
GO(xmlUCSIsCJKUnifiedIdeographs, iFi)
GO(xmlUCSIsCJKUnifiedIdeographsExtensionA, iFi)
GO(xmlUCSIsCJKUnifiedIdeographsExtensionB, iFi)
GO(xmlUCSIsCombiningDiacriticalMarks, iFi)
GO(xmlUCSIsCombiningDiacriticalMarksforSymbols, iFi)
GO(xmlUCSIsCombiningHalfMarks, iFi)
GO(xmlUCSIsCombiningMarksforSymbols, iFi)
GO(xmlUCSIsControlPictures, iFi)
GO(xmlUCSIsCurrencySymbols, iFi)
GO(xmlUCSIsCypriotSyllabary, iFi)
GO(xmlUCSIsCyrillic, iFi)
GO(xmlUCSIsCyrillicSupplement, iFi)
GO(xmlUCSIsDeseret, iFi)
GO(xmlUCSIsDevanagari, iFi)
GO(xmlUCSIsDingbats, iFi)
GO(xmlUCSIsEnclosedAlphanumerics, iFi)
GO(xmlUCSIsEnclosedCJKLettersandMonths, iFi)
GO(xmlUCSIsEthiopic, iFi)
GO(xmlUCSIsGeneralPunctuation, iFi)
GO(xmlUCSIsGeometricShapes, iFi)
GO(xmlUCSIsGeorgian, iFi)
GO(xmlUCSIsGothic, iFi)
GO(xmlUCSIsGreek, iFi)
GO(xmlUCSIsGreekandCoptic, iFi)
GO(xmlUCSIsGreekExtended, iFi)
GO(xmlUCSIsGujarati, iFi)
GO(xmlUCSIsGurmukhi, iFi)
GO(xmlUCSIsHalfwidthandFullwidthForms, iFi)
GO(xmlUCSIsHangulCompatibilityJamo, iFi)
GO(xmlUCSIsHangulJamo, iFi)
GO(xmlUCSIsHangulSyllables, iFi)
GO(xmlUCSIsHanunoo, iFi)
GO(xmlUCSIsHebrew, iFi)
GO(xmlUCSIsHighPrivateUseSurrogates, iFi)
GO(xmlUCSIsHighSurrogates, iFi)
GO(xmlUCSIsHiragana, iFi)
GO(xmlUCSIsIdeographicDescriptionCharacters, iFi)
GO(xmlUCSIsIPAExtensions, iFi)
GO(xmlUCSIsKanbun, iFi)
GO(xmlUCSIsKangxiRadicals, iFi)
GO(xmlUCSIsKannada, iFi)
GO(xmlUCSIsKatakana, iFi)
GO(xmlUCSIsKatakanaPhoneticExtensions, iFi)
GO(xmlUCSIsKhmer, iFi)
GO(xmlUCSIsKhmerSymbols, iFi)
GO(xmlUCSIsLao, iFi)
GO(xmlUCSIsLatin1Supplement, iFi)
GO(xmlUCSIsLatinExtendedA, iFi)
GO(xmlUCSIsLatinExtendedAdditional, iFi)
GO(xmlUCSIsLatinExtendedB, iFi)
GO(xmlUCSIsLetterlikeSymbols, iFi)
GO(xmlUCSIsLimbu, iFi)
GO(xmlUCSIsLinearBIdeograms, iFi)
GO(xmlUCSIsLinearBSyllabary, iFi)
GO(xmlUCSIsLowSurrogates, iFi)
GO(xmlUCSIsMalayalam, iFi)
GO(xmlUCSIsMathematicalAlphanumericSymbols, iFi)
GO(xmlUCSIsMathematicalOperators, iFi)
GO(xmlUCSIsMiscellaneousMathematicalSymbolsA, iFi)
GO(xmlUCSIsMiscellaneousMathematicalSymbolsB, iFi)
GO(xmlUCSIsMiscellaneousSymbols, iFi)
GO(xmlUCSIsMiscellaneousSymbolsandArrows, iFi)
GO(xmlUCSIsMiscellaneousTechnical, iFi)
GO(xmlUCSIsMongolian, iFi)
GO(xmlUCSIsMusicalSymbols, iFi)
GO(xmlUCSIsMyanmar, iFi)
GO(xmlUCSIsNumberForms, iFi)
GO(xmlUCSIsOgham, iFi)
GO(xmlUCSIsOldItalic, iFi)
GO(xmlUCSIsOpticalCharacterRecognition, iFi)
GO(xmlUCSIsOriya, iFi)
GO(xmlUCSIsOsmanya, iFi)
GO(xmlUCSIsPhoneticExtensions, iFi)
GO(xmlUCSIsPrivateUse, iFi)
GO(xmlUCSIsPrivateUseArea, iFi)
GO(xmlUCSIsRunic, iFi)
GO(xmlUCSIsShavian, iFi)
GO(xmlUCSIsSinhala, iFi)
GO(xmlUCSIsSmallFormVariants, iFi)
GO(xmlUCSIsSpacingModifierLetters, iFi)
GO(xmlUCSIsSpecials, iFi)
GO(xmlUCSIsSuperscriptsandSubscripts, iFi)
GO(xmlUCSIsSupplementalArrowsA, iFi)
GO(xmlUCSIsSupplementalArrowsB, iFi)
GO(xmlUCSIsSupplementalMathematicalOperators, iFi)
GO(xmlUCSIsSupplementaryPrivateUseAreaA, iFi)
GO(xmlUCSIsSupplementaryPrivateUseAreaB, iFi)
GO(xmlUCSIsSyriac, iFi)
GO(xmlUCSIsTagalog, iFi)
GO(xmlUCSIsTagbanwa, iFi)
GO(xmlUCSIsTags, iFi)
GO(xmlUCSIsTaiLe, iFi)
GO(xmlUCSIsTaiXuanJingSymbols, iFi)
GO(xmlUCSIsTamil, iFi)
GO(xmlUCSIsTelugu, iFi)
GO(xmlUCSIsThaana, iFi)
GO(xmlUCSIsThai, iFi)
GO(xmlUCSIsTibetan, iFi)
GO(xmlUCSIsUgaritic, iFi)
GO(xmlUCSIsUnifiedCanadianAboriginalSyllabics, iFi)
GO(xmlUCSIsVariationSelectors, iFi)
GO(xmlUCSIsVariationSelectorsSupplement, iFi)
GO(xmlUCSIsYijingHexagramSymbols, iFi)
GO(xmlUCSIsYiRadicals, iFi)
GO(xmlUCSIsYiSyllables, iFi)
GO(xmlUnlinkNode, vFp)
GO(xmlUnlockLibrary, vFv)
GO(xmlUnsetNsProp, iFppp)
GO(xmlUnsetProp, iFpp)
//GO(xmlURIEscape, 
//GO(xmlURIEscapeStr, 
//GO(xmlURIUnescapeString, 
GO(xmlUTF8Charcmp, iFpp)
GO(xmlUTF8Size, iFp)
GO(xmlUTF8Strlen, iFp)
GO(xmlUTF8Strloc, iFpp)
GO(xmlUTF8Strndup, pFpi)
GO(xmlUTF8Strpos, pFpi)
GO(xmlUTF8Strsize, iFpi)
GO(xmlUTF8Strsub, pFpii)
GO(xmlValidateAttributeDecl, iFppp)
GO(xmlValidateAttributeValue, iFup)
GO(xmlValidateDocument, iFpp)
GO(xmlValidateDocumentFinal, iFpp)
GO(xmlValidateDtd, iFpp)
GO(xmlValidateDtdFinal, iFpp)
GO(xmlValidateElement, iFppp)
GO(xmlValidateElementDecl, iFppp)
GO(xmlValidateName, iFpi)
GO(xmlValidateNamesValue, iFp)
GO(xmlValidateNameValue, iFp)
GO(xmlValidateNCName, iFpi)
GO(xmlValidateNMToken, iFpi)
GO(xmlValidateNmtokensValue, iFp)
GO(xmlValidateNmtokenValue, iFp)
GO(xmlValidateNotationDecl, iFppp)
GO(xmlValidateNotationUse, iFppp)
GO(xmlValidateOneAttribute, iFppppp)
GO(xmlValidateOneElement, iFppp)
GO(xmlValidateOneNamespace, iFpppppp)
GO(xmlValidatePopElement, iFpppp)
GO(xmlValidatePushCData, iFppi)
GO(xmlValidatePushElement, iFpppp)
GO(xmlValidateQName, iFpi)
GO(xmlValidateRoot, iFpp)
GO(xmlValidBuildContentModel, iFpp)
GO(xmlValidCtxtNormalizeAttributeValue, pFppppp)
GO(xmlValidGetPotentialChildren, iFpppi)
GO(xmlValidGetValidElements, iFpppi)
GO(xmlValidNormalizeAttributeValue, pFpppp)
//GO(xmlXIncludeFreeContext, 
//GO(xmlXIncludeNewContext, 
//GO(xmlXIncludeProcess, 
//GO(xmlXIncludeProcessFlags, 
//GO(xmlXIncludeProcessFlagsData, 
//GO(xmlXIncludeProcessNode, 
//GO(xmlXIncludeProcessTree, 
//GO(xmlXIncludeProcessTreeFlags, 
//GO(xmlXIncludeProcessTreeFlagsData, 
//GO(xmlXIncludeSetFlags, 
GO(xmlXPathAddValues, vFp)
GO(xmlXPathBooleanFunction, vFpi)
GO(xmlXPathCastBooleanToNumber, dFi)
GO(xmlXPathCastBooleanToString, pFi)
GO(xmlXPathCastNodeSetToBoolean, iFp)
GO(xmlXPathCastNodeSetToNumber, dFp)
GO(xmlXPathCastNodeSetToString, pFp)
GO(xmlXPathCastNodeToNumber, dFp)
GO(xmlXPathCastNodeToString, pFp)
GO(xmlXPathCastNumberToBoolean, iFd)
GO(xmlXPathCastNumberToString, pFd)
GO(xmlXPathCastStringToBoolean, iFp)
GO(xmlXPathCastStringToNumber, dFp)
GO(xmlXPathCastToBoolean, iFp)
GO(xmlXPathCastToNumber, dFp)
GO(xmlXPathCastToString, pFp)
GO(xmlXPathCeilingFunction, vFpi)
GO(xmlXPathCmpNodes, iFpp)
GO(xmlXPathCompareValues, iFpii)
GO(xmlXPathCompile, pFp)
GO(xmlXPathCompiledEval, pFpp)
GO(xmlXPathCompiledEvalToBoolean, iFpp)
GO(xmlXPathConcatFunction, vFpi)
GO(xmlXPathContainsFunction, vFpi)
GO(xmlXPathContextSetCache, iFpiii)
GO(xmlXPathConvertBoolean, pFp)
GO(xmlXPathConvertNumber, pFp)
GO(xmlXPathConvertString, pFp)
GO(xmlXPathCountFunction, vFpi)
GO(xmlXPathCtxtCompile, pFpp)
GO(xmlXPathDebugDumpCompExpr, vFSpi)
GO(xmlXPathDebugDumpObject, vFSpi)
GO(xmlXPathDifference, pFpp)
GO(xmlXPathDistinct, pFp)
GO(xmlXPathDistinctSorted, pFp)
GO(xmlXPathDivValues, vFp)
GO(xmlXPathEqualValues, iFp)
GO(xmlXPathErr, vFpi)
GO(xmlXPatherror, vFppii)
GO(xmlXPathEval, pFpp)
GO(xmlXPathEvalExpr, vFp)
GO(xmlXPathEvalExpression, pFpp)
GO(xmlXPathEvalPredicate, iFpp)
GO(xmlXPathEvaluatePredicateResult, iFpp)
GO(xmlXPathFalseFunction, vFpi)
GO(xmlXPathFloorFunction, vFpi)
GO(xmlXPathFreeCompExpr, vFp)
GO(xmlXPathFreeContext, vFp)
GO(xmlXPathFreeNodeSet, vFp)
GO(xmlXPathFreeNodeSetList, vFp)
GO(xmlXPathFreeObject, vFp)
GO(xmlXPathFreeParserContext, vFp)
//GOM(xmlXPathFunctionLookup, BFEpp)
//GOM(xmlXPathFunctionLookupNS, BFEppp)
GO(xmlXPathHasSameNodes, iFpp)
//GO(xmlXPathIdFunction, 
GO(xmlXPathInit, vFv)
GO(xmlXPathIntersection, pFpp)
GO(xmlXPathIsInf, iFd)
GO(xmlXPathIsNaN, iFd)
GO(xmlXPathIsNodeType, iFp)
GO(xmlXPathLangFunction, vFpi)
GO(xmlXPathLastFunction, vFpi)
GO(xmlXPathLeading, pFpp)
GO(xmlXPathLeadingSorted, pFpp)
GO(xmlXPathLocalNameFunction, vFpi)
GO(xmlXPathModValues, vFp)
GO(xmlXPathMultValues, vFp)
GO(xmlXPathNamespaceURIFunction, vFpi)
GO(xmlXPathNewBoolean, pFi)
GO(xmlXPathNewContext, pFp)
GO(xmlXPathNewCString, pFp)
GO(xmlXPathNewFloat, pFd)
GO(xmlXPathNewNodeSet, pFp)
GO(xmlXPathNewNodeSetList, pFp)
GO(xmlXPathNewParserContext, pFpp)
GO(xmlXPathNewString, pFp)
GO(xmlXPathNewValueTree, pFp)
GO(xmlXPathNextAncestor, pFpp)
GO(xmlXPathNextAncestorOrSelf, pFpp)
GO(xmlXPathNextAttribute, pFpp)
GO(xmlXPathNextChild, pFpp)
GO(xmlXPathNextDescendant, pFpp)
GO(xmlXPathNextDescendantOrSelf, pFpp)
GO(xmlXPathNextFollowing, pFpp)
GO(xmlXPathNextFollowingSibling, pFpp)
GO(xmlXPathNextNamespace, pFpp)
GO(xmlXPathNextParent, pFpp)
GO(xmlXPathNextPreceding, pFpp)
GO(xmlXPathNextPrecedingSibling, pFpp)
GO(xmlXPathNextSelf, pFpp)
GO(xmlXPathNodeEval, pFppp)
GO(xmlXPathNodeLeading, pFpp)
GO(xmlXPathNodeLeadingSorted, pFpp)
GO(xmlXPathNodeSetAdd, iFpp)
GO(xmlXPathNodeSetAddNs, iFppp)
GO(xmlXPathNodeSetAddUnique, iFpp)
GO(xmlXPathNodeSetContains, iFpp)
GO(xmlXPathNodeSetCreate, pFp)
GO(xmlXPathNodeSetDel, vFpp)
GO(xmlXPathNodeSetFreeNs, vFp)
GO(xmlXPathNodeSetMerge, pFpp)
GO(xmlXPathNodeSetRemove, vFpi)
GO(xmlXPathNodeSetSort, vFp)
GO(xmlXPathNodeTrailing, pFpp)
GO(xmlXPathNodeTrailingSorted, pFpp)
GO(xmlXPathNormalizeFunction, vFpi)
GO(xmlXPathNotEqualValues, iFp)
GO(xmlXPathNotFunction, vFpi)
GO(xmlXPathNsLookup, pFpp)
GO(xmlXPathNumberFunction, vFpi)
GO(xmlXPathObjectCopy, pFp)
GO(xmlXPathOrderDocElems, lFp)
GO(xmlXPathParseName, pFp)
GO(xmlXPathParseNCName, pFp)
GO(xmlXPathPopBoolean, iFp)
GO(xmlXPathPopExternal, pFp)
GO(xmlXPathPopNodeSet, pFp)
GO(xmlXPathPopNumber, dFp)
GO(xmlXPathPopString, pFp)
GO(xmlXPathPositionFunction, vFpi)
GO(xmlXPathRegisterAllFunctions, vFp)
GO(xmlXPathRegisteredFuncsCleanup, vFp)
GO(xmlXPathRegisteredNsCleanup,vFp)
GO(xmlXPathRegisteredVariablesCleanup, vFp)
GOM(xmlXPathRegisterFunc, iFEppp)
//GOM(xmlXPathRegisterFuncLookup, vFEpBp)
//GOM(xmlXPathRegisterFuncNS, iFEpppB)
GO(xmlXPathRegisterNs, iFppp)
GO(xmlXPathRegisterVariable, iFppp)
//GOM(xmlXPathRegisterVariableLookup, vFEpBp)
GO(xmlXPathRegisterVariableNS, iFpppp)
GO(xmlXPathRoot, vFp)
GO(xmlXPathRoundFunction, vFpi)
GO(xmlXPathSetContextNode, iFpp)
GO(xmlXPathStartsWithFunction, vFpi)
GO(xmlXPathStringEvalNumber, dFp)
GO(xmlXPathStringFunction, vFpi)
GO(xmlXPathStringLengthFunction, vFpi)
GO(xmlXPathSubstringAfterFunction, vFpi)
GO(xmlXPathSubstringBeforeFunction, vFpi)
GO(xmlXPathSubstringFunction, vFpi)
GO(xmlXPathSubValues, vFp)
GO(xmlXPathSumFunction, vFpi)
GO(xmlXPathTrailing, pFpp)
GO(xmlXPathTrailingSorted, pFpp)
GO(xmlXPathTranslateFunction, vFpi)
GO(xmlXPathTrueFunction, vFpi)
GO(xmlXPathValueFlipSign, vFp)
GO(xmlXPathValuePush, iFpp)
GO(xmlXPathVariableLookup, pFpp)
GO(xmlXPathVariableLookupNS, pFppp)
GO(xmlXPathWrapCString, pFp)
GO(xmlXPathWrapExternal, pFp)
GO(xmlXPathWrapNodeSet, pFp)
GO(xmlXPathWrapString, pFp)
//GO(xmlXPtrAdvanceNode, 
//GO(xmlXPtrBuildNodeList, 
//GOM(xmlXPtrEval, pFEpp)
//GO(xmlXPtrEvalRangePredicate, 
//GO(xmlXPtrFreeLocationSet, 
//GO(xmlXPtrLocationSetAdd, 
//GO(xmlXPtrLocationSetCreate, 
//GO(xmlXPtrLocationSetDel, 
//GO(xmlXPtrLocationSetMerge, 
//GO(xmlXPtrLocationSetRemove, 
//GO(xmlXPtrNewCollapsedRange, 
//GOM(xmlXPtrNewContext, pFEppp)
//GO(xmlXPtrNewLocationSetNodes, 
//GO(xmlXPtrNewLocationSetNodeSet, 
//GO(xmlXPtrNewRange, 
//GO(xmlXPtrNewRangeNodeObject, 
//GO(xmlXPtrNewRangeNodePoint, 
//GO(xmlXPtrNewRangeNodes, 
//GO(xmlXPtrNewRangePointNode, 
//GO(xmlXPtrNewRangePoints, 
//GO(xmlXPtrRangeToFunction, 
//GO(xmlXPtrWrapLocationSet, 

DATAM(xmlFree, sizeof(void*))
DATAM(xmlMalloc, sizeof(void*))
DATAM(xmlRealloc, sizeof(void*))
DATAM(xmlMemStrdup, sizeof(void*))

DATA(xmlIsDigitGroup, 2*sizeof(void*)+2*sizeof(int))
DATA(xmlStringText, sizeof(void*))
DATA(xmlStringTextNoenc, sizeof(void*))
DATA(xmlIsCombiningGroup, 2*sizeof(void*)+2*sizeof(int))
DATA(xmlIsExtenderGroup, 2*sizeof(void*)+2*sizeof(int))
DATA(xmlIsBaseCharGroup, 2*sizeof(void*)+2*sizeof(int))
