/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibdlTYPES32_H_
#define __wrappedlibdlTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef int32_t (*iEp_t)(void*);
typedef void* (*pEv_t)(void);
typedef int32_t (*iEpp_t)(void*, void*);
typedef void* (*pEpi_t)(void*, int32_t);
typedef void* (*pEpp_t)(void*, void*);
typedef int32_t (*iEpip_t)(void*, int32_t, void*);
typedef void* (*pEppi_t)(void*, void*, int32_t);
typedef void* (*pEppp_t)(void*, void*, void*);
typedef int32_t (*iEpppi_t)(void*, void*, void*, int32_t);

#define SUPER() ADDED_FUNCTIONS() \
	GO(dlclose, iEp_t) \
	GO(dlerror, pEv_t) \
	GO(_dl_find_object, iEpp_t) \
	GO(dladdr, iEpp_t) \
	GO(dlopen, pEpi_t) \
	GO(dlsym, pEpp_t) \
	GO(dlinfo, iEpip_t) \
	GO(dlmopen, pEppi_t) \
	GO(dlvsym, pEppp_t) \
	GO(dladdr1, iEpppi_t)

#endif // __wrappedlibdlTYPES32_H_
