#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_75, pFpp)
GO(ubrk_close_75, vFp)
GO(ubrk_countAvailable_75, iFv)
GO(ubrk_current_75, iFp)
GO(ubrk_first_75, iFp)
GO(ubrk_following_75, iFpi)
GO(ubrk_getAvailable_75, pFi)
GO(ubrk_getBinaryRules_75, iFppip)
GO(ubrk_getLocaleByType_75, pFpip)
GO(ubrk_getRuleStatus_75, iFp)
GO(ubrk_getRuleStatusVec_75, iFppip)
GO(ubrk_isBoundary_75, cFpi)
GO(ubrk_last_75, iFp)
GO(ubrk_next_75, iFp)
GO(ubrk_open_75, pFppip)
GO(ubrk_openBinaryRules_75, pFpipip)
GO(ubrk_openRules_75, pFpipipp)
GO(ubrk_preceding_75, iFpi)
GO(ubrk_previous_75, iFp)
GO(ubrk_refreshUText_75, vFppp)
GO(ubrk_safeClone_75, pFpppp)
GO(ubrk_setText_75, vFppip)
GO(ubrk_setUText_75, vFppp)
GO(ubrk_swap_75, iFppipp)

GO(ucnv_getDefaultName_75, pFv)
GO(ucnv_open_75, pFpp)
GO(ucnv_setDefaultName_75, vFp)

GO(ucurr_forLocale_75, iFppip)
GO(ucurr_getName_75, pFppippp)

GO(uloc_canonicalize_75, iFppip)
GO(uloc_countAvailable_75, iFv)
GO(uloc_getAvailable_75, pFi)
GO(uloc_getBaseName_75, iFppip)
GO(uloc_getCharacterOrientation_75, iFpp)
GO(uloc_getCountry_75, iFppip)
GO(uloc_getDefault_75, pFv)
GO(uloc_getDisplayCountry_75, iFpppip)
GO(uloc_getDisplayLanguage_75, iFpppip)
GO(uloc_getDisplayName_75, iFpppip)
GO(uloc_getISO3Country_75, pFp)
GO(uloc_getISO3Language_75, pFp)
GO(uloc_getKeywordValue_75, iFpppip)
GO(uloc_getLanguage_75, iFppip)
GO(uloc_getLCID_75, uFp)
GO(uloc_getName_75, iFppip)
GO(uloc_getParent_75, iFppip)
GO(uloc_setKeywordValue_75, iFpppip)

GO(ures_close_75, vFp)
GO(ures_getByKey_75, pFpppp)
GO(ures_getSize_75, iFp)
GO(ures_getStringByIndex_75, pFpipp)
GO(ures_open_75, pFppp)

GO(uldn_close_75, vFp)
GO(uldn_keyValueDisplayName_75, iFppppip)
GO(uldn_open_75, pFpip)

GO(uenum_close_75, vFp)
GO(uenum_count_75, iFpp)
GO(uenum_next_75, pFppp)

GO(uidna_close_75, vFp)
GO(uidna_nameToASCII_75, iFppipipp)
GO(uidna_nameToUnicode_75, iFppipipp)
GO(uidna_openUTS46_75, pFpp)

GO(unorm2_getNFCInstance_75, pFp)
GO(unorm2_getNFDInstance_75, pFp)
GO(unorm2_getNFKCInstance_75, pFp)
GO(unorm2_getNFKDInstance_75, pFp)
GO(unorm2_isNormalized_75, CFppip)
GO(unorm2_normalize_75, iFppipip)

GO(u_charsToUChars_75, vFppi)
GO(u_getVersion_75, vFp)
GO(u_strcmp_75, iFpp)
GO(u_strcpy_75, pFpp)
GO(u_strlen_75, iFp)
GO(u_strncpy_75, pFppi)
GO(u_strToLower_75, iFpipipp)
GO(u_strToUpper_75, iFpipipp)
GO(u_tolower_75, uFu)
GO(u_toupper_75, uFu)
GO(u_uastrncpy_75, pFppi)
