#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error meh!
#endif

//GO(_atspi_accessible_add_cache, 
//GO(atspi_accessible_clear_cache, 
//GO(atspi_accessible_get_accessible_id, 
//GO(atspi_accessible_get_action, 
//GO(atspi_accessible_get_action_iface, 
GO(atspi_accessible_get_application, pFpp)
//GO(atspi_accessible_get_atspi_version, 
GO(atspi_accessible_get_attributes, pFpp)
//GO(atspi_accessible_get_attributes_as_array, 
//GO(_atspi_accessible_get_cache_mask, 
GO(atspi_accessible_get_child_at_index, pFpip)
GO(atspi_accessible_get_child_count, iFpp)
//GO(atspi_accessible_get_collection, 
//GO(atspi_accessible_get_collection_iface, 
//GO(atspi_accessible_get_component, 
//GO(atspi_accessible_get_component_iface, 
GO(atspi_accessible_get_description, pFpp)
//GO(atspi_accessible_get_document, 
//GO(atspi_accessible_get_document_iface, 
//GO(atspi_accessible_get_editable_text, 
//GO(atspi_accessible_get_editable_text_iface, 
//GO(atspi_accessible_get_hyperlink, 
//GO(atspi_accessible_get_hypertext, 
//GO(atspi_accessible_get_hypertext_iface, 
//GO(atspi_accessible_get_id, 
//GO(atspi_accessible_get_image, 
//GO(atspi_accessible_get_image_iface, 
//GO(atspi_accessible_get_index_in_parent, 
GO(atspi_accessible_get_interfaces, pFp)
//GO(atspi_accessible_get_localized_role_name, 
GO(atspi_accessible_get_name, pFpp)
//GO(atspi_accessible_get_object_locale, 
GO(atspi_accessible_get_parent, pFpp)
GO(atspi_accessible_get_process_id, uFpp)
GO(atspi_accessible_get_relation_set, pFpp)
GO(atspi_accessible_get_role, iFpp)
GO(atspi_accessible_get_role_name, pFpp)
//GO(atspi_accessible_get_selection, 
//GO(atspi_accessible_get_selection_iface, 
GO(atspi_accessible_get_state_set, pFpp)
//GO(atspi_accessible_get_table, 
//GO(atspi_accessible_get_table_cell, 
//GO(atspi_accessible_get_table_iface, 
//GO(atspi_accessible_get_text, 
//GO(atspi_accessible_get_text_iface, 
//GO(atspi_accessible_get_toolkit_name, 
//GO(atspi_accessible_get_toolkit_version, 
//GO(atspi_accessible_get_type, 
//GO(atspi_accessible_get_value, 
//GO(atspi_accessible_get_value_iface, 
//GO(atspi_accessible_is_action, 
//GO(atspi_accessible_is_application, 
//GO(atspi_accessible_is_collection, 
//GO(atspi_accessible_is_component, 
//GO(atspi_accessible_is_document, 
//GO(atspi_accessible_is_editable_text, 
//GO(atspi_accessible_is_hyperlink, 
//GO(atspi_accessible_is_hypertext, 
//GO(atspi_accessible_is_image, 
//GO(atspi_accessible_is_selection, 
//GO(atspi_accessible_is_streamable_content, 
//GO(atspi_accessible_is_table, 
//GO(atspi_accessible_is_table_cell, 
//GO(atspi_accessible_is_text, 
//GO(atspi_accessible_is_value, 
//GO(_atspi_accessible_new, 
//GO(_atspi_accessible_ref_cache, 
//GO(atspi_accessible_set_cache_mask, 
//GO(_atspi_accessible_test_cache, 
//GO(_atspi_accessible_unref_cache, 
//GO(atspi_action_do_action, 
//GO(atspi_action_get_action_description, 
//GO(atspi_action_get_action_name, 
//GO(atspi_action_get_description, 
//GO(atspi_action_get_key_binding, 
//GO(atspi_action_get_localized_name, 
//GO(atspi_action_get_n_actions, 
//GO(atspi_action_get_name, 
//GO(atspi_action_get_type, 
//GO(atspi_application_get_type, 
//GO(_atspi_application_new, 
//GO(_atspi_bus, 
//DATA(atspi_bus_registry, 
//GO(atspi_cache_get_type, 
//GO(atspi_collection_get_active_descendant, 
//GO(atspi_collection_get_matches, 
//GO(atspi_collection_get_matches_from, 
//GO(atspi_collection_get_matches_to, 
//GO(atspi_collection_get_type, 
//GO(atspi_collection_is_ancestor_of, 
//GO(atspi_collection_match_type_get_type, 
//GO(atspi_collection_sort_order_get_type, 
//GO(atspi_collection_tree_traversal_type_get_type, 
//GO(atspi_component_contains, 
//GO(atspi_component_get_accessible_at_point, 
//GO(atspi_component_get_alpha, 
//GO(atspi_component_get_extents, 
//GO(atspi_component_get_layer, 
//GO(atspi_component_get_mdi_z_order, 
//GO(atspi_component_get_position, 
//GO(atspi_component_get_size, 
//GO(atspi_component_get_type, 
//GO(atspi_component_grab_focus, 
//GO(atspi_component_layer_get_type, 
//GO(atspi_component_scroll_to, 
//GO(atspi_component_scroll_to_point, 
//GO(atspi_component_set_extents, 
//GO(atspi_component_set_position, 
//GO(atspi_component_set_size, 
//GO(atspi_coord_type_get_type, 
//GO(_atspi_dbus_attribute_array_from_iter, 
//GO(_atspi_dbus_call, 
//GO(_atspi_dbus_call_partial, 
//GO(_atspi_dbus_call_partial_va, 
//GO(atspi_dbus_connection_setup_with_g_main, 
//GO(_atspi_dbus_get_property, 
//GO(_atspi_dbus_handle_DeviceEvent, 
//GO(_atspi_dbus_handle_event, 
//GO(_atspi_dbus_hash_from_iter, 
//GO(_atspi_dbus_return_accessible_from_iter, 
//GO(_atspi_dbus_return_accessible_from_message, 
//GO(_atspi_dbus_return_attribute_array_from_message, 
//GO(_atspi_dbus_return_hash_from_message, 
//GO(_atspi_dbus_return_hyperlink_from_iter, 
//GO(_atspi_dbus_return_hyperlink_from_message, 
//GO(_atspi_dbus_send_with_reply_and_block, 
//GO(atspi_dbus_server_setup_with_g_main, 
//GO(_atspi_dbus_set_interfaces, 
//GO(_atspi_dbus_set_state, 
//GO(_atspi_dbus_update_cache_from_dict, 
//GO(atspi_deregister_device_event_listener, 
//GO(atspi_deregister_keystroke_listener, 
//GO(atspi_device_event_free, 
//GO(atspi_device_event_get_type, 
//GO(atspi_device_listener_add_callback, 
//GO(_atspi_device_listener_get_path, 
//GO(atspi_device_listener_get_type, 
//GO(atspi_device_listener_new, 
//GO(atspi_device_listener_new_simple, 
//GO(atspi_device_listener_remove_callback, 
//GO(atspi_document_get_attributes, 
//GO(atspi_document_get_attribute_value, 
//GO(atspi_document_get_current_page_number, 
//GO(atspi_document_get_document_attributes, 
//GO(atspi_document_get_document_attribute_value, 
//GO(atspi_document_get_locale, 
//GO(atspi_document_get_page_count, 
//GO(atspi_document_get_type, 
//GO(atspi_editable_text_copy_text, 
//GO(atspi_editable_text_cut_text, 
//GO(atspi_editable_text_delete_text, 
//GO(atspi_editable_text_get_type, 
//GO(atspi_editable_text_insert_text, 
//GO(atspi_editable_text_paste_text, 
//GO(atspi_editable_text_set_text_contents, 
//GO(_atspi_error_quark, 
GO(atspi_event_get_type, LFv)
GO(atspi_event_listener_deregister, iFppp)
//GO(atspi_event_listener_deregister_from_callback, 
//GO(atspi_event_listener_deregister_no_data, 
//GO(atspi_event_listener_get_type, 
GOM(atspi_event_listener_new, pFppp)
//GO(atspi_event_listener_new_simple, 
GO(atspi_event_listener_register, iFppp)
//GO(atspi_event_listener_register_from_callback, 
//GO(atspi_event_listener_register_from_callback_full, 
//GO(atspi_event_listener_register_full, 
//GO(atspi_event_listener_register_no_data, 
//GO(atspi_event_main, 
//GO(atspi_event_quit, 
//GO(atspi_event_type_get_type, 
//GO(atspi_exit, 
//GO(atspi_generate_keyboard_event, 
//GO(atspi_generate_mouse_event, 
//GO(atspi_get_a11y_bus, 
//GO(_atspi_get_allow_sync, 
GO(atspi_get_desktop, pFi)
//GO(atspi_get_desktop_count, 
//GO(atspi_get_desktop_list, 
//GO(_atspi_get_iface_num, 
//GO(_atspi_get_live_refs, 
//GO(atspi_hyperlink_get_end_index, 
//GO(atspi_hyperlink_get_index_range, 
//GO(atspi_hyperlink_get_n_anchors, 
//GO(atspi_hyperlink_get_object, 
//GO(atspi_hyperlink_get_start_index, 
//GO(atspi_hyperlink_get_type, 
//GO(atspi_hyperlink_get_uri, 
//GO(atspi_hyperlink_is_valid, 
//GO(_atspi_hyperlink_new, 
//GO(atspi_hypertext_get_link, 
//GO(atspi_hypertext_get_link_index, 
//GO(atspi_hypertext_get_n_links, 
//GO(atspi_hypertext_get_type, 
//GO(atspi_image_get_image_description, 
//GO(atspi_image_get_image_extents, 
//GO(atspi_image_get_image_locale, 
//GO(atspi_image_get_image_position, 
//GO(atspi_image_get_image_size, 
//GO(atspi_image_get_type, 
GO(atspi_init, iFv)
//DATA(atspi_interface_accessible, 
//DATA(atspi_interface_action, 
//DATA(atspi_interface_application, 
//DATA(atspi_interface_cache, 
//DATA(atspi_interface_collection, 
//DATA(atspi_interface_component, 
//DATA(atspi_interface_dec, 
//DATA(atspi_interface_device_event_listener, 
//DATA(atspi_interface_document, 
//DATA(atspi_interface_editable_text, 
//DATA(atspi_interface_event_object, 
//DATA(atspi_interface_hyperlink, 
//DATA(atspi_interface_hypertext, 
//DATA(atspi_interface_image, 
//DATA(atspi_interface_registry, 
//DATA(atspi_interface_selection, 
//DATA(atspi_interface_table, 
//DATA(atspi_interface_table_cell, 
//DATA(atspi_interface_text, 
//DATA(atspi_interface_value, 
//GO(atspi_is_initialized, 
//GO(atspi_key_definition_copy, 
//GO(atspi_key_definition_free, 
//GO(atspi_key_definition_get_type, 
//GO(atspi_key_event_type_get_type, 
//GO(atspi_key_listener_sync_type_get_type, 
//GO(atspi_key_synth_type_get_type, 
//GO(atspi_locale_type_get_type, 
//DATAB(atspi_main_context, 
//DATAB(atspi_main_loop, 
//GO(atspi_marshal_VOID__INT_INT, 
//GO(atspi_match_rule_get_type, 
//GO(_atspi_match_rule_marshal, 
//GO(atspi_match_rule_new, 
//GO(atspi_modifier_type_get_type, 
//GO(_atspi_mutter_generate_keyboard_event, 
//GO(_atspi_mutter_generate_mouse_event, 
//GO(_atspi_mutter_set_reference_window, 
//GO(_atspi_name_compat, 
//DATAB(atspi_no_cache, 
//GO(atspi_object_get_type, 
//DATA(atspi_path_dec, 
//DATA(atspi_path_registry, 
//DATA(atspi_path_root, 
//GO(atspi_point_copy, 
//GO(atspi_point_get_type, 
//GO(_atspi_prepare_screen_reader_interface, 
//GO(atspi_range_copy, 
//GO(atspi_range_get_type, 
//GO(atspi_rect_copy, 
//GO(atspi_rect_free, 
//GO(atspi_rect_get_type, 
//GO(_atspi_ref_accessible, 
//GO(atspi_register_device_event_listener, 
//GO(atspi_register_keystroke_listener, 
GO(atspi_relation_get_n_targets, iFp)
GO(atspi_relation_get_relation_type, iFp)
GO(atspi_relation_get_target, pFpi)
//GO(atspi_relation_get_type, 
//GO(_atspi_relation_new_from_iter, 
//GO(atspi_relation_type_get_type, 
//GO(_atspi_reregister_device_listeners, 
//GO(_atspi_reregister_event_listeners, 
//GO(atspi_role_get_name, 
//GO(atspi_role_get_type, 
//GO(atspi_scroll_type_get_type, 
//GO(atspi_selection_clear_selection, 
//GO(atspi_selection_deselect_child, 
//GO(atspi_selection_deselect_selected_child, 
//GO(atspi_selection_get_n_selected_children, 
//GO(atspi_selection_get_selected_child, 
//GO(atspi_selection_get_type, 
//GO(atspi_selection_is_child_selected, 
//GO(atspi_selection_select_all, 
//GO(atspi_selection_select_child, 
//GO(_atspi_send_event, 
//GO(_atspi_set_allow_sync, 
//GO(_atspi_set_error_no_sync, 
//GO(atspi_set_main_context, 
//GO(atspi_set_reference_window, 
//GO(atspi_set_timeout, 
//GO(atspi_state_set_add, 
//GO(atspi_state_set_compare, 
//GO(atspi_state_set_contains, 
//GO(atspi_state_set_equals, 
GO(atspi_state_set_get_states, pFp)
//GO(atspi_state_set_get_type, 
//GO(atspi_state_set_is_empty, 
//GO(atspi_state_set_new, 
//GO(_atspi_state_set_new_internal, 
//GO(atspi_state_set_remove, 
//GO(atspi_state_set_set_by_name, 
//GO(atspi_state_type_get_type, 
//GO(atspi_table_add_column_selection, 
//GO(atspi_table_add_row_selection, 
//GO(atspi_table_cell_get_column_header_cells, 
//GO(atspi_table_cell_get_column_span, 
//GO(atspi_table_cell_get_position, 
//GO(atspi_table_cell_get_row_column_span, 
//GO(atspi_table_cell_get_row_header_cells, 
//GO(atspi_table_cell_get_row_span, 
//GO(atspi_table_cell_get_table, 
//GO(atspi_table_cell_get_type, 
//GO(atspi_table_get_accessible_at, 
//GO(atspi_table_get_caption, 
//GO(atspi_table_get_column_at_index, 
//GO(atspi_table_get_column_description, 
//GO(atspi_table_get_column_extent_at, 
//GO(atspi_table_get_column_header, 
//GO(atspi_table_get_index_at, 
//GO(atspi_table_get_n_columns, 
//GO(atspi_table_get_n_rows, 
//GO(atspi_table_get_n_selected_columns, 
//GO(atspi_table_get_n_selected_rows, 
//GO(atspi_table_get_row_at_index, 
//GO(atspi_table_get_row_column_extents_at_index, 
//GO(atspi_table_get_row_description, 
//GO(atspi_table_get_row_extent_at, 
//GO(atspi_table_get_row_header, 
//GO(atspi_table_get_selected_columns, 
//GO(atspi_table_get_selected_rows, 
//GO(atspi_table_get_summary, 
//GO(atspi_table_get_type, 
//GO(atspi_table_is_column_selected, 
//GO(atspi_table_is_row_selected, 
//GO(atspi_table_is_selected, 
//GO(atspi_table_remove_column_selection, 
//GO(atspi_table_remove_row_selection, 
//GO(atspi_text_add_selection, 
//GO(atspi_text_boundary_type_get_type, 
//GO(atspi_text_clip_type_get_type, 
//GO(atspi_text_get_attribute_run, 
//GO(atspi_text_get_attributes, 
//GO(atspi_text_get_attribute_value, 
//GO(atspi_text_get_bounded_ranges, 
//GO(atspi_text_get_caret_offset, 
//GO(atspi_text_get_character_at_offset, 
//GO(atspi_text_get_character_count, 
//GO(atspi_text_get_character_extents, 
//GO(atspi_text_get_default_attributes, 
//GO(atspi_text_get_n_selections, 
//GO(atspi_text_get_offset_at_point, 
//GO(atspi_text_get_range_extents, 
//GO(atspi_text_get_selection, 
//GO(atspi_text_get_string_at_offset, 
//GO(atspi_text_get_text, 
//GO(atspi_text_get_text_after_offset, 
//GO(atspi_text_get_text_at_offset, 
//GO(atspi_text_get_text_attributes, 
//GO(atspi_text_get_text_attribute_value, 
//GO(atspi_text_get_text_before_offset, 
//GO(atspi_text_get_type, 
//GO(atspi_text_granularity_get_type, 
//GO(atspi_text_range_get_type, 
//GO(atspi_text_remove_selection, 
//GO(atspi_text_scroll_substring_to, 
//GO(atspi_text_scroll_substring_to_point, 
//GO(atspi_text_set_caret_offset, 
//GO(atspi_text_set_selection, 
//GO(atspi_value_get_current_value, 
//GO(atspi_value_get_maximum_value, 
//GO(atspi_value_get_minimum_increment, 
//GO(atspi_value_get_minimum_value, 
//GO(atspi_value_get_type, 
//GO(atspi_value_set_current_value, 
//DATA(cache_signal_type, 
//GO(callback_ref, 
//GO(callback_unref, 
//GO(dbind_any_demarshal, 
//GO(dbind_any_demarshal_va, 
//GO(dbind_any_free, 
//GO(dbind_any_free_ptr, 
//GO(dbind_any_marshal, 
//GO(dbind_any_marshal_va, 
//GO(dbind_emit_signal, 
//GO(dbind_emit_signal_va, 
//GO(dbind_find_c_alignment, 
//GO(dbind_method_call_reentrant, 
//GO(dbind_method_call_reentrant_va, 
//GO(dbind_send_and_allow_reentry, 
//GO(dbind_set_timeout, 
//DATA(_dbus_gmain_connection_slot, 
//DATAB(device_parent_class, 
//GO(free_value, 
//GO(get_reference_from_iter, 
//DATA(old_cache_signal_type, 
//DATA(str_parent, 
