#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif
GO(TTF_AddFallbackFont, iFpp)
GO(TTF_AppendTextString, iFppL)
GO(TTF_ClearFallbackFonts, vFp)
GO(TTF_CloseFont, vFp)
GO(TTF_CopyFont, pFp)
GO(TTF_CreateGPUTextEngine, pFp)
GO(TTF_CreateGPUTextEngineWithProperties, pFu)
GO(TTF_CreateRendererTextEngine, pFp)
GO(TTF_CreateRendererTextEngineWithProperties, pFu)
GO(TTF_CreateSurfaceTextEngine, pFv)
GO(TTF_CreateText, pFpppL)
GO(TTF_DeleteTextString, iFpii)
GO(TTF_DestroyGPUTextEngine, vFp)
GO(TTF_DestroyRendererTextEngine, vFp)
GO(TTF_DestroySurfaceTextEngine, vFp)
GO(TTF_DestroyText, vFp)
GO(TTF_DrawRendererText, iFpff)
GO(TTF_DrawSurfaceText, iFpiip)
GO(TTF_FontHasGlyph, iFpu)
GO(TTF_FontIsFixedWidth, iFp)
GO(TTF_FontIsScalable, iFp)
GO(TTF_GetFontAscent, iFp)
GO(TTF_GetFontDescent, iFp)
GO(TTF_GetFontDirection, uFp)
GO(TTF_GetFontDPI, iFppp)
GO(TTF_GetFontFamilyName, pFp)
GO(TTF_GetFontGeneration, uFp)
GO(TTF_GetFontHeight, iFp)
GO(TTF_GetFontHinting, iFp)
GO(TTF_GetFontKerning, iFp)
GO(TTF_GetFontLineSkip, iFp)
GO(TTF_GetFontOutline, iFp)
GO(TTF_GetFontProperties, uFp)
GO(TTF_GetFontScript, uFp)
GO(TTF_GetFontSDF, iFp)
GO(TTF_GetFontSize, fFp)
GO(TTF_GetFontStyle, uFp)
GO(TTF_GetFontStyleName, pFp)
GO(TTF_GetFontWeight, iFp)
GO(TTF_GetFontWrapAlignment, iFp)
GO(TTF_GetFreeTypeVersion, vFppp)
GO(TTF_GetGlyphImage, pFpup)
GO(TTF_GetGlyphImageForIndex, pFpup)
GO(TTF_GetGlyphKerning, iFpuup)
GO(TTF_GetGlyphMetrics, iFpuppppp)
GO(TTF_GetGlyphScript, uFu)
GO(TTF_GetGPUTextDrawData, pFp)
GO(TTF_GetGPUTextEngineWinding, iFp)
GO(TTF_GetHarfBuzzVersion, vFppp)
GO(TTF_GetNextTextSubString, iFppp)
GO(TTF_GetNumFontFaces, iFp)
GO(TTF_GetPreviousTextSubString, iFppp)
GO(TTF_GetStringSize, iFppLpp)
GO(TTF_GetStringSizeWrapped, iFppLipp)
GO(TTF_GetTextColor, iFppppp)
GO(TTF_GetTextColorFloat, iFppppp)
GO(TTF_GetTextDirection, uFp)
GO(TTF_GetTextEngine, pFp)
GO(TTF_GetTextFont, pFp)
GO(TTF_GetTextPosition, iFppp)
GO(TTF_GetTextProperties, uFp)
GO(TTF_GetTextScript, uFp)
GO(TTF_GetTextSize, iFppp)
GO(TTF_GetTextSubString, iFpip)
GO(TTF_GetTextSubStringForLine, iFpip)
GO(TTF_GetTextSubStringForPoint, iFpiip)
GO(TTF_GetTextSubStringsForRange, pFpiip)
GO(TTF_GetTextWrapWidth, iFpp)
GO(TTF_Init, iFv)
GO(TTF_InsertTextString, iFpipL)
GO(TTF_MeasureString, iFppLipp)
GO(TTF_OpenFont, pFpf)
GO(TTF_OpenFontIO, pFpif)
GO(TTF_OpenFontWithProperties, pFu)
GO(TTF_Quit, vFv)
GO(TTF_RemoveFallbackFont, vFpp)
GO(TTF_RenderGlyph_Blended, pFpuu)
GO(TTF_RenderGlyph_LCD, pFpuuu)
GO(TTF_RenderGlyph_Shaded, pFpuuu)
GO(TTF_RenderGlyph_Solid, pFpuu)
GO(TTF_RenderText_Blended, pFppLu)
GO(TTF_RenderText_Blended_Wrapped, pFppLui)
GO(TTF_RenderText_LCD, pFppLuu)
GO(TTF_RenderText_LCD_Wrapped, pFppLuui)
GO(TTF_RenderText_Shaded, pFppLuu)
GO(TTF_RenderText_Shaded_Wrapped, pFppLuui)
GO(TTF_RenderText_Solid, pFppLu)
GO(TTF_RenderText_Solid_Wrapped, pFppLui)
GO(TTF_SetFontDirection, iFpu)
GO(TTF_SetFontHinting, vFpi)
GO(TTF_SetFontKerning, vFpi)
GO(TTF_SetFontLanguage, iFpp)
GO(TTF_SetFontLineSkip, vFpi)
GO(TTF_SetFontOutline, iFpi)
GO(TTF_SetFontScript, iFpu)
GO(TTF_SetFontSDF, iFpi)
GO(TTF_SetFontSize, iFpf)
GO(TTF_SetFontSizeDPI, iFpfii)
GO(TTF_SetFontStyle, vFpu)
GO(TTF_SetFontWrapAlignment, vFpi)
GO(TTF_SetGPUTextEngineWinding, vFpi)
GO(TTF_SetTextColor, iFpCCCC)
GO(TTF_SetTextColorFloat, iFpffff)
GO(TTF_SetTextDirection, iFpu)
GO(TTF_SetTextEngine, iFpp)
GO(TTF_SetTextFont, iFpp)
GO(TTF_SetTextPosition, iFpii)
GO(TTF_SetTextScript, iFpu)
GO(TTF_SetTextString, iFppL)
GO(TTF_SetTextWrapWhitespaceVisible, iFpi)
GO(TTF_SetTextWrapWidth, iFpi)
GO(TTF_StringToTag, uFp)
GO(TTF_TagToString, vFupL)
GO(TTF_TextWrapWhitespaceVisible, iFp)
GO(TTF_UpdateText, iFp)
GO(TTF_Version, iFv)
GO(TTF_WasInit, iFv)
