#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(xcb_glx_are_textures_resident, 
//GO(xcb_glx_are_textures_resident_data, 
//GO(xcb_glx_are_textures_resident_data_end, 
//GO(xcb_glx_are_textures_resident_data_length, 
//GO(xcb_glx_are_textures_resident_reply, 
//GO(xcb_glx_are_textures_resident_sizeof, 
//GO(xcb_glx_are_textures_resident_unchecked, 
//GO(xcb_glx_bool32_end, 
//GO(xcb_glx_bool32_next, 
//GO(xcb_glx_change_drawable_attributes, 
//GO(xcb_glx_change_drawable_attributes_attribs, 
//GO(xcb_glx_change_drawable_attributes_attribs_end, 
//GO(xcb_glx_change_drawable_attributes_attribs_length, 
//GO(xcb_glx_change_drawable_attributes_checked, 
//GO(xcb_glx_change_drawable_attributes_sizeof, 
//GO(xcb_glx_client_info, 
//GO(xcb_glx_client_info_checked, 
//GO(xcb_glx_client_info_sizeof, 
//GO(xcb_glx_client_info_string, 
//GO(xcb_glx_client_info_string_end, 
//GO(xcb_glx_client_info_string_length, 
//GO(xcb_glx_context_end, 
//GO(xcb_glx_context_next, 
//GO(xcb_glx_context_tag_end, 
//GO(xcb_glx_context_tag_next, 
//GO(xcb_glx_copy_context, 
//GO(xcb_glx_copy_context_checked, 
//GO(xcb_glx_create_context, 
//GO(xcb_glx_create_context_attribs_arb, 
//GO(xcb_glx_create_context_attribs_arb_attribs, 
//GO(xcb_glx_create_context_attribs_arb_attribs_end, 
//GO(xcb_glx_create_context_attribs_arb_attribs_length, 
//GO(xcb_glx_create_context_attribs_arb_checked, 
//GO(xcb_glx_create_context_attribs_arb_sizeof, 
//GO(xcb_glx_create_context_checked, 
//GO(xcb_glx_create_glx_pixmap, 
//GO(xcb_glx_create_glx_pixmap_checked, 
//GO(xcb_glx_create_new_context, 
//GO(xcb_glx_create_new_context_checked, 
//GO(xcb_glx_create_pbuffer, 
//GO(xcb_glx_create_pbuffer_attribs, 
//GO(xcb_glx_create_pbuffer_attribs_end, 
//GO(xcb_glx_create_pbuffer_attribs_length, 
//GO(xcb_glx_create_pbuffer_checked, 
//GO(xcb_glx_create_pbuffer_sizeof, 
//GO(xcb_glx_create_pixmap, 
//GO(xcb_glx_create_pixmap_attribs, 
//GO(xcb_glx_create_pixmap_attribs_end, 
//GO(xcb_glx_create_pixmap_attribs_length, 
//GO(xcb_glx_create_pixmap_checked, 
//GO(xcb_glx_create_pixmap_sizeof, 
//GO(xcb_glx_create_window, 
//GO(xcb_glx_create_window_attribs, 
//GO(xcb_glx_create_window_attribs_end, 
//GO(xcb_glx_create_window_attribs_length, 
//GO(xcb_glx_create_window_checked, 
//GO(xcb_glx_create_window_sizeof, 
//GO(xcb_glx_delete_lists, 
//GO(xcb_glx_delete_lists_checked, 
//GO(xcb_glx_delete_queries_arb, 
//GO(xcb_glx_delete_queries_arb_checked, 
//GO(xcb_glx_delete_queries_arb_ids, 
//GO(xcb_glx_delete_queries_arb_ids_end, 
//GO(xcb_glx_delete_queries_arb_ids_length, 
//GO(xcb_glx_delete_queries_arb_sizeof, 
//GO(xcb_glx_delete_textures, 
//GO(xcb_glx_delete_textures_checked, 
//GO(xcb_glx_delete_textures_sizeof, 
//GO(xcb_glx_delete_textures_textures, 
//GO(xcb_glx_delete_textures_textures_end, 
//GO(xcb_glx_delete_textures_textures_length, 
//GO(xcb_glx_delete_window, 
//GO(xcb_glx_delete_window_checked, 
//GO(xcb_glx_destroy_context, 
//GO(xcb_glx_destroy_context_checked, 
//GO(xcb_glx_destroy_glx_pixmap, 
//GO(xcb_glx_destroy_glx_pixmap_checked, 
//GO(xcb_glx_destroy_pbuffer, 
//GO(xcb_glx_destroy_pbuffer_checked, 
//GO(xcb_glx_destroy_pixmap, 
//GO(xcb_glx_destroy_pixmap_checked, 
//GO(xcb_glx_drawable_end, 
//GO(xcb_glx_drawable_next, 
//GO(xcb_glx_end_list, 
//GO(xcb_glx_end_list_checked, 
//GO(xcb_glx_fbconfig_end, 
//GO(xcb_glx_fbconfig_next, 
//GO(xcb_glx_feedback_buffer, 
//GO(xcb_glx_feedback_buffer_checked, 
//GO(xcb_glx_finish, 
//GO(xcb_glx_finish_reply, 
//GO(xcb_glx_finish_unchecked, 
//GO(xcb_glx_float32_end, 
//GO(xcb_glx_float32_next, 
//GO(xcb_glx_float64_end, 
//GO(xcb_glx_float64_next, 
//GO(xcb_glx_flush, 
//GO(xcb_glx_flush_checked, 
//GO(xcb_glx_gen_lists, 
//GO(xcb_glx_gen_lists_reply, 
//GO(xcb_glx_gen_lists_unchecked, 
//GO(xcb_glx_gen_queries_arb, 
//GO(xcb_glx_gen_queries_arb_data, 
//GO(xcb_glx_gen_queries_arb_data_end, 
//GO(xcb_glx_gen_queries_arb_data_length, 
//GO(xcb_glx_gen_queries_arb_reply, 
//GO(xcb_glx_gen_queries_arb_sizeof, 
//GO(xcb_glx_gen_queries_arb_unchecked, 
//GO(xcb_glx_gen_textures, 
//GO(xcb_glx_gen_textures_data, 
//GO(xcb_glx_gen_textures_data_end, 
//GO(xcb_glx_gen_textures_data_length, 
//GO(xcb_glx_gen_textures_reply, 
//GO(xcb_glx_gen_textures_sizeof, 
//GO(xcb_glx_gen_textures_unchecked, 
//GO(xcb_glx_get_booleanv, 
//GO(xcb_glx_get_booleanv_data, 
//GO(xcb_glx_get_booleanv_data_end, 
//GO(xcb_glx_get_booleanv_data_length, 
//GO(xcb_glx_get_booleanv_reply, 
//GO(xcb_glx_get_booleanv_sizeof, 
//GO(xcb_glx_get_booleanv_unchecked, 
//GO(xcb_glx_get_clip_plane, 
//GO(xcb_glx_get_clip_plane_data, 
//GO(xcb_glx_get_clip_plane_data_end, 
//GO(xcb_glx_get_clip_plane_data_length, 
//GO(xcb_glx_get_clip_plane_reply, 
//GO(xcb_glx_get_clip_plane_sizeof, 
//GO(xcb_glx_get_clip_plane_unchecked, 
//GO(xcb_glx_get_color_table, 
//GO(xcb_glx_get_color_table_data, 
//GO(xcb_glx_get_color_table_data_end, 
//GO(xcb_glx_get_color_table_data_length, 
//GO(xcb_glx_get_color_table_parameterfv, 
//GO(xcb_glx_get_color_table_parameterfv_data, 
//GO(xcb_glx_get_color_table_parameterfv_data_end, 
//GO(xcb_glx_get_color_table_parameterfv_data_length, 
//GO(xcb_glx_get_color_table_parameterfv_reply, 
//GO(xcb_glx_get_color_table_parameterfv_sizeof, 
//GO(xcb_glx_get_color_table_parameterfv_unchecked, 
//GO(xcb_glx_get_color_table_parameteriv, 
//GO(xcb_glx_get_color_table_parameteriv_data, 
//GO(xcb_glx_get_color_table_parameteriv_data_end, 
//GO(xcb_glx_get_color_table_parameteriv_data_length, 
//GO(xcb_glx_get_color_table_parameteriv_reply, 
//GO(xcb_glx_get_color_table_parameteriv_sizeof, 
//GO(xcb_glx_get_color_table_parameteriv_unchecked, 
//GO(xcb_glx_get_color_table_reply, 
//GO(xcb_glx_get_color_table_sizeof, 
//GO(xcb_glx_get_color_table_unchecked, 
//GO(xcb_glx_get_compressed_tex_image_arb, 
//GO(xcb_glx_get_compressed_tex_image_arb_data, 
//GO(xcb_glx_get_compressed_tex_image_arb_data_end, 
//GO(xcb_glx_get_compressed_tex_image_arb_data_length, 
//GO(xcb_glx_get_compressed_tex_image_arb_reply, 
//GO(xcb_glx_get_compressed_tex_image_arb_sizeof, 
//GO(xcb_glx_get_compressed_tex_image_arb_unchecked, 
//GO(xcb_glx_get_convolution_filter, 
//GO(xcb_glx_get_convolution_filter_data, 
//GO(xcb_glx_get_convolution_filter_data_end, 
//GO(xcb_glx_get_convolution_filter_data_length, 
//GO(xcb_glx_get_convolution_filter_reply, 
//GO(xcb_glx_get_convolution_filter_sizeof, 
//GO(xcb_glx_get_convolution_filter_unchecked, 
//GO(xcb_glx_get_convolution_parameterfv, 
//GO(xcb_glx_get_convolution_parameterfv_data, 
//GO(xcb_glx_get_convolution_parameterfv_data_end, 
//GO(xcb_glx_get_convolution_parameterfv_data_length, 
//GO(xcb_glx_get_convolution_parameterfv_reply, 
//GO(xcb_glx_get_convolution_parameterfv_sizeof, 
//GO(xcb_glx_get_convolution_parameterfv_unchecked, 
//GO(xcb_glx_get_convolution_parameteriv, 
//GO(xcb_glx_get_convolution_parameteriv_data, 
//GO(xcb_glx_get_convolution_parameteriv_data_end, 
//GO(xcb_glx_get_convolution_parameteriv_data_length, 
//GO(xcb_glx_get_convolution_parameteriv_reply, 
//GO(xcb_glx_get_convolution_parameteriv_sizeof, 
//GO(xcb_glx_get_convolution_parameteriv_unchecked, 
//GO(xcb_glx_get_doublev, 
//GO(xcb_glx_get_doublev_data, 
//GO(xcb_glx_get_doublev_data_end, 
//GO(xcb_glx_get_doublev_data_length, 
//GO(xcb_glx_get_doublev_reply, 
//GO(xcb_glx_get_doublev_sizeof, 
//GO(xcb_glx_get_doublev_unchecked, 
//GO(xcb_glx_get_drawable_attributes, 
//GO(xcb_glx_get_drawable_attributes_attribs, 
//GO(xcb_glx_get_drawable_attributes_attribs_end, 
//GO(xcb_glx_get_drawable_attributes_attribs_length, 
//GO(xcb_glx_get_drawable_attributes_reply, 
//GO(xcb_glx_get_drawable_attributes_sizeof, 
//GO(xcb_glx_get_drawable_attributes_unchecked, 
//GO(xcb_glx_get_error, 
//GO(xcb_glx_get_error_reply, 
//GO(xcb_glx_get_error_unchecked, 
//GO(xcb_glx_get_fb_configs, 
//GO(xcb_glx_get_fb_configs_property_list, 
//GO(xcb_glx_get_fb_configs_property_list_end, 
//GO(xcb_glx_get_fb_configs_property_list_length, 
//GO(xcb_glx_get_fb_configs_reply, 
//GO(xcb_glx_get_fb_configs_sizeof, 
//GO(xcb_glx_get_fb_configs_unchecked, 
//GO(xcb_glx_get_floatv, 
//GO(xcb_glx_get_floatv_data, 
//GO(xcb_glx_get_floatv_data_end, 
//GO(xcb_glx_get_floatv_data_length, 
//GO(xcb_glx_get_floatv_reply, 
//GO(xcb_glx_get_floatv_sizeof, 
//GO(xcb_glx_get_floatv_unchecked, 
//GO(xcb_glx_get_histogram, 
//GO(xcb_glx_get_histogram_data, 
//GO(xcb_glx_get_histogram_data_end, 
//GO(xcb_glx_get_histogram_data_length, 
//GO(xcb_glx_get_histogram_parameterfv, 
//GO(xcb_glx_get_histogram_parameterfv_data, 
//GO(xcb_glx_get_histogram_parameterfv_data_end, 
//GO(xcb_glx_get_histogram_parameterfv_data_length, 
//GO(xcb_glx_get_histogram_parameterfv_reply, 
//GO(xcb_glx_get_histogram_parameterfv_sizeof, 
//GO(xcb_glx_get_histogram_parameterfv_unchecked, 
//GO(xcb_glx_get_histogram_parameteriv, 
//GO(xcb_glx_get_histogram_parameteriv_data, 
//GO(xcb_glx_get_histogram_parameteriv_data_end, 
//GO(xcb_glx_get_histogram_parameteriv_data_length, 
//GO(xcb_glx_get_histogram_parameteriv_reply, 
//GO(xcb_glx_get_histogram_parameteriv_sizeof, 
//GO(xcb_glx_get_histogram_parameteriv_unchecked, 
//GO(xcb_glx_get_histogram_reply, 
//GO(xcb_glx_get_histogram_sizeof, 
//GO(xcb_glx_get_histogram_unchecked, 
//GO(xcb_glx_get_integerv, 
//GO(xcb_glx_get_integerv_data, 
//GO(xcb_glx_get_integerv_data_end, 
//GO(xcb_glx_get_integerv_data_length, 
//GO(xcb_glx_get_integerv_reply, 
//GO(xcb_glx_get_integerv_sizeof, 
//GO(xcb_glx_get_integerv_unchecked, 
//GO(xcb_glx_get_lightfv, 
//GO(xcb_glx_get_lightfv_data, 
//GO(xcb_glx_get_lightfv_data_end, 
//GO(xcb_glx_get_lightfv_data_length, 
//GO(xcb_glx_get_lightfv_reply, 
//GO(xcb_glx_get_lightfv_sizeof, 
//GO(xcb_glx_get_lightfv_unchecked, 
//GO(xcb_glx_get_lightiv, 
//GO(xcb_glx_get_lightiv_data, 
//GO(xcb_glx_get_lightiv_data_end, 
//GO(xcb_glx_get_lightiv_data_length, 
//GO(xcb_glx_get_lightiv_reply, 
//GO(xcb_glx_get_lightiv_sizeof, 
//GO(xcb_glx_get_lightiv_unchecked, 
//GO(xcb_glx_get_mapdv, 
//GO(xcb_glx_get_mapdv_data, 
//GO(xcb_glx_get_mapdv_data_end, 
//GO(xcb_glx_get_mapdv_data_length, 
//GO(xcb_glx_get_mapdv_reply, 
//GO(xcb_glx_get_mapdv_sizeof, 
//GO(xcb_glx_get_mapdv_unchecked, 
//GO(xcb_glx_get_mapfv, 
//GO(xcb_glx_get_mapfv_data, 
//GO(xcb_glx_get_mapfv_data_end, 
//GO(xcb_glx_get_mapfv_data_length, 
//GO(xcb_glx_get_mapfv_reply, 
//GO(xcb_glx_get_mapfv_sizeof, 
//GO(xcb_glx_get_mapfv_unchecked, 
//GO(xcb_glx_get_mapiv, 
//GO(xcb_glx_get_mapiv_data, 
//GO(xcb_glx_get_mapiv_data_end, 
//GO(xcb_glx_get_mapiv_data_length, 
//GO(xcb_glx_get_mapiv_reply, 
//GO(xcb_glx_get_mapiv_sizeof, 
//GO(xcb_glx_get_mapiv_unchecked, 
//GO(xcb_glx_get_materialfv, 
//GO(xcb_glx_get_materialfv_data, 
//GO(xcb_glx_get_materialfv_data_end, 
//GO(xcb_glx_get_materialfv_data_length, 
//GO(xcb_glx_get_materialfv_reply, 
//GO(xcb_glx_get_materialfv_sizeof, 
//GO(xcb_glx_get_materialfv_unchecked, 
//GO(xcb_glx_get_materialiv, 
//GO(xcb_glx_get_materialiv_data, 
//GO(xcb_glx_get_materialiv_data_end, 
//GO(xcb_glx_get_materialiv_data_length, 
//GO(xcb_glx_get_materialiv_reply, 
//GO(xcb_glx_get_materialiv_sizeof, 
//GO(xcb_glx_get_materialiv_unchecked, 
//GO(xcb_glx_get_minmax, 
//GO(xcb_glx_get_minmax_data, 
//GO(xcb_glx_get_minmax_data_end, 
//GO(xcb_glx_get_minmax_data_length, 
//GO(xcb_glx_get_minmax_parameterfv, 
//GO(xcb_glx_get_minmax_parameterfv_data, 
//GO(xcb_glx_get_minmax_parameterfv_data_end, 
//GO(xcb_glx_get_minmax_parameterfv_data_length, 
//GO(xcb_glx_get_minmax_parameterfv_reply, 
//GO(xcb_glx_get_minmax_parameterfv_sizeof, 
//GO(xcb_glx_get_minmax_parameterfv_unchecked, 
//GO(xcb_glx_get_minmax_parameteriv, 
//GO(xcb_glx_get_minmax_parameteriv_data, 
//GO(xcb_glx_get_minmax_parameteriv_data_end, 
//GO(xcb_glx_get_minmax_parameteriv_data_length, 
//GO(xcb_glx_get_minmax_parameteriv_reply, 
//GO(xcb_glx_get_minmax_parameteriv_sizeof, 
//GO(xcb_glx_get_minmax_parameteriv_unchecked, 
//GO(xcb_glx_get_minmax_reply, 
//GO(xcb_glx_get_minmax_sizeof, 
//GO(xcb_glx_get_minmax_unchecked, 
//GO(xcb_glx_get_pixel_mapfv, 
//GO(xcb_glx_get_pixel_mapfv_data, 
//GO(xcb_glx_get_pixel_mapfv_data_end, 
//GO(xcb_glx_get_pixel_mapfv_data_length, 
//GO(xcb_glx_get_pixel_mapfv_reply, 
//GO(xcb_glx_get_pixel_mapfv_sizeof, 
//GO(xcb_glx_get_pixel_mapfv_unchecked, 
//GO(xcb_glx_get_pixel_mapuiv, 
//GO(xcb_glx_get_pixel_mapuiv_data, 
//GO(xcb_glx_get_pixel_mapuiv_data_end, 
//GO(xcb_glx_get_pixel_mapuiv_data_length, 
//GO(xcb_glx_get_pixel_mapuiv_reply, 
//GO(xcb_glx_get_pixel_mapuiv_sizeof, 
//GO(xcb_glx_get_pixel_mapuiv_unchecked, 
//GO(xcb_glx_get_pixel_mapusv, 
//GO(xcb_glx_get_pixel_mapusv_data, 
//GO(xcb_glx_get_pixel_mapusv_data_end, 
//GO(xcb_glx_get_pixel_mapusv_data_length, 
//GO(xcb_glx_get_pixel_mapusv_reply, 
//GO(xcb_glx_get_pixel_mapusv_sizeof, 
//GO(xcb_glx_get_pixel_mapusv_unchecked, 
//GO(xcb_glx_get_polygon_stipple, 
//GO(xcb_glx_get_polygon_stipple_data, 
//GO(xcb_glx_get_polygon_stipple_data_end, 
//GO(xcb_glx_get_polygon_stipple_data_length, 
//GO(xcb_glx_get_polygon_stipple_reply, 
//GO(xcb_glx_get_polygon_stipple_sizeof, 
//GO(xcb_glx_get_polygon_stipple_unchecked, 
//GO(xcb_glx_get_queryiv_arb, 
//GO(xcb_glx_get_queryiv_arb_data, 
//GO(xcb_glx_get_queryiv_arb_data_end, 
//GO(xcb_glx_get_queryiv_arb_data_length, 
//GO(xcb_glx_get_queryiv_arb_reply, 
//GO(xcb_glx_get_queryiv_arb_sizeof, 
//GO(xcb_glx_get_queryiv_arb_unchecked, 
//GO(xcb_glx_get_query_objectiv_arb, 
//GO(xcb_glx_get_query_objectiv_arb_data, 
//GO(xcb_glx_get_query_objectiv_arb_data_end, 
//GO(xcb_glx_get_query_objectiv_arb_data_length, 
//GO(xcb_glx_get_query_objectiv_arb_reply, 
//GO(xcb_glx_get_query_objectiv_arb_sizeof, 
//GO(xcb_glx_get_query_objectiv_arb_unchecked, 
//GO(xcb_glx_get_query_objectuiv_arb, 
//GO(xcb_glx_get_query_objectuiv_arb_data, 
//GO(xcb_glx_get_query_objectuiv_arb_data_end, 
//GO(xcb_glx_get_query_objectuiv_arb_data_length, 
//GO(xcb_glx_get_query_objectuiv_arb_reply, 
//GO(xcb_glx_get_query_objectuiv_arb_sizeof, 
//GO(xcb_glx_get_query_objectuiv_arb_unchecked, 
//GO(xcb_glx_get_separable_filter, 
//GO(xcb_glx_get_separable_filter_reply, 
//GO(xcb_glx_get_separable_filter_rows_and_cols, 
//GO(xcb_glx_get_separable_filter_rows_and_cols_end, 
//GO(xcb_glx_get_separable_filter_rows_and_cols_length, 
//GO(xcb_glx_get_separable_filter_sizeof, 
//GO(xcb_glx_get_separable_filter_unchecked, 
//GO(xcb_glx_get_string, 
//GO(xcb_glx_get_string_reply, 
//GO(xcb_glx_get_string_sizeof, 
//GO(xcb_glx_get_string_string, 
//GO(xcb_glx_get_string_string_end, 
//GO(xcb_glx_get_string_string_length, 
//GO(xcb_glx_get_string_unchecked, 
//GO(xcb_glx_get_tex_envfv, 
//GO(xcb_glx_get_tex_envfv_data, 
//GO(xcb_glx_get_tex_envfv_data_end, 
//GO(xcb_glx_get_tex_envfv_data_length, 
//GO(xcb_glx_get_tex_envfv_reply, 
//GO(xcb_glx_get_tex_envfv_sizeof, 
//GO(xcb_glx_get_tex_envfv_unchecked, 
//GO(xcb_glx_get_tex_enviv, 
//GO(xcb_glx_get_tex_enviv_data, 
//GO(xcb_glx_get_tex_enviv_data_end, 
//GO(xcb_glx_get_tex_enviv_data_length, 
//GO(xcb_glx_get_tex_enviv_reply, 
//GO(xcb_glx_get_tex_enviv_sizeof, 
//GO(xcb_glx_get_tex_enviv_unchecked, 
//GO(xcb_glx_get_tex_gendv, 
//GO(xcb_glx_get_tex_gendv_data, 
//GO(xcb_glx_get_tex_gendv_data_end, 
//GO(xcb_glx_get_tex_gendv_data_length, 
//GO(xcb_glx_get_tex_gendv_reply, 
//GO(xcb_glx_get_tex_gendv_sizeof, 
//GO(xcb_glx_get_tex_gendv_unchecked, 
//GO(xcb_glx_get_tex_genfv, 
//GO(xcb_glx_get_tex_genfv_data, 
//GO(xcb_glx_get_tex_genfv_data_end, 
//GO(xcb_glx_get_tex_genfv_data_length, 
//GO(xcb_glx_get_tex_genfv_reply, 
//GO(xcb_glx_get_tex_genfv_sizeof, 
//GO(xcb_glx_get_tex_genfv_unchecked, 
//GO(xcb_glx_get_tex_geniv, 
//GO(xcb_glx_get_tex_geniv_data, 
//GO(xcb_glx_get_tex_geniv_data_end, 
//GO(xcb_glx_get_tex_geniv_data_length, 
//GO(xcb_glx_get_tex_geniv_reply, 
//GO(xcb_glx_get_tex_geniv_sizeof, 
//GO(xcb_glx_get_tex_geniv_unchecked, 
//GO(xcb_glx_get_tex_image, 
//GO(xcb_glx_get_tex_image_data, 
//GO(xcb_glx_get_tex_image_data_end, 
//GO(xcb_glx_get_tex_image_data_length, 
//GO(xcb_glx_get_tex_image_reply, 
//GO(xcb_glx_get_tex_image_sizeof, 
//GO(xcb_glx_get_tex_image_unchecked, 
//GO(xcb_glx_get_tex_level_parameterfv, 
//GO(xcb_glx_get_tex_level_parameterfv_data, 
//GO(xcb_glx_get_tex_level_parameterfv_data_end, 
//GO(xcb_glx_get_tex_level_parameterfv_data_length, 
//GO(xcb_glx_get_tex_level_parameterfv_reply, 
//GO(xcb_glx_get_tex_level_parameterfv_sizeof, 
//GO(xcb_glx_get_tex_level_parameterfv_unchecked, 
//GO(xcb_glx_get_tex_level_parameteriv, 
//GO(xcb_glx_get_tex_level_parameteriv_data, 
//GO(xcb_glx_get_tex_level_parameteriv_data_end, 
//GO(xcb_glx_get_tex_level_parameteriv_data_length, 
//GO(xcb_glx_get_tex_level_parameteriv_reply, 
//GO(xcb_glx_get_tex_level_parameteriv_sizeof, 
//GO(xcb_glx_get_tex_level_parameteriv_unchecked, 
//GO(xcb_glx_get_tex_parameterfv, 
//GO(xcb_glx_get_tex_parameterfv_data, 
//GO(xcb_glx_get_tex_parameterfv_data_end, 
//GO(xcb_glx_get_tex_parameterfv_data_length, 
//GO(xcb_glx_get_tex_parameterfv_reply, 
//GO(xcb_glx_get_tex_parameterfv_sizeof, 
//GO(xcb_glx_get_tex_parameterfv_unchecked, 
//GO(xcb_glx_get_tex_parameteriv, 
//GO(xcb_glx_get_tex_parameteriv_data, 
//GO(xcb_glx_get_tex_parameteriv_data_end, 
//GO(xcb_glx_get_tex_parameteriv_data_length, 
//GO(xcb_glx_get_tex_parameteriv_reply, 
//GO(xcb_glx_get_tex_parameteriv_sizeof, 
//GO(xcb_glx_get_tex_parameteriv_unchecked, 
//GO(xcb_glx_get_visual_configs, 
//GO(xcb_glx_get_visual_configs_property_list, 
//GO(xcb_glx_get_visual_configs_property_list_end, 
//GO(xcb_glx_get_visual_configs_property_list_length, 
//GO(xcb_glx_get_visual_configs_reply, 
//GO(xcb_glx_get_visual_configs_sizeof, 
//GO(xcb_glx_get_visual_configs_unchecked, 
DATA(xcb_glx_id, 8)
//GO(xcb_glx_is_direct, 
//GO(xcb_glx_is_direct_reply, 
//GO(xcb_glx_is_direct_unchecked, 
//GO(xcb_glx_is_enabled, 
//GO(xcb_glx_is_enabled_reply, 
//GO(xcb_glx_is_enabled_unchecked, 
//GO(xcb_glx_is_list, 
//GO(xcb_glx_is_list_reply, 
//GO(xcb_glx_is_list_unchecked, 
//GO(xcb_glx_is_query_arb, 
//GO(xcb_glx_is_query_arb_reply, 
//GO(xcb_glx_is_query_arb_unchecked, 
//GO(xcb_glx_is_texture, 
//GO(xcb_glx_is_texture_reply, 
//GO(xcb_glx_is_texture_unchecked, 
//GO(xcb_glx_make_context_current, 
//GO(xcb_glx_make_context_current_reply, 
//GO(xcb_glx_make_context_current_unchecked, 
//GO(xcb_glx_make_current, 
//GO(xcb_glx_make_current_reply, 
//GO(xcb_glx_make_current_unchecked, 
//GO(xcb_glx_new_list, 
//GO(xcb_glx_new_list_checked, 
//GO(xcb_glx_pbuffer_end, 
//GO(xcb_glx_pbuffer_next, 
//GO(xcb_glx_pixel_storef, 
//GO(xcb_glx_pixel_storef_checked, 
//GO(xcb_glx_pixel_storei, 
//GO(xcb_glx_pixel_storei_checked, 
//GO(xcb_glx_pixmap_end, 
//GO(xcb_glx_pixmap_next, 
//GO(xcb_glx_query_context, 
//GO(xcb_glx_query_context_attribs, 
//GO(xcb_glx_query_context_attribs_end, 
//GO(xcb_glx_query_context_attribs_length, 
//GO(xcb_glx_query_context_reply, 
//GO(xcb_glx_query_context_sizeof, 
//GO(xcb_glx_query_context_unchecked, 
//GO(xcb_glx_query_extensions_string, 
//GO(xcb_glx_query_extensions_string_reply, 
//GO(xcb_glx_query_extensions_string_unchecked, 
//GO(xcb_glx_query_server_string, 
//GO(xcb_glx_query_server_string_reply, 
//GO(xcb_glx_query_server_string_sizeof, 
//GO(xcb_glx_query_server_string_string, 
//GO(xcb_glx_query_server_string_string_end, 
//GO(xcb_glx_query_server_string_string_length, 
//GO(xcb_glx_query_server_string_unchecked, 
GO(xcb_glx_query_version, UFbuu)
GO(xcb_glx_query_version_reply, pFbup)
GO(xcb_glx_query_version_unchecked, UFbuu)
//GO(xcb_glx_read_pixels, 
//GO(xcb_glx_read_pixels_data, 
//GO(xcb_glx_read_pixels_data_end, 
//GO(xcb_glx_read_pixels_data_length, 
//GO(xcb_glx_read_pixels_reply, 
//GO(xcb_glx_read_pixels_sizeof, 
//GO(xcb_glx_read_pixels_unchecked, 
//GO(xcb_glx_render, 
//GO(xcb_glx_render_checked, 
//GO(xcb_glx_render_data, 
//GO(xcb_glx_render_data_end, 
//GO(xcb_glx_render_data_length, 
//GO(xcb_glx_render_large, 
//GO(xcb_glx_render_large_checked, 
//GO(xcb_glx_render_large_data, 
//GO(xcb_glx_render_large_data_end, 
//GO(xcb_glx_render_large_data_length, 
//GO(xcb_glx_render_large_sizeof, 
//GO(xcb_glx_render_mode, 
//GO(xcb_glx_render_mode_data, 
//GO(xcb_glx_render_mode_data_end, 
//GO(xcb_glx_render_mode_data_length, 
//GO(xcb_glx_render_mode_reply, 
//GO(xcb_glx_render_mode_sizeof, 
//GO(xcb_glx_render_mode_unchecked, 
//GO(xcb_glx_render_sizeof, 
//GO(xcb_glx_select_buffer, 
//GO(xcb_glx_select_buffer_checked, 
//GO(xcb_glx_set_client_info_2arb, 
//GO(xcb_glx_set_client_info_2arb_checked, 
//GO(xcb_glx_set_client_info_2arb_gl_extension_string, 
//GO(xcb_glx_set_client_info_2arb_gl_extension_string_end, 
//GO(xcb_glx_set_client_info_2arb_gl_extension_string_length, 
//GO(xcb_glx_set_client_info_2arb_gl_versions, 
//GO(xcb_glx_set_client_info_2arb_gl_versions_end, 
//GO(xcb_glx_set_client_info_2arb_gl_versions_length, 
//GO(xcb_glx_set_client_info_2arb_glx_extension_string, 
//GO(xcb_glx_set_client_info_2arb_glx_extension_string_end, 
//GO(xcb_glx_set_client_info_2arb_glx_extension_string_length, 
//GO(xcb_glx_set_client_info_2arb_sizeof, 
//GO(xcb_glx_set_client_info_arb, 
//GO(xcb_glx_set_client_info_arb_checked, 
//GO(xcb_glx_set_client_info_arb_gl_extension_string, 
//GO(xcb_glx_set_client_info_arb_gl_extension_string_end, 
//GO(xcb_glx_set_client_info_arb_gl_extension_string_length, 
//GO(xcb_glx_set_client_info_arb_gl_versions, 
//GO(xcb_glx_set_client_info_arb_gl_versions_end, 
//GO(xcb_glx_set_client_info_arb_gl_versions_length, 
//GO(xcb_glx_set_client_info_arb_glx_extension_string, 
//GO(xcb_glx_set_client_info_arb_glx_extension_string_end, 
//GO(xcb_glx_set_client_info_arb_glx_extension_string_length, 
//GO(xcb_glx_set_client_info_arb_sizeof, 
//GO(xcb_glx_swap_buffers, 
//GO(xcb_glx_swap_buffers_checked, 
//GO(xcb_glx_use_x_font, 
//GO(xcb_glx_use_x_font_checked, 
//GO(xcb_glx_vendor_private, 
//GO(xcb_glx_vendor_private_checked, 
//GO(xcb_glx_vendor_private_data, 
//GO(xcb_glx_vendor_private_data_end, 
//GO(xcb_glx_vendor_private_data_length, 
//GO(xcb_glx_vendor_private_sizeof, 
//GO(xcb_glx_vendor_private_with_reply, 
//GO(xcb_glx_vendor_private_with_reply_data_2, 
//GO(xcb_glx_vendor_private_with_reply_data_2_end, 
//GO(xcb_glx_vendor_private_with_reply_data_2_length, 
//GO(xcb_glx_vendor_private_with_reply_reply, 
//GO(xcb_glx_vendor_private_with_reply_sizeof, 
//GO(xcb_glx_vendor_private_with_reply_unchecked, 
//GO(xcb_glx_wait_gl, 
//GO(xcb_glx_wait_gl_checked, 
//GO(xcb_glx_wait_x, 
//GO(xcb_glx_wait_x_checked, 
//GO(xcb_glx_window_end, 
//GO(xcb_glx_window_next, 
