#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(asn1_array2tree, iFppp)
GO(asn1_bit_der, vFpipp)
GO(asn1_check_version, pFp)
GO(asn1_copy_node, iFpppp)
GO(asn1_create_element, iFppp)
GO(asn1_decode_simple_ber, iFupuppp)
GO(asn1_decode_simple_der, iFupupp)
GO(asn1_delete_element, iFpp)
GO(asn1_delete_structure, iFp)
GO(asn1_delete_structure2, iFpu)
GO(asn1_der_coding, iFppppp)
GO(asn1_der_decoding, iFppip)
GO(asn1_der_decoding2, iFpppup)
GO(asn1_der_decoding_element, iFpppip)
GO(asn1_der_decoding_startEnd, iFppippp)
GO(asn1_dup_node, pFpp)
GO(asn1_encode_simple_der, iFupupp)
GO(asn1_expand_any_defined_by, iFpp)
GO(asn1_expand_octet_string, iFpppp)
GO(asn1_find_node, pFpp)
GO(asn1_find_structure_from_oid, pFpp)
GO(asn1_get_bit_der, iFpippip)
GO(asn1_get_length_ber, lFpip)
GO(asn1_get_length_der, lFpip)
GO(asn1_get_object_id_der, iFpippi)
GO(asn1_get_octet_der, iFpippip)
GO(asn1_get_tag_der, iFpippp)
GO(asn1_length_der, vFLpp)
GO(asn1_number_of_elements, iFppp)
GO(asn1_object_id_der, iFpppu)
GO(asn1_octet_der, vFpipp)
GO(asn1_parser2array, iFpppp)
GO(asn1_parser2tree, iFppp)
GO(asn1_perror, vFi)
GO(asn1_print_structure, vFSppi)
GO(asn1_read_node_value, iFpp)
GO(asn1_read_tag, iFpppp)
GO(asn1_read_value, iFpppp)
GO(asn1_read_value_type, iFppppp)
GO(asn1_strerror, pFi)
GO(asn1_write_value, iFpppi)

