#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GOM(new_fieldtype, pFPP)
GO(link_fieldtype, pFpp)
GO(free_fieldtype, iFp)
//GOM(set_fieldtype_arg, iFpPPP)
//GOM(set_fieldtype_choice, iFpPP)
GO(new_field, pFiiiiii)
GO(dup_field, pFpii)
GO(link_field, pFpii)
GO(free_field, iFp)
GO(field_info, iFppppppp)
GO(dynamic_field_info, iFpppp)
GO(set_max_field, iFpi)
GO(move_field, iFpii)
//GO(set_field_type, iFpp,...)
GO(set_new_page, iFpi)
GO(set_field_just, iFpi)
GO(field_just, iFp)
GO(set_field_fore, iFpL)
GO(set_field_back, iFpL)
GO(set_field_pad, iFpi)
GO(field_pad, iFp)
GO(set_field_buffer, iFpip)
GO(set_field_status, iFpi)
GO(set_field_userptr, iFpp)
GO(set_field_opts, iFpi)
GO(field_opts_on, iFpi)
GO(field_opts_off, iFpi)
GO(field_fore, LFp)
GO(field_back, LFp)
GO(new_page, iFp)
GO(field_status, iFp)
GO(field_arg, pFp)
GO(field_userptr, pFp)
GO(field_type, pFp)
GO(field_buffer, pFpi)
GO(field_opts, iFp)
GO(new_form, pFp)
GO(form_fields, pFp)
GO(current_field, pFp)
GO(form_win, pFp)
GO(form_sub, pFp)
//GOM(form_init, PFp)
//GOM(form_term, PFp)
//GOM(field_init, PFp)
//GOM(field_term, PFp)
GO(free_form, iFp)
GO(set_form_fields, iFpp)
GO(field_count, iFp)
GO(set_form_win, iFpp)
GO(set_form_sub, iFpp)
GO(set_current_field, iFpp)
GO(unfocus_current_field, iFp)
GO(field_index, iFp)
GO(set_form_page, iFpi)
GO(form_page, iFp)
GO(scale_form, iFppp)
//GOM(set_form_init, iFpP)
//GOM(set_form_term, iFpP)
//GOM(set_field_init, iFpP)
//GOM(set_field_term, iFpP)
GO(post_form, iFp)
GO(unpost_form, iFp)
GO(pos_form_cursor, iFp)
GO(form_driver, iFpi)
GO(form_driver_w, iFpiu)
GO(set_form_userptr, iFpp)
GO(set_form_opts, iFpi)
GO(form_opts_on, iFpi)
GO(form_opts_off, iFpi)
GO(form_request_by_name, iFp)
GO(form_request_name, pFi)
GO(form_userptr, pFp)
GO(form_opts, iFp)
GO(data_ahead, iFp)
GO(data_behind, iFp)
GO(new_form_sp, pFpp)
