#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//DATA(dbus_menu_clean_xml, 
GOM(dbusmenu_client_add_type_handler, iFEppp)
//GO(dbusmenu_client_add_type_handler_full, 
GO(dbusmenu_client_get_icon_paths, pFp)
GO(dbusmenu_client_get_root, pFp)
//GO(dbusmenu_client_get_status, 
//GO(dbusmenu_client_get_text_direction, 
GO(dbusmenu_client_get_type, pFv)
//GO(dbusmenu_client_menuitem_get_type, 
//GO(dbusmenu_client_menuitem_new, 
//GO(dbusmenu_client_new, 
//GO(dbusmenu_client_send_about_to_show, 
//GO(dbusmenu_client_send_event, 
//GO(dbusmenu_defaults_default_get, 
//GO(dbusmenu_defaults_default_get_type, 
//GO(dbusmenu_defaults_default_set, 
//GO(dbusmenu_defaults_get_type, 
//GO(dbusmenu_defaults_ref_default, 
//GO(dbusmenu_menuitem_build_variant, 
GO(dbusmenu_menuitem_child_add_position, iFppu)
GO(dbusmenu_menuitem_child_append, iFpp)
GO(dbusmenu_menuitem_child_delete, iFpp)
GO(dbusmenu_menuitem_child_find, pFpi)
GO(dbusmenu_menuitem_child_prepend, iFpp)
GO(dbusmenu_menuitem_child_reorder, iFppu)
//GO(dbusmenu_menuitem_exposed, 
GO(dbusmenu_menuitem_find_id, pFpi)
GOM(dbusmenu_menuitem_foreach, vFEppp)
GO(dbusmenu_menuitem_get_children, pFp)
GO(dbusmenu_menuitem_get_id, iFp)
GO(dbusmenu_menuitem_get_parent, pFp)
GO(dbusmenu_menuitem_get_position, uFpp)
GO(dbusmenu_menuitem_get_position_realized, uFpp)
GO(dbusmenu_menuitem_get_root, iFp)
GO(dbusmenu_menuitem_get_type, iFv)
GO(dbusmenu_menuitem_handle_event, vFpppu)
GO(dbusmenu_menuitem_new, pFv)
GO(dbusmenu_menuitem_new_with_id, pFi)
GO(dbusmenu_menuitem_properties_copy, pFp)
GO(dbusmenu_menuitem_properties_list, pFp)
//GO(dbusmenu_menuitem_properties_variant, 
GO(dbusmenu_menuitem_property_exist, iFpp)
GO(dbusmenu_menuitem_property_get, pFpp)
GO(dbusmenu_menuitem_property_get_bool, iFpp)
GO(dbusmenu_menuitem_property_get_byte_array, pFppp)
GO(dbusmenu_menuitem_property_get_int, iFpp)
GO(dbusmenu_menuitem_property_get_variant, pFpp)
//GO(dbusmenu_menuitem_property_is_default, 
GO(dbusmenu_menuitem_property_remove, vFpp)
GO(dbusmenu_menuitem_property_set, iFppp)
GO(dbusmenu_menuitem_property_set_bool, iFppi)
GO(dbusmenu_menuitem_property_set_byte_array, iFpppL)
GO(dbusmenu_menuitem_property_set_int, iFppi)
GO(dbusmenu_menuitem_property_set_variant, iFppp)
GO(dbusmenu_menuitem_proxy_get_type, iFv)
//GO(dbusmenu_menuitem_proxy_get_wrapped, 
//GO(dbusmenu_menuitem_proxy_new, 
//GO(dbusmenu_menuitem_realized, 
GOM(dbusmenu_menuitem_send_about_to_show, vFEppp)
//GO(dbusmenu_menuitem_set_parent, 
//GO(dbusmenu_menuitem_set_realized, 
GO(dbusmenu_menuitem_set_root, vFpi)
GO(dbusmenu_menuitem_show_to_user, vFpu)
GO(dbusmenu_menuitem_take_children, pFp)
//GO(dbusmenu_menuitem_unparent, 
GO(dbusmenu_server_get_icon_paths, pFp)
GO(dbusmenu_server_get_status, iFp)
GO(dbusmenu_server_get_text_direction, iFp)
GO(dbusmenu_server_get_type, iFv)
GO(dbusmenu_server_new, pFp)
GO(dbusmenu_server_set_icon_paths, vFpp)
GO(dbusmenu_server_set_root, vFpp)
GO(dbusmenu_server_set_status, vFpi)
GO(dbusmenu_server_set_text_direction, vFpi)
//GO(dbusmenu_status_get_nick, 
GO(dbusmenu_status_get_type, iFv)
//GO(dbusmenu_status_get_value_from_nick, 
//GO(dbusmenu_text_direction_get_nick, 
//GO(dbusmenu_text_direction_get_type, 
//GO(dbusmenu_text_direction_get_value_from_nick, 
