#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(XFixesChangeCursor, vFpLL)
GO(XFixesChangeCursorByName, vFpLp)
GO(XFixesChangeSaveSet, vFpLiii)
GO(XFixesCopyRegion, vFpLL)
GO(XFixesCreatePointerBarrier, LFpLiiiiiip)
GO(XFixesCreateRegion, LFppi)
GO(XFixesCreateRegionFromBitmap, LFpL)
GO(XFixesCreateRegionFromGC, LFpp)
GO(XFixesCreateRegionFromPicture, LFpL)
GO(XFixesCreateRegionFromWindow, LFpLi)
GO(XFixesDestroyPointerBarrier, vFpL)
GO(XFixesDestroyRegion, vFpL)
GO(XFixesExpandRegion, vFpLLuuuu)
DATA(XFixesExtensionInfo, sizeof(void*))    //B
DATA(XFixesExtensionName, sizeof(void*))    //D
GO(XFixesFetchRegion, pFpLp)
GO(XFixesFetchRegionAndBounds, pFpLpp)
GO(XFixesFindDisplay, pFp)
GO(XFixesGetCursorImage, pFp)
GO(XFixesGetCursorName, pFpLp)
GO(XFixesHideCursor, vFpL)
GO(XFixesIntersectRegion, vFpLLL)
GO(XFixesInvertRegion, vFpLpL)
GO(XFixesQueryExtension, iFppp)
GO(XFixesQueryVersion, iFppp)
GO(XFixesRegionExtents, vFpLL)
GO(XFixesSelectCursorInput, vFpLL)
GO(XFixesSelectSelectionInput, vFpLLL)
GO(XFixesSetCursorName, vFpLp)
GO(XFixesSetGCClipRegion, vFppiiL)
GO(XFixesSetPictureClipRegion, vFpLiiL)
GO(XFixesSetRegion, vFpLpi)
GO(XFixesSetWindowShapeRegion, vFpLiiiL)
GO(XFixesShowCursor, vFpL)
GO(XFixesSubtractRegion, vFpLLL)
GO(XFixesTranslateRegion, vFpLii)
GO(XFixesUnionRegion, vFpLLL)
GO(XFixesVersion, iFv)
