#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

// __glXGLLoadGLXFunction
GO(glXChooseFBConfig, pFpipp)
GO(glXChooseVisual, pFpip)
GO(glXCopyContext, vFpppL)
GO(glXCreateContext, pFpppi)
GO(glXCreateGLXPixmap, LFppL)
GO(glXCreateNewContext, pFppipi)
GO(glXCreatePbuffer, LFppp)
GO(glXCreatePixmap, LFppLp)
GO(glXCreateWindow, LFppLp)
GO(glXDestroyContext, vFpp)
GO(glXDestroyGLXPixmap, vFpL)
GO(glXDestroyPbuffer, vFpL)
GO(glXDestroyPixmap, vFpL)
GO(glXDestroyWindow, vFpL)
GO(glXGetClientString, pFpi)
GO(glXGetConfig, iFppip)
GO(glXGetCurrentContext, pFv)
GO(glXGetCurrentDisplay, pFv)
GO(glXGetCurrentDrawable, LFv)
GO(glXGetCurrentReadDrawable, LFv)
GO(glXGetFBConfigAttrib, iFppip)
GO(glXGetFBConfigs, pFpip)
GOM(glXGetProcAddress, pFEp)
GOM(glXGetProcAddressARB, pFEp)
GO(glXGetSelectedEvent, vFpLp)
GO(glXGetVisualFromFBConfig, pFpp)
GO(glXIsDirect, iFpp)
GO(glXMakeContextCurrent, iFpLLp)
GO(glXMakeCurrent, iFpLp)
GO(glXQueryContext, iFppip)
GO(glXQueryDrawable, vFpLip)
GO(glXQueryExtension, iFppp)
GO(glXQueryExtensionsString, pFpi)
GO(glXQueryServerString, pFpii)
GO(glXQueryVersion, iFppp)
GO(glXSelectEvent, vFpLL)
GO(glXSwapBuffers, vFpL)
GO(glXUseXFont, vFLiii)
GO(glXWaitGL, vFv)
GO(glXWaitX, vFv)



