#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GO(C_GetFunctionList, 
//GO(C_GetInterface, 
//GO(C_GetInterfaceList, 
//GO(FC_GetFunctionList, 
//GO(FC_GetInterface, 
//GO(FC_GetInterfaceList, 
//GO(NSC_GetFunctionList, 
//GO(NSC_GetInterface, 
//GO(NSC_GetInterfaceList, 
//GO(NSC_ModuleDBFunc, 
