#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

// __glXGLLoadGLXFunction
GOM(glXChooseFBConfig, pFEXipp)
GOM(glXChooseVisual, pFEXip)
GO(glXCopyContext, vFXppL)
GOM(glXCreateContext, pFEXppi)
//GOM(glXCreateGLXPixmap, LFEXbpLiiiLLLii_L)
GO(glXCreateNewContext, pFXpipi)
GO(glXCreatePbuffer, LFXpp)
GO(glXCreatePixmap, LFXpLp)
GO(glXCreateWindow, LFXpLp)
GO(glXDestroyContext, vFXp)
GO(glXDestroyGLXPixmap, vFXL)
GO(glXDestroyPbuffer, vFXL)
GO(glXDestroyPixmap, vFXL)
GO(glXDestroyWindow, vFXL)
GO(glXGetClientString, pFXi)
//GOM(glXGetConfig, iFEXbpLiiiLLLii_ip)
GO(glXGetCurrentContext, pFv)
GO(glXGetCurrentDisplay, XFv)
GO(glXGetCurrentDrawable, LFv)
GO(glXGetCurrentReadDrawable, LFv)
GO(glXGetFBConfigAttrib, iFXpip)
GOM(glXGetFBConfigs, pFEXip)
GOM(glXGetProcAddress, pFEp)
GOM(glXGetProcAddressARB, pFEp)
GO(glXGetSelectedEvent, vFXLbL_)
GOM(glXGetVisualFromFBConfig, pFEXp)
GO(glXIsDirect, iFXp)
GO(glXMakeContextCurrent, iFXLLp)
GO(glXMakeCurrent, iFXLp)
GO(glXQueryContext, iFXpip)
GO(glXQueryDrawable, vFXLip)
GO(glXQueryExtension, iFXpp)
GO(glXQueryExtensionsString, pFXi)
GO(glXQueryServerString, pFXii)
GO(glXQueryVersion, iFXpp)
GO(glXSelectEvent, vFXLL)
GO(glXSwapBuffers, vFXL)
GO(glXUseXFont, vFLiii)
GO(glXWaitGL, vFv)
GO(glXWaitX, vFv)



