#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GOM(XcursorAnimateCreate, pFEbXiip_)
//GOM(XcursorAnimateDestroy, vFEbpi_)
//GOM(XcursorAnimateNext, LFEbpi_)
//GOM(XcursorCommentCreate, pFEui)
//GO(XcursorCommentDestroy, vFbuup_)
//GOM(XcursorCommentsCreate, pFEi)
//GOM(XcursorCommentsDestroy, vFEbip_)
//GO(_XcursorCreateFontCursor, 
//GO(_XcursorCreateGlyphCursor, 
GOM(XcursorCursorsCreate, pFEXi)
GOM(XcursorCursorsDestroy, vFEp)
//GOM(XcursorFileLoad, iFESbp_bp_)
//GOM(XcursorFileLoadAllImages, pFES)
//GOM(XcursorFileLoadImage, pFESi)
//GOM(XcursorFileLoadImages, pFESi)
//GOM(XcursorFilenameLoad, iFEpbp_bp_)
//GOM(XcursorFilenameLoadAllImages, pFEp)
GO(XcursorFilenameLoadCursor, LFXp)
//GOM(XcursorFilenameLoadCursors, pFEXp)
//GOM(XcursorFilenameLoadImage, pFEpi)
//GOM(XcursorFilenameLoadImages, pFEpi)
//GOM(XcursorFilenameSave, iFEpbip_bipp_)
//GOM(XcursorFilenameSaveImages, iFEpbipp_)
//GOM(XcursorFileSave, iFESbip_bipp_)
//GOM(XcursorFileSaveImages, iFESbipp_)
GO(XcursorGetDefaultSize, iFX)
//GO(_XcursorGetDisplayInfo, 
GO(XcursorGetTheme, pFX)
GO(XcursorGetThemeCore, iFX)
GOM(XcursorImageCreate, pFEii)
GOM(XcursorImageDestroy, vFEp)
//GOM(XcursorImageHash, vFEpp)    // needs unbridge XImage (first p)
GOM(XcursorImageLoadCursor, LFEXp)
GOM(XcursorImagesCreate, pFEi)
GOM(XcursorImagesDestroy, vFEp)
GOM(XcursorImagesLoadCursor, LFEXp)
GOM(XcursorImagesLoadCursors, pFEXp)
//GOM(XcursorImagesSetName, vFEpp)
GO(XcursorLibraryLoadCursor, LFXp)
//GOM(XcursorLibraryLoadCursors, pFEXp)
//GOM(XcursorLibraryLoadImage, pFEppi)
GOM(XcursorLibraryLoadImages, pFEppi)
GO(XcursorLibraryPath, pFv)
GO(XcursorLibraryShape, iFp)
GO(XcursorNoticeCreateBitmap, vFXLuu)
//GOM(XcursorNoticePutBitmap, vFEXLp)   //needs unbridge XImage (last p)
GO(XcursorSetDefaultSize, iFXi)
GO(XcursorSetTheme, iFXp)
GO(XcursorSetThemeCore, iFXi)
GO(XcursorShapeLoadCursor, LFXu)
//GOM(XcursorShapeLoadCursors, pFEXu)
//GOM(XcursorShapeLoadImage, pFEupi)
GOM(XcursorShapeLoadImages, pFEupi)
GO(XcursorSupportsAnim, iFX)
GO(XcursorSupportsARGB, iFX)
//GO(XcursorTryShapeBitmapCursor, LFXLLbLWWWcc_bLWWWcc_uu)
//GO(XcursorTryShapeCursor, LFXLLuubLWWWcc_bLWWWcc_)
//GOM(XcursorXcFileLoad, iFEpbp_bp_)
//GOM(XcursorXcFileLoadAllImages, pFEp)
//GOM(XcursorXcFileLoadImage, pFEpi)
//GOM(XcursorXcFileLoadImages, pFEpi)
//GOM(XcursorXcFileSave, iFEpbip_bipp_)
