/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibmTYPES32_H_
#define __wrappedlibmTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef uint64_t (*UEs_t)(void*);
typedef float (*fEf_t)(float);
typedef double (*dEd_t)(double);
typedef long double (*DED_t)(long double);
typedef double (*KEK_t)(double);
typedef float (*fEff_t)(float, float);
typedef double (*dEdd_t)(double, double);
typedef double (*KEKi_t)(double, int32_t);
typedef double (*KEKK_t)(double, double);
typedef double (*KEKp_t)(double, void*);
typedef uint64_t (*UEsvvs_t)(void*, void, void, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(cacosf, UEs_t) \
	GO(cacoshf, UEs_t) \
	GO(casinf, UEs_t) \
	GO(casinhf, UEs_t) \
	GO(catanf, UEs_t) \
	GO(catanhf, UEs_t) \
	GO(ccosf, UEs_t) \
	GO(ccoshf, UEs_t) \
	GO(cexpf, UEs_t) \
	GO(clogf, UEs_t) \
	GO(cprojf, UEs_t) \
	GO(csinf, UEs_t) \
	GO(csinhf, UEs_t) \
	GO(csqrtf, UEs_t) \
	GO(ctanf, UEs_t) \
	GO(ctanhf, UEs_t) \
	GO(__acosf_finite, fEf_t) \
	GO(__acoshf_finite, fEf_t) \
	GO(__asinf_finite, fEf_t) \
	GO(__coshf_finite, fEf_t) \
	GO(__exp2f_finite, fEf_t) \
	GO(__expf_finite, fEf_t) \
	GO(__log10f_finite, fEf_t) \
	GO(__log2f_finite, fEf_t) \
	GO(__logf_finite, fEf_t) \
	GO(__sinhf_finite, fEf_t) \
	GO(__sqrtf_finite, fEf_t) \
	GO(__acos_finite, dEd_t) \
	GO(__acosh_finite, dEd_t) \
	GO(__asin_finite, dEd_t) \
	GO(__cosh_finite, dEd_t) \
	GO(__exp2_finite, dEd_t) \
	GO(__exp_finite, dEd_t) \
	GO(__log10_finite, dEd_t) \
	GO(__log2_finite, dEd_t) \
	GO(__log_finite, dEd_t) \
	GO(__sinh_finite, dEd_t) \
	GO(__sqrt_finite, dEd_t) \
	GO(pow10l, DED_t) \
	GO(acoshl, KEK_t) \
	GO(acosl, KEK_t) \
	GO(asinhl, KEK_t) \
	GO(asinl, KEK_t) \
	GO(atanhl, KEK_t) \
	GO(cbrtl, KEK_t) \
	GO(erfcl, KEK_t) \
	GO(erfl, KEK_t) \
	GO(lgammal, KEK_t) \
	GO(logl, KEK_t) \
	GO(pow10l, KEK_t) \
	GO(tgammal, KEK_t) \
	GO(__atan2f_finite, fEff_t) \
	GO(__hypotf_finite, fEff_t) \
	GO(__powf_finite, fEff_t) \
	GO(__atan2_finite, dEdd_t) \
	GO(__hypot_finite, dEdd_t) \
	GO(__pow_finite, dEdd_t) \
	GO(ldexpl, KEKi_t) \
	GO(fmodl, KEKK_t) \
	GO(powl, KEKK_t) \
	GO(frexpl, KEKp_t) \
	GO(lgammal_r, KEKp_t) \
	GO(cpowf, UEsvvs_t)

#endif // __wrappedlibmTYPES32_H_
