#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh
#endif

GO(adler32, LFLpu)
GO(adler32_combine, LFLLl)
GO(adler32_combine64, LFLLl)
GO(adler32_z, LFLpL)
GO(compress, iFpppL)
GO(compress2, iFpppLi)
GO(compressBound, LFL)
GO(crc32, LFLpu)
GO(crc32_combine, LFLLl)
GO(crc32_combine64, LFLLl)
GO(crc32_z, LFLpL)
GO(deflate, iFpi)
GO(deflateBound, LFpL)
GO(deflateCopy, iFpp)
GOM(deflateEnd, iFEp) // TODO: remove "M"?
GO(deflateGetDictionary, iFppp)
GOM(deflateInit_, iFEpipi)
GOM(deflateInit2_, iFEpiiiiipi)
GO(deflateParams, iFpii)
GO(deflatePending, iFppp)
GO(deflatePrime, iFpii)
GO(deflateReset, iFp)
GO(deflateResetKeep, iFp)
GO(deflateSetDictionary, iFppu)
GO(deflateSetHeader, iFpp)
GO(deflateTune, iFpiiii)
GO(get_crc_table, pFv)
GO(gzbuffer, iFpu)
GO(gzclearerr, vFp)
GO(gzclose, iFp)
GO(gzclose_r, iFp)
GO(gzclose_w, iFp)
GO(gzdirect, iFp)
GO(gzdopen, pFip)
GO(gzeof, iFp)
GO(gzerror, pFpp)
GO(gzflush, iFpi)
GO(gzfread, LFpLLp)
GO(gzfwrite, LFpLLp)
GO(gzgetc, iFp)
GO(gzgetc_, iFp)
GO(gzgets, pFppi)
GO(gzoffset, lFp)
GO(gzoffset64, lFp)
GO(gzopen, pFpp)
GO(gzopen64, pFpp)
//GO(gzprintf, iFppV)
GO(gzputc, iFpi)
GO(gzputs, iFpp)
GO(gzread, iFppu)
GO(gzrewind, iFp)
GO(gzseek, lFpli)
GO(gzseek64, lFpli)
GO(gzsetparams, iFpii)
GO(gztell, lFp)
GO(gztell64, lFp)
GO(gzungetc, iFip)
//GO(gzvprintf, iFppA)
GO(gzwrite, iFppu)
GOM(inflate, iFEpi)
//GO(inflateBack, iFp@p@p)
GO(inflateBackEnd, iFp)
GOM(inflateBackInit_, iFEpippi)
GO(inflateCodesUsed, LFp)
GO(inflateCopy, iFpp)
GOM(inflateEnd, iFEp) // TODO: remove "M"?
GO(inflateGetDictionary, iFppp)
GO(inflateGetHeader, iFpp)
GOM(inflateInit_, iFEppi)
GOM(inflateInit2_, iFEpipi)
GO(inflateMark, lFp)
GO(inflatePrime, iFpii)
GO(inflateReset, iFp)
GO(inflateReset2, iFpi)
GO(inflateResetKeep, iFp)
GO(inflateSetDictionary, iFppu)
GO(inflateSync, iFp)
GO(inflateSyncPoint, iFp)
GO(inflateUndermine, iFpi)
GO(inflateValidate, iFpi)
GO(uncompress, iFpppL)
GO(uncompress2, iFpppp)
GO(zError, pFi)
GO(zlibCompileFlags, LFv)
GO(zlibVersion, pFv)
