/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibglxTYPES32_H_
#define __wrappedlibglxTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void* (*pFp_t)(void*);
typedef void* (*pFXp_t)(void*, void*);
typedef void* (*pFXip_t)(void*, int32_t, void*);
typedef void* (*pFXipp_t)(void*, int32_t, void*, void*);
typedef void* (*pFXppi_t)(void*, void*, void*, int32_t);

#define SUPER() ADDED_FUNCTIONS() \
	GO(glXGetProcAddress, pFp_t) \
	GO(glXGetProcAddressARB, pFp_t) \
	GO(glXGetVisualFromFBConfig, pFXp_t) \
	GO(glXChooseVisual, pFXip_t) \
	GO(glXGetFBConfigs, pFXip_t) \
	GO(glXChooseFBConfig, pFXipp_t) \
	GO(glXCreateContext, pFXppi_t)

#endif // __wrappedlibglxTYPES32_H_
