/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibcupsTYPES32_H_
#define __wrappedlibcupsTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef int32_t (*iFp_t)(void*);
typedef void (*vFip_t)(int32_t, void*);
typedef void* (*pFpip_t)(void*, int32_t, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(cupsGetDests, iFp_t) \
	GO(cupsFreeDests, vFip_t) \
	GO(cupsGetOption, pFpip_t)

#endif // __wrappedlibcupsTYPES32_H_
