#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(xcb_atom_name_by_resource, 
//GO(xcb_atom_name_by_screen, 
//GO(xcb_atom_name_unique, 
//GO(xcb_aux_change_gc, 
//GO(xcb_aux_change_gc_checked, 
//GO(xcb_aux_change_keyboard_control, 
//GO(xcb_aux_change_window_attributes, 
//GO(xcb_aux_change_window_attributes_checked, 
//GO(xcb_aux_clear_window, 
//GO(xcb_aux_configure_window, 
//GO(xcb_aux_create_gc, 
//GO(xcb_aux_create_gc_checked, 
//GO(xcb_aux_create_window, 
//GO(xcb_aux_create_window_checked, 
//GO(xcb_aux_find_visual_by_attrs, 
//GO(xcb_aux_find_visual_by_id, 
//GO(xcb_aux_get_depth, 
//GO(xcb_aux_get_depth_of_visual, 
//GO(xcb_aux_get_screen, 
//GO(xcb_aux_get_visualtype, 
//GO(xcb_aux_parse_color, 
//GO(xcb_aux_set_line_attributes_checked, 
//GO(xcb_aux_sync, 
//GO(xcb_event_get_error_label, 
//GO(xcb_event_get_label, 
//GO(xcb_event_get_request_label, 
