#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(__create_ib_request, 
//GO(__do_niscall3, 
//GO(__follow_path, 
//GO(__free_fdresult, 
//GO(nis_add, 
//GO(nis_add_entry, 
//GO(nis_addmember, 
//GO(__nisbind_connect, 
//GO(__nisbind_create, 
//GO(__nisbind_destroy, 
//GO(__nisbind_next, 
//GO(nis_checkpoint, 
//GO(nis_clone_directory, 
//GO(nis_clone_object, 
//GO(nis_clone_result, 
//GO(nis_creategroup, 
//GO(__nis_default_access, 
//GO(__nis_default_group, 
//GO(__nis_default_owner, 
//GO(__nis_default_ttl, 
//GO(nis_destroygroup, 
//GO(nis_destroy_object, 
//GO(nis_dir_cmp, 
//GO(nis_domain_of, 
//GO(nis_domain_of_r, 
//GO(__nis_finddirectory, 
//GO(nis_first_entry, 
//GO(nis_free_directory, 
//GO(nis_freenames, 
//GO(nis_free_object, 
//GO(nis_free_request, 
//GO(nis_freeresult, 
//GO(nis_freeservlist, 
//GO(nis_freetags, 
//GO(nis_getnames, 
//GO(nis_getservlist, 
//GO(__nis_hash, 
//GO(nis_ismember, 
//GO(nis_leaf_of, 
//GO(nis_leaf_of_r, 
//GO(nis_lerror, 
//GO(nis_list, 
//GO(nis_local_directory, 
//GO(nis_local_group, 
//GO(nis_local_host, 
//GO(nis_local_principal, 
//GO(nis_lookup, 
//GO(nis_mkdir, 
//GO(nis_modify, 
//GO(nis_modify_entry, 
//GO(nis_name_of, 
//GO(nis_name_of_r, 
//GO(nis_next_entry, 
//GO(nis_perror, 
//GO(nis_ping, 
//GO(nis_print_directory, 
//GO(nis_print_entry, 
//GO(nis_print_group, 
//GO(nis_print_group_entry, 
//GO(nis_print_link, 
//GO(nis_print_object, 
//GO(nis_print_result, 
//GO(nis_print_rights, 
//GO(nis_print_table, 
//GO(nis_read_obj, 
//GO(nis_remove, 
//GO(nis_remove_entry, 
//GO(nis_removemember, 
//GO(nis_rmdir, 
//GO(nis_servstate, 
//GO(nis_sperrno, 
//GO(nis_sperror, 
//GO(nis_sperror_r, 
//GO(nis_stats, 
//GO(nis_verifygroup, 
//GO(nis_write_obj, 
//GO(_nsl_default_nss, 
//GO(__prepare_niscall, 
//GO(readColdStartFile, 
//GO(writeColdStartFile, 
//GO(xdr_cback_data, 
//GO(xdr_domainname, 
//GO(_xdr_ib_request, 
//GO(xdr_keydat, 
//GO(xdr_mapname, 
//GO(_xdr_nis_result, 
//GO(xdr_obj_p, 
//GO(xdr_peername, 
//GO(xdr_valdat, 
//GO(xdr_ypall, 
//GO(xdr_ypbind_binding, 
//GO(xdr_ypbind_resp, 
//GO(xdr_ypbind_resptype, 
//GO(xdr_ypbind_setdom, 
//GO(xdr_yp_buf, 
//GO(xdr_ypdelete_args, 
//GO(xdr_ypmaplist, 
//GO(xdr_ypmap_parms, 
//GO(xdr_yppushresp_xfr, 
//GO(xdr_yppush_status, 
//GO(xdr_ypreq_key, 
//GO(xdr_ypreq_nokey, 
//GO(xdr_ypreq_xfr, 
//GO(xdr_ypresp_all, 
//GO(xdr_ypresp_key_val, 
//GO(xdr_ypresp_maplist, 
//GO(xdr_ypresp_master, 
//GO(xdr_ypresp_order, 
//GO(xdr_ypresp_val, 
//GO(xdr_ypresp_xfr, 
//GO(xdr_ypstat, 
//GO(xdr_ypupdate_args, 
//GO(xdr_ypxfrstat, 
//GOM(yp_all, iFEppbpp_)
GO(yp_bind, iFp)
GO(ypbinderr_string, pFi)
GO(__yp_check, iFbp_)
GO(yperr_string, pFi)
GO(yp_first, iFppbp_pbp_p)
GO(yp_get_default_domain, iFbp_)
//GO(yp_maplist, 
GO(yp_master, iFppbp_)
GO(yp_match, iFpppibp_p)
GO(yp_next, iFpppibp_pbp_p)
GO(yp_order, iFppp)
GO(ypprot_err, iFi)
GO(yp_unbind, vFp)
GO(yp_update, iFppupipi)
