#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(png_access_version_number, uFv)
GO(png_benign_error, vFpp)
GO(png_build_grayscale_palette, vFip)
GO(png_calloc, pFpL)
GO(png_chunk_benign_error, vFpp)
GO(png_chunk_error, vFpp)
GO(png_chunk_warning, vFpp)
GO(png_convert_from_struct_tm, vFpp)
GO(png_convert_from_time_t, vFpl)
GO(png_convert_to_rfc1123, pFpp)
GO(png_convert_to_rfc1123_buffer, iFpp)
GO(png_create_info_struct, pFp)
GOM(png_create_read_struct, pFEpppp)
GOM(png_create_read_struct_2, pFEppppppp)
//GOM(png_create_write_struct, pFEpppp)
GOM(png_create_write_struct_2, pFEppppppp)
GO(png_data_freer, vFppiu)
GO(png_destroy_info_struct, vFpp)
GO(png_destroy_read_struct, vFppp)
GO(png_destroy_write_struct, vFpp)
GO(png_error, vFpp)
GO(png_free, vFpp)
GO(png_free_data, vFppui)
GO(png_free_default, vFpp)
GO(png_get_bit_depth, CFpp)
GO(png_get_bKGD, uFppp)
GO(png_get_channels, CFpp)
GO(png_get_cHRM, uFpppppppppp)
GO(png_get_cHRM_fixed, uFpppppppppp)
GO(png_get_cHRM_XYZ, uFppppppppppp)
GO(png_get_cHRM_XYZ_fixed, uFppppppppppp)
GO(png_get_chunk_cache_max, uFp)
GO(png_get_chunk_malloc_max, LFp)
GO(png_get_cICP, uFpppppp)
GO(png_get_cLLI, uFpppp)
GO(png_get_cLLI_fixed, uFpppp)
GO(png_get_color_type, CFpp)
GO(png_get_compression_buffer_size, LFp)
GO(png_get_compression_type, CFpp)
GO(png_get_copyright, pFp)
GO(png_get_current_pass_number, CFp)
GO(png_get_current_row_number, uFp)
GO(png_get_error_ptr, pFp)
GO(png_get_eXIf, uFppp)
GO(png_get_eXIf_1, uFpppp)
GO(png_get_filter_type, CFpp)
GO(png_get_gAMA, uFppp)
GO(png_get_gAMA_fixed, uFppp)
GO(png_get_header_ver, pFp)
GO(png_get_header_version, pFp)
GO(png_get_hIST, uFppp)
GO(png_get_iCCP, uFpppppp)
GO(png_get_IHDR, uFppppppppp)
GO(png_get_image_height, uFpp)
GO(png_get_image_width, uFpp)
GO(png_get_int_32, iFp)
GO(png_get_interlace_type, CFpp)
GO(png_get_io_chunk_type, uFp)
GO(png_get_io_ptr, pFp)
GO(png_get_io_state, uFp)
GO(png_get_libpng_ver, pFp)
GO(png_get_mDCV, uFpppppppppppp)
GO(png_get_mDCV_fixed, uFpppppppppppp)
GO(png_get_mem_ptr, pFp)
GO(png_get_oFFs, uFppppp)
GO(png_get_palette_max, iFpp)
GO(png_get_pCAL, uFppppppppp)
GO(png_get_pHYs, uFppppp)
GO(png_get_pHYs_dpi, uFppppp)
GO(png_get_pixel_aspect_ratio, fFpp)
GO(png_get_pixel_aspect_ratio_fixed, iFpp)
GO(png_get_pixels_per_inch, uFpp)
GO(png_get_pixels_per_meter, uFpp)
GO(png_get_PLTE, uFpppp)
GO(png_get_progressive_ptr, pFp)
GO(png_get_rgb_to_gray_status, CFp)
GO(png_get_rowbytes, LFpp)
GO(png_get_rows, pFpp)
GO(png_get_sBIT, uFppp)
GO(png_get_sCAL, uFppppp)
GO(png_get_sCAL_fixed, uFppppp)
GO(png_get_sCAL_s, uFppppp)
GO(png_get_signature, pFpp)
GO(png_get_sPLT, iFppp)
GO(png_get_sRGB, uFppp)
GO(png_get_text, iFpppp)
GO(png_get_tIME, uFppp)
GO(png_get_tRNS, uFppppp)
GO(png_get_uint_16, WFp)
GO(png_get_uint_31, uFpp)
GO(png_get_uint_32, uFp)
GO(png_get_unknown_chunks, iFppp)
GO(png_get_user_chunk_ptr, pFp)
GO(png_get_user_height_max, uFp)
GO(png_get_user_transform_ptr, pFp)
GO(png_get_user_width_max, uFp)
GO(png_get_valid, uFppu)
GO(png_get_x_offset_inches, fFpp)
GO(png_get_x_offset_inches_fixed, iFpp)
GO(png_get_x_offset_microns, iFpp)
GO(png_get_x_offset_pixels, iFpp)
GO(png_get_x_pixels_per_inch, uFpp)
GO(png_get_x_pixels_per_meter, uFpp)
GO(png_get_y_offset_inches, fFpp)
GO(png_get_y_offset_inches_fixed, iFpp)
GO(png_get_y_offset_microns, iFpp)
GO(png_get_y_offset_pixels, iFpp)
GO(png_get_y_pixels_per_inch, uFpp)
GO(png_get_y_pixels_per_meter, uFpp)
GO(png_handle_as_unknown, iFpp)
GO(png_image_begin_read_from_file, iFpp)
GO(png_image_begin_read_from_memory, iFppL)
GO(png_image_begin_read_from_stdio, iFpS)
GO(png_image_finish_read, iFpppip)
GO(png_image_free, vFp)
GO(png_image_write_to_file, iFppipip)
GO(png_image_write_to_memory, iFpppipip)
GO(png_image_write_to_stdio, iFpSipip)
GO(png_info_init_3, vFpL)
GO(png_init_io, vFpS)
GO(png_longjmp, vFpi)
GO(png_malloc, pFpL)
GO(png_malloc_default, pFpL)
GO(png_malloc_warn, pFpL)
GO(png_permit_mng_features, uFpu)
GO(png_process_data, vFpppL)
GO(png_process_data_pause, LFpi)
GO(png_process_data_skip, uFp)
GO(png_progressive_combine_row, vFppp)
GO(png_read_end, vFpp)
GO(png_read_image, vFpp)
GO(png_read_info, vFpp)
GO(png_read_png, vFppip)
GO(png_read_row, vFppp)
GO(png_read_rows, vFpppu)
GO(png_read_update_info, vFpp)
GO(png_reset_zstream, iFp)
GO(png_save_int_32, vFpi)
GO(png_save_uint_16, vFpu)
GO(png_save_uint_32, vFpu)
GO(png_set_add_alpha, vFpui)
GO(png_set_alpha_mode, vFpid)
GO(png_set_alpha_mode_fixed, vFpii)
GO(png_set_background, vFppiid)
GO(png_set_background_fixed, vFppiii)
GO(png_set_benign_errors, vFpi)
GO(png_set_bgr, vFp)
GO(png_set_bKGD, vFppp)
GO(png_set_check_for_invalid_index, vFpi)
GO(png_set_cHRM, vFppdddddddd)
GO(png_set_cHRM_fixed, vFppiiiiiiii)
GO(png_set_cHRM_XYZ, vFppddddddddd)
GO(png_set_cHRM_XYZ_fixed, vFppiiiiiiiii)
GO(png_set_chunk_cache_max, vFpu)
GO(png_set_chunk_malloc_max, vFpL)
GO(png_set_cICP, vFppCCCC)
GO(png_set_cLLI, vFppdd)
GO(png_set_cLLI_fixed, vFppuu)
GO(png_set_compression_buffer_size, vFpL)
GO(png_set_compression_level, vFpi)
GO(png_set_compression_mem_level, vFpi)
GO(png_set_compression_method, vFpi)
GO(png_set_compression_strategy, vFpi)
GO(png_set_compression_window_bits, vFpi)
GO(png_set_crc_action, vFpii)
GOM(png_set_error_fn, vFEpppp)
GO(png_set_eXIf, vFppp)
GO(png_set_eXIf_1, vFppup)
GO(png_set_expand, vFp)
GO(png_set_expand_16, vFp)
GO(png_set_expand_gray_1_2_4_to_8, vFp)
GO(png_set_filler, vFpui)
GO(png_set_filter, vFpii)
GO(png_set_filter_heuristics, vFpiipp)
GO(png_set_filter_heuristics_fixed, vFpiipp)
GO(png_set_flush, vFpi)
GO(png_set_gAMA, vFppd)
GO(png_set_gAMA_fixed, vFppi)
GO(png_set_gamma, vFpdd)
GO(png_set_gamma_fixed, vFpii)
GO(png_set_gray_to_rgb, vFp)
GO(png_set_hIST, vFppp)
GO(png_set_iCCP, vFpppipu)
GO(png_set_IHDR, vFppuuiiiii)
GO(png_set_interlace_handling, iFp)
GO(png_set_invalid, vFppi)
GO(png_set_invert_alpha, vFp)
GO(png_set_invert_mono, vFp)
GO(png_set_keep_unknown_chunks, vFpipi)
//GOM(png_set_longjmp_fn, pFEppL)
GO(png_set_mDCV, vFppdddddddddd)
GO(png_set_mDCV_fixed, vFppiiiiiiiiuu)
//GOM(png_set_mem_fn, vFEpppp)
GO(png_set_oFFs, vFppiii)
GO(png_set_option, iFpii)
GO(png_set_packing, vFp)
GO(png_set_packswap, vFp)
GO(png_set_palette_to_rgb, vFp)
GO(png_set_pCAL, vFpppiiiipp)
GO(png_set_pHYs, vFppuui)
GO(png_set_PLTE, vFpppi)
GOM(png_set_progressive_read_fn, vFEppppp)
GO(png_set_quantize, vFppiipi)
GOM(png_set_read_fn, vFEppp)
//GOM(png_set_read_status_fn, vFEpp)
//GOM(png_set_read_user_chunk_fn, vFEppp)
GOM(png_set_read_user_transform_fn, vFEpp)
GO(png_set_rgb_to_gray, vFpidd)
GO(png_set_rgb_to_gray_fixed, vFpiii)
GO(png_set_rows, vFppp)
GO(png_set_sBIT, vFppp)
GO(png_set_sCAL, vFppidd)
GO(png_set_scale_16, vFp)
GO(png_set_sCAL_fixed, vFppiii)
GO(png_set_sCAL_s, vFppipp)
GO(png_set_shift, vFpp)
GO(png_set_sig_bytes, vFpi)
GO(png_set_sPLT, vFpppi)
GO(png_set_sRGB, vFppi)
GO(png_set_sRGB_gAMA_and_cHRM, vFppi)
GO(png_set_strip_16, vFp)
GO(png_set_strip_alpha, vFp)
GO(png_set_swap, vFp)
GO(png_set_swap_alpha, vFp)
GO(png_set_text, vFpppi)
GO(png_set_text_compression_level, vFpi)
GO(png_set_text_compression_mem_level, vFpi)
GO(png_set_text_compression_method, vFpi)
GO(png_set_text_compression_strategy, vFpi)
GO(png_set_text_compression_window_bits, vFpi)
GO(png_set_tIME, vFppp)
GO(png_set_tRNS, vFpppip)
GO(png_set_tRNS_to_alpha, vFp)
GO(png_set_unknown_chunk_location, vFppii)
GO(png_set_unknown_chunks, vFpppi)
GO(png_set_user_limits, vFpuu)
GO(png_set_user_transform_info, vFppii)
GOM(png_set_write_fn, vFEpppp)
//GOM(png_set_write_status_fn, vFEpp)
//GOM(png_set_write_user_transform_fn, vFEpp)
GO(png_sig_cmp, iFpLL)
GO(png_start_read_image, vFp)
GO(png_warning, vFpp)
GO(png_write_chunk, vFpppL)
GO(png_write_chunk_data, vFppL)
GO(png_write_chunk_end, vFp)
GO(png_write_chunk_start, vFppu)
GO(png_write_end, vFpp)
GO(png_write_flush, vFp)
GO(png_write_image, vFpp)
GO(png_write_info, vFpp)
GO(png_write_info_before_PLTE, vFpp)
GO(png_write_png, vFppip)
GO(png_write_row, vFpp)
GO(png_write_rows, vFppu)
GO(png_write_sig, vFp)
