#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

// ayatanamenuitemfactory.h
GO(ayatana_menu_item_factory_create_menu_item, pFpppp)
GO(ayatana_menu_item_factory_get_all, pFv)
GO(ayatana_menu_item_factory_get_type, pFv)

// idoactionhelper (no public header, from nm)
GO(ido_action_helper_activate, vFp)
GO(ido_action_helper_activate_with_parameter, vFpp)
GO(ido_action_helper_change_action_state, vFpp)
GO(ido_action_helper_get_action_target, pFp)
GO(ido_action_helper_get_type, pFv)
GO(ido_action_helper_get_widget, pFp)
GO(ido_action_helper_new, pFppp)

// various *_new_from_model functions (no public header, from nm)
GO(ido_alarm_menu_item_new_from_model, pFpp)
GO(ido_application_menu_item_get_type, pFv)
GO(ido_application_menu_item_new_from_model, pFpp)
GO(ido_appointment_menu_item_new_from_model, pFpp)

// idobasicmenuitem (no public header, from nm)
GO(ido_basic_menu_item_get_type, pFv)
GO(ido_basic_menu_item_new, pFv)
GO(ido_basic_menu_item_new_from_model, pFpp)
GO(ido_basic_menu_item_set_icon, vFpp)
GO(ido_basic_menu_item_set_icon_from_file, vFpp)
GO(ido_basic_menu_item_set_pixbuf, vFpp)
GO(ido_basic_menu_item_set_secondary_count, vFpi)
GO(ido_basic_menu_item_set_secondary_text, vFpp)
GO(ido_basic_menu_item_set_text, vFpp)

// idocalendarmenuitem.h
GO(ido_calendar_menu_item_clear_marks, vFp)
GO(ido_calendar_menu_item_get_calendar, pFp)
GO(ido_calendar_menu_item_get_date, vFpppp)
GO(ido_calendar_menu_item_get_display_options, iFp)
GO(ido_calendar_menu_item_get_type, pFv)
GO(ido_calendar_menu_item_mark_day, iFpu)
GO(ido_calendar_menu_item_new, pFv)
GO(ido_calendar_menu_item_new_from_model, pFpp)
GO(ido_calendar_menu_item_set_date, iFpuuu)
GO(ido_calendar_menu_item_set_display_options, vFpi)
GO(ido_calendar_menu_item_unmark_day, iFpu)

// idodetaillabel (no public header, from nm)
GO(ido_detail_label_get_text, pFp)
GO(ido_detail_label_get_type, pFv)
GO(ido_detail_label_new, pFp)
GO(ido_detail_label_set_count, vFpi)
GO(ido_detail_label_set_text, vFpp)

// idoentrymenuitem.h
GO(ido_entry_menu_item_get_entry, pFp)
GO(ido_entry_menu_item_get_type, pFv)
GO(ido_entry_menu_item_new, pFv)

// idoguestmenuitem (no public header, from nm)
GO(ido_guest_menu_item_new_from_model, pFpp)

// libayatana-ido.h
GO(ido_init, vFv)

// idolevelmenuitem (no public header, from nm)
GO(ido_level_menu_item_get_type, pFv)
GO(ido_level_menu_item_new, pFv)
GO(ido_level_menu_item_new_from_model, pFpp)
GO(idoLevelMenuItemSetIcon, vFpp)
GO(idoLevelMenuItemSetLevel, vFpu)
GO(idoLevelMenuItemSetText, vFpp)

// idolocationmenuitem (no public header, from nm)
GO(ido_location_menu_item_get_type, pFv)
GO(ido_location_menu_item_new, pFv)
GO(ido_location_menu_item_new_from_model, pFpp)
GO(ido_location_menu_item_set_timezone, vFpp)

// idomediaplayermenuitem (no public header, from nm)
GO(ido_media_player_menu_item_get_type, pFv)
GO(ido_media_player_menu_item_new_from_model, pFpp)

// idomenuitemfactory (no public header, from nm)
GO(ido_menu_item_factory_get_type, pFv)

// idoplaybackmenuitem (no public header, from nm)
GO(ido_playback_menu_item_get_type, pFv)
GO(ido_playback_menu_item_new_from_model, pFpp)

// idoprogressmenuitem (no public header, from nm)
GO(ido_progress_menu_item_new_from_model, pFpp)

// idorange.h
GO(ido_range_get_type, pFv)
GO(ido_range_new, pFpi)
GO(ido_range_style_get_type, pFv)

// idoremovablemenuitem (no public header, from nm)
GO(ido_removable_menu_item_get_type, pFv)
GO(ido_removable_menu_item_new, pFv)
GO(ido_removable_menu_item_new_from_model, pFpp)
GO(idoRemovableMenuItemSetIcon, vFpp)
GO(idoRemovableMenuItemSetIconFromFile, vFpp)
GO(idoRemovableMenuItemSetText, vFpp)
GO(idoRemovableMenuItemUseMarkup, vFpi)

// idoscalemenuitem.h
GO(ido_scale_menu_item_get_primary_image, pFp)
GO(ido_scale_menu_item_get_primary_label, pFp)
GO(ido_scale_menu_item_get_scale, pFp)
GO(ido_scale_menu_item_get_secondary_image, pFp)
GO(ido_scale_menu_item_get_secondary_label, pFp)
GO(ido_scale_menu_item_get_style, iFp)
GO(ido_scale_menu_item_get_type, pFv)
GO(ido_scale_menu_item_new, pFpip)
GO(ido_scale_menu_item_new_from_model, pFpp)
GO(ido_scale_menu_item_new_with_range, pFpidddd)
GO(ido_scale_menu_item_primary_clicked, vFp)
GO(ido_scale_menu_item_secondary_clicked, vFp)
GO(ido_scale_menu_item_set_primary_label, vFpp)
GO(ido_scale_menu_item_set_secondary_label, vFpp)
GO(ido_scale_menu_item_set_style, vFpi)
GO(ido_scale_menu_item_style_get_type, pFv)

// idosourcemenuitem (no public header, from nm)
GO(ido_source_menu_item_get_type, pFv)
GO(ido_source_menu_item_new_from_menu_model, pFpp)

// idoswitchmenuitem.h
GO(ido_switch_menu_item_get_content_area, pFp)
GO(ido_switch_menu_item_get_type, pFv)
GO(ido_switch_menu_item_new, pFv)
GO(ido_switch_menu_item_new_from_menu_model, pFpp)
GO(ido_switch_menu_item_set_accelerator, vFpp)
GO(ido_switch_menu_item_set_icon, vFpp)
GO(ido_switch_menu_item_set_label, vFpp)

// idotimeline.h
GO(ido_timeline_calculate_progress, dFdi)
GO(ido_timeline_direction_get_type, pFv)
GO(ido_timeline_get_direction, iFp)
GO(ido_timeline_get_duration, uFp)
GO(ido_timeline_get_fps, uFp)
GO(ido_timeline_get_loop, iFp)
GO(ido_timeline_get_progress, dFp)
GO(ido_timeline_get_screen, pFp)
GO(ido_timeline_get_type, pFv)
GO(ido_timeline_is_running, iFp)
GO(ido_timeline_new, pFu)
GO(ido_timeline_new_for_screen, pFup)
GO(ido_timeline_pause, vFp)
GO(ido_timeline_progress_type_get_type, pFv)
GO(ido_timeline_rewind, vFp)
GO(ido_timeline_set_direction, vFpi)
GO(ido_timeline_set_duration, vFpu)
GO(ido_timeline_set_fps, vFpu)
GO(ido_timeline_set_loop, vFpi)
GO(ido_timeline_set_progress, vFpd)
GO(ido_timeline_set_screen, vFpp)
GO(ido_timeline_start, vFp)

// idotimestampmenuitem (no public header, from nm)
GO(ido_time_stamp_menu_item_get_format, pFp)
GO(ido_time_stamp_menu_item_get_type, pFv)
GO(ido_time_stamp_menu_item_new, pFv)
GO(ido_time_stamp_menu_item_set_date_time, vFpp)
GO(ido_time_stamp_menu_item_set_format, vFpp)

// idousermenuitem (no public header, from nm)
GO(ido_user_menu_item_get_type, pFv)
GO(ido_user_menu_item_new, pFv)
GO(ido_user_menu_item_new_from_model, pFpp)
GO(ido_user_menu_item_set_current_user, vFpi)
GO(ido_user_menu_item_set_icon, vFpp)
GO(ido_user_menu_item_set_icon_from_file, vFpp)
GO(ido_user_menu_item_set_label, vFpp)
GO(ido_user_menu_item_set_logged_in, vFpi)
