#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(xcb_res_client_end, 
//GO(xcb_res_client_id_spec_end, 
GO(xcb_res_client_id_spec_next, vFp)
//GO(xcb_res_client_id_value_end, 
GO(xcb_res_client_id_value_next, vFp)
GO(xcb_res_client_id_value_sizeof, iFp)
GO(xcb_res_client_id_value_value, pFp)
GO(xcb_res_client_id_value_value_end, HFp)
GO(xcb_res_client_id_value_value_length, iFp)
GO(xcb_res_client_next, vFp)
DATA(xcb_res_id, 16)
GO(xcb_res_query_client_ids, UFbup)
GO(xcb_res_query_client_ids_ids_iterator, HFp)
GO(xcb_res_query_client_ids_ids_length, iFp)
GO(xcb_res_query_client_ids_reply, pFbup)
GO(xcb_res_query_client_ids_sizeof, iFp)
GO(xcb_res_query_client_ids_unchecked, UFbup)
GO(xcb_res_query_client_pixmap_bytes, UFbu)
GO(xcb_res_query_client_pixmap_bytes_reply, pFbup)
GO(xcb_res_query_client_pixmap_bytes_unchecked, UFbu)
GO(xcb_res_query_client_resources, UFbu)
GO(xcb_res_query_client_resources_reply, pFbup)
GO(xcb_res_query_client_resources_sizeof, iFp)
GO(xcb_res_query_client_resources_types, pFp)
GO(xcb_res_query_client_resources_types_iterator, HFp)
GO(xcb_res_query_client_resources_types_length, iFp)
GO(xcb_res_query_client_resources_unchecked, UFbu)
GO(xcb_res_query_clients, UFb)
GO(xcb_res_query_clients_clients, pFp)
GO(xcb_res_query_clients_clients_iterator, HFp)
GO(xcb_res_query_clients_clients_length, iFp)
GO(xcb_res_query_clients_reply, pFbup)
GO(xcb_res_query_clients_sizeof, iFp)
GO(xcb_res_query_clients_unchecked, UFb)
GO(xcb_res_query_resource_bytes, UFbuup)
GO(xcb_res_query_resource_bytes_reply, pFbup)
GO(xcb_res_query_resource_bytes_sizeof, iFp)
GO(xcb_res_query_resource_bytes_sizes_iterator, HFp)
GO(xcb_res_query_resource_bytes_sizes_length, iFp)
GO(xcb_res_query_resource_bytes_unchecked, UFbuup)
GO(xcb_res_query_version, UFbCC)
GO(xcb_res_query_version_reply, pFbup)
GO(xcb_res_query_version_unchecked, UFbCC)
//GO(xcb_res_resource_id_spec_end, 
GO(xcb_res_resource_id_spec_next, vFp)
//GO(xcb_res_resource_size_spec_end, 
GO(xcb_res_resource_size_spec_next, vFp)
GO(xcb_res_resource_size_value_cross_references, pFp)
GO(xcb_res_resource_size_value_cross_references_iterator, HFp)
GO(xcb_res_resource_size_value_cross_references_length, iFp)
//GO(xcb_res_resource_size_value_end, 
GO(xcb_res_resource_size_value_next, vFp)
GO(xcb_res_resource_size_value_sizeof, iFp)
//GO(xcb_res_type_end, 
GO(xcb_res_type_next, vFp)
