#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

// XEventClass is a 32bits value, on 32bits and 64bits version, no need to wrap

//GO(XAllowDeviceEvents, iFXbLip_iL)
//GO(XChangeDeviceControl, iFXbLip_ibLi_)
//GO(XChangeDeviceDontPropagateList, iFXLibL_i)
//GO(XChangeDeviceKeyMapping, iFXbLip_iibL_i)
//GO(XChangeDeviceProperty, vFXbLip_LLiipi)
//GO(XChangeFeedbackControl, iFXbLip_LbLiL_)
//GO(XChangeKeyboardDevice, iFXbLip_)
//GO(XChangePointerDevice, iFXbLip_ii)
GOM(XCloseDevice, iFEXp)
//GO(XDeleteDeviceProperty, vFXbLip_L)
//GO(XDeviceBell, iFXbLip_LLi)
//GO(XFreeDeviceControl, vFbLi_)
GOM(XFreeDeviceList, vFEp)
GOM(XFreeDeviceMotionEvents, vFEp)
GOM(XFreeDeviceState, vFEp)
//GO(XFreeFeedbackList, vFbLiL_)
GOM(XGetDeviceButtonMapping, iFEXppu)
//GO(XGetDeviceControl, bLi_FXbLip_i)
//GO(XGetDeviceDontPropagateList, bL_FXLp)
//GO(XGetDeviceFocus, iFXbLip_bL_pbL_)
//GO(XGetDeviceKeyMapping, bL_FXbLip_Cip)
//GO(XGetDeviceModifierMapping, bip_FXbLip_)
GOM(XGetDeviceMotionEvents, pFEXpLLppp)
//GO(XGetDeviceProperty, iFXbLip_LlliLbL_pbL_bL_bp_)
GO(XGetExtensionVersion, pFXp)
//GO(XGetFeedbackControl, bLiL_FXbLip_p)
//GO(XGetSelectedExtensionEvents, iFXLpbbL__pbbL__)
GOM(XGrabDevice, iFEXpLiipiiL)
//GO(XGrabDeviceButton, iFXbLip_uubLip_LiubL_ii)
//GO(XGrabDeviceKey, iFXbLip_uubLip_LiubL_ii)
GO(XIAllowEvents, iFXiiL)
GO(XIAllowTouchEvents, iFXiuLi)
GO(XIBarrierReleasePointer, vFXiLu)
//GO(XIBarrierReleasePointers, vFXbiLu_i)
//GO(XIChangeHierarchy, iFppi)
GO(XIChangeProperty, vFXiLLiipi)
GO(XIDefineCursor, iFXiLL)
GO(XIDeleteProperty, vFXiL)
GOM(XIFreeDeviceInfo, vFEp)
GO(XIGetClientPointer, iFXLp)
GO(XIGetFocus, iFXibL_)
GO(XIGetProperty, iFXiLlliLBL_pBL_BL_bp_)
GOM(XIGetSelectedEvents, pFEXLp)
GO(XIGrabButton, iFXiiLLiiibiip_ip)
GO(XIGrabDevice, iFXiLLLiiibiip_)
GO(XIGrabEnter, iFXiLLiiibiip_ip)
GO(XIGrabFocusIn, iFXiLiiibiip_ip)
GO(XIGrabKeycode, iFXiiLiiibiip_ip)
GO(XIGrabTouchBegin, iFXiLibiip_ip)
//GO(XIListProperties, bL_FXip)
GOM(XIQueryDevice, pFEXip)
GO(XIQueryPointer, iFXiLbL_bL_ppppbip_pp)
GO(XIQueryVersion, iFXpp)
GOM(XISelectEvents, iFEXLpi)
GO(XISetClientPointer, iFXLi)
GO(XISetFocus, iFXiLL)
GO(XIUndefineCursor, iFXiL)
GO(XIUngrabButton, iFXiiLip)
GO(XIUngrabDevice, iFXiL)
GO(XIUngrabEnter, iFXiLip)
GO(XIUngrabFocusIn, iFXiLip)
GO(XIUngrabKeycode, iFXiiLip)
GO(XIUngrabTouchBegin, iFXiLip)
GO(XIWarpPointer, iFXiLLdduudd)
//GO(XListDeviceProperties, bL_FXbLip_p)
GOM(XListInputDevices, pFEXp)
GOM(XOpenDevice, pFEXL)
GOM(XQueryDeviceState, pFEXp)
GO(XSelectExtensionEvent, iFXLpi)
//GO(XSendExtensionEvent, iFppLiipp)
//GO(XSetDeviceButtonMapping, iFXbLip_pi)
//GO(XSetDeviceFocus, iFXbLip_LiL)
//GO(XSetDeviceMode, iFXbLip_i)
//GO(XSetDeviceModifierMapping, iFXbLip_bip_)
//GO(XSetDeviceValuators, iFXbLip_pii)
GO(XUngrabDevice, iFXbLip_L)
GO(XUngrabDeviceButton, iFXbLip_uubLip_L)
GO(XUngrabDeviceKey, iFXbLip_uubLip_L)
GO(_XiGetDevicePresenceNotifyEvent, iFX)
GO(_xibadclass, vFXp)
GO(_xibaddevice, vFXp)
GO(_xibadevent, vFXp)
GO(_xibadmode, vFXp)
GO(_xidevicebusy, vFXp)
