#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error meh!
#endif


GO(XDamageAdd, vFpLL)
GO(XDamageCreate, LFpLi)
GO(XDamageDestroy, vFpL)
GO(XDamageFindDisplay, pFp)
GO(XDamageQueryExtension, iFppp)
GO(XDamageQueryVersion, iFppp)
GO(XDamageSubtract, vFpLLL)
