#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

// UDate is double
// UBool is int8_t

GO(ucal_add_72, vFpiip)
GO(ucal_clear_72, vFp)
GO(ucal_clearField_72, vFpi)
GO(ucal_clone_72, pFpp)
GO(ucal_close_72, vFp)
GO(ucal_countAvailable_72, iFv)
GO(ucal_equivalentTo_72, cFpp)
GO(ucal_get_72, iFpip)
GO(ucal_getAttribute_72, iFpi)
GO(ucal_getAvailable_72, pFi)
GO(ucal_getCanonicalTimeZoneID_72, iFpipipp)
GO(ucal_getDayOfWeekType_72, iFpip)
GO(ucal_getDefaultTimeZone_72, iFpip)
GO(ucal_getDSTSavings_72, iFpp)
GO(ucal_getFieldDifference_72, iFpdip)
GO(ucal_getGregorianChange_72, dFpp)
GO(ucal_getHostTimeZone_72, iFpip)
GO(ucal_getKeywordValuesForLocale_72, pFppCp)
GO(ucal_getLimit_72, iFpiip)
GO(ucal_getLocaleByType_72, pFpip)
GO(ucal_getMillis_72, dFpp)
GO(ucal_getNow_72, dFv)
GO(ucal_getTimeZoneDisplayName_72, iFpippip)
GO(ucal_getTimeZoneID_72, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_72, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_72, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_72, cFpipp)
GO(ucal_getType_72, pFpp)
GO(ucal_getTZDataVersion_72, pFp)
GO(ucal_getWeekendTransition_72, iFpip)
GO(ucal_getWindowsTimeZoneID_72, iFpipip)
GO(ucal_inDaylightTime_72, cFpp)
GO(ucal_isSet_72, cFpi)
GO(ucal_isWeekend_72, cFpdp)
GO(ucal_open_72, pFpipip)
GO(ucal_openCountryTimeZones_72, pFpp)
GO(ucal_openTimeZoneIDEnumeration_72, pFippp)
GO(ucal_openTimeZones_72, pFp)
GO(ucal_roll_72, vFpiip)
GO(ucal_set_72, vFpii)
GO(ucal_setAttribute_72, vFpii)
GO(ucal_setDate_72, vFpiiip)
GO(ucal_setDateTime_72, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_72, vFpp)
GO(ucal_setGregorianChange_72, vFpdp)
GO(ucal_setMillis_72, vFpdp)
GO(ucal_setTimeZone_72, vFppip)

GO(ucol_clone_72, pFpp)
GO(ucol_close_72, vFp)
GO(ucol_closeElements_72, vFp)
GO(ucol_setMaxVariable_72, vFpip)
GO(ucol_getOffset_72, iFp)
GO(ucol_getRules_72, pFpp)
GO(ucol_getSortKey_72, iFppipi)
GO(ucol_getStrength_72, iFp)
GO(ucol_getVersion_72, vFpp)
GO(ucol_next_72, iFpp)
GO(ucol_previous_72, iFpp)
GO(ucol_open_72, pFpp)
GO(ucol_openElements_72, pFppip)
GO(ucol_openRules_72, pFpiiipp)
GO(ucol_safeClone_72, pFpppp)
GO(ucol_setAttribute_72, vFpiip)
GO(ucol_setVariableTop_72, uFppip)
GO(ucol_strcoll_72, iFppipi)

GO(udat_close_72, vFp)
GO(udat_countSymbols_72, iFpi)
GO(udat_getSymbols_72, iFpiipip)
GO(udat_format_72, iFpdpipp)
GO(udat_open_72, pFiippipip)
GO(udat_setCalendar_72, vFpp)
GO(udat_toPattern_72, iFpCpip)

GO(unum_close_72, vFp)
GO(unum_getAttribute_72, iFpi)
GO(unum_getSymbol_72, iFpipip)
GO(unum_open_72, pFipippp)
GO(unum_toPattern_72, iFpCpip)

GO(udatpg_close_72, vFp)
GO(udatpg_getBestPattern_72, iFppipip)
GO(udatpg_open_72, pFpp)

GO(ulocdata_getCLDRVersion_72, vFpp)
GO(ulocdata_getMeasurementSystem_72, iFpp)

GO(usearch_close_72, vFp)
GO(usearch_first_72, iFpp)
GO(usearch_getBreakIterator_72, pFp)
GO(usearch_getMatchedLength_72, iFp)
GO(usearch_last_72, iFpp)
GO(usearch_openFromCollator_72, pFpipippp)
GO(usearch_setPattern_72, vFppip)
GO(usearch_setText_72, vFppip)
