#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(cblas_caxpby, vFlpplppl)
GO(cblas_caxpy, vFlpplpl)
GO(cblas_caxpyc, vFlpplpl)
GO(cblas_ccopy, vFlplpl)
GO(cblas_cdotc, xFlplpl)
GO(cblas_cdotc_sub, vFlplplp)
GO(cblas_cdotu, xFlplpl)
GO(cblas_cdotu_sub, vFlplplp)
GO(cblas_cgbmv, vFuullllpplplppl)
GO(cblas_cgeadd, vFullpplppl)
GO(cblas_cgemm, vFuuulllpplplppl)
GO(cblas_cgemm_batch, vFuppppppppppppplp)
GO(cblas_cgemmt, vFuuuullpplplppl)
//GO(cblas_cgemmtr, 
GO(cblas_cgemv, vFuullpplplppl)
GO(cblas_cgerc, vFullpplplpl)
GO(cblas_cgeru, vFullpplplpl)
GO(cblas_chbmv, vFuullpplplppl)
GO(cblas_chemm, vFuuullpplplppl)
GO(cblas_chemv, vFuulpplplppl)
GO(cblas_cher, vFuulfplpl)
GO(cblas_cher2, vFuulpplplpl)
GO(cblas_cher2k, vFuuullpplplfpl)
GO(cblas_cherk, vFuuullfplfpl)
GO(cblas_chpmv, vFuulppplppl)
GO(cblas_chpr, vFuulfplp)
GO(cblas_chpr2, vFuulpplplp)
GO(cblas_cimatcopy, vFuullppll)
GO(cblas_comatcopy, vFuullpplpl)
GO(cblas_crotg, vFpppp)
GO(cblas_cscal, vFlppl)
GO(cblas_csrot, vFlplplff)
GO(cblas_csscal, vFlfpl)
GO(cblas_cswap, vFlplpl)
GO(cblas_csymm, vFuuullpplplppl)
GO(cblas_csyr2k, vFuuullpplplppl)
GO(cblas_csyrk, vFuuullpplppl)
GO(cblas_ctbmv, vFuuuullplpl)
GO(cblas_ctbsv, vFuuuullplpl)
GO(cblas_ctpmv, vFuuuulppl)
GO(cblas_ctpsv, vFuuuulppl)
GO(cblas_ctrmm, vFuuuuullpplpl)
GO(cblas_ctrmv, vFuuuulplpl)
GO(cblas_ctrsm, vFuuuuullpplpl)
GO(cblas_ctrsv, vFuuuulplpl)
GO(cblas_damax, dFlpl)
GO(cblas_damin, dFlpl)
GO(cblas_dasum, dFlpl)
GO(cblas_daxpby, vFldpldpl)
GO(cblas_daxpy, vFldplpl)
GO(cblas_dcopy, vFlplpl)
GO(cblas_ddot, dFlplpl)
GO(cblas_dgbmv, vFuulllldplpldpl)
GO(cblas_dgeadd, vFulldpldpl)
GO(cblas_dgemm, vFuuullldplpldpl)
GO(cblas_dgemm_batch, vFuppppppppppppplp)
GO(cblas_dgemmt, vFuuuulldplpldpl)
//GO(cblas_dgemmtr, 
GO(cblas_dgemv, vFuulldplpldpl)
GO(cblas_dger, vFulldplplpl)
GO(cblas_dimatcopy, vFuulldpll)
GO(cblas_dnrm2, dFlpl)
GO(cblas_domatcopy, vFuulldplpl)
GO(cblas_drot, vFlplpldd)
GO(cblas_drotg, vFpppp)
GO(cblas_drotm, vFlplplp)
GO(cblas_drotmg, vFpppdp)
GO(cblas_dsbmv, vFuulldplpldpl)
GO(cblas_dscal, vFldpl)
GO(cblas_dsdot, dFlplpl)
GO(cblas_dspmv, vFuuldppldpl)
GO(cblas_dspr, vFuuldplp)
GO(cblas_dspr2, vFuuldplplp)
GO(cblas_dsum, dFlpl)
GO(cblas_dswap, vFlplpl)
GO(cblas_dsymm, vFuuulldplpldpl)
GO(cblas_dsymv, vFuuldplpldpl)
GO(cblas_dsyr, vFuuldplpl)
GO(cblas_dsyr2, vFuuldplplpl)
GO(cblas_dsyr2k, vFuuulldplpldpl)
GO(cblas_dsyrk, vFuuulldpldpl)
GO(cblas_dtbmv, vFuuuullplpl)
GO(cblas_dtbsv, vFuuuullplpl)
GO(cblas_dtpmv, vFuuuulppl)
GO(cblas_dtpsv, vFuuuulppl)
GO(cblas_dtrmm, vFuuuuulldplpl)
GO(cblas_dtrmv, vFuuuulplpl)
GO(cblas_dtrsm, vFuuuuulldplpl)
GO(cblas_dtrsv, vFuuuulplpl)
GO(cblas_dzamax, dFlpl)
GO(cblas_dzamin, dFlpl)
GO(cblas_dzasum, dFlpl)
GO(cblas_dznrm2, dFlpl)
GO(cblas_dzsum, dFlpl)
GO(cblas_icamax, LFlpl)
GO(cblas_icamin, LFlpl)
GO(cblas_icmax, LFlpl)
GO(cblas_icmin, LFlpl)
GO(cblas_idamax, LFlpl)
GO(cblas_idamin, LFlpl)
GO(cblas_idmax, LFlpl)
GO(cblas_idmin, LFlpl)
GO(cblas_isamax, LFlpl)
GO(cblas_isamin, LFlpl)
GO(cblas_ismax, LFlpl)
GO(cblas_ismin, LFlpl)
GO(cblas_izamax, LFlpl)
GO(cblas_izamin, LFlpl)
GO(cblas_izmax, LFlpl)
GO(cblas_izmin, LFlpl)
GO(cblas_samax, fFlpl)
GO(cblas_samin, fFlpl)
GO(cblas_sasum, fFlpl)
GO(cblas_saxpby, vFlfplfpl)
GO(cblas_saxpy, vFlfplpl)
GO(cblas_scamax, fFlpl)
GO(cblas_scamin, fFlpl)
GO(cblas_scasum, fFlpl)
GO(cblas_scnrm2, fFlpl)
GO(cblas_scopy, vFlplpl)
GO(cblas_scsum, fFlpl)
GO(cblas_sdot, fFlplpl)
GO(cblas_sdsdot, fFlfplpl)
GO(cblas_sgbmv, vFuullllfplplfpl)
GO(cblas_sgeadd, vFullfplfpl)
GO(cblas_sgemm, vFuuulllfplplfpl)
GO(cblas_sgemm_batch, vFuppppppppppppplp)
GO(cblas_sgemmt, vFuuuullfplplfpl)
//GO(cblas_sgemmtr, 
GO(cblas_sgemv, vFuullfplplfpl)
GO(cblas_sger, vFullfplplpl)
GO(cblas_simatcopy, vFuullfpll)
GO(cblas_snrm2, fFlpl)
GO(cblas_somatcopy, vFuullfplpl)
GO(cblas_srot, vFlplplff)
GO(cblas_srotg, vFpppp)
GO(cblas_srotm, vFlplplp)
GO(cblas_srotmg, vFpppfp)
GO(cblas_ssbmv, vFuullfplplfpl)
GO(cblas_sscal, vFlfpl)
GO(cblas_sspmv, vFuulfpplfpl)
GO(cblas_sspr, vFuulfplp)
GO(cblas_sspr2, vFuulfplplp)
GO(cblas_ssum, fFlpl)
GO(cblas_sswap, vFlplpl)
GO(cblas_ssymm, vFuuullfplplfpl)
GO(cblas_ssymv, vFuulfplplfpl)
GO(cblas_ssyr, vFuulfplpl)
GO(cblas_ssyr2, vFuulfplplpl)
GO(cblas_ssyr2k, vFuuullfplplfpl)
GO(cblas_ssyrk, vFuuullfplfpl)
GO(cblas_stbmv, vFuuuullplpl)
GO(cblas_stbsv, vFuuuullplpl)
GO(cblas_stpmv, vFuuuulppl)
GO(cblas_stpsv, vFuuuulppl)
GO(cblas_strmm, vFuuuuullfplpl)
GO(cblas_strmv, vFuuuulplpl)
GO(cblas_strsm, vFuuuuullfplpl)
GO(cblas_strsv, vFuuuulplpl)
GO(cblas_zaxpby, vFlpplppl)
GO(cblas_zaxpy, vFlpplpl)
GO(cblas_zaxpyc, vFlpplpl)
GO(cblas_zcopy, vFlplpl)
GO(cblas_zdotc, XFlplpl)
GO(cblas_zdotc_sub, vFlplplp)
GO(cblas_zdotu, XFlplpl)
GO(cblas_zdotu_sub, vFlplplp)
GO(cblas_zdrot, vFlplpldd)
GO(cblas_zdscal, vFldpl)
GO(cblas_zgbmv, vFuullllpplplppl)
GO(cblas_zgeadd, vFullpplppl)
GO(cblas_zgemm, vFuuulllpplplppl)
GO(cblas_zgemm_batch, vFuppppppppppppplp)
GO(cblas_zgemmt, vFuuuullpplplppl)
//GO(cblas_zgemmtr, 
GO(cblas_zgemv, vFuullpplplppl)
GO(cblas_zgerc, vFullpplplpl)
GO(cblas_zgeru, vFullpplplpl)
GO(cblas_zhbmv, vFuullpplplppl)
GO(cblas_zhemm, vFuuullpplplppl)
GO(cblas_zhemv, vFuulpplplppl)
GO(cblas_zher, vFuuldplpl)
GO(cblas_zher2, vFuulpplplpl)
GO(cblas_zher2k, vFuuullpplpldpl)
GO(cblas_zherk, vFuuulldpldpl)
GO(cblas_zhpmv, vFuulppplppl)
GO(cblas_zhpr, vFuuldplp)
GO(cblas_zhpr2, vFuulpplplp)
GO(cblas_zimatcopy, vFuullppll)
GO(cblas_zomatcopy, vFuullpplpl)
GO(cblas_zrotg, vFpppp)
GO(cblas_zscal, vFlppl)
GO(cblas_zswap, vFlplpl)
GO(cblas_zsymm, vFuuullpplplppl)
GO(cblas_zsyr2k, vFuuullpplplppl)
GO(cblas_zsyrk, vFuuullpplppl)
GO(cblas_ztbmv, vFuuuullplpl)
GO(cblas_ztbsv, vFuuuullplpl)
GO(cblas_ztpmv, vFuuuulppl)
GO(cblas_ztpsv, vFuuuulppl)
GO(cblas_ztrmm, vFuuuuullpplpl)
GO(cblas_ztrmv, vFuuuulplpl)
GO(cblas_ztrsm, vFuuuuullpplpl)
GO(cblas_ztrsv, vFuuuulplpl)
