#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(xcb_create_pixmap_from_bitmap_data, uFbupuuuuup)
GO(xcb_image_annotate, vFp)
GO(xcb_image_convert, pFpp)
GO(xcb_image_create, pFWWuCCCCuupup)
GO(xcb_image_create_from_bitmap_data, pFpuu)
GO(xcb_image_create_native, pFbWWuCpup)
GO(xcb_image_destroy, vFp)
GO(xcb_image_get, pFbuwwWWuu)
GO(xcb_image_get_pixel, uFpuu)
GO(xcb_image_native, pFbpi)
GO(xcb_image_put, UFbuupwwC)
GO(xcb_image_put_pixel, vFpuuu)
GO(xcb_image_shm_get, iFbupppWWu)    //xcb_shm_segment_info_t is a struct with "u u p" => transform to pp?
GO(xcb_image_shm_put, pFbuupppwwwwWWC)
GO(xcb_image_subimage, pFpuuuupup)
