#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_72, pFpp)
GO(ubrk_close_72, vFp)
GO(ubrk_countAvailable_72, iFv)
GO(ubrk_current_72, iFp)
GO(ubrk_first_72, iFp)
GO(ubrk_following_72, iFpi)
GO(ubrk_getAvailable_72, pFi)
GO(ubrk_getBinaryRules_72, iFppip)
GO(ubrk_getLocaleByType_72, pFpip)
GO(ubrk_getRuleStatus_72, iFp)
GO(ubrk_getRuleStatusVec_72, iFppip)
GO(ubrk_isBoundary_72, cFpi)
GO(ubrk_last_72, iFp)
GO(ubrk_next_72, iFp)
GO(ubrk_open_72, pFppip)
GO(ubrk_openBinaryRules_72, pFpipip)
GO(ubrk_openRules_72, pFpipipp)
GO(ubrk_preceding_72, iFpi)
GO(ubrk_previous_72, iFp)
GO(ubrk_refreshUText_72, vFppp)
GO(ubrk_safeClone_72, pFpppp)
GO(ubrk_setText_72, vFppip)
GO(ubrk_setUText_72, vFppp)
GO(ubrk_swap_72, iFppipp)

GO(ucnv_getDefaultName_72, pFv)
GO(ucnv_open_72, pFpp)
GO(ucnv_setDefaultName_72, vFp)

GO(ucurr_forLocale_72, iFppip)
GO(ucurr_getName_72, pFppippp)

GO(uloc_canonicalize_72, iFppip)
GO(uloc_countAvailable_72, iFv)
GO(uloc_getAvailable_72, pFi)
GO(uloc_getBaseName_72, iFppip)
GO(uloc_getCharacterOrientation_72, iFpp)
GO(uloc_getCountry_72, iFppip)
GO(uloc_getDefault_72, pFv)
GO(uloc_getDisplayCountry_72, iFpppip)
GO(uloc_getDisplayLanguage_72, iFpppip)
GO(uloc_getDisplayName_72, iFpppip)
GO(uloc_getISO3Country_72, pFp)
GO(uloc_getISO3Language_72, pFp)
GO(uloc_getKeywordValue_72, iFpppip)
GO(uloc_getLanguage_72, iFppip)
GO(uloc_getLCID_72, uFp)
GO(uloc_getName_72, iFppip)
GO(uloc_getParent_72, iFppip)
GO(uloc_setKeywordValue_72, iFpppip)

GO(ures_close_72, vFp)
GO(ures_getByKey_72, pFpppp)
GO(ures_getSize_72, iFp)
GO(ures_getStringByIndex_72, pFpipp)
GO(ures_open_72, pFppp)

GO(uldn_close_72, vFp)
GO(uldn_keyValueDisplayName_72, iFppppip)
GO(uldn_open_72, pFpip)

GO(uenum_close_72, vFp)
GO(uenum_count_72, iFpp)
GO(uenum_next_72, pFppp)

GO(uidna_close_72, vFp)
GO(uidna_nameToASCII_72, iFppipipp)
GO(uidna_nameToUnicode_72, iFppipipp)
GO(uidna_openUTS46_72, pFpp)

GO(unorm2_getNFCInstance_72, pFp)
GO(unorm2_getNFDInstance_72, pFp)
GO(unorm2_getNFKCInstance_72, pFp)
GO(unorm2_getNFKDInstance_72, pFp)
GO(unorm2_isNormalized_72, CFppip)
GO(unorm2_normalize_72, iFppipip)

GO(u_charsToUChars_72, vFppi)
GO(u_getVersion_72, vFp)
GO(u_strcmp_72, iFpp)
GO(u_strcpy_72, pFpp)
GO(u_strlen_72, iFp)
GO(u_strncpy_72, pFppi)
GO(u_strToLower_72, iFpipipp)
GO(u_strToUpper_72, iFpipipp)
GO(u_tolower_72, uFu)
GO(u_toupper_72, uFu)
GO(u_uastrncpy_72, pFppi)
