#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GO(PL_Base64Decode, 
//GO(PL_Base64Encode, 
//GO(PL_CreateLongOptState, 
//GO(PL_CreateOptState, 
//GO(PL_DestroyOptState, 
//GO(PL_FPrintError, 
//GO(PL_GetNextOpt, 
//GO(PL_PrintError, 
//GO(PL_strcasecmp, 
//GO(PL_strcaserstr, 
//GO(PL_strcasestr, 
//GO(PL_strcat, 
//GO(PL_strcatn, 
//GO(PL_strchr, 
//GO(PL_strcmp, 
//GO(PL_strcpy, 
//GO(PL_strdup, 
//GO(PL_strfree, 
//GO(PL_strlen, 
//GO(PL_strncasecmp, 
//GO(PL_strncaserstr, 
//GO(PL_strncasestr, 
//GO(PL_strncat, 
//GO(PL_strnchr, 
//GO(PL_strncmp, 
//GO(PL_strncpy, 
//GO(PL_strncpyz, 
//GO(PL_strndup, 
//GO(PL_strnlen, 
//GO(PL_strnpbrk, 
//GO(PL_strnprbrk, 
//GO(PL_strnrchr, 
//GO(PL_strnrstr, 
//GO(PL_strnstr, 
//GO(PL_strpbrk, 
//GO(PL_strprbrk, 
//GO(PL_strrchr, 
//GO(PL_strrstr, 
//GO(PL_strstr, 
//GO(PL_strtok_r, 