/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedgnutlsTYPES32_H_
#define __wrappedgnutlsTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void (*vFp_t)(void*);
typedef void (*vFpp_t)(void*, void*);
typedef void* (*pFpp_t)(void*, void*);
typedef int32_t (*iFppppppppu_t)(void*, void*, void*, void*, void*, void*, void*, void*, uint32_t);

#define SUPER() ADDED_FUNCTIONS() \
	GO(gnutls_global_set_log_function, vFp_t) \
	GO(gnutls_transport_set_pull_function, vFpp_t) \
	GO(gnutls_transport_set_pull_timeout_function, vFpp_t) \
	GO(gnutls_transport_set_push_function, vFpp_t) \
	GO(gnutls_certificate_get_peers, pFpp_t) \
	GO(gnutls_pkcs12_simple_parse, iFppppppppu_t)

#endif // __wrappedgnutlsTYPES32_H_
