#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(avahi_address_cmp, iFpp)
GO(avahi_address_parse, pFpip)
GO(avahi_address_snprint, pFpLp)
GO(avahi_af_to_proto, iFi)
GO(avahi_age, IFp)
GO(avahi_alternative_host_name, pFp)
GO(avahi_alternative_service_name, pFp)
GO(avahi_domain_equal, iFpp)
GO(avahi_domain_hash, uFp)
GO(avahi_elapse_time, pFpuu)
GO(avahi_escape_label, pFpLpp)
GO(avahi_free, vFp)
GO(avahi_get_type_from_subtype, pFp)
GO(avahi_init_i18n, vFv)
GO(avahi_is_valid_domain_name, iFp)
GO(avahi_is_valid_fqdn, iFp)
GO(avahi_is_valid_host_name, iFp)
GO(avahi_is_valid_service_name, iFp)
GO(avahi_is_valid_service_subtype, iFp)
GO(avahi_is_valid_service_type_generic, iFp)
GO(avahi_is_valid_service_type_strict, iFp)
GO(avahi_malloc, pFL)
GO(avahi_malloc0, pFL)
GO(avahi_memdup, pFpL)
GO(avahi_normalize_name, pFppL)
GO(avahi_normalize_name_strdup, pFp)
GO(avahi_proto_to_af, iFi)
GO(avahi_proto_to_string, pFi)
GO(avahi_realloc, pFpL)
GO(avahi_reverse_lookup_name, pFppL)
GO(avahi_rlist_prepend, pFpp)
GO(avahi_rlist_remove, pFpp)
GO(avahi_rlist_remove_by_link, pFpp)
GO(avahi_service_name_join, iFpLppp)
GO(avahi_service_name_split, iFppLpLpL)
GOM(avahi_set_allocator, vFEp)
GO(avahi_simple_poll_dispatch, iFp)
GO(avahi_simple_poll_free, vFp)
GO(avahi_simple_poll_get, pFp)
GO(avahi_simple_poll_iterate, iFpi)
GO(avahi_simple_poll_loop, iFp)
GO(avahi_simple_poll_new, pFv)
GO(avahi_simple_poll_prepare, iFpi)
GO(avahi_simple_poll_quit, vFp)
GO(avahi_simple_poll_run, iFp)
GOM(avahi_simple_poll_set_func, vFEppp)
GO(avahi_simple_poll_wakeup, vFp)
GO(avahi_strdup, pFp)
GOM(avahi_strdup_printf, pFEpV)
GOM(avahi_strdup_vprintf, pFEpA)
GO(avahi_strerror, pFi)
GO(avahi_string_list_add, pFpp)
GO(avahi_string_list_add_anonymous, pFpL)
GO(avahi_string_list_add_arbitrary, pFppL)
GOM(avahi_string_list_add_many, pFEpV)
GOM(avahi_string_list_add_many_va, pFEpA)
GO(avahi_string_list_add_pair, pFppp)
GO(avahi_string_list_add_pair_arbitrary, pFpppL)
GOM(avahi_string_list_add_printf, pFEppV)
GOM(avahi_string_list_add_vprintf, pFEppA)
GO(avahi_string_list_copy, pFp)
GO(avahi_string_list_equal, iFpp)
GO(avahi_string_list_find, pFpp)
GO(avahi_string_list_free, vFp)
GO(avahi_string_list_get_next, pFp)
GO(avahi_string_list_get_pair, iFpppp)
GO(avahi_string_list_get_service_cookie, uFp)
GO(avahi_string_list_get_size, LFp)
GO(avahi_string_list_get_text, pFp)
GO(avahi_string_list_length, uFp)
GOM(avahi_string_list_new, pFEpV)
GO(avahi_string_list_new_from_array, pFpi)
GOM(avahi_string_list_new_va, pFEA)
GO(avahi_string_list_parse, iFpLp)
GO(avahi_string_list_reverse, pFp)
GO(avahi_string_list_serialize, LFppL)
GO(avahi_string_list_to_string, pFp)
GO(avahi_strndup, pFpL)
GO(avahi_threaded_poll_free, vFp)
GO(avahi_threaded_poll_get, pFp)
GO(avahi_threaded_poll_lock, vFp)
GO(avahi_threaded_poll_new, pFv)
GO(avahi_threaded_poll_quit, vFp)
GO(avahi_threaded_poll_start, iFp)
GO(avahi_threaded_poll_stop, iFp)
GO(avahi_threaded_poll_unlock, vFp)
GO(avahi_timeval_add, pFpI)
GO(avahi_timeval_compare, iFpp)
GO(avahi_timeval_diff, IFpp)
