#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_68, pFpp)
GO(ubrk_close_68, vFp)
GO(ubrk_countAvailable_68, iFv)
GO(ubrk_current_68, iFp)
GO(ubrk_first_68, iFp)
GO(ubrk_following_68, iFpi)
GO(ubrk_getAvailable_68, pFi)
GO(ubrk_getBinaryRules_68, iFppip)
GO(ubrk_getLocaleByType_68, pFpip)
GO(ubrk_getRuleStatus_68, iFp)
GO(ubrk_getRuleStatusVec_68, iFppip)
GO(ubrk_isBoundary_68, cFpi)
GO(ubrk_last_68, iFp)
GO(ubrk_next_68, iFp)
GO(ubrk_open_68, pFppip)
GO(ubrk_openBinaryRules_68, pFpipip)
GO(ubrk_openRules_68, pFpipipp)
GO(ubrk_preceding_68, iFpi)
GO(ubrk_previous_68, iFp)
GO(ubrk_refreshUText_68, vFppp)
GO(ubrk_safeClone_68, pFpppp)
GO(ubrk_setText_68, vFppip)
GO(ubrk_setUText_68, vFppp)
GO(ubrk_swap_68, iFppipp)

GO(ucnv_getDefaultName_68, pFv)
GO(ucnv_open_68, pFpp)
GO(ucnv_setDefaultName_68, vFp)

GO(ucurr_forLocale_68, iFppip)
GO(ucurr_getName_68, pFppippp)

GO(uloc_canonicalize_68, iFppip)
GO(uloc_countAvailable_68, iFv)
GO(uloc_getAvailable_68, pFi)
GO(uloc_getBaseName_68, iFppip)
GO(uloc_getCharacterOrientation_68, iFpp)
GO(uloc_getCountry_68, iFppip)
GO(uloc_getDefault_68, pFv)
GO(uloc_getDisplayCountry_68, iFpppip)
GO(uloc_getDisplayLanguage_68, iFpppip)
GO(uloc_getDisplayName_68, iFpppip)
GO(uloc_getISO3Country_68, pFp)
GO(uloc_getISO3Language_68, pFp)
GO(uloc_getKeywordValue_68, iFpppip)
GO(uloc_getLanguage_68, iFppip)
GO(uloc_getLCID_68, uFp)
GO(uloc_getName_68, iFppip)
GO(uloc_getParent_68, iFppip)
GO(uloc_setKeywordValue_68, iFpppip)

GO(ures_close_68, vFp)
GO(ures_getByKey_68, pFpppp)
GO(ures_getSize_68, iFp)
GO(ures_getStringByIndex_68, pFpipp)
GO(ures_open_68, pFppp)

GO(uldn_close_68, vFp)
GO(uldn_keyValueDisplayName_68, iFppppip)
GO(uldn_open_68, pFpip)

GO(uenum_close_68, vFp)
GO(uenum_count_68, iFpp)
GO(uenum_next_68, pFppp)

GO(uidna_close_68, vFp)
GO(uidna_nameToASCII_68, iFppipipp)
GO(uidna_nameToUnicode_68, iFppipipp)
GO(uidna_openUTS46_68, pFpp)

GO(unorm2_getNFCInstance_68, pFp)
GO(unorm2_getNFDInstance_68, pFp)
GO(unorm2_getNFKCInstance_68, pFp)
GO(unorm2_getNFKDInstance_68, pFp)
GO(unorm2_isNormalized_68, CFppip)
GO(unorm2_normalize_68, iFppipip)

GO(u_charsToUChars_68, vFppi)
GO(u_getVersion_68, vFp)
GO(u_strcmp_68, iFpp)
GO(u_strcpy_68, pFpp)
GO(u_strlen_68, iFp)
GO(u_strncpy_68, pFppi)
GO(u_strToLower_68, iFpipipp)
GO(u_strToUpper_68, iFpipipp)
GO(u_tolower_68, uFu)
GO(u_toupper_68, uFu)
GO(u_uastrncpy_68, pFppi)
