#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GOM(pcre2_callout_enumerate_16, iFEppp)
GO(pcre2_code_copy_16, pFp)
GO(pcre2_code_copy_with_tables_16, pFp)
GO(pcre2_code_free_16, vFp)
GO(pcre2_compile_16, pFpLuppp)
GO(pcre2_compile_context_copy_16, pFp)
GO(pcre2_compile_context_create_16, pFp)
GO(pcre2_compile_context_free_16, vFp)
GO(pcre2_config_16, iFup)
GO(pcre2_convert_context_copy_16, pFp)
GO(pcre2_convert_context_create_16, pFp)
GO(pcre2_convert_context_free_16, vFp)
GO(pcre2_converted_pattern_free_16, vFp)
GO(pcre2_dfa_match_16, iFppLLupppL)
GO(pcre2_general_context_copy_16, pFp)
GOM(pcre2_general_context_create_16, pFEppp)
GO(pcre2_general_context_free_16, vFp)
GO(pcre2_get_error_message_16, iFipL)
GO(pcre2_get_mark_16, pFp)
GO(pcre2_get_match_data_heapframes_size_16, LFp)
GO(pcre2_get_match_data_size_16, LFp)
GO(pcre2_get_ovector_count_16, uFp)
GO(pcre2_get_ovector_pointer_16, pFp)
GO(pcre2_get_startchar_16, LFp)
GO(pcre2_jit_compile_16, iFpu)
GO(pcre2_jit_free_unused_memory_16, vFp)
GO(pcre2_jit_match_16, iFppLLupp)
GOM(pcre2_jit_stack_assign_16, vFEppp)
GO(pcre2_jit_stack_create_16, pFLLp)
GO(pcre2_jit_stack_free_16, vFp)
GO(pcre2_maketables_16, pFp)
GO(pcre2_maketables_free_16, vFpp)
GO(pcre2_match_16, iFppLLupp)
GO(pcre2_match_context_copy_16, pFp)
GO(pcre2_match_context_create_16, pFp)
GO(pcre2_match_context_free_16, vFp)
GO(pcre2_match_data_create_16, pFup)
GO(pcre2_match_data_create_from_pattern_16, pFpp)
GO(pcre2_match_data_free_16, vFp)
GO(pcre2_pattern_convert_16, iFpLuppp)
GO(pcre2_pattern_info_16, iFpup)
GO(pcre2_serialize_decode_16, iFpipp)
GO(pcre2_serialize_encode_16, iFpippp)
GO(pcre2_serialize_free_16, vFp)
GO(pcre2_serialize_get_number_of_codes_16, iFp)
GO(pcre2_set_bsr_16, iFpu)
GOM(pcre2_set_callout_16, iFEppp)
GO(pcre2_set_character_tables_16, iFpp)
GO(pcre2_set_compile_extra_options_16, iFpu)
GOM(pcre2_set_compile_recursion_guard_16, iFEppp)
GO(pcre2_set_depth_limit_16, iFpu)
GO(pcre2_set_glob_escape_16, iFpu)
GO(pcre2_set_glob_separator_16, iFpu)
GO(pcre2_set_heap_limit_16, iFpu)
GO(pcre2_set_match_limit_16, iFpu)
GO(pcre2_set_max_pattern_compiled_length_16, iFpL)
GO(pcre2_set_max_pattern_length_16, iFpL)
GO(pcre2_set_max_varlookbehind_16, iFpu)
GO(pcre2_set_newline_16, iFpu)
GO(pcre2_set_offset_limit_16, iFpL)
GO(pcre2_set_optimize_16, iFpu)
GO(pcre2_set_parens_nest_limit_16, iFpu)
GO(pcre2_set_recursion_limit_16, iFpu)
GOM(pcre2_set_recursion_memory_management_16, iFEpppp)
GOM(pcre2_set_substitute_callout_16, iFEppp)
GOM(pcre2_set_substitute_case_callout_16, iFEppp)
GO(pcre2_substitute_16, iFppLLupppLpp)
GO(pcre2_substring_copy_byname_16, iFpppp)
GO(pcre2_substring_copy_bynumber_16, iFpupp)
GO(pcre2_substring_free_16, vFp)
GO(pcre2_substring_get_byname_16, iFpppp)
GO(pcre2_substring_get_bynumber_16, iFpupp)
GO(pcre2_substring_length_byname_16, iFppp)
GO(pcre2_substring_length_bynumber_16, iFpup)
GO(pcre2_substring_list_free_16, vFp)
GO(pcre2_substring_list_get_16, iFppp)
GO(pcre2_substring_nametable_scan_16, iFpppp)
GO(pcre2_substring_number_from_name_16, iFpp)
