#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GOM(vdp_device_create_x11, iFEpipp) // last is a pointer to a VdpGetProcAddress function

// dummy signatures for getprocaddress
GO(dummy_iFuuppp, iFuuppp)
GO(dummy_iFuuup, iFuuup)
GO(dummy_iFuuuup, iFuuuup)
GO(dummy_iFuupppup, iFuupppup)
GO(dummy_iFuupppp, iFuupppp)
GO(dummy_iFuuuuip, iFuuuuip)
GO(dummy_iFupppp, iFupppp)
GO(dummy_iFupupppu, iFupupppu)
GO(dummy_iFuuppppp, iFuuppppp)
GO(dummy_iFuuuuup, iFuuuuup)
GO(dummy_iFuupup, iFuupup)
GO(dummy_iFuupuppp, iFuupuppp)
GO(dummy_iFuupiupuuppuppup, iFuupiupuuppuppup)
GO(dummy_iFuuuuU, iFuuuuU)
