#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(decode_krb5_ad_kdcissued, 
//GO(decode_krb5_ad_signedpath, 
//GO(decode_krb5_ap_rep, 
//GO(decode_krb5_ap_rep_enc_part, 
//GO(decode_krb5_ap_req, 
//GO(decode_krb5_as_rep, 
//GO(decode_krb5_as_req, 
//GO(decode_krb5_authdata, 
//GO(decode_krb5_authenticator, 
//GO(decode_krb5_cred, 
//GO(decode_krb5_enc_cred_part, 
//GO(decode_krb5_enc_data, 
//GO(decode_krb5_enc_kdc_rep_part, 
//GO(decode_krb5_enc_priv_part, 
//GO(decode_krb5_encryption_key, 
//GO(decode_krb5_enc_sam_response_enc_2, 
//GO(decode_krb5_enc_tkt_part, 
//GO(decode_krb5_error, 
//GO(decode_krb5_etype_info, 
//GO(decode_krb5_etype_info2, 
//GO(decode_krb5_fast_req, 
//GO(decode_krb5_fast_response, 
//GO(decode_krb5_iakerb_finished, 
//GO(decode_krb5_iakerb_header, 
//GO(decode_krb5_kdc_req_body, 
//GO(decode_krb5_kkdcp_message, 
//GO(decode_krb5_otp_tokeninfo, 
//GO(decode_krb5_padata_sequence, 
//GO(decode_krb5_pa_enc_ts, 
//GO(decode_krb5_pa_for_user, 
//GO(decode_krb5_pa_fx_fast_reply, 
//GO(decode_krb5_pa_fx_fast_request, 
//GO(decode_krb5_pa_otp_challenge, 
//GO(decode_krb5_pa_otp_enc_req, 
//GO(decode_krb5_pa_otp_req, 
//GO(decode_krb5_pa_pac_req, 
//GO(decode_krb5_pa_s4u_x509_user, 
//GO(decode_krb5_priv, 
//GO(decode_krb5_safe, 
//GO(decode_krb5_sam_challenge_2, 
//GO(decode_krb5_sam_challenge_2_body, 
//GO(decode_krb5_sam_response_2, 
//GO(decode_krb5_setpw_req, 
//GO(decode_krb5_tgs_rep, 
//GO(decode_krb5_tgs_req, 
//GO(decode_krb5_ticket, 
//GO(decode_krb5_typed_data, 
//GO(encode_krb5_ad_kdcissued, 
//GO(encode_krb5_ad_signedpath, 
//GO(encode_krb5_ad_signedpath_data, 
//GO(encode_krb5_ap_rep, 
//GO(encode_krb5_ap_rep_enc_part, 
//GO(encode_krb5_ap_req, 
//GO(encode_krb5_as_rep, 
//GO(encode_krb5_as_req, 
//GO(encode_krb5_authdata, 
//GO(encode_krb5_authenticator, 
//GO(encode_krb5_checksum, 
//GO(encode_krb5_cred, 
//GO(encode_krb5_enc_cred_part, 
//GO(encode_krb5_enc_data, 
//GO(encode_krb5_enc_kdc_rep_part, 
//GO(encode_krb5_enc_priv_part, 
//GO(encode_krb5_encryption_key, 
//GO(encode_krb5_enc_sam_response_enc_2, 
//GO(encode_krb5_enc_tkt_part, 
//GO(encode_krb5_error, 
//GO(encode_krb5_etype_info, 
//GO(encode_krb5_etype_info2, 
//GO(encode_krb5_fast_response, 
//GO(encode_krb5_iakerb_finished, 
//GO(encode_krb5_iakerb_header, 
//GO(encode_krb5_kdc_req_body, 
//GO(encode_krb5_kkdcp_message, 
//GO(encode_krb5_otp_tokeninfo, 
//GO(encode_krb5_padata_sequence, 
//GO(encode_krb5_pa_enc_ts, 
//GO(encode_krb5_pa_for_user, 
//GO(encode_krb5_pa_fx_fast_reply, 
//GO(encode_krb5_pa_otp_challenge, 
//GO(encode_krb5_pa_otp_enc_req, 
//GO(encode_krb5_pa_otp_req, 
//GO(encode_krb5_pa_s4u_x509_user, 
//GO(encode_krb5_pkinit_supp_pub_info, 
//GO(encode_krb5_priv, 
//GO(encode_krb5_s4u_userid, 
//GO(encode_krb5_safe, 
//GO(encode_krb5_sam_challenge_2, 
//GO(encode_krb5_sam_challenge_2_body, 
//GO(encode_krb5_sam_response_2, 
//GO(encode_krb5_sp80056a_other_info, 
//GO(encode_krb5_tgs_rep, 
//GO(encode_krb5_tgs_req, 
//GO(encode_krb5_ticket, 
//GO(encode_krb5_typed_data, 
//DATA(et_asn1_error_table, 
//DATA(et_k524_error_table, 
//DATA(et_kdb5_error_table, 
//DATA(et_krb5_error_table, 
//DATA(et_kv5m_error_table, 
//DATA(et_prof_error_table, 
//GO(initialize_asn1_error_table, 
//GO(initialize_k524_error_table, 
//GO(initialize_k5e1_error_table, 
//GO(initialize_kdb5_error_table, 
//GO(initialize_krb5_error_table, 
//GO(initialize_kv5m_error_table, 
//GO(initialize_prof_error_table, 
//GO(k5_build_conf_principals, 
//GO(k5_ccselect_free_context, 
//GO(k5_change_error_message_code, 
//GO(k5_etypes_contains, 
//GO(k5_expand_path_tokens, 
//GO(k5_expand_path_tokens_extra, 
//GO(k5_free_algorithm_identifier, 
//GO(k5_free_kkdcp_message, 
//GO(k5_free_otp_tokeninfo, 
//GO(k5_free_pa_otp_challenge, 
//GO(k5_free_pa_otp_req, 
//GO(k5_free_serverlist, 
//GO(k5_hostrealm_free_context, 
//GO(k5_init_trace, 
//GO(k5_kt_get_principal, 
//GO(k5_localauth_free_context, 
//GO(k5_locate_kdc, 
//GO(k5_marshal_cred, 
//GO(k5_marshal_princ, 
//GO(k5_os_free_context, 
//GO(k5_os_init_context, 
//GO(k5_plugin_free_modules, 
//GO(k5_plugin_load, 
//GO(k5_plugin_load_all, 
//GO(k5_plugin_register, 
//GO(k5_plugin_register_dyn, 
//GO(k5_unmarshal_cred, 
//GO(k5_unmarshal_princ, 
//GO(krb524_convert_creds_kdc, 
//GO(krb524_init_ets, 
//GO(krb5_425_conv_principal, 
//GO(krb5_524_convert_creds, 
//GO(krb5_524_conv_principal, 
//GO(krb5_address_compare, 
//GO(krb5_address_order, 
//GO(krb5_address_search, 
//GO(krb5_allow_weak_crypto, 
//GO(krb5_aname_to_localname, 
//GO(krb5_anonymous_principal, 
//GO(krb5_anonymous_realm, 
//GO(krb5_appdefault_boolean, 
//GO(krb5_appdefault_string, 
//GO(krb5_auth_con_free, 
//GO(krb5_auth_con_genaddrs, 
//GO(krb5_auth_con_getaddrs, 
//GO(krb5_auth_con_get_authdata_context, 
//GO(krb5_auth_con_getauthenticator, 
//GO(krb5_auth_con_get_checksum_func, 
//GO(krb5_auth_con_getflags, 
//GO(krb5_auth_con_getivector, 
//GO(krb5_auth_con_getkey, 
//GO(krb5_auth_con_getkey_k, 
//GO(krb5_auth_con_getlocalseqnumber, 
//GO(krb5_auth_con_getlocalsubkey, 
//GO(krb5_auth_con_getpermetypes, 
//GO(krb5_auth_con_getrcache, 
//GO(krb5_auth_con_getrecvsubkey, 
//GO(krb5_auth_con_getrecvsubkey_k, 
//GO(krb5_auth_con_getremoteseqnumber, 
//GO(krb5_auth_con_getremotesubkey, 
//GO(krb5_auth_con_getsendsubkey, 
//GO(krb5_auth_con_getsendsubkey_k, 
//GO(krb5_auth_con_init, 
//GO(krb5_auth_con_initivector, 
//GO(krb5_auth_con_setaddrs, 
//GO(krb5_auth_con_set_authdata_context, 
//GO(krb5_auth_con_set_checksum_func, 
//GO(krb5_auth_con_setflags, 
//GO(krb5_auth_con_setivector, 
//GO(krb5_auth_con_setpermetypes, 
//GO(krb5_auth_con_setports, 
//GO(krb5_auth_con_setrcache, 
//GO(krb5_auth_con_setrecvsubkey, 
//GO(krb5_auth_con_setrecvsubkey_k, 
//GO(krb5_auth_con_set_req_cksumtype, 
//GO(krb5_auth_con_set_safe_cksumtype, 
//GO(krb5_auth_con_setsendsubkey, 
//GO(krb5_auth_con_setsendsubkey_k, 
//GO(krb5_auth_con_setuseruserkey, 
//GO(krb5_authdata_context_copy, 
//GO(krb5_authdata_context_free, 
//GO(krb5_authdata_context_init, 
//GO(krb5_authdata_delete_attribute, 
//GO(krb5_authdata_export_attributes, 
//GO(krb5_authdata_export_authdata, 
//GO(krb5_authdata_export_internal, 
//GO(krb5_authdata_free_internal, 
//GO(krb5_authdata_get_attribute, 
//GO(krb5_authdata_get_attribute_types, 
//GO(krb5_authdata_import_attributes, 
//GO(krb5_authdata_set_attribute, 
//GO(krb5_auth_to_rep, 
//GO(krb5_build_principal, 
//GO(krb5_build_principal_alloc_va, 
//GO(krb5_build_principal_ext, 
//GO(krb5_build_principal_va, 
//GO(krb5_cc_cache_match, 
GO(krb5_cc_close, iFpp)
GO(krb5_cccol_cursor_free, iFpp)
GO(krb5_cccol_cursor_new, iFpp)
GO(krb5_cccol_cursor_next, iFppp)
//GO(krb5_cccol_have_content, 
//GO(krb5_cc_copy_creds, 
GO(krb5_cc_default, iFpp)
//GO(krb5_cc_default_name, 
//GO(krb5_cc_destroy, 
//DATA(krb5_cc_dfl_ops, 
//GO(krb5_cc_dup, 
GO(krb5_cc_end_seq_get, iFppp)
//DATA(krb5_cc_file_ops, 
//GO(krb5_cc_gen_new, 
//GO(krb5_cc_get_config, 
//GO(krb5_cc_get_full_name, 
//GO(krb5_cc_get_name, 
//GO(krb5_cc_get_principal, 
//GO(krb5_cc_get_type, 
GO(krb5_cc_initialize, iFppp)
//GO(krb5_cc_move, 
//GO(krb5_cc_new_unique, 
GO(krb5_cc_next_cred, iFpppp)
//GO(krb5_cc_register, 
//GO(krb5_cc_remove_cred, 
//GO(krb5_cc_resolve, 
//GO(krb5_cc_retrieve_cred, 
//GO(krb5_cc_select, 
//GO(krb5_cc_set_config, 
//GO(krb5_cc_set_default_name, 
//GO(krb5_cc_set_flags, 
GO(krb5_cc_start_seq_get, iFppp)
GO(krb5_cc_store_cred, iFppp)
//GO(krb5_cc_support_switch, 
//GO(krb5_cc_switch, 
//GO(krb5_change_cache, 
//GO(krb5_change_password, 
//GO(krb5_check_clockskew, 
//GO(krb5_check_transited_list, 
//GO(krb5_chpw_message, 
//GO(krb5_chpw_result_code_string, 
//GO(krb5_clear_error_message, 
//GO(_krb5_conf_boolean, 
//GO(krb5_copy_addr, 
//GO(krb5_copy_addresses, 
//GO(krb5_copy_authdata, 
//GO(krb5_copy_authenticator, 
//GO(krb5_copy_checksum, 
//GO(krb5_copy_context, 
//GO(krb5_copy_creds, 
//GO(krb5_copy_data, 
//GO(krb5_copy_error_message, 
//GO(krb5_copy_keyblock, 
//GO(krb5_copy_keyblock_contents, 
//GO(krb5_copy_principal, 
//GO(krb5_copy_ticket, 
//GO(krb5_crypto_us_timeofday, 
//GO(krb5_decode_authdata_container, 
GO(krb5_decode_ticket, iFpp)
//GO(krb5_decrypt_tkt_part, 
//GO(krb5_deltat_to_string, 
//GO(krb5_encode_authdata_container, 
//GO(krb5_encode_kdc_rep, 
//GO(krb5_encrypt_helper, 
//GO(krb5_encrypt_tkt_part, 
//GO(krb5_externalize_data, 
//GO(krb5_externalize_opaque, 
//DATA(krb5_fcc_ops, 
//GO(krb5_find_authdata, 
//GO(krb5_find_serializer, 
//GO(krb5_free_address, 
//GO(krb5_free_addresses, 
//GO(krb5_free_ad_kdcissued, 
//GO(krb5_free_ad_signedpath, 
//GO(krb5_free_ap_rep, 
//GO(krb5_free_ap_rep_enc_part, 
//GO(krb5_free_ap_req, 
//GO(krb5_free_authdata, 
//GO(krb5_free_authenticator, 
//GO(krb5_free_authenticator_contents, 
//GO(krb5_free_checksum, 
//GO(krb5_free_checksum_contents, 
//GO(krb5_free_config_files, 
GO(krb5_free_context, vFp)
//GO(krb5_free_cred, 
GO(krb5_free_cred_contents, vFpp)
//GO(krb5_free_cred_enc_part, 
//GO(krb5_free_creds, 
//GO(krb5_free_data, 
//GO(krb5_free_data_contents, 
//GO(krb5_free_default_realm, 
//GO(krb5_free_enc_data, 
//GO(krb5_free_enc_kdc_rep_part, 
//GO(krb5_free_enc_sam_response_enc_2, 
//GO(krb5_free_enc_sam_response_enc_2_contents, 
//GO(krb5_free_enc_tkt_part, 
//GO(krb5_free_enctypes, 
//GO(krb5_free_error, 
//GO(krb5_free_error_message, 
//GO(krb5_free_etype_info, 
//GO(krb5_free_fast_armored_req, 
//GO(krb5_free_fast_req, 
//GO(krb5_free_fast_response, 
//GO(krb5_free_host_realm, 
//GO(krb5_free_iakerb_finished, 
//GO(krb5_free_iakerb_header, 
//GO(krb5_free_kdc_rep, 
//GO(krb5_free_kdc_req, 
//GO(krb5_free_keyblock, 
//GO(krb5_free_keyblock_contents, 
//GO(krb5_free_keytab_entry_contents, 
//GO(krb5_free_last_req, 
//GO(krb5_free_octet_data, 
//GO(krb5_free_pa_data, 
//GO(krb5_free_pa_enc_ts, 
//GO(krb5_free_pa_for_user, 
//GO(krb5_free_pa_pac_req, 
//GO(krb5_free_pa_s4u_x509_user, 
GO(krb5_free_principal, vFpp)
//GO(krb5_free_priv, 
//GO(krb5_free_priv_enc_part, 
//GO(krb5_free_realm_tree, 
//GO(krb5_free_safe, 
//GO(krb5_free_sam_challenge_2, 
//GO(krb5_free_sam_challenge_2_body, 
//GO(krb5_free_sam_challenge_2_body_contents, 
//GO(krb5_free_sam_challenge_2_contents, 
//GO(krb5_free_sam_response_2, 
//GO(krb5_free_sam_response_2_contents, 
//GO(krb5_free_string, 
//GO(krb5_free_tgt_creds, 
GO(krb5_free_ticket, vFpp)
//GO(krb5_free_tickets, 
//GO(krb5_free_tkt_authent, 
GO(krb5_free_unparsed_name, vFpp)
//GO(krb5_fwd_tgt_creds, 
//GO(krb5_generate_seq_number, 
//GO(krb5_generate_subkey, 
//GO(krb5_gen_portaddr, 
//GO(krb5_gen_replay_name, 
//GO(krb5_get_credentials, 
//GO(krb5_get_credentials_for_proxy, 
//GO(krb5_get_credentials_for_user, 
//GO(krb5_get_credentials_renew, 
//GO(krb5_get_credentials_validate, 
//GO(krb5_get_cred_via_tkt, 
//GO(krb5_get_default_config_files, 
//GO(krb5_get_default_in_tkt_ktypes, 
//GO(krb5_get_default_realm, 
//GO(krb5_get_error_message, 
//GO(krb5_get_fallback_host_realm, 
//GO(krb5_get_host_realm, 
//GO(krb5_get_init_creds_keytab, 
GO(krb5_get_init_creds_opt_alloc, iFpp)
GO(krb5_get_init_creds_opt_free, vFpp)
//GO(krb5_get_init_creds_opt_free_pa, 
//GO(krb5_get_init_creds_opt_get_fast_flags, 
//GO(krb5_get_init_creds_opt_get_pa, 
//GO(krb5_get_init_creds_opt_init, 
//GO(krb5_get_init_creds_opt_set_address_list, 
//GO(krb5_get_init_creds_opt_set_anonymous, 
//GO(krb5_get_init_creds_opt_set_canonicalize, 
//GO(krb5_get_init_creds_opt_set_change_password_prompt, 
//GO(krb5_get_init_creds_opt_set_etype_list, 
//GO(krb5_get_init_creds_opt_set_expire_callback, 
//GO(krb5_get_init_creds_opt_set_fast_ccache, 
//GO(krb5_get_init_creds_opt_set_fast_ccache_name, 
//GO(krb5_get_init_creds_opt_set_fast_flags, 
//GO(krb5_get_init_creds_opt_set_forwardable, 
//GO(krb5_get_init_creds_opt_set_in_ccache, 
GO(krb5_get_init_creds_opt_set_out_ccache, iFppp)
//GO(krb5_get_init_creds_opt_set_pa, 
//GO(krb5_get_init_creds_opt_set_preauth_list, 
//GO(krb5_get_init_creds_opt_set_proxiable, 
//GO(krb5_get_init_creds_opt_set_renew_life, 
//GO(krb5_get_init_creds_opt_set_responder, 
//GO(krb5_get_init_creds_opt_set_salt, 
//GO(krb5_get_init_creds_opt_set_tkt_life, 
GOM(krb5_get_init_creds_password, iFEppppppipp)
//GO(krb5_get_in_tkt_with_keytab, 
//GO(krb5_get_in_tkt_with_password, 
//GO(krb5_get_in_tkt_with_skey, 
//GO(krb5_get_notification_message, 
//GO(krb5_get_permitted_enctypes, 
//GO(krb5_get_profile, 
//GO(krb5_get_prompt_types, 
//GO(krb5_get_realm_domain, 
//GO(krb5_get_renewed_creds, 
//GO(krb5_get_server_rcache, 
//GO(krb5_get_tgs_ktypes, 
//GO(krb5_get_time_offsets, 
//GO(krb5_get_validated_creds, 
GO(krb5_init_context, iFp)
//GO(krb5_init_context_profile, 
//GO(krb5_init_creds_free, 
//GO(krb5_init_creds_get, 
//GO(krb5_init_creds_get_creds, 
//GO(krb5_init_creds_get_error, 
//GO(krb5_init_creds_get_times, 
//GO(krb5_init_creds_init, 
//GO(krb5_init_creds_set_keytab, 
//GO(krb5_init_creds_set_password, 
//GO(krb5_init_creds_set_service, 
//GO(krb5_init_creds_step, 
//GO(krb5_init_keyblock, 
//GO(krb5_init_secure_context, 
//GO(krb5int_accessor, 
//GO(krb5int_cc_default, 
//GO(krb5int_cleanup_library, 
//GO(krb5int_copy_data_contents, 
//GO(krb5int_copy_data_contents_add0, 
//GO(krb5_internalize_opaque, 
//GO(krb5int_find_pa_data, 
//GO(krb5int_foreach_localaddr, 
//GO(krb5int_free_data_list, 
//GO(krb5int_get_authdata_containee_types, 
//GO(krb5int_init_context_kdc, 
//GO(krb5int_initialize_library, 
//GO(krb5int_parse_enctype_list, 
//GO(krb5int_random_string, 
//GO(krb5int_tgtname, 
//GO(krb5int_trace, 
GO(krb5_is_config_principal, uFpp)
//GO(krb5_is_permitted_enctype, 
//GO(krb5_is_referral_realm, 
//GO(krb5_is_thread_safe, 
//GO(krb5_kdc_rep_decrypt_proc, 
//GO(krb5_kt_add_entry, 
//GO(krb5_kt_client_default, 
//GO(krb5_kt_close, 
//GO(krb5_kt_default, 
//GO(krb5_kt_default_name, 
//DATA(krb5_kt_dfl_ops, 
//GO(krb5_kt_dup, 
//GO(krb5_kt_end_seq_get, 
//DATA(krb5_ktf_ops, 
//GO(krb5_kt_free_entry, 
//DATA(krb5_ktf_writable_ops, 
//GO(krb5_kt_get_entry, 
//GO(krb5_kt_get_name, 
//GO(krb5_kt_get_type, 
//GO(krb5_kt_have_content, 
//GO(krb5_kt_next_entry, 
//GO(krb5_kt_read_service_key, 
//GO(krb5_kt_register, 
//GO(krb5_kt_remove_entry, 
//GO(krb5_kt_resolve, 
//DATA(krb5_kts_ops, 
//GO(krb5_kt_start_seq_get, 
//GO(krb5_kuserok, 
//GO(krb5_lock_file, 
//GO(krb5_make_authdata_kdc_issued, 
//GO(krb5_make_fulladdr, 
//GO(krb5_make_full_ipaddr, 
//DATA(krb5_mcc_ops, 
//GO(krb5_merge_authdata, 
//GO(krb5_mk_1cred, 
//GO(krb5_mk_error, 
//GO(krb5_mk_ncred, 
//GO(krb5_mk_priv, 
//GO(krb5_mk_rep, 
//GO(krb5_mk_rep_dce, 
//GO(krb5_mk_req, 
//GO(krb5_mk_req_extended, 
//GO(krb5_mk_safe, 
//GO(krb5_net_read, 
//GO(krb5_net_write, 
//GO(krb5_os_localaddr, 
//DATAB(krb5_overridekeyname, 
//GO(krb5_pac_add_buffer, 
//GO(krb5_pac_free, 
//GO(krb5_pac_get_buffer, 
//GO(krb5_pac_get_types, 
//GO(krb5_pac_init, 
//GO(krb5_pac_parse, 
//GO(krb5_pac_sign, 
//GO(krb5_pac_verify, 
//GO(krb5_parse_name, 
GO(krb5_parse_name_flags, iFppip)
//GO(krb5_principal2salt, 
//GO(krb5_principal2salt_norealm, 
//GO(krb5_principal_compare, 
//GO(krb5_principal_compare_any_realm, 
//GO(krb5_principal_compare_flags, 
//GO(krb5_prompter_posix, 
//GO(krb5_rc_close, 
//GO(krb5_rc_default, 
//GO(krb5_rc_default_name, 
//GO(krb5_rc_default_type, 
//GO(krb5_rc_destroy, 
//GO(krb5_rc_dfl_close, 
//GO(krb5_rc_dfl_close_no_free, 
//GO(krb5_rc_dfl_destroy, 
//GO(krb5_rc_dfl_expunge, 
//GO(krb5_rc_dfl_get_name, 
//GO(krb5_rc_dfl_get_span, 
//GO(krb5_rc_dfl_init, 
//DATA(krb5_rc_dfl_ops, 
//GO(krb5_rc_dfl_recover, 
//GO(krb5_rc_dfl_resolve, 
//GO(krb5_rc_dfl_store, 
//GO(krb5_rc_expunge, 
//GO(krb5_rc_free_entry, 
//GO(krb5_rc_get_lifespan, 
//GO(krb5_rc_get_name, 
//GO(krb5_rc_get_type, 
//GO(krb5_rc_hash_message, 
//GO(krb5_rc_initialize, 
//GO(krb5_rc_io_close, 
//GO(krb5_rc_io_creat, 
//GO(krb5_rc_io_destroy, 
//GO(krb5_rc_io_mark, 
//GO(krb5_rc_io_move, 
//GO(krb5_rc_io_open, 
//GO(krb5_rc_io_read, 
//GO(krb5_rc_io_size, 
//GO(krb5_rc_io_sync, 
//GO(krb5_rc_io_unmark, 
//GO(krb5_rc_io_write, 
//GO(krb5_rc_recover, 
//GO(krb5_rc_recover_or_initialize, 
//GO(krb5_rc_register_type, 
//GO(krb5_rc_resolve, 
//GO(krb5_rc_resolve_full, 
//GO(krb5_rc_resolve_type, 
//GO(krb5_rc_store, 
//GO(krb5_rd_cred, 
//GO(krb5_rd_error, 
//GO(krb5_rd_priv, 
//GO(krb5_rd_rep, 
//GO(krb5_rd_rep_dce, 
//GO(krb5_rd_req, 
//GO(krb5_rd_req_decoded, 
//GO(krb5_rd_req_decoded_anyflag, 
//GO(krb5_rd_safe, 
//GO(krb5_read_message, 
//GO(krb5_read_password, 
//GO(krb5_realm_compare, 
//GO(krb5_recvauth, 
//GO(krb5_recvauth_version, 
//GO(krb5_register_serializer, 
//GO(krb5_responder_get_challenge, 
//GO(krb5_responder_list_questions, 
//GO(krb5_responder_otp_challenge_free, 
//GO(krb5_responder_otp_get_challenge, 
//GO(krb5_responder_otp_set_answer, 
//GO(krb5_responder_pkinit_challenge_free, 
//GO(krb5_responder_pkinit_get_challenge, 
//GO(krb5_responder_pkinit_set_answer, 
//GO(krb5_responder_set_answer, 
//GO(krb5_salttype_to_string, 
//GO(krb5_sendauth, 
//GO(krb5_sendto_kdc, 
//GO(krb5_ser_address_init, 
//GO(krb5_ser_auth_context_init, 
//GO(krb5_ser_authdata_init, 
//GO(krb5_ser_authenticator_init, 
//GO(krb5_ser_ccache_init, 
//GO(krb5_ser_checksum_init, 
//GO(krb5_ser_context_init, 
//GO(krb5_ser_keyblock_init, 
//GO(krb5_ser_keytab_init, 
//GO(krb5_ser_pack_bytes, 
//GO(krb5_ser_pack_int32, 
//GO(krb5_ser_pack_int64, 
//GO(krb5_ser_principal_init, 
//GO(krb5_ser_rcache_init, 
//GO(krb5_ser_unpack_bytes, 
//GO(krb5_ser_unpack_int32, 
//GO(krb5_ser_unpack_int64, 
//GO(krb5_server_decrypt_ticket_keytab, 
//GO(krb5_set_config_files, 
//GO(krb5_set_debugging_time, 
//GO(krb5_set_default_in_tkt_ktypes, 
//GO(krb5_set_default_realm, 
//GO(krb5_set_default_tgs_enctypes, 
//GO(krb5_set_default_tgs_ktypes, 
//GO(krb5_set_error_message, 
//GO(krb5_set_password, 
//GO(krb5_set_password_using_ccache, 
//GO(krb5_set_principal_realm, 
//GO(krb5_set_real_time, 
//GO(krb5_set_time_offsets, 
//GO(krb5_set_trace_callback, 
//GO(krb5_set_trace_filename, 
//GO(krb5_size_opaque, 
//GO(krb5_sname_match, 
//GO(krb5_sname_to_principal, 
//GO(krb5_string_to_deltat, 
//GO(krb5_string_to_salttype, 
//GO(krb5_string_to_timestamp, 
//GO(krb5_timeofday, 
//GO(krb5_timestamp_to_sfstring, 
//GO(krb5_timestamp_to_string, 
//GO(krb5_tkt_creds_free, 
//GO(krb5_tkt_creds_get, 
//GO(krb5_tkt_creds_get_creds, 
//GO(krb5_tkt_creds_get_times, 
//GO(krb5_tkt_creds_init, 
//GO(krb5_tkt_creds_step, 
//GO(krb5_unlock_file, 
//GO(krb5_unpack_full_ipaddr, 
//GO(krb5_unparse_name, 
//GO(krb5_unparse_name_ext, 
GO(krb5_unparse_name_flags, iFppip)
//GO(krb5_unparse_name_flags_ext, 
//GO(krb5_use_natural_time, 
//GO(krb5_us_timeofday, 
//GO(krb5_verify_authdata_kdc_issued, 
//GO(krb5_verify_init_creds, 
//GO(krb5_verify_init_creds_opt_init, 
//GO(krb5_verify_init_creds_opt_set_ap_req_nofail, 
//GO(krb5_vset_error_message, 
//GO(krb5_walk_realm_tree, 
//GO(krb5_write_message, 
//GO(profile_abandon, 
//GO(profile_add_relation, 
//GO(profile_clear_relation, 
//GO(profile_flush, 
//GO(profile_flush_to_buffer, 
//GO(profile_flush_to_file, 
//GO(profile_free_buffer, 
//GO(profile_free_list, 
//GO(profile_get_boolean, 
//GO(profile_get_integer, 
//GO(profile_get_relation_names, 
//GO(profile_get_string, 
//GO(profile_get_subsection_names, 
//GO(profile_get_values, 
//GO(profile_init, 
//GO(profile_init_flags, 
//GO(profile_init_path, 
//GO(profile_init_vtable, 
//GO(profile_iterator, 
//GO(profile_iterator_create, 
//GO(profile_iterator_free, 
//GO(profile_release, 
//GO(profile_release_string, 
//GO(profile_rename_section, 
//GO(profile_ser_externalize, 
//GO(profile_ser_internalize, 
//GO(profile_ser_size, 
//GO(profile_update_relation, 
