#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(xcb_render_util_change_glyphset, 
//GO(xcb_render_util_composite_text, 
//GO(xcb_render_util_composite_text_checked, 
//GO(xcb_render_util_composite_text_free, 
//GO(xcb_render_util_composite_text_stream, 
//GO(xcb_render_util_disconnect, 
//GO(xcb_render_util_find_format, 
GO(xcb_render_util_find_standard_format, pFpu)
GO(xcb_render_util_find_visual_format, pFpu)
//GO(xcb_render_util_glyphs_16, 
//GO(xcb_render_util_glyphs_32, 
//GO(xcb_render_util_glyphs_8, 
//GO(xcb_render_util_query_formats, 
//GO(xcb_render_util_query_version, 

