/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibxcbDEFS32_H_
#define __wrappedlibxcbDEFS32_H_

#define ZFEpp pFEppp
#define ZFEpp_32 pFEppp_32
#define zFEnuu pFEpnuu
#define zFEnuu_32 pFEpnuu_32
#define zFEnCWp pFEpnCWp
#define zFEnCWp_32 pFEpnCWp_32
#define zFEnCuuuuu pFEpnCuuuuu
#define zFEnCuuuuu_32 pFEpnCuuuuu_32
#define zFEnCuuuCup pFEpnCuuuCup
#define zFEnCuuuCup_32 pFEpnCuuuCup_32
#define zFEpnCuuwwWWWWuup pFEppnCuuwwWWWWuup
#define zFEpnCuuwwWWWWuup_32 pFEppnCuuwwWWWWuup_32

#endif // __wrappedlibxcbDEFS32_H_
