#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(XkbAccessXDetailText, pFuu)
GO(XkbActionText, pFpppu)
GO(XkbActionTypeText, pFuu)
GO(XkbAtomGetString, pFpL)
GO(XkbAtomText, pFpLu)
GO(XkbBehaviorText, pFppu)
GO(XkbCFAddModByName, pFpipip)
//GOM(XkbCFAddPrivate, pFEppp)
GO(XkbCFApplyMods, iFpip)
//GOM(XkbCFApplyRtrnValues, iFEppp)
GO(XkbCFBindMods, iFpp)
//DATAM(XkbCFDflts, 8)
//GOM(XkbCFDup, pFEp)
//GOM(XkbCFFree, pFEpi)
//GOM(XkbCFFreeRtrn, vFEppp)
//GOM(XkbCFParse, iFESppp)
GO(XkbCFReportError, vFSpii)
GO(XkbCFScan, iFSpp)
GO(XkbChangeAtomDisplay, LFppL)
GO(XkbChangeKbdDisplay, iFpp)
GO(XkbConfigText, pFuu)
GO(XkbControlsMaskText, pFuu)
GO(XkbConvertGetByNameComponents, uFiu)
GO(XkbConvertXkbComponents, uFiu)
GO(XkbDetermineFileType, iFpip)
GO(XkbDoodadTypeText, pFuu)
GO(XkbEnsureSafeMapName, vFp)
//DATAB(_XkbErrCode, 4)
//DATAB(_XkbErrData, 4)
//DATAB(_XkbErrLocation, 8)
//DATA(_XkbErrMessages, 
GO(XkbFindKeycodeByName, iFppi)
GO(XkbFreeOrderedDrawables, vFp)
GO(XkbGeomFPText, pFiu)
GO(XkbGetOrderedDrawables, pFpp)
GO(XkbIMWhichStateMaskText, pFuu)
GO(XkbIndentText, pFu)
GO(XkbInitAtoms, vFp)
GO(XkbInternAtom, LFppi)
GO(XkbKeyNameText, pFpu)
GO(XkbKeysymText, pFLu)
GO(_XkbKSCheckCase, uFL)
GO(XkbLookupCanonicalRGBColor, iFpp)
GO(XkbLookupGroupAndLevel, iFpippp)
//GO(XkbMergeFile, 
GO(XkbModIndexText, pFuu)
GO(XkbModMaskText, pFuu)
GO(XkbNameMatchesPattern, iFpp)
GO(XkbNKNDetailMaskText, pFuu)
GO(XkbReadFromServer, uFpuup)
GO(XkbRF_AddGroup, pFp)
GO(XkbRF_AddRule, pFp)
GO(XkbRF_AddVarDesc, pFp)
GO(XkbRF_AddVarDescCopy, pFpp)
GO(XkbRF_AddVarToDescribe, pFpp)
GO(XkbRF_Create, pFii)
GO(XkbRF_Free, vFpi)
GO(XkbRF_GetComponents, iFppp)
GO(XkbRF_GetNamesProp, iFppp)
GO(XkbRF_Load, pFppii)
GO(XkbRF_LoadDescriptions, iFSp)
GO(XkbRF_LoadDescriptionsByName, iFppp)
GO(XkbRF_LoadRules, iFSp)
GO(XkbRF_LoadRulesByName, iFppp)
GO(XkbRF_SetNamesProp, iFppp)
GO(XkbSIMatchText, pFuu)
GO(XkbStdBell, iFpLii)
GO(XkbStdBellEvent, iFpLii)
GO(XkbStringText, pFpu)
GO(XkbVModIndexText, pFppuu)
GO(XkbVModMaskText, pFppuuu)
GO(XkbWriteCFile, iFSpp)
GO(XkbWriteToServer, iFp)
//GOM(XkbWriteXKBCompatMap, iFESpiipp)
//GOM(XkbWriteXKBFile, iFESpipp)
//GOM(XkbWriteXKBGeometry, iFESpiipp)
//GOM(XkbWriteXKBKeycodes, iFESpiipp)
//GOM(XkbWriteXKBKeymap, iFESpiipp)
GO(XkbWriteXKBKeymapForNames, iFSpppuu)
//GOM(XkbWriteXKBKeyTypes, iFESpiipp)
//GOM(XkbWriteXKBLayout, iFESpiipp)
//GOM(XkbWriteXKBSemantics, iFESpiipp)
//GOM(XkbWriteXKBSymbols, iFESpiipp)
GO(XkbWriteXKMFile, iFSp)
GO(XkmFindTOCEntry, pFppu)
GO(XkmProbe, iFS)
GO(XkmReadFile, uFSuup)
GO(XkmReadFileSection, iFSppp)
GO(XkmReadFileSectionName, pFSp)
GO(XkmReadTOC, iFSpip)
