#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(xcb_present_event_end, 
//GO(xcb_present_event_next, 
DATA(xcb_present_id, 8)
//GO(xcb_present_notify_end, 
GO(xcb_present_notify_msc, UFbuuUUU)
//GO(xcb_present_notify_msc_checked, 
//GO(xcb_present_notify_next, 
GO(xcb_present_pixmap, UFbuuuuuwwuuuuUUUup)
GO(xcb_present_pixmap_checked, UFbuuuuuwwuuuuUUUup)
//GO(xcb_present_pixmap_notifies, 
//GO(xcb_present_pixmap_notifies_iterator, 
//GO(xcb_present_pixmap_notifies_length, 
//GO(xcb_present_pixmap_sizeof, 
//GO(xcb_present_query_capabilities, 
//GO(xcb_present_query_capabilities_reply, 
//GO(xcb_present_query_capabilities_unchecked, 
GO(xcb_present_query_version, UFbuu)
GO(xcb_present_query_version_reply, pFbup)
GO(xcb_present_query_version_unchecked, UFbuu)
//GO(xcb_present_redirect_notify_notifies, 
//GO(xcb_present_redirect_notify_notifies_iterator, 
//GO(xcb_present_redirect_notify_notifies_length, 
//GO(xcb_present_redirect_notify_sizeof, 
GO(xcb_present_select_input, UFbuuu)
GO(xcb_present_select_input_checked, UFbuuu)
