/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibxcbUNDEFS32_H_
#define __wrappedlibxcbUNDEFS32_H_

#undef ZFEpp
#undef ZFEpp_32
#undef zFEnuu
#undef zFEnuu_32
#undef zFEnCWp
#undef zFEnCWp_32
#undef zFEnCuuuuu
#undef zFEnCuuuuu_32
#undef zFEnCuuuCup
#undef zFEnCuuuCup_32
#undef zFEpnCuuwwWWWWuup
#undef zFEpnCuuwwWWWWuup_32

#endif // __wrappedlibxcbUNDEFS32_H_
