/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibxfixesTYPES32_H_
#define __wrappedlibxfixesTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void* (*pFX_t)(void*);
typedef int32_t (*iFXpp_t)(void*, void*, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(XFixesGetCursorImage, pFX_t) \
	GO(XFixesQueryExtension, iFXpp_t)

#endif // __wrappedlibxfixesTYPES32_H_
