#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(gluBeginCurve,vFp)
GO(gluBeginPolygon, vFp)
GO(gluBeginSurface, vFp)
GO(gluBeginTrim, vFp)
GO(gluBuild1DMipmapLevels, iFuiiuuiiip)
GO(gluBuild1DMipmaps, iFuiiuup)
GO(gluBuild2DMipmapLevels, iFuiiiuuiiip)
GO(gluBuild2DMipmaps, iFuiiiuup)
GO(gluBuild3DMipmapLevels, iFuiiiiuuiiip)
GO(gluBuild3DMipmaps, iFuiiiiuup)
GO(gluCheckExtension, CFpp)
GO(gluCylinder, vFpdddii)
GO(gluDeleteNurbsRenderer, vFp)
GO(gluDeleteQuadric, vFp)
GO(gluDeleteTess, vFp)
GO(gluDisk, vFpddii)
GO(gluEndCurve, vFp)
GO(gluEndPolygon, vFp)
GO(gluEndSurface, vFp)
GO(gluEndTrim, vFp)
GO(gluErrorString, pFu)
GO(gluGetNurbsProperty, vFpup)
GO(gluGetString, pFu)
GO(gluGetTessProperty, vFpup)
GO(gluLoadSamplingMatrices, vFpppp)
GO(gluLookAt, vFddddddddd)
GO(gluNewNurbsRenderer, pFv)
GO(gluNewQuadric, pFv)
GO(gluNewTess, pFv)
GO(gluNextContour, vFpu)
GOM(gluNurbsCallback, vFEpup)
GO(gluNurbsCallbackData, vFpp)
GO(gluNurbsCallbackDataEXT, vFpp)
GO(gluNurbsCurve, vFpipipiu)
GO(gluNurbsProperty, vFpuf)
GO(gluNurbsSurface, vFpipipiipiiu)
GO(gluOrtho2D, vFdddd)
GO(gluPartialDisk, vFpddiidd)
GO(gluPerspective, vFdddd)
GO(gluPickMatrix, vFddddp)
GO(gluProject, iFdddpppppp)
GO(gluPwlCurve, vFpipiu)
GOM(gluQuadricCallback, vFEpup)
GO(gluQuadricDrawStyle, vFpu)
GO(gluQuadricNormals, vFpu)
GO(gluQuadricOrientation, vFpu)
GO(gluQuadricTexture, vFpC)
GO(gluScaleImage, iFuiiupiiup)
GO(gluSphere, vFpdii)
GO(gluTessBeginContour, vFp)
GO(gluTessBeginPolygon, vFpp)
GOM(gluTessCallback, vFEpup)
GO(gluTessEndContour, vFp)
GO(gluTessEndPolygon, vFp)
GO(gluTessNormal, vFpddd)
GO(gluTessProperty, vFpud)
GO(gluTessVertex, vFppp)
GO(gluUnProject, iFdddpppppp)
GO(gluUnProject4, iFddddpppddpppp)

