#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(eglBindAPI, uFu)
GO(eglBindTexImage, uFppi)
GO(eglChooseConfig, uFpppip)
GO(eglClientWaitSync, iFppiU)
GO(eglCopyBuffers, uFppL)
GO(eglCreateContext, pFpppp)
GO(eglCreateImage, pFppupp)
GO(eglCreatePbufferFromClientBuffer, pFpuppp)
GO(eglCreatePbufferSurface, pFppp)
GO(eglCreatePixmapSurface, pFppLp)
GO(eglCreatePlatformPixmapSurface, pFpppp)
GO(eglCreatePlatformWindowSurface, pFpppp)
GO(eglCreateSync, pFpup)
GO(eglCreateWindowSurface, pFppLp)
GO(eglDestroyContext, uFpp)
GO(eglDestroyImage, uFpp)
GO(eglDestroySurface, uFpp)
GO(eglDestroySync, uFpp)
GO(eglGetConfigAttrib, uFppip)
GO(eglGetConfigs, uFppip)
GO(eglGetCurrentContext, pFv)
GO(eglGetCurrentDisplay, pFv)
GO(eglGetCurrentSurface, pFi)
GO(eglGetDisplay, pFp)
GO(eglGetError, iFv)
GO(eglGetPlatformDisplay, pFupp)
GOM(eglGetProcAddress, pFEp)
GO(eglGetSyncAttrib, uFppip)
GO(eglInitialize, uFppp)
GO(eglMakeCurrent, uFpppp)
GO(eglQueryAPI, uFv)
GO(eglQueryContext, uFppip)
GO(eglQueryString, pFpi)
GO(eglQuerySurface, uFppip)
GO(eglReleaseTexImage, uFppi)
GO(eglReleaseThread, uFv)
GO(eglSurfaceAttrib, uFppii)
GO(eglSwapBuffers, uFpp)
GO(eglSwapInterval, uFpi)
GO(eglTerminate, uFp)
GO(eglWaitClient, uFv)
GO(eglWaitGL, uFv)
GO(eglWaitNative, uFi)
GO(eglWaitSync, uFppi)

// EGL_EXT_platform_base
GO(eglGetPlatformDisplayEXT, pFupp)
GO(eglCreatePlatformWindowSurfaceEXT, pFpppp)
GO(eglCreatePlatformPixmapSurfaceEXT, pFpppp)
