#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(cuArray3DCreate, uFpp)
GO(cuArray3DCreate_v2, uFpp)
GO(cuArray3DGetDescriptor, uFpp)
GO(cuArray3DGetDescriptor_v2, uFpp)
GO(cuArrayCreate, uFpp)
GO(cuArrayCreate_v2, uFpp)
GO(cuArrayDestroy, uFp)
GO(cuArrayGetDescriptor, uFpp)
GO(cuArrayGetDescriptor_v2, uFpp)
GO(cuArrayGetMemoryRequirements, uFppi)
GO(cuArrayGetPlane, uFppu)
GO(cuArrayGetSparseProperties, uFpp)
GO(cuCoredumpGetAttribute, uFupp)
GO(cuCoredumpGetAttributeGlobal, uFupp)
GO(cuCoredumpSetAttribute, uFupp)
GO(cuCoredumpSetAttributeGlobal, uFupp)
GO(cuCtxAttach, uFpu)
GO(cuCtxCreate, uFpui)
GO(cuCtxCreate_v2, uFpui)
GO(cuCtxCreate_v3, uFppiui)
GO(cuCtxCreate_v4, uFppup)
GO(cuCtxDestroy, uFp)
GO(cuCtxDestroy_v2, uFp)
GO(cuCtxDetach, uFp)
GO(cuCtxDisablePeerAccess, uFp)
GO(cuCtxEnablePeerAccess, uFpu)
GO(cuCtxFromGreenCtx, uFpp)
GO(cuCtxGetApiVersion, uFpp)
GO(cuCtxGetCacheConfig, uFp)
GO(cuCtxGetCurrent, uFp)
GO(cuCtxGetDevice, uFp)
GO(cuCtxGetDevResource, uFppu)
GO(cuCtxGetExecAffinity, uFpu)
GO(cuCtxGetFlags, uFp)
GO(cuCtxGetId, uFpp)
GO(cuCtxGetLimit, uFpu)
GO(cuCtxGetSharedMemConfig, uFp)
GO(cuCtxGetStreamPriorityRange, uFpp)
GO(cuCtxPopCurrent, uFp)
GO(cuCtxPopCurrent_v2, uFp)
GO(cuCtxPushCurrent, uFp)
GO(cuCtxPushCurrent_v2, uFp)
GO(cuCtxRecordEvent, uFpp)
GO(cuCtxResetPersistingL2Cache, uFv)
GO(cuCtxSetCacheConfig, uFu)
GO(cuCtxSetCurrent, uFp)
GO(cuCtxSetFlags, uFu)
GO(cuCtxSetLimit, uFuL)
GO(cuCtxSetSharedMemConfig, uFu)
GO(cuCtxSynchronize, uFv)
GO(cuCtxWaitEvent, uFpp)
//GO(cudbgApiAttach, 
//DATA(cudbgApiClientPid, 
//DATA(cudbgApiClientRevision, 
//GO(cudbgApiDetach, 
//GO(cudbgApiInit, 
//DATA(cudbgAttachHandlerAvailable, 
//DATA(cudbgDebuggerCapabilities, 
//DATA(cudbgDebuggerInitialized, 
//DATA(cudbgDetachSuspendedDevicesMask, 
//DATA(cudbgEnableIntegratedMemcheck, 
//DATA(cudbgEnableLaunchBlocking, 
//DATA(cudbgEnablePreemptionDebugging, 
//GO(cudbgGetAPI, 
//GO(cudbgGetAPIVersion, 
//DATA(cudbgInjectionPath, 
//DATA(cudbgIpcFlag, 
//GO(cudbgMain, 
//GO(cudbgReportDriverApiError, 
//DATA(cudbgReportDriverApiErrorFlags, 
//GO(cudbgReportDriverInternalError, 
//DATA(cudbgReportedDriverApiErrorCode, 
//DATA(cudbgReportedDriverApiErrorFuncNameAddr, 
//DATA(cudbgReportedDriverApiErrorFuncNameSize, 
//DATA(cudbgReportedDriverInternalErrorCode, 
//DATA(cudbgResumeForAttachDetach, 
//DATA(cudbgRpcEnabled, 
//DATA(cudbgSessionId, 
//DATA(cudbgUseExternalDebugger, 
GO(cuDestroyExternalMemory, uFp)
GO(cuDestroyExternalSemaphore, uFp)
GO(cuDeviceCanAccessPeer, uFpii)
GO(cuDeviceComputeCapability, uFppi)
GO(cuDeviceGet, uFpi)
GO(cuDeviceGetAttribute, uFpui)
GO(cuDeviceGetByPCIBusId, uFpp)
GO(cuDeviceGetCount, uFp)
GO(cuDeviceGetDefaultMemPool, uFpi)
GO(cuDeviceGetDevResource, uFppu)
GO(cuDeviceGetExecAffinitySupport, uFpui)
GO(cuDeviceGetGraphMemAttribute, uFiup)
GO(cuDeviceGetLuid, uFppi)
GO(cuDeviceGetMemPool, uFpi)
GO(cuDeviceGetName, uFpii)
GO(cuDeviceGetNvSciSyncAttributes, uFpii)
GO(cuDeviceGetP2PAttribute, uFpuii)
GO(cuDeviceGetPCIBusId, uFpii)
GO(cuDeviceGetProperties, uFpi)
GO(cuDeviceGetTexture1DLinearMaxWidth, uFpuui)
GO(cuDeviceGetUuid, uFpi)
GO(cuDeviceGetUuid_v2, uFpi)
GO(cuDeviceGraphMemTrim, uFi)
GO(cuDevicePrimaryCtxGetState, uFipp)
GO(cuDevicePrimaryCtxRelease, uFi)
GO(cuDevicePrimaryCtxRelease_v2, uFi)
GO(cuDevicePrimaryCtxReset, uFi)
GO(cuDevicePrimaryCtxReset_v2, uFi)
GO(cuDevicePrimaryCtxRetain, uFpi)
GO(cuDevicePrimaryCtxSetFlags, uFiu)
GO(cuDevicePrimaryCtxSetFlags_v2, uFiu)
GOM(cuDeviceRegisterAsyncNotification, uFEpppp)
GO(cuDeviceSetGraphMemAttribute, uFiup)
GO(cuDeviceSetMemPool, uFip)
GO(cuDeviceTotalMem, uFpi)
GO(cuDeviceTotalMem_v2, uFpi)
GO(cuDevResourceGenerateDesc, uFppu)
GO(cuDevSmResourceSplitByCount, uFppppuu)
GO(cuDeviceUnregisterAsyncNotification, uFpp)
GO(cuDriverGetVersion, uFp)
//GO(cuEGLApiInit, 
GO(cuEGLStreamConsumerAcquireFrame, uFpppu)
GO(cuEGLStreamConsumerConnect, uFpp)
GO(cuEGLStreamConsumerConnectWithFlags, uFppu)
GO(cuEGLStreamConsumerDisconnect, uFp)
GO(cuEGLStreamConsumerReleaseFrame, uFppp)
GO(cuEGLStreamProducerConnect, uFppii)
GO(cuEGLStreamProducerDisconnect, uFp)
//GO(cuEGLStreamProducerPresentFrame, 
GO(cuEGLStreamProducerReturnFrame, uFppp)
GO(cuEventCreate, uFpu)
GO(cuEventCreateFromEGLSync, uFppu)
GO(cuEventDestroy, uFp)
GO(cuEventDestroy_v2, uFp)
GO(cuEventElapsedTime, uFppp)
GO(cuEventElapsedTime_v2, uFppp)
GO(cuEventQuery, uFp)
GO(cuEventRecord, uFpp)
GO(cuEventRecord_ptsz, uFpp)
GO(cuEventRecordWithFlags, uFppu)
GO(cuEventRecordWithFlags_ptsz, uFppu)
GO(cuEventSynchronize, uFp)
GO(cuExternalMemoryGetMappedBuffer, uFppp)
GO(cuExternalMemoryGetMappedMipmappedArray, uFppp)
GO(cuFlushGPUDirectRDMAWrites, uFuu)
GO(cuFuncGetAttribute, uFpup)
GO(cuFuncGetModule, uFpp)
GO(cuFuncGetName, uFpp)
GO(cuFuncGetParamInfo, uFpLpp)
GO(cuFuncLoad, uFp)
GO(cuFuncIsLoaded, uFpp)
GO(cuFuncSetAttribute, uFpui)
GO(cuFuncSetBlockShape, uFpiii)
GO(cuFuncSetCacheConfig, uFpu)
GO(cuFuncSetSharedMemConfig, uFpu)
GO(cuFuncSetSharedSize, uFpu)
GO(cuGetErrorName, uFup)
GO(cuGetErrorString, uFup)
GO(cuGetExportTable, uFpp)
GOM(cuGetProcAddress, uFppiUp)
GOM(cuGetProcAddress_v2, uFppiUp)
GO(cuGLCtxCreate, uFpui)
GO(cuGLCtxCreate_v2, uFpui)
GO(cuGLGetDevices, uFppuu)
GO(cuGLGetDevices_v2, uFppuu)
GO(cuGLInit, uFv)
GO(cuGLMapBufferObject, uFppu)
GO(cuGLMapBufferObjectAsync, uFppup)
GO(cuGLMapBufferObjectAsync_v2, uFppup)
GO(cuGLMapBufferObjectAsync_v2_ptsz, uFppup)
GO(cuGLMapBufferObject_v2, uFppu)
GO(cuGLMapBufferObject_v2_ptds, uFppu)
GO(cuGLRegisterBufferObject, uFu)
GO(cuGLSetBufferObjectMapFlags, uFuu)
GO(cuGLUnmapBufferObject, uFu)
GO(cuGLUnmapBufferObjectAsync, uFup)
GO(cuGLUnregisterBufferObject, uFu)
GO(cuGraphAddBatchMemOpNode, uFpppLp)
GO(cuGraphAddChildGraphNode, uFpppLp)
GO(cuGraphAddDependencies, uFpppL)
GO(cuGraphAddDependencies_v2, uFppppL)
GO(cuGraphAddEmptyNode, uFpppL)
GO(cuGraphAddEventRecordNode, uFpppLp)
GO(cuGraphAddEventWaitNode, uFpppLp)
GO(cuGraphAddExternalSemaphoresSignalNode, uFpppLp)
GO(cuGraphAddExternalSemaphoresWaitNode, uFpppLp)
GOM(cuGraphAddHostNode, uFEpppLp)
GO(cuGraphAddKernelNode, uFpppLp)
GO(cuGraphAddKernelNode_v2, uFpppLp)
GO(cuGraphAddMemAllocNode, uFpppLp)
GO(cuGraphAddMemcpyNode, uFpppLpp)
GO(cuGraphAddMemFreeNode, uFpppLU)
GO(cuGraphAddMemsetNode, uFpppLpp)
GO(cuGraphAddNode, uFpppLp) // should be GOM for last param?
GO(cuGraphAddNode_v2, uFppppLp) // should be GOM for last param?
GO(cuGraphBatchMemOpNodeGetParams, uFpp)
GO(cuGraphBatchMemOpNodeSetParams, uFpp)
GO(cuGraphChildGraphNodeGetGraph, uFpp)
GO(cuGraphClone, uFpp)
GO(cuGraphConditionalHandleCreate, uFpppuu)
GO(cuGraphCreate, uFpu)
GO(cuGraphDebugDotPrint, uFppu)
GO(cuGraphDestroy, uFp)
GO(cuGraphDestroyNode, uFp)
GO(cuGraphEventRecordNodeGetEvent, uFpp)
GO(cuGraphEventRecordNodeSetEvent, uFpp)
GO(cuGraphEventWaitNodeGetEvent, uFpp)
GO(cuGraphEventWaitNodeSetEvent, uFpp)
GO(cuGraphExecBatchMemOpNodeSetParams, uFppp)
GO(cuGraphExecChildGraphNodeSetParams, uFppp)
GO(cuGraphExecDestroy, uFp)
GO(cuGraphExecEventRecordNodeSetEvent, uFppp)
GO(cuGraphExecEventWaitNodeSetEvent, uFppp)
GO(cuGraphExecExternalSemaphoresSignalNodeSetParams, uFppp)
GO(cuGraphExecExternalSemaphoresWaitNodeSetParams, uFppp)
GO(cuGraphExecGetFlags, uFpp)
GOM(cuGraphExecHostNodeSetParams, uFEppp)
GO(cuGraphExecKernelNodeSetParams, uFppp)
GO(cuGraphExecKernelNodeSetParams_v2, uFppp)
GO(cuGraphExecMemcpyNodeSetParams, uFpppp)
GO(cuGraphExecMemsetNodeSetParams, uFpppp)
GO(cuGraphExecNodeSetParams, uFppp) //should be GOM?
GO(cuGraphExecUpdate, uFppp)
GO(cuGraphExecUpdate_v2, uFppp)
GO(cuGraphExternalSemaphoresSignalNodeGetParams, uFpp)
GO(cuGraphExternalSemaphoresSignalNodeSetParams, uFpp)
GO(cuGraphExternalSemaphoresWaitNodeGetParams, uFpp)
GO(cuGraphExternalSemaphoresWaitNodeSetParams, uFpp)
GO(cuGraphGetEdges, uFpppp)
GO(cuGraphGetEdges_v2, uFppppp)
GO(cuGraphGetNodes, uFppp)
GO(cuGraphGetRootNodes, uFppp)
GOM(cuGraphHostNodeGetParams, uFEpp)
GOM(cuGraphHostNodeSetParams, uFEpp)
GO(cuGraphicsEGLRegisterImage, uFppu)
GO(cuGraphicsGLRegisterBuffer, uFpuu)
GO(cuGraphicsGLRegisterImage, uFpuuu)
GO(cuGraphicsMapResources, uFupp)
GO(cuGraphicsMapResources_ptsz, uFupp)
GO(cuGraphicsResourceGetMappedEglFrame, uFppuu)
GO(cuGraphicsResourceGetMappedMipmappedArray, uFpp)
GO(cuGraphicsResourceGetMappedPointer, uFppp)
GO(cuGraphicsResourceGetMappedPointer_v2, uFppp)
GO(cuGraphicsResourceSetMapFlags, uFpu)
GO(cuGraphicsResourceSetMapFlags_v2, uFpu)
GO(cuGraphicsSubResourceGetMappedArray, uFppuu)
GO(cuGraphicsUnmapResources, uFupp)
GO(cuGraphicsUnmapResources_ptsz, uFupp)
GO(cuGraphicsUnregisterResource, uFp)
GO(cuGraphicsVDPAURegisterOutputSurface, uFpuu)
GO(cuGraphicsVDPAURegisterVideoSurface, uFpuu)
GO(cuGraphInstantiate, uFppppL)
GO(cuGraphInstantiate_v2, uFppppL)
GO(cuGraphInstantiateWithFlags, uFppU)
GO(cuGraphInstantiateWithParams, uFppp)
GO(cuGraphInstantiateWithParams_ptsz, uFppp)
GO(cuGraphKernelNodeCopyAttributes, uFpp)
GO(cuGraphKernelNodeGetAttribute, uFpup)
GO(cuGraphKernelNodeGetParams, uFpp)
GO(cuGraphKernelNodeGetParams_v2, uFpp)
GO(cuGraphKernelNodeSetAttribute, uFpup)
GO(cuGraphKernelNodeSetParams, uFpp)
GO(cuGraphKernelNodeSetParams_v2, uFpp)
GO(cuGraphLaunch, uFpp)
GO(cuGraphLaunch_ptsz, uFpp)
GO(cuGraphMemAllocNodeGetParams, uFpp)
GO(cuGraphMemcpyNodeGetParams, uFpp)
GO(cuGraphMemcpyNodeSetParams, uFpp)
GO(cuGraphMemFreeNodeGetParams, uFpp)
GO(cuGraphMemsetNodeGetParams, uFpp)
GO(cuGraphMemsetNodeSetParams, uFpp)
GO(cuGraphNodeFindInClone, uFppp)
GO(cuGraphNodeGetDependencies, uFppp)
GO(cuGraphNodeGetDependencies_v2, uFpppp)
GO(cuGraphNodeGetDependentNodes, uFppp)
GO(cuGraphNodeGetDependentNodes_v2, uFpppp)
GO(cuGraphNodeGetEnabled, uFppp)
GO(cuGraphNodeGetType, uFpp)
GO(cuGraphNodeSetEnabled, uFppu)
GO(cuGraphNodeSetParams, uFpp)  // should be GOM?
GO(cuGraphReleaseUserObject, uFppu)
GO(cuGraphRemoveDependencies, uFpppL)
GO(cuGraphRemoveDependencies_v2, uFppppL)
GO(cuGraphRetainUserObject, uFppuu)
GO(cuGraphUpload, uFpp)
GO(cuGraphUpload_ptsz, uFpp)
GO(cuGreenCtxCreate, uFpppu)
GO(cuGreenCtxDestroy, uFp)
GO(cuGreenCtxGetDevResource, uFppu)
GO(cuGreenCtxRecordEvent, uFpp)
GO(cuGreenCtxStreamCreate, uFppui)
GO(cuGreenCtxWaitEvent, uFpp)
GO(cuImportExternalMemory, uFpp)
GO(cuImportExternalSemaphore, uFpp)
GO(cuInit, uFu)
GO(cuIpcCloseMemHandle, uFU)
GO(cuIpcGetEventHandle, uFpp)
GO(cuIpcGetMemHandle, uFpU)
GO(cuIpcOpenEventHandle, uFpU)
GO(cuIpcOpenMemHandle, uFpUu)
GO(cuIpcOpenMemHandle_v2, uFpuU)
GO(cuKernelGetAttribute, uFpupi)
GO(cuKernelGetFunction, uFpp)
GO(cuKernelGetLibrary, uFpp)
GO(cuKernelGetName, uFpp)
GO(cuKernelGetParamInfo, uFpLpp)
GO(cuKernelSetAttribute, uFuipi)
GO(cuKernelSetCacheConfig, uFpui)
GO(cuLaunch, uFp)
GO(cuLaunchCooperativeKernel, uFpuuuuuuupp)
GO(cuLaunchCooperativeKernelMultiDevice, uFpuu)
GO(cuLaunchCooperativeKernel_ptsz, uFpuuuuuuupp)
GO(cuLaunchGrid, uFpii)
GO(cuLaunchGridAsync, uFpiip)
GOM(cuLaunchHostFunc, uFEppp)
GOM(cuLaunchHostFunc_ptsz, uFEppp)
GO(cuLaunchKernel, uFpuuuuuuuppp)
GO(cuLaunchKernelEx, uFpppp)
GO(cuLaunchKernelEx_ptsz, uFpppp)
GO(cuLaunchKernel_ptsz, uFpuuuuuuuppp)
GO(cuLibraryEnumerateKernels, uFpup)
GO(cuLibraryGetGlobal, uFpppp)
GO(cuLibraryGetKernel, uFppp)
GO(cuLibraryGetKernelCount, uFpp)
GO(cuLibraryGetManaged, uFpppp)
GO(cuLibraryGetModule, uFpp)
GO(cuLibraryGetUnifiedFunction, uFppp)
GO(cuLibraryLoadData, uFppppuppu)
GO(cuLibraryLoadFromFile, uFppppuppu)
GO(cuLibraryUnload, uFp)
GO(cuLinkAddData, uFpupLpupp)
GO(cuLinkAddData_v2, uFpupLpupp)
GO(cuLinkAddFile, uFpupupp)
GO(cuLinkAddFile_v2, uFpupupp)
GO(cuLinkComplete, uFppp)
GO(cuLinkCreate, uFuppp)
GO(cuLinkCreate_v2, uFuppp)
GO(cuLinkDestroy, uFp)
GO(cuMemAddressFree, uFUL)
GO(cuMemAddressReserve, uFpLLUU)
GO(cuMemAdvise, uFULui)
GO(cuMemAdvise_v2, uFULui)
GO(cuMemAlloc, uFpLp)
GO(cuMemAllocAsync, uFpLp)
GO(cuMemAllocAsync_ptsz, uFpLp)
GO(cuMemAllocFromPoolAsync, uFpLpp)
GO(cuMemAllocFromPoolAsync_ptsz, uFpLpp)
GO(cuMemAllocHost, uFpL)
GO(cuMemAllocHost_v2, uFpL)
GO(cuMemAllocManaged, uFpLu)
GO(cuMemAllocPitch, uFppLLu)
GO(cuMemAllocPitch_v2, uFppLLu)
GO(cuMemAlloc_v2, uFpL)
GO(cuMemcpy, uFUUL)
GO(cuMemcpy2D, uFp)
GO(cuMemcpy2DAsync, uFpp)
GO(cuMemcpy2DAsync_v2, uFpp)
GO(cuMemcpy2DAsync_v2_ptsz, uFpp)
GO(cuMemcpy2DUnaligned, uFp)
GO(cuMemcpy2DUnaligned_v2, uFp)
GO(cuMemcpy2DUnaligned_v2_ptds, uFp)
GO(cuMemcpy2D_v2, uFp)
GO(cuMemcpy2D_v2_ptds, uFp)
GO(cuMemcpy3D, uFp)
GO(cuMemcpy3DAsync, uFpp)
GO(cuMemcpy3DAsync_v2, uFpp)
GO(cuMemcpy3DAsync_v2_ptsz, uFpp)
GO(cuMemcpy3DBatchAsync, uFLpUp)
GO(cuMemcpy3DBatchAsync_ptsz, uFLpUp)
GO(cuMemcpy3DPeer, uFp)
GO(cuMemcpy3DPeerAsync, uFpp)
GO(cuMemcpy3DPeerAsync_ptsz, uFpp)
GO(cuMemcpy3DPeer_ptds, uFp)
GO(cuMemcpy3D_v2, uFp)
GO(cuMemcpy3D_v2_ptds, uFp)
GO(cuMemcpyAsync, uFUULp)
GO(cuMemcpyAsync_ptsz, uFUULp)
GO(cuMemcpyAtoA, uFpLpLL)
GO(cuMemcpyAtoA_v2, uFpLpLL)
GO(cuMemcpyAtoA_v2_ptds, uFpLpLL)
GO(cuMemcpyAtoD, uFUpLL)
GO(cuMemcpyAtoD_v2, uFUpLL)
GO(cuMemcpyAtoD_v2_ptds, uFUpLL)
GO(cuMemcpyAtoH, uFppLL)
GO(cuMemcpyAtoHAsync, uFppLLp)
GO(cuMemcpyAtoHAsync_v2, uFppLLp)
GO(cuMemcpyAtoHAsync_v2_ptsz, uFppLLp)
GO(cuMemcpyAtoH_v2, uFppLL)
GO(cuMemcpyAtoH_v2_ptds, uFppLL)
GO(cuMemcpyBatchAsync, uFpppLppLp)
GO(cuMemcpyBatchAsync_ptsz, uFpppLppLp)
GO(cuMemcpyDtoA, uFpLUL)
GO(cuMemcpyDtoA_v2, uFpLUL)
GO(cuMemcpyDtoA_v2_ptds, uFpLUL)
GO(cuMemcpyDtoD, uFUUL)
GO(cuMemcpyDtoDAsync, uFUULp)
GO(cuMemcpyDtoDAsync_v2, uFUULp)
GO(cuMemcpyDtoDAsync_v2_ptsz, uFUULp)
GO(cuMemcpyDtoD_v2, uFUUL)
GO(cuMemcpyDtoD_v2_ptds, uFUUL)
GO(cuMemcpyDtoH, uFpUL)
GO(cuMemcpyDtoHAsync, uFpULp)
GO(cuMemcpyDtoHAsync_v2, uFpULp)
GO(cuMemcpyDtoHAsync_v2_ptsz, uFpULp)
GO(cuMemcpyDtoH_v2, uFpUL)
GO(cuMemcpyDtoH_v2_ptds, uFpUL)
GO(cuMemcpyHtoA, uFpLpL)
GO(cuMemcpyHtoAAsync, uFpLpLp)
GO(cuMemcpyHtoAAsync_v2, uFpLpLp)
GO(cuMemcpyHtoAAsync_v2_ptsz, uFpLpLp)
GO(cuMemcpyHtoA_v2, uFpLpL)
GO(cuMemcpyHtoA_v2_ptds, uFpLpL)
GO(cuMemcpyHtoD, uFUpL)
GO(cuMemcpyHtoDAsync, uFUpLp)
GO(cuMemcpyHtoDAsync_v2, uFUpLp)
GO(cuMemcpyHtoDAsync_v2_ptsz, uFUpLp)
GO(cuMemcpyHtoD_v2, uFUpL)
GO(cuMemcpyHtoD_v2_ptds, uFUpL)
GO(cuMemcpyPeer, uFUpUpL)
GO(cuMemcpyPeerAsync, uFUpUpLp)
GO(cuMemcpyPeerAsync_ptsz, uFUpUpLp)
GO(cuMemcpyPeer_ptds, uFUpUpL)
GO(cuMemcpy_ptds, uFUUL)
GO(cuMemCreate, uFpLpU)
GO(cuMemExportToShareableHandle, uFpUuU)
GO(cuMemFree, uFU)
GO(cuMemFreeAsync, uFUp)
GO(cuMemFreeAsync_ptsz, uFUp)
GO(cuMemFreeHost, uFp)
GO(cuMemFree_v2, uFU)
GO(cuMemGetAccess, uFppU)
GO(cuMemGetAddressRange, uFppU)
GO(cuMemGetAddressRange_v2, uFppU)
GO(cuMemGetAllocationGranularity, uFppu)
GO(cuMemGetAllocationPropertiesFromHandle, uFpU)
//GO(cuMemGetAttribute, 
//GO(cuMemGetAttribute_v2, 
GO(cuMemGetHandleForAddressRange, uFpULuU)
GO(cuMemGetInfo, uFpp)
GO(cuMemGetInfo_v2, uFpp)
GO(cuMemHostAlloc, uFpLu)
GO(cuMemHostGetDevicePointer, uFppu)
GO(cuMemHostGetDevicePointer_v2, uFppu)
GO(cuMemHostGetFlags, uFpp)
GO(cuMemHostRegister, uFpLu)
GO(cuMemHostRegister_v2, uFpLu)
GO(cuMemHostUnregister, uFp)
GO(cuMemImportFromShareableHandle, uFppu)
GO(cuMemMap, uFULLUU)
GO(cuMemMapArrayAsync, uFpup)
GO(cuMemMapArrayAsync_ptsz, uFpup)
GO(cuMemPoolCreate, uFpp)
GO(cuMemPoolDestroy, uFp)
GO(cuMemPoolExportPointer, uFpU)
GO(cuMemPoolExportToShareableHandle, uFppuU)
GO(cuMemPoolGetAccess, uFppp)
GO(cuMemPoolGetAttribute, uFpup)
GO(cuMemPoolImportFromShareableHandle, uFppuU)
GO(cuMemPoolImportPointer, uFppp)
GO(cuMemPoolSetAccess, uFppL)
GO(cuMemPoolSetAttribute, uFpup)
GO(cuMemPoolTrimTo, uFpL)
GO(cuMemPrefetchAsync, uFULip)
GO(cuMemPrefetchAsync_ptsz, uFULip)
GO(cuMemPrefetchAsync_v2, uFULip)
GO(cuMemPrefetchAsync_v2_ptsz, uFULip)
GO(cuMemRangeGetAttribute, uFpLuUL)
GO(cuMemRangeGetAttributes, uFpppLUL)
GO(cuMemRelease, uFU)
GO(cuMemRetainAllocationHandle, uFpp)
GO(cuMemSetAccess, uFULpL)
GO(cuMemsetD16, uFUWL)
GO(cuMemsetD16Async, uFUWLp)
GO(cuMemsetD16Async_ptsz, uFUWLp)
GO(cuMemsetD16_v2, uFUWL)
GO(cuMemsetD16_v2_ptds, uFUWL)
GO(cuMemsetD2D16, uFULWLL) 
GO(cuMemsetD2D16Async, uFULWLLp)
GO(cuMemsetD2D16Async_ptsz, uFULWLLp)
GO(cuMemsetD2D16_v2, uFULWLL)
GO(cuMemsetD2D16_v2_ptds, uFULWLL)
GO(cuMemsetD2D32, uFULuLL)
GO(cuMemsetD2D32Async, uFULuLLp)
GO(cuMemsetD2D32Async_ptsz, uFULuLLp)
GO(cuMemsetD2D32_v2, uFULuLL)
GO(cuMemsetD2D32_v2_ptds, uFULuLL)
GO(cuMemsetD2D8, uFULCLL)
GO(cuMemsetD2D8Async, uFULCLLp)
GO(cuMemsetD2D8Async_ptsz, uFULCLLp)
GO(cuMemsetD2D8_v2, uFULCLL)
GO(cuMemsetD2D8_v2_ptds, uFULCLL)
GO(cuMemsetD32, uFUuL)
GO(cuMemsetD32Async, uFUuLp)
GO(cuMemsetD32Async_ptsz, uFUuLp)
GO(cuMemsetD32_v2, uFUuL)
GO(cuMemsetD32_v2_ptds, uFUuL)
GO(cuMemsetD8, uFUCL)
GO(cuMemsetD8Async, uFUCLp)
GO(cuMemsetD8Async_ptsz, uFUCLp)
GO(cuMemsetD8_v2, uFUCL)
GO(cuMemsetD8_v2_ptds, uFUCL)
GO(cuMemUnmap, uFUL)
GO(cuMipmappedArrayCreate, uFppu)
GO(cuMipmappedArrayDestroy, uFp)
GO(cuMipmappedArrayGetLevel, uFppu)
GO(cuMipmappedArrayGetMemoryRequirements, uFppi)
GO(cuMipmappedArrayGetSparseProperties, uFpp)
GO(cuModuleEnumerateFunctions, uFpup)
GO(cuModuleGetFunction, uFppp)
GO(cuModuleGetFunctionCount, uFpp)
GO(cuModuleGetGlobal, uFpppp)
GO(cuModuleGetGlobal_v2, uFpppp)
GO(cuModuleGetLoadingMode, uFp)
GO(cuModuleGetSurfRef, uFppp)
GO(cuModuleGetTexRef, uFppp)
GO(cuModuleLoad, uFpp)
GO(cuModuleLoadData, uFpp)
GO(cuModuleLoadDataEx, uFppupp)
GO(cuModuleLoadFatBinary, uFpp)
GO(cuModuleUnload, uFp)
GO(cuMulticastAddDevice, uFUi)
GO(cuMulticastBindAddr, uFULULU)
GO(cuMulticastBindMem, uFULULLU)
GO(cuMulticastCreate, uFpp)
GO(cuMulticastGetGranularity, uFppu)
GO(cuMulticastUnbind, uFUiLL)
GO(cuOccupancyAvailableDynamicSMemPerBlock, uFppii)
GO(cuOccupancyMaxActiveBlocksPerMultiprocessor, uFppiL)
GO(cuOccupancyMaxActiveBlocksPerMultiprocessorWithFlags, uFppiLu)
GO(cuOccupancyMaxActiveClusters, uFppp)
GO(cuOccupancyMaxPotentialBlockSize, uFppppLi)
GO(cuOccupancyMaxPotentialBlockSizeWithFlags, uFppppLiu)
GO(cuOccupancyMaxPotentialClusterSize, uFppp)
GO(cuParamSetf, uFpif)
GO(cuParamSeti, uFpiu)
GO(cuParamSetSize, uFpu)
GO(cuParamSetTexRef, uFpip)
GO(cuParamSetv, uFpipu)
GO(cuPointerGetAttribute, uFpuU)
GO(cuPointerGetAttributes, uFuppU)
GO(cuPointerSetAttribute, uFpuU)
GO(cuProfilerInitialize, uFppu)
GO(cuProfilerStart, uFv)
GO(cuProfilerStop, uFv)
GO(cuSignalExternalSemaphoresAsync, uFppup)
GO(cuSignalExternalSemaphoresAsync_ptsz, uFppup)
GOM(cuStreamAddCallback, uFEpppu)
GOM(cuStreamAddCallback_ptsz, uFEpppu)
GO(cuStreamAttachMemAsync, uFpULu)
GO(cuStreamAttachMemAsync_ptsz, uFpULu)
GO(cuStreamBatchMemOp, uFpupu)
GO(cuStreamBatchMemOp_ptsz, uFpupu)
GO(cuStreamBatchMemOp_v2, uFpupu)
GO(cuStreamBatchMemOp_v2_ptsz, uFpupu)
GO(cuStreamBeginCapture, uFpu)
GO(cuStreamBeginCapture_ptsz, uFpu)
GO(cuStreamBeginCapture_v2, uFpu)
GO(cuStreamBeginCapture_v2_ptsz, uFpu)
GO(cuStreamBeginCaptureToGraph, uFppppLu)
GO(cuStreamBeginCaptureToGraph_ptsz, uFppppLu)
GO(cuStreamCopyAttributes, uFpp)
GO(cuStreamCopyAttributes_ptsz, uFpp)
GO(cuStreamCreate, uFpu)
GO(cuStreamCreateWithPriority, uFpui)
GO(cuStreamDestroy, uFp)
GO(cuStreamDestroy_v2, uFp)
GO(cuStreamEndCapture, uFpp)
GO(cuStreamEndCapture_ptsz, uFpp)
GO(cuStreamGetAttribute, uFpup)
GO(cuStreamGetAttribute_ptsz, uFpup)
GO(cuStreamGetCaptureInfo, uFpppppp)
GO(cuStreamGetCaptureInfo_ptsz, uFpppppp)
GO(cuStreamGetCaptureInfo_v2, uFpppppp)
GO(cuStreamGetCaptureInfo_v2_ptsz, uFpppppp)
GO(cuStreamGetCaptureInfo_v3, uFppppppp)
GO(cuStreamGetCaptureInfo_v3_ptsz, uFppppppp)
GO(cuStreamGetCtx, uFpp)
GO(cuStreamGetCtx_ptsz, uFpp)
GO(cuStreamGetCtx_v2, uFppp)
GO(cuStreamGetCtx_v2_ptsz, uFppp)
GO(cuStreamGetDevice, uFpp)
GO(cuStreamGetDevice_ptsz, uFpp)
GO(cuStreamGetFlags, uFpp)
GO(cuStreamGetFlags_ptsz, uFpp)
GO(cuStreamGetGreenCtx, uFpp)
GO(cuStreamGetId, uFpp)
GO(cuStreamGetId_ptsz, uFpp)
GO(cuStreamGetPriority, uFpp)
GO(cuStreamGetPriority_ptsz, uFpp)
GO(cuStreamIsCapturing, uFpp)
GO(cuStreamIsCapturing_ptsz, uFpp)
GO(cuStreamQuery, uFp)
GO(cuStreamQuery_ptsz, uFp)
GO(cuStreamSetAttribute, uFpup)
GO(cuStreamSetAttribute_ptsz, uFpup)
GO(cuStreamSynchronize, uFp)
GO(cuStreamSynchronize_ptsz, uFp)
GO(cuStreamUpdateCaptureDependencies, uFppLu)
GO(cuStreamUpdateCaptureDependencies_ptsz, uFppLu)
GO(cuStreamUpdateCaptureDependencies_v2, uFpppLu)
GO(cuStreamUpdateCaptureDependencies_v2_ptsz, uFpppLu)
GO(cuStreamWaitEvent, uFppu)
GO(cuStreamWaitEvent_ptsz, uFppu)
GO(cuStreamWaitValue32, uFpUuu)
GO(cuStreamWaitValue32_ptsz, uFpUuu)
GO(cuStreamWaitValue32_v2, uFpUuu)
GO(cuStreamWaitValue32_v2_ptsz, uFpUuu)
GO(cuStreamWaitValue64, uFpUUu)
GO(cuStreamWaitValue64_ptsz, uFpUUu)
GO(cuStreamWaitValue64_v2, uFpUUu)
GO(cuStreamWaitValue64_v2_ptsz, uFpUUu)
GO(cuStreamWriteValue32, uFpUuu)
GO(cuStreamWriteValue32_ptsz, uFpUuu)
GO(cuStreamWriteValue32_v2, uFpUuu)
GO(cuStreamWriteValue32_v2_ptsz, uFpUuu)
GO(cuStreamWriteValue64, uFpUUu)
GO(cuStreamWriteValue64_ptsz, uFpUUu)
GO(cuStreamWriteValue64_v2, uFpUUu)
GO(cuStreamWriteValue64_v2_ptsz, uFpUUu)
GO(cuSurfObjectCreate, uFpp)
GO(cuSurfObjectDestroy, uFU)
GO(cuSurfObjectGetResourceDesc, uFpU)
GO(cuSurfRefGetArray, uFpp)
GO(cuSurfRefSetArray, uFppu)
GO(cuTensorMapEncodeIm2col, uFpuupppppuupuuuu)
GO(cuTensorMapEncodeTiled, uFpuupppppuuuu)
GO(cuTensorMapReplaceAddress, uFpp)
GO(cuTexObjectCreate, uFpppp)
GO(cuTexObjectDestroy, uFU)
GO(cuTexObjectGetResourceDesc, uFpU)
GO(cuTexObjectGetResourceViewDesc, uFpU)
GO(cuTexObjectGetTextureDesc, uFpU)
GO(cuTexRefCreate, uFp)
GO(cuTexRefDestroy, uFp)
GO(cuTexRefGetAddress, uFpp)
GO(cuTexRefGetAddressMode, uFppi)
GO(cuTexRefGetAddress_v2, uFpp)
GO(cuTexRefGetArray, uFpp)
GO(cuTexRefGetBorderColor, uFpp)
GO(cuTexRefGetFilterMode, uFpp)
GO(cuTexRefGetFlags, uFpp)
GO(cuTexRefGetFormat, uFppp)
GO(cuTexRefGetMaxAnisotropy, uFpp)
GO(cuTexRefGetMipmapFilterMode, uFpp)
GO(cuTexRefGetMipmapLevelBias, uFpp)
GO(cuTexRefGetMipmapLevelClamp, uFppp)
GO(cuTexRefGetMipmappedArray, uFpp)
GO(cuTexRefSetAddress, uFppUL)
GO(cuTexRefSetAddress2D, uFppUL)
GO(cuTexRefSetAddress2D_v2, uFppUL)
GO(cuTexRefSetAddress2D_v3, uFppUL)
GO(cuTexRefSetAddressMode, uFpiu)
GO(cuTexRefSetAddress_v2, uFppUL)
GO(cuTexRefSetArray, uFppu)
GO(cuTexRefSetBorderColor, uFpp)
GO(cuTexRefSetFilterMode, uFpu)
GO(cuTexRefSetFlags, uFpu)
GO(cuTexRefSetFormat, uFpui)
GO(cuTexRefSetMaxAnisotropy, uFpu)
GO(cuTexRefSetMipmapFilterMode, uFpu)
GO(cuTexRefSetMipmapLevelBias, uFpf)
GO(cuTexRefSetMipmapLevelClamp, uFpff)
GO(cuTexRefSetMipmappedArray, uFppu)
GO(cuThreadExchangeStreamCaptureMode, uFp)
GOM(cuUserObjectCreate, uFEpppuu)
GO(cuUserObjectRelease, uFpu)
GO(cuUserObjectRetain, uFpu)
//GO(cuVDPAUCtxCreate, 
//GOM(cuVDPAUCtxCreate_v2, uFEpuiup)
//GOM(cuVDPAUGetDevice, uFEpup)
GO(cuWaitExternalSemaphoresAsync, uFppup)
GO(cuWaitExternalSemaphoresAsync_ptsz, uFppup)
