#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(pango_alignment_get_type, iFv)
GO(pango_attr_background_new, pFWWW)
GO(pango_attr_fallback_new, pFi)
GO(pango_attr_family_new, pFp)
GO(pango_attr_font_desc_new, pFp)
GO(pango_attr_font_features_new, pFp)   //since 1.38
GO(pango_attr_foreground_new, pFWWW)
GO(pango_attr_gravity_hint_new, pFu)
GO(pango_attr_gravity_new, pFu)
GO(pango_attribute_copy, pFp)
GO(pango_attribute_destroy, vFp)
GO(pango_attribute_equal, iFpp)
GOM(pango_attribute_init, vFEpp)
GO(pango_attr_iterator_copy, pFp)
GO(pango_attr_iterator_destroy, vFp)
GO(pango_attr_iterator_get, pFpu)
GO(pango_attr_iterator_get_attrs, pFp)
GO(pango_attr_iterator_get_font, vFpppp)
GO(pango_attr_iterator_next, iFp)
GO(pango_attr_iterator_range, vFppp)
GO(pango_attr_language_new, pFp)
GO(pango_attr_letter_spacing_new, pFi)
GO(pango_attr_list_change, vFpp)
GO(pango_attr_list_copy, pFp)
//GOM(pango_attr_list_filter, pFEppp)
GO(pango_attr_list_get_iterator, pFp)
GO(pango_attr_list_get_type, LFv)
GO(pango_attr_list_insert, vFpp)
GO(pango_attr_list_insert_before, vFpp)
GO(pango_attr_list_new, pFv)
GO(pango_attr_list_ref, pFp)
GO(pango_attr_list_splice, vFppii)
GO(pango_attr_list_unref, vFp)
GO(pango_attr_overline_new, pFu)
GO(pango_attr_rise_new, pFi)
GO(pango_attr_scale_new, pFd)
GO(pango_attr_shape_new, pFpp)
//GOM(pango_attr_shape_new_with_data, pFEppppp)
GO(pango_attr_size_new, pFi)
GO(pango_attr_size_new_absolute, pFi)
GO(pango_attr_stretch_new, pFu)
GO(pango_attr_strikethrough_color_new, pFWWW)
GO(pango_attr_strikethrough_new, pFi)
GO(pango_attr_style_new, pFu)
GO(pango_attr_type_get_name, pFu)
GO(pango_attr_type_get_type, iFv)
GO(pango_attr_type_register, uFp)
GO(pango_attr_underline_color_new, pFWWW)
GO(pango_attr_underline_new, pFu)
GO(pango_attr_variant_new, pFu)
GO(pango_attr_weight_new, pFu)
GO(pango_bidi_type_for_unichar, uFu)
//GO(pango_bidi_type_get_type, 
//GO(pango_break, 
GO(pango_color_copy, pFp)
GO(pango_color_free, vFp)
GO(pango_color_get_type, LFv)
GO(pango_color_parse, iFpp)
GO(pango_color_to_string, pFp)
GO(pango_config_key_get, pFp)
//GO(pango_config_key_get_system, 
GO(pango_context_changed, vFp)
GO(pango_context_get_base_dir, uFp)
GO(pango_context_get_base_gravity, iFp)
GO(pango_context_get_font_description, pFp)
GO(pango_context_get_font_map, pFp)
GO(pango_context_get_gravity, uFp)
GO(pango_context_get_gravity_hint, uFp)
GO(pango_context_get_language, pFp)
GO(pango_context_get_matrix, pFp)
GO(pango_context_get_metrics, pFppp)
GO(pango_context_get_serial, uFp)
GO(pango_context_get_type, iFv)
GO(pango_context_list_families, vFppp)
GO(pango_context_load_font, pFpp)
GO(pango_context_load_fontset, pFppp)
GO(pango_context_new, pFv)
GO(pango_context_set_base_dir, vFpu)
GO(pango_context_set_base_gravity, vFpu)
GO(pango_context_set_font_description, vFpp)
GO(pango_context_set_font_map, vFpp)
GO(pango_context_set_gravity_hint, vFpu)
GO(pango_context_set_language, vFpp)
GO(pango_context_set_matrix, vFpp)
GO(pango_context_set_round_glyph_positions, vFpu)
GO(pango_coverage_copy, pFp)
GO(pango_coverage_from_bytes, pFpi)
GO(pango_coverage_get, uFpi)
//GO(pango_coverage_level_get_type, 
GO(pango_coverage_max, vFpp)
GO(pango_coverage_new, pFv)
GO(pango_coverage_ref, pFp)
GO(pango_coverage_set, vFpiu)
GO(pango_coverage_to_bytes, vFppp)
GO(pango_coverage_unref, vFp)
GO(pango_default_break, vFpippi)
//GO(pango_direction_get_type, 
GO(pango_ellipsize_mode_get_type, lFv)
//GO(pango_engine_get_type, 
//GO(pango_engine_lang_get_type, 
//GO(pango_engine_shape_get_type, 
GO(pango_extents_to_pixels, vFpp)
GO(pango_find_base_dir, uFpi)
//GO(pango_find_map, 
GO(pango_find_paragraph_boundary, vFpipp)
GO(pango_font_describe, pFp)
GO(pango_font_describe_with_absolute_size, pFp)
GO(pango_font_description_better_match, iFppp)
GO(pango_font_description_copy, pFp)
GO(pango_font_description_copy_static, pFp)
GO(pango_font_description_equal, iFpp)
GO(pango_font_description_free, vFp)
GO(pango_font_description_from_string, pFp)
GO(pango_font_description_get_family, pFp)
GO(pango_font_description_get_gravity, uFp)
GO(pango_font_description_get_set_fields, uFp)
GO(pango_font_description_get_size, iFp)
GO(pango_font_description_get_size_is_absolute, iFp)
GO(pango_font_description_get_stretch, uFp)
GO(pango_font_description_get_style, uFp)
GO(pango_font_description_get_type, LFv)
GO(pango_font_description_get_variant, uFp)
GO(pango_font_description_get_weight, uFp)
GO(pango_font_description_hash, uFp)
GO(pango_font_description_merge, vFppi)
GO(pango_font_description_merge_static, vFppi)
GO(pango_font_description_new, pFv)
GO(pango_font_description_set_absolute_size, vFpd)
GO(pango_font_description_set_family, vFpp)
GO(pango_font_description_set_family_static, vFpp)
GO(pango_font_description_set_gravity, vFpu)
GO(pango_font_description_set_size, vFpi)
GO(pango_font_description_set_stretch, vFpu)
GO(pango_font_description_set_style, vFpu)
GO(pango_font_description_set_variant, vFpu)
GO(pango_font_description_set_weight, vFpu)
GO(pango_font_descriptions_free, vFpi)
GO(pango_font_description_to_filename, pFp)
GO(pango_font_description_to_string, pFp)
GO(pango_font_description_unset_fields, vFpu)
GO(pango_font_face_describe, pFp)
GO(pango_font_face_get_face_name, pFp)
GO(pango_font_face_get_type, LFv)
GO(pango_font_face_is_synthesized, iFp)
GO(pango_font_face_list_sizes, vFppp)
GO(pango_font_family_get_name, pFp)
GO(pango_font_family_get_type, LFv)
GO(pango_font_family_is_monospace, iFp)
GO(pango_font_family_list_faces, vFppp)
GO(pango_font_find_shaper, pFppu)
GO(pango_font_get_coverage, pFpp)
GO(pango_font_get_font_map, pFp)
GO(pango_font_get_glyph_extents, vFpupp)
GO(pango_font_get_metrics, pFpp)
GO(pango_font_get_type, LFv)
GO(pango_font_map_create_context, pFp)
GO(pango_font_map_get_serial, uFp)
//GO(pango_font_map_get_shape_engine_type, 
GO(pango_font_map_get_type, iFv)
GO(pango_font_map_list_families, vFppp)
GO(pango_font_map_load_font, pFppp)
GO(pango_font_map_load_fontset, pFpppp)
GO(pango_font_mask_get_type, iFv)
GO(pango_font_metrics_get_approximate_char_width, iFp)
GO(pango_font_metrics_get_approximate_digit_width, iFp)
GO(pango_font_metrics_get_ascent, iFp)
GO(pango_font_metrics_get_descent, iFp)
GO(pango_font_metrics_get_height, iFp)
GO(pango_font_metrics_get_strikethrough_position, iFp)
GO(pango_font_metrics_get_strikethrough_thickness, iFp)
GO(pango_font_metrics_get_type, LFv)
GO(pango_font_metrics_get_underline_position, iFp)
GO(pango_font_metrics_get_underline_thickness, iFp)
//GO(pango_font_metrics_new, 
GO(pango_font_metrics_ref, pFp)
GO(pango_font_metrics_unref, vFp)
//GO(pango_fontset_foreach, 
GO(pango_fontset_get_font, pFpu)
GO(pango_fontset_get_metrics, pFp)
//GO(pango_fontset_get_type, 
GO(pango_fontset_simple_append, vFpp)
//GO(pango_fontset_simple_get_type, 
GO(pango_fontset_simple_new, pFp)
GO(pango_fontset_simple_size, iFp)
//GO(pango_get_lib_subdirectory, 
GO(pango_get_log_attrs, vFpiippi)
GO(pango_get_mirror_char, iFup)
//GO(pango_get_sysconf_subdirectory, 
GO(pango_glyph_item_apply_attrs, pFppp)
GO(pango_glyph_item_copy, pFp)
GO(pango_glyph_item_free, vFp)
GO(pango_glyph_item_get_logical_widths, vFppp)
GO(pango_glyph_item_get_type, iFv)
GO(pango_glyph_item_iter_copy, pFp)
GO(pango_glyph_item_iter_free, vFp)
GO(pango_glyph_item_iter_get_type, iFv)
GO(pango_glyph_item_iter_init_end, iFppp)
GO(pango_glyph_item_iter_init_start, iFppp)
GO(pango_glyph_item_iter_next_cluster, iFp)
GO(pango_glyph_item_iter_prev_cluster, iFp)
GO(pango_glyph_item_letter_space, vFpppi)
GO(pango_glyph_item_split, pFppi)
GO(pango_glyph_string_copy, pFp)
GO(pango_glyph_string_extents, vFpppp)
GO(pango_glyph_string_extents_range, vFpiippp)
GO(pango_glyph_string_free, vFp)
GO(pango_glyph_string_get_logical_widths, vFppiip)
GO(pango_glyph_string_get_type, iFv)
GO(pango_glyph_string_get_width, iFp)
GO(pango_glyph_string_index_to_x, vFppipiip)
GO(pango_glyph_string_new, pFv)
GO(pango_glyph_string_set_size, vFpi)
GO(pango_glyph_string_x_to_index, vFppipipp)
GO(pango_gravity_get_for_matrix, uFp)
GO(pango_gravity_get_for_script, uFiuu)
GO(pango_gravity_get_for_script_and_width, uFiiuu)
GO(pango_gravity_get_type, iFv)
//GO(pango_gravity_hint_get_type, 
GO(pango_gravity_to_rotation, dFu)
GO(pango_is_zero_width, iFu)
GO(pango_item_copy, pFp)
GO(pango_item_free, vFp)
//GO(pango_item_get_type, 
GO(pango_itemize, pFppiipp)
GO(pango_itemize_with_base_dir, pFpupiipp)
GO(pango_item_new, pFv)
GO(pango_item_split, pFpii)
GO(pango_language_from_string, pFp)
GO(pango_language_get_default, pFv)
GO(pango_language_get_sample_string, pFp)
GO(pango_language_get_scripts, pFpp)
GO(pango_language_get_type, LFv)
GO(pango_language_includes_script, iFpi)
GO(pango_language_matches, iFpp)
GO(pango_language_to_string, pFp)
GO(pango_layout_context_changed, vFp)
GO(pango_layout_copy, pFp)
GO(pango_layout_get_alignment, uFp)
GO(pango_layout_get_attributes, pFp)
GO(pango_layout_get_auto_dir, iFp)
GO(pango_layout_get_baseline, iFp)
GO(pango_layout_get_character_count, iFp)
GO(pango_layout_get_context, pFp)
GO(pango_layout_get_cursor_pos, vFpipp)
GO(pango_layout_get_ellipsize, uFp)
GO(pango_layout_get_extents, vFppp)
GO(pango_layout_get_font_description, pFp)
GO(pango_layout_get_height, iFp)
GO(pango_layout_get_indent, iFp)
GO(pango_layout_get_iter, pFp)
GO(pango_layout_get_justify, iFp)
GO(pango_layout_get_line, pFpi)
GO(pango_layout_get_line_count, iFp)
GO(pango_layout_get_line_readonly, pFpi)
GO(pango_layout_get_lines, pFp)
GO(pango_layout_get_lines_readonly, pFp)
GO(pango_layout_get_log_attrs, vFppp)
GO(pango_layout_get_log_attrs_readonly, pFpp)
GO(pango_layout_get_pixel_extents, vFppp)
GO(pango_layout_get_pixel_size, vFppp)
GO(pango_layout_get_serial, uFp)
GO(pango_layout_get_single_paragraph_mode, iFp)
GO(pango_layout_get_size, vFppp)
GO(pango_layout_get_spacing, iFp)
GO(pango_layout_get_tabs, pFp)
GO(pango_layout_get_text, pFp)
GO(pango_layout_get_type, iFv)
GO(pango_layout_get_unknown_glyphs_count, iFp)
GO(pango_layout_get_width, iFp)
GO(pango_layout_get_wrap, uFp)
GO(pango_layout_index_to_line_x, vFpiipp)
GO(pango_layout_index_to_pos, vFpip)
GO(pango_layout_is_ellipsized, iFp)
GO(pango_layout_is_wrapped, iFp)
GO(pango_layout_iter_at_last_line, iFp)
GO(pango_layout_iter_copy, pFp)
GO(pango_layout_iter_free, vFp)
GO(pango_layout_iter_get_baseline, iFp)
GO(pango_layout_iter_get_char_extents, vFpp)
GO(pango_layout_iter_get_cluster_extents, vFppp)
GO(pango_layout_iter_get_index, iFp)
GO(pango_layout_iter_get_layout, pFp)
GO(pango_layout_iter_get_layout_extents, vFppp)
GO(pango_layout_iter_get_line, pFp)
GO(pango_layout_iter_get_line_extents, vFppp)
GO(pango_layout_iter_get_line_readonly, pFp)
GO(pango_layout_iter_get_line_yrange, vFppp)
GO(pango_layout_iter_get_run, pFp)
GO(pango_layout_iter_get_run_extents, vFppp)
GO(pango_layout_iter_get_run_readonly, pFp)
GO(pango_layout_iter_get_type, iFv)
GO(pango_layout_iter_next_char, iFp)
GO(pango_layout_iter_next_cluster, iFp)
GO(pango_layout_iter_next_line, iFp)
GO(pango_layout_iter_next_run, iFp)
GO(pango_layout_line_get_extents, vFppp)
GO(pango_layout_line_get_pixel_extents, vFppp)
GO(pango_layout_line_get_type, iFv)
GO(pango_layout_line_get_x_ranges, vFpiipp)
GO(pango_layout_line_index_to_x, vFpiip)
GO(pango_layout_line_ref, pFp)
GO(pango_layout_line_unref, vFp)
GO(pango_layout_line_x_to_index, iFpipp)
GO(pango_layout_move_cursor_visually, vFpiiiipp)
GO(pango_layout_new, pFp)
GO(pango_layout_set_alignment, vFpu)
GO(pango_layout_set_attributes, vFpp)
GO(pango_layout_set_auto_dir, vFpi)
GO(pango_layout_set_ellipsize, vFpu)
GO(pango_layout_set_font_description, vFpp)
GO(pango_layout_set_height, vFpi)
GO(pango_layout_set_indent, vFpi)
GO(pango_layout_set_justify, vFpi)
GO(pango_layout_set_markup, vFppi)
GO(pango_layout_set_markup_with_accel, vFppiup)
GO(pango_layout_set_single_paragraph_mode, vFpi)
GO(pango_layout_set_spacing, vFpi)
GO(pango_layout_set_tabs, vFpp)
GO(pango_layout_set_text, vFppi)
GO(pango_layout_set_width, vFpi)
GO(pango_layout_set_wrap, vFpu)
GO(pango_layout_xy_to_index, iFpiipp)
GO(pango_log2vis_get_embedding_levels, pFpip)
//GO(pango_lookup_aliases, 
//GO(pango_map_get_engine, 
//GO(pango_map_get_engines, 
GO(pango_markup_parser_finish, iFppppp)
GO(pango_markup_parser_new, pFu)
GO(pango_matrix_concat, vFpp)
GO(pango_matrix_copy, pFp)
GO(pango_matrix_free, vFp)
GO(pango_matrix_get_font_scale_factor, dFp)
GO(pango_matrix_get_font_scale_factors, vFppp)  //1.38+
GO(pango_matrix_get_type, LFv)
GO(pango_matrix_rotate, vFpd)
GO(pango_matrix_scale, vFpdd)
GO(pango_matrix_transform_distance, vFppp)
GO(pango_matrix_transform_pixel_rectangle, vFpp)
GO(pango_matrix_transform_point, vFppp)
GO(pango_matrix_transform_rectangle, vFpp)
GO(pango_matrix_translate, vFpdd)
//GO(pango_module_register, 
GO(pango_parse_enum, iFLppip)
GO(pango_parse_markup, iFpiupppp)
GO(pango_parse_stretch, iFppi)
GO(pango_parse_style, iFppi)
GO(pango_parse_variant, iFppi)
GO(pango_parse_weight, iFppi)
GO(pango_quantize_line_geometry, vFpp)
GO(pango_read_line, iFpp)
//GO(pango_renderer_activate, 
GO(pango_renderer_deactivate, vFp)
GO(pango_renderer_draw_error_underline, vFpiiii)
GO(pango_renderer_draw_glyph, vFppudd)
GO(pango_renderer_draw_glyph_item, vFpppii)
GO(pango_renderer_draw_glyphs, vFpppii)
GO(pango_renderer_draw_layout, vFppii)
GO(pango_renderer_draw_layout_line, vFppii)
GO(pango_renderer_draw_rectangle, vFpuiiii)
GO(pango_renderer_draw_trapezoid, vFpudddddd)
GO(pango_renderer_get_color, pFpu)
GO(pango_renderer_get_layout, pFp)
GO(pango_renderer_get_layout_line, pFp)
GO(pango_renderer_get_matrix, pFp)
GO(pango_renderer_get_type, LFv)
GO(pango_renderer_part_changed, vFpu)
GO(pango_renderer_set_color, vFpup)
GO(pango_renderer_set_matrix, vFpp)
//GO(pango_render_part_get_type, 
GO(pango_reorder_items, pFp)
GO(pango_scan_int, iFpp)
GO(pango_scan_string, iFpp)
GO(pango_scan_word, iFpp)
GO(pango_script_for_unichar, iFu)
GO(pango_script_get_sample_language, pFi)
GO(pango_script_get_type, iFv)
GO(pango_script_iter_free, vFp)
GO(pango_script_iter_get_range, vFpppp)
GO(pango_script_iter_new, pFpi)
GO(pango_script_iter_next, iFp)
GO(pango_shape, vFpipp)
GO(pango_shape_full, vFpipipp)
GO(pango_skip_space, iFp)
GO(pango_split_file_list, pFp)
GO(pango_stretch_get_type, iFv)
GO(pango_style_get_type, iFv)
GO(pango_tab_align_get_type, iFv)
GO(pango_tab_array_copy, pFp)
GO(pango_tab_array_free, vFp)
GO(pango_tab_array_get_positions_in_pixels, iFp)
GO(pango_tab_array_get_size, iFp)
GO(pango_tab_array_get_tab, vFpipp)
GO(pango_tab_array_get_tabs, vFppp)
GO(pango_tab_array_get_type, iFv)
GO(pango_tab_array_new, pFii)
GO(pango_tab_array_new_with_positions, pFiiiippppppppppppppp) //vaarg
GO(pango_tab_array_resize, vFpi)
GO(pango_tab_array_set_tab, vFpiui)
GO(pango_trim_string, pFp)
GO(pango_underline_get_type, iFv)
GO(pango_unichar_direction, uFu)
GO(pango_units_from_double, iFd)
GO(pango_units_to_double, dFi)
GO(pango_variant_get_type, pFv)
GO(pango_version, iFv)
GO(pango_version_check, pFiii)
GO(pango_version_string, pFv)
GO(pango_weight_get_type, iFv)
GO(pango_wrap_mode_get_type, iFv)
