#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GO(avc_add_callback, 
//GO(avc_audit, 
//GO(avc_av_stats, 
//GO(avc_cache_stats, 
//GO(avc_cleanup, 
//GO(avc_compute_create, 
//GO(avc_compute_member, 
//GO(avc_context_to_sid, 
//GO(avc_context_to_sid_raw, 
//GO(avc_destroy, 
//GO(avc_get_initial_sid, 
//GO(avc_has_perm, 
//GO(avc_has_perm_noaudit, 
//GO(avc_init, 
//GO(avc_netlink_acquire_fd, 
//GO(avc_netlink_check_nb, 
//GO(avc_netlink_close, 
//GO(avc_netlink_loop, 
//GO(avc_netlink_open, 
//GO(avc_netlink_release_fd, 
//GO(avc_open, 
//GO(avc_reset, 
//GO(avc_sid_stats, 
//GO(avc_sid_to_context, 
//GO(avc_sid_to_context_raw, 
//GO(checkPasswdAccess, 
GO(context_free, vFp)
GO(context_new, pFp)
//GO(context_range_get, 
GO(context_range_set, iFpp)
//GO(context_role_get, 
GO(context_role_set, iFpp)
GO(context_str, pFp)
GO(context_type_get, iFpp)
GO(context_type_set, iFpp)
//GO(context_user_get, 
GO(context_user_set, iFpp)
//DATAB(dir_xattr_list, 
GO(fgetfilecon, iFip)
GO(fgetfilecon_raw, iFip)
//GO(fini_selinuxmnt, 
GO(freecon, vFp)
GO(freeconary, vFp)
//GO(fsetfilecon, 
GO(fsetfilecon_raw, iFip)
//GO(getcon, 
GO(getcon_raw, iFp)
//GO(get_default_context, 
//GO(get_default_context_with_level, 
//GO(get_default_context_with_role, 
//GO(get_default_context_with_rolelevel, 
//GO(get_default_type, 
//GO(getexeccon, 
//GO(getexeccon_raw, 
GO(getfilecon, iFpp)
GO(getfilecon_raw, iFpp)
GO(getfscreatecon, iFp)
GO(getfscreatecon_raw, iFp)
//GO(getkeycreatecon, 
//GO(getkeycreatecon_raw, 
//GO(get_ordered_context_list, 
//GO(get_ordered_context_list_with_level, 
//GO(getpeercon, 
//GO(getpeercon_raw, 
//GO(getpidcon, 
//GO(getpidcon_raw, 
//GO(getprevcon, 
GO(getprevcon_raw, iFp)
//GO(getseuser, 
//GO(getseuserbyname, 
//GO(getsockcreatecon, 
//GO(getsockcreatecon_raw, 
//GO(is_context_customizable, 
GO(is_selinux_enabled, iFv)
//GO(is_selinux_mls_enabled, 
GO(lgetfilecon, iFpp)
GO(lgetfilecon_raw, iFpp)
GO(lsetfilecon, iFpp)
GO(lsetfilecon_raw, iFpp)
//GO(manual_user_enter_context, 
//GO(map_class, 
//GO(map_decision, 
//GO(map_perm, 
//GO(matchmediacon, 
//GO(matchpathcon, 
//GO(matchpathcon_checkmatches, 
//GO(matchpathcon_filespec_add, 
//GO(matchpathcon_filespec_destroy, 
//GO(matchpathcon_filespec_eval, 
//GO(matchpathcon_fini, 
//GO(matchpathcon_index, 
//GO(matchpathcon_init, 
//GO(matchpathcon_init_prefix, 
GO(mode_to_security_class, iFup)
//DATAB(myprintf_compat, 
//GO(print_access_vector, 
//GO(query_user_context, 
//GO(realpath_not_final, 
//GO(rpm_execcon, 
//GO(security_av_perm_to_string, 
//GO(security_av_string, 
//GO(security_canonicalize_context, 
//GO(security_canonicalize_context_raw, 
GO(security_check_context, iFp)
//GO(security_check_context_raw, 
//GO(security_class_to_string, 
//GO(security_commit_booleans, 
//GO(security_compute_av, 
//GO(security_compute_av_flags, 
//GO(security_compute_av_flags_raw, 
//GO(security_compute_av_raw, 
//GO(security_compute_create, 
//GO(security_compute_create_name, 
//GO(security_compute_create_name_raw, 
GO(security_compute_create_raw, iFpppp)
//GO(security_compute_member, 
//GO(security_compute_member_raw, 
//GO(security_compute_relabel, 
//GO(security_compute_relabel_raw, 
//GO(security_compute_user, 
//GO(security_compute_user_raw, 
//GO(security_deny_unknown, 
//GO(security_disable, 
GO(security_get_boolean_active, iFp)
GO(security_get_boolean_names, iFpp)
//GO(security_get_boolean_pending, 
//GO(security_get_checkreqprot, 
GO(security_getenforce, iFv)
//GO(security_get_initial_context, 
//GO(security_get_initial_context_raw, 
//GO(security_load_booleans, 
//GO(security_load_policy, 
//GO(security_policyvers, 
//GO(security_reject_unknown, 
//GO(security_set_boolean, 
GO(security_set_boolean_list, iFLpi)
//GO(security_setenforce, 
//GO(security_validatetrans, 
//GO(security_validatetrans_raw, 
GO(selabel_close, vFp)
//GO(selabel_cmp, 
//GO(selabel_digest, 
//GO(selabel_get_digests_all_partial_matches, 
//GO(selabel_hash_all_partial_matches, 
GO(selabel_lookup, iFpppi)
//GO(selabel_lookup_best_match, 
//GO(selabel_lookup_best_match_raw, 
GO(selabel_lookup_raw, iFppp)
GO(selabel_open, pFipL)
//GO(selabel_partial_match, 
//GO(selabel_stats, 
GO(selinux_binary_policy_path, pFv)
//GO(selinux_booleans_path, 
//GO(selinux_booleans_subs_path, 
GO(selinux_boolean_sub, pFp)
GO(selinux_check_access, iFppppv)
//GO(selinux_check_passwd_access, 
//GO(selinux_check_securetty_context, 
//GO(selinux_colors_path, 
//GO(selinux_contexts_path, 
//GO(selinux_current_policy_path, 
//GO(selinux_customizable_types_path, 
//GO(selinux_default_context_path, 
//GO(selinux_default_type_path, 
//GO(selinux_failsafe_context_path, 
//GO(selinux_file_context_cmp, 
GO(selinux_file_context_homedir_path, pFv)
GO(selinux_file_context_local_path, pFv)
GO(selinux_file_context_path, pFv)
//GO(selinux_file_context_subs_dist_path, 
//GO(selinux_file_context_subs_path, 
//GO(selinux_file_context_verify, 
//GO(selinuxfs_exists, 
GOM(selinux_get_callback, pFi)
//GO(selinux_getenforcemode, 
//GO(selinux_getpolicytype, 
//GO(selinux_homedir_context_path, 
//GO(selinux_init_load_policy, 
//GO(selinux_lsetfilecon_default, 
//GO(selinux_lxc_contexts_path, 
//GO(selinux_media_context_path, 
//GO(selinux_mkload_policy, 
//DATAB(selinux_mnt, 
GO(selinux_netfilter_context_path, pFv)
//GO(selinux_openrc_contexts_path, 
//GO(selinux_openssh_contexts_path, 
GO(selinux_path, pFv)
GO(selinux_policy_root, pFv)
//GO(selinux_raw_context_to_color, 
//GO(selinux_raw_to_trans_context, 
//GO(selinux_removable_context_path, 
//GO(selinux_reset_config, 
GO(selinux_restorecon, iFpu)
GO(selinux_restorecon_default_handle, pFv)
//GO(selinux_restorecon_set_alt_rootpath, 
//GO(selinux_restorecon_set_exclude_list, 
GO(selinux_restorecon_set_sehandle, vFp)
//GO(selinux_restorecon_xattr, 
//GO(selinux_securetty_types_path, 
//GO(selinux_sepgsql_context_path, 
GOM(selinux_set_callback, vFEip)
//GO(selinux_set_mapping, 
GO(selinux_set_policy_root, iFp)
//GO(selinux_snapperd_contexts_path, 
GO(selinux_status_close, vFv)
//GO(selinux_status_deny_unknown, 
//GO(selinux_status_getenforce, 
GO(selinux_status_open, iFi)
//GO(selinux_status_policyload, 
GO(selinux_status_updated, iFv)
//GO(selinux_systemd_contexts_path, 
//GO(selinux_translations_path, 
GO(selinux_trans_to_raw_context, iFpp)
//GO(selinux_user_contexts_path, 
GO(selinux_usersconf_path, pFv)
//GO(selinux_users_path, 
//GO(selinux_virtual_domain_context_path, 
//GO(selinux_virtual_image_context_path, 
//GO(selinux_x_context_path, 
GO(setcon, iFp)
//GO(setcon_raw, 
//GO(setexeccon, 
//GO(setexeccon_raw, 
GO(setexecfilecon, iFpp)
GO(setfilecon, iFpp)
GO(setfilecon_raw, iFpp)
GO(setfscreatecon, iFp)
GO(setfscreatecon_raw, iFp)
//GO(setkeycreatecon, 
//GO(setkeycreatecon_raw, 
//GO(set_matchpathcon_canoncon, 
//GO(set_matchpathcon_flags, 
//GO(set_matchpathcon_invalidcon, 
//GO(set_matchpathcon_printf, 
//GO(set_selinuxmnt, 
//GO(setsockcreatecon, 
//GO(setsockcreatecon_raw, 
//GO(sidget, 
//GO(sidput, 
//GO(string_to_av_perm, 
//GO(string_to_security_class, 
//GO(unmap_class, 
//GO(unmap_perm, 
