#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(DPMSCapable, iFp)
GO(DPMSDisable, iFp)
GO(DPMSEnable, iFp)
GO(DPMSForceLevel, iFpW)
GO(DPMSGetTimeouts, iFpppp)
GO(DPMSGetVersion, iFppp)
GO(DPMSInfo, iFppp)
GO(DPMSQueryExtension, iFppp)
GO(DPMSSetTimeouts, iFpWWW)
GO(XagCreateAssociation, iFppp)
GO(XagCreateEmbeddedApplicationGroup, iFpLLLLp)
GO(XagCreateNonembeddedApplicationGroup, iFpp)
GO(XagDestroyApplicationGroup, iFpL)
GO(XagDestroyAssociation, iFpL)
//GOM(XagGetApplicationGroupAttributes, iFEpLV)
GO(XagQueryApplicationGroup, iFpLp)
GO(XagQueryVersion, iFppp)
GO(XcupGetReservedColormapEntries, iFpipp)
GO(XcupQueryVersion, iFppp)
GO(XcupStoreColors, iFpLpi)
GO(XdbeAllocateBackBufferName, LFpLC)
GO(XdbeBeginIdiom, iFp)
GO(XdbeDeallocateBackBufferName, iFpL)
GO(XdbeEndIdiom, iFp)
GO(XdbeFreeVisualInfo, vFp)
GO(XdbeGetBackBufferAttributes, pFpL)
GO(XdbeGetVisualInfo, pFppp)
GO(XdbeQueryExtension, iFppp)
GO(XdbeSwapBuffers, iFppi)
GO(XeviGetVisualInfo, iFppipp)
GO(XeviQueryExtension, iFp)
GO(XeviQueryVersion, iFppp)
GOM(XextAddDisplay, pFEppppip)
GO(XextCreateExtension, pFv)
GO(XextDestroyExtension, vFp)
DATAB(_XExtensionErrorFunction, sizeof(void*))
GO(XextFindDisplay, pFpp)
GO(XextRemoveDisplay, iFpp)
GO(XGEQueryExtension, iFppp)
GO(XGEQueryVersion, iFppp)
GO(XLbxGetEventBase, iFp)
GO(XLbxQueryExtension, iFpppp)
GO(XLbxQueryVersion, iFppp)
GO(XmbufChangeBufferAttributes, vFpLLp)
GO(XmbufChangeWindowAttributes, vFpLLp)
GO(XmbufClearBufferArea, vFpLiiuui)
GO(XmbufCreateBuffers, iFpLiiip)
//GOM(XmbufCreateStereoWindow, LFEpLiiuuuiupLppp)
GO(XmbufDestroyBuffers, vFpL)
GO(XmbufDisplayBuffers, vFpipii)
GO(XmbufGetBufferAttributes, iFpLp)
GO(XmbufGetScreenInfo, iFpLpppp)
GO(XmbufGetVersion, iFppp)
GO(XmbufGetWindowAttributes, iFpLp)
GO(XmbufQueryExtension, iFppp)
GO(XMissingExtension, iFpp)
GO(XMITMiscGetBugMode, iFp)
GO(XMITMiscQueryExtension, iFppp)
GO(XMITMiscSetBugMode, iFpi)
GO(XSecurityAllocXauth, pFv)
GO(XSecurityFreeXauth, vFp)
GO(XSecurityGenerateAuthorization, pFppLpp)
GO(XSecurityQueryExtension, iFppp)
GO(XSecurityRevokeAuthorization, iFpL)
GOM(XSetExtensionErrorHandler, pFEp)
GO(XShapeCombineMask, vFpLiiiLi)
GO(XShapeCombineRectangles, vFpLiiipiii)
GO(XShapeCombineRegion, vFpLiiipi)
GO(XShapeCombineShape, vFpLiiiLii)
GO(XShapeGetRectangles, pFpLipp)
GO(XShapeInputSelected, LFpL)
GO(XShapeOffsetShape, vFpLiii)
GO(XShapeQueryExtension, iFppp)
GO(XShapeQueryExtents, iFpLpppppppppp)
GO(XShapeQueryVersion, iFppp)
GO(XShapeSelectInput, vFpLL)
GO(XShmAttach, iFpp)
GOM(XShmCreateImage, pFEppuippuu)       //need brige/unbridge...
GO(XShmCreatePixmap, LFpLppuuu)
GO(XShmDetach, iFpp)
GO(XShmGetEventBase, iFp)
GOM(XShmGetImage, iFEpLpiiL)        //need brige/unbridge...
GO(XShmPixmapFormat, iFp)
GOM(XShmPutImage, iFEpLppiiiiuui)       //need brige/unbridge...
GO(XShmQueryExtension, iFp)
GO(XShmQueryVersion, iFpppp)
GO(XSyncAwait, iFppi)
GO(XSyncAwaitFence, iFppi)
GO(XSyncChangeAlarm, iFpLLp)
GO(XSyncChangeCounter, iFpuU)
GO(XSyncCreateAlarm, LFpLp)     // XSyncAlarm is an XID
GO(XSyncCreateCounter, uFpU)    // XSyncValue is a struct of 2 int
GO(XSyncCreateFence, LFpLi)
GO(XSyncDestroyAlarm, iFpL)
GO(XSyncDestroyCounter, iFpL)
GO(XSyncDestroyFence, iFpL)
GO(XSyncFreeSystemCounterList, vFp)
GO(XSyncGetPriority, iFpLp)
GO(XSyncInitialize, iFppp)
GO(XSyncIntsToValue, vFpui)
GO(XSyncIntToValue, vFpi)
GO(XSyncListSystemCounters, pFpp)
GO(XSyncMaxValue, vFp)
GO(XSyncMinValue, vFp)
GO(XSyncQueryAlarm, iFpLp)
GO(XSyncQueryCounter, iFpLp)
GO(XSyncQueryExtension, iFppp)
GO(XSyncQueryFence, iFpLp)
GO(XSyncResetFence, iFpL)
GO(XSyncSetCounter, iFpuU)      // XSyncCounter is an XID
GO(XSyncSetPriority, iFpLi)
GO(XSyncTriggerFence, iFpL)
//GO(XSyncValueAdd, 
//GO(XSyncValueEqual, 
//GO(XSyncValueGreaterOrEqual, 
//GO(XSyncValueGreaterThan, 
//GO(XSyncValueHigh32, 
//GO(XSyncValueIsNegative, 
//GO(XSyncValueIsPositive, 
GO(XSyncValueIsZero, iFp)
//GO(XSyncValueLessOrEqual, 
//GO(XSyncValueLessThan, 
//GO(XSyncValueLow32, 
//GO(XSyncValueSubtract, 
DATA(XTestFakeAckType, sizeof(void*))
GO(XTestFakeInput, iFppii)
GO(XTestFlush, iFp)
GO(XTestGetInput, iFpi)
DATAB(XTestInputActionType, sizeof(void*))
GO(XTestMovePointer, iFpipppu)
GO(XTestPressButton, iFpiLuu)
GO(XTestPressKey, iFpiLuu)
GO(XTestQueryInputSize, iFpp)
GO(XTestReset, iFp)
GO(XTestStopInput, iFp)
