/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedsdl1TYPES32_H_
#define __wrappedsdl1TYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void (*vFv_t)(void);
typedef void (*vFp_t)(void*);
typedef int32_t (*iFv_t)(void);
typedef int32_t (*iFp_t)(void*);
typedef uint32_t (*uFp_t)(void*);
typedef void* (*pFv_t)(void);
typedef void* (*pFp_t)(void*);
typedef void (*vFpp_t)(void*, void*);
typedef int32_t (*iFup_t)(uint32_t, void*);
typedef int32_t (*iFpp_t)(void*, void*);
typedef void* (*pFpi_t)(void*, int32_t);
typedef void* (*pFpu_t)(void*, uint32_t);
typedef void* (*pFpp_t)(void*, void*);
typedef int32_t (*iFpuu_t)(void*, uint32_t, uint32_t);
typedef void* (*pFupp_t)(uint32_t, void*, void*);
typedef int32_t (*iFpiuu_t)(void*, int32_t, uint32_t, uint32_t);
typedef uint32_t (*uFpCCC_t)(void*, uint8_t, uint8_t, uint8_t);
typedef void* (*pFiiiu_t)(int32_t, int32_t, int32_t, uint32_t);
typedef void (*vFupppp_t)(uint32_t, void*, void*, void*, void*);
typedef int32_t (*iFpWCiWCi_t)(void*, uint16_t, uint8_t, int32_t, uint16_t, uint8_t, int32_t);
typedef void* (*pFpiiiiuuuu_t)(void*, int32_t, int32_t, int32_t, int32_t, uint32_t, uint32_t, uint32_t, uint32_t);

#define SUPER() ADDED_FUNCTIONS() \
	GO(SDL_Quit, vFv_t) \
	GO(SDL_FreeSurface, vFp_t) \
	GO(SDL_KillThread, vFp_t) \
	GO(SDL_UnloadObject, vFp_t) \
	GO(SDL_UnlockSurface, vFp_t) \
	GO(SDL_Has3DNow, iFv_t) \
	GO(SDL_Has3DNowExt, iFv_t) \
	GO(SDL_HasAltiVec, iFv_t) \
	GO(SDL_HasMMX, iFv_t) \
	GO(SDL_HasMMXExt, iFv_t) \
	GO(SDL_HasRDTSC, iFv_t) \
	GO(SDL_HasSSE, iFv_t) \
	GO(SDL_HasSSE2, iFv_t) \
	GO(SDL_ConvertAudio, iFp_t) \
	GO(SDL_Flip, iFp_t) \
	GO(SDL_LockSurface, iFp_t) \
	GO(SDL_PollEvent, iFp_t) \
	GO(SDL_PushEvent, iFp_t) \
	GO(SDL_ReadLE32, uFp_t) \
	GO(SDL_RemoveTimer, uFp_t) \
	GO(SDL_GetVideoInfo, pFv_t) \
	GO(SDL_GetVideoSurface, pFv_t) \
	GO(SDL_GL_GetProcAddress, pFp_t) \
	GO(SDL_LoadObject, pFp_t) \
	GO(SDL_WM_SetIcon, vFpp_t) \
	GO(SDL_SetTimer, iFup_t) \
	GO(SDL_OpenAudio, iFpp_t) \
	GO(SDL_LoadBMP_RW, pFpi_t) \
	GO(SDL_RWFromMem, pFpi_t) \
	GO(SDL_ListModes, pFpu_t) \
	GO(SDL_CreateThread, pFpp_t) \
	GO(SDL_LoadFunction, pFpp_t) \
	GO(SDL_RWFromFile, pFpp_t) \
	GO(SDL_SetColorKey, iFpuu_t) \
	GO(SDL_AddTimer, pFupp_t) \
	GO(SDL_PeepEvents, iFpiuu_t) \
	GO(SDL_MapRGB, uFpCCC_t) \
	GO(SDL_SetVideoMode, pFiiiu_t) \
	GO(SDL_GetRGB, vFupppp_t) \
	GO(SDL_BuildAudioCVT, iFpWCiWCi_t) \
	GO(SDL_CreateRGBSurfaceFrom, pFpiiiiuuuu_t)

#endif // __wrappedsdl1TYPES32_H_
