#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

GO(app_indicator_build_menu_from_desktop, vFppp)
GO(app_indicator_category_get_type, pFv)
GO(app_indicator_get_attention_icon, pFp)
GO(app_indicator_get_attention_icon_desc, pFp)
GO(app_indicator_get_category, iFp)
GO(app_indicator_get_icon, pFp)
GO(app_indicator_get_icon_desc, pFp)
GO(app_indicator_get_icon_theme_path, pFp)
GO(app_indicator_get_id, pFp)
GO(app_indicator_get_label, pFp)
GO(app_indicator_get_label_guide, pFp)
GO(app_indicator_get_menu, pFp)
GO(app_indicator_get_ordering_index, uFp)
GO(app_indicator_get_secondary_activate_target, pFp)
GO(app_indicator_get_status, iFp)
GO(app_indicator_get_title, pFp)
GO(app_indicator_get_type, iFv)
GO(app_indicator_new, pFppi)
GO(app_indicator_new_with_path, pFppip)
GO(app_indicator_set_attention_icon, vFpp)
GO(app_indicator_set_attention_icon_full, vFppp)
GO(app_indicator_set_icon, vFpp)
GO(app_indicator_set_icon_full, vFppp)
GO(app_indicator_set_icon_theme_path, vFpp)
GO(app_indicator_set_label, vFppp)
GO(app_indicator_set_menu, vFpp)
GO(app_indicator_set_ordering_index, vFpu)
GO(app_indicator_set_secondary_activate_target, vFpp)
GO(app_indicator_set_status, vFpi)
GO(app_indicator_set_title, vFpp)
GO(app_indicator_status_get_type, iFv)
//GO(_application_service_marshal_VOID__BOOLEAN_STRING_OBJECT, 
//GO(_application_service_marshal_VOID__INT_INT, 
//GO(_application_service_marshal_VOID__INT_STRING, 
//GO(_application_service_marshal_VOID__INT_STRING_STRING, 
//GO(_application_service_marshal_VOID__INT_UINT, 
//GO(_application_service_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING, 
//GO(_application_service_marshal_VOID__STRING_STRING, 
//GO(_generate_id, 
//DATA(_notification_item, 
//DATA(_notification_watcher, 
