#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif


GO(gcry_calloc, pFLL)
GO(gcry_calloc_secure, pFLL)
GO(gcry_check_version, pFp)
GO(gcry_cipher_algo_info, uFiipp)
GO(gcry_cipher_algo_name, pFi)
GO(gcry_cipher_authenticate, uFppL)
GO(gcry_cipher_checktag, uFppL)
GO(gcry_cipher_close, vFp)
GO(gcry_cipher_ctl, uFpipL)
GO(gcry_cipher_decrypt, uFppLpL)
GO(gcry_cipher_encrypt, uFppLpL)
GO(gcry_cipher_get_algo_blklen, LFi)
GO(gcry_cipher_get_algo_keylen, LFi)
GO(gcry_cipher_gettag, uFppL)
GO(gcry_cipher_info, uFpipp)
GO(gcry_cipher_map_name, iFp)
GO(gcry_cipher_mode_from_oid, iFp)
GO(gcry_cipher_open, uFpiiu)
GO(gcry_cipher_setctr, uFppL)
GO(gcry_cipher_setiv, uFppL)
GO(gcry_cipher_setkey, uFppL)
GO(gcry_control, uFuNN)
GO(gcry_create_nonce, vFpL)
GO(gcry_ctx_release, vFp)
GO(gcry_ecc_get_algo_keylen, uFi)
GO(gcry_ecc_mul_point, uFippp)
GO(gcry_err_code_from_errno, uFi)
GO(gcry_err_code_to_errno, iFu)
GO(gcry_err_make_from_errno, uFui)
GO(gcry_error_from_errno, uFi)
GO(gcry_free, vFp)
GO(gcry_get_config, pFip)
GO(gcry_is_secure, iFp)
GO(gcry_kdf_close, vFp)
//GOM(gcry_kdf_compute, uFEpp)
GO(gcry_kdf_derive, uFpLiipLLLp)
GO(gcry_kdf_final, uFpLp)
GO(gcry_kdf_open, uFpiipupLpLpLpL)
//GOM(gcry_log_debug, vFEpV)
GO(gcry_log_debughex, vFppL)
GO(gcry_log_debugmpi, vFpp)
GO(gcry_log_debugpnt, vFppp)
GO(gcry_log_debugsxp, vFpp)
GO(gcry_mac_algo_info, uFiipp)
GO(gcry_mac_algo_name, pFi)
GO(gcry_mac_close, vFp)
GO(gcry_mac_ctl, uFpipL)
GO(gcry_mac_get_algo, iFp)
GO(gcry_mac_get_algo_keylen, uFi)
GO(gcry_mac_get_algo_maclen, uFi)
GO(gcry_mac_map_name, iFp)
GO(gcry_mac_open, uFpiup)
GO(gcry_mac_read, uFppp)
GO(gcry_mac_setiv, uFppL)
GO(gcry_mac_setkey, uFppL)
GO(gcry_mac_verify, uFppL)
GO(gcry_mac_write, uFppL)
GO(gcry_malloc, pFL)
GO(gcry_malloc_secure, pFL)
GO(gcry_md_algo_info, uFiipp)
GO(gcry_md_algo_name, pFi)
GO(gcry_md_close, vFp)
GO(gcry_md_copy, uFpp)
GO(gcry_md_ctl, uFpipL)
GO(gcry_md_debug, vFpp)
GO(gcry_md_enable, uFpi)
GO(gcry_md_extract, uFpipL)
GO(gcry_md_get_algo, iFp)
GO(gcry_md_get_algo_dlen, uFi)
GO(gcry_md_hash_buffer, vFippL)
GO(gcry_md_hash_buffers, uFiuppi)
GO(gcry_md_info, uFpipp)
GO(gcry_md_is_enabled, iFpi)
GO(gcry_md_putc, vFpi)
GO(gcry_md_final, vFp)
GO(gcry_md_get_asnoid, uFipL)
GO(gcry_md_test_algo, uFi)
GO(gcry_md_is_secure, iFp)
GO(gcry_md_map_name, iFp)
GO(gcry_md_open, uFpiu)
GO(gcry_md_read, pFpi)
GO(gcry_md_reset, vFp)
GO(gcry_md_setkey, uFppL)
GO(gcry_md_write, vFppL)
GO(gcry_mpi_abs, vFp)
GO(gcry_mpi_add, vFppp)
GO(gcry_mpi_addm, vFpppp)
GO(gcry_mpi_add_ui, vFppL)
GO(gcry_mpi_aprint, uFuppp)
GO(gcry_mpi_clear_bit, vFpu)
GO(gcry_mpi_clear_flag, vFpu)
GO(gcry_mpi_clear_highbit, vFpu)
GO(gcry_mpi_cmp, iFpp)
GO(gcry_mpi_cmp_ui, iFpL)
GO(gcry_mpi_copy, pFp)
GO(gcry_mpi_div, vFppppi)
GO(gcry_mpi_dump, vFp)
GO(gcry_mpi_ec_add, vFpppp)
GO(gcry_mpi_ec_curve_point, iFpp)
GO(gcry_mpi_ec_decode_point, uFppp)
GO(gcry_mpi_ec_dup, vFppp)
GO(gcry_mpi_ec_get_affine, iFpppp)
GO(gcry_mpi_ec_get_mpi, pFppi)
GO(gcry_mpi_ec_get_point, pFppi)
GO(gcry_mpi_ec_mul, vFpppp)
GO(gcry_mpi_ec_new, uFppp)
GO(gcry_mpi_ec_set_mpi, uFppp)
GO(gcry_mpi_ec_set_point, uFppp)
GO(gcry_mpi_ec_sub, vFpppp)
GO(gcry_mpi_gcd, iFppp)
GO(_gcry_mpi_get_const, pFi)
GO(gcry_mpi_get_flag, iFpu)
GO(gcry_mpi_get_nbits, uFp)
GO(gcry_mpi_get_opaque, pFpp)
GO(gcry_mpi_get_ui, uFpp)
GO(gcry_mpi_invm, iFppp)
GO(gcry_mpi_is_neg, iFp)
GO(gcry_mpi_lshift, vFppu)
GO(gcry_mpi_mod, vFppp)
GO(gcry_mpi_mul, vFppp)
GO(gcry_mpi_mul_2exp, vFppL)
GO(gcry_mpi_mulm, vFpppp)
GO(gcry_mpi_mul_ui, vFppL)
GO(gcry_mpi_neg, vFpp)
GO(gcry_mpi_new, pFu)
GO(gcry_mpi_point_copy, pFp)
GO(gcry_mpi_point_get, vFpppp)
GO(gcry_mpi_point_new, pFu)
GO(gcry_mpi_point_release, vFp)
GO(gcry_mpi_point_set, pFpppp)
GO(gcry_mpi_point_snatch_get, vFpppp)
GO(gcry_mpi_point_snatch_set, pFpppp)
GO(gcry_mpi_powm, vFpppp)
GO(gcry_mpi_print, uFupLpp)
GO(gcry_mpi_randomize, vFpuu)
GO(gcry_mpi_release, vFp)
GO(gcry_mpi_rshift, vFppu)
GO(gcry_mpi_scan, uFpupLp)
GO(gcry_mpi_set, pFpp)
GO(gcry_mpi_set_bit, vFpu)
GO(gcry_mpi_set_flag, vFpu)
GO(gcry_mpi_set_highbit, vFpu)
GO(gcry_mpi_set_opaque, pFppu)
GO(gcry_mpi_set_opaque_copy, pFppu)
GO(gcry_mpi_set_ui, pFpL)
GO(gcry_mpi_snatch, vFpp)
GO(gcry_mpi_snew, pFu)
GO(gcry_mpi_sub, vFppp)
GO(gcry_mpi_subm, vFpppp)
GO(gcry_mpi_sub_ui, vFppL)
GO(gcry_mpi_swap, vFpp)
GO(gcry_mpi_test_bit, iFpu)
GO(gcry_pk_algo_info, uFiipp)
GO(gcry_pk_algo_name, pFi)
GO(gcry_pk_ctl, uFipL)
GO(gcry_pk_decrypt, uFppp)
GO(gcry_pk_encrypt, uFppp)
GO(gcry_pk_genkey, uFpp)
GO(gcry_pk_get_curve, pFpip)
GO(gcry_pk_get_keygrip, pFpp)
GO(gcry_pk_get_nbits, uFp)
GO(gcry_pk_get_param, pFip)
GO(gcry_pk_hash_sign, uFppppp)
GO(gcry_pk_hash_verify, uFppppp)
GO(gcry_pk_map_name, iFp)
GO(gcry_pk_random_override_new, uFppL)
GO(gcry_pk_sign, uFppp)
GO(gcry_pk_testkey, uFp)
GO(gcry_pk_verify, uFppp)
GO(gcry_prime_check, uFpu)
//GOM(gcry_prime_generate, uFEpuupppuu)
GO(gcry_prime_group_generator, uFpppp)
GO(gcry_prime_release_factors, vFp)
GO(gcry_pubkey_get_sexp, uFpip)
GO(gcry_random_add_bytes, uFpLi)
GO(gcry_random_bytes, pFLu)
GO(gcry_random_bytes_secure, pFLu)
GO(gcry_randomize, vFpLu)
GO(gcry_realloc, pFpL)
//GOM(gcry_set_allocation_handler, vFEppppp)
//GOM(gcry_set_fatalerror_handler, vFEpp)
//GOM(gcry_set_gettext_handler, vFEp)
//GOM(gcry_set_log_handler, vFEpp)
//GOM(gcry_set_outofcore_handler, vFEpp)
//GOM(gcry_set_progress_handler, vFEpp)
GO(gcry_sexp_alist, pFp)
GO(gcry_sexp_append, pFpp)
GOM(gcry_sexp_build, uFEpppV)
GO(gcry_sexp_build_array, uFpppp)
GO(gcry_sexp_cadr, pFp)
GO(gcry_sexp_canon_len, LFpLpp)
GO(gcry_sexp_car, pFp)
GO(gcry_sexp_cdr, pFp)
GO(gcry_sexp_cons, pFpp)
//GOM(gcry_sexp_create, uFEppLip)
GO(gcry_sexp_dump, vFp)
//GOM(gcry_sexp_extract_param, uFEpppV)
GO(gcry_sexp_find_token, pFppL)
GO(gcry_sexp_length, iFp)
GO(gcry_sexp_new, uFppLi)
GO(gcry_sexp_nth, pFpi)
GO(gcry_sexp_nth_buffer, pFpip)
GO(gcry_sexp_nth_data, pFpip)
GO(gcry_sexp_nth_mpi, pFpii)
GO(gcry_sexp_nth_string, pFpi)
GO(gcry_sexp_prepend, pFpp)
GO(gcry_sexp_release, vFp)
GO(gcry_sexp_sprint, LFpipL)
GO(gcry_sexp_sscan, uFpppL)
//GOM(gcry_sexp_vlist, pFEpV)
GO(gcry_strdup, pFp)
GO(gcry_strerror, pFu)
GO(gcry_strsource, pFu)
GO(gcry_xcalloc, pFLL)
GO(gcry_xcalloc_secure, pFLL)
GO(gcry_xmalloc, pFL)
GO(gcry_xmalloc_secure, pFL)
GO(gcry_xrealloc, pFpL)
GO(gcry_xstrdup, pFp)
