#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(avahi_address_resolver_event, iFpip)
GO(avahi_address_resolver_free, iFp)
GO(avahi_address_resolver_get_client, pFp)
GOM(avahi_address_resolver_new, pFEpiipupp)
GO(avahi_client_errno, iFp)
GO(avahi_client_free, vFp)
GO(avahi_client_get_domain_name, pFp)
GO(avahi_client_get_host_name, pFp)
GO(avahi_client_get_host_name_fqdn, pFp)
GO(avahi_client_get_local_service_cookie, uFp)
GO(avahi_client_get_state, uFp)
GO(avahi_client_get_version_string, pFp)
GO(avahi_client_is_connected, iFp)
GOM(avahi_client_new, pFEpuppp)
GO(avahi_client_set_dbus_error, iFpp)
GO(avahi_client_set_errno, iFpi)
GO(avahi_client_set_host_name, iFpp)
GO(avahi_client_simple_method_call, iFpppp)
GO(avahi_dbus_connection_glue, iFpp)
GO(avahi_domain_browser_event, iFpip)
GO(avahi_domain_browser_free, iFp)
GO(avahi_domain_browser_get_client, pFp)
GOM(avahi_domain_browser_new, pFEpiipuupp)
GO(avahi_entry_group_add_address, iFpiiupp)
GO(avahi_entry_group_add_record, iFpiiupWWupL)
GO(avahi_entry_group_add_service, iFpiiuppppWppppppp)       //should be iFEpiiuppppWV
GO(avahi_entry_group_add_service_strlst, iFpiiuppppWp)
GO(avahi_entry_group_add_service_subtype, iFpiiupppp)
GO(avahi_entry_group_commit, iFp)
GO(avahi_entry_group_free, iFp)
GO(avahi_entry_group_get_client, pFp)
GO(avahi_entry_group_get_state, iFp)
GO(avahi_entry_group_is_empty, iFp)
GOM(avahi_entry_group_new, pFEppp)
GO(avahi_entry_group_reset, iFp)
GO(avahi_entry_group_set_state, vFpi)
GO(avahi_entry_group_update_service_txt, iFpiiuppppppppp)   //should be iFEpiiupppV
GO(avahi_entry_group_update_service_txt_strlst, iFpiiupppp)
GO(avahi_error_dbus_to_number, iFp)
GO(avahi_error_number_to_dbus, pFi)
GO(avahi_host_name_resolver_event, iFpip)
GO(avahi_host_name_resolver_free, iFp)
GO(avahi_host_name_resolver_get_client, pFp)
GOM(avahi_host_name_resolver_new, pFEpiipiupp)
GO(avahi_nss_support, iFv)
GO(avahi_record_browser_event, iFpip)
GO(avahi_record_browser_free, iFp)
GO(avahi_record_browser_get_client, pFp)
GOM(avahi_record_browser_new, pFEpiipWWupp)
GO(avahi_service_browser_event, iFpip)
GO(avahi_service_browser_free, iFp)
GO(avahi_service_browser_get_client, pFp)
GOM(avahi_service_browser_new, pFEpiippupp)
GO(avahi_service_resolver_event, pFpip)
GO(avahi_service_resolver_free, iFp)
GO(avahi_service_resolver_get_client, pFp)
GOM(avahi_service_resolver_new, pFEpiipppiupp)
GO(avahi_service_type_browser_event, iFpip)
GO(avahi_service_type_browser_free, iFp)
GO(avahi_service_type_browser_get_client, pFp)
GOM(avahi_service_type_browser_new, pFEpiipupp)
GO(avahi_xdg_config_open, pFp)
