#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ucal_add_64, vFpiip)
GO(ucal_clear_64, vFp)
GO(ucal_clearField_64, vFpi)
GO(ucal_clone_64, pFpp)
GO(ucal_close_64, vFp)
GO(ucal_countAvailable_64, iFv)
GO(ucal_equivalentTo_64, cFpp)
GO(ucal_get_64, iFpip)
GO(ucal_getAttribute_64, iFpi)
GO(ucal_getAvailable_64, pFi)
GO(ucal_getCanonicalTimeZoneID_64, iFpipipp)
GO(ucal_getDayOfWeekType_64, iFpip)
GO(ucal_getDefaultTimeZone_64, iFpip)
GO(ucal_getDSTSavings_64, iFpp)
GO(ucal_getFieldDifference_64, iFpdip)
GO(ucal_getGregorianChange_64, dFpp)
GO(ucal_getHostTimeZone_64, iFpip)
GO(ucal_getKeywordValuesForLocale_64, pFppCp)
GO(ucal_getLimit_64, iFpiip)
GO(ucal_getLocaleByType_64, pFpip)
GO(ucal_getMillis_64, dFpp)
GO(ucal_getNow_64, dFv)
GO(ucal_getTimeZoneDisplayName_64, iFpippip)
GO(ucal_getTimeZoneID_64, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_64, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_64, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_64, cFpipp)
GO(ucal_getType_64, pFpp)
GO(ucal_getTZDataVersion_64, pFp)
GO(ucal_getWeekendTransition_64, iFpip)
GO(ucal_getWindowsTimeZoneID_64, iFpipip)
GO(ucal_inDaylightTime_64, cFpp)
GO(ucal_isSet_64, cFpi)
GO(ucal_isWeekend_64, cFpdp)
GO(ucal_open_64, pFpipip)
GO(ucal_openCountryTimeZones_64, pFpp)
GO(ucal_openTimeZoneIDEnumeration_64, pFippp)
GO(ucal_openTimeZones_64, pFp)
GO(ucal_roll_64, vFpiip)
GO(ucal_set_64, vFpii)
GO(ucal_setAttribute_64, vFpii)
GO(ucal_setDate_64, vFpiiip)
GO(ucal_setDateTime_64, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_64, vFpp)
GO(ucal_setGregorianChange_64, vFpdp)
GO(ucal_setMillis_64, vFpdp)
GO(ucal_setTimeZone_64, vFppip)

// GO(ucol_clone_64, pFpp) // does not exists
GO(ucol_close_64, vFp)
GO(ucol_closeElements_64, vFp)
GO(ucol_setMaxVariable_64, vFpip)
GO(ucol_getOffset_64, iFp)
GO(ucol_getRules_64, pFpp)
GO(ucol_getSortKey_64, iFppipi)
GO(ucol_getStrength_64, iFp)
GO(ucol_getVersion_64, vFpp)
GO(ucol_next_64, iFpp)
GO(ucol_previous_64, iFpp)
GO(ucol_open_64, pFpp)
GO(ucol_openElements_64, pFppip)
GO(ucol_openRules_64, pFpiiipp)
GO(ucol_safeClone_64, pFpppp)
GO(ucol_setAttribute_64, vFpiip)
GO(ucol_setVariableTop_64, uFppip)
GO(ucol_strcoll_64, iFppipi)

GO(udat_close_64, vFp)
GO(udat_countSymbols_64, iFpi)
GO(udat_getSymbols_64, iFpiipip)
GO(udat_format_64, iFpdpipp)
GO(udat_open_64, pFiippipip)
GO(udat_setCalendar_64, vFpp)
GO(udat_toPattern_64, iFpCpip)

GO(unum_close_64, vFp)
GO(unum_getAttribute_64, iFpi)
GO(unum_getSymbol_64, iFpipip)
GO(unum_open_64, pFipippp)
GO(unum_toPattern_64, iFpCpip)

GO(udatpg_close_64, vFp)
GO(udatpg_getBestPattern_64, iFppipip)
GO(udatpg_open_64, pFpp)

GO(ulocdata_getCLDRVersion_64, vFpp)
GO(ulocdata_getMeasurementSystem_64, iFpp)

GO(usearch_close_64, vFp)
GO(usearch_first_64, iFpp)
GO(usearch_getBreakIterator_64, pFp)
GO(usearch_getMatchedLength_64, iFp)
GO(usearch_last_64, iFpp)
GO(usearch_openFromCollator_64, pFpipippp)
GO(usearch_setPattern_64, vFppip)
GO(usearch_setText_64, vFppip)
