#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error meh!
#endif

GO(arc4random, uFv)
GO(arc4random_addrandom, vFpi)
GO(arc4random_buf, vFpL)
GO(arc4random_stir, vFv)
GO(arc4random_uniform, uFu)
//GO(bsd_getopt, 
GO(closefrom, iFi)
//GO(dehumanize_number, 
//GO(errc, 
//GO(expand_number, 
//GO(explicit_bzero, 
//GO(__fdnlist, 
//GO(fgetln, 
//GO(fgetwln, 
//GO(flopen, 
//GO(flopenat, 
//GO(fmtcheck, 
//GO(fparseln, 
//GO(fpurge, 
//GO(freezero, 
//GO(funopen, 
//GO(getbsize, 
//GO(getmode, 
//GO(getpeereid, 
GO(getprogname, pFv)
//GO(gid_from_group, 
//GO(group_from_gid, 
//GO(heapsort, 
//GO(humanize_number, 
//GO(inet_net_pton@@LIBBSD_0.0, 
//GO(_int_to_time, 
//GO(_long_to_time, 
//GO(MD5Data, 
//GO(MD5End, 
//GO(MD5File, 
//GO(MD5FileChunk, 
//GO(MD5Final, 
//GO(MD5Init, 
//GO(MD5Pad, 
//GO(MD5Transform, 
//GO(MD5Update, 
//GO(mergesort, 
//GO(nlist, 
//GO(nvis, 
//DATAB(optreset, 
//GO(pidfile_close, 
//GO(pidfile_fileno, 
//GO(pidfile_open, 
//GO(pidfile_remove, 
//GO(pidfile_write, 
//GO(radixsort, 
//GO(readpassphrase, 
//GO(reallocarray, 
//GO(reallocf, 
//GO(recallocarray, 
//GO(setmode, 
//GO(setproctitle@LIBBSD_0.2, 
//GO(setproctitle, 
//GO(setproctitle_init, 
GO(setprogname, vFp)
//GO(sl_add, 
//GO(sl_find, 
//GO(sl_free, 
//GO(sl_init, 
//GO(snvis, 
//GO(sradixsort, 
//GO(stravis, 
//GO(strenvisx, 
//GO(strlcat, 
//GO(strlcpy, 
//GO(strmode, 
//GO(strnstr, 
//GO(strnunvis, 
//GO(strnunvis@LIBBSD_0.9.1, 
//GO(strnunvis_netbsd.1, 
//GO(strnunvisx, 
//GO(strnvis@LIBBSD_0.9.1, 
//GO(strnvis, 
//GO(strnvis_netbsd, 
//GO(strnvisx, 
//GO(strsenvisx, 
//GO(strsnvis, 
//GO(strsnvisx, 
//GO(strsvis, 
//GO(strsvisx, 
//GO(strtoi, 
//GO(strtonum, 
//GO(strtou, 
//GO(strunvis, 
//GO(strunvisx, 
//GO(strvis, 
//GO(strvisx, 
//GO(svis, 
//GO(_time32_to_time, 
//GO(_time64_to_time, 
//GO(_time_to_int, 
//GO(_time_to_long, 
//GO(_time_to_time32, 
//GO(_time_to_time64, 
//GO(uid_from_user, 
//GO(unvis, 
//GO(user_from_uid, 
//GO(verrc, 
//GO(vis, 
//GO(vwarnc, 
//GO(warnc, 
GO(wcslcat, LFppL)
GO(wcslcpy, LFppL)
