/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedcurlTYPES32_H_
#define __wrappedcurlTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef int32_t (*iFpup_t)(void*, uint32_t, void*);
typedef uint32_t (*uFpup_t)(void*, uint32_t, void*);
typedef int32_t (*iFlppppp_t)(intptr_t, void*, void*, void*, void*, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(curl_multi_setopt, iFpup_t) \
	GO(curl_easy_setopt, uFpup_t) \
	GO(curl_share_setopt, uFpup_t) \
	GO(curl_global_init_mem, iFlppppp_t)

#endif // __wrappedcurlTYPES32_H_
