#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(gst_audio_aggregator_convert_pad_get_type, LFv)
GO(gst_audio_aggregator_get_type, LFv)
GO(gst_audio_aggregator_pad_get_type, LFv)
GO(gst_audio_aggregator_set_sink_caps, vFppp)
//GOM(gst_audio_base_sink_create_ringbuffer, pFEp)
GO(gst_audio_base_sink_discont_reason_get_type, LFv)
//GOM(gst_audio_base_sink_get_alignment_threshold, LFEp)
//GOM(gst_audio_base_sink_get_discont_wait, LFEp)
//GOM(gst_audio_base_sink_get_drift_tolerance, lFEp)
//GOM(gst_audio_base_sink_get_provide_clock, iFEp)
//GOM(gst_audio_base_sink_get_slave_method, uFEp)
GO(gst_audio_base_sink_get_type, LFv)
//GOM(gst_audio_base_sink_report_device_failure, vFEp)
//GOM(gst_audio_base_sink_set_alignment_threshold, vFEpL)
//GOM(gst_audio_base_sink_set_custom_slaving_callback, vFEpppp)
//GOM(gst_audio_base_sink_set_discont_wait, vFEpL)
//GOM(gst_audio_base_sink_set_drift_tolerance, vFEpl)
//GOM(gst_audio_base_sink_set_provide_clock, vFEpi)
//GOM(gst_audio_base_sink_set_slave_method, vFEpu)
GO(gst_audio_base_sink_slave_method_get_type, LFv)
//GOM(gst_audio_base_src_create_ringbuffer, pFEp)
//GOM(gst_audio_base_src_get_provide_clock, iFEp)
//GOM(gst_audio_base_src_get_slave_method, uFEp)
GO(gst_audio_base_src_get_type, LFv)
//GOM(gst_audio_base_src_set_provide_clock, vFEpi)
//GOM(gst_audio_base_src_set_slave_method, vFEpu)
GO(gst_audio_base_src_slave_method_get_type, LFv)
GO(gst_audio_buffer_clip, pFppii)
GO(gst_audio_buffer_map, iFpppu)
GO(gst_audio_buffer_reorder_channels, iFpuipp)
GO(gst_audio_buffer_truncate, pFpiLL)
GO(gst_audio_buffer_unmap, vFp)
GO(gst_audio_cd_src_add_track, iFpp)
GO(gst_audio_cd_src_get_type, LFv)
GO(gst_audio_cd_src_mode_get_type, LFv)
GO(gst_audio_channel_get_fallback_mask, LFi)
GO(gst_audio_channel_mixer_flags_get_type, LFv)
GO(gst_audio_channel_mixer_free, vFp)
GO(gst_audio_channel_mixer_is_passthrough, iFp)
GO(gst_audio_channel_mixer_new, pFuuipip)
GO(gst_audio_channel_mixer_new_with_matrix, pFuuiip)
GO(gst_audio_channel_mixer_samples, vFpppi)
GO(gst_audio_channel_position_get_type, LFv)
GO(gst_audio_channel_positions_from_mask, iFiLp)
GO(gst_audio_channel_positions_to_mask, iFpiip)
GO(gst_audio_channel_positions_to_string, pFpi)
GO(gst_audio_channel_positions_to_valid_order, iFpi)
GO(gst_audio_check_valid_channel_positions, iFpii)
GO(gst_audio_clipping_meta_api_get_type, LFv)
GO(gst_audio_clipping_meta_get_info, pFv)
//GOM(gst_audio_clock_adjust, LFEpL)
//GOM(gst_audio_clock_get_time, LFEp)
GO(gst_audio_clock_get_type, LFv)
//GOM(gst_audio_clock_invalidate, vFEp)
//GOM(gst_audio_clock_new, pFEpppp)
//GOM(gst_audio_clock_reset, vFEpL)
GO(gst_audio_converter_convert, iFpupLpp)
GO(gst_audio_converter_flags_get_type, LFv)
GO(gst_audio_converter_free, vFp)
GO(gst_audio_converter_get_config, pFppp)
GO(gst_audio_converter_get_in_frames, LFpL)
GO(gst_audio_converter_get_max_latency, LFp)
GO(gst_audio_converter_get_out_frames, LFpL)
GO(gst_audio_converter_get_type, LFv)
GO(gst_audio_converter_is_passthrough, iFp)
GO(gst_audio_converter_new, pFuppp)
GO(gst_audio_converter_reset, vFp)
GO(gst_audio_converter_samples, iFpupLpL)
GO(gst_audio_converter_supports_inplace, iFp)
GO(gst_audio_converter_update_config, iFpiip)
GO(gst_audio_decoder_allocate_output_buffer, pFpL)
GO(_gst_audio_decoder_error, iFpiuippppi)
GO(gst_audio_decoder_finish_frame, iFppi)
GO(gst_audio_decoder_finish_subframe, iFpp)
GO(gst_audio_decoder_get_allocator, vFppp)
GO(gst_audio_decoder_get_audio_info, pFp)
GO(gst_audio_decoder_get_delay, iFp)
GO(gst_audio_decoder_get_drainable, iFp)
GO(gst_audio_decoder_get_estimate_rate, iFp)
GO(gst_audio_decoder_get_latency, vFppp)
GO(gst_audio_decoder_get_max_errors, iFp)
GO(gst_audio_decoder_get_min_latency, LFp)
GO(gst_audio_decoder_get_needs_format, iFp)
GO(gst_audio_decoder_get_parse_state, vFppp)
GO(gst_audio_decoder_get_plc, iFp)
GO(gst_audio_decoder_get_plc_aware, iFp)
GO(gst_audio_decoder_get_tolerance, LFp)
GO(gst_audio_decoder_get_type, LFv)
GO(gst_audio_decoder_merge_tags, vFppu)
GO(gst_audio_decoder_negotiate, iFp)
GO(gst_audio_decoder_proxy_getcaps, pFppp)
GO(gst_audio_decoder_set_allocation_caps, vFpp)
GO(gst_audio_decoder_set_drainable, vFpi)
GO(gst_audio_decoder_set_estimate_rate, vFpi)
GO(gst_audio_decoder_set_latency, vFpLL)
GO(gst_audio_decoder_set_max_errors, vFpi)
GO(gst_audio_decoder_set_min_latency, vFpL)
GO(gst_audio_decoder_set_needs_format, vFpi)
GO(gst_audio_decoder_set_output_caps, iFpp)
GO(gst_audio_decoder_set_output_format, iFpp)
GO(gst_audio_decoder_set_plc, vFpi)
GO(gst_audio_decoder_set_plc_aware, vFpi)
GO(gst_audio_decoder_set_tolerance, vFpL)
GO(gst_audio_decoder_set_use_default_pad_acceptcaps, vFpi)
GO(gst_audio_dither_method_get_type, LFv)
GO(gst_audio_downmix_meta_api_get_type, LFv)
GO(gst_audio_downmix_meta_get_info, pFv)
GO(gst_audio_encoder_allocate_output_buffer, pFpL)
GO(gst_audio_encoder_finish_frame, iFppi)
GO(gst_audio_encoder_get_allocator, vFppp)
GO(gst_audio_encoder_get_audio_info, pFp)
GO(gst_audio_encoder_get_drainable, iFp)
GO(gst_audio_encoder_get_frame_max, iFp)
GO(gst_audio_encoder_get_frame_samples_max, iFp)
GO(gst_audio_encoder_get_frame_samples_min, iFp)
GO(gst_audio_encoder_get_hard_min, iFp)
GO(gst_audio_encoder_get_hard_resync, iFp)
GO(gst_audio_encoder_get_latency, vFppp)
GO(gst_audio_encoder_get_lookahead, iFp)
GO(gst_audio_encoder_get_mark_granule, iFp)
GO(gst_audio_encoder_get_perfect_timestamp, iFp)
GO(gst_audio_encoder_get_tolerance, LFp)
GO(gst_audio_encoder_get_type, LFv)
GO(gst_audio_encoder_merge_tags, vFppu)
GO(gst_audio_encoder_negotiate, iFp)
GO(gst_audio_encoder_proxy_getcaps, pFppp)
GO(gst_audio_encoder_set_allocation_caps, vFpp)
GO(gst_audio_encoder_set_drainable, vFpi)
GO(gst_audio_encoder_set_frame_max, vFpi)
GO(gst_audio_encoder_set_frame_samples_max, vFpi)
GO(gst_audio_encoder_set_frame_samples_min, vFpi)
GO(gst_audio_encoder_set_hard_min, vFpi)
GO(gst_audio_encoder_set_hard_resync, vFpi)
GO(gst_audio_encoder_set_headers, vFpp)
GO(gst_audio_encoder_set_latency, vFpLL)
GO(gst_audio_encoder_set_lookahead, vFpi)
GO(gst_audio_encoder_set_mark_granule, vFpi)
GO(gst_audio_encoder_set_output_format, iFpp)
GO(gst_audio_encoder_set_perfect_timestamp, vFpi)
GO(gst_audio_encoder_set_tolerance, vFpL)
GO(gst_audio_filter_class_add_pad_templates, vFpp)
GO(gst_audio_filter_get_type, LFv)
GO(gst_audio_flags_get_type, LFv)
GO(gst_audio_format_build_integer, uFiiii)
GO(gst_audio_format_fill_silence, vFppL)
GO(gst_audio_format_flags_get_type, LFv)
GO(gst_audio_format_from_string, uFp)
GO(gst_audio_format_get_info, pFu)
GO(gst_audio_format_get_type, LFv)
GO(gst_audio_format_info_fill_silence, vFppL)
GO(gst_audio_format_info_get_type, LFv)
GO(gst_audio_formats_raw, pFp)
GO(gst_audio_format_to_string, pFu)
GO(gst_audio_get_channel_reorder_map, iFippp)
GO(gst_audio_iec61937_frame_size, uFp)
GO(gst_audio_iec61937_payload, iFpupupi)
GO(gst_audio_info_convert, iFpulup)
GO(gst_audio_info_copy, pFp)
GO(gst_audio_info_free, vFp)
GO(gst_audio_info_from_caps, iFpp)
GO(gst_audio_info_get_type, LFv)
GO(gst_audio_info_init, vFp)
GO(gst_audio_info_is_equal, iFpp)
GO(gst_audio_info_new, pFv)
GO(gst_audio_info_set_format, vFpuiip)
GO(gst_audio_info_to_caps, pFp)
GO(gst_audio_layout_get_type, LFv)
GO(gst_audio_make_raw_caps, pFpuu)
GO(gst_audio_meta_api_get_type, LFv)
GO(gst_audio_meta_get_info, pFv)
GO(gst_audio_noise_shaping_method_get_type, LFv)
GO(gst_audio_pack_flags_get_type, LFv)
GO(gst_audio_quantize_flags_get_type, LFv)
GO(gst_audio_quantize_free, vFp)
GO(gst_audio_quantize_new, pFuuuuuu)
GO(gst_audio_quantize_reset, vFp)
GO(gst_audio_quantize_samples, vFpppu)
GO(gst_audio_reorder_channels, iFpLuipp)
GO(gst_audio_resampler_filter_interpolation_get_type, LFv)
GO(gst_audio_resampler_filter_mode_get_type, LFv)
GO(gst_audio_resampler_flags_get_type, LFv)
GO(gst_audio_resampler_free, vFp)
GO(gst_audio_resampler_get_in_frames, LFpL)
GO(gst_audio_resampler_get_max_latency, LFp)
GO(gst_audio_resampler_get_out_frames, LFpL)
GO(gst_audio_resampler_method_get_type, LFv)
GO(gst_audio_resampler_new, pFuuuiiip)
GO(gst_audio_resampler_options_set_quality, vFuuiip)
GO(gst_audio_resampler_resample, vFppLpL)
GO(gst_audio_resampler_reset, vFp)
GO(gst_audio_resampler_update, iFpiip)
//GOM(gst_audio_ring_buffer_acquire, iFEpp)
//GOM(gst_audio_ring_buffer_activate, iFEpi)
//GOM(gst_audio_ring_buffer_advance, vFEpu)
//GOM(gst_audio_ring_buffer_clear, vFEpi)
//GOM(gst_audio_ring_buffer_clear_all, vFEp)
//GOM(gst_audio_ring_buffer_close_device, iFEp)
//GOM(gst_audio_ring_buffer_commit, uFEpppiip)
//GOM(gst_audio_ring_buffer_convert, iFEpulup)
GO(gst_audio_ring_buffer_debug_spec_buff, vFp)
GO(gst_audio_ring_buffer_debug_spec_caps, vFp)
//GOM(gst_audio_ring_buffer_delay, uFEp)
//GOM(gst_audio_ring_buffer_device_is_open, iFEp)
GO(gst_audio_ring_buffer_format_type_get_type, LFv)
GO(gst_audio_ring_buffer_get_type, LFv)
//GOM(gst_audio_ring_buffer_is_acquired, iFEp)
//GOM(gst_audio_ring_buffer_is_active, iFEp)
//GOM(gst_audio_ring_buffer_is_flushing, iFEp)
//GOM(gst_audio_ring_buffer_may_start, vFEpi)
//GOM(gst_audio_ring_buffer_open_device, iFEp)
GO(gst_audio_ring_buffer_parse_caps, iFpp)
//GOM(gst_audio_ring_buffer_pause, iFEp)
//GOM(gst_audio_ring_buffer_prepare_read, iFEpppp)
//GOM(gst_audio_ring_buffer_read, uFEpLpup)
//GOM(gst_audio_ring_buffer_release, iFEp)
//GOM(gst_audio_ring_buffer_samples_done, LFEp)
//GOM(gst_audio_ring_buffer_set_callback, vFEppp)
//GOM(gst_audio_ring_buffer_set_callback_full, vFEpppp)
//GOM(gst_audio_ring_buffer_set_channel_positions, vFEpp)
//GOM(gst_audio_ring_buffer_set_flushing, vFEpi)
//GOM(gst_audio_ring_buffer_set_sample, vFEpL)
//GOM(gst_audio_ring_buffer_set_timestamp, vFEpiL)
//GOM(gst_audio_ring_buffer_start, iFEp)
GO(gst_audio_ring_buffer_state_get_type, LFv)
//GOM(gst_audio_ring_buffer_stop, iFEp)
GO(gst_audio_sink_get_type, LFv)
GO(gst_audio_src_get_type, LFv)
GO(gst_audio_stream_align_copy, pFp)
GO(gst_audio_stream_align_free, vFp)
GO(gst_audio_stream_align_get_alignment_threshold, LFp)
GO(gst_audio_stream_align_get_discont_wait, LFp)
GO(gst_audio_stream_align_get_rate, iFp)
GO(gst_audio_stream_align_get_samples_since_discont, LFp)
GO(gst_audio_stream_align_get_timestamp_at_discont, LFp)
GO(gst_audio_stream_align_get_type, LFv)
GO(gst_audio_stream_align_mark_discont, vFp)
GO(gst_audio_stream_align_new, pFiLL)
GO(gst_audio_stream_align_process, iFpiLuppp)
GO(gst_audio_stream_align_set_alignment_threshold, vFpL)
GO(gst_audio_stream_align_set_discont_wait, vFpL)
GO(gst_audio_stream_align_set_rate, vFpi)
GO(gst_buffer_add_audio_clipping_meta, pFpuLL)
GO(gst_buffer_add_audio_downmix_meta, pFppipip)
GO(gst_buffer_add_audio_meta, pFppLp)
GO(gst_buffer_get_audio_downmix_meta_for_channels, pFppi)
GO(gst_stream_volume_convert_volume, dFuud)
GO(gst_stream_volume_get_mute, iFp)
GO(gst_stream_volume_get_type, LFv)
GO(gst_stream_volume_get_volume, dFpu)
GO(gst_stream_volume_set_mute, vFpi)
GO(gst_stream_volume_set_volume, vFpud)
