#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_67, pFpp)
GO(ubrk_close_67, vFp)
GO(ubrk_countAvailable_67, iFv)
GO(ubrk_current_67, iFp)
GO(ubrk_first_67, iFp)
GO(ubrk_following_67, iFpi)
GO(ubrk_getAvailable_67, pFi)
GO(ubrk_getBinaryRules_67, iFppip)
GO(ubrk_getLocaleByType_67, pFpip)
GO(ubrk_getRuleStatus_67, iFp)
GO(ubrk_getRuleStatusVec_67, iFppip)
GO(ubrk_isBoundary_67, cFpi)
GO(ubrk_last_67, iFp)
GO(ubrk_next_67, iFp)
GO(ubrk_open_67, pFppip)
GO(ubrk_openBinaryRules_67, pFpipip)
GO(ubrk_openRules_67, pFpipipp)
GO(ubrk_preceding_67, iFpi)
GO(ubrk_previous_67, iFp)
GO(ubrk_refreshUText_67, vFppp)
GO(ubrk_safeClone_67, pFpppp)
GO(ubrk_setText_67, vFppip)
GO(ubrk_setUText_67, vFppp)
GO(ubrk_swap_67, iFppipp)

GO(ucnv_getDefaultName_67, pFv)
GO(ucnv_open_67, pFpp)
GO(ucnv_setDefaultName_67, vFp)

GO(ucurr_forLocale_67, iFppip)
GO(ucurr_getName_67, pFppippp)

GO(uloc_canonicalize_67, iFppip)
GO(uloc_countAvailable_67, iFv)
GO(uloc_getAvailable_67, pFi)
GO(uloc_getBaseName_67, iFppip)
GO(uloc_getCharacterOrientation_67, iFpp)
GO(uloc_getCountry_67, iFppip)
GO(uloc_getDefault_67, pFv)
GO(uloc_getDisplayCountry_67, iFpppip)
GO(uloc_getDisplayLanguage_67, iFpppip)
GO(uloc_getDisplayName_67, iFpppip)
GO(uloc_getISO3Country_67, pFp)
GO(uloc_getISO3Language_67, pFp)
GO(uloc_getKeywordValue_67, iFpppip)
GO(uloc_getLanguage_67, iFppip)
GO(uloc_getLCID_67, uFp)
GO(uloc_getName_67, iFppip)
GO(uloc_getParent_67, iFppip)
GO(uloc_setKeywordValue_67, iFpppip)

GO(ures_close_67, vFp)
GO(ures_getByKey_67, pFpppp)
GO(ures_getSize_67, iFp)
GO(ures_getStringByIndex_67, pFpipp)
GO(ures_open_67, pFppp)

GO(uldn_close_67, vFp)
GO(uldn_keyValueDisplayName_67, iFppppip)
GO(uldn_open_67, pFpip)

GO(uenum_close_67, vFp)
GO(uenum_count_67, iFpp)
GO(uenum_next_67, pFppp)

GO(uidna_close_67, vFp)
GO(uidna_nameToASCII_67, iFppipipp)
GO(uidna_nameToUnicode_67, iFppipipp)
GO(uidna_openUTS46_67, pFpp)

GO(unorm2_getNFCInstance_67, pFp)
GO(unorm2_getNFDInstance_67, pFp)
GO(unorm2_getNFKCInstance_67, pFp)
GO(unorm2_getNFKDInstance_67, pFp)
GO(unorm2_isNormalized_67, CFppip)
GO(unorm2_normalize_67, iFppipip)

GO(u_charsToUChars_67, vFppi)
GO(u_getVersion_67, vFp)
GO(u_strcmp_67, iFpp)
GO(u_strcpy_67, pFpp)
GO(u_strlen_67, iFp)
GO(u_strncpy_67, pFppi)
GO(u_strToLower_67, iFpipipp)
GO(u_strToUpper_67, iFpipipp)
GO(u_tolower_67, uFu)
GO(u_toupper_67, uFu)
GO(u_uastrncpy_67, pFppi)
