#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif
//GO(JNI_OnLoad, vFv)
GO(SDL_abs, iFi)
GO(SDL_acos, dFd)
GO(SDL_acosf, fFf)
GO(SDL_AcquireCameraFrame, pFpp)
GO(SDL_AcquireGPUCommandBuffer, pFp)
GO(SDL_AcquireGPUSwapchainTexture, iFppppp)
GO(SDL_AddAtomicInt, iFpi)
GO(SDL_AddAtomicU32, uFpi)
GOM(SDL_AddEventWatch, iFEpp)
GO(SDL_AddGamepadMapping, iFp)
GO(SDL_AddGamepadMappingsFromFile, iFp)
GO(SDL_AddGamepadMappingsFromIO, iFpi)
GOM(SDL_AddHintCallback, iFEppp)
GO(SDL_AddSurfaceAlternateImage, iFpp)
GOM(SDL_AddTimer, uFEupp)
GOM(SDL_AddTimerNS, uFEUpp)
GO(SDL_AddVulkanRenderSemaphores, iFpuII)
GO(SDL_aligned_alloc, pFLL)
GO(SDL_aligned_free, vFp)
GO(SDL_asin, dFd)
GO(SDL_asinf, fFf)
//GOM(SDL_asprintf, iFEppV)
GO(SDL_AsyncIOFromFile, pFpp)
GO(SDL_atan, dFd)
GO(SDL_atan2, dFdd)
GO(SDL_atan2f, fFff)
GO(SDL_atanf, fFf)
GO(SDL_atof, dFp)
GO(SDL_atoi, iFp)
//GOM(SDL_AttachVirtualJoystick, uFEp)
GO(SDL_AudioDevicePaused, iFu)
GO(SDL_AudioStreamDevicePaused, iFp)
GO(SDL_BeginGPUComputePass, pFppupu)
GO(SDL_BeginGPUCopyPass, pFp)
GO(SDL_BeginGPURenderPass, pFppup)
GO(SDL_BindAudioStream, iFup)
GO(SDL_BindAudioStreams, iFupi)
GO(SDL_BindGPUComputePipeline, vFpp)
GO(SDL_BindGPUComputeSamplers, vFpupu)
GO(SDL_BindGPUComputeStorageBuffers, vFpupu)
GO(SDL_BindGPUComputeStorageTextures, vFpupu)
GO(SDL_BindGPUFragmentSamplers, vFpupu)
GO(SDL_BindGPUFragmentStorageBuffers, vFpupu)
GO(SDL_BindGPUFragmentStorageTextures, vFpupu)
GO(SDL_BindGPUGraphicsPipeline, vFpp)
GO(SDL_BindGPUIndexBuffer, vFppu)
GO(SDL_BindGPUVertexBuffers, vFpupu)
GO(SDL_BindGPUVertexSamplers, vFpupu)
GO(SDL_BindGPUVertexStorageBuffers, vFpupu)
GO(SDL_BindGPUVertexStorageTextures, vFpupu)
GO(SDL_BlitGPUTexture, vFpp)
GO(SDL_BlitSurface, iFpppp)
GO(SDL_BlitSurface9Grid, iFppiiiifipp)
GO(SDL_BlitSurfaceScaled, iFppppi)
GO(SDL_BlitSurfaceTiled, iFpppp)
GO(SDL_BlitSurfaceTiledWithScale, iFppfipp)
GO(SDL_BlitSurfaceUnchecked, iFpppp)
GO(SDL_BlitSurfaceUncheckedScaled, iFppppi)
GO(SDL_BroadcastCondition, vFp)
//GOM(SDL_bsearch, pFEppLLp)
//GOM(SDL_bsearch_r, pFEppLLpp)
GO(SDL_CalculateGPUTextureFormatSize, uFuuuu)
GO(SDL_calloc, pFLL)
GO(SDL_CancelGPUCommandBuffer, iFp)
GO(SDL_CaptureMouse, iFi)
GO(SDL_ceil, dFd)
GO(SDL_ceilf, fFf)
GO(SDL_ClaimWindowForGPUDevice, iFpp)
GO(SDL_CleanupTLS, vFv)
GO(SDL_ClearAudioStream, iFp)
GO(SDL_ClearClipboardData, iFv)
GO(SDL_ClearComposition, iFp)
GO(SDL_ClearError, iFv)
GO(SDL_ClearProperty, iFup)
GO(SDL_ClearSurface, iFpffff)
GO(SDL_ClickTrayEntry, vFp)
GO(SDL_CloseAsyncIO, iFpipp)
GO(SDL_CloseAudioDevice, vFu)
GO(SDL_CloseCamera, vFp)
GO(SDL_CloseGamepad, vFp)
GO(SDL_CloseHaptic, vFp)
GO(SDL_CloseIO, iFp)
GO(SDL_CloseJoystick, vFp)
GO(SDL_CloseSensor, vFp)
GO(SDL_CloseStorage, iFp)
GO(SDL_CompareAndSwapAtomicInt, iFpii)
GO(SDL_CompareAndSwapAtomicPointer, iFppp)
GO(SDL_CompareAndSwapAtomicU32, iFpuu)
GO(SDL_ComposeCustomBlendMode, uFuuuuuu)
GO(SDL_ConvertAudioSamples, iFppippp)
GO(SDL_ConvertEventToRenderCoordinates, iFpp)
GO(SDL_ConvertPixels, iFiiupiupi)
GO(SDL_ConvertPixelsAndColorspace, iFiiuuupiuuupi)
GO(SDL_ConvertSurface, pFpu)
GO(SDL_ConvertSurfaceAndColorspace, pFpupuu)
GO(SDL_CopyFile, iFpp)
GO(SDL_CopyGPUBufferToBuffer, vFpppui)
GO(SDL_CopyGPUTextureToTexture, vFpppuuui)
GO(SDL_CopyProperties, iFuu)
GO(SDL_copysign, dFdd)
GO(SDL_copysignf, fFff)
GO(SDL_CopyStorageFile, iFppp)
GO(SDL_cos, dFd)
GO(SDL_cosf, fFf)
GO(SDL_crc16, WFWpL)
GO(SDL_crc32, uFupL)
GO(SDL_CreateAnimatedCursor, pFpiii)
GO(SDL_CreateAsyncIOQueue, pFv)
GO(SDL_CreateAudioStream, pFpp)
GO(SDL_CreateColorCursor, pFpii)
GO(SDL_CreateCondition, pFv)
GO(SDL_CreateCursor, pFppiiii)
GO(SDL_CreateDirectory, iFp)
GO(SDL_CreateEnvironment, pFi)
GO(SDL_CreateGPUBuffer, pFpp)
GO(SDL_CreateGPUComputePipeline, pFpp)
GO(SDL_CreateGPUDevice, pFuip)
GO(SDL_CreateGPUDeviceWithProperties, pFu)
GO(SDL_CreateGPUGraphicsPipeline, pFpp)
GO(SDL_CreateGPURenderer, pFpp)
GO(SDL_CreateGPURenderState, pFpp)
GO(SDL_CreateGPUSampler, pFpp)
GO(SDL_CreateGPUShader, pFpp)
GO(SDL_CreateGPUTexture, pFpp)
GO(SDL_CreateGPUTransferBuffer, pFpp)
GO(SDL_CreateHapticEffect, iFpp)
GO(SDL_CreateMutex, pFv)
GO(SDL_CreatePalette, pFi)
GO(SDL_CreatePopupWindow, pFpiiiiU)
GO(SDL_CreateProcess, pFpi)
GO(SDL_CreateProcessWithProperties, pFu)
GO(SDL_CreateProperties, uFv)
GO(SDL_CreateRenderer, pFpp)
GO(SDL_CreateRendererWithProperties, pFu)
GO(SDL_CreateRWLock, pFv)
GO(SDL_CreateSemaphore, pFu)
GO(SDL_CreateSoftwareRenderer, pFp)
GO(SDL_CreateStorageDirectory, iFpp)
GO(SDL_CreateSurface, pFiiu)
GO(SDL_CreateSurfaceFrom, pFiiupi)
GO(SDL_CreateSurfacePalette, pFp)
GO(SDL_CreateSystemCursor, pFu)
GO(SDL_CreateTexture, pFpuuii)
GO(SDL_CreateTextureFromSurface, pFpp)
GO(SDL_CreateTextureWithProperties, pFpu)
//GOM(SDL_CreateThreadRuntime, pFEppppp)
//GOM(SDL_CreateThreadWithPropertiesRuntime, pFEupp)
GO(SDL_CreateTray, pFpp)
GO(SDL_CreateTrayMenu, pFp)
GO(SDL_CreateTraySubmenu, pFp)
GO(SDL_CreateWindow, pFpiiU)
GO(SDL_CreateWindowAndRenderer, iFpiiUpp)
GO(SDL_CreateWindowWithProperties, pFu)
GO(SDL_CursorVisible, iFv)
GO(SDL_DateTimeToTime, iFpp)
GO(SDL_Delay, vFu)
GO(SDL_DelayNS, vFU)
GO(SDL_DelayPrecise, vFU)
GO(SDL_DestroyAsyncIOQueue, vFp)
GO(SDL_DestroyAudioStream, vFp)
GO(SDL_DestroyCondition, vFp)
GO(SDL_DestroyCursor, vFp)
GO(SDL_DestroyEnvironment, vFp)
GO(SDL_DestroyGPUDevice, vFp)
GO(SDL_DestroyGPURenderState, vFp)
GO(SDL_DestroyHapticEffect, vFpi)
GO(SDL_DestroyMutex, vFp)
GO(SDL_DestroyPalette, vFp)
GO(SDL_DestroyProcess, vFp)
GO(SDL_DestroyProperties, vFu)
GO(SDL_DestroyRenderer, vFp)
GO(SDL_DestroyRWLock, vFp)
GO(SDL_DestroySemaphore, vFp)
GO(SDL_DestroySurface, vFp)
GO(SDL_DestroyTexture, vFp)
GO(SDL_DestroyTray, vFp)
GO(SDL_DestroyWindow, vFp)
GO(SDL_DestroyWindowSurface, iFp)
GO(SDL_DetachThread, vFp)
GO(SDL_DetachVirtualJoystick, iFu)
GO(SDL_DisableScreenSaver, iFv)
GO(SDL_DispatchGPUCompute, vFpuuu)
GO(SDL_DispatchGPUComputeIndirect, vFppu)
GO(SDL_DownloadFromGPUBuffer, vFppp)
GO(SDL_DownloadFromGPUTexture, vFppp)
GO(SDL_DrawGPUIndexedPrimitives, vFpuuuiu)
GO(SDL_DrawGPUIndexedPrimitivesIndirect, vFppuu)
GO(SDL_DrawGPUPrimitives, vFpuuuu)
GO(SDL_DrawGPUPrimitivesIndirect, vFppuu)
GO(SDL_DuplicateSurface, pFp)
//GO(SDL_DYNAPI_entry, vFv)
GO(SDL_EGL_GetCurrentConfig, pFv)
GO(SDL_EGL_GetCurrentDisplay, pFv)
//GOM(SDL_EGL_GetProcAddress, pFEp)
GO(SDL_EGL_GetWindowSurface, pFp)
//GOM(SDL_EGL_SetAttributeCallbacks, vFEpppp)
GO(SDL_EnableScreenSaver, iFv)
GO(SDL_EndGPUComputePass, vFp)
GO(SDL_EndGPUCopyPass, vFp)
GO(SDL_EndGPURenderPass, vFp)
//GO(SDL_EnterAppMainCallbacks, vFv)
//GOM(SDL_EnumerateDirectory, iFEppp)
//GOM(SDL_EnumerateProperties, iFEupp)
//GOM(SDL_EnumerateStorageDirectory, iFEpppp)
GO(SDL_EventEnabled, iFu)
GO(SDL_exp, dFd)
GO(SDL_expf, fFf)
GO(SDL_fabs, dFd)
GO(SDL_fabsf, fFf)
GO(SDL_FillSurfaceRect, iFppu)
GO(SDL_FillSurfaceRects, iFppiu)
GOM(SDL_FilterEvents, vFEpp)
GO(SDL_FlashWindow, iFpu)
GO(SDL_FlipSurface, iFpu)
GO(SDL_floor, dFd)
GO(SDL_floorf, fFf)
GO(SDL_FlushAudioStream, iFp)
GO(SDL_FlushEvent, vFu)
GO(SDL_FlushEvents, vFuu)
GO(SDL_FlushIO, iFp)
GO(SDL_FlushRenderer, iFp)
GO(SDL_fmod, dFdd)
GO(SDL_fmodf, fFff)
GO(SDL_free, vFp)
GO(SDL_GamepadConnected, iFp)
GO(SDL_GamepadEventsEnabled, iFv)
GO(SDL_GamepadHasAxis, iFpi)
GO(SDL_GamepadHasButton, iFpi)
GO(SDL_GamepadHasSensor, iFpi)
GO(SDL_GamepadSensorEnabled, iFpi)
//GO(SDL_GDKResumeGPU, vFv)
//GO(SDL_GDKSuspendComplete, vFv)
//GO(SDL_GDKSuspendGPU, vFv)
GO(SDL_GenerateMipmapsForGPUTexture, vFpp)
//GO(SDL_GetAndroidActivity, vFv)
//GO(SDL_GetAndroidCachePath, vFv)
//GO(SDL_GetAndroidExternalStoragePath, vFv)
//GO(SDL_GetAndroidExternalStorageState, vFv)
//GO(SDL_GetAndroidInternalStoragePath, vFv)
//GO(SDL_GetAndroidJNIEnv, vFv)
//GO(SDL_GetAndroidSDKVersion, vFv)
GO(SDL_GetAppMetadataProperty, pFp)
//GOM(SDL_GetAssertionHandler, pFEp)
GO(SDL_GetAssertionReport, pFv)
GO(SDL_GetAsyncIOResult, iFpp)
GO(SDL_GetAsyncIOSize, IFp)
GO(SDL_GetAtomicInt, iFp)
GO(SDL_GetAtomicPointer, pFp)
GO(SDL_GetAtomicU32, uFp)
GO(SDL_GetAudioDeviceChannelMap, pFup)
GO(SDL_GetAudioDeviceFormat, iFupp)
GO(SDL_GetAudioDeviceGain, fFu)
GO(SDL_GetAudioDeviceName, pFu)
GO(SDL_GetAudioDriver, pFi)
GO(SDL_GetAudioFormatName, pFu)
GO(SDL_GetAudioPlaybackDevices, pFp)
GO(SDL_GetAudioRecordingDevices, pFp)
GO(SDL_GetAudioStreamAvailable, iFp)
GO(SDL_GetAudioStreamData, iFppi)
GO(SDL_GetAudioStreamDevice, uFp)
GO(SDL_GetAudioStreamFormat, iFppp)
GO(SDL_GetAudioStreamFrequencyRatio, fFp)
GO(SDL_GetAudioStreamGain, fFp)
GO(SDL_GetAudioStreamInputChannelMap, pFpp)
GO(SDL_GetAudioStreamOutputChannelMap, pFpp)
GO(SDL_GetAudioStreamProperties, uFp)
GO(SDL_GetAudioStreamQueued, iFp)
GO(SDL_GetBasePath, pFv)
GO(SDL_GetBooleanProperty, iFupi)
GO(SDL_GetCameraDriver, pFi)
GO(SDL_GetCameraFormat, iFpp)
GO(SDL_GetCameraID, uFp)
GO(SDL_GetCameraName, pFu)
GO(SDL_GetCameraPermissionState, iFp)
GO(SDL_GetCameraPosition, uFu)
GO(SDL_GetCameraProperties, uFp)
GO(SDL_GetCameras, pFp)
GO(SDL_GetCameraSupportedFormats, pFup)
GO(SDL_GetClipboardData, pFpp)
GO(SDL_GetClipboardMimeTypes, pFp)
GO(SDL_GetClipboardText, pFv)
GO(SDL_GetClosestFullscreenDisplayMode, iFuiifip)
GO(SDL_GetCPUCacheLineSize, iFv)
GO(SDL_GetCurrentAudioDriver, pFv)
GO(SDL_GetCurrentCameraDriver, pFv)
GO(SDL_GetCurrentDirectory, pFv)
GO(SDL_GetCurrentDisplayMode, pFu)
GO(SDL_GetCurrentDisplayOrientation, uFu)
GO(SDL_GetCurrentRenderOutputSize, iFppp)
GO(SDL_GetCurrentThreadID, UFv)
GO(SDL_GetCurrentTime, iFp)
GO(SDL_GetCurrentVideoDriver, pFv)
GO(SDL_GetCursor, pFv)
GO(SDL_GetDateTimeLocalePreferences, iFpp)
GO(SDL_GetDayOfWeek, iFiii)
GO(SDL_GetDayOfYear, iFiii)
GO(SDL_GetDaysInMonth, iFii)
//GOM(SDL_GetDefaultAssertionHandler, pFEv)
GO(SDL_GetDefaultCursor, pFv)
//GOM(SDL_GetDefaultLogOutputFunction, pFEv)
GO(SDL_GetDefaultTextureScaleMode, iFpp)
GO(SDL_GetDesktopDisplayMode, pFu)
//GO(SDL_GetDirect3D9AdapterIndex, vFv)
GO(SDL_GetDisplayBounds, iFup)
GO(SDL_GetDisplayContentScale, fFu)
GO(SDL_GetDisplayForPoint, uFp)
GO(SDL_GetDisplayForRect, uFp)
GO(SDL_GetDisplayForWindow, uFp)
GO(SDL_GetDisplayName, pFu)
GO(SDL_GetDisplayProperties, uFu)
GO(SDL_GetDisplays, pFp)
GO(SDL_GetDisplayUsableBounds, iFup)
//GO(SDL_GetDXGIOutputInfo, vFv)
GO(SDL_getenv, pFp)
GO(SDL_GetEnvironment, pFv)
GO(SDL_GetEnvironmentVariable, pFpp)
GO(SDL_GetEnvironmentVariables, pFp)
GO(SDL_getenv_unsafe, pFp)
GO(SDL_GetError, pFv)
GO(SDL_GetEventDescription, iFppi)
GOM(SDL_GetEventFilter, iFEpp)
GO(SDL_GetFloatProperty, fFupf)
GO(SDL_GetFullscreenDisplayModes, pFup)
GO(SDL_GetGamepadAppleSFSymbolsNameForAxis, pFpi)
GO(SDL_GetGamepadAppleSFSymbolsNameForButton, pFpi)
GO(SDL_GetGamepadAxis, wFpi)
GO(SDL_GetGamepadAxisFromString, iFp)
GO(SDL_GetGamepadBindings, pFpp)
GO(SDL_GetGamepadButton, iFpi)
GO(SDL_GetGamepadButtonFromString, iFp)
GO(SDL_GetGamepadButtonLabel, uFpi)
GO(SDL_GetGamepadButtonLabelForType, uFui)
GO(SDL_GetGamepadConnectionState, iFp)
GO(SDL_GetGamepadFirmwareVersion, WFp)
GO(SDL_GetGamepadFromID, pFu)
GO(SDL_GetGamepadFromPlayerIndex, pFi)
GO(SDL_GetGamepadGUIDForID, HFu)
GO(SDL_GetGamepadID, uFp)
GO(SDL_GetGamepadJoystick, pFp)
GO(SDL_GetGamepadMapping, pFp)
//GO(SDL_GetGamepadMappingForGUID, vFv)
GO(SDL_GetGamepadMappingForID, pFu)
GO(SDL_GetGamepadMappings, pFp)
GO(SDL_GetGamepadName, pFp)
GO(SDL_GetGamepadNameForID, pFu)
GO(SDL_GetGamepadPath, pFp)
GO(SDL_GetGamepadPathForID, pFu)
GO(SDL_GetGamepadPlayerIndex, iFp)
GO(SDL_GetGamepadPlayerIndexForID, iFu)
GO(SDL_GetGamepadPowerInfo, iFpp)
GO(SDL_GetGamepadProduct, WFp)
GO(SDL_GetGamepadProductForID, WFu)
GO(SDL_GetGamepadProductVersion, WFp)
GO(SDL_GetGamepadProductVersionForID, WFu)
GO(SDL_GetGamepadProperties, uFp)
GO(SDL_GetGamepads, pFp)
GO(SDL_GetGamepadSensorData, iFpipi)
GO(SDL_GetGamepadSensorDataRate, fFpi)
GO(SDL_GetGamepadSerial, pFp)
GO(SDL_GetGamepadSteamHandle, UFp)
GO(SDL_GetGamepadStringForAxis, pFi)
GO(SDL_GetGamepadStringForButton, pFi)
GO(SDL_GetGamepadStringForType, pFu)
GO(SDL_GetGamepadTouchpadFinger, iFpiipppp)
GO(SDL_GetGamepadType, uFp)
GO(SDL_GetGamepadTypeForID, uFu)
GO(SDL_GetGamepadTypeFromString, uFp)
GO(SDL_GetGamepadVendor, WFp)
GO(SDL_GetGamepadVendorForID, WFu)
//GO(SDL_GetGDKDefaultUser, vFv)
//GO(SDL_GetGDKTaskQueue, vFv)
GO(SDL_GetGlobalMouseState, uFpp)
GO(SDL_GetGlobalProperties, uFv)
GO(SDL_GetGPUDeviceDriver, pFp)
GO(SDL_GetGPUDeviceProperties, uFp)
GO(SDL_GetGPUDriver, pFi)
GO(SDL_GetGPURendererDevice, pFp)
GO(SDL_GetGPUShaderFormats, uFp)
GO(SDL_GetGPUSwapchainTextureFormat, uFpp)
GO(SDL_GetGPUTextureFormatFromPixelFormat, uFu)
GO(SDL_GetGrabbedWindow, pFv)
GO(SDL_GetHapticEffectStatus, iFpi)
GO(SDL_GetHapticFeatures, uFp)
GO(SDL_GetHapticFromID, pFu)
GO(SDL_GetHapticID, uFp)
GO(SDL_GetHapticName, pFp)
GO(SDL_GetHapticNameForID, pFu)
GO(SDL_GetHaptics, pFp)
GO(SDL_GetHint, pFp)
GO(SDL_GetHintBoolean, iFpi)
GO(SDL_GetIOProperties, uFp)
GO(SDL_GetIOSize, IFp)
GO(SDL_GetIOStatus, uFp)
GO(SDL_GetJoystickAxis, wFpi)
GO(SDL_GetJoystickAxisInitialState, iFpip)
GO(SDL_GetJoystickBall, iFpipp)
GO(SDL_GetJoystickButton, iFpi)
GO(SDL_GetJoystickConnectionState, iFp)
GO(SDL_GetJoystickFirmwareVersion, WFp)
GO(SDL_GetJoystickFromID, pFu)
GO(SDL_GetJoystickFromPlayerIndex, pFi)
GO(SDL_GetJoystickGUID, HFp)
GO(SDL_GetJoystickGUIDForID, HFu)
//GO(SDL_GetJoystickGUIDInfo, vFv)
GO(SDL_GetJoystickHat, CFpi)
GO(SDL_GetJoystickID, uFp)
GO(SDL_GetJoystickName, pFp)
GO(SDL_GetJoystickNameForID, pFu)
GO(SDL_GetJoystickPath, pFp)
GO(SDL_GetJoystickPathForID, pFu)
GO(SDL_GetJoystickPlayerIndex, iFp)
GO(SDL_GetJoystickPlayerIndexForID, iFu)
GO(SDL_GetJoystickPowerInfo, iFpp)
GO(SDL_GetJoystickProduct, WFp)
GO(SDL_GetJoystickProductForID, WFu)
GO(SDL_GetJoystickProductVersion, WFp)
GO(SDL_GetJoystickProductVersionForID, WFu)
GO(SDL_GetJoystickProperties, uFp)
GO(SDL_GetJoysticks, pFp)
GO(SDL_GetJoystickSerial, pFp)
GO(SDL_GetJoystickType, uFp)
GO(SDL_GetJoystickTypeForID, uFu)
GO(SDL_GetJoystickVendor, WFp)
GO(SDL_GetJoystickVendorForID, WFu)
GO(SDL_GetKeyboardFocus, pFv)
GO(SDL_GetKeyboardNameForID, pFu)
GO(SDL_GetKeyboards, pFp)
GO(SDL_GetKeyboardState, pFp)
GO(SDL_GetKeyFromName, uFp)
GO(SDL_GetKeyFromScancode, uFuWi)
GO(SDL_GetKeyName, pFu)
GOM(SDL_GetLogOutputFunction, vFEpp)
GO(SDL_GetLogPriority, uFi)
GO(SDL_GetMasksForPixelFormat, iFuppppp)
GO(SDL_GetMaxHapticEffects, iFp)
GO(SDL_GetMaxHapticEffectsPlaying, iFp)
//GOM(SDL_GetMemoryFunctions, vFEpppp)
GO(SDL_GetMice, pFp)
GO(SDL_GetModState, WFv)
GO(SDL_GetMouseFocus, pFv)
GO(SDL_GetMouseNameForID, pFu)
GO(SDL_GetMouseState, uFpp)
GO(SDL_GetNaturalDisplayOrientation, uFu)
GO(SDL_GetNumAllocations, iFv)
GO(SDL_GetNumAudioDrivers, iFv)
GO(SDL_GetNumberProperty, IFupI)
GO(SDL_GetNumCameraDrivers, iFv)
GO(SDL_GetNumGamepadTouchpadFingers, iFpi)
GO(SDL_GetNumGamepadTouchpads, iFp)
GO(SDL_GetNumGPUDrivers, iFv)
GO(SDL_GetNumHapticAxes, iFp)
GO(SDL_GetNumJoystickAxes, iFp)
GO(SDL_GetNumJoystickBalls, iFp)
GO(SDL_GetNumJoystickButtons, iFp)
GO(SDL_GetNumJoystickHats, iFp)
GO(SDL_GetNumLogicalCPUCores, iFv)
GO(SDL_GetNumRenderDrivers, iFv)
GO(SDL_GetNumVideoDrivers, iFv)
//GOM(SDL_GetOriginalMemoryFunctions, vFEpppp)
GO(SDL_GetPathInfo, iFpp)
GO(SDL_GetPenDeviceType, iFu)
GO(SDL_GetPerformanceCounter, UFv)
GO(SDL_GetPerformanceFrequency, UFv)
GO(SDL_GetPixelFormatDetails, pFu)
GO(SDL_GetPixelFormatForMasks, uFiuuuu)
GO(SDL_GetPixelFormatFromGPUTextureFormat, uFu)
GO(SDL_GetPixelFormatName, pFu)
GO(SDL_GetPlatform, pFv)
GO(SDL_GetPointerProperty, pFupp)
GO(SDL_GetPowerInfo, iFpp)
GO(SDL_GetPreferredLocales, pFp)
GO(SDL_GetPrefPath, pFpp)
GO(SDL_GetPrimaryDisplay, uFv)
GO(SDL_GetPrimarySelectionText, pFv)
GO(SDL_GetProcessInput, pFp)
GO(SDL_GetProcessOutput, pFp)
GO(SDL_GetProcessProperties, uFp)
GO(SDL_GetPropertyType, uFup)
GO(SDL_GetRealGamepadType, uFp)
GO(SDL_GetRealGamepadTypeForID, uFu)
GO(SDL_GetRectAndLineIntersection, iFppppp)
GO(SDL_GetRectAndLineIntersectionFloat, iFppppp)
GO(SDL_GetRectEnclosingPoints, iFpipp)
GO(SDL_GetRectEnclosingPointsFloat, iFpipp)
GO(SDL_GetRectIntersection, iFppp)
GO(SDL_GetRectIntersectionFloat, iFppp)
GO(SDL_GetRectUnion, iFppp)
GO(SDL_GetRectUnionFloat, iFppp)
GO(SDL_GetRelativeMouseState, uFpp)
GO(SDL_GetRenderClipRect, iFpp)
GO(SDL_GetRenderColorScale, iFpp)
GO(SDL_GetRenderDrawBlendMode, iFpp)
GO(SDL_GetRenderDrawColor, iFppppp)
GO(SDL_GetRenderDrawColorFloat, iFppppp)
GO(SDL_GetRenderDriver, pFi)
GO(SDL_GetRenderer, pFp)
GO(SDL_GetRendererFromTexture, pFp)
GO(SDL_GetRendererName, pFp)
GO(SDL_GetRendererProperties, uFp)
GO(SDL_GetRenderLogicalPresentation, iFpppp)
GO(SDL_GetRenderLogicalPresentationRect, iFpp)
GO(SDL_GetRenderMetalCommandEncoder, pFp)
GO(SDL_GetRenderMetalLayer, pFp)
GO(SDL_GetRenderOutputSize, iFppp)
GO(SDL_GetRenderSafeArea, iFpp)
GO(SDL_GetRenderScale, iFppp)
GO(SDL_GetRenderTarget, pFp)
GO(SDL_GetRenderTextureAddressMode, iFppp)
GO(SDL_GetRenderViewport, iFpp)
GO(SDL_GetRenderVSync, iFpp)
GO(SDL_GetRenderWindow, pFp)
GO(SDL_GetRevision, pFv)
GO(SDL_GetRGB, vFuppppp)
GO(SDL_GetRGBA, vFupppppp)
GO(SDL_GetSandbox, uFv)
GO(SDL_GetScancodeFromKey, uFup)
GO(SDL_GetScancodeFromName, uFp)
GO(SDL_GetScancodeName, pFu)
GO(SDL_GetSemaphoreValue, uFp)
GO(SDL_GetSensorData, iFppi)
GO(SDL_GetSensorFromID, pFu)
GO(SDL_GetSensorID, uFp)
GO(SDL_GetSensorName, pFp)
GO(SDL_GetSensorNameForID, pFu)
GO(SDL_GetSensorNonPortableType, iFp)
GO(SDL_GetSensorNonPortableTypeForID, iFu)
GO(SDL_GetSensorProperties, uFp)
GO(SDL_GetSensors, pFp)
GO(SDL_GetSensorType, iFp)
GO(SDL_GetSensorTypeForID, iFu)
GO(SDL_GetSilenceValueForFormat, iFu)
GO(SDL_GetSIMDAlignment, LFv)
GO(SDL_GetStorageFileSize, iFppp)
GO(SDL_GetStoragePathInfo, iFppp)
GO(SDL_GetStorageSpaceRemaining, UFp)
GO(SDL_GetStringProperty, pFupp)
GO(SDL_GetSurfaceAlphaMod, iFpp)
GO(SDL_GetSurfaceBlendMode, iFpp)
GO(SDL_GetSurfaceClipRect, iFpp)
GO(SDL_GetSurfaceColorKey, iFpp)
GO(SDL_GetSurfaceColorMod, iFpppp)
GO(SDL_GetSurfaceColorspace, uFp)
GO(SDL_GetSurfaceImages, pFpp)
GO(SDL_GetSurfacePalette, pFp)
GO(SDL_GetSurfaceProperties, uFp)
GO(SDL_GetSystemPageSize, iFv)
GO(SDL_GetSystemRAM, iFv)
GO(SDL_GetSystemTheme, uFv)
GO(SDL_GetTextInputArea, iFppp)
GO(SDL_GetTextureAlphaMod, iFpp)
GO(SDL_GetTextureAlphaModFloat, iFpp)
GO(SDL_GetTextureBlendMode, iFpp)
GO(SDL_GetTextureColorMod, iFpppp)
GO(SDL_GetTextureColorModFloat, iFpppp)
GO(SDL_GetTexturePalette, pFp)
GO(SDL_GetTextureProperties, uFp)
GO(SDL_GetTextureScaleMode, iFpp)
GO(SDL_GetTextureSize, iFppp)
GO(SDL_GetThreadID, UFp)
GO(SDL_GetThreadName, pFp)
GO(SDL_GetThreadState, uFp)
GO(SDL_GetTicks, UFv)
GO(SDL_GetTicksNS, UFv)
GO(SDL_GetTLS, pFp)
GO(SDL_GetTouchDeviceName, pFU)
GO(SDL_GetTouchDevices, pFp)
GO(SDL_GetTouchDeviceType, iFU)
GO(SDL_GetTouchFingers, pFUp)
GO(SDL_GetTrayEntries, pFpp)
GO(SDL_GetTrayEntryChecked, iFp)
GO(SDL_GetTrayEntryEnabled, iFp)
GO(SDL_GetTrayEntryLabel, pFp)
GO(SDL_GetTrayEntryParent, pFp)
GO(SDL_GetTrayMenu, pFp)
GO(SDL_GetTrayMenuParentEntry, pFp)
GO(SDL_GetTrayMenuParentTray, pFp)
GO(SDL_GetTraySubmenu, pFp)
GO(SDL_GetUserFolder, pFu)
GO(SDL_GetVersion, iFv)
GO(SDL_GetVideoDriver, pFi)
GO(SDL_GetWindowAspectRatio, iFppp)
GO(SDL_GetWindowBordersSize, iFppppp)
GO(SDL_GetWindowDisplayScale, fFp)
GO(SDL_GetWindowFlags, UFp)
GO(SDL_GetWindowFromEvent, pFp)
GO(SDL_GetWindowFromID, pFu)
GO(SDL_GetWindowFullscreenMode, pFp)
GO(SDL_GetWindowICCProfile, pFpp)
GO(SDL_GetWindowID, uFp)
GO(SDL_GetWindowKeyboardGrab, iFp)
GO(SDL_GetWindowMaximumSize, iFppp)
GO(SDL_GetWindowMinimumSize, iFppp)
GO(SDL_GetWindowMouseGrab, iFp)
GO(SDL_GetWindowMouseRect, pFp)
GO(SDL_GetWindowOpacity, fFp)
GO(SDL_GetWindowParent, pFp)
GO(SDL_GetWindowPixelDensity, fFp)
GO(SDL_GetWindowPixelFormat, uFp)
GO(SDL_GetWindowPosition, iFppp)
GO(SDL_GetWindowProgressState, iFp)
GO(SDL_GetWindowProgressValue, fFp)
GO(SDL_GetWindowProperties, uFp)
GO(SDL_GetWindowRelativeMouseMode, iFp)
GO(SDL_GetWindows, pFp)
GO(SDL_GetWindowSafeArea, iFpp)
GO(SDL_GetWindowSize, iFppp)
GO(SDL_GetWindowSizeInPixels, iFppp)
GO(SDL_GetWindowSurface, pFp)
GO(SDL_GetWindowSurfaceVSync, iFpp)
GO(SDL_GetWindowTitle, pFp)
GO(SDL_GL_CreateContext, pFp)
GO(SDL_GL_DestroyContext, iFp)
GO(SDL_GL_ExtensionSupported, iFp)
GO(SDL_GL_GetAttribute, iFup)
GO(SDL_GL_GetCurrentContext, pFv)
GO(SDL_GL_GetCurrentWindow, pFv)
GOM(SDL_GL_GetProcAddress, pFEp)
GO(SDL_GL_GetSwapInterval, iFp)
GO(SDL_GL_LoadLibrary, iFp)
GO(SDL_GL_MakeCurrent, iFpp)
GO(SDL_GlobDirectory, pFppup)
GO(SDL_GlobStorageDirectory, pFpppup)
GO(SDL_GL_ResetAttributes, vFv)
GO(SDL_GL_SetAttribute, iFui)
GO(SDL_GL_SetSwapInterval, iFi)
GO(SDL_GL_SwapWindow, iFp)
GO(SDL_GL_UnloadLibrary, vFv)
GO(SDL_GPUSupportsProperties, iFu)
GO(SDL_GPUSupportsShaderFormats, iFup)
GO(SDL_GPUTextureFormatTexelBlockSize, uFu)
GO(SDL_GPUTextureSupportsFormat, iFpuuu)
GO(SDL_GPUTextureSupportsSampleCount, iFpuu)
//GO(SDL_GUIDToString, vFv)
GO(SDL_HapticEffectSupported, iFpp)
GO(SDL_HapticRumbleSupported, iFp)
GO(SDL_HasAltiVec, iFv)
GO(SDL_HasARMSIMD, iFv)
GO(SDL_HasAVX, iFv)
GO(SDL_HasAVX2, iFv)
GO(SDL_HasAVX512F, iFv)
GO(SDL_HasClipboardData, iFp)
GO(SDL_HasClipboardText, iFv)
GO(SDL_HasEvent, iFu)
GO(SDL_HasEvents, iFuu)
GO(SDL_HasGamepad, iFv)
GO(SDL_HasJoystick, iFv)
GO(SDL_HasKeyboard, iFv)
GO(SDL_HasLASX, iFv)
GO(SDL_HasLSX, iFv)
GO(SDL_HasMMX, iFv)
GO(SDL_HasMouse, iFv)
GO(SDL_HasNEON, iFv)
GO(SDL_HasPrimarySelectionText, iFv)
GO(SDL_HasProperty, iFup)
GO(SDL_HasRectIntersection, iFpp)
GO(SDL_HasRectIntersectionFloat, iFpp)
GO(SDL_HasScreenKeyboardSupport, iFv)
GO(SDL_HasSSE, iFv)
GO(SDL_HasSSE2, iFv)
GO(SDL_HasSSE3, iFv)
GO(SDL_HasSSE41, iFv)
GO(SDL_HasSSE42, iFv)
GO(SDL_hid_ble_scan, vFi)
GO(SDL_hid_close, iFp)
GO(SDL_hid_device_change_count, uFv)
GO(SDL_HideCursor, iFv)
GO(SDL_hid_enumerate, pFWW)
GO(SDL_HideWindow, iFp)
GO(SDL_hid_exit, iFv)
GO(SDL_hid_free_enumeration, vFp)
GO(SDL_hid_get_device_info, pFp)
GO(SDL_hid_get_feature_report, iFppL)
GO(SDL_hid_get_indexed_string, iFpipL)
GO(SDL_hid_get_input_report, iFppL)
GO(SDL_hid_get_manufacturer_string, iFppL)
GO(SDL_hid_get_product_string, iFppL)
GO(SDL_hid_get_properties, uFp)
GO(SDL_hid_get_report_descriptor, iFppL)
GO(SDL_hid_get_serial_number_string, iFppL)
GO(SDL_hid_init, iFv)
GO(SDL_hid_open, pFWWp)
GO(SDL_hid_open_path, pFp)
GO(SDL_hid_read, iFppL)
GO(SDL_hid_read_timeout, iFppLi)
GO(SDL_hid_send_feature_report, iFppL)
GO(SDL_hid_set_nonblocking, iFpi)
GO(SDL_hid_write, iFppL)
GO(SDL_iconv, LFppppp)
GO(SDL_iconv_close, iFp)
GO(SDL_iconv_open, pFpp)
GO(SDL_iconv_string, pFpppL)
GO(SDL_Init, iFu)
GO(SDL_InitHapticRumble, iFp)
GO(SDL_InitSubSystem, iFu)
GO(SDL_InsertGPUDebugLabel, vFpp)
GO(SDL_InsertTrayEntryAt, pFpipu)
GO(SDL_IOFromConstMem, pFpL)
GO(SDL_IOFromDynamicMem, pFv)
GO(SDL_IOFromFile, pFpp)
GO(SDL_IOFromMem, pFpL)
//GOM(SDL_IOprintf, LFEppV)
//GOM(SDL_IOvprintf, LFEppA)
GO(SDL_isalnum, iFi)
GO(SDL_isalpha, iFi)
GO(SDL_IsAudioDevicePhysical, iFu)
GO(SDL_IsAudioDevicePlayback, iFu)
GO(SDL_isblank, iFi)
//GO(SDL_IsChromebook, vFv)
GO(SDL_iscntrl, iFi)
//GO(SDL_IsDeXMode, vFv)
GO(SDL_isdigit, iFi)
GO(SDL_IsGamepad, iFu)
GO(SDL_isgraph, iFi)
GO(SDL_isinf, iFd)
GO(SDL_isinff, iFf)
GO(SDL_IsJoystickHaptic, iFp)
GO(SDL_IsJoystickVirtual, iFu)
GO(SDL_islower, iFi)
GO(SDL_IsMainThread, iFv)
GO(SDL_IsMouseHaptic, iFv)
GO(SDL_isnan, iFd)
GO(SDL_isnanf, iFf)
GO(SDL_isprint, iFi)
GO(SDL_ispunct, iFi)
GO(SDL_isspace, iFi)
GO(SDL_IsTablet, iFv)
GO(SDL_IsTV, iFv)
GO(SDL_isupper, iFi)
GO(SDL_isxdigit, iFi)
GO(SDL_itoa, pFipi)
GO(SDL_JoystickConnected, iFp)
GO(SDL_JoystickEventsEnabled, iFv)
GO(SDL_KillProcess, iFpi)
GO(SDL_lltoa, pFIpi)
GO(SDL_LoadBMP, pFp)
GO(SDL_LoadBMP_IO, pFpi)
GO(SDL_LoadFile, pFpp)
GO(SDL_LoadFileAsync, iFppp)
GO(SDL_LoadFile_IO, pFppi)
//GOM(SDL_LoadFunction, pFEpp)
GO(SDL_LoadObject, pFp)
GO(SDL_LoadPNG, pFp)
GO(SDL_LoadPNG_IO, pFpi)
GO(SDL_LoadSurface, pFp)
GO(SDL_LoadSurface_IO, pFpi)
GO(SDL_LoadWAV, iFpppp)
GO(SDL_LoadWAV_IO, iFpippp)
GO(SDL_LockAudioStream, iFp)
GO(SDL_LockJoysticks, vFv)
GO(SDL_LockMutex, vFp)
GO(SDL_LockProperties, iFu)
GO(SDL_LockRWLockForReading, vFp)
GO(SDL_LockRWLockForWriting, vFp)
GO(SDL_LockSpinlock, vFp)
GO(SDL_LockSurface, iFp)
GO(SDL_LockTexture, iFpppp)
GO(SDL_LockTextureToSurface, iFppp)
GO(SDL_log, dFd)
//GOM(SDL_Log, vFEpV)
GO(SDL_log10, dFd)
GO(SDL_log10f, fFf)
//GOM(SDL_LogCritical, vFEipV)
//GOM(SDL_LogDebug, vFEipV)
//GOM(SDL_LogError, vFEipV)
GO(SDL_logf, fFf)
//GOM(SDL_LogInfo, vFEipV)
//GOM(SDL_LogMessage, vFEiupV)
//GOM(SDL_LogMessageV, vFEiupA)
//GOM(SDL_LogTrace, vFEipV)
//GOM(SDL_LogVerbose, vFEipV)
//GOM(SDL_LogWarn, vFEipV)
GO(SDL_lround, lFd)
GO(SDL_lroundf, lFf)
GO(SDL_ltoa, pFlpi)
GO(SDL_malloc, pFL)
GO(SDL_MapGPUTransferBuffer, pFppi)
GO(SDL_MapRGB, uFppCCC)
GO(SDL_MapRGBA, uFppCCCC)
GO(SDL_MapSurfaceRGB, uFpCCC)
GO(SDL_MapSurfaceRGBA, uFpCCCC)
GO(SDL_MaximizeWindow, iFp)
GO(SDL_memcmp, iFppL)
GO(SDL_memcpy, pFppL)
GO(SDL_memmove, pFppL)
GO(SDL_MemoryBarrierAcquireFunction, vFv)
GO(SDL_MemoryBarrierReleaseFunction, vFv)
GO(SDL_memset, pFpiL)
GO(SDL_memset4, pFpuL)
GO(SDL_Metal_CreateView, pFp)
GO(SDL_Metal_DestroyView, vFp)
GO(SDL_Metal_GetLayer, pFp)
GO(SDL_MinimizeWindow, iFp)
GO(SDL_MixAudio, iFppuuf)
GO(SDL_modf, dFdp)
GO(SDL_modff, fFfp)
GO(SDL_murmur3_32, uFpLu)
//GO(SDL_OnApplicationDidChangeStatusBarOrientation, vFv)
GO(SDL_OnApplicationDidEnterBackground, vFv)
GO(SDL_OnApplicationDidEnterForeground, vFv)
GO(SDL_OnApplicationDidReceiveMemoryWarning, vFv)
GO(SDL_OnApplicationWillEnterBackground, vFv)
GO(SDL_OnApplicationWillEnterForeground, vFv)
GO(SDL_OnApplicationWillTerminate, vFv)
GO(SDL_OpenAudioDevice, uFup)
//GOM(SDL_OpenAudioDeviceStream, pFEuppp)
GO(SDL_OpenCamera, pFup)
GO(SDL_OpenFileStorage, pFp)
GO(SDL_OpenGamepad, pFu)
GO(SDL_OpenHaptic, pFu)
GO(SDL_OpenHapticFromJoystick, pFp)
GO(SDL_OpenHapticFromMouse, pFv)
//GOM(SDL_OpenIO, pFEpp)
GO(SDL_OpenJoystick, pFu)
GO(SDL_OpenSensor, pFu)
//GOM(SDL_OpenStorage, pFEpp)
GO(SDL_OpenTitleStorage, pFpu)
GO(SDL_OpenURL, iFp)
GO(SDL_OpenUserStorage, pFppu)
GO(SDL_OutOfMemory, iFv)
GO(SDL_PauseAudioDevice, iFu)
GO(SDL_PauseAudioStreamDevice, iFp)
GO(SDL_PauseHaptic, iFp)
GO(SDL_PeepEvents, iFpiuuu)
GO(SDL_PlayHapticRumble, iFpfu)
GO(SDL_PollEvent, iFp)
GO(SDL_PopGPUDebugGroup, vFp)
GO(SDL_pow, dFdd)
GO(SDL_powf, fFff)
GO(SDL_PremultiplyAlpha, iFiiupiupii)
GO(SDL_PremultiplySurfaceAlpha, iFpi)
GO(SDL_PumpEvents, vFv)
GO(SDL_PushEvent, iFp)
GO(SDL_PushGPUComputeUniformData, vFpupu)
GO(SDL_PushGPUDebugGroup, vFpp)
GO(SDL_PushGPUFragmentUniformData, vFpupu)
GO(SDL_PushGPUVertexUniformData, vFpupu)
GO(SDL_PutAudioStreamData, iFppi)
//GOM(SDL_PutAudioStreamDataNoCopy, iFEppipp)
GO(SDL_PutAudioStreamPlanarData, iFppii)
//GOM(SDL_qsort, vFEpLLp)
//GOM(SDL_qsort_r, vFEpLLpp)
GO(SDL_QueryGPUFence, iFpp)
GO(SDL_Quit, vFv)
GO(SDL_QuitSubSystem, vFu)
GO(SDL_RaiseWindow, iFp)
GO(SDL_rand, iFi)
GO(SDL_rand_bits, uFv)
GO(SDL_rand_bits_r, uFp)
GO(SDL_randf, fFv)
GO(SDL_randf_r, fFp)
GO(SDL_rand_r, iFpi)
GO(SDL_ReadAsyncIO, iFppUUpp)
GO(SDL_ReadIO, LFppL)
GO(SDL_ReadProcess, pFppp)
GO(SDL_ReadS16BE, iFpp)
GO(SDL_ReadS16LE, iFpp)
GO(SDL_ReadS32BE, iFpp)
GO(SDL_ReadS32LE, iFpp)
GO(SDL_ReadS64BE, iFpp)
GO(SDL_ReadS64LE, iFpp)
GO(SDL_ReadS8, iFpp)
GO(SDL_ReadStorageFile, iFpppU)
GO(SDL_ReadSurfacePixel, iFpiipppp)
GO(SDL_ReadSurfacePixelFloat, iFpiipppp)
GO(SDL_ReadU16BE, iFpp)
GO(SDL_ReadU16LE, iFpp)
GO(SDL_ReadU32BE, iFpp)
GO(SDL_ReadU32LE, iFpp)
GO(SDL_ReadU64BE, iFpp)
GO(SDL_ReadU64LE, iFpp)
GO(SDL_ReadU8, iFpp)
GO(SDL_realloc, pFpL)
//GO(SDL_RegisterApp, vFv)
GO(SDL_RegisterEvents, uFi)
GO(SDL_ReleaseCameraFrame, vFpp)
GO(SDL_ReleaseGPUBuffer, vFpp)
GO(SDL_ReleaseGPUComputePipeline, vFpp)
GO(SDL_ReleaseGPUFence, vFpp)
GO(SDL_ReleaseGPUGraphicsPipeline, vFpp)
GO(SDL_ReleaseGPUSampler, vFpp)
GO(SDL_ReleaseGPUShader, vFpp)
GO(SDL_ReleaseGPUTexture, vFpp)
GO(SDL_ReleaseGPUTransferBuffer, vFpp)
GO(SDL_ReleaseWindowFromGPUDevice, vFpp)
GO(SDL_ReloadGamepadMappings, iFv)
GOM(SDL_RemoveEventWatch, vFEpp)
GOM(SDL_RemoveHintCallback, vFEppp)
GO(SDL_RemovePath, iFp)
GO(SDL_RemoveStoragePath, iFpp)
GO(SDL_RemoveSurfaceAlternateImages, vFp)
GO(SDL_RemoveTimer, iFu)
GO(SDL_RemoveTrayEntry, vFp)
GO(SDL_RenamePath, iFpp)
GO(SDL_RenameStoragePath, iFppp)
GO(SDL_RenderClear, iFp)
GO(SDL_RenderClipEnabled, iFp)
GO(SDL_RenderCoordinatesFromWindow, iFpffpp)
GO(SDL_RenderCoordinatesToWindow, iFpffpp)
GO(SDL_RenderDebugText, iFpffp)
//GOM(SDL_RenderDebugTextFormat, iFEpffpV)
GO(SDL_RenderFillRect, iFpp)
GO(SDL_RenderFillRects, iFppi)
GO(SDL_RenderGeometry, iFpppipi)
GO(SDL_RenderGeometryRaw, iFpppipipiipii)
GO(SDL_RenderLine, iFpffff)
GO(SDL_RenderLines, iFppi)
GO(SDL_RenderPoint, iFpff)
GO(SDL_RenderPoints, iFppi)
GO(SDL_RenderPresent, iFp)
GO(SDL_RenderReadPixels, pFpp)
GO(SDL_RenderRect, iFpp)
GO(SDL_RenderRects, iFppi)
GO(SDL_RenderTexture, iFpppp)
GO(SDL_RenderTexture9Grid, iFpppfffffp)
GO(SDL_RenderTexture9GridTiled, iFpppfffffpf)
GO(SDL_RenderTextureAffine, iFpppppp)
GO(SDL_RenderTextureRotated, iFppppdpu)
GO(SDL_RenderTextureTiled, iFpppfp)
GO(SDL_RenderViewportSet, iFp)
GO(SDL_ReportAssertion, uFpppi)
//GO(SDL_RequestAndroidPermission, vFv)
GO(SDL_ResetAssertionReport, vFv)
GO(SDL_ResetHint, iFp)
GO(SDL_ResetHints, vFv)
GO(SDL_ResetKeyboard, vFv)
GO(SDL_ResetLogPriorities, vFv)
GO(SDL_RestoreWindow, iFp)
GO(SDL_ResumeAudioDevice, iFu)
GO(SDL_ResumeAudioStreamDevice, iFp)
GO(SDL_ResumeHaptic, iFp)
GO(SDL_RotateSurface, pFpf)
GO(SDL_round, dFd)
GO(SDL_roundf, fFf)
GO(SDL_RumbleGamepad, iFpWWu)
GO(SDL_RumbleGamepadTriggers, iFpWWu)
GO(SDL_RumbleJoystick, iFpWWu)
GO(SDL_RumbleJoystickTriggers, iFpWWu)
//GO(SDL_RunApp, vFv)
GO(SDL_RunHapticEffect, iFpiu)
//GOM(SDL_RunOnMainThread, iFEppi)
GO(SDL_SaveBMP, iFpp)
GO(SDL_SaveBMP_IO, iFppi)
GO(SDL_SaveFile, iFppL)
GO(SDL_SaveFile_IO, iFppLi)
GO(SDL_SavePNG, iFpp)
GO(SDL_SavePNG_IO, iFppi)
GO(SDL_scalbn, dFdi)
GO(SDL_scalbnf, fFfi)
GO(SDL_ScaleSurface, pFpiii)
GO(SDL_ScreenKeyboardShown, iFp)
GO(SDL_ScreenSaverEnabled, iFv)
GO(SDL_SeekIO, IFpIu)
//GO(SDL_SendAndroidBackButton, vFv)
//GO(SDL_SendAndroidMessage, vFv)
GO(SDL_SendGamepadEffect, iFppi)
GO(SDL_SendJoystickEffect, iFppi)
GO(SDL_SendJoystickVirtualSensorData, iFpiUpi)
GO(SDL_SetAppMetadata, iFppp)
GO(SDL_SetAppMetadataProperty, iFpp)
//GOM(SDL_SetAssertionHandler, vFEpp)
GO(SDL_SetAtomicInt, iFpi)
GO(SDL_SetAtomicPointer, pFpp)
GO(SDL_SetAtomicU32, uFpu)
GO(SDL_SetAudioDeviceGain, iFuf)
//GOM(SDL_SetAudioPostmixCallback, iFEupp)
GO(SDL_SetAudioStreamFormat, iFppp)
GO(SDL_SetAudioStreamFrequencyRatio, iFpf)
GO(SDL_SetAudioStreamGain, iFpf)
//GOM(SDL_SetAudioStreamGetCallback, iFEppp)
GO(SDL_SetAudioStreamInputChannelMap, iFppi)
GO(SDL_SetAudioStreamOutputChannelMap, iFppi)
//GOM(SDL_SetAudioStreamPutCallback, iFEppp)
GO(SDL_SetBooleanProperty, iFupi)
//GOM(SDL_SetClipboardData, iFEppppL)
GO(SDL_SetClipboardText, iFp)
GO(SDL_SetCurrentThreadPriority, iFu)
GO(SDL_SetCursor, iFp)
GO(SDL_SetDefaultTextureScaleMode, iFpi)
GO(SDL_SetEnvironmentVariable, iFpppi)
GO(SDL_setenv_unsafe, iFppi)
//GOM(SDL_SetError, iFEpV)
//GOM(SDL_SetErrorV, iFEpA)
GO(SDL_SetEventEnabled, vFui)
GOM(SDL_SetEventFilter, vFEpp)
GO(SDL_SetFloatProperty, iFupf)
GO(SDL_SetGamepadEventsEnabled, vFi)
GO(SDL_SetGamepadLED, iFpCCC)
GO(SDL_SetGamepadMapping, iFup)
GO(SDL_SetGamepadPlayerIndex, iFpi)
GO(SDL_SetGamepadSensorEnabled, iFpii)
GO(SDL_SetGPUAllowedFramesInFlight, iFpu)
//GO(SDL_SetGPUBlendConstants, vFv)
GO(SDL_SetGPUBufferName, vFppp)
GO(SDL_SetGPURenderState, iFpp)
GO(SDL_SetGPURenderStateFragmentUniforms, iFpupu)
GO(SDL_SetGPUScissor, vFpp)
GO(SDL_SetGPUStencilReference, vFpC)
GO(SDL_SetGPUSwapchainParameters, iFppuu)
GO(SDL_SetGPUTextureName, vFppp)
GO(SDL_SetGPUViewport, vFpp)
GO(SDL_SetHapticAutocenter, iFpi)
GO(SDL_SetHapticGain, iFpi)
GO(SDL_SetHint, iFpp)
GO(SDL_SetHintWithPriority, iFppu)
GO(SDL_SetInitialized, vFpi)
//GO(SDL_SetiOSAnimationCallback, vFv)
//GO(SDL_SetiOSEventPump, vFv)
GO(SDL_SetJoystickEventsEnabled, vFi)
GO(SDL_SetJoystickLED, iFpCCC)
GO(SDL_SetJoystickPlayerIndex, iFpi)
GO(SDL_SetJoystickVirtualAxis, iFpiw)
GO(SDL_SetJoystickVirtualBall, iFpiww)
GO(SDL_SetJoystickVirtualButton, iFpii)
GO(SDL_SetJoystickVirtualHat, iFpiC)
GO(SDL_SetJoystickVirtualTouchpad, iFpiiifff)
GO(SDL_SetLinuxThreadPriority, iFIi)
GO(SDL_SetLinuxThreadPriorityAndPolicy, iFIii)
GOM(SDL_SetLogOutputFunction, vFEpp)
GO(SDL_SetLogPriorities, vFu)
GO(SDL_SetLogPriority, vFiu)
GO(SDL_SetLogPriorityPrefix, iFup)
//GO(SDL_SetMainReady, vFv)
//GOM(SDL_SetMemoryFunctions, iFEpppp)
GO(SDL_SetModState, vFW)
GO(SDL_SetNumberProperty, iFupI)
GO(SDL_SetPaletteColors, iFppii)
GO(SDL_SetPointerProperty, iFupp)
//GOM(SDL_SetPointerPropertyWithCleanup, iFEupppp)
GO(SDL_SetPrimarySelectionText, iFp)
//GOM(SDL_SetRelativeMouseTransform, iFEpp)
GO(SDL_SetRenderClipRect, iFpp)
GO(SDL_SetRenderColorScale, iFpf)
GO(SDL_SetRenderDrawBlendMode, iFpu)
GO(SDL_SetRenderDrawColor, iFpCCCC)
GO(SDL_SetRenderDrawColorFloat, iFpffff)
GO(SDL_SetRenderLogicalPresentation, iFpiiu)
GO(SDL_SetRenderScale, iFpff)
GO(SDL_SetRenderTarget, iFpp)
GO(SDL_SetRenderTextureAddressMode, iFpii)
GO(SDL_SetRenderViewport, iFpp)
GO(SDL_SetRenderVSync, iFpi)
GO(SDL_SetScancodeName, iFup)
GO(SDL_SetStringProperty, iFupp)
GO(SDL_SetSurfaceAlphaMod, iFpC)
GO(SDL_SetSurfaceBlendMode, iFpu)
GO(SDL_SetSurfaceClipRect, iFpp)
GO(SDL_SetSurfaceColorKey, iFpiu)
GO(SDL_SetSurfaceColorMod, iFpCCC)
GO(SDL_SetSurfaceColorspace, iFpu)
GO(SDL_SetSurfacePalette, iFpp)
GO(SDL_SetSurfaceRLE, iFpi)
GO(SDL_SetTextInputArea, iFppi)
GO(SDL_SetTextureAlphaMod, iFpC)
GO(SDL_SetTextureAlphaModFloat, iFpf)
GO(SDL_SetTextureBlendMode, iFpu)
GO(SDL_SetTextureColorMod, iFpCCC)
GO(SDL_SetTextureColorModFloat, iFpfff)
GO(SDL_SetTexturePalette, iFpp)
GO(SDL_SetTextureScaleMode, iFpi)
//GOM(SDL_SetTLS, iFEppp)
//GOM(SDL_SetTrayEntryCallback, vFEppp)
GO(SDL_SetTrayEntryChecked, vFpi)
GO(SDL_SetTrayEntryEnabled, vFpi)
GO(SDL_SetTrayEntryLabel, vFpp)
GO(SDL_SetTrayIcon, vFpp)
GO(SDL_SetTrayTooltip, vFpp)
GO(SDL_SetWindowAlwaysOnTop, iFpi)
GO(SDL_SetWindowAspectRatio, iFpff)
GO(SDL_SetWindowBordered, iFpi)
GO(SDL_SetWindowFillDocument, iFpi)
GO(SDL_SetWindowFocusable, iFpi)
GO(SDL_SetWindowFullscreen, iFpi)
GO(SDL_SetWindowFullscreenMode, iFpp)
//GOM(SDL_SetWindowHitTest, iFEppp)
GO(SDL_SetWindowIcon, iFpp)
GO(SDL_SetWindowKeyboardGrab, iFpi)
GO(SDL_SetWindowMaximumSize, iFpii)
GO(SDL_SetWindowMinimumSize, iFpii)
GO(SDL_SetWindowModal, iFpi)
GO(SDL_SetWindowMouseGrab, iFpi)
GO(SDL_SetWindowMouseRect, iFpp)
GO(SDL_SetWindowOpacity, iFpf)
GO(SDL_SetWindowParent, iFpp)
GO(SDL_SetWindowPosition, iFpii)
GO(SDL_SetWindowProgressState, iFpi)
GO(SDL_SetWindowProgressValue, iFpf)
GO(SDL_SetWindowRelativeMouseMode, iFpi)
GO(SDL_SetWindowResizable, iFpi)
GO(SDL_SetWindowShape, iFpp)
GO(SDL_SetWindowSize, iFpii)
//GO(SDL_SetWindowsMessageHook, vFv)
GO(SDL_SetWindowSurfaceVSync, iFpi)
GO(SDL_SetWindowTitle, iFpp)
//GOM(SDL_SetX11EventHook, vFEpp)
GO(SDL_ShouldInit, iFp)
GO(SDL_ShouldQuit, iFp)
//GO(SDL_ShowAndroidToast, vFv)
GO(SDL_ShowCursor, iFv)
//GOM(SDL_ShowFileDialogWithProperties, vFEuppu)
GO(SDL_ShowMessageBox, iFpp)
//GOM(SDL_ShowOpenFileDialog, vFEppppipi)
//GOM(SDL_ShowOpenFolderDialog, vFEppppi)
//GOM(SDL_ShowSaveFileDialog, vFEppppip)
GO(SDL_ShowSimpleMessageBox, iFuppp)
GO(SDL_ShowWindow, iFp)
GO(SDL_ShowWindowSystemMenu, iFpii)
GO(SDL_SignalAsyncIOQueue, vFp)
GO(SDL_SignalCondition, vFp)
GO(SDL_SignalSemaphore, vFp)
GO(SDL_sin, dFd)
GO(SDL_sinf, fFf)
//GOM(SDL_snprintf, iFEpLpV)
GO(SDL_sqrt, dFd)
GO(SDL_sqrtf, fFf)
GO(SDL_srand, vFU)
//GOM(SDL_sscanf, iFEppV)
GO(SDL_StartTextInput, iFp)
GO(SDL_StartTextInputWithProperties, iFpu)
GO(SDL_StepBackUTF8, uFpp)
GO(SDL_StepUTF8, uFpp)
GO(SDL_StopHapticEffect, iFpi)
GO(SDL_StopHapticEffects, iFp)
GO(SDL_StopHapticRumble, iFp)
GO(SDL_StopTextInput, iFp)
GO(SDL_StorageReady, iFp)
GO(SDL_strcasecmp, iFpp)
GO(SDL_strcasestr, pFpp)
GO(SDL_strchr, pFpi)
GO(SDL_strcmp, iFpp)
GO(SDL_strdup, pFp)
GO(SDL_StretchSurface, iFppppi)
GO(SDL_StringToGUID, HFp)
GO(SDL_strlcat, LFppL)
GO(SDL_strlcpy, LFppL)
GO(SDL_strlen, LFp)
GO(SDL_strlwr, pFp)
GO(SDL_strncasecmp, iFppL)
GO(SDL_strncmp, iFppL)
GO(SDL_strndup, pFpL)
GO(SDL_strnlen, LFpL)
GO(SDL_strnstr, pFppL)
GO(SDL_strpbrk, pFpp)
GO(SDL_strrchr, pFpi)
GO(SDL_strrev, pFp)
GO(SDL_strstr, pFpp)
GO(SDL_strtod, dFpp)
GO(SDL_strtok_r, pFppp)
GO(SDL_strtol, lFppi)
GO(SDL_strtoll, IFppi)
GO(SDL_strtoul, LFppi)
GO(SDL_strtoull, UFppi)
GO(SDL_strupr, pFp)
GO(SDL_SubmitGPUCommandBuffer, iFp)
GO(SDL_SubmitGPUCommandBufferAndAcquireFence, pFp)
GO(SDL_SurfaceHasAlternateImages, iFp)
GO(SDL_SurfaceHasColorKey, iFp)
GO(SDL_SurfaceHasRLE, iFp)
//GOM(SDL_swprintf, iFEpLpV)
GO(SDL_SyncWindow, iFp)
GO(SDL_tan, dFd)
GO(SDL_tanf, fFf)
GO(SDL_TellIO, IFp)
GO(SDL_TextInputActive, iFp)
GO(SDL_TimeFromWindows, IFuu)
GO(SDL_TimeToDateTime, iFIpi)
GO(SDL_TimeToWindows, vFIpp)
GO(SDL_tolower, iFi)
GO(SDL_toupper, iFi)
GO(SDL_trunc, dFd)
GO(SDL_truncf, fFf)
GO(SDL_TryLockMutex, iFp)
GO(SDL_TryLockRWLockForReading, iFp)
GO(SDL_TryLockRWLockForWriting, iFp)
GO(SDL_TryLockSpinlock, iFp)
GO(SDL_TryWaitSemaphore, iFp)
GO(SDL_UCS4ToUTF8, pFup)
GO(SDL_uitoa, pFupi)
GO(SDL_ulltoa, pFUpi)
GO(SDL_ultoa, pFLpi)
GO(SDL_UnbindAudioStream, vFp)
GO(SDL_UnbindAudioStreams, vFpi)
GO(SDL_UnloadObject, vFp)
GO(SDL_UnlockAudioStream, iFp)
GO(SDL_UnlockJoysticks, vFv)
GO(SDL_UnlockMutex, vFp)
GO(SDL_UnlockProperties, vFu)
GO(SDL_UnlockRWLock, vFp)
GO(SDL_UnlockSpinlock, vFp)
GO(SDL_UnlockSurface, vFp)
GO(SDL_UnlockTexture, vFp)
GO(SDL_UnmapGPUTransferBuffer, vFpp)
//GO(SDL_UnregisterApp, vFv)
GO(SDL_UnsetEnvironmentVariable, iFpp)
GO(SDL_unsetenv_unsafe, iFp)
GO(SDL_UpdateGamepads, vFv)
GO(SDL_UpdateHapticEffect, iFpip)
GO(SDL_UpdateJoysticks, vFv)
GO(SDL_UpdateNVTexture, iFpppipi)
GO(SDL_UpdateSensors, vFv)
GO(SDL_UpdateTexture, iFpppi)
GO(SDL_UpdateTrays, vFv)
GO(SDL_UpdateWindowSurface, iFp)
GO(SDL_UpdateWindowSurfaceRects, iFppi)
GO(SDL_UpdateYUVTexture, iFpppipipi)
GO(SDL_UploadToGPUBuffer, vFpppi)
GO(SDL_UploadToGPUTexture, vFpppi)
GO(SDL_utf8strlcpy, LFppL)
GO(SDL_utf8strlen, LFp)
GO(SDL_utf8strnlen, LFpL)
//GOM(SDL_vasprintf, iFEppA)
//GOM(SDL_vsnprintf, iFEpLpA)
//GOM(SDL_vsscanf, iFEppA)
//GOM(SDL_vswprintf, iFEpLpA)
GO(SDL_Vulkan_CreateSurface, iFpppp)
GO(SDL_Vulkan_DestroySurface, vFppp)
GO(SDL_Vulkan_GetInstanceExtensions, pFp)
GO(SDL_Vulkan_GetPresentationSupport, iFppu)
GOM(SDL_Vulkan_GetVkGetInstanceProcAddr, pFEv)
GO(SDL_Vulkan_LoadLibrary, iFp)
GO(SDL_Vulkan_UnloadLibrary, vFv)
GO(SDL_WaitAndAcquireGPUSwapchainTexture, iFppppp)
GO(SDL_WaitAsyncIOResult, iFppi)
GO(SDL_WaitCondition, vFpp)
GO(SDL_WaitConditionTimeout, iFppi)
GO(SDL_WaitEvent, iFp)
GO(SDL_WaitEventTimeout, iFpi)
GO(SDL_WaitForGPUFences, iFpipu)
GO(SDL_WaitForGPUIdle, iFp)
GO(SDL_WaitForGPUSwapchain, iFpp)
GO(SDL_WaitProcess, iFpip)
GO(SDL_WaitSemaphore, vFp)
GO(SDL_WaitSemaphoreTimeout, iFpi)
GO(SDL_WaitThread, vFpp)
GO(SDL_WarpMouseGlobal, iFff)
GO(SDL_WarpMouseInWindow, vFpff)
GO(SDL_WasInit, uFu)
GO(SDL_wcscasecmp, iFpp)
GO(SDL_wcscmp, iFpp)
GO(SDL_wcsdup, pFp)
GO(SDL_wcslcat, LFppL)
GO(SDL_wcslcpy, LFppL)
GO(SDL_wcslen, LFp)
GO(SDL_wcsncasecmp, iFppL)
GO(SDL_wcsncmp, iFppL)
GO(SDL_wcsnlen, LFpL)
GO(SDL_wcsnstr, pFppL)
GO(SDL_wcsstr, pFpp)
GO(SDL_wcstol, lFppi)
GO(SDL_WindowHasSurface, iFp)
GO(SDL_WindowSupportsGPUPresentMode, iFppu)
GO(SDL_WindowSupportsGPUSwapchainComposition, iFppu)
GO(SDL_WriteAsyncIO, iFppUUpp)
GO(SDL_WriteIO, LFppL)
GO(SDL_WriteS16BE, iFpw)
GO(SDL_WriteS16LE, iFpw)
GO(SDL_WriteS32BE, iFpi)
GO(SDL_WriteS32LE, iFpi)
GO(SDL_WriteS64BE, iFpI)
GO(SDL_WriteS64LE, iFpI)
GO(SDL_WriteS8, iFpc)
GO(SDL_WriteStorageFile, iFpppU)
GO(SDL_WriteSurfacePixel, iFpiiCCCC)
GO(SDL_WriteSurfacePixelFloat, iFpiiffff)
GO(SDL_WriteU16BE, iFpW)
GO(SDL_WriteU16LE, iFpW)
GO(SDL_WriteU32BE, iFpu)
GO(SDL_WriteU32LE, iFpu)
GO(SDL_WriteU64BE, iFpU)
GO(SDL_WriteU64LE, iFpU)
GO(SDL_WriteU8, iFpC)
