GO(AddDirtyBox, iWpp)
GO(AddDirtyRect, iWpip)
GO(AddDirtyRect, iWpp)
GO(AddRef, uWp)
GO(Apply, iWp)
GO(BeginScene, iWp)
GO(BeginStateBlock, iWp)
GO(Capture, iWp)
GO(CheckDepthStencilMatch, iWpiiii)
GO(CheckDepthStencilMatch, iWpuiiii)
GO(CheckDeviceFormatConversion, iWpiii)
GO(CheckDeviceFormatConversion, iWpuiii)
GO(CheckDeviceFormat, iWpiiuii)
GO(CheckDeviceFormat, iWpuiiuii)
GO(CheckDeviceMultiSampleType, iWpiiiip)
GO(CheckDeviceMultiSampleType, iWpuiiiip)
GO(CheckDeviceOverlayType, iWpuiuuipip)
GO(CheckDeviceState, iWpp)
GO(CheckDeviceType, iWpiiii)
GO(CheckDeviceType, iWpuiiii)
GO(Clear, iWpupuufu)
GO(ColorFill, iWpppu)
GO(ComposeRects, iWppppupiii)
GO(Configure, iWpupp)
GO(CreateDeviceEx, iWpuipuppp)
GOM(CreateDeviceEx, iWEpuipuppppp)  // Note that this one is unique (not a real GOM as it is assigned manually)
GOM(CreateDevice, iWEpuipupppp) // Note that this one is unique (not a real GOM as it is assigned manually)
GO(CreateDevice, iWpuipupp)
GO(DecryptionBlt, iWpppuppp)
GO(DeletePatch, iWpu)
GO(DrawIndexedPrimitive, iWpiiuuuu)
GO(DrawIndexedPrimitiveUP, iWpiuuupipu)
GO(DrawPrimitive, iWpiuu)
GO(DrawPrimitiveUP, iWpiupu)
GO(DrawRectPatch, iWpupp)
GO(DrawTriPatch, iWpupp)
GO(EncryptionBlt, iWpppup)
GO(EndScene, iWp)
GO(EnumAdapterModesEx, iWpupup)
GO(EnumAdapterModes, iWpuiup)
GO(EvictManagedResources, iWp)
GO(FinishSessionKeyRefresh, iWp)
GO(FreePrivateData, iWpp)
GO(GenerateMipSubLevels, vWp)
GO(GetAdapterCount, uWp)
GO(GetAdapterDisplayModeEx, iWpupp)
GO(GetAdapterDisplayMode, iWpup)
GO(GetAdapterIdentifier, iWpup)
GO(GetAdapterIdentifier, iWpuup)
GO(GetAdapterLUID, iWpup)
GO(GetAdapterModeCountEx, uWpup)
GO(GetAdapterModeCount, uWpui)
GO(GetAdapterMonitor, pWpu)
GO(GetAutoGenFilterType, iWp)
GO(GetAvailableTextureMem, uWp)
GO(GetBackBuffer, iWpuip)
GO(GetCertificate, iWpup)
GO(GetCertificateSize, iWpp)
GO(GetClipPlane, iWpup)
GO(GetClipStatus, iWpp)
GO(GetContainer, iWppp)
GO(GetContentProtectionCaps, iWpppp)
GO(GetCreationParameters, iWpp)
GO(GetCurrentTexturePalette, iWpp)
GO(GetData, iWppuu)
GO(GetDataSize, uWp)
GO(GetDC, iWpp)
GO(GetDeclaration, iWppp)
GO(GetDesc, iWpp)
GO(GetDeviceCaps, iWpip)
GO(GetDeviceCaps, iWpp)
GO(GetDeviceCaps, iWpuip)
GO(GetDevice, iWpp)
GO(GetDirect3D, iWpp)
GO(GetDisplayModeEx, iWppp)
GO(GetDisplayModeEx, iWpupp)
GO(GetDisplayMode, iWpp)
GO(GetDisplayMode, iWpup)
GO(GetEncryptionBltKey, iWppu)
GO(GetFrontBufferData, iWpp)
GO(GetFrontBufferData, iWpup)
GO(GetFunction, iWppp)
GO(GetFVF, iWpp)
GO(GetGammaRamp, vWpup)
GO(GetGPUThreadPriority, iWpp)
GO(GetIndices, iWpp)
GO(GetLastPresentCount, iWpp)
GO(GetLevelCount, uWp)
GO(GetLevelDesc, iWpup)
GO(GetLightEnable, iWpup)
GO(GetLight, iWpup)
GO(GetLOD, uWp)
GO(GetMaterial, iWpp)
GO(GetMaximumFrameLatency, iWpp)
GO(GetNPatchMode, fWp)
GO(GetNumberOfSwapChains, uWp)
GO(GetPaletteEntries, iWpup)
GO(GetPixelShaderConstantB, iWpupu)
GO(GetPixelShaderConstantF, iWpupu)
GO(GetPixelShaderConstantI, iWpupu)
GO(GetPixelShader, iWpp)
GO(GetPresentParameters, iWpp)
GO(GetPresentStats, iWpp)
GO(GetPriority, uWp)
GO(GetPrivateData, iWpppp)
GO(GetRasterStatus, iWpp)
GO(GetRasterStatus, iWpup)
GO(GetRenderState, iWpip)
GO(GetRenderTargetData, iWppp)
GO(GetSamplerState, iWpuip)
GO(GetScissorRect, iWpp)
GO(GetSoftwareVertexProcessing, iWp)
GO(GetStreamSourceFreq, iWpup)
GO(GetStreamSource, iWpuppp)
GO(GetSurfacePitch, iWppp)
GO(GetTexture, iWpup)
GO(GetTextureStageState, iWpuip)
GO(GetTransform, iWpip)
GO(GetType, iWp)
GO(GetVertexDeclaration, iWpp)
GO(GetVertexShaderConstantB, iWpupu)
GO(GetVertexShaderConstantF, iWpupu)
GO(GetVertexShaderConstantI, iWpupu)
GO(GetVertexShader, iWpp)
GO(GetViewport, iWpp)
GO(Issue, iWpu)
GO(LightEnable, iWpui)
GO(LockBox, iWpppu)
GO(LockBox, iWpuppu)
GO(Lock, iWpuupu)
GO(LockRect, iWpiuppu)
GO(LockRect, iWpppu)
GO(LockRect, iWpuppu)
GOM(CheckResourceResidency, iWEppu)
GOM(CreateAdditionalSwapChain, iWEppp)
GOM(CreateAuthenticatedChannel, iWEpipp)
GOM(CreateCryptoSession, iWEppppp)
GOM(CreateCubeTexture, iWEpuuuiipp)
GOM(CreateDepthStencilSurfaceEx, iWEpuuiiuippu)
GOM(CreateDepthStencilSurface, iWEpuuiiuipp)
GOM(CreateIndexBuffer, iWEpuuiipp)
GOM(CreateOffscreenPlainSurfaceEx, iWEpuuiippu)
GOM(CreateOffscreenPlainSurface, iWEpuuiipp)
GOM(CreatePixelShader, iWEppp)
GOM(CreateQuery, iWEpip)
GOM(CreateRenderTargetEx, iWEpuuiiuippu)
GOM(CreateRenderTarget, iWEpuuiiuipp)
GOM(CreateStateBlock, iWEpip)
GOM(CreateTexture, iWEpuuuuiipp)
GOM(CreateVertexBuffer, iWEpuuuipp)
GOM(CreateVertexDeclaration, iWEppp)
GOM(CreateVertexShader, iWEppp)
GOM(CreateVolumeTexture, iWEpuuuuuiipp)
GOM(EndStateBlock, iWEpp)
GOM(GetBackBuffer, iWEpuuip)
GOM(GetCubeMapSurface, iWEpiup)
GOM(GetDepthStencilSurface, iWEpp)
GOM(GetDirect3D, iWEpp)
GOM(GetRenderTarget, iWEpup)
GOM(GetSurfaceLevel, iWEpup)
GOM(GetSwapChain, iWEpup)
GOM(GetVolumeLevel, iWEpup)
GO(MultiplyTransform, iWpip)
GO(NegotiateKeyExchange, iWpup)
GO(PreLoad, vWp)
GO(PresentEx, iWpppppu)
GO(Present, iWppppp)
GO(Present, iWpppppu)
GO(ProcessVertices, iWpuuuppu)
GO(Query, iWpupup)
GO(QueryInterface, iWppp)
GO(RegisterSoftwareDevice, iWpp)
GO(ReleaseDC, iWpp)
GO(Release, uWp)
GO(ResetEx, iWppp)
GO(Reset, iWpp)
GO(SetAutoGenFilterType, iWpi)
GO(SetClipPlane, iWpup)
GO(SetClipStatus, iWpp)
GO(SetConvolutionMonoKernel, iWpuupp)
GO(SetCurrentTexturePalette, iWpu)
GO(SetCursorPosition, vWpiiu)
GO(SetCursorProperties, iWpuup)
GO(SetDepthStencilSurface, iWpp)
GO(SetDialogBoxMode, iWpi)
GO(SetFVF, iWpu)
GO(SetGammaRamp, vWpuup)
GO(SetGPUThreadPriority, iWpi)
GO(SetIndices, iWpp)
GO(SetLight, iWpup)
GO(SetLOD, uWpu)
GO(SetMaterial, iWpp)
GO(SetMaximumFrameLatency, iWpu)
GO(SetNPatchMode, iWpf)
GO(SetPaletteEntries, iWpup)
GO(SetPixelShaderConstantB, iWpupu)
GO(SetPixelShaderConstantF, iWpupu)
GO(SetPixelShaderConstantI, iWpupu)
GO(SetPixelShader, iWpp)
GO(SetPriority, uWpu)
GO(SetPrivateData, iWpppuu)
GO(SetRenderState, iWpiu)
GO(SetRenderTarget, iWpup)
GO(SetSamplerState, iWpuiu)
GO(SetScissorRect, iWpp)
GO(SetSoftwareVertexProcessing, iWpi)
GO(SetStreamSourceFreq, iWpuu)
GO(SetStreamSource, iWpupuu)
GO(SetTexture, iWpup)
GO(SetTextureStageState, iWpuiu)
GO(SetTransform, iWpip)
GO(SetVertexDeclaration, iWpp)
GO(SetVertexShaderConstantB, iWpupu)
GO(SetVertexShaderConstantF, iWpupu)
GO(SetVertexShaderConstantI, iWpupu)
GO(SetVertexShader, iWpp)
GO(SetViewport, iWpp)
GO(ShowCursor, iWpi)
GO(StartSessionKeyRefresh, iWppu)
GO(StretchRect, iWpppppi)
GO(TestCooperativeLevel, iWp)
GO(UnlockBox, iWp)
GO(UnlockBox, iWpu)
GO(Unlock, iWp)
GO(UnlockRect, iWp)
GO(UnlockRect, iWpiu)
GO(UnlockRect, iWpu)
GO(UpdateSurface, iWppppp)
GO(UpdateTexture, iWppp)
GO(ValidateDevice, iWpp)
GO(WaitForVBlank, iWpu)
