#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GO(Breakpad_SetSteamID, 
//DATAB(__bss_start, 
//GO(CreateInterface, 
//DATAB(_edata, 
//DATAB(_end, 
//GO(_fini, 
//GO(_init, 
//GO(SteamRealPath, 
//GO(__wrap_access, 
//GO(__wrap_chmod, 
//GO(__wrap_chown, 
//GO(__wrap_dlmopen, 
//GO(__wrap_dlopen, 
//GO(__wrap_fopen, 
//GO(__wrap_fopen64, 
//GO(__wrap_freopen, 
//GO(__wrap_lchown, 
//GO(__wrap_link, 
//GO(__wrap_lstat, 
//GO(__wrap_lstat64, 
//GO(__wrap___lxstat, 
//GO(__wrap___lxstat64, 
//GO(__wrap_mkdir, 
//GO(__wrap_mkfifo, 
//GO(__wrap_mknod, 
//GO(__wrap_mount, 
//GO(__wrap_open, 
//GO(__wrap_open64, 
//GO(__wrap_opendir, 
//GO(__wrap_rename, 
//GO(__wrap_rmdir, 
//GO(__wrap_scandir, 
//GO(__wrap_scandir64, 
//GO(__wrap_stat, 
//GO(__wrap_stat64, 
//GO(__wrap_statfs, 
//GO(__wrap_statfs64, 
//GO(__wrap_statvfs, 
//GO(__wrap_statvfs64, 
//GO(__wrap_symlink, 
//GO(__wrap_unlink, 
//GO(__wrap_utime, 
//GO(__wrap_utimes, 
//GO(__wrap___xstat, 
//GO(__wrap___xstat64, 
