#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(sd_booted, iFv)
//GOM(sd_bus_add_fallback, iFEppppp)
//GOM(sd_bus_add_fallback_vtable, iFEppppppp)
//GOM(sd_bus_add_filter, iFEpppp)
//GOM(sd_bus_add_match, iFEppppp)
//GOM(sd_bus_add_match_async, iFEpppppp)
//GOM(sd_bus_add_node_enumerator, iFEppppp)
//GOM(sd_bus_add_object, iFEppppp)
GO(sd_bus_add_object_manager, iFppp)
//GOM(sd_bus_add_object_vtable, iFEpppppp)
GO(sd_bus_attach_event, iFppi)
GO(sd_bus_call, iFppUpp)
//GOM(sd_bus_call_async, iFEpppppU)
//GOM(sd_bus_call_method, iFEppppppppV)
//GOM(sd_bus_call_method_async, iFEpppppppppV)
//GOM(sd_bus_call_method_asyncv, iFEpppppppppA)
//GOM(sd_bus_call_methodv, iFEppppppppA)
GO(sd_bus_can_send, iFpc)
GO(sd_bus_close, vFp)
GO(sd_bus_close_unref, pFp)
GO(sd_bus_creds_get_audit_login_uid, iFpp)
GO(sd_bus_creds_get_audit_session_id, iFpp)
GO(sd_bus_creds_get_augmented_mask, UFp)
GO(sd_bus_creds_get_cgroup, iFpp)
GO(sd_bus_creds_get_cmdline, iFpp)
GO(sd_bus_creds_get_comm, iFpp)
GO(sd_bus_creds_get_description, iFpp)
GO(sd_bus_creds_get_egid, iFpp)
GO(sd_bus_creds_get_euid, iFpp)
GO(sd_bus_creds_get_exe, iFpp)
GO(sd_bus_creds_get_fsgid, iFpp)
GO(sd_bus_creds_get_fsuid, iFpp)
GO(sd_bus_creds_get_gid, iFpp)
GO(sd_bus_creds_get_mask, UFp)
GO(sd_bus_creds_get_owner_uid, iFpp)
GO(sd_bus_creds_get_pid, iFpp)
GO(sd_bus_creds_get_pidfd_dup, iFpp)
GO(sd_bus_creds_get_ppid, iFpp)
GO(sd_bus_creds_get_selinux_context, iFpp)
GO(sd_bus_creds_get_session, iFpp)
GO(sd_bus_creds_get_sgid, iFpp)
GO(sd_bus_creds_get_slice, iFpp)
GO(sd_bus_creds_get_suid, iFpp)
GO(sd_bus_creds_get_supplementary_gids, iFpp)
GO(sd_bus_creds_get_tid, iFpp)
GO(sd_bus_creds_get_tid_comm, iFpp)
GO(sd_bus_creds_get_tty, iFpp)
GO(sd_bus_creds_get_uid, iFpp)
GO(sd_bus_creds_get_unique_name, iFpp)
GO(sd_bus_creds_get_unit, iFpp)
GO(sd_bus_creds_get_user_slice, iFpp)
GO(sd_bus_creds_get_user_unit, iFpp)
GO(sd_bus_creds_get_well_known_names, iFpp)
GO(sd_bus_creds_has_bounding_cap, iFpi)
GO(sd_bus_creds_has_effective_cap, iFpi)
GO(sd_bus_creds_has_inheritable_cap, iFpi)
GO(sd_bus_creds_has_permitted_cap, iFpi)
GO(sd_bus_creds_new_from_pid, iFpiU)
GO(sd_bus_creds_new_from_pidfd, iFpiU)
GO(sd_bus_creds_ref, pFp)
GO(sd_bus_creds_unref, pFp)
GO(sd_bus_default, iFp)
GO(sd_bus_default_flush_close, vFv)
GO(sd_bus_default_system, iFp)
GO(sd_bus_default_user, iFp)
GO(sd_bus_detach_event, iFp)
//GOM(sd_bus_emit_interfaces_added, iFEpppV)
GO(sd_bus_emit_interfaces_added_strv, iFppp)
//GOM(sd_bus_emit_interfaces_removed, iFEpppV)
GO(sd_bus_emit_interfaces_removed_strv, iFppp)
GO(sd_bus_emit_object_added, iFpp)
GO(sd_bus_emit_object_removed, iFpp)
//GOM(sd_bus_emit_properties_changed, iFEppppV)
GO(sd_bus_emit_properties_changed_strv, iFpppp)
//GOM(sd_bus_emit_signal, iFEpppppV)
//GOM(sd_bus_emit_signal_to, iFEppppppV)
//GOM(sd_bus_emit_signal_tov, iFEppppppA)
//GOM(sd_bus_emit_signalv, iFEpppppA)
GO(sd_bus_enqueue_for_read, iFpp)
GO(sd_bus_error_add_map, iFp)
GO(sd_bus_error_copy, iFpp)
GO(sd_bus_error_free, vFp)
GO(sd_bus_error_get_errno, iFp)
GO(sd_bus_error_has_name, iFpp)
//GOM(sd_bus_error_has_names_sentinel, iFEpV)
GO(sd_bus_error_is_set, iFp)
GO(sd_bus_error_move, iFpp)
GO(sd_bus_error_set, iFppp)
GO(sd_bus_error_set_const, iFppp)
GO(sd_bus_error_set_errno, iFpi)
//GOM(sd_bus_error_set_errnof, iFEpipV)
//GOM(sd_bus_error_set_errnofv, iFEpipA)
//GOM(sd_bus_error_setf, iFEpppV)
//GOM(sd_bus_error_setfv, iFEpppA)
GO(sd_bus_flush, iFp)
GO(sd_bus_flush_close_unref, pFp)
GO(sd_bus_get_address, iFpp)
GO(sd_bus_get_allow_interactive_authorization, iFp)
GO(sd_bus_get_bus_id, iFpp)
GO(sd_bus_get_close_on_exit, iFp)
GO(sd_bus_get_connected_signal, iFp)
GO(sd_bus_get_creds_mask, iFpp)
//GOM(sd_bus_get_current_handler, pFEp)
GO(sd_bus_get_current_message, pFp)
GO(sd_bus_get_current_slot, pFp)
GO(sd_bus_get_current_userdata, pFp)
GO(sd_bus_get_description, iFpp)
GO(sd_bus_get_event, pFp)
GO(sd_bus_get_events, iFp)
GO(sd_bus_get_exit_on_disconnect, iFp)
GO(sd_bus_get_fd, iFp)
GO(sd_bus_get_method_call_timeout, iFpp)
GO(sd_bus_get_name_creds, iFppUp)
GO(sd_bus_get_name_machine_id, iFppp)
GO(sd_bus_get_n_queued_read, iFpp)
GO(sd_bus_get_n_queued_write, iFpp)
GO(sd_bus_get_owner_creds, iFpUp)
GO(sd_bus_get_property, iFpppppppp)
GO(sd_bus_get_property_string, iFppppppp)
GO(sd_bus_get_property_strv, iFppppppp)
GO(sd_bus_get_property_trivial, iFppppppcp)
GO(sd_bus_get_scope, iFpp)
GO(sd_bus_get_sender, iFpp)
GO(sd_bus_get_tid, iFpp)
GO(sd_bus_get_timeout, iFpp)
GO(sd_bus_get_unique_name, iFpp)
GO(sd_bus_get_watch_bind, iFp)
GO(sd_bus_interface_name_is_valid, iFp)
GO(sd_bus_is_anonymous, iFp)
GO(sd_bus_is_bus_client, iFp)
GO(sd_bus_is_monitor, iFp)
GO(sd_bus_is_open, iFp)
GO(sd_bus_is_ready, iFp)
GO(sd_bus_is_server, iFp)
GO(sd_bus_is_trusted, iFp)
GO(sd_bus_list_names, iFppp)
//GOM(sd_bus_match_signal, iFEpppppppp)
//GOM(sd_bus_match_signal_async, iFEppppppppp)
GO(sd_bus_member_name_is_valid, iFp)
//GOM(sd_bus_message_append, iFEppV)
GO(sd_bus_message_append_array, iFpcpL)
GO(sd_bus_message_append_array_iovec, iFpcpu)
GO(sd_bus_message_append_array_memfd, iFpciUU)
GO(sd_bus_message_append_array_space, iFpcLp)
GO(sd_bus_message_append_basic, iFpcp)
GO(sd_bus_message_append_string_iovec, iFppu)
GO(sd_bus_message_append_string_memfd, iFpiUU)
GO(sd_bus_message_append_string_space, iFpLp)
GO(sd_bus_message_append_strv, iFpp)
//GOM(sd_bus_message_appendv, iFEppA)
GO(sd_bus_message_at_end, iFpi)
GO(sd_bus_message_close_container, iFp)
GO(sd_bus_message_copy, iFppi)
GO(sd_bus_message_dump, iFpSU)
GO(sd_bus_message_dump_json, iFpUp)
GO(sd_bus_message_enter_container, iFpcp)
GO(sd_bus_message_exit_container, iFp)
GO(sd_bus_message_get_allow_interactive_authorization, iFp)
GO(sd_bus_message_get_auto_start, iFp)
GO(sd_bus_message_get_bus, pFp)
GO(sd_bus_message_get_cookie, iFpp)
GO(sd_bus_message_get_creds, pFp)
GO(sd_bus_message_get_destination, pFp)
GO(sd_bus_message_get_errno, iFp)
GO(sd_bus_message_get_error, pFp)
GO(sd_bus_message_get_expect_reply, iFp)
GO(sd_bus_message_get_interface, pFp)
GO(sd_bus_message_get_member, pFp)
GO(sd_bus_message_get_monotonic_usec, iFpp)
GO(sd_bus_message_get_path, pFp)
GO(sd_bus_message_get_priority, iFpp)
GO(sd_bus_message_get_realtime_usec, iFpp)
GO(sd_bus_message_get_reply_cookie, iFpp)
GO(sd_bus_message_get_sender, pFp)
GO(sd_bus_message_get_seqnum, iFpp)
GO(sd_bus_message_get_signature, pFpi)
GO(sd_bus_message_get_type, iFpp)
GO(sd_bus_message_has_signature, iFpp)
GO(sd_bus_message_is_empty, iFp)
GO(sd_bus_message_is_method_call, iFppp)
GO(sd_bus_message_is_method_error, iFpp)
GO(sd_bus_message_is_signal, iFppp)
GO(sd_bus_message_new, iFppC)
GO(sd_bus_message_new_method_call, iFpppppp)
GO(sd_bus_message_new_method_errno, iFppip)
//GOM(sd_bus_message_new_method_errnof, iFEppipV)
GO(sd_bus_message_new_method_error, iFppp)
//GOM(sd_bus_message_new_method_errorf, iFEppppV)
GO(sd_bus_message_new_method_return, iFpp)
GO(sd_bus_message_new_signal, iFppppp)
GO(sd_bus_message_new_signal_to, iFpppppp)
GO(sd_bus_message_open_container, iFpcp)
GO(sd_bus_message_peek_type, iFppp)
//GOM(sd_bus_message_read, iFEppV)
GO(sd_bus_message_read_array, iFpcpp)
GO(sd_bus_message_read_basic, iFpcp)
GO(sd_bus_message_read_strv, iFpp)
GO(sd_bus_message_read_strv_extend, iFpp)
//GOM(sd_bus_message_readv, iFEppA)
GO(sd_bus_message_ref, pFp)
GO(sd_bus_message_rewind, iFpi)
GO(sd_bus_message_seal, iFpUU)
GO(sd_bus_message_send, iFp)
GO(sd_bus_message_sensitive, iFp)
GO(sd_bus_message_set_allow_interactive_authorization, iFpi)
GO(sd_bus_message_set_auto_start, iFpi)
GO(sd_bus_message_set_destination, iFpp)
GO(sd_bus_message_set_expect_reply, iFpi)
GO(sd_bus_message_set_priority, iFpI)
GO(sd_bus_message_set_sender, iFpp)
GO(sd_bus_message_skip, iFpp)
GO(sd_bus_message_unref, pFp)
GO(sd_bus_message_verify_type, iFpcp)
GO(sd_bus_negotiate_creds, iFpiU)
GO(sd_bus_negotiate_fds, iFpi)
GO(sd_bus_negotiate_timestamp, iFpi)
GO(sd_bus_new, iFp)
GO(sd_bus_object_path_is_valid, iFp)
GO(sd_bus_open, iFp)
GO(sd_bus_open_system, iFp)
GO(sd_bus_open_system_machine, iFpp)
GO(sd_bus_open_system_remote, iFpp)
GO(sd_bus_open_system_with_description, iFpp)
GO(sd_bus_open_user, iFp)
GO(sd_bus_open_user_machine, iFpp)
GO(sd_bus_open_user_with_description, iFpp)
GO(sd_bus_open_with_description, iFpp)
GO(sd_bus_path_decode, iFppp)
//GOM(sd_bus_path_decode_many, iFEppV)
GO(sd_bus_path_encode, iFppp)
//GOM(sd_bus_path_encode_many, iFEppV)
GO(sd_bus_pending_method_calls, iFp)
GO(sd_bus_process, iFpp)
GO(sd_bus_process_priority, iFpIp)
GO(sd_bus_query_sender_creds, iFpUp)
GO(sd_bus_query_sender_privilege, iFpi)
GO(sd_bus_ref, pFp)
GO(sd_bus_release_name, iFpp)
//GOM(sd_bus_release_name_async, iFEppppp)
GO(sd_bus_reply_method_errno, iFpip)
//GOM(sd_bus_reply_method_errnof, iFEpipV)
//GOM(sd_bus_reply_method_errnofv, iFEpipA)
GO(sd_bus_reply_method_error, iFpp)
//GOM(sd_bus_reply_method_errorf, iFEpppV)
//GOM(sd_bus_reply_method_errorfv, iFEpppA)
//GOM(sd_bus_reply_method_return, iFEppV)
//GOM(sd_bus_reply_method_returnv, iFEppA)
GO(sd_bus_request_name, iFppU)
//GOM(sd_bus_request_name_async, iFEpppUpp)
GO(sd_bus_send, iFppp)
GO(sd_bus_send_to, iFpppp)
GO(sd_bus_service_name_is_valid, iFp)
GO(sd_bus_set_address, iFpp)
GO(sd_bus_set_allow_interactive_authorization, iFpi)
GO(sd_bus_set_anonymous, iFpi)
GO(sd_bus_set_bus_client, iFpi)
GO(sd_bus_set_close_on_exit, iFpi)
GO(sd_bus_set_connected_signal, iFpi)
GO(sd_bus_set_description, iFpp)
GO(sd_bus_set_exec, iFppp)
GO(sd_bus_set_exit_on_disconnect, iFpi)
GO(sd_bus_set_fd, iFpii)
GO(sd_bus_set_method_call_timeout, iFpU)
GO(sd_bus_set_monitor, iFpi)
//GOM(sd_bus_set_property, iFEpppppppV)
//GOM(sd_bus_set_propertyv, iFEpppppppA)
GO(sd_bus_set_sender, iFpp)
GO(sd_bus_set_server, iFpiH)
GO(sd_bus_set_trusted, iFpi)
GO(sd_bus_set_watch_bind, iFpi)
GO(sd_bus_slot_get_bus, pFp)
//GOM(sd_bus_slot_get_current_handler, pFEp)
GO(sd_bus_slot_get_current_message, pFp)
GO(sd_bus_slot_get_current_userdata, pFp)
GO(sd_bus_slot_get_description, iFpp)
//GOM(sd_bus_slot_get_destroy_callback, iFEpp)
GO(sd_bus_slot_get_floating, iFp)
GO(sd_bus_slot_get_userdata, pFp)
GO(sd_bus_slot_ref, pFp)
GO(sd_bus_slot_set_description, iFpp)
//GOM(sd_bus_slot_set_destroy_callback, iFEpp)
GO(sd_bus_slot_set_floating, iFpi)
GO(sd_bus_slot_set_userdata, pFpp)
GO(sd_bus_slot_unref, pFp)
GO(sd_bus_start, iFp)
GO(sd_bus_track_add_name, iFpp)
GO(sd_bus_track_add_sender, iFpp)
GO(sd_bus_track_contains, pFpp)
GO(sd_bus_track_count, uFp)
GO(sd_bus_track_count_name, iFpp)
GO(sd_bus_track_count_sender, iFpp)
GO(sd_bus_track_first, pFp)
GO(sd_bus_track_get_bus, pFp)
//GOM(sd_bus_track_get_destroy_callback, iFEpp)
GO(sd_bus_track_get_recursive, iFp)
GO(sd_bus_track_get_userdata, pFp)
//GOM(sd_bus_track_new, iFEpppp)
GO(sd_bus_track_next, pFp)
GO(sd_bus_track_ref, pFp)
GO(sd_bus_track_remove_name, iFpp)
GO(sd_bus_track_remove_sender, iFpp)
//GOM(sd_bus_track_set_destroy_callback, iFEpp)
GO(sd_bus_track_set_recursive, iFpi)
GO(sd_bus_track_set_userdata, pFpp)
GO(sd_bus_track_unref, pFp)
GO(sd_bus_try_close, iFp)
GO(sd_bus_unref, pFp)
GO(sd_bus_wait, iFpU)
GO(sd_device_enumerator_add_all_parents, iFp)
GO(sd_device_enumerator_add_match_parent, iFpp)
GO(sd_device_enumerator_add_match_property, iFppp)
GO(sd_device_enumerator_add_match_property_required, iFppp)
GO(sd_device_enumerator_add_match_subsystem, iFppi)
GO(sd_device_enumerator_add_match_sysattr, iFpppi)
GO(sd_device_enumerator_add_match_sysname, iFpp)
GO(sd_device_enumerator_add_match_tag, iFpp)
GO(sd_device_enumerator_add_nomatch_sysname, iFpp)
GO(sd_device_enumerator_allow_uninitialized, iFp)
GO(sd_device_enumerator_get_device_first, pFp)
GO(sd_device_enumerator_get_device_next, pFp)
GO(sd_device_enumerator_get_subsystem_first, pFp)
GO(sd_device_enumerator_get_subsystem_next, pFp)
GO(sd_device_enumerator_new, iFp)
GO(sd_device_enumerator_ref, pFp)
GO(sd_device_enumerator_unref, pFp)
GO(sd_device_get_action, iFpp)
GO(sd_device_get_child_first, pFpp)
GO(sd_device_get_child_next, pFpp)
GO(sd_device_get_current_tag_first, pFp)
GO(sd_device_get_current_tag_next, pFp)
GO(sd_device_get_device_id, iFpp)
GO(sd_device_get_devlink_first, pFp)
GO(sd_device_get_devlink_next, pFp)
GO(sd_device_get_devname, iFpp)
GO(sd_device_get_devnum, iFpp)
GO(sd_device_get_devpath, iFpp)
GO(sd_device_get_devtype, iFpp)
GO(sd_device_get_diskseq, iFpp)
GO(sd_device_get_driver, iFpp)
GO(sd_device_get_driver_subsystem, iFpp)
GO(sd_device_get_ifindex, iFpp)
GO(sd_device_get_is_initialized, iFp)
GO(sd_device_get_parent, iFpp)
GO(sd_device_get_parent_with_subsystem_devtype, iFpppp)
GO(sd_device_get_property_first, pFpp)
GO(sd_device_get_property_next, pFpp)
GO(sd_device_get_property_value, iFppp)
GO(sd_device_get_seqnum, iFpp)
GO(sd_device_get_subsystem, iFpp)
GO(sd_device_get_sysattr_first, pFp)
GO(sd_device_get_sysattr_next, pFp)
GO(sd_device_get_sysattr_value, iFppp)
GO(sd_device_get_sysattr_value_with_size, iFpppp)
GO(sd_device_get_sysname, iFpp)
GO(sd_device_get_sysnum, iFpp)
GO(sd_device_get_syspath, iFpp)
GO(sd_device_get_tag_first, pFp)
GO(sd_device_get_tag_next, pFp)
GO(sd_device_get_trigger_uuid, iFpp)
GO(sd_device_get_usec_initialized, iFpp)
GO(sd_device_get_usec_since_initialized, iFpp)
GO(sd_device_has_current_tag, iFpp)
GO(sd_device_has_tag, iFpp)
GO(sd_device_monitor_attach_event, iFpp)
GO(sd_device_monitor_detach_event, iFp)
GO(sd_device_monitor_filter_add_match_parent, iFppi)
GO(sd_device_monitor_filter_add_match_subsystem_devtype, iFppp)
GO(sd_device_monitor_filter_add_match_sysattr, iFpppi)
GO(sd_device_monitor_filter_add_match_tag, iFpp)
GO(sd_device_monitor_filter_remove, iFp)
GO(sd_device_monitor_filter_update, iFp)
GO(sd_device_monitor_get_description, iFpp)
GO(sd_device_monitor_get_event, pFp)
GO(sd_device_monitor_get_events, iFp)
GO(sd_device_monitor_get_event_source, pFp)
GO(sd_device_monitor_get_fd, iFp)
GO(sd_device_monitor_get_timeout, iFpp)
GO(sd_device_monitor_is_running, iFp)
GO(sd_device_monitor_new, iFp)
GO(sd_device_monitor_receive, iFpp)
GO(sd_device_monitor_ref, pFp)
GO(sd_device_monitor_set_description, iFpp)
GO(sd_device_monitor_set_receive_buffer_size, iFpL)
//GOM(sd_device_monitor_start, iFEppp)
GO(sd_device_monitor_stop, iFp)
GO(sd_device_monitor_unref, pFp)
GO(sd_device_new_child, iFppp)
GO(sd_device_new_from_device_id, iFpp)
GO(sd_device_new_from_devname, iFpp)
GO(sd_device_new_from_devnum, iFpcU)
GO(sd_device_new_from_ifindex, iFpi)
GO(sd_device_new_from_ifname, iFpp)
GO(sd_device_new_from_path, iFpp)
GO(sd_device_new_from_stat_rdev, iFpp)
GO(sd_device_new_from_subsystem_sysname, iFppp)
GO(sd_device_new_from_syspath, iFpp)
GO(sd_device_open, iFpi)
GO(sd_device_ref, pFp)
GO(sd_device_set_sysattr_value, iFppp)
//GOM(sd_device_set_sysattr_valuef, iFEpppV)
GO(sd_device_trigger, iFpI)
GO(sd_device_trigger_with_uuid, iFpIp)
GO(sd_device_unref, pFp)
//GOM(sd_event_add_child, iFEppiipp)
//GOM(sd_event_add_child_pidfd, iFEppiipp)
//GOM(sd_event_add_defer, iFEpppp)
//GOM(sd_event_add_exit, iFEpppp)
//GOM(sd_event_add_inotify, iFEpppupp)
//GOM(sd_event_add_inotify_fd, iFEppiupp)
//GOM(sd_event_add_io, iFEppiupp)
//GOM(sd_event_add_memory_pressure, iFEpppp)
//GOM(sd_event_add_post, iFEpppp)
//GOM(sd_event_add_signal, iFEppipp)
//GOM(sd_event_add_time, iFEppiUUpp)
//GOM(sd_event_add_time_relative, iFEppiUUpp)
GO(sd_event_default, iFp)
GO(sd_event_dispatch, iFp)
GO(sd_event_exit, iFpi)
GO(sd_event_get_exit_code, iFpp)
GO(sd_event_get_exit_on_idle, iFp)
GO(sd_event_get_fd, iFp)
GO(sd_event_get_iteration, iFpp)
GO(sd_event_get_state, iFp)
GO(sd_event_get_tid, iFpp)
GO(sd_event_get_watchdog, iFp)
GO(sd_event_loop, iFp)
GO(sd_event_new, iFp)
GO(sd_event_now, iFpip)
GO(sd_event_prepare, iFp)
GO(sd_event_ref, pFp)
GO(sd_event_run, iFpU)
GO(sd_event_set_exit_on_idle, iFpi)
GO(sd_event_set_signal_exit, iFpi)
GO(sd_event_set_watchdog, iFpi)
GO(sd_event_source_disable_unref, pFp)
GO(sd_event_source_get_child_pid, iFpp)
GO(sd_event_source_get_child_pidfd, iFp)
GO(sd_event_source_get_child_pidfd_own, iFp)
GO(sd_event_source_get_child_process_own, iFp)
GO(sd_event_source_get_description, iFpp)
//GOM(sd_event_source_get_destroy_callback, iFEpp)
GO(sd_event_source_get_enabled, iFpp)
GO(sd_event_source_get_event, pFp)
GO(sd_event_source_get_exit_on_failure, iFp)
GO(sd_event_source_get_floating, iFp)
GO(sd_event_source_get_inotify_mask, iFpp)
GO(sd_event_source_get_inotify_path, iFpp)
GO(sd_event_source_get_io_events, iFpp)
GO(sd_event_source_get_io_fd, iFp)
GO(sd_event_source_get_io_fd_own, iFp)
GO(sd_event_source_get_io_revents, iFpp)
GO(sd_event_source_get_pending, iFp)
GO(sd_event_source_get_priority, iFpp)
GO(sd_event_source_get_ratelimit, iFppp)
GO(sd_event_source_get_signal, iFp)
GO(sd_event_source_get_time, iFpp)
GO(sd_event_source_get_time_accuracy, iFpp)
GO(sd_event_source_get_time_clock, iFpp)
GO(sd_event_source_get_userdata, pFp)
GO(sd_event_source_is_ratelimited, iFp)
GO(sd_event_source_leave_ratelimit, iFp)
GO(sd_event_source_ref, pFp)
GO(sd_event_source_send_child_signal, iFpipu)
GO(sd_event_source_set_child_pidfd_own, iFpi)
GO(sd_event_source_set_child_process_own, iFpi)
GO(sd_event_source_set_description, iFpp)
//GOM(sd_event_source_set_destroy_callback, iFEpp)
GO(sd_event_source_set_enabled, iFpi)
GO(sd_event_source_set_exit_on_failure, iFpi)
GO(sd_event_source_set_floating, iFpi)
GO(sd_event_source_set_io_events, iFpu)
GO(sd_event_source_set_io_fd, iFpi)
GO(sd_event_source_set_io_fd_own, iFpi)
GO(sd_event_source_set_memory_pressure_period, iFpUU)
GO(sd_event_source_set_memory_pressure_type, iFpp)
//GOM(sd_event_source_set_prepare, iFEpp)
GO(sd_event_source_set_priority, iFpI)
GO(sd_event_source_set_ratelimit, iFpUu)
//GOM(sd_event_source_set_ratelimit_expire_callback, iFEpp)
GO(sd_event_source_set_time, iFpU)
GO(sd_event_source_set_time_accuracy, iFpU)
GO(sd_event_source_set_time_relative, iFpU)
GO(sd_event_source_set_userdata, pFpp)
GO(sd_event_source_unref, pFp)
GO(sd_event_trim_memory, iFv)
GO(sd_event_unref, pFp)
GO(sd_event_wait, iFpU)
GO(sd_get_machine_names, iFp)
GO(sd_get_seats, iFp)
GO(sd_get_sessions, iFp)
GO(sd_get_uids, iFp)
GO(sd_hwdb_enumerate, iFppp)
GO(sd_hwdb_get, iFpppp)
GO(sd_hwdb_new, iFp)
GO(sd_hwdb_new_from_path, iFpp)
GO(sd_hwdb_ref, pFp)
GO(sd_hwdb_seek, iFpp)
GO(sd_hwdb_unref, pFp)
GO(sd_id128_from_string, iFpp)
GO(sd_id128_get_app_specific, iFHHp)
GO(sd_id128_get_boot, iFp)
GO(sd_id128_get_boot_app_specific, iFHp)
GO(sd_id128_get_invocation, iFp)
GO(sd_id128_get_invocation_app_specific, iFHp)
GO(sd_id128_get_machine, iFp)
GO(sd_id128_get_machine_app_specific, iFHp)
GO(sd_id128_randomize, iFp)
GO(sd_id128_string_equal, iFpH)
GO(sd_id128_to_string, pFHp)
GO(sd_id128_to_uuid_string, pFHp)
GO(sd_is_fifo, iFip)
GO(sd_is_mq, iFip)
GO(sd_is_socket, iFiiii)
GO(sd_is_socket_inet, iFiiiiW)
GO(sd_is_socket_sockaddr, iFiipui)
GO(sd_is_socket_unix, iFiiipL)
GO(sd_is_special, iFip)
GO(sd_journal_add_conjunction, iFp)
GO(sd_journal_add_disjunction, iFp)
GO(sd_journal_add_match, iFppL)
GO(sd_journal_close, vFp)
GO(sd_journal_enumerate_available_data, iFppp)
GO(sd_journal_enumerate_available_unique, iFppp)
GO(sd_journal_enumerate_data, iFppp)
GO(sd_journal_enumerate_fields, iFpp)
GO(sd_journal_enumerate_unique, iFppp)
GO(sd_journal_flush_matches, vFp)
GO(sd_journal_get_catalog, iFpp)
GO(sd_journal_get_catalog_for_message_id, iFHp)
GO(sd_journal_get_cursor, iFpp)
GO(sd_journal_get_cutoff_monotonic_usec, iFpHpp)
GO(sd_journal_get_cutoff_realtime_usec, iFppp)
GO(sd_journal_get_data, iFpppp)
GO(sd_journal_get_data_threshold, iFpp)
GO(sd_journal_get_events, iFp)
GO(sd_journal_get_fd, iFp)
GO(sd_journal_get_monotonic_usec, iFppp)
GO(sd_journal_get_realtime_usec, iFpp)
GO(sd_journal_get_seqnum, iFppp)
GO(sd_journal_get_timeout, iFpp)
GO(sd_journal_get_usage, iFpp)
GO(sd_journal_has_persistent_files, iFp)
GO(sd_journal_has_runtime_files, iFp)
GO(sd_journal_next, iFp)
GO(sd_journal_next_skip, iFpU)
GO(sd_journal_open, iFpi)
GO(sd_journal_open_container, iFppi)
GO(sd_journal_open_directory, iFppi)
GO(sd_journal_open_directory_fd, iFpii)
GO(sd_journal_open_files, iFppi)
GO(sd_journal_open_files_fd, iFppui)
GO(sd_journal_open_namespace, iFppi)
GO(sd_journal_perror, iFp)
GO(sd_journal_perror_with_location, iFpppp)
GO(sd_journal_previous, iFp)
GO(sd_journal_previous_skip, iFpU)
//GOM(sd_journal_print, iFEipV)
//GOM(sd_journal_printv, iFEipA)
//GOM(sd_journal_printv_with_location, iFEippppA)
//GOM(sd_journal_print_with_location, iFEippppV)
GO(sd_journal_process, iFp)
GO(sd_journal_query_unique, iFpp)
GO(sd_journal_reliable_fd, iFp)
GO(sd_journal_restart_data, vFp)
GO(sd_journal_restart_fields, vFp)
GO(sd_journal_restart_unique, vFp)
GO(sd_journal_seek_cursor, iFpp)
GO(sd_journal_seek_head, iFp)
GO(sd_journal_seek_monotonic_usec, iFpHU)
GO(sd_journal_seek_realtime_usec, iFpU)
GO(sd_journal_seek_tail, iFp)
GO(sd_journal_send, iFppppppppppppp)        // signature should be iFEpV
GO(sd_journal_sendv, iFpi)
GO(sd_journal_sendv_with_location, iFppppi)
//GOM(sd_journal_send_with_location, iFEppppV)
GO(sd_journal_set_data_threshold, iFpL)
GO(sd_journal_step_one, iFpi)
GO(sd_journal_stream_fd, iFpii)
GO(sd_journal_stream_fd_with_namespace, iFppii)
GO(sd_journal_test_cursor, iFpp)
GO(sd_journal_wait, iFpU)
//GOM(sd_json_build, iFEpV)
//GOM(sd_json_buildv, iFEpA)
//GOM(sd_json_dispatch, iFEppIp)
GO(sd_json_dispatch_const_string, iFppIp)
GO(sd_json_dispatch_double, iFppIp)
//GOM(sd_json_dispatch_full, iFEpppIpp)
GO(sd_json_dispatch_id128, iFppIp)
GO(sd_json_dispatch_int16, iFppIp)
GO(sd_json_dispatch_int32, iFppIp)
GO(sd_json_dispatch_int64, iFppIp)
GO(sd_json_dispatch_int8, iFppIp)
GO(sd_json_dispatch_intbool, iFppIp)
GO(sd_json_dispatch_signal, iFppIp)
GO(sd_json_dispatch_stdbool, iFppIp)
GO(sd_json_dispatch_string, iFppIp)
GO(sd_json_dispatch_strv, iFppIp)
GO(sd_json_dispatch_tristate, iFppIp)
GO(sd_json_dispatch_uid_gid, iFppIp)
GO(sd_json_dispatch_uint16, iFppIp)
GO(sd_json_dispatch_uint32, iFppIp)
GO(sd_json_dispatch_uint64, iFppIp)
GO(sd_json_dispatch_uint8, iFppIp)
GO(sd_json_dispatch_unsupported, iFppIp)
GO(sd_json_dispatch_variant, iFppIp)
GO(sd_json_dispatch_variant_noref, iFppIp)
GO(sd_json_parse, iFpIppp)
GO(sd_json_parse_continue, iFpIppp)
GO(sd_json_parse_file, iFSpIppp)
GO(sd_json_parse_file_at, iFSipIppp)
GO(sd_json_parse_with_source, iFppIppp)
GO(sd_json_parse_with_source_continue, iFppIppp)
GO(sd_json_variant_append_array, iFpp)
//GOM(sd_json_variant_append_arrayb, iFEpV)
GO(sd_json_variant_append_array_nodup, iFpp)
GO(sd_json_variant_boolean, iFp)
GO(sd_json_variant_by_index, pFpL)
GO(sd_json_variant_by_key, pFpp)
GO(sd_json_variant_by_key_full, pFppp)
GO(sd_json_variant_dump, iFpISp)
GO(sd_json_variant_elements, LFp)
GO(sd_json_variant_equal, iFpp)
GO(sd_json_variant_filter, iFpp)
GO(sd_json_variant_find, pFpp)
GO(sd_json_variant_format, iFpIp)
GO(sd_json_variant_get_source, iFpppp)
GO(sd_json_variant_has_type, iFpI)
GO(sd_json_variant_integer, IFp)
GO(sd_json_variant_is_array, iFp)
GO(sd_json_variant_is_blank_array, iFp)
GO(sd_json_variant_is_blank_object, iFp)
GO(sd_json_variant_is_boolean, iFp)
GO(sd_json_variant_is_integer, iFp)
GO(sd_json_variant_is_negative, iFp)
GO(sd_json_variant_is_normalized, iFp)
GO(sd_json_variant_is_null, iFp)
GO(sd_json_variant_is_number, iFp)
GO(sd_json_variant_is_object, iFp)
GO(sd_json_variant_is_real, iFp)
GO(sd_json_variant_is_sensitive, iFp)
GO(sd_json_variant_is_sensitive_recursive, iFp)
GO(sd_json_variant_is_sorted, iFp)
GO(sd_json_variant_is_string, iFp)
GO(sd_json_variant_is_unsigned, iFp)
GO(sd_json_variant_merge_object, iFpp)
//GOM(sd_json_variant_merge_objectb, iFEpV)
GO(sd_json_variant_new_array, iFppL)
GO(sd_json_variant_new_array_bytes, iFppL)
GO(sd_json_variant_new_array_strv, iFpp)
GO(sd_json_variant_new_base32hex, iFppL)
GO(sd_json_variant_new_base64, iFppL)
GO(sd_json_variant_new_boolean, iFpi)
GO(sd_json_variant_new_hex, iFppL)
GO(sd_json_variant_new_id128, iFpH)
GO(sd_json_variant_new_integer, iFpI)
GO(sd_json_variant_new_null, iFp)
GO(sd_json_variant_new_object, iFppL)
GO(sd_json_variant_new_octescape, iFppL)
GO(sd_json_variant_new_real, iFpd)
GO(sd_json_variant_new_string, iFpp)
GO(sd_json_variant_new_stringn, iFppL)
GO(sd_json_variant_new_unsigned, iFpU)
GO(sd_json_variant_new_uuid, iFpH)
GO(sd_json_variant_normalize, iFp)
GO(sd_json_variant_real, dFp)
GO(sd_json_variant_ref, pFp)
GO(sd_json_variant_sensitive, vFp)
GO(sd_json_variant_set_field, iFppp)
//GOM(sd_json_variant_set_fieldb, iFEppV)
GO(sd_json_variant_set_field_boolean, iFppi)
GO(sd_json_variant_set_field_id128, iFppH)
GO(sd_json_variant_set_field_integer, iFppI)
GO(sd_json_variant_set_field_string, iFppp)
GO(sd_json_variant_set_field_strv, iFppp)
GO(sd_json_variant_set_field_unsigned, iFppU)
GO(sd_json_variant_set_field_uuid, iFppH)
GO(sd_json_variant_sort, iFp)
GO(sd_json_variant_string, pFp)
GO(sd_json_variant_strv, iFpp)
GO(sd_json_variant_type, IFp)
GO(sd_json_variant_type_from_string, IFp)
GO(sd_json_variant_type_to_string, pFI)
GO(sd_json_variant_unbase64, iFppp)
GO(sd_json_variant_unhex, iFppp)
GO(sd_json_variant_unref, pFp)
GO(sd_json_variant_unref_many, vFpL)
GO(sd_json_variant_unsigned, UFp)
GO(sd_listen_fds, iFi)
GO(sd_listen_fds_with_names, iFip)
GO(sd_login_monitor_flush, iFp)
GO(sd_login_monitor_get_events, iFp)
GO(sd_login_monitor_get_fd, iFp)
GO(sd_login_monitor_get_timeout, iFpp)
GO(sd_login_monitor_new, iFpp)
GO(sd_login_monitor_unref, pFp)
GO(sd_machine_get_class, iFpp)
GO(sd_machine_get_ifindices, iFpp)
GO(sd_notify, iFip)
GO(sd_notify_barrier, iFiU)
//GOM(sd_notifyf, iFEipV)
GO(sd_path_lookup, iFUpp)
GO(sd_path_lookup_strv, iFUpp)
GO(sd_peer_get_cgroup, iFip)
GO(sd_peer_get_machine_name, iFip)
GO(sd_peer_get_owner_uid, iFip)
GO(sd_peer_get_session, iFip)
GO(sd_peer_get_slice, iFip)
GO(sd_peer_get_unit, iFip)
GO(sd_peer_get_user_slice, iFip)
GO(sd_peer_get_user_unit, iFip)
GO(sd_pidfd_get_cgroup, iFip)
GO(sd_pidfd_get_inode_id, iFip)
GO(sd_pidfd_get_machine_name, iFip)
GO(sd_pidfd_get_owner_uid, iFip)
GO(sd_pidfd_get_session, iFip)
GO(sd_pidfd_get_slice, iFip)
GO(sd_pidfd_get_unit, iFip)
GO(sd_pidfd_get_user_slice, iFip)
GO(sd_pidfd_get_user_unit, iFip)
GO(sd_pid_get_cgroup, iFip)
GO(sd_pid_get_machine_name, iFip)
GO(sd_pid_get_owner_uid, iFip)
GO(sd_pid_get_session, iFip)
GO(sd_pid_get_slice, iFip)
GO(sd_pid_get_unit, iFip)
GO(sd_pid_get_user_slice, iFip)
GO(sd_pid_get_user_unit, iFip)
GO(sd_pid_notify, iFiip)
GO(sd_pid_notify_barrier, iFiiU)
//GOM(sd_pid_notifyf, iFEiipV)
//GOM(sd_pid_notifyf_with_fds, iFEiipLpV)
GO(sd_pid_notify_with_fds, iFiippu)
GO(sd_seat_can_graphical, iFp)
GO(sd_seat_can_multi_session, iFp)
GO(sd_seat_can_tty, iFp)
GO(sd_seat_get_active, iFppp)
GO(sd_seat_get_sessions, iFpppp)
GO(sd_session_get_class, iFpp)
GO(sd_session_get_desktop, iFpp)
GO(sd_session_get_display, iFpp)
GO(sd_session_get_extra_device_access, iFpp)
GO(sd_session_get_leader, iFpp)
GO(sd_session_get_remote_host, iFpp)
GO(sd_session_get_remote_user, iFpp)
GO(sd_session_get_seat, iFpp)
GO(sd_session_get_service, iFpp)
GO(sd_session_get_start_time, iFpp)
GO(sd_session_get_state, iFpp)
GO(sd_session_get_tty, iFpp)
GO(sd_session_get_type, iFpp)
GO(sd_session_get_uid, iFpp)
GO(sd_session_get_username, iFpp)
GO(sd_session_get_vt, iFpp)
GO(sd_session_is_active, iFp)
GO(sd_session_is_remote, iFp)
GO(sd_uid_get_display, iFup)
GO(sd_uid_get_login_time, iFup)
GO(sd_uid_get_seats, iFuip)
GO(sd_uid_get_sessions, iFuip)
GO(sd_uid_get_state, iFup)
GO(sd_uid_is_on_seat, iFuip)
GO(sd_varlink_attach_event, iFppI)
//GOM(sd_varlink_bind_reply, iFEpp)
GO(sd_varlink_call, iFppppp)
//GOM(sd_varlink_callb, iFEppppV)
//GOM(sd_varlink_callb_ap, iFEpppppA)
//GOM(sd_varlink_callb_full, iFEpppppV)
GO(sd_varlink_call_full, iFpppppp)
GO(sd_varlink_close, iFp)
GO(sd_varlink_close_unref, pFp)
GO(sd_varlink_collect, iFppppp)
//GOM(sd_varlink_collectb, iFEppppV)
GO(sd_varlink_collect_full, iFpppppp)
GO(sd_varlink_connect_address, iFpp)
GO(sd_varlink_connect_exec, iFppp)
GO(sd_varlink_connect_fd, iFpi)
GO(sd_varlink_connect_fd_pair, iFpiip)
GO(sd_varlink_connect_url, iFpp)
GO(sd_varlink_detach_event, vFp)
//GOM(sd_varlink_dispatch, iFEpppp)
GO(sd_varlink_dispatch_again, iFp)
GO(sd_varlink_error, iFppp)
//GOM(sd_varlink_errorb, iFEppV)
GO(sd_varlink_error_errno, iFpi)
GO(sd_varlink_error_invalid_parameter, iFpp)
GO(sd_varlink_error_invalid_parameter_name, iFpp)
GO(sd_varlink_error_is_invalid_parameter, iFppp)
GO(sd_varlink_error_to_errno, iFpp)
GO(sd_varlink_flush, iFp)
GO(sd_varlink_flush_close_unref, pFp)
GO(sd_varlink_get_current_method, iFpp)
GO(sd_varlink_get_current_parameters, iFpp)
GO(sd_varlink_get_description, pFp)
GO(sd_varlink_get_event, pFp)
GO(sd_varlink_get_events, iFp)
GO(sd_varlink_get_fd, iFp)
GO(sd_varlink_get_input_fd, iFp)
GO(sd_varlink_get_n_fds, iFp)
GO(sd_varlink_get_output_fd, iFp)
GO(sd_varlink_get_peer_gid, iFpp)
GO(sd_varlink_get_peer_pid, iFpp)
GO(sd_varlink_get_peer_pidfd, iFp)
GO(sd_varlink_get_peer_uid, iFpp)
GO(sd_varlink_get_server, pFp)
GO(sd_varlink_get_timeout, iFpp)
GO(sd_varlink_get_userdata, pFp)
//GOM(sd_varlink_idl_dump, iFESpIL)
//GOM(sd_varlink_idl_format, iFEpp)
//GOM(sd_varlink_idl_format_full, iFEpILp)
//GOM(sd_varlink_idl_parse, iFEpppp)
//GOM(sd_varlink_interface_free, pFEp)
GO(sd_varlink_invocation, iFI)
GO(sd_varlink_invoke, iFppp)
//GOM(sd_varlink_invokeb, iFEppV)
GO(sd_varlink_is_connected, iFp)
GO(sd_varlink_is_idle, iFp)
GO(sd_varlink_notify, iFpp)
//GOM(sd_varlink_notifyb, iFEpV)
GO(sd_varlink_observe, iFppp)
//GOM(sd_varlink_observeb, iFEppV)
GO(sd_varlink_peek_dup_fd, iFpL)
GO(sd_varlink_peek_fd, iFpL)
GO(sd_varlink_process, iFp)
GO(sd_varlink_push_dup_fd, iFpi)
GO(sd_varlink_push_fd, iFpi)
GO(sd_varlink_ref, pFp)
GO(sd_varlink_reply, iFpp)
//GOM(sd_varlink_replyb, iFEpV)
GO(sd_varlink_reset_fds, iFp)
GO(sd_varlink_send, iFppp)
//GOM(sd_varlink_sendb, iFEppV)
GO(sd_varlink_server_add_connection, iFpip)
GO(sd_varlink_server_add_connection_pair, iFpiipp)
GO(sd_varlink_server_add_connection_stdio, iFpp)
//GOM(sd_varlink_server_add_interface, iFEpp)
//GOM(sd_varlink_server_add_interface_many_internal, iFEpV)
GO(sd_varlink_server_attach_event, iFppI)
//GOM(sd_varlink_server_bind_connect, iFEpp)
//GOM(sd_varlink_server_bind_disconnect, iFEpp)
//GOM(sd_varlink_server_bind_method, iFEppp)
//GOM(sd_varlink_server_bind_method_many_internal, iFEpV)
GO(sd_varlink_server_connections_max, uFp)
GO(sd_varlink_server_connections_per_uid_max, uFp)
GO(sd_varlink_server_current_connections, uFp)
GO(sd_varlink_server_detach_event, iFp)
GO(sd_varlink_server_get_event, pFp)
GO(sd_varlink_server_get_userdata, pFp)
GO(sd_varlink_server_listen_address, iFppu)
GO(sd_varlink_server_listen_auto, iFp)
GO(sd_varlink_server_listen_fd, iFpi)
GO(sd_varlink_server_listen_name, iFpp)
GO(sd_varlink_server_loop_auto, iFp)
GO(sd_varlink_server_new, iFpI)
GO(sd_varlink_server_ref, pFp)
GO(sd_varlink_server_set_connections_max, iFpu)
GO(sd_varlink_server_set_connections_per_uid_max, iFpu)
GO(sd_varlink_server_set_description, iFpp)
GO(sd_varlink_server_set_exit_on_idle, iFpi)
GO(sd_varlink_server_set_info, iFppppp)
GO(sd_varlink_server_set_userdata, pFpp)
GO(sd_varlink_server_shutdown, iFp)
GO(sd_varlink_server_unref, pFp)
GO(sd_varlink_set_allow_fd_passing_input, iFpi)
GO(sd_varlink_set_allow_fd_passing_output, iFpi)
GO(sd_varlink_set_description, iFpp)
GO(sd_varlink_set_input_sensitive, iFp)
GO(sd_varlink_set_relative_timeout, iFpU)
GO(sd_varlink_set_userdata, pFpp)
GO(sd_varlink_take_fd, iFpL)
GO(sd_varlink_unref, pFp)
GO(sd_varlink_wait, iFpU)
GO(sd_watchdog_enabled, iFip)
