#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(libssh2_agent_connect, iFp)
GO(libssh2_agent_disconnect, iFp)
GO(libssh2_agent_free, vFp)
GO(libssh2_agent_get_identity, iFppp)
GO(libssh2_agent_get_identity_path, pFp)
GO(libssh2_agent_init, pFp)
GO(libssh2_agent_list_identities, iFp)
GO(libssh2_agent_set_identity_path, vFpp)
GO(libssh2_agent_sign, iFpppppLpu)
GO(libssh2_agent_userauth, iFppp)
GO(libssh2_banner_set, iFpp)
GO(libssh2_base64_decode, iFppppu)
GO(libssh2_channel_close, iFp)
GO(libssh2_channel_direct_streamlocal_ex, pFpppi)
GO(libssh2_channel_direct_tcpip_ex, pFppipi)
GO(libssh2_channel_eof, iFp)
GO(libssh2_channel_flush_ex, iFpi)
GO(libssh2_channel_forward_accept, pFp)
GO(libssh2_channel_forward_cancel, iFp)
GO(libssh2_channel_forward_listen_ex, pFppipi)
GO(libssh2_channel_free, iFp)
GO(libssh2_channel_get_exit_signal, iFppppppp)
GO(libssh2_channel_get_exit_status, iFp)
GO(libssh2_channel_handle_extended_data, vFpi)
GO(libssh2_channel_handle_extended_data2, iFpi)
GO(libssh2_channel_open_ex, pFppuuupu)
GO(libssh2_channel_process_startup, iFppupu)
GO(libssh2_channel_read_ex, lFpipL)
GO(libssh2_channel_receive_window_adjust, LFpLC)
GO(libssh2_channel_receive_window_adjust2, iFpLCp)
GO(libssh2_channel_request_auth_agent, iFp)
GO(libssh2_channel_request_pty_ex, iFppupuiiii)
GO(libssh2_channel_request_pty_size_ex, iFpiiii)
GO(libssh2_channel_send_eof, iFp)
GO(libssh2_channel_set_blocking, vFpi)
GO(libssh2_channel_setenv_ex, iFppupu)
GO(libssh2_channel_signal_ex, iFppL)
GO(libssh2_channel_wait_closed, iFp)
GO(libssh2_channel_wait_eof, iFp)
GO(libssh2_channel_window_read_ex, LFppp)
GO(libssh2_channel_window_write_ex, LFpp)
GO(libssh2_channel_write_ex, lFpipL)
GO(libssh2_channel_x11_req_ex, iFpippi)
//GO(libssh2_crypt_methods, 
GO(libssh2_crypto_engine, uFv)
GO(libssh2_exit, vFv)
GO(libssh2_free, vFpp)
GO(libssh2_hostkey_hash, pFpi)
//GO(libssh2_hostkey_methods, 
GO(libssh2_init, iFi)
GO(libssh2_keepalive_config, vFpiu)
GO(libssh2_keepalive_send, iFpp)
GO(libssh2_knownhost_add, iFppppLip)
GO(libssh2_knownhost_addc, iFppppLpLip)
GO(libssh2_knownhost_check, iFpppLip)
GO(libssh2_knownhost_checkp, iFppipLip)
GO(libssh2_knownhost_del, iFpp)
GO(libssh2_knownhost_free, vFp)
GO(libssh2_knownhost_get, iFppp)
GO(libssh2_knownhost_init, pFp)
GO(libssh2_knownhost_readfile, iFppi)
GO(libssh2_knownhost_readline, iFppLi)
GO(libssh2_knownhost_writefile, iFppi)
GO(libssh2_knownhost_writeline, iFpppLpi)
GO(libssh2_poll, iFpul)
GO(libssh2_poll_channel_read, iFpi)
//GO(libssh2_publickey_add_ex, 
//GO(libssh2_publickey_init, 
//GO(libssh2_publickey_list_fetch, 
//GO(libssh2_publickey_list_free, 
//GO(libssh2_publickey_remove_ex, 
//GO(libssh2_publickey_shutdown, 
GO(libssh2_scp_recv, pFppp)
GO(libssh2_scp_recv2, pFppp)
GO(libssh2_scp_send64, pFppiIll)
GO(libssh2_scp_send_ex, pFppiLll)
GO(libssh2_session_abstract, pFp)
GO(libssh2_session_banner_get, pFp)
GO(libssh2_session_banner_set, iFpp)
GO(libssh2_session_block_directions, iFp)
GO(libssh2_session_callback_set, pFpip)
GO(libssh2_session_callback_set2, pFpip)
GO(libssh2_session_disconnect_ex, iFpipp)
GO(libssh2_session_flag, iFpii)
GO(libssh2_session_free, iFp)
GO(libssh2_session_get_blocking, iFp)
GO(libssh2_session_get_read_timeout, lFp)
GO(libssh2_session_get_timeout, lFp)
GO(libssh2_session_handshake, iFpi)
GO(libssh2_session_hostkey, pFppp)
GOM(libssh2_session_init_ex, pFEpppp)
GO(libssh2_session_last_errno, iFp)
GO(libssh2_session_last_error, iFpppi)
GO(libssh2_session_method_pref, iFpip)
GO(libssh2_session_methods, pFpi)
GO(libssh2_session_set_blocking, vFpi)
GO(libssh2_session_set_last_error, iFpip)
GO(libssh2_session_set_read_timeout, vFpl)
GO(libssh2_session_set_timeout, vFpl)
GO(libssh2_session_startup, iFpi)
GO(libssh2_session_supported_algs, iFpip)
GO(libssh2_sftp_close_handle, iFp)
//GO(libssh2_sftp_dtor, 
GO(libssh2_sftp_fstat_ex, iFppi)
GO(libssh2_sftp_fstatvfs, iFpp)
GO(libssh2_sftp_fsync, iFp)
GO(libssh2_sftp_get_channel, pFp)
GO(libssh2_sftp_init, pFp)
GO(libssh2_sftp_last_error, LFp)
GO(libssh2_sftp_mkdir_ex, iFppul)
GO(libssh2_sftp_open_ex, pFppuLli)
GO(libssh2_sftp_open_ex_r, pFppLLlip)
GO(libssh2_sftp_posix_rename_ex, iFppLpL)
GO(libssh2_sftp_read, lFppL)
GO(libssh2_sftp_readdir_ex, iFppLpLp)
GO(libssh2_sftp_rename_ex, iFppupul)
GO(libssh2_sftp_rmdir_ex, iFppu)
GO(libssh2_sftp_seek, vFpL)
GO(libssh2_sftp_seek64, vFpU)
GO(libssh2_sftp_shutdown, iFp)
GO(libssh2_sftp_stat_ex, iFppuip)
GO(libssh2_sftp_statvfs, iFppLp)
GO(libssh2_sftp_symlink_ex, iFppupui)
GO(libssh2_sftp_tell, LFp)
GO(libssh2_sftp_tell64, UFp)
GO(libssh2_sftp_unlink_ex, iFppu)
GO(libssh2_sftp_write, lFppL)
GO(libssh2_sign_sk, iFppppLp)
GO(libssh2_trace, iFpi)
//GOM(libssh2_trace_sethandler, iFEppp)
GO(libssh2_userauth_authenticated, iFp)
GO(libssh2_userauth_banner, iFpp)
GO(libssh2_userauth_hostbased_fromfile_ex, iFppuppppupu)
GOM(libssh2_userauth_keyboard_interactive_ex, iFEppup)
GO(libssh2_userauth_list, pFppu)
GOM(libssh2_userauth_password_ex, iFEppupup)
//GOM(libssh2_userauth_publickey, iFEpppLpp)
GO(libssh2_userauth_publickey_fromfile_ex, iFppuppp)
GO(libssh2_userauth_publickey_frommemory, iFppLpLpLp)
//GOM(libssh2_userauth_publickey_sk, iFEppLpLpLppp)
GO(libssh2_version, pFi)
