/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibxxf86vmTYPES32_H_
#define __wrappedlibxxf86vmTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef int32_t (*iFXipp_t)(void*, int32_t, void*, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(XF86VidModeGetAllModeLines, iFXipp_t)

#endif // __wrappedlibxxf86vmTYPES32_H_
