#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh!
#endif

GO(SDLNet_AddSocket, iFpp)
GO(SDLNet_AllocPacket, pFi)
GO(SDLNet_AllocPacketV, pFii)
GO(SDLNet_AllocSocketSet, pFi)
GO(SDLNet_CheckSockets, iFpu)
GO(SDLNet_DelSocket, iFpp)
GO(SDLNet_FreePacket, vFp)
GO(SDLNet_FreePacketV, vFp)
GO(SDLNet_FreeSocketSet, vFp)
GO(SDLNet_GetError, pFv)
//GO(SDLNet_GetLastError, 
GO(SDLNet_GetLocalAddresses, iFpi)
GO(SDLNet_Init, iFv)
GO(SDLNet_Linked_Version, pFv)
GO(SDLNet_Quit, vFv)
GO(SDLNet_Read16, WFp)
GO(SDLNet_Read32, uFp)
GO(SDLNet_ResizePacket, iFpi)
GO(SDLNet_ResolveHost, iFppW)
GO(SDLNet_ResolveIP, pFp)
//GOM(SDLNet_SetError, vFEpV)
//GO(SDLNet_SetLastError, 
GO(SDLNet_TCP_Accept, pFp)
GO(SDLNet_TCP_Close, vFp)
GO(SDLNet_TCP_GetPeerAddress, pFp)
GO(SDLNet_TCP_Open, pFp)
GO(SDLNet_TCP_Recv, iFppi)
GO(SDLNet_TCP_Send, iFppi)
GO(SDLNet_UDP_Bind, iFpip)
GO(SDLNet_UDP_Close, vFp)
GO(SDLNet_UDP_GetPeerAddress, pFpi)
GO(SDLNet_UDP_Open, pFW)
GO(SDLNet_UDP_Recv, iFpp)
GO(SDLNet_UDP_RecvV, iFpp)
GO(SDLNet_UDP_Send, iFpip)
GO(SDLNet_UDP_SendV, iFppi)
GO(SDLNet_UDP_SetPacketLoss, vFpi)
GO(SDLNet_UDP_Unbind, vFpi)
GO(SDLNet_Write16, vFWp)
GO(SDLNet_Write32, vFup)
GO(SDLNet_version, pFv)
