/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedtcmallocminimalTYPES32_H_
#define __wrappedtcmallocminimalTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void* (*pFp_t)(void*);
typedef int32_t (*iFpL_t)(void*, uintptr_t);
typedef void* (*pFpLiiii_t)(void*, uintptr_t, int32_t, int32_t, int32_t, int32_t);
typedef void* (*pFpLiiiI_t)(void*, uintptr_t, int32_t, int32_t, int32_t, int64_t);

#define SUPER() ADDED_FUNCTIONS() \
	GO(munmap, iFpL_t) \
	GO(mmap, pFpLiiii_t) \
	GO(mmap64, pFpLiiiI_t)

#endif // __wrappedtcmallocminimalTYPES32_H_
