/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibxcursorTYPES32_H_
#define __wrappedlibxcursorTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void (*vFp_t)(void*);
typedef void* (*pFi_t)(int32_t);
typedef uintptr_t (*LFXp_t)(void*, void*);
typedef void* (*pFii_t)(int32_t, int32_t);
typedef void* (*pFXi_t)(void*, int32_t);
typedef void* (*pFXp_t)(void*, void*);
typedef void* (*pFupi_t)(uint32_t, void*, int32_t);
typedef void* (*pFppi_t)(void*, void*, int32_t);

#define SUPER() ADDED_FUNCTIONS() \
	GO(XcursorCursorsDestroy, vFp_t) \
	GO(XcursorImageDestroy, vFp_t) \
	GO(XcursorImagesDestroy, vFp_t) \
	GO(XcursorImagesCreate, pFi_t) \
	GO(XcursorImageLoadCursor, LFXp_t) \
	GO(XcursorImagesLoadCursor, LFXp_t) \
	GO(XcursorImageCreate, pFii_t) \
	GO(XcursorCursorsCreate, pFXi_t) \
	GO(XcursorImagesLoadCursors, pFXp_t) \
	GO(XcursorShapeLoadImages, pFupi_t) \
	GO(XcursorLibraryLoadImages, pFppi_t)

#endif // __wrappedlibxcursorTYPES32_H_
