#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(XauDisposeAuth, vFp)
GO(XauFileName, pFv)
GO(XauGetAuthByAddr, pFuupupup)
GO(XauGetBestAuthByAddr, pFuupupipp)
GO(XauLockAuth, iFpiil)
GO(XauReadAuth, pFS)
GO(XauUnlockAuth, iFp)
GO(XauWriteAuth, iFSp)
