#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(XRRQueryVersion, iFXpp)
GO(XRRFreeScreenConfigInfo, vFp)
GOM(XRRGetCrtcInfo, pFEXpL)
GOM(XRRListOutputProperties, pFEXLp)
GOM(XRRQueryExtension, iFEXpp)
//GO(XRRAllocModeInfo, bLuuLuuuuuuupuL_Fpi)
//GO(XRRGetProviderProperty, iFXLLlliiLbL_pbL_bL_bp_)
//GO(XRRListProviderProperties, bL_FXLp)
GO(XRRRotations, WFXip)
GO(XRRSetCrtcTransform, vFXLpppi)
GO(XRRGetCrtcGammaSize, iFXL)
GO(XRRConfigRotations, WFpp)
GOM(XRRGetProviderInfo, pFEXpL)
GO(XRRConfigureOutputProperty, vFXLLiiibl_)
GO(XRRSetOutputPrimary, vFXLL)
GO(XRRFreeOutputInfo, vFp)
GOM(XRRUpdateConfiguration, iFEp)
GOM(XRRGetScreenResources, pFEXL)
GO(XRRConfigSizes, pFpp)
//GO(XRRQueryProviderProperty, biiiibl__FXLL)
GO(XRRDeleteOutputProperty, vFXLL)
GO(XRRDeleteProviderProperty, vFXLL)
GO(XRRSetScreenConfig, iFXpLiWL)
//GO(XRRAllocGamma, bippp_Fi)
GO(XRRSetScreenSize, vFXLiiii)
GO(XRRSetScreenConfigAndRate, iFXpLiWwL)
GO(XRRFreeScreenResources, vFp)
GO(XRRGetOutputPrimary, LFXL)
//GO(XRRCreateMode, LFXLbLuuLuuuuuuupuL_)
GO(XRRConfigCurrentRate, wFp)
GO(XRRDestroyMode, vFXL)
GOM(XRRSetCrtcConfig, iFEXpLLiiLWpi)
GO(XRRConfigCurrentConfiguration, WFpp)
GO(XRRSizes, pFXip)
GO(XRRAddOutputMode, vFXLL)
GO(XRRFreePanning, vFp)
GOM(XRRFreeProviderResources, vFEp)
GO(XRRChangeProviderProperty, vFXLLLiipi)
GOM(XRRGetPanning, pFEXpL)
GO(XRRSetProviderOffloadSink, iFXLL)
GOM(XRRGetScreenResourcesCurrent, pFEXL)
GO(XRRConfigTimes, LFpbL_)
//GO(XRRSetCrtcGamma, vFXLbippp_)
GO(XRRSetProviderOutputSource, iFXLL)
GO(XRRGetScreenSizeRange, iFXLpppp)
GO(XRRRates, pFXiip)
GOM(XRRFreeProviderInfo, vFEp)
GO(XRRConfigRates, pFpip)
GOM(XRRQueryOutputProperty, pFEXLL)
GO(XRRGetOutputProperty, iFXLLlliiLBL_pBL_BL_Bp_)
//GO(XRRFreeGamma, vFbippp_)
GO(XRRRootToScreen, iFXL)
GO(XRRGetScreenInfo, pFXL)
GO(XRRFreeCrtcInfo, vFp)
GOM(XRRGetProviderResources, pFEXL)
//GO(XRRFreeModeInfo, vFbLuuLuuuuuuupuL_)
GO(XRRChangeOutputProperty, vFXLLLiipi)
//GO(XRRGetCrtcGamma, bippp_FXL)
GOM(XRRSetPanning, iFEXpLp)
GO(XRRSelectInput, vFXLi)
GOM(XRRGetCrtcTransform, iFEXLp)
//GO(XRRTimes, LFXibL_)
GO(XRRDeleteOutputMode, vFXLL)
GOM(XRRGetOutputInfo, pFEXpL)
GO(XRRConfigureProviderProperty, vFXLLiiibl_)
//GO(XRRAllocateMonitor, bLiiiiiiiiibL__FXi)
GOM(XRRGetMonitors, pFEXLip)
//GO(XRRSetMonitor, vFXLbLiiiiiiiiibL__)
GOM(XRRFreeMonitors, vFEp)
