#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(gst_fft_f32_fft, vFppp)
GO(gst_fft_f32_free, vFp)
GO(gst_fft_f32_inverse_fft, vFppp)
GO(gst_fft_f32_new, pFii)
GO(gst_fft_f32_window, vFppu)
GO(gst_fft_f64_fft, vFppp)
GO(gst_fft_f64_free, vFp)
GO(gst_fft_f64_inverse_fft, vFppp)
GO(gst_fft_f64_new, pFii)
GO(gst_fft_f64_window, vFppu)
GO(gst_fft_next_fast_length, iFi)
GO(gst_fft_s16_fft, vFppp)
GO(gst_fft_s16_free, vFp)
GO(gst_fft_s16_inverse_fft, vFppp)
GO(gst_fft_s16_new, pFii)
GO(gst_fft_s16_window, vFppu)
GO(gst_fft_s32_fft, vFppp)
GO(gst_fft_s32_free, vFp)
GO(gst_fft_s32_inverse_fft, vFppp)
GO(gst_fft_s32_new, pFii)
GO(gst_fft_s32_window, vFppu)
