#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(buffer_free, vFp)
GO(buffer_get, pFp)
GO(buffer_get_len, uFp)
GO(buffer_new, pFv)
GO(channel_accept_x11, pFpi)
GO(channel_change_pty_size, iFpii)
GO(channel_close, iFp)
GO(channel_forward_accept, pFpi)
GO(channel_forward_cancel, iFppi)
GO(channel_forward_listen, iFppip)
GO(channel_free, vFp)
GO(channel_get_exit_status, iFp)
GO(channel_get_session, pFp)
GO(channel_is_closed, iFp)
GO(channel_is_eof, iFp)
GO(channel_is_open, iFp)
GO(channel_new, pFp)
GO(channel_open_forward, iFppipi)
GO(channel_open_session, iFp)
GO(channel_poll, iFpi)
GO(channel_read, iFppui)
GO(channel_read_buffer, iFppui)
GO(channel_read_nonblocking, iFppui)
GO(channel_request_env, iFppp)
GO(channel_request_exec, iFpp)
GO(channel_request_pty, iFp)
GO(channel_request_pty_size, iFppii)
GO(channel_request_send_signal, iFpp)
GO(channel_request_sftp, iFp)
GO(channel_request_shell, iFp)
GO(channel_request_subsystem, iFpp)
GO(channel_request_x11, iFpippi)
GO(channel_select, iFpppp)
GO(channel_send_eof, iFp)
GO(channel_set_blocking, vFpi)
GO(channel_write, iFppu)
GO(channel_write_stderr, iFppu)
GO(privatekey_free, vFp)
GO(privatekey_from_file, pFppip)
GO(publickey_free, vFp)
GO(publickey_from_file, pFppp)
GO(publickey_from_privatekey, pFp)
GO(publickey_to_string, pFp)
GO(sftp_aio_begin_read, lFpLp)
GO(sftp_aio_begin_write, lFppLp)
GO(sftp_aio_free, vFp)
GO(sftp_aio_wait_read, lFppL)
GO(sftp_aio_wait_write, lFp)
GO(sftp_async_read, iFppuu)
GO(sftp_async_read_begin, iFpu)
GO(sftp_attributes_free, vFp)
GO(sftp_canonicalize_path, pFpp)
GO(sftp_channel_default_data_callback, iFpppuip)
GO(sftp_channel_default_subsystem_request, iFpppp)
GO(sftp_chmod, iFppu)
GO(sftp_chown, iFppuu)
GO(sftp_client_message_free, vFp)
GO(sftp_client_message_get_data, pFp)
GO(sftp_client_message_get_filename, pFp)
GO(sftp_client_message_get_flags, uFp)
GO(sftp_client_message_get_submessage, pFp)
GO(sftp_client_message_get_type, CFp)
GO(sftp_client_message_set_filename, vFpp)
GO(sftp_close, iFp)
GO(sftp_closedir, iFp)
GO(sftp_dir_eof, iFp)
GO(sftp_expand_path, pFpp)
GO(sftp_extensions_get_count, uFp)
GO(sftp_extensions_get_data, pFpu)
GO(sftp_extensions_get_name, pFpu)
GO(sftp_extension_supported, iFppp)
GO(sftp_file_set_blocking, vFp)
GO(sftp_file_set_nonblocking, vFp)
GO(sftp_free, vFp)
GO(sftp_fstat, pFp)
GO(sftp_fstatvfs, pFp)
GO(sftp_fsync, iFp)
GO(sftp_get_client_message, pFp)
GO(sftp_get_error, iFp)
GO(sftp_handle, pFpp)
GO(sftp_handle_alloc, pFpp)
GO(sftp_handle_remove, vFpp)
GO(sftp_hardlink, iFppp)
GO(sftp_home_directory, pFpp)
GO(sftp_init, iFp)
GO(sftp_limits, pFp)
GO(sftp_limits_free, vFp)
GO(sftp_lsetstat, iFppp)
GO(sftp_lstat, pFpp)
GO(sftp_mkdir, iFppu)
GO(sftp_new, pFp)
GO(sftp_new_channel, pFpp)
GO(sftp_open, pFppiu)
GO(sftp_opendir, pFpp)
GO(sftp_read, lFppL)
GO(sftp_readdir, pFpp)
GO(sftp_readlink, pFpp)
GO(sftp_rename, iFppp)
GO(sftp_reply_attr, iFpp)
GO(sftp_reply_data, iFppi)
GO(sftp_reply_handle, iFpp)
GO(sftp_reply_name, iFppp)
GO(sftp_reply_names, iFp)
GO(sftp_reply_names_add, iFpppp)
GO(sftp_reply_status, iFpup)
GO(sftp_rewind, vFp)
GO(sftp_rmdir, iFpp)
GO(sftp_seek, iFpu)
GO(sftp_seek64, iFpU)
GO(sftp_send_client_message, iFpp)
GO(sftp_server_free, vFp)
GO(sftp_server_init, iFp)
GO(sftp_server_new, pFpp)
GO(sftp_server_version, iFp)
GO(sftp_setstat, iFppp)
GO(sftp_stat, pFpp)
GO(sftp_statvfs, pFpp)
GO(sftp_statvfs_free, vFp)
GO(sftp_symlink, iFppp)
GO(sftp_tell, LFp)
GO(sftp_tell64, UFp)
GO(sftp_unlink, iFpp)
GO(sftp_utimes, iFppp)
GO(sftp_write, lFppL)
GO(ssh_accept, iFp)
//GOM(ssh_add_channel_callbacks, iFEpp)
GO(ssh_auth_list, iFp)
GO(ssh_basename, pFp)
GO(ssh_bind_accept, iFpp)
GO(ssh_bind_accept_fd, iFppi)
GO(ssh_bind_fd_toaccept, vFp)
GO(ssh_bind_free, vFp)
GO(ssh_bind_get_fd, iFp)
GO(ssh_bind_listen, iFp)
GO(ssh_bind_new, pFv)
GO(ssh_bind_options_parse_config, iFpp)
GO(ssh_bind_options_set, iFpup)
GO(ssh_bind_set_blocking, vFpi)
//GOM(ssh_bind_set_callbacks, iFEppp)
GO(ssh_bind_set_fd, vFpi)
GO(ssh_blocking_flush, iFpi)
GO(ssh_buffer_add_data, iFppu)
GO(ssh_buffer_free, vFp)
GO(ssh_buffer_get, pFp)
GO(ssh_buffer_get_data, uFppu)
GO(ssh_buffer_get_len, uFp)
GO(ssh_buffer_new, pFv)
GO(ssh_buffer_reinit, iFp)
GO(ssh_channel_accept_forward, pFpip)
GO(ssh_channel_accept_x11, pFpi)
GO(ssh_channel_cancel_forward, iFppi)
GO(ssh_channel_change_pty_size, iFpii)
GO(ssh_channel_close, iFp)
GO(ssh_channel_free, vFp)
GO(ssh_channel_get_exit_state, iFpppp)
GO(ssh_channel_get_exit_status, iFp)
GO(ssh_channel_get_session, pFp)
GO(ssh_channel_is_closed, iFp)
GO(ssh_channel_is_eof, iFp)
GO(ssh_channel_is_open, iFp)
GO(ssh_channel_listen_forward, iFppip)
GO(ssh_channel_new, pFp)
GO(ssh_channel_open_auth_agent, iFp)
GO(ssh_channel_open_forward, iFppipi)
GO(ssh_channel_open_forward_port, pFpippp)
GO(ssh_channel_open_forward_unix, iFpppi)
GO(ssh_channel_open_reverse_forward, iFppipi)
GO(ssh_channel_open_session, iFp)
GO(ssh_channel_open_x11, iFppi)
GO(ssh_channel_poll, iFpi)
GO(ssh_channel_poll_timeout, iFpii)
GO(ssh_channel_read, iFppui)
GO(ssh_channel_read_nonblocking, iFppui)
GO(ssh_channel_read_timeout, iFppuii)
GO(ssh_channel_request_auth_agent, iFp)
GO(ssh_channel_request_env, iFppp)
GO(ssh_channel_request_exec, iFpp)
GO(ssh_channel_request_pty, iFp)
GO(ssh_channel_request_pty_size, iFppii)
GO(ssh_channel_request_pty_size_modes, iFppiipL)
GO(ssh_channel_request_send_break, iFpu)
GO(ssh_channel_request_send_exit_signal, iFppipp)
GO(ssh_channel_request_send_exit_status, iFpi)
GO(ssh_channel_request_send_signal, iFpp)
GO(ssh_channel_request_sftp, iFp)
GO(ssh_channel_request_shell, iFp)
GO(ssh_channel_request_subsystem, iFpp)
GO(ssh_channel_request_x11, iFpippi)
GO(ssh_channel_select, iFpppp)
GO(ssh_channel_send_eof, iFp)
GO(ssh_channel_set_blocking, vFpi)
GO(ssh_channel_set_counter, vFpp)
GO(ssh_channel_window_size, uFp)
GO(ssh_channel_write, iFppu)
GO(ssh_channel_write_stderr, iFppu)
GO(ssh_clean_pubkey_hash, vFp)
GO(ssh_connect, iFp)
GO(ssh_connector_free, vFp)
GO(ssh_connector_new, pFp)
GO(ssh_connector_set_in_channel, iFppu)
GO(ssh_connector_set_in_fd, vFpi)
GO(ssh_connector_set_out_channel, iFppu)
GO(ssh_connector_set_out_fd, vFpi)
GO(ssh_copyright, pFv)
GO(ssh_dirname, pFp)
GO(ssh_disconnect, vFp)
GO(ssh_dump_knownhost, pFp)
GO(ssh_event_add_connector, iFpp)
//GOM(ssh_event_add_fd, iFEpiwpp)
GO(ssh_event_add_session, iFpp)
GO(ssh_event_dopoll, iFpi)
GO(ssh_event_free, vFp)
GO(ssh_event_new, pFv)
GO(ssh_event_remove_connector, iFpp)
GO(ssh_event_remove_fd, iFpi)
GO(ssh_event_remove_session, iFpp)
GO(ssh_execute_message_callbacks, iFp)
GO(ssh_finalize, iFv)
GO(ssh_forward_accept, pFpi)
GO(ssh_forward_cancel, iFppi)
GO(ssh_forward_listen, iFppip)
GO(ssh_free, vFp)
GO(ssh_get_cipher_in, pFp)
GO(ssh_get_cipher_out, pFp)
GO(ssh_get_clientbanner, pFp)
GO(ssh_get_disconnect_message, pFp)
GO(ssh_get_error, pFp)
GO(ssh_get_error_code, iFp)
GO(ssh_get_fd, iFp)
GO(ssh_get_fingerprint_hash, pFupL)
GO(ssh_get_hexa, pFpL)
GO(ssh_get_hmac_in, pFp)
GO(ssh_get_hmac_out, pFp)
GO(ssh_get_issue_banner, pFp)
GO(ssh_get_kex_algo, pFp)
//GOM(ssh_get_log_callback, pFEv)
GO(ssh_get_log_level, iFv)
GO(ssh_get_log_userdata, pFv)
GO(ssh_get_openssh_version, iFp)
GO(ssh_getpass, iFppLii)
GO(ssh_get_poll_flags, iFp)
GO(ssh_get_pubkey, pFp)
GO(ssh_get_pubkey_hash, iFpp)
GO(ssh_get_publickey, iFpp)
GO(ssh_get_publickey_hash, iFpupp)
GO(ssh_get_random, iFpii)
GO(ssh_get_serverbanner, pFp)
GO(ssh_get_server_publickey, iFpp)
GO(ssh_get_status, iFp)
GO(ssh_get_version, iFp)
GO(ssh_gssapi_get_creds, pFp)
GO(ssh_gssapi_set_creds, vFpp)
GO(ssh_handle_key_exchange, iFp)
GO(ssh_init, iFv)
GO(ssh_is_blocking, iFp)
GO(ssh_is_connected, iFp)
GO(ssh_is_server_known, iFp)
GO(ssh_key_cmp, iFppu)
GO(ssh_key_dup, pFp)
GO(ssh_key_free, vFp)
GO(ssh_key_is_private, iFp)
GO(ssh_key_is_public, iFp)
GO(ssh_key_new, pFv)
GO(ssh_key_type, uFp)
GO(ssh_key_type_from_name, uFp)
GO(ssh_key_type_to_char, pFu)
GO(ssh_knownhosts_entry_free, vFp)
GO(ssh_known_hosts_parse_line, iFppp)
//GOM(_ssh_log, vFEippV)
//GOM(ssh_log, vFEpipV)
GO(ssh_message_auth_interactive_request, iFpppupp)
GO(ssh_message_auth_kbdint_is_response, iFp)
GO(ssh_message_auth_password, pFp)
GO(ssh_message_auth_pubkey, pFp)
GO(ssh_message_auth_publickey, pFp)
GO(ssh_message_auth_publickey_state, iFp)
GO(ssh_message_auth_reply_pk_ok, iFppp)
GO(ssh_message_auth_reply_pk_ok_simple, iFp)
GO(ssh_message_auth_reply_success, iFpi)
GO(ssh_message_auth_set_methods, iFpi)
GO(ssh_message_auth_user, pFp)
GO(ssh_message_channel_request_channel, pFp)
GO(ssh_message_channel_request_command, pFp)
GO(ssh_message_channel_request_env_name, pFp)
GO(ssh_message_channel_request_env_value, pFp)
GO(ssh_message_channel_request_open_destination, pFp)
GO(ssh_message_channel_request_open_destination_port, iFp)
GO(ssh_message_channel_request_open_originator, pFp)
GO(ssh_message_channel_request_open_originator_port, iFp)
GO(ssh_message_channel_request_open_reply_accept, pFp)
GO(ssh_message_channel_request_open_reply_accept_channel, iFpp)
GO(ssh_message_channel_request_pty_height, iFp)
GO(ssh_message_channel_request_pty_pxheight, iFp)
GO(ssh_message_channel_request_pty_pxwidth, iFp)
GO(ssh_message_channel_request_pty_term, pFp)
GO(ssh_message_channel_request_pty_width, iFp)
GO(ssh_message_channel_request_reply_success, iFp)
GO(ssh_message_channel_request_subsystem, pFp)
GO(ssh_message_channel_request_x11_auth_cookie, pFp)
GO(ssh_message_channel_request_x11_auth_protocol, pFp)
GO(ssh_message_channel_request_x11_screen_number, iFp)
GO(ssh_message_channel_request_x11_single_connection, iFp)
GO(ssh_message_free, vFp)
GO(ssh_message_get, pFp)
GO(ssh_message_global_request_address, pFp)
GO(ssh_message_global_request_port, iFp)
GO(ssh_message_global_request_reply_success, iFpW)
GO(ssh_message_reply_default, iFp)
GO(ssh_message_retrieve, pFpu)
GO(ssh_message_service_reply_success, iFp)
GO(ssh_message_service_service, pFp)
GO(ssh_message_subtype, iFp)
GO(ssh_message_type, iFp)
GO(ssh_mkdir, iFpu)
GO(ssh_new, pFv)
GO(ssh_options_copy, iFpp)
GO(ssh_options_get, iFpup)
GO(ssh_options_getopt, iFppp)
GO(ssh_options_get_port, iFpp)
GO(ssh_options_parse_config, iFpp)
GO(ssh_options_set, iFpup)
GO(ssh_pcap_file_close, iFp)
GO(ssh_pcap_file_free, vFp)
GO(ssh_pcap_file_new, pFv)
GO(ssh_pcap_file_open, iFpp)
GO(ssh_pki_copy_cert_to_privkey, iFpp)
//GOM(ssh_pki_export_privkey_base64, iFEppppp)
//GOM(ssh_pki_export_privkey_base64_format, iFEpppppu)
//GOM(ssh_pki_export_privkey_file, iFEppppp)
//GOM(ssh_pki_export_privkey_file_format, iFEpppppu)
GO(ssh_pki_export_privkey_to_pubkey, iFpp)
GO(ssh_pki_export_pubkey_base64, iFpp)
GO(ssh_pki_export_pubkey_file, iFpp)
GO(ssh_pki_generate, iFuip)
GO(ssh_pki_import_cert_base64, iFpup)
GO(ssh_pki_import_cert_file, iFpp)
//GOM(ssh_pki_import_privkey_base64, iFEppppp)
//GOM(ssh_pki_import_privkey_file, iFEppppp)
GO(ssh_pki_import_pubkey_base64, iFpup)
GO(ssh_pki_import_pubkey_file, iFpp)
GO(ssh_pki_key_ecdsa_name, pFp)
GO(ssh_print_hash, vFupL)
GO(ssh_print_hexa, vFppL)
GO(ssh_privatekey_type, uFp)
GO(ssh_publickey_to_file, iFpppi)
//GOM(ssh_remove_channel_callbacks, iFEpp)
GO(ssh_request_no_more_sessions, iFp)
GO(ssh_scp_accept_request, iFp)
GO(ssh_scp_close, iFp)
GO(ssh_scp_deny_request, iFpp)
GO(ssh_scp_free, vFp)
GO(ssh_scp_init, iFp)
GO(ssh_scp_leave_directory, iFp)
GO(ssh_scp_new, pFpip)
GO(ssh_scp_pull_request, iFp)
GO(ssh_scp_push_directory, iFppi)
GO(ssh_scp_push_file, iFppLi)
GO(ssh_scp_push_file64, iFppUi)
GO(ssh_scp_read, iFppL)
GO(ssh_scp_request_get_filename, pFp)
GO(ssh_scp_request_get_permissions, iFp)
GO(ssh_scp_request_get_size, LFp)
GO(ssh_scp_request_get_size64, UFp)
GO(ssh_scp_request_get_warning, pFp)
GO(ssh_scp_write, iFppL)
GO(ssh_select, iFppipp)
GO(ssh_send_debug, iFppi)
GO(ssh_send_ignore, iFpp)
GO(ssh_send_issue_banner, iFpp)
GO(ssh_send_keepalive, iFp)
GO(ssh_server_init_kex, iFp)
GO(ssh_service_request, iFpp)
GO(ssh_session_export_known_hosts_entry, iFpp)
GO(ssh_session_get_known_hosts_entry, iFpp)
GO(ssh_session_has_known_hosts_entry, iFp)
GO(ssh_session_is_known_server, iFp)
GO(ssh_session_set_disconnect_message, iFpp)
GO(ssh_session_update_known_hosts, iFp)
GO(ssh_set_agent_channel, iFpp)
GO(ssh_set_agent_socket, iFpi)
GO(ssh_set_auth_methods, vFpi)
GO(ssh_set_blocking, vFpi)
//GOM(ssh_set_callbacks, iFEpp)
//GOM(ssh_set_channel_callbacks, iFEpp)
GO(ssh_set_counters, vFppp)
GO(ssh_set_fd_except, vFp)
GO(ssh_set_fd_toread, vFp)
GO(ssh_set_fd_towrite, vFp)
//GOM(ssh_set_log_callback, iFEp)
GO(ssh_set_log_level, iFi)
GO(ssh_set_log_userdata, iFp)
//GOM(ssh_set_message_callback, vFEppp)
GO(ssh_set_pcap_file, iFpp)
//GOM(ssh_set_server_callbacks, iFEpp)
GO(ssh_silent_disconnect, vFp)
GO(ssh_string_burn, vFp)
GO(ssh_string_copy, pFp)
GO(ssh_string_data, pFp)
GO(ssh_string_fill, iFppL)
GO(ssh_string_free, vFp)
GO(ssh_string_free_char, vFp)
GO(ssh_string_from_char, pFp)
GO(ssh_string_get_char, pFp)
GO(ssh_string_len, LFp)
GO(ssh_string_new, pFL)
GO(ssh_string_to_char, pFp)
//GOM(ssh_threads_get_default, pFEv)
//GOM(ssh_threads_get_noop, pFEv)
//GOM(ssh_threads_get_pthread, pFEv)
//GOM(ssh_threads_set_callbacks, iFEp)
GO(ssh_try_publickey_from_file, iFpppp)
GO(ssh_userauth_agent, iFpp)
GO(ssh_userauth_agent_pubkey, iFppp)
GO(ssh_userauth_autopubkey, iFpp)
GO(ssh_userauth_gssapi, iFp)
GO(ssh_userauth_kbdint, iFppp)
GO(ssh_userauth_kbdint_getanswer, pFpu)
GO(ssh_userauth_kbdint_getinstruction, pFp)
GO(ssh_userauth_kbdint_getname, pFp)
GO(ssh_userauth_kbdint_getnanswers, iFp)
GO(ssh_userauth_kbdint_getnprompts, iFp)
GO(ssh_userauth_kbdint_getprompt, pFpup)
GO(ssh_userauth_kbdint_setanswer, iFpup)
GO(ssh_userauth_list, iFpp)
GO(ssh_userauth_none, iFpp)
GO(ssh_userauth_offer_pubkey, iFppip)
GO(ssh_userauth_password, iFppp)
GO(ssh_userauth_privatekey_file, iFpppp)
GO(ssh_userauth_pubkey, iFpppp)
GO(ssh_userauth_publickey, iFppp)
GO(ssh_userauth_publickey_auto, iFppp)
GO(ssh_userauth_publickey_auto_get_current_identity, iFpp)
GO(ssh_userauth_try_publickey, iFppp)
GO(ssh_version, pFi)
GO(ssh_vlog, vFippp)
GO(ssh_write_knownhost, iFp)
GO(string_burn, vFp)
GO(string_copy, pFp)
GO(string_data, pFp)
GO(string_fill, iFppL)
GO(string_free, vFp)
GO(string_from_char, pFp)
GO(string_len, LFp)
GO(string_new, pFL)
GO(string_to_char, pFp)
