#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error meh!
#endif

//GO(notify_get_app_name, 
//GO(notify_get_server_caps, 
//GO(notify_get_server_info, 
//GO(notify_init, 
//GO(notify_is_initted, 
//GO(notify_marshal_VOID__UINT_STRING, 
//GO(notify_marshal_VOID__UINT_UINT, 
//GO(notify_notification_add_action, 
//GO(notify_notification_clear_actions, 
//GO(notify_notification_clear_hints, 
//GO(notify_notification_close, 
//GO(notify_notification_get_closed_reason, 
//GO(notify_notification_get_type, 
//GO(notify_notification_new, 
//GO(notify_notification_set_app_name, 
//GO(notify_notification_set_category, 
//GO(notify_notification_set_hint, 
//GO(notify_notification_set_hint_byte, 
//GO(notify_notification_set_hint_byte_array, 
//GO(notify_notification_set_hint_double, 
//GO(notify_notification_set_hint_int32, 
//GO(notify_notification_set_hint_string, 
//GO(notify_notification_set_hint_uint32, 
//GO(notify_notification_set_icon_from_pixbuf, 
//GO(notify_notification_set_image_from_pixbuf, 
//GO(notify_notification_set_timeout, 
//GO(notify_notification_set_urgency, 
//GO(notify_notification_show, 
//GO(notify_notification_update, 
//GO(notify_set_app_name, 
//GO(notify_uninit, 
//GO(notify_urgency_get_type, 
