#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(XRecordAllocRange, pFv)
GO(XRecordCreateContext, LFpipipi)
GO(XRecordDisableContext, iFpL)
GOM(XRecordEnableContext, iFEpppp)
GOM(XRecordEnableContextAsync, iFEpppp)
GO(XRecordFreeContext, iFpL)
GO(XRecordFreeData, vFp)
GO(XRecordFreeState, vFp)
GO(XRecordGetContext, iFpLp)
GO(XRecordIdBaseMask, uFp)
GO(XRecordProcessReplies, vFp)
GO(XRecordQueryVersion, iFpii)
GO(XRecordRegisterClients, iFpLipipi)
GO(XRecordUnregisterClients, iFpLpi)
GO(XTestCompareCurrentCursorWithWindow, iFpL)
GO(XTestCompareCursorWithWindow, iFpLL)
GO(XTestDiscard, iFp)
GO(XTestFakeButtonEvent, iFpuiL)
GO(XTestFakeDeviceButtonEvent, iFppuipiL)
GO(XTestFakeDeviceKeyEvent, iFppuipiL)
GO(XTestFakeDeviceMotionEvent, iFppiipiL)
GO(XTestFakeKeyEvent, iFpuiL)
GO(XTestFakeMotionEvent, iFpiiiL)
GO(XTestFakeProximityEvent, iFppipiL)
GO(XTestFakeRelativeMotionEvent, iFpiiL)
GO(XTestGrabControl, iFpi)
GO(XTestQueryExtension, iFppppp)
GO(XTestSetGContextOfGC, vFpL)
GO(XTestSetVisualIDOfVisual, vFpL)
