#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(gpgme_addrspec_from_uid, pFp)
GO(gpgme_cancel, uFp)
GO(gpgme_cancel_async, uFp)
GO(gpgme_check_version, pFp)
GO(gpgme_check_version_internal, pFpL)
GO(gpgme_conf_arg_new, uFpup)
GO(gpgme_conf_arg_release, vFpu)
GO(gpgme_conf_opt_change, uFpip)
GO(gpgme_conf_release, vFp)
GO(gpgme_ctx_get_engine_info, pFp)
GO(gpgme_ctx_set_engine_info, uFpupp)
GO(gpgme_data_get_encoding, uFp)
GO(gpgme_data_get_file_name, pFp)
GO(gpgme_data_identify, uFpi)
GO(gpgme_data_new, uFp)
GOM(gpgme_data_new_from_cbs, uFEppp)
GO(gpgme_data_new_from_estream, uFpp)
GO(gpgme_data_new_from_fd, uFpi)
GO(gpgme_data_new_from_file, uFppi)
GO(gpgme_data_new_from_filepart, uFppSlL)
GO(gpgme_data_new_from_mem, uFppLi)
GO(gpgme_data_new_from_stream, uFpS)
//GOM(gpgme_data_new_with_read_cb, uFEppp)
GO(gpgme_data_read, lFppL)
GO(gpgme_data_release, vFp)
GO(gpgme_data_release_and_get_mem, pFpp)
GO(gpgme_data_rewind, uFp)
GO(gpgme_data_seek, lFpli)
GO(gpgme_data_set_encoding, uFpu)
GO(gpgme_data_set_file_name, uFpp)
GO(gpgme_data_set_flag, uFppp)
GO(gpgme_data_write, lFppL)
GO(gpgme_engine_check_version, uFu)
GO(gpgme_err_code_from_errno, uFi)
GO(gpgme_err_code_from_syserror, uFv)
GO(gpgme_err_code_to_errno, iFu)
GO(gpgme_err_make_from_errno, uFui)
GO(gpgme_error_from_errno, uFi)
GO(gpgme_err_set_errno, vFi)
GO(gpgme_free, vFp)
GO(gpgme_get_armor, iFp)
GO(gpgme_get_ctx_flag, pFpp)
GO(gpgme_get_dirinfo, pFp)
GO(gpgme_get_engine_info, uFp)
GO(gpgme_get_include_certs, iFp)
//GOM(gpgme_get_io_cbs, vFEpp)
GO(gpgme_get_key, uFpppi)
GO(gpgme_get_keylist_mode, uFp)
GO(gpgme_get_offline, iFp)
//GOM(gpgme_get_passphrase_cb, vFEppp)
GO(gpgme_get_pinentry_mode, uFp)
//GOM(gpgme_get_progress_cb, vFEppp)
GO(gpgme_get_protocol, uFp)
GO(gpgme_get_protocol_name, pFu)
GO(gpgme_get_sender, pFp)
GO(gpgme_get_sig_key, uFpip)
GO(gpgme_get_sig_status, pFpipp)
//GO(gpgme_get_sig_string_attr, 
//GO(gpgme_get_sig_ulong_attr, 
//GOM(gpgme_get_status_cb, vFEppp)
GO(gpgme_get_sub_protocol, uFp)
GO(gpgme_get_textmode, iFp)
GO(gpgme_hash_algo_name, pFu)
GO(gpgme_io_read, lFipL)
GO(gpgme_io_write, lFipL)
GO(gpgme_io_writen, iFipL)
GO(gpgme_key_from_uid, uFpp)
//GO(gpgme_key_get_string_attr, 
//GO(gpgme_key_get_ulong_attr, 
GO(gpgme_key_ref, vFp)
GO(gpgme_key_release, vFp)
//GO(gpgme_key_sig_get_string_attr, 
//GO(gpgme_key_sig_get_ulong_attr, 
GO(gpgme_key_unref, vFp)
GO(gpgme_new, uFp)
GO(gpgme_op_adduid, uFpppu)
GO(gpgme_op_adduid_start, uFpppu)
GO(gpgme_op_assuan_result, pFp)
//GOM(gpgme_op_assuan_transact, uFEpppppppp)
//GOM(gpgme_op_assuan_transact_ext, uFEppppppppp)
//GOM(gpgme_op_assuan_transact_start, uFEpppppppp)
//GOM(gpgme_op_card_edit, uFEppppp)
//GOM(gpgme_op_card_edit_start, uFEppppp)
GO(gpgme_op_conf_dir, uFppp)
GO(gpgme_op_conf_load, uFpp)
GO(gpgme_op_conf_save, uFpp)
GO(gpgme_op_createkey, uFpppLLpu)
GO(gpgme_op_createkey_start, uFpppLLpu)
GO(gpgme_op_createsubkey, uFpppLLu)
GO(gpgme_op_createsubkey_start, uFpppLLu)
GO(gpgme_op_decrypt, uFppp)
GO(gpgme_op_decrypt_ext, uFpupp)
GO(gpgme_op_decrypt_ext_start, uFpupp)
GO(gpgme_op_decrypt_result, pFp)
GO(gpgme_op_decrypt_start, uFppp)
GO(gpgme_op_decrypt_verify, uFppp)
GO(gpgme_op_decrypt_verify_start, uFppp)
GO(gpgme_op_delete, uFppi)
GO(gpgme_op_delete_ext, uFppu)
GO(gpgme_op_delete_ext_start, uFppu)
GO(gpgme_op_delete_start, uFppi)
//GOM(gpgme_op_edit, uFEppppp)
//GOM(gpgme_op_edit_start, uFEppppp)
GO(gpgme_op_encrypt, uFppupp)
GO(gpgme_op_encrypt_ext, uFpppupp)
GO(gpgme_op_encrypt_ext_start, uFpppupp)
GO(gpgme_op_encrypt_result, pFp)
GO(gpgme_op_encrypt_sign, uFppupp)
GO(gpgme_op_encrypt_sign_ext, uFpppupp)
GO(gpgme_op_encrypt_sign_ext_start, uFpppupp)
GO(gpgme_op_encrypt_sign_start, uFppupp)
GO(gpgme_op_encrypt_start, uFppupp)
GO(gpgme_op_export, uFppup)
GO(gpgme_op_export_ext, uFppup)
GO(gpgme_op_export_ext_start, uFppup)
GO(gpgme_op_export_keys, uFppup)
GO(gpgme_op_export_keys_start, uFppup)
GO(gpgme_op_export_start, uFppup)
GO(gpgme_op_genkey, uFpppp)
GO(gpgme_op_genkey_result, pFp)
GO(gpgme_op_genkey_start, uFpppp)
GO(gpgme_op_getauditlog, uFppu)
GO(gpgme_op_getauditlog_start, uFppu)
GO(gpgme_op_import, uFpp)
GO(gpgme_op_import_ext, uFppp)
GO(gpgme_op_import_keys, uFpp)
GO(gpgme_op_import_keys_start, uFpp)
GO(gpgme_op_import_result, pFp)
GO(gpgme_op_import_start, uFpp)
//GOM(gpgme_op_interact, uFEppuppp)
//GOM(gpgme_op_interact_start, uFEppuppp)
GO(gpgme_op_keylist_end, uFp)
GO(gpgme_op_keylist_ext_start, uFppii)
GO(gpgme_op_keylist_from_data_start, uFppi)
GO(gpgme_op_keylist_next, uFpp)
GO(gpgme_op_keylist_result, pFp)
GO(gpgme_op_keylist_start, uFppi)
GO(gpgme_op_keysign, uFpppLu)
GO(gpgme_op_keysign_start, uFpppLu)
GO(gpgme_op_passwd, uFppu)
GO(gpgme_op_passwd_start, uFppu)
GO(gpgme_op_query_swdb, uFpppu)
GO(gpgme_op_query_swdb_result, pFp)
GO(gpgme_op_receive_keys, uFpp)
GO(gpgme_op_receive_keys_start, uFpp)
GO(gpgme_op_revsig, uFppppu)
GO(gpgme_op_revsig_start, uFppppu)
GO(gpgme_op_revuid, uFpppu)
GO(gpgme_op_revuid_start, uFpppu)
GO(gpgme_op_setexpire, uFppLpu)
GO(gpgme_op_setexpire_start, uFppLpu)
GO(gpgme_op_setownertrust, uFppp)
GO(gpgme_op_setownertrust_start, uFppp)
GO(gpgme_op_set_uid_flag, uFppppp)
GO(gpgme_op_set_uid_flag_start, uFppppp)
GO(gpgme_op_sign, uFpppu)
GO(gpgme_op_sign_result, pFp)
GO(gpgme_op_sign_start, uFpppu)
GO(gpgme_op_spawn, uFppppppu)
GO(gpgme_op_spawn_start, uFppppppu)
GO(gpgme_op_tofu_policy, uFppu)
GO(gpgme_op_tofu_policy_start, uFppu)
//GO(gpgme_op_trustlist_end, 
//GO(gpgme_op_trustlist_next, 
//GO(gpgme_op_trustlist_start, 
GO(gpgme_op_verify, uFpppp)
GO(gpgme_op_verify_ext, uFpuppp)
GO(gpgme_op_verify_ext_start, uFpuppp)
GO(gpgme_op_verify_result, pFp)
GO(gpgme_op_verify_start, uFpppp)
GO(gpgme_op_vfs_create, uFpppup)
GO(gpgme_op_vfs_mount, uFpppup)
GO(gpgme_op_vfs_mount_result, pFp)
GO(gpgme_pubkey_algo_name, pFu)
GO(gpgme_pubkey_algo_string, pFp)
GO(gpgme_release, vFp)
GO(gpgme_result_ref, vFp)
GO(gpgme_result_unref, vFp)
GO(gpgme_set_armor, vFpi)
GO(gpgme_set_ctx_flag, uFppp)
GO(gpgme_set_engine_info, uFupp)
GO(gpgme_set_global_flag, iFpp)
GO(gpgme_set_include_certs, vFpi)
//GOM(gpgme_set_io_cbs, vFEpp)
GO(gpgme_set_keylist_mode, uFpu)
GO(gpgme_set_locale, uFpip)
GO(gpgme_set_offline, vFpi)
//GOM(gpgme_set_passphrase_cb, vFEppp)
GO(gpgme_set_pinentry_mode, uFpu)
//GOM(gpgme_set_progress_cb, vFEppp)
GO(gpgme_set_protocol, uFpu)
GO(gpgme_set_sender, uFpp)
//GOM(gpgme_set_status_cb, vFEppp)
GO(gpgme_set_sub_protocol, uFpu)
GO(gpgme_set_textmode, vFpi)
GO(gpgme_signers_add, uFpp)
GO(gpgme_signers_clear, vFp)
GO(gpgme_signers_count, uFp)
GO(gpgme_signers_enum, pFpi)
GO(gpgme_sig_notation_add, uFpppu)
GO(gpgme_sig_notation_clear, vFp)
GO(gpgme_sig_notation_get, pFp)
GO(gpgme_strerror, pFu)
GO(gpgme_strerror_r, iFupL)
GO(gpgme_strsource, pFu)
//GO(gpgme_trust_item_get_int_attr, 
//GO(gpgme_trust_item_get_string_attr, 
//GO(gpgme_trust_item_ref, 
//GO(gpgme_trust_item_release, 
//GO(gpgme_trust_item_unref, 
GO(gpgme_wait, pFppi)
GO(gpgme_wait_ext, pFpppi)
