#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//%S z my_xcb_cookie_t u
//%S Z my_xcb_XXX_iterator_t pii

//GOS(xcb_alloc_color, pFpnuWWW)
//GOS(xcb_alloc_color_cells, pFpnCuWW)
GO(xcb_alloc_color_cells_masks, pFp)
//GOS(xcb_alloc_color_cells_masks_end, pFpp)
GO(xcb_alloc_color_cells_masks_length, iFp)
GO(xcb_alloc_color_cells_pixels, pFp)
//GOS(xcb_alloc_color_cells_pixels_end, pFpp)
GO(xcb_alloc_color_cells_pixels_length, iFp)
GO(xcb_alloc_color_cells_reply, pFnubp_)
GO(xcb_alloc_color_cells_sizeof, iFp)
//GOS(xcb_alloc_color_cells_unchecked, pFpnCuWW)
//GOS(xcb_alloc_color_planes, pFpnCuWWWW)
GO(xcb_alloc_color_planes_pixels, pFp)
//GOS(xcb_alloc_color_planes_pixels_end, pFpp)
GO(xcb_alloc_color_planes_pixels_length, iFp)
GO(xcb_alloc_color_planes_reply, pFnubp_)
GO(xcb_alloc_color_planes_sizeof, iFp)
//GOS(xcb_alloc_color_planes_unchecked, pFpnCuWWWW)
GO(xcb_alloc_color_reply, pFnubp_)
//GOS(xcb_alloc_color_unchecked, pFpnuWWW)
//GOS(xcb_alloc_named_color, pFpnuWp)
GO(xcb_alloc_named_color_reply, pFnubp_)
GO(xcb_alloc_named_color_sizeof, iFp)
//GOS(xcb_alloc_named_color_unchecked, pFpnuWp)
//GOS(xcb_allow_events, pFpnCu)
//GOS(xcb_allow_events_checked, pFpnCu)
//GO(xcb_arc_end, 
GO(xcb_arc_next, vFbpii_)
//GO(xcb_atom_end, 
GO(xcb_atom_next, vFbpii_)
//GOS(xcb_bell, pFpnc)
//GOS(xcb_bell_checked, pFpnc)
//GO(xcb_big_requests_enable, 
//GO(xcb_big_requests_enable_reply, 
//GO(xcb_big_requests_enable_unchecked, 
DATA(xcb_big_requests_id, 4) // Warning: failed to confirm
//GO(xcb_bool32_end, 
GO(xcb_bool32_next, vFbpii_)
//GO(xcb_button_end, 
GO(xcb_button_next, vFbpii_)
//GOS(xcb_change_active_pointer_grab, pFpnuuW)
//GOS(xcb_change_active_pointer_grab_checked, pFpnuuW)
//GOS(xcb_change_gc, pFpnuup)
//GOS(xcb_change_gc_aux, pFpnuup)
//GOS(xcb_change_gc_aux_checked, pFpnuup)
//GOS(xcb_change_gc_checked, pFpnuup)
GO(xcb_change_gc_sizeof, iFp)
GO(xcb_change_gc_value_list, pFp)
GO(xcb_change_gc_value_list_serialize, iFbp_up)
GO(xcb_change_gc_value_list_sizeof, iFpu)
GO(xcb_change_gc_value_list_unpack, iFpup)
//GOS(xcb_change_hosts, pFpnCCWp)
GO(xcb_change_hosts_address, pFp)
//GOS(xcb_change_hosts_address_end, pFpp)
GO(xcb_change_hosts_address_length, iFp)
//GOS(xcb_change_hosts_checked, pFpnCCWp)
GO(xcb_change_hosts_sizeof, iFp)
//GOS(xcb_change_keyboard_control, pFpnup)
//GOS(xcb_change_keyboard_control_aux, pFpnup)
//GOS(xcb_change_keyboard_control_aux_checked, pFpnup)
//GOS(xcb_change_keyboard_control_checked, pFpnup)
GO(xcb_change_keyboard_control_sizeof, iFp)
GO(xcb_change_keyboard_control_value_list, pFp)
GO(xcb_change_keyboard_control_value_list_serialize, iFbp_up)
GO(xcb_change_keyboard_control_value_list_sizeof, iFpu)
GO(xcb_change_keyboard_control_value_list_unpack, iFpup)
//GOS(xcb_change_keyboard_mapping, pFpnCCCp)
//GOS(xcb_change_keyboard_mapping_checked, pFpnCCCp)
GO(xcb_change_keyboard_mapping_keysyms, pFp)
//GOS(xcb_change_keyboard_mapping_keysyms_end, pFpp)
GO(xcb_change_keyboard_mapping_keysyms_length, iFp)
GO(xcb_change_keyboard_mapping_sizeof, iFp)
//GOS(xcb_change_pointer_control, pFpnwwwCC)
//GOS(xcb_change_pointer_control_checked, pFpnwwwCC)
GOS(xcb_change_property, zFEnCuuuCup)  //%%
//GOS(xcb_change_property_checked, pFpnCuuuCup)
GO(xcb_change_property_data, pFp)
//GOS(xcb_change_property_data_end, pFpp)
GO(xcb_change_property_data_length, iFp)
GO(xcb_change_property_sizeof, iFp)
//GOS(xcb_change_save_set, pFpnCu)
//GOS(xcb_change_save_set_checked, pFpnCu)
//GOS(xcb_change_window_attributes, pFpnuup)
//GOS(xcb_change_window_attributes_aux, pFpnuup)
//GOS(xcb_change_window_attributes_aux_checked, pFpnuup)
//GOS(xcb_change_window_attributes_checked, pFpnuup)
GO(xcb_change_window_attributes_sizeof, iFp)
GO(xcb_change_window_attributes_value_list, pFp)
GO(xcb_change_window_attributes_value_list_serialize, iFbp_up)
GO(xcb_change_window_attributes_value_list_sizeof, iFpu)
GO(xcb_change_window_attributes_value_list_unpack, iFpup)
//GO(xcb_char2b_end, 
GO(xcb_char2b_next, vFbpii_)
//GO(xcb_charinfo_end, 
GO(xcb_charinfo_next, vFbpii_)
//GOS(xcb_circulate_window, pFpnCu)
//GOS(xcb_circulate_window_checked, pFpnCu)
//GOS(xcb_clear_area, pFpnCuwwWW)
//GOS(xcb_clear_area_checked, pFpnCuwwWW)
//GO(xcb_client_message_data_end, 
GO(xcb_client_message_data_next, vFbpii_)
//GOS(xcb_close_font, pFpnu)
//GOS(xcb_close_font_checked, pFpnu)
//GO(xcb_coloritem_end, 
GO(xcb_coloritem_next, vFbpii_)
//GO(xcb_colormap_end, 
GO(xcb_colormap_next, vFbpii_)
//GOS(xcb_configure_window, pFpnuWp)
//GOS(xcb_configure_window_aux, pFpnuWp)
//GOS(xcb_configure_window_aux_checked, pFpnuWp)
//GOS(xcb_configure_window_checked, pFpnuWp)
GO(xcb_configure_window_sizeof, iFp)
GO(xcb_configure_window_value_list, pFp)
GO(xcb_configure_window_value_list_serialize, iFbp_Wp)
GO(xcb_configure_window_value_list_sizeof, iFpW)
GO(xcb_configure_window_value_list_unpack, iFpWp)
GOM(xcb_connect, pFEpp)
GO(xcb_connection_has_error, iFn)
//GO(xcb_connect_to_display_with_auth_info, nFpbipip_p)
//GO(xcb_connect_to_fd, nFibipip_)
//GOS(xcb_convert_selection, pFpnuuuuu)
//GOS(xcb_convert_selection_checked, pFpnuuuuu)
//GOS(xcb_copy_area, pFpnuuuwwwwWW)
//GOS(xcb_copy_area_checked, pFpnuuuwwwwWW)
//GOS(xcb_copy_colormap_and_free, pFpnuu)
//GOS(xcb_copy_colormap_and_free_checked, pFpnuu)
//GOS(xcb_copy_gc, pFpnuuu)
//GOS(xcb_copy_gc_checked, pFpnuuu)
//GOS(xcb_copy_plane, pFpnuuuwwwwWWu)
//GOS(xcb_copy_plane_checked, pFpnuuuwwwwWWu)
//GOS(xcb_create_colormap, pFpnCuuu)
//GOS(xcb_create_colormap_checked, pFpnCuuu)
//GOS(xcb_create_cursor, pFpnuuuWWWWWWWW)
//GOS(xcb_create_cursor_checked, pFpnuuuWWWWWWWW)
//GOS(xcb_create_gc, pFpnuuup)
//GOS(xcb_create_gc_aux, pFpnuuup)
//GOS(xcb_create_gc_aux_checked, pFpnuuup)
//GOS(xcb_create_gc_checked, pFpnuuup)
GO(xcb_create_gc_sizeof, iFp)
GO(xcb_create_gc_value_list, pFp)
GO(xcb_create_gc_value_list_serialize, iFbp_up)
GO(xcb_create_gc_value_list_sizeof, iFpu)
GO(xcb_create_gc_value_list_unpack, iFpup)
//GOS(xcb_create_glyph_cursor, pFpnuuuWWWWWWWW)
//GOS(xcb_create_glyph_cursor_checked, pFpnuuuWWWWWWWW)
//GOS(xcb_create_pixmap, pFpnCuuWW)
//GOS(xcb_create_pixmap_checked, pFpnCuuWW)
GOS(xcb_create_window, zFEpnCuuwwWWWWuup)    //%%
//GOS(xcb_create_window_aux, pFpnCuuwwWWWWuup)
//GOS(xcb_create_window_aux_checked, pFpnCuuwwWWWWuup)
GOS(xcb_create_window_checked, zFEpnCuuwwWWWWuup)    //%%
GO(xcb_create_window_sizeof, iFp)
GO(xcb_create_window_value_list, pFp)
GO(xcb_create_window_value_list_serialize, iFbp_up)
GO(xcb_create_window_value_list_sizeof, iFpu)
GO(xcb_create_window_value_list_unpack, iFpup)
//GO(xcb_cursor_end, 
GO(xcb_cursor_next, vFbpii_)
GOS(xcb_delete_property, zFEnuu)    //%%
//GOS(xcb_delete_property_checked, pFpnuu)
//GO(xcb_depth_end, 
GO(xcb_depth_next, vFbpii_)
GO(xcb_depth_sizeof, iFp)
GO(xcb_depth_visuals, pFp)
//GOS(xcb_depth_visuals_iterator, pFpp)
GO(xcb_depth_visuals_length, iFp)
//GOS(xcb_destroy_subwindows, pFpnu)
//GOS(xcb_destroy_subwindows_checked, pFpnu)
//GOS(xcb_destroy_window, pFpnu)
//GOS(xcb_destroy_window_checked, pFpnu)
GO(xcb_discard_reply, vFnu)
GO(xcb_discard_reply64, vFnU)
GOM(xcb_disconnect, vFEp)
//GO(xcb_drawable_end, 
GO(xcb_drawable_next, vFbpii_)
//GOS(xcb_fill_poly, pFpnuuCCup)
//GOS(xcb_fill_poly_checked, pFpnuuCCup)
GO(xcb_fill_poly_points, pFp)
//GOS(xcb_fill_poly_points_iterator, pFpp)
GO(xcb_fill_poly_points_length, iFp)
GO(xcb_fill_poly_sizeof, iFpu)
GO(xcb_flush, iFn)
//GO(xcb_fontable_end, 
GO(xcb_fontable_next, vFbpii_)
//GO(xcb_font_end, 
GO(xcb_font_next, vFbpii_)
//GO(xcb_fontprop_end, 
GO(xcb_fontprop_next, vFbpii_)
//GOS(xcb_force_screen_saver, pFpnC)
//GOS(xcb_force_screen_saver_checked, pFpnC)
//GO(xcb_format_end, 
GO(xcb_format_next, vFbpii_)
//GOS(xcb_free_colormap, pFpnu)
//GOS(xcb_free_colormap_checked, pFpnu)
//GOS(xcb_free_colors, pFpnuuup)
//GOS(xcb_free_colors_checked, pFpnuuup)
GO(xcb_free_colors_pixels, pFp)
//GOS(xcb_free_colors_pixels_end, pFpp)
GO(xcb_free_colors_pixels_length, iFp)
GO(xcb_free_colors_sizeof, iFpu)
//GOS(xcb_free_cursor, pFpnu)
//GOS(xcb_free_cursor_checked, pFpnu)
//GOS(xcb_free_gc, pFpnu)
//GOS(xcb_free_gc_checked, pFpnu)
//GOS(xcb_free_pixmap, pFpnu)
//GOS(xcb_free_pixmap_checked, pFpnu)
//GO(xcb_gcontext_end, 
GO(xcb_gcontext_next, vFbpii_)
GO(xcb_generate_id, uFn)
//GOS(xcb_get_atom_name, pFpnu)
GO(xcb_get_atom_name_name, pFp)
//GOS(xcb_get_atom_name_name_end, pFpp)
GO(xcb_get_atom_name_name_length, iFp)
GO(xcb_get_atom_name_reply, pFnubp_)
GO(xcb_get_atom_name_sizeof, iFp)
//GOS(xcb_get_atom_name_unchecked, pFpnu)
GO(xcb_get_extension_data, pFnbpi_)
GO(xcb_get_file_descriptor, iFn)
//GOS(xcb_get_font_path, pFpn)
//GOS(xcb_get_font_path_path_iterator, pFpp)
GO(xcb_get_font_path_path_length, iFp)
GO(xcb_get_font_path_reply, pFnubp_)
GO(xcb_get_font_path_sizeof, iFp)
//GOS(xcb_get_font_path_unchecked, pFpn)
//GOS(xcb_get_geometry, pFpnu)
GO(xcb_get_geometry_reply, pFnubp_)
//GOS(xcb_get_geometry_unchecked, pFpnu)
//GOS(xcb_get_image, pFpnCuwwWWu)
GO(xcb_get_image_data, pFp)
//GOS(xcb_get_image_data_end, pFpp)
GO(xcb_get_image_data_length, iFp)
GO(xcb_get_image_reply, pFnubp_)
GO(xcb_get_image_sizeof, iFp)
//GOS(xcb_get_image_unchecked, pFpnCuwwWWu)
//GOS(xcb_get_input_focus, pFpn)
GO(xcb_get_input_focus_reply, pFnubp_)
//GOS(xcb_get_input_focus_unchecked, pFpn)
//GOS(xcb_get_keyboard_control, pFpn)
GO(xcb_get_keyboard_control_reply, pFnubp_)
//GOS(xcb_get_keyboard_control_unchecked, pFpn)
//GOS(xcb_get_keyboard_mapping, pFpnCC)
GO(xcb_get_keyboard_mapping_keysyms, pFp)
//GOS(xcb_get_keyboard_mapping_keysyms_end, pFpp)
GO(xcb_get_keyboard_mapping_keysyms_length, iFp)
GO(xcb_get_keyboard_mapping_reply, pFnubp_)
GO(xcb_get_keyboard_mapping_sizeof, iFp)
//GOS(xcb_get_keyboard_mapping_unchecked, pFpnCC)
GO(xcb_get_maximum_request_length, uFn)
//GOS(xcb_get_modifier_mapping, pFpn)
GO(xcb_get_modifier_mapping_keycodes, pFp)
//GOS(xcb_get_modifier_mapping_keycodes_end, pFpp)
GO(xcb_get_modifier_mapping_keycodes_length, iFp)
GO(xcb_get_modifier_mapping_reply, pFnubp_)
GO(xcb_get_modifier_mapping_sizeof, iFp)
//GOS(xcb_get_modifier_mapping_unchecked, pFpn)
//GOS(xcb_get_motion_events, pFpnuuu)
GO(xcb_get_motion_events_events, pFp)
//GOS(xcb_get_motion_events_events_iterator, pFpp)
GO(xcb_get_motion_events_events_length, iFp)
GO(xcb_get_motion_events_reply, pFnubp_)
GO(xcb_get_motion_events_sizeof, iFp)
//GOS(xcb_get_motion_events_unchecked, pFpnuuu)
//GOS(xcb_get_pointer_control, pFpn)
GO(xcb_get_pointer_control_reply, pFnubp_)
//GOS(xcb_get_pointer_control_unchecked, pFpn)
//GOS(xcb_get_pointer_mapping, pFpn)
GO(xcb_get_pointer_mapping_map, pFp)
//GOS(xcb_get_pointer_mapping_map_end, pFpp)
GO(xcb_get_pointer_mapping_map_length, iFp)
GO(xcb_get_pointer_mapping_reply, pFnubp_)
GO(xcb_get_pointer_mapping_sizeof, iFp)
//GOS(xcb_get_pointer_mapping_unchecked, pFpn)
GOS(xcb_get_property, zFEnCuuuuu)  //%%
GO(xcb_get_property_reply, pFnubp_)
GO(xcb_get_property_sizeof, iFp)
//GOS(xcb_get_property_unchecked, pFpnCuuuuu)
GO(xcb_get_property_value, pFp)
//GOS(xcb_get_property_value_end, pFpp)
GO(xcb_get_property_value_length, iFp)
GO(xcb_get_reply_fds, pFnpL)
//GOS(xcb_get_screen_saver, pFpn)
GO(xcb_get_screen_saver_reply, pFnubp_)
//GOS(xcb_get_screen_saver_unchecked, pFpn)
//GOS(xcb_get_selection_owner, pFpnu)
GO(xcb_get_selection_owner_reply, pFnubp_)
//GOS(xcb_get_selection_owner_unchecked, pFpnu)
GO(xcb_get_setup, pFn)
//GOS(xcb_get_window_attributes, pFpnu)
GO(xcb_get_window_attributes_reply, pFnubp_)
//GOS(xcb_get_window_attributes_unchecked, pFpnu)
//GOS(xcb_grab_button, pFpnCuWCCuuCW)
//GOS(xcb_grab_button_checked, pFpnCuWCCuuCW)
//GOS(xcb_grab_key, pFpnCuWCCC)
//GOS(xcb_grab_keyboard, pFpnCuuCC)
GO(xcb_grab_keyboard_reply, pFnubp_)
//GOS(xcb_grab_keyboard_unchecked, pFpnCuuCC)
//GOS(xcb_grab_key_checked, pFpnCuWCCC)
//GOS(xcb_grab_pointer, pFpnCuWCCuuu)
GO(xcb_grab_pointer_reply, pFnubp_)
//GOS(xcb_grab_pointer_unchecked, pFpnCuWCCuuu)
//GOS(xcb_grab_server, pFpn)
//GOS(xcb_grab_server_checked, pFpn)
GO(xcb_host_address, pFp)
//GOS(xcb_host_address_end, pFpp)
GO(xcb_host_address_length, iFp)
//GO(xcb_host_end, 
GO(xcb_host_next, vFbpii_)
GO(xcb_host_sizeof, iFp)
//GOS(xcb_image_text_16, pFpnCuuwwp)
//GOS(xcb_image_text_16_checked, pFpnCuuwwp)
GO(xcb_image_text_16_sizeof, iFp)
GO(xcb_image_text_16_string, pFp)
//GOS(xcb_image_text_16_string_iterator, pFpp)
GO(xcb_image_text_16_string_length, iFp)
//GOS(xcb_image_text_8, pFpnCuuwwp)
//GOS(xcb_image_text_8_checked, pFpnCuuwwp)
GO(xcb_image_text_8_sizeof, iFp)
GO(xcb_image_text_8_string, pFp)
//GOS(xcb_image_text_8_string_end, pFpp)
GO(xcb_image_text_8_string_length, iFp)
//GOS(xcb_install_colormap, pFpnu)
//GOS(xcb_install_colormap_checked, pFpnu)
GOS(xcb_intern_atom, zFEnCWp)   //%%
GO(xcb_intern_atom_reply, pFnubp_)
GO(xcb_intern_atom_sizeof, iFp)
//GOS(xcb_intern_atom_unchecked, pFpnCWp)
//GO(xcb_keycode32_end, 
GO(xcb_keycode32_next, vFbpii_)
//GO(xcb_keycode_end, 
GO(xcb_keycode_next, vFbpii_)
//GO(xcb_keysym_end, 
GO(xcb_keysym_next, vFbpii_)
//GOS(xcb_kill_client, pFpnu)
//GOS(xcb_kill_client_checked, pFpnu)
//GOS(xcb_list_extensions, pFpn)
//GOS(xcb_list_extensions_names_iterator, pFpp)
GO(xcb_list_extensions_names_length, iFp)
GO(xcb_list_extensions_reply, pFnubp_)
GO(xcb_list_extensions_sizeof, iFp)
//GOS(xcb_list_extensions_unchecked, pFpn)
//GOS(xcb_list_fonts, pFpnWWp)
//GOS(xcb_list_fonts_names_iterator, pFpp)
GO(xcb_list_fonts_names_length, iFp)
GO(xcb_list_fonts_reply, pFnubp_)
GO(xcb_list_fonts_sizeof, iFp)
//GOS(xcb_list_fonts_unchecked, pFpnWWp)
//GOS(xcb_list_fonts_with_info, pFpnWWp)
GO(xcb_list_fonts_with_info_name, pFp)
//GOS(xcb_list_fonts_with_info_name_end, pFpp)
GO(xcb_list_fonts_with_info_name_length, iFp)
GO(xcb_list_fonts_with_info_properties, pFp)
//GOS(xcb_list_fonts_with_info_properties_iterator, pFpp)
GO(xcb_list_fonts_with_info_properties_length, iFp)
GO(xcb_list_fonts_with_info_reply, pFnubp_)
GO(xcb_list_fonts_with_info_sizeof, iFp)
//GOS(xcb_list_fonts_with_info_unchecked, pFpnWWp)
//GOS(xcb_list_hosts, pFpn)
//GOS(xcb_list_hosts_hosts_iterator, pFpp)
GO(xcb_list_hosts_hosts_length, iFp)
GO(xcb_list_hosts_reply, pFnubp_)
GO(xcb_list_hosts_sizeof, iFp)
//GOS(xcb_list_hosts_unchecked, pFpn)
//GOS(xcb_list_installed_colormaps, pFpnu)
GO(xcb_list_installed_colormaps_cmaps, pFp)
//GOS(xcb_list_installed_colormaps_cmaps_end, pFpp)
GO(xcb_list_installed_colormaps_cmaps_length, iFp)
GO(xcb_list_installed_colormaps_reply, pFnubp_)
GO(xcb_list_installed_colormaps_sizeof, iFp)
//GOS(xcb_list_installed_colormaps_unchecked, pFpnu)
//GOS(xcb_list_properties, pFpnu)
GO(xcb_list_properties_atoms, pFp)
//GOS(xcb_list_properties_atoms_end, pFpp)
GO(xcb_list_properties_atoms_length, iFp)
GO(xcb_list_properties_reply, pFnubp_)
GO(xcb_list_properties_sizeof, iFp)
//GOS(xcb_list_properties_unchecked, pFpnu)
//GOS(xcb_lookup_color, pFpnuWp)
GO(xcb_lookup_color_reply, pFnubp_)
GO(xcb_lookup_color_sizeof, iFp)
//GOS(xcb_lookup_color_unchecked, pFpnuWp)
//GOS(xcb_map_subwindows, pFpnu)
//GOS(xcb_map_subwindows_checked, pFpnu)
//GOS(xcb_map_window, pFpnu)
//GOS(xcb_map_window_checked, pFpnu)
//GOS(xcb_no_operation, pFpn)
//GOS(xcb_no_operation_checked, pFpn)
//GOS(xcb_open_font, pFpnuWp)
//GOS(xcb_open_font_checked, pFpnuWp)
GO(xcb_open_font_name, pFp)
//GOS(xcb_open_font_name_end, pFpp)
GO(xcb_open_font_name_length, iFp)
GO(xcb_open_font_sizeof, iFp)
GO(xcb_parse_display, iFpbp_pp)
//GO(xcb_pixmap_end, 
GO(xcb_pixmap_next, vFbpii_)
//GO(xcb_point_end, 
GO(xcb_point_next, vFbpii_)
GO(xcb_poll_for_event, pFn)
GO(xcb_poll_for_queued_event, pFn)
GO(xcb_poll_for_reply, iFnubp_bp_)
GO(xcb_poll_for_reply64, iFnUbp_bp_)
GO(xcb_poll_for_special_event, pFnp)
//GOS(xcb_poly_arc, pFpnuuup)
GO(xcb_poly_arc_arcs, pFp)
//GOS(xcb_poly_arc_arcs_iterator, pFpp)
GO(xcb_poly_arc_arcs_length, iFp)
//GOS(xcb_poly_arc_checked, pFpnuuup)
GO(xcb_poly_arc_sizeof, iFpu)
//GOS(xcb_poly_fill_arc, pFpnuuup)
GO(xcb_poly_fill_arc_arcs, pFp)
//GOS(xcb_poly_fill_arc_arcs_iterator, pFpp)
GO(xcb_poly_fill_arc_arcs_length, iFp)
//GOS(xcb_poly_fill_arc_checked, pFpnuuup)
GO(xcb_poly_fill_arc_sizeof, iFpu)
//GOS(xcb_poly_fill_rectangle, pFpnuuup)
//GOS(xcb_poly_fill_rectangle_checked, pFpnuuup)
GO(xcb_poly_fill_rectangle_rectangles, pFp)
//GOS(xcb_poly_fill_rectangle_rectangles_iterator, pFpp)
GO(xcb_poly_fill_rectangle_rectangles_length, iFp)
GO(xcb_poly_fill_rectangle_sizeof, iFpu)
//GOS(xcb_poly_line, pFpnCuuup)
//GOS(xcb_poly_line_checked, pFpnCuuup)
GO(xcb_poly_line_points, pFp)
//GOS(xcb_poly_line_points_iterator, pFpp)
GO(xcb_poly_line_points_length, iFp)
GO(xcb_poly_line_sizeof, iFpu)
//GOS(xcb_poly_point, pFpnCuuup)
//GOS(xcb_poly_point_checked, pFpnCuuup)
GO(xcb_poly_point_points, pFp)
//GOS(xcb_poly_point_points_iterator, pFpp)
GO(xcb_poly_point_points_length, iFp)
GO(xcb_poly_point_sizeof, iFpu)
//GOS(xcb_poly_rectangle, pFpnuuup)
//GOS(xcb_poly_rectangle_checked, pFpnuuup)
GO(xcb_poly_rectangle_rectangles, pFp)
//GOS(xcb_poly_rectangle_rectangles_iterator, pFpp)
GO(xcb_poly_rectangle_rectangles_length, iFp)
GO(xcb_poly_rectangle_sizeof, iFpu)
//GOS(xcb_poly_segment, pFpnuuup)
//GOS(xcb_poly_segment_checked, pFpnuuup)
GO(xcb_poly_segment_segments, pFp)
//GOS(xcb_poly_segment_segments_iterator, pFpp)
GO(xcb_poly_segment_segments_length, iFp)
GO(xcb_poly_segment_sizeof, iFpu)
//GOS(xcb_poly_text_16, pFpnuuwwup)
//GOS(xcb_poly_text_16_checked, pFpnuuwwup)
GO(xcb_poly_text_16_items, pFp)
//GOS(xcb_poly_text_16_items_end, pFpp)
GO(xcb_poly_text_16_items_length, iFp)
GO(xcb_poly_text_16_sizeof, iFpu)
//GOS(xcb_poly_text_8, pFpnuuwwup)
//GOS(xcb_poly_text_8_checked, pFpnuuwwup)
GO(xcb_poly_text_8_items, pFp)
//GOS(xcb_poly_text_8_items_end, pFpp)
GO(xcb_poly_text_8_items_length, iFp)
GO(xcb_poly_text_8_sizeof, iFpu)
GO(xcb_popcount, iFu)
GO(xcb_prefetch_extension_data, vFnbpi_)
GO(xcb_prefetch_maximum_request_length, vFn)
//GOS(xcb_put_image, pFpnCuuWWwwCCup)
//GOS(xcb_put_image_checked, pFpnCuuWWwwCCup)
GO(xcb_put_image_data, pFp)
//GOS(xcb_put_image_data_end, pFpp)
GO(xcb_put_image_data_length, iFp)
GO(xcb_put_image_sizeof, iFpu)
//GOS(xcb_query_best_size, pFpnCuWW)
GO(xcb_query_best_size_reply, pFnubp_)
//GOS(xcb_query_best_size_unchecked, pFpnCuWW)
//GOS(xcb_query_colors, pFpnuup)
GO(xcb_query_colors_colors, pFp)
//GOS(xcb_query_colors_colors_iterator, pFpp)
GO(xcb_query_colors_colors_length, iFp)
GO(xcb_query_colors_reply, pFnubp_)
GO(xcb_query_colors_sizeof, iFpu)
//GOS(xcb_query_colors_unchecked, pFpnuup)
//GOS(xcb_query_extension, pFpnWp)
GO(xcb_query_extension_reply, pFnubp_)
GO(xcb_query_extension_sizeof, iFp)
//GOS(xcb_query_extension_unchecked, pFpnWp)
//GOS(xcb_query_font, pFpnu)
GO(xcb_query_font_char_infos, pFp)
//GOS(xcb_query_font_char_infos_iterator, pFpp)
GO(xcb_query_font_char_infos_length, iFp)
GO(xcb_query_font_properties, pFp)
//GOS(xcb_query_font_properties_iterator, pFpp)
GO(xcb_query_font_properties_length, iFp)
GO(xcb_query_font_reply, pFnubp_)
GO(xcb_query_font_sizeof, iFp)
//GOS(xcb_query_font_unchecked, pFpnu)
//GOS(xcb_query_keymap, pFpn)
GO(xcb_query_keymap_reply, pFnubp_)
//GOS(xcb_query_keymap_unchecked, pFpn)
//GOS(xcb_query_pointer, pFpnu)
GO(xcb_query_pointer_reply, pFnubp_)
//GOS(xcb_query_pointer_unchecked, pFpnu)
//GOS(xcb_query_text_extents, pFpnuup)
GO(xcb_query_text_extents_reply, pFnubp_)
GO(xcb_query_text_extents_sizeof, iFpu)
//GOS(xcb_query_text_extents_unchecked, pFpnuup)
//GOS(xcb_query_tree, pFpnu)
GO(xcb_query_tree_children, pFp)
//GOS(xcb_query_tree_children_end, pFpp)
GO(xcb_query_tree_children_length, iFp)
GO(xcb_query_tree_reply, pFnubp_)
GO(xcb_query_tree_sizeof, iFp)
//GOS(xcb_query_tree_unchecked, pFpnu)
//GOS(xcb_recolor_cursor, pFpnuWWWWWW)
//GOS(xcb_recolor_cursor_checked, pFpnuWWWWWW)
//GO(xcb_rectangle_end, 
GO(xcb_rectangle_next, vFbpii_)
GO(xcb_register_for_special_xge, pFnbpi_up)
//GOS(xcb_reparent_window, pFpnuuww)
//GOS(xcb_reparent_window_checked, pFpnuuww)
GO(xcb_request_check, pFnu)
//GO(xcb_rgb_end, 
GO(xcb_rgb_next, vFbpii_)
//GOS(xcb_rotate_properties, pFpnuWwp)
GO(xcb_rotate_properties_atoms, pFp)
//GOS(xcb_rotate_properties_atoms_end, pFpp)
GO(xcb_rotate_properties_atoms_length, iFp)
//GOS(xcb_rotate_properties_checked, pFpnuWwp)
GO(xcb_rotate_properties_sizeof, iFp)
//GOS(xcb_screen_allowed_depths_iterator, pFpp)
GO(xcb_screen_allowed_depths_length, iFp)
//GO(xcb_screen_end, 
GO(xcb_screen_next, vFbpii_)
GO(xcb_screen_sizeof, iFp)
//GO(xcb_segment_end, 
GO(xcb_segment_next, vFbpii_)
//GOS(xcb_send_event, pFpnCuup)
//GOS(xcb_send_event_checked, pFpnCuup)
GO(xcb_send_fd, vFni)
GO(xcb_send_request, uFnibpL_bLpCC_)
//GOM(xcb_send_request64, UFEnibpL_bLpCC_)
GO(xcb_send_request_with_fds, uFnibpL_bLpCC_up)
//GOM(xcb_send_request_with_fds64, UFEnibpL_bLpCC_up)
//GOS(xcb_set_access_control, pFpnC)
//GOS(xcb_set_access_control_checked, pFpnC)
//GOS(xcb_set_clip_rectangles, pFpnCuwwup)
//GOS(xcb_set_clip_rectangles_checked, pFpnCuwwup)
GO(xcb_set_clip_rectangles_rectangles, pFp)
//GOS(xcb_set_clip_rectangles_rectangles_iterator, pFpp)
GO(xcb_set_clip_rectangles_rectangles_length, iFp)
GO(xcb_set_clip_rectangles_sizeof, iFpu)
//GOS(xcb_set_close_down_mode, pFpnC)
//GOS(xcb_set_close_down_mode_checked, pFpnC)
//GOS(xcb_set_dashes, pFpnuWWp)
//GOS(xcb_set_dashes_checked, pFpnuWWp)
GO(xcb_set_dashes_dashes, pFp)
//GOS(xcb_set_dashes_dashes_end, pFpp)
GO(xcb_set_dashes_dashes_length, iFp)
GO(xcb_set_dashes_sizeof, iFp)
//GOS(xcb_set_font_path, pFpnWp)
//GOS(xcb_set_font_path_checked, pFpnWp)
//GOS(xcb_set_font_path_font_iterator, pFpp)
GO(xcb_set_font_path_font_length, iFp)
GO(xcb_set_font_path_sizeof, iFp)
//GOS(xcb_set_input_focus, pFpnCuu)
//GOS(xcb_set_input_focus_checked, pFpnCuu)
//GOS(xcb_set_modifier_mapping, pFpnCp)
GO(xcb_set_modifier_mapping_reply, pFnubp_)
GO(xcb_set_modifier_mapping_sizeof, iFp)
//GOS(xcb_set_modifier_mapping_unchecked, pFpnCp)
//GOS(xcb_set_pointer_mapping, pFpnCp)
GO(xcb_set_pointer_mapping_reply, pFnubp_)
GO(xcb_set_pointer_mapping_sizeof, iFp)
//GOS(xcb_set_pointer_mapping_unchecked, pFpnCp)
//GOS(xcb_set_screen_saver, pFpnwwCC)
//GOS(xcb_set_screen_saver_checked, pFpnwwCC)
//GOS(xcb_set_selection_owner, pFpnuuu)
//GOS(xcb_set_selection_owner_checked, pFpnuuu)
//GO(xcb_setup_authenticate_end, 
GO(xcb_setup_authenticate_next, vFbpii_)
GO(xcb_setup_authenticate_reason, pFp)
//GOS(xcb_setup_authenticate_reason_end, pFpp)
GO(xcb_setup_authenticate_reason_length, iFp)
GO(xcb_setup_authenticate_sizeof, iFp)
//GO(xcb_setup_end, 
//GO(xcb_setup_failed_end, 
GO(xcb_setup_failed_next, vFbpii_)
GO(xcb_setup_failed_reason, pFp)
//GOS(xcb_setup_failed_reason_end, pFpp)
GO(xcb_setup_failed_reason_length, iFp)
GO(xcb_setup_failed_sizeof, iFp)
GO(xcb_setup_next, vFbpii_)
GO(xcb_setup_pixmap_formats, pFp)
//GOS(xcb_setup_pixmap_formats_iterator, pFpp)
GO(xcb_setup_pixmap_formats_length, iFp)
GO(xcb_setup_request_authorization_protocol_data, pFp)
//GOS(xcb_setup_request_authorization_protocol_data_end, pFpp)
GO(xcb_setup_request_authorization_protocol_data_length, iFp)
GO(xcb_setup_request_authorization_protocol_name, pFp)
//GOS(xcb_setup_request_authorization_protocol_name_end, pFpp)
GO(xcb_setup_request_authorization_protocol_name_length, iFp)
//GO(xcb_setup_request_end, 
GO(xcb_setup_request_next, vFbpii_)
GO(xcb_setup_request_sizeof, iFp)
GOS(xcb_setup_roots_iterator, ZFEpp)    //%%
GO(xcb_setup_roots_length, iFp)
GO(xcb_setup_sizeof, iFp)
GO(xcb_setup_vendor, pFp)
//GOS(xcb_setup_vendor_end, pFpp)
GO(xcb_setup_vendor_length, iFp)
//GOS(xcb_store_colors, pFpnuup)
//GOS(xcb_store_colors_checked, pFpnuup)
GO(xcb_store_colors_items, pFp)
//GOS(xcb_store_colors_items_iterator, pFpp)
GO(xcb_store_colors_items_length, iFp)
GO(xcb_store_colors_sizeof, iFpu)
//GOS(xcb_store_named_color, pFpnCuuWp)
//GOS(xcb_store_named_color_checked, pFpnCuuWp)
GO(xcb_store_named_color_name, pFp)
//GOS(xcb_store_named_color_name_end, pFpp)
GO(xcb_store_named_color_name_length, iFp)
GO(xcb_store_named_color_sizeof, iFp)
//GO(xcb_str_end, 
GO(xcb_str_name, pFp)
//GOS(xcb_str_name_end, pFpp)
GO(xcb_str_name_length, iFp)
GO(xcb_str_next, vFbpii_)
GO(xcb_str_sizeof, iFp)
GO(xcb_sumof, iFpi)
//GOM(xcb_take_socket, iFEnppip)
//GO(xcb_timecoord_end, 
GO(xcb_timecoord_next, vFbpii_)
//GO(xcb_timestamp_end, 
GO(xcb_timestamp_next, vFbpii_)
//GOS(xcb_translate_coordinates, pFpnuuww)
GO(xcb_translate_coordinates_reply, pFnubp_)
//GOS(xcb_translate_coordinates_unchecked, pFpnuuww)
//GOS(xcb_ungrab_button, pFpnCuW)
//GOS(xcb_ungrab_button_checked, pFpnCuW)
//GOS(xcb_ungrab_key, pFpnCuW)
//GOS(xcb_ungrab_keyboard, pFpnu)
//GOS(xcb_ungrab_keyboard_checked, pFpnu)
//GOS(xcb_ungrab_key_checked, pFpnCuW)
//GOS(xcb_ungrab_pointer, pFpnu)
//GOS(xcb_ungrab_pointer_checked, pFpnu)
//GOS(xcb_ungrab_server, pFpn)
//GOS(xcb_ungrab_server_checked, pFpn)
//GOS(xcb_uninstall_colormap, pFpnu)
//GOS(xcb_uninstall_colormap_checked, pFpnu)
//GOS(xcb_unmap_subwindows, pFpnu)
//GOS(xcb_unmap_subwindows_checked, pFpnu)
//GOS(xcb_unmap_window, pFpnu)
//GOS(xcb_unmap_window_checked, pFpnu)
GO(xcb_unregister_for_special_event, vFnp)
//GO(xcb_visualid_end, 
GO(xcb_visualid_next, vFbpii_)
//GO(xcb_visualtype_end, 
GO(xcb_visualtype_next, vFbpii_)
GO(xcb_wait_for_event, pFn)
GO(xcb_wait_for_reply, pFnubp_)
GO(xcb_wait_for_reply64, pFnUbp_)
GO(xcb_wait_for_special_event, pFnp)
//GOS(xcb_warp_pointer, pFpnuuwwWWww)
//GOS(xcb_warp_pointer_checked, pFpnuuwwWWww)
//GO(xcb_window_end, 
GO(xcb_window_next, vFbpii_)
GO(xcb_writev, iFnbpL_iU)
//GO(xcb_xc_misc_get_version, 
//GO(xcb_xc_misc_get_version_reply, 
//GO(xcb_xc_misc_get_version_unchecked, 
//GO(xcb_xc_misc_get_xid_list, 
//GO(xcb_xc_misc_get_xid_list_ids, 
//GO(xcb_xc_misc_get_xid_list_ids_end, 
//GO(xcb_xc_misc_get_xid_list_ids_length, 
//GO(xcb_xc_misc_get_xid_list_reply, 
//GO(xcb_xc_misc_get_xid_list_sizeof, 
//GO(xcb_xc_misc_get_xid_list_unchecked, 
//GO(xcb_xc_misc_get_xid_range, 
//GO(xcb_xc_misc_get_xid_range_reply, 
//GO(xcb_xc_misc_get_xid_range_unchecked, 
DATA(xcb_xc_misc_id, 4) // Warning: failed to confirm
