#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_76, pFpp)
GO(ubrk_close_76, vFp)
GO(ubrk_countAvailable_76, iFv)
GO(ubrk_current_76, iFp)
GO(ubrk_first_76, iFp)
GO(ubrk_following_76, iFpi)
GO(ubrk_getAvailable_76, pFi)
GO(ubrk_getBinaryRules_76, iFppip)
GO(ubrk_getLocaleByType_76, pFpip)
GO(ubrk_getRuleStatus_76, iFp)
GO(ubrk_getRuleStatusVec_76, iFppip)
GO(ubrk_isBoundary_76, cFpi)
GO(ubrk_last_76, iFp)
GO(ubrk_next_76, iFp)
GO(ubrk_open_76, pFppip)
GO(ubrk_openBinaryRules_76, pFpipip)
GO(ubrk_openRules_76, pFpipipp)
GO(ubrk_preceding_76, iFpi)
GO(ubrk_previous_76, iFp)
GO(ubrk_refreshUText_76, vFppp)
GO(ubrk_safeClone_76, pFpppp)
GO(ubrk_setText_76, vFppip)
GO(ubrk_setUText_76, vFppp)
GO(ubrk_swap_76, iFppipp)

GO(ucnv_getDefaultName_76, pFv)
GO(ucnv_open_76, pFpp)
GO(ucnv_setDefaultName_76, vFp)

GO(ucurr_forLocale_76, iFppip)
GO(ucurr_getName_76, pFppippp)

GO(uloc_canonicalize_76, iFppip)
GO(uloc_countAvailable_76, iFv)
GO(uloc_getAvailable_76, pFi)
GO(uloc_getBaseName_76, iFppip)
GO(uloc_getCharacterOrientation_76, iFpp)
GO(uloc_getCountry_76, iFppip)
GO(uloc_getDefault_76, pFv)
GO(uloc_getDisplayCountry_76, iFpppip)
GO(uloc_getDisplayLanguage_76, iFpppip)
GO(uloc_getDisplayName_76, iFpppip)
GO(uloc_getISO3Country_76, pFp)
GO(uloc_getISO3Language_76, pFp)
GO(uloc_getKeywordValue_76, iFpppip)
GO(uloc_getLanguage_76, iFppip)
GO(uloc_getLCID_76, uFp)
GO(uloc_getName_76, iFppip)
GO(uloc_getParent_76, iFppip)
GO(uloc_setKeywordValue_76, iFpppip)

GO(ures_close_76, vFp)
GO(ures_getByKey_76, pFpppp)
GO(ures_getSize_76, iFp)
GO(ures_getStringByIndex_76, pFpipp)
GO(ures_open_76, pFppp)

GO(uldn_close_76, vFp)
GO(uldn_keyValueDisplayName_76, iFppppip)
GO(uldn_open_76, pFpip)

GO(uenum_close_76, vFp)
GO(uenum_count_76, iFpp)
GO(uenum_next_76, pFppp)

GO(uidna_close_76, vFp)
GO(uidna_nameToASCII_76, iFppipipp)
GO(uidna_nameToUnicode_76, iFppipipp)
GO(uidna_openUTS46_76, pFpp)

GO(unorm2_getNFCInstance_76, pFp)
GO(unorm2_getNFDInstance_76, pFp)
GO(unorm2_getNFKCInstance_76, pFp)
GO(unorm2_getNFKDInstance_76, pFp)
GO(unorm2_isNormalized_76, CFppip)
GO(unorm2_normalize_76, iFppipip)

GO(u_charsToUChars_76, vFppi)
GO(u_getVersion_76, vFp)
GO(u_strcmp_76, iFpp)
GO(u_strcpy_76, pFpp)
GO(u_strlen_76, iFp)
GO(u_strncpy_76, pFppi)
GO(u_strToLower_76, iFpipipp)
GO(u_strToUpper_76, iFpipipp)
GO(u_tolower_76, uFu)
GO(u_toupper_76, uFu)
GO(u_uastrncpy_76, pFppi)
