#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ucal_add_66, vFpiip)
GO(ucal_clear_66, vFp)
GO(ucal_clearField_66, vFpi)
GO(ucal_clone_66, pFpp)
GO(ucal_close_66, vFp)
GO(ucal_countAvailable_66, iFv)
GO(ucal_equivalentTo_66, cFpp)
GO(ucal_get_66, iFpip)
GO(ucal_getAttribute_66, iFpi)
GO(ucal_getAvailable_66, pFi)
GO(ucal_getCanonicalTimeZoneID_66, iFpipipp)
GO(ucal_getDayOfWeekType_66, iFpip)
GO(ucal_getDefaultTimeZone_66, iFpip)
GO(ucal_getDSTSavings_66, iFpp)
GO(ucal_getFieldDifference_66, iFpdip)
GO(ucal_getGregorianChange_66, dFpp)
GO(ucal_getHostTimeZone_66, iFpip)
GO(ucal_getKeywordValuesForLocale_66, pFppCp)
GO(ucal_getLimit_66, iFpiip)
GO(ucal_getLocaleByType_66, pFpip)
GO(ucal_getMillis_66, dFpp)
GO(ucal_getNow_66, dFv)
GO(ucal_getTimeZoneDisplayName_66, iFpippip)
GO(ucal_getTimeZoneID_66, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_66, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_66, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_66, cFpipp)
GO(ucal_getType_66, pFpp)
GO(ucal_getTZDataVersion_66, pFp)
GO(ucal_getWeekendTransition_66, iFpip)
GO(ucal_getWindowsTimeZoneID_66, iFpipip)
GO(ucal_inDaylightTime_66, cFpp)
GO(ucal_isSet_66, cFpi)
GO(ucal_isWeekend_66, cFpdp)
GO(ucal_open_66, pFpipip)
GO(ucal_openCountryTimeZones_66, pFpp)
GO(ucal_openTimeZoneIDEnumeration_66, pFippp)
GO(ucal_openTimeZones_66, pFp)
GO(ucal_roll_66, vFpiip)
GO(ucal_set_66, vFpii)
GO(ucal_setAttribute_66, vFpii)
GO(ucal_setDate_66, vFpiiip)
GO(ucal_setDateTime_66, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_66, vFpp)
GO(ucal_setGregorianChange_66, vFpdp)
GO(ucal_setMillis_66, vFpdp)
GO(ucal_setTimeZone_66, vFppip)

GO(ucol_clone_66, pFpp)
GO(ucol_close_66, vFp)
GO(ucol_closeElements_66, vFp)
GO(ucol_setMaxVariable_66, vFpip)
GO(ucol_getOffset_66, iFp)
GO(ucol_getRules_66, pFpp)
GO(ucol_getSortKey_66, iFppipi)
GO(ucol_getStrength_66, iFp)
GO(ucol_getVersion_66, vFpp)
GO(ucol_next_66, iFpp)
GO(ucol_previous_66, iFpp)
GO(ucol_open_66, pFpp)
GO(ucol_openElements_66, pFppip)
GO(ucol_openRules_66, pFpiiipp)
GO(ucol_safeClone_66, pFpppp)
GO(ucol_setAttribute_66, vFpiip)
GO(ucol_setVariableTop_66, uFppip)
GO(ucol_strcoll_66, iFppipi)

GO(udat_close_66, vFp)
GO(udat_countSymbols_66, iFpi)
GO(udat_getSymbols_66, iFpiipip)
GO(udat_format_66, iFpdpipp)
GO(udat_open_66, pFiippipip)
GO(udat_setCalendar_66, vFpp)
GO(udat_toPattern_66, iFpCpip)

GO(unum_close_66, vFp)
GO(unum_getAttribute_66, iFpi)
GO(unum_getSymbol_66, iFpipip)
GO(unum_open_66, pFipippp)
GO(unum_toPattern_66, iFpCpip)

GO(udatpg_close_66, vFp)
GO(udatpg_getBestPattern_66, iFppipip)
GO(udatpg_open_66, pFpp)

GO(ulocdata_getCLDRVersion_66, vFpp)
GO(ulocdata_getMeasurementSystem_66, iFpp)

GO(usearch_close_66, vFp)
GO(usearch_first_66, iFpp)
GO(usearch_getBreakIterator_66, pFp)
GO(usearch_getMatchedLength_66, iFp)
GO(usearch_last_66, iFpp)
GO(usearch_openFromCollator_66, pFpipippp)
GO(usearch_setPattern_66, vFppip)
GO(usearch_setText_66, vFppip)
