#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error meh!
#endif
//GO(acc_async_test, 
//GO(acc_async_test_all, 
//GO(acc_async_test_all_h_, 
//GO(acc_async_test_h_, 
//GO(acc_async_wait, 
//GO(acc_async_wait_all, 
//GO(acc_attach, 
//GO(acc_attach_async, 
//GO(acc_copyin, 
//GO(acc_copyin_32_h_, 
//GO(acc_copyin_64_h_, 
//GO(acc_copyin_array_h_, 
//GO(acc_copyin_async, 
//GO(acc_copyin_async_32_h_, 
//GO(acc_copyin_async_64_h_, 
//GO(acc_copyin_async_array_h_, 
//GO(acc_copyout, 
//GO(acc_copyout_32_h_, 
//GO(acc_copyout_64_h_, 
//GO(acc_copyout_array_h_, 
//GO(acc_copyout_async, 
//GO(acc_copyout_async_32_h_, 
//GO(acc_copyout_async_64_h_, 
//GO(acc_copyout_async_array_h_, 
//GO(acc_copyout_finalize, 
//GO(acc_copyout_finalize_32_h_, 
//GO(acc_copyout_finalize_64_h_, 
//GO(acc_copyout_finalize_array_h_, 
//GO(acc_copyout_finalize_async, 
//GO(acc_create, 
//GO(acc_create_32_h_, 
//GO(acc_create_64_h_, 
//GO(acc_create_array_h_, 
//GO(acc_create_async, 
//GO(acc_create_async_32_h_, 
//GO(acc_create_async_64_h_, 
//GO(acc_create_async_array_h_, 
//GO(acc_delete, 
//GO(acc_delete_32_h_, 
//GO(acc_delete_64_h_, 
//GO(acc_delete_array_h_, 
//GO(acc_delete_async, 
//GO(acc_delete_async_32_h_, 
//GO(acc_delete_async_64_h_, 
//GO(acc_delete_async_array_h_, 
//GO(acc_delete_finalize, 
//GO(acc_delete_finalize_32_h_, 
//GO(acc_delete_finalize_64_h_, 
//GO(acc_delete_finalize_array_h_, 
//GO(acc_delete_finalize_async, 
//GO(acc_detach, 
//GO(acc_detach_async, 
//GO(acc_detach_finalize, 
//GO(acc_detach_finalize_async, 
//GO(acc_deviceptr, 
//GO(acc_free, 
//GO(acc_get_cuda_stream, 
//GO(acc_get_current_cuda_context, 
//GO(acc_get_current_cuda_device, 
//GO(acc_get_device_num, 
//GO(acc_get_device_num_h_, 
//GO(acc_get_device_type, 
//GO(acc_get_device_type_h_, 
//GO(acc_get_num_devices, 
//GO(acc_get_num_devices_h_, 
//GO(acc_get_property, 
//GO(acc_get_property_h_, 
//GO(acc_get_property_string, 
//GO(acc_get_property_string_h_, 
//GO(acc_hostptr, 
//GO(acc_init, 
//GO(acc_init_h_, 
//GO(acc_is_present, 
//GO(acc_is_present_32_h_, 
//GO(acc_is_present_64_h_, 
//GO(acc_is_present_array_h_, 
//GO(acc_malloc, 
//GO(acc_map_data, 
//GO(acc_memcpy_from_device, 
//GO(acc_memcpy_from_device_async, 
//GO(acc_memcpy_to_device, 
//GO(acc_memcpy_to_device_async, 
//GO(acc_on_device, 
//GO(acc_on_device_h_, 
//GO(acc_pcopyin, 
//GO(acc_pcreate, 
//GO(acc_present_or_copyin, 
//GO(acc_present_or_copyin_32_h_, 
//GO(acc_present_or_copyin_64_h_, 
//GO(acc_present_or_copyin_array_h_, 
//GO(acc_present_or_create, 
//GO(acc_present_or_create_32_h_, 
//GO(acc_present_or_create_64_h_, 
//GO(acc_present_or_create_array_h_, 
//GO(acc_prof_lookup, 
//GO(acc_prof_register, 
//GO(acc_prof_unregister, 
//GO(acc_register_library, 
//GO(acc_set_cuda_stream, 
//GO(acc_set_device_num, 
//GO(acc_set_device_num_h_, 
//GO(acc_set_device_type, 
//GO(acc_set_device_type_h_, 
//GO(acc_shutdown, 
//GO(acc_shutdown_h_, 
//GO(acc_unmap_data, 
//GO(acc_update_device, 
//GO(acc_update_device_32_h_, 
//GO(acc_update_device_64_h_, 
//GO(acc_update_device_array_h_, 
//GO(acc_update_device_async, 
//GO(acc_update_device_async_32_h_, 
//GO(acc_update_device_async_64_h_, 
//GO(acc_update_device_async_array_h_, 
//GO(acc_update_self, 
//GO(acc_update_self_32_h_, 
//GO(acc_update_self_64_h_, 
//GO(acc_update_self_array_h_, 
//GO(acc_update_self_async, 
//GO(acc_update_self_async_32_h_, 
//GO(acc_update_self_async_64_h_, 
//GO(acc_update_self_async_array_h_, 
//GO(acc_wait, 
//GO(acc_wait_all, 
//GO(acc_wait_all_async, 
//GO(acc_wait_all_async_h_, 
//GO(acc_wait_all_h_, 
//GO(acc_wait_async, 
//GO(acc_wait_async_h_, 
//GO(acc_wait_h_, 

//GO(GOMP_alloc, 
GO(GOMP_atomic_end, vFv)
GO(GOMP_atomic_start, vFv)
GO(GOMP_barrier, vFv)
GO(GOMP_barrier_cancel, iFv)
GO(GOMP_cancel, iFii)
//GO(GOMP_cancellation_point, 
GO(GOMP_critical_end, vFv)
GO(GOMP_critical_name_end, vFp)
GO(GOMP_critical_name_start, vFp)
GO(GOMP_critical_start, vFv)
//GO(GOMP_doacross_post, 
//GO(GOMP_doacross_ull_post, 
//GO(GOMP_doacross_ull_wait, 
//GO(GOMP_doacross_wait, 
//GO(GOMP_error, 
//GO(GOMP_free, 
//GO(GOMP_loop_doacross_dynamic_start, 
//GO(GOMP_loop_doacross_guided_start, 
//GO(GOMP_loop_doacross_runtime_start, 
//GO(GOMP_loop_doacross_start, 
//GO(GOMP_loop_doacross_static_start, 
GO(GOMP_loop_dynamic_next, iFpp)
GO(GOMP_loop_dynamic_start, iFllllpp)
GO(GOMP_loop_end, vFv)
//GO(GOMP_loop_end_cancel, 
GO(GOMP_loop_end_nowait, vFv)
//GO(GOMP_loop_guided_next, 
//GO(GOMP_loop_guided_start, 
//GO(GOMP_loop_maybe_nonmonotonic_runtime_next, 
//GO(GOMP_loop_maybe_nonmonotonic_runtime_start, 
//GO(GOMP_loop_nonmonotonic_dynamic_next, 
//GO(GOMP_loop_nonmonotonic_dynamic_start, 
//GO(GOMP_loop_nonmonotonic_guided_next, 
//GO(GOMP_loop_nonmonotonic_guided_start, 
//GO(GOMP_loop_nonmonotonic_runtime_next, 
//GO(GOMP_loop_nonmonotonic_runtime_start, 
GO(GOMP_loop_ordered_dynamic_next, iFpp)
GO(GOMP_loop_ordered_dynamic_start, iFllllpp)
//GO(GOMP_loop_ordered_guided_next, 
//GO(GOMP_loop_ordered_guided_start, 
//GO(GOMP_loop_ordered_runtime_next, 
//GO(GOMP_loop_ordered_runtime_start, 
//GO(GOMP_loop_ordered_start,
//GO(GOMP_loop_ordered_static_next, 
//GO(GOMP_loop_ordered_static_start, 
//GO(GOMP_loop_runtime_next, 
//GO(GOMP_loop_runtime_start, 
//GO(GOMP_loop_start, 
//GO(GOMP_loop_static_next, 
//GO(GOMP_loop_static_start, 
//GO(GOMP_loop_ull_doacross_dynamic_start, 
//GO(GOMP_loop_ull_doacross_guided_start, 
//GO(GOMP_loop_ull_doacross_runtime_start, 
//GO(GOMP_loop_ull_doacross_start, 
//GO(GOMP_loop_ull_doacross_static_start, 
//GO(GOMP_loop_ull_dynamic_next, 
//GO(GOMP_loop_ull_dynamic_start, 
//GO(GOMP_loop_ull_guided_next, 
//GO(GOMP_loop_ull_guided_start, 
//GO(GOMP_loop_ull_maybe_nonmonotonic_runtime_next, 
//GO(GOMP_loop_ull_maybe_nonmonotonic_runtime_start, 
//GO(GOMP_loop_ull_nonmonotonic_dynamic_next, 
//GO(GOMP_loop_ull_nonmonotonic_dynamic_start, 
//GO(GOMP_loop_ull_nonmonotonic_guided_next, 
//GO(GOMP_loop_ull_nonmonotonic_guided_start, 
//GO(GOMP_loop_ull_nonmonotonic_runtime_next, 
//GO(GOMP_loop_ull_nonmonotonic_runtime_start, 
//GO(GOMP_loop_ull_ordered_dynamic_next, 
//GO(GOMP_loop_ull_ordered_dynamic_start, 
//GO(GOMP_loop_ull_ordered_guided_next, 
//GO(GOMP_loop_ull_ordered_guided_start, 
//GO(GOMP_loop_ull_ordered_runtime_next, 
//GO(GOMP_loop_ull_ordered_runtime_start, 
//GO(GOMP_loop_ull_ordered_start, 
//GO(GOMP_loop_ull_ordered_static_next, 
//GO(GOMP_loop_ull_ordered_static_start, 
//GO(GOMP_loop_ull_runtime_next, 
//GO(GOMP_loop_ull_runtime_start, 
//GO(GOMP_loop_ull_start, 
//GO(GOMP_loop_ull_static_next, 
//GO(GOMP_loop_ull_static_start, 
//GO(GOMP_offload_register, 
//GO(GOMP_offload_register_ver, 
//GO(GOMP_offload_unregister, 
//GO(GOMP_offload_unregister_ver, 
GO(GOMP_ordered_end, vFv)
GO(GOMP_ordered_start, vFv)
GOM(GOMP_parallel, vFEppuu)
//GO(GOMP_parallel_end, 
//GO(GOMP_parallel_loop_dynamic, 
//GO(GOMP_parallel_loop_dynamic_start, 
//GO(GOMP_parallel_loop_guided, 
//GO(GOMP_parallel_loop_guided_start, 
//GO(GOMP_parallel_loop_maybe_nonmonotonic_runtime, 
//GO(GOMP_parallel_loop_nonmonotonic_dynamic, 
//GO(GOMP_parallel_loop_nonmonotonic_guided, 
//GO(GOMP_parallel_loop_nonmonotonic_runtime, 
//GO(GOMP_parallel_loop_runtime, 
//GO(GOMP_parallel_loop_runtime_start, 
//GO(GOMP_parallel_loop_static, 
//GO(GOMP_parallel_loop_static_start, 
//GO(GOMP_parallel_reductions, 
//GO(GOMP_parallel_sections, 
//GO(GOMP_parallel_sections_start, 
//GO(GOMP_parallel_start, 
//GO(GOMP_PLUGIN_acc_default_dim, 
//GO(GOMP_PLUGIN_acc_thread, 
//GO(GOMP_PLUGIN_async_unmap_vars, 
//GO(GOMP_PLUGIN_debug, 
//GO(GOMP_PLUGIN_error, 
//GO(GOMP_PLUGIN_fatal, 
//GO(GOMP_PLUGIN_goacc_profiling_dispatch, 
//GO(GOMP_PLUGIN_goacc_thread, 
//GO(GOMP_PLUGIN_malloc, 
//GO(GOMP_PLUGIN_malloc_cleared, 
//GO(GOMP_PLUGIN_realloc, 
//GO(GOMP_PLUGIN_target_rev, 
//GO(GOMP_PLUGIN_target_task_completion, 
//GO(GOMP_scope_start, 
//GO(GOMP_sections2_start, 
//GO(GOMP_sections_end, 
//GO(GOMP_sections_end_cancel, 
GO(GOMP_sections_end_nowait, vFv)
GO(GOMP_sections_next, uFv)
GO(GOMP_sections_start, uFu)
//GO(GOMP_single_copy_end, 
//GO(GOMP_single_copy_start, 
GO(GOMP_single_start, iFv)
//GO(GOMP_target, 
//GO(GOMP_target_data, 
//GO(GOMP_target_data_ext, 
//GO(GOMP_target_end_data, 
//GO(GOMP_target_enter_exit_data, 
//GO(GOMP_target_ext, 
//GO(GOMP_target_update, 
//GO(GOMP_target_update_ext, 
GOM(GOMP_task, vFEppplliupi)
//GO(GOMP_taskgroup_end, 
//GO(GOMP_taskgroup_reduction_register, 
//GO(GOMP_taskgroup_reduction_unregister, 
//GO(GOMP_taskgroup_start, 
//GO(GOMP_taskloop, 
//GO(GOMP_taskloop_ull, 
//GO(GOMP_task_reduction_remap, 
//GO(GOMP_taskwait, 
//GO(GOMP_taskwait_depend, 
//GO(GOMP_taskwait_depend_nowait, 
//GO(GOMP_taskyield, 
//GO(GOMP_teams, 
//GO(GOMP_teams4, 
//GO(GOMP_teams_reg, 
//GO(GOMP_warning, 
//GO(GOMP_workshare_task_reduction_unregister, 

//GO(omp_aligned_alloc, 
//GO(omp_aligned_calloc, 
//GO(omp_alloc, 
//GO(omp_calloc, 
//GO(omp_capture_affinity, 
//GO(omp_capture_affinity_, 
//GO(omp_destroy_allocator, 
//GO(omp_destroy_allocator_, 
//GO(omp_destroy_lock@OMP_1.0
//GO(omp_destroy_lock, 
//GO(omp_destroy_lock_, 
//GO(omp_destroy_lock_@OMP_1.0
//GO(omp_destroy_nest_lock@OMP_1.0
//GO(omp_destroy_nest_lock, 
//GO(omp_destroy_nest_lock_, 
//GO(omp_destroy_nest_lock_@OMP_1.0
//GO(omp_display_affinity, 
//GO(omp_display_affinity_, 
//GO(omp_display_env, 
//GO(omp_display_env_, 
//GO(omp_display_env_8_, 
//GO(omp_free, 
//GO(omp_fulfill_event, 
//GO(omp_fulfill_event_, 
//GO(omp_get_active_level, 
//GO(omp_get_active_level_, 
//GO(omp_get_affinity_format, 
//GO(omp_get_affinity_format_, 
//GO(omp_get_ancestor_thread_num, 
//GO(omp_get_ancestor_thread_num_, 
//GO(omp_get_ancestor_thread_num_8_, 
//GO(omp_get_cancellation, 
//GO(omp_get_cancellation_, 
//GO(omp_get_default_allocator, 
//GO(omp_get_default_allocator_, 
//GO(omp_get_default_device, 
//GO(omp_get_default_device_, 
//GO(omp_get_device_num, 
//GO(omp_get_device_num_, 
//GO(omp_get_dynamic, 
//GO(omp_get_dynamic_, 
//GO(omp_get_initial_device, 
//GO(omp_get_initial_device_, 
//GO(omp_get_level, 
//GO(omp_get_level_, 
//GO(omp_get_mapped_ptr, 
//GO(omp_get_max_active_levels, 
//GO(omp_get_max_active_levels_, 
//GO(omp_get_max_task_priority, 
//GO(omp_get_max_task_priority_, 
//GO(omp_get_max_teams, 
//GO(omp_get_max_teams_, 
GO(omp_get_max_threads, iFv)
//GO(omp_get_max_threads_, 
//GO(omp_get_nested, 
//GO(omp_get_nested_, 
//GO(omp_get_num_devices, 
//GO(omp_get_num_devices_, 
//GO(omp_get_num_places, 
//GO(omp_get_num_places_, 
GO(omp_get_num_procs, iFv)
//GO(omp_get_num_procs_, 
//GO(omp_get_num_teams, 
//GO(omp_get_num_teams_, 
GO(omp_get_num_threads, iFv)
//GO(omp_get_num_threads_, 
//GO(omp_get_partition_num_places, 
//GO(omp_get_partition_num_places_, 
//GO(omp_get_partition_place_nums, 
//GO(omp_get_partition_place_nums_, 
//GO(omp_get_partition_place_nums_8_, 
//GO(omp_get_place_num, 
//GO(omp_get_place_num_, 
//GO(omp_get_place_num_procs, 
//GO(omp_get_place_num_procs_, 
//GO(omp_get_place_num_procs_8_, 
//GO(omp_get_place_proc_ids, 
//GO(omp_get_place_proc_ids_, 
//GO(omp_get_place_proc_ids_8_, 
//GO(omp_get_proc_bind, 
//GO(omp_get_proc_bind_, 
//GO(omp_get_schedule, 
//GO(omp_get_schedule_, 
//GO(omp_get_schedule_8_, 
//GO(omp_get_supported_active_levels, 
//GO(omp_get_supported_active_levels_, 
//GO(omp_get_team_num, 
//GO(omp_get_team_num_, 
//GO(omp_get_team_size, 
//GO(omp_get_team_size_, 
//GO(omp_get_team_size_8_, 
//GO(omp_get_teams_thread_limit, 
//GO(omp_get_teams_thread_limit_, 
//GO(omp_get_thread_limit, 
//GO(omp_get_thread_limit_, 
GO(omp_get_thread_num, iFv)
//GO(omp_get_thread_num_, 
//GO(omp_get_wtick, 
//GO(omp_get_wtick_, 
//GO(omp_get_wtime, 
//GO(omp_get_wtime_, 
//GO(omp_in_explicit_task, 
//GO(omp_in_explicit_task_, 
//GO(omp_in_final, 
//GO(omp_in_final_, 
//GO(omp_init_allocator, 
//GO(omp_init_allocator_, 
//GO(omp_init_allocator_8_, 
//GO(omp_init_lock, 
//GO(omp_init_lock@OMP_1.0
//GO(omp_init_lock_, 
//GO(omp_init_lock_@OMP_1.0
//GO(omp_init_nest_lock, 
//GO(omp_init_nest_lock@OMP_1.0
//GO(omp_init_nest_lock_, 
//GO(omp_init_nest_lock_@OMP_1.0
GO(omp_in_parallel, iFv)
//GO(omp_in_parallel_, 
//GO(omp_is_initial_device, 
//GO(omp_is_initial_device_, 
//GO(omp_pause_resource, 
//GO(omp_pause_resource_, 
//GO(omp_pause_resource_all, 
//GO(omp_pause_resource_all_, 
//GO(omp_realloc, 
//GO(omp_set_affinity_format, 
//GO(omp_set_affinity_format_, 
//GO(omp_set_default_allocator, 
//GO(omp_set_default_allocator_, 
//GO(omp_set_default_device, 
//GO(omp_set_default_device_, 
//GO(omp_set_default_device_8_, 
//GO(omp_set_dynamic, 
//GO(omp_set_dynamic_, 
//GO(omp_set_dynamic_8_, 
//GO(omp_set_lock, 
//GO(omp_set_lock@OMP_1.0
//GO(omp_set_lock_, 
//GO(omp_set_lock_@OMP_1.0
//GO(omp_set_max_active_levels, 
//GO(omp_set_max_active_levels_, 
//GO(omp_set_max_active_levels_8_, 
//GO(omp_set_nested, 
//GO(omp_set_nested_, 
//GO(omp_set_nested_8_, 
//GO(omp_set_nest_lock@OMP_1.0
//GO(omp_set_nest_lock, 
//GO(omp_set_nest_lock_, 
//GO(omp_set_nest_lock_@OMP_1.0
//GO(omp_set_num_teams, 
//GO(omp_set_num_teams_, 
//GO(omp_set_num_teams_8_, 
GO(omp_set_num_threads, vFi)
//GO(omp_set_num_threads_, 
//GO(omp_set_num_threads_8_, 
//GO(omp_set_schedule, 
//GO(omp_set_schedule_, 
//GO(omp_set_schedule_8_, 
//GO(omp_set_teams_thread_limit, 
//GO(omp_set_teams_thread_limit_, 
//GO(omp_set_teams_thread_limit_8_, 
//GO(omp_target_alloc, 
//GO(omp_target_associate_ptr, 
//GO(omp_target_disassociate_ptr, 
//GO(omp_target_free, 
//GO(omp_target_is_accessible, 
//GO(omp_target_is_present, 
//GO(omp_target_memcpy, 
//GO(omp_target_memcpy_async, 
//GO(omp_target_memcpy_rect, 
//GO(omp_target_memcpy_rect_async, 
//GO(omp_test_lock@OMP_1.0
//GO(omp_test_lock, 
//GO(omp_test_lock_, 
//GO(omp_test_lock_@OMP_1.0
//GO(omp_test_nest_lock@OMP_1.0
//GO(omp_test_nest_lock, 
//GO(omp_test_nest_lock_@OMP_1.0
//GO(omp_test_nest_lock_, 
//GO(omp_unset_lock, 
//GO(omp_unset_lock@OMP_1.0
//GO(omp_unset_lock_@OMP_1.0
//GO(omp_unset_lock_, 
//GO(omp_unset_nest_lock, 
//GO(omp_unset_nest_lock@OMP_1.0
//GO(omp_unset_nest_lock_, 
//GO(omp_unset_nest_lock_@OMP_1.0

