#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_74, pFpp)
GO(ubrk_close_74, vFp)
GO(ubrk_countAvailable_74, iFv)
GO(ubrk_current_74, iFp)
GO(ubrk_first_74, iFp)
GO(ubrk_following_74, iFpi)
GO(ubrk_getAvailable_74, pFi)
GO(ubrk_getBinaryRules_74, iFppip)
GO(ubrk_getLocaleByType_74, pFpip)
GO(ubrk_getRuleStatus_74, iFp)
GO(ubrk_getRuleStatusVec_74, iFppip)
GO(ubrk_isBoundary_74, cFpi)
GO(ubrk_last_74, iFp)
GO(ubrk_next_74, iFp)
GO(ubrk_open_74, pFppip)
GO(ubrk_openBinaryRules_74, pFpipip)
GO(ubrk_openRules_74, pFpipipp)
GO(ubrk_preceding_74, iFpi)
GO(ubrk_previous_74, iFp)
GO(ubrk_refreshUText_74, vFppp)
GO(ubrk_safeClone_74, pFpppp)
GO(ubrk_setText_74, vFppip)
GO(ubrk_setUText_74, vFppp)
GO(ubrk_swap_74, iFppipp)

GO(ucnv_getDefaultName_74, pFv)
GO(ucnv_open_74, pFpp)
GO(ucnv_setDefaultName_74, vFp)

GO(ucurr_forLocale_74, iFppip)
GO(ucurr_getName_74, pFppippp)

GO(uloc_canonicalize_74, iFppip)
GO(uloc_countAvailable_74, iFv)
GO(uloc_getAvailable_74, pFi)
GO(uloc_getBaseName_74, iFppip)
GO(uloc_getCharacterOrientation_74, iFpp)
GO(uloc_getCountry_74, iFppip)
GO(uloc_getDefault_74, pFv)
GO(uloc_getDisplayCountry_74, iFpppip)
GO(uloc_getDisplayLanguage_74, iFpppip)
GO(uloc_getDisplayName_74, iFpppip)
GO(uloc_getISO3Country_74, pFp)
GO(uloc_getISO3Language_74, pFp)
GO(uloc_getKeywordValue_74, iFpppip)
GO(uloc_getLanguage_74, iFppip)
GO(uloc_getLCID_74, uFp)
GO(uloc_getName_74, iFppip)
GO(uloc_getParent_74, iFppip)
GO(uloc_setKeywordValue_74, iFpppip)

GO(ures_close_74, vFp)
GO(ures_getByKey_74, pFpppp)
GO(ures_getSize_74, iFp)
GO(ures_getStringByIndex_74, pFpipp)
GO(ures_open_74, pFppp)

GO(uldn_close_74, vFp)
GO(uldn_keyValueDisplayName_74, iFppppip)
GO(uldn_open_74, pFpip)

GO(uenum_close_74, vFp)
GO(uenum_count_74, iFpp)
GO(uenum_next_74, pFppp)

GO(uidna_close_74, vFp)
GO(uidna_nameToASCII_74, iFppipipp)
GO(uidna_nameToUnicode_74, iFppipipp)
GO(uidna_openUTS46_74, pFpp)

GO(unorm2_getNFCInstance_74, pFp)
GO(unorm2_getNFDInstance_74, pFp)
GO(unorm2_getNFKCInstance_74, pFp)
GO(unorm2_getNFKDInstance_74, pFp)
GO(unorm2_isNormalized_74, CFppip)
GO(unorm2_normalize_74, iFppipip)

GO(u_charsToUChars_74, vFppi)
GO(u_getVersion_74, vFp)
GO(u_strcmp_74, iFpp)
GO(u_strcpy_74, pFpp)
GO(u_strlen_74, iFp)
GO(u_strncpy_74, pFppi)
GO(u_strToLower_74, iFpipipp)
GO(u_strToUpper_74, iFpipipp)
GO(u_tolower_74, uFu)
GO(u_toupper_74, uFu)
GO(u_uastrncpy_74, pFppi)
