/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedliboggTYPES32_H_
#define __wrappedliboggTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void (*vFp_t)(void*);
typedef int32_t (*iFp_t)(void*);
typedef int64_t (*IFp_t)(void*);
typedef intptr_t (*lFp_t)(void*);
typedef void* (*pFp_t)(void*);
typedef void (*vFpi_t)(void*, int32_t);
typedef void (*vFpl_t)(void*, intptr_t);
typedef int32_t (*iFpi_t)(void*, int32_t);
typedef int32_t (*iFpp_t)(void*, void*);
typedef intptr_t (*lFpi_t)(void*, int32_t);
typedef void* (*pFpi_t)(void*, int32_t);
typedef void (*vFpui_t)(void*, uint32_t, int32_t);
typedef void (*vFpLi_t)(void*, uintptr_t, int32_t);
typedef void (*vFppi_t)(void*, void*, int32_t);
typedef int32_t (*iFppi_t)(void*, void*, int32_t);

#define SUPER() ADDED_FUNCTIONS() \
	GO(oggpackB_reset, vFp_t) \
	GO(oggpackB_writeclear, vFp_t) \
	GO(oggpackB_writeinit, vFp_t) \
	GO(oggpack_reset, vFp_t) \
	GO(oggpack_writeclear, vFp_t) \
	GO(oggpack_writeinit, vFp_t) \
	GO(ogg_page_bos, iFp_t) \
	GO(ogg_page_checksum_set, iFp_t) \
	GO(ogg_page_continued, iFp_t) \
	GO(ogg_page_eos, iFp_t) \
	GO(ogg_page_packets, iFp_t) \
	GO(ogg_page_serialno, iFp_t) \
	GO(ogg_stream_clear, iFp_t) \
	GO(ogg_stream_reset, iFp_t) \
	GO(ogg_sync_clear, iFp_t) \
	GO(ogg_sync_init, iFp_t) \
	GO(ogg_sync_reset, iFp_t) \
	GO(oggpackB_bytes, iFp_t) \
	GO(oggpackB_writecheck, iFp_t) \
	GO(oggpack_writecheck, iFp_t) \
	GO(ogg_page_granulepos, IFp_t) \
	GO(ogg_page_pageno, lFp_t) \
	GO(oggpack_bytes, lFp_t) \
	GO(oggpackB_get_buffer, pFp_t) \
	GO(oggpack_get_buffer, pFp_t) \
	GO(oggpack_adv, vFpi_t) \
	GO(oggpack_writetrunc, vFpl_t) \
	GO(ogg_stream_init, iFpi_t) \
	GO(ogg_stream_reset_serialno, iFpi_t) \
	GO(ogg_sync_wrote, iFpi_t) \
	GO(ogg_stream_flush, iFpp_t) \
	GO(ogg_stream_packetin, iFpp_t) \
	GO(ogg_stream_packetout, iFpp_t) \
	GO(ogg_stream_packetpeek, iFpp_t) \
	GO(ogg_stream_pagein, iFpp_t) \
	GO(ogg_stream_pageout, iFpp_t) \
	GO(ogg_sync_pageout, iFpp_t) \
	GO(ogg_sync_pageseek, iFpp_t) \
	GO(oggpackB_read, lFpi_t) \
	GO(oggpack_look, lFpi_t) \
	GO(oggpack_read, lFpi_t) \
	GO(ogg_sync_buffer, pFpi_t) \
	GO(oggpackB_write, vFpui_t) \
	GO(oggpack_write, vFpLi_t) \
	GO(oggpackB_readinit, vFppi_t) \
	GO(oggpack_readinit, vFppi_t) \
	GO(ogg_stream_flush_fill, iFppi_t) \
	GO(ogg_stream_pageout_fill, iFppi_t)

#endif // __wrappedliboggTYPES32_H_
