#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(gst_argb_control_binding_get_type, LFv)
//GOM(gst_argb_control_binding_new, pFEpppppp)
GO(gst_control_point_copy, pFp)
GO(gst_control_point_free, vFp)
GO(gst_control_point_get_type, LFv)
GO(gst_direct_control_binding_get_type, LFv)
//GOM(gst_direct_control_binding_new, pFEppp)
//GOM(gst_direct_control_binding_new_absolute, pFEppp)
GO(gst_interpolation_control_source_get_type, LFv)
//GOM(gst_interpolation_control_source_new, pFEv)
GO(gst_interpolation_mode_get_type, LFv)
GO(gst_lfo_control_source_get_type, LFv)
//GOM(gst_lfo_control_source_new, pFEv)
GO(gst_lfo_waveform_get_type, LFv)
GO(gst_proxy_control_binding_get_type, LFv)
GO(gst_proxy_control_binding_new, pFpppp)
//GOM(gst_timed_value_control_invalidate_cache, vFEp)
//GOM(gst_timed_value_control_source_find_control_point_iter, pFEpL)
//GOM(gst_timed_value_control_source_get_all, pFEp)
//GOM(gst_timed_value_control_source_get_count, iFEp)
GO(gst_timed_value_control_source_get_type, LFv)
//GOM(gst_timed_value_control_source_set, iFEpLd)
//GOM(gst_timed_value_control_source_set_from_list, iFEpp)
//GOM(gst_timed_value_control_source_unset, iFEpL)
//GOM(gst_timed_value_control_source_unset_all, vFEp)
GO(gst_trigger_control_source_get_type, LFv)
//GOM(gst_trigger_control_source_new, pFEv)
