#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error meh!
#endif

// issue is the pci_access structure have 3 function callback that can be override by user program

GO(pci_alloc, pFv)
GO(pci_cleanup, vFp)
GO(pci_fill_info, iFpi)
GO(pci_filter_init, vFpp)
GO(pci_filter_match, iFpp)
GO(pci_filter_parse_id, pFpp)
GO(pci_filter_parse_slot, pFpp)
GO(pci_find_cap, pFpuu)
GO(pci_find_cap_nr, pFpuup)
GO(pci_free_dev, vFp)
GO(pci_free_name_list, vFp)
GO(pci_get_dev, pFpiiii)
GO(pci_get_method_name, pFi)
GO(pci_get_param, pFpp)
GO(pci_get_string_property, pFpu)
GO(pci_id_cache_flush, vFp)
GO(pci_init, vFp)
GO(pci_load_name_list, iFp)
GO(pci_lookup_method, iFp)
GO(pci_lookup_name, pFpiipppp)  //vaarg, with up to 4 element depending on flags
GO(pci_read_block, iFpipi)
GO(pci_read_byte, CFpi)
GO(pci_read_long, uFpi)
GO(pci_read_vpd, iFpipi)
GO(pci_read_word, WFpi)
GO(pci_scan_bus, vFp)
GO(pci_set_name_list_path, vFppi)
GO(pci_set_param, iFppp)
GO(pci_setup_cache, vFppi)
GO(pci_walk_params, pFpp)
GO(pci_write_block, iFpipi)
GO(pci_write_byte, iFpiC)
GO(pci_write_long, iFpiu)
GO(pci_write_word, iFpiW)
