#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ucal_add_67, vFpiip)
GO(ucal_clear_67, vFp)
GO(ucal_clearField_67, vFpi)
GO(ucal_clone_67, pFpp)
GO(ucal_close_67, vFp)
GO(ucal_countAvailable_67, iFv)
GO(ucal_equivalentTo_67, cFpp)
GO(ucal_get_67, iFpip)
GO(ucal_getAttribute_67, iFpi)
GO(ucal_getAvailable_67, pFi)
GO(ucal_getCanonicalTimeZoneID_67, iFpipipp)
GO(ucal_getDayOfWeekType_67, iFpip)
GO(ucal_getDefaultTimeZone_67, iFpip)
GO(ucal_getDSTSavings_67, iFpp)
GO(ucal_getFieldDifference_67, iFpdip)
GO(ucal_getGregorianChange_67, dFpp)
GO(ucal_getHostTimeZone_67, iFpip)
GO(ucal_getKeywordValuesForLocale_67, pFppCp)
GO(ucal_getLimit_67, iFpiip)
GO(ucal_getLocaleByType_67, pFpip)
GO(ucal_getMillis_67, dFpp)
GO(ucal_getNow_67, dFv)
GO(ucal_getTimeZoneDisplayName_67, iFpippip)
GO(ucal_getTimeZoneID_67, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_67, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_67, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_67, cFpipp)
GO(ucal_getType_67, pFpp)
GO(ucal_getTZDataVersion_67, pFp)
GO(ucal_getWeekendTransition_67, iFpip)
GO(ucal_getWindowsTimeZoneID_67, iFpipip)
GO(ucal_inDaylightTime_67, cFpp)
GO(ucal_isSet_67, cFpi)
GO(ucal_isWeekend_67, cFpdp)
GO(ucal_open_67, pFpipip)
GO(ucal_openCountryTimeZones_67, pFpp)
GO(ucal_openTimeZoneIDEnumeration_67, pFippp)
GO(ucal_openTimeZones_67, pFp)
GO(ucal_roll_67, vFpiip)
GO(ucal_set_67, vFpii)
GO(ucal_setAttribute_67, vFpii)
GO(ucal_setDate_67, vFpiiip)
GO(ucal_setDateTime_67, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_67, vFpp)
GO(ucal_setGregorianChange_67, vFpdp)
GO(ucal_setMillis_67, vFpdp)
GO(ucal_setTimeZone_67, vFppip)

GO(ucol_clone_67, pFpp)
GO(ucol_close_67, vFp)
GO(ucol_closeElements_67, vFp)
GO(ucol_setMaxVariable_67, vFpip)
GO(ucol_getOffset_67, iFp)
GO(ucol_getRules_67, pFpp)
GO(ucol_getSortKey_67, iFppipi)
GO(ucol_getStrength_67, iFp)
GO(ucol_getVersion_67, vFpp)
GO(ucol_next_67, iFpp)
GO(ucol_previous_67, iFpp)
GO(ucol_open_67, pFpp)
GO(ucol_openElements_67, pFppip)
GO(ucol_openRules_67, pFpiiipp)
GO(ucol_safeClone_67, pFpppp)
GO(ucol_setAttribute_67, vFpiip)
GO(ucol_setVariableTop_67, uFppip)
GO(ucol_strcoll_67, iFppipi)

GO(udat_close_67, vFp)
GO(udat_countSymbols_67, iFpi)
GO(udat_getSymbols_67, iFpiipip)
GO(udat_format_67, iFpdpipp)
GO(udat_open_67, pFiippipip)
GO(udat_setCalendar_67, vFpp)
GO(udat_toPattern_67, iFpCpip)

GO(unum_close_67, vFp)
GO(unum_getAttribute_67, iFpi)
GO(unum_getSymbol_67, iFpipip)
GO(unum_open_67, pFipippp)
GO(unum_toPattern_67, iFpCpip)

GO(udatpg_close_67, vFp)
GO(udatpg_getBestPattern_67, iFppipip)
GO(udatpg_open_67, pFpp)

GO(ulocdata_getCLDRVersion_67, vFpp)
GO(ulocdata_getMeasurementSystem_67, iFpp)

GO(usearch_close_67, vFp)
GO(usearch_first_67, iFpp)
GO(usearch_getBreakIterator_67, pFp)
GO(usearch_getMatchedLength_67, iFp)
GO(usearch_last_67, iFpp)
GO(usearch_openFromCollator_67, pFpipippp)
GO(usearch_setPattern_67, vFppip)
GO(usearch_setText_67, vFppip)
