#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(XFixesChangeCursor, vFXLL)
GO(XFixesChangeCursorByName, vFXLp)
GO(XFixesChangeSaveSet, vFXLiii)
GO(XFixesCopyRegion, vFXLL)
GO(XFixesCreatePointerBarrier, LFXLiiiiiip)
GO(XFixesCreateRegion, LFXpi)
GO(XFixesCreateRegionFromBitmap, LFXL)
GO(XFixesCreateRegionFromGC, LFXp)
GO(XFixesCreateRegionFromPicture, LFXL)
GO(XFixesCreateRegionFromWindow, LFXLi)
GO(XFixesDestroyPointerBarrier, vFXL)
GO(XFixesDestroyRegion, vFXL)
GO(XFixesExpandRegion, vFXLLuuuu)
//DATA(XFixesExtensionInfo, sizeof(ptr_t))    //B
//DATA(XFixesExtensionName, sizeof(ptr_t))    //D
GO(XFixesFetchRegion, pFXLp)
GO(XFixesFetchRegionAndBounds, pFXLpp)
//GO(XFixesFindDisplay, pFp)
GOM(XFixesGetCursorImage, pFEX)
GO(XFixesGetCursorName, pFXLbL_)
GO(XFixesHideCursor, vFXL)
GO(XFixesIntersectRegion, vFXLLL)
GO(XFixesInvertRegion, vFXLpL)
GOM(XFixesQueryExtension, iFEXpp)
GO(XFixesQueryVersion, iFXpp)
GO(XFixesRegionExtents, vFXLL)
GO(XFixesSelectCursorInput, vFXLL)
GO(XFixesSelectSelectionInput, vFXLLL)
GO(XFixesSetCursorName, vFXLp)
GO(XFixesSetGCClipRegion, vFXpiiL)
GO(XFixesSetPictureClipRegion, vFXLiiL)
GO(XFixesSetRegion, vFXLpi)
GO(XFixesSetWindowShapeRegion, vFXLiiiL)
GO(XFixesShowCursor, vFXL)
GO(XFixesSubtractRegion, vFXLLL)
GO(XFixesTranslateRegion, vFXLii)
GO(XFixesUnionRegion, vFXLLL)
GO(XFixesVersion, iFv)
