#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
error Meh...
#endif

GO(alureGetVersion,vFpp)
GO(alureGetErrorString,pFv)
GO(alureGetDeviceNames,pFip)
GO(alureFreeDeviceNames,vFp)
GO(alureInitDevice,iFpp)
GO(alureShutdownDevice,iFv)
GO(alureGetSampleFormat,iFuuu)
GO(alureSleep,iFf)
GO(alureStreamSizeIsMicroSec,iFi)
GO(alureCreateBufferFromFile,uFp)
GO(alureCreateBufferFromMemory,uFpi)
GO(alureBufferDataFromFile,iFpu)
GO(alureBufferDataFromMemory,iFpiu)
GO(alureCreateStreamFromFile,pFpiip)
GO(alureCreateStreamFromMemory,pFpuiip)
GO(alureCreateStreamFromStaticMemory,pFpuiip)
//GOM(alureCreateStreamFromCallback,pFEBpiuiip)
GO(alureGetStreamLength,IFp)
GO(alureGetStreamFrequency,iFp)
GO(alureGetStreamOrder,uFp)
GO(alureBufferDataFromStream,iFpip)
GO(alureRewindStream,iFp)
GO(alureSetStreamOrder,iFpu)
GO(alureSetStreamPatchset,iFpp)
GO(alureDestroyStream,iFpip)
GO(alureUpdate,vFv)
GO(alureUpdateInterval,iFf)
//GOM(alurePlaySourceStream,iFEupiiBp)
//GOM(alurePlaySource,iFEuBp)
GO(alureStopSource,iFui)
GO(alurePauseSource,iFu)
GO(alureResumeSource,iFu)
//GOM(alureInstallDecodeCallbacks,iFEiBBBBBB)
//GOM(alureSetIOCallbacks,iFEBBBBB)
//GOM(alureGetProcAddress,pFEp)