#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

// UDate is double
// UBool is int8_t

GO(ucal_add_75, vFpiip)
GO(ucal_clear_75, vFp)
GO(ucal_clearField_75, vFpi)
GO(ucal_clone_75, pFpp)
GO(ucal_close_75, vFp)
GO(ucal_countAvailable_75, iFv)
GO(ucal_equivalentTo_75, cFpp)
GO(ucal_get_75, iFpip)
GO(ucal_getAttribute_75, iFpi)
GO(ucal_getAvailable_75, pFi)
GO(ucal_getCanonicalTimeZoneID_75, iFpipipp)
GO(ucal_getDayOfWeekType_75, iFpip)
GO(ucal_getDefaultTimeZone_75, iFpip)
GO(ucal_getDSTSavings_75, iFpp)
GO(ucal_getFieldDifference_75, iFpdip)
GO(ucal_getGregorianChange_75, dFpp)
GO(ucal_getHostTimeZone_75, iFpip)
GO(ucal_getKeywordValuesForLocale_75, pFppCp)
GO(ucal_getLimit_75, iFpiip)
GO(ucal_getLocaleByType_75, pFpip)
GO(ucal_getMillis_75, dFpp)
GO(ucal_getNow_75, dFv)
GO(ucal_getTimeZoneDisplayName_75, iFpippip)
GO(ucal_getTimeZoneID_75, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_75, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_75, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_75, cFpipp)
GO(ucal_getType_75, pFpp)
GO(ucal_getTZDataVersion_75, pFp)
GO(ucal_getWeekendTransition_75, iFpip)
GO(ucal_getWindowsTimeZoneID_75, iFpipip)
GO(ucal_inDaylightTime_75, cFpp)
GO(ucal_isSet_75, cFpi)
GO(ucal_isWeekend_75, cFpdp)
GO(ucal_open_75, pFpipip)
GO(ucal_openCountryTimeZones_75, pFpp)
GO(ucal_openTimeZoneIDEnumeration_75, pFippp)
GO(ucal_openTimeZones_75, pFp)
GO(ucal_roll_75, vFpiip)
GO(ucal_set_75, vFpii)
GO(ucal_setAttribute_75, vFpii)
GO(ucal_setDate_75, vFpiiip)
GO(ucal_setDateTime_75, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_75, vFpp)
GO(ucal_setGregorianChange_75, vFpdp)
GO(ucal_setMillis_75, vFpdp)
GO(ucal_setTimeZone_75, vFppip)

GO(ucol_clone_75, pFpp)
GO(ucol_close_75, vFp)
GO(ucol_closeElements_75, vFp)
GO(ucol_setMaxVariable_75, vFpip)
GO(ucol_getOffset_75, iFp)
GO(ucol_getRules_75, pFpp)
GO(ucol_getSortKey_75, iFppipi)
GO(ucol_getStrength_75, iFp)
GO(ucol_getVersion_75, vFpp)
GO(ucol_next_75, iFpp)
GO(ucol_previous_75, iFpp)
GO(ucol_open_75, pFpp)
GO(ucol_openElements_75, pFppip)
GO(ucol_openRules_75, pFpiiipp)
GO(ucol_safeClone_75, pFpppp)
GO(ucol_setAttribute_75, vFpiip)
GO(ucol_setVariableTop_75, uFppip)
GO(ucol_strcoll_75, iFppipi)

GO(udat_close_75, vFp)
GO(udat_countSymbols_75, iFpi)
GO(udat_getSymbols_75, iFpiipip)
GO(udat_format_75, iFpdpipp)
GO(udat_open_75, pFiippipip)
GO(udat_setCalendar_75, vFpp)
GO(udat_toPattern_75, iFpCpip)

GO(unum_close_75, vFp)
GO(unum_getAttribute_75, iFpi)
GO(unum_getSymbol_75, iFpipip)
GO(unum_open_75, pFipippp)
GO(unum_toPattern_75, iFpCpip)

GO(udatpg_close_75, vFp)
GO(udatpg_getBestPattern_75, iFppipip)
GO(udatpg_open_75, pFpp)

GO(ulocdata_getCLDRVersion_75, vFpp)
GO(ulocdata_getMeasurementSystem_75, iFpp)

GO(usearch_close_75, vFp)
GO(usearch_first_75, iFpp)
GO(usearch_getBreakIterator_75, pFp)
GO(usearch_getMatchedLength_75, iFp)
GO(usearch_last_75, iFpp)
GO(usearch_openFromCollator_75, pFpipippp)
GO(usearch_setPattern_75, vFppip)
GO(usearch_setText_75, vFppip)
