#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(smbc_chmod, iFpu)
GO(smbc_close, iFi)
GO(smbc_closedir, iFi)
GO(smbc_creat, iFpu)
GO(smbc_fgetxattr, iFippL)
GO(smbc_flistxattr, iFipL)
GO(smbc_free_context, iFpi)
GO(smbc_fremovexattr, iFip)
GO(smbc_fsetxattr, iFippLi)
GO(smbc_fstat, iFip)
GO(smbc_fstatvfs, iFip)
GO(smbc_ftruncate, iFil)
//GOM(smbc_getDebug, iFEp)
GO(smbc_getdents, iFupi)
//GOM(smbc_getFunctionAddCachedServer, pFEp)
//GOM(smbc_getFunctionAuthData, pFEp)
//GOM(smbc_getFunctionAuthDataWithContext, pFEp)
//GOM(smbc_getFunctionCheckServer, pFEp)
//GOM(smbc_getFunctionChmod, pFEp)
//GOM(smbc_getFunctionClose, pFEp)
//GOM(smbc_getFunctionClosedir, pFEp)
//GOM(smbc_getFunctionCreat, pFEp)
//GOM(smbc_getFunctionFGetxattr, pFEp)
//GOM(smbc_getFunctionFstat, pFEp)
//GOM(smbc_getFunctionFstatdir, pFEp)
//GOM(smbc_getFunctionFstatVFS, pFEp)
//GOM(smbc_getFunctionFtruncate, pFEp)
//GOM(smbc_getFunctionGetCachedServer, pFEp)
//GOM(smbc_getFunctionGetdents, pFEp)
//GOM(smbc_getFunctionGetxattr, pFEp)
//GOM(smbc_getFunctionListPrintJobs, pFEp)
//GOM(smbc_getFunctionListxattr, pFEp)
//GOM(smbc_getFunctionLseek, pFEp)
//GOM(smbc_getFunctionLseekdir, pFEp)
//GOM(smbc_getFunctionMkdir, pFEp)
//GOM(smbc_getFunctionNotify, pFEp)
//GOM(smbc_getFunctionOpen, pFEp)
//GOM(smbc_getFunctionOpendir, pFEp)
//GOM(smbc_getFunctionOpenPrintJob, pFEp)
//GOM(smbc_getFunctionPrintFile, pFEp)
//GOM(smbc_getFunctionPurgeCachedServers, pFEp)
//GOM(smbc_getFunctionRead, pFEp)
//GOM(smbc_getFunctionReaddir, pFEp)
//GOM(smbc_getFunctionReaddirPlus, pFEp)
//GOM(smbc_getFunctionReaddirPlus2, pFEp)
//GOM(smbc_getFunctionRemoveCachedServer, pFEp)
//GOM(smbc_getFunctionRemoveUnusedServer, pFEp)
//GOM(smbc_getFunctionRemovexattr, pFEp)
//GOM(smbc_getFunctionRename, pFEp)
//GOM(smbc_getFunctionRmdir, pFEp)
//GOM(smbc_getFunctionSetxattr, pFEp)
//GOM(smbc_getFunctionSplice, pFEp)
//GOM(smbc_getFunctionStat, pFEp)
//GOM(smbc_getFunctionStatVFS, pFEp)
//GOM(smbc_getFunctionTelldir, pFEp)
//GOM(smbc_getFunctionUnlink, pFEp)
//GOM(smbc_getFunctionUnlinkPrintJob, pFEp)
//GOM(smbc_getFunctionUtimes, pFEp)
//GOM(smbc_getFunctionWrite, pFEp)
//GOM(smbc_getNetbiosName, pFEp)
//GOM(smbc_getOptionBrowseMaxLmbCount, iFEp)
//GOM(smbc_getOptionCaseSensitive, iFEp)
//GOM(smbc_getOptionDebugToStderr, iFEp)
//GOM(smbc_getOptionFallbackAfterKerberos, iFEp)
//GOM(smbc_getOptionFullTimeNames, iFEp)
//GOM(smbc_getOptionNoAutoAnonymousLogin, iFEp)
//GOM(smbc_getOptionOneSharePerServer, iFEp)
//GOM(smbc_getOptionOpenShareMode, uFEp)
//GOM(smbc_getOptionPosixExtensions, iFEp)
//GOM(smbc_getOptionSmbEncryptionLevel, iFEp)
//GOM(smbc_getOptionUrlEncodeReaddirEntries, iFEp)
//GOM(smbc_getOptionUseCCache, iFEp)
//GOM(smbc_getOptionUseKerberos, iFEp)
//GOM(smbc_getOptionUseNTHash, iFEp)
//GOM(smbc_getOptionUserData, pFEp)
//GOM(smbc_getPort, WFEp)
//GOM(smbc_getServerCacheData, pFEp)
//GOM(smbc_getTimeout, iFEp)
//GOM(smbc_getUser, pFEp)
//GOM(smbc_getWorkgroup, pFEp)
GO(smbc_getxattr, iFpppL)
GOM(smbc_init, iFEpi)
GO(smbc_init_context, pFp)
GO(smbc_lgetxattr, iFpppL)
//GOM(smbc_list_print_jobs, iFEpp)
GO(smbc_listxattr, iFppL)
GO(smbc_llistxattr, iFppL)
GO(smbc_lremovexattr, iFpp)
GO(smbc_lseek, lFili)
GO(smbc_lseekdir, iFil)
GO(smbc_lsetxattr, iFpppLi)
GO(smbc_mkdir, iFpu)
GO(smbc_new_context, pFv)
//GOM(smbc_notify, iFEiiuupp)
GO(smbc_open, iFpiu)
GO(smbc_opendir, iFp)
GO(smbc_open_print_job, iFp)
//GOM(smbc_option_get, pFEpp)
//GOM(smbc_option_set, vFEppV)
GO(smbc_print_file, iFpp)
GO(smbc_read, lFipL)
GO(smbc_readdir, pFu)
GO(smbc_readdirplus, pFu)
GO(smbc_readdirplus2, pFup)
GO(smbc_removexattr, iFpp)
GO(smbc_rename, iFpp)
GO(smbc_rmdir, iFp)
//GOM(smbc_setConfiguration, iFEpp)
GO(smbc_set_context, pFp)
GO(smbc_set_credentials, vFpppip)
//GOM(smbc_set_credentials_with_fallback, vFEpppp)
//GOM(smbc_setDebug, vFEpi)
//GOM(smbc_setFunctionAddCachedServer, vFEpp)
//GOM(smbc_setFunctionAuthData, vFEpp)
GOM(smbc_setFunctionAuthDataWithContext, vFEpp)
//GOM(smbc_setFunctionCheckServer, vFEpp)
//GOM(smbc_setFunctionChmod, vFEpp)
//GOM(smbc_setFunctionClose, vFEpp)
//GOM(smbc_setFunctionClosedir, vFEpp)
//GOM(smbc_setFunctionCreat, vFEpp)
//GOM(smbc_setFunctionFGetxattr, vFEpp)
//GOM(smbc_setFunctionFstat, vFEpp)
//GOM(smbc_setFunctionFstatdir, vFEpp)
//GOM(smbc_setFunctionFstatVFS, vFEpp)
//GOM(smbc_setFunctionFtruncate, vFEpp)
//GOM(smbc_setFunctionGetCachedServer, vFEpp)
//GOM(smbc_setFunctionGetdents, vFEpp)
//GOM(smbc_setFunctionGetxattr, vFEpp)
//GOM(smbc_setFunctionListPrintJobs, vFEpp)
//GOM(smbc_setFunctionListxattr, vFEpp)
//GOM(smbc_setFunctionLseek, vFEpp)
//GOM(smbc_setFunctionLseekdir, vFEpp)
//GOM(smbc_setFunctionMkdir, vFEpp)
//GOM(smbc_setFunctionNotify, vFEpp)
//GOM(smbc_setFunctionOpen, vFEpp)
//GOM(smbc_setFunctionOpendir, vFEpp)
//GOM(smbc_setFunctionOpenPrintJob, vFEpp)
//GOM(smbc_setFunctionPrintFile, vFEpp)
//GOM(smbc_setFunctionPurgeCachedServers, vFEpp)
//GOM(smbc_setFunctionRead, vFEpp)
//GOM(smbc_setFunctionReaddir, vFEpp)
//GOM(smbc_setFunctionReaddirPlus, vFEpp)
//GOM(smbc_setFunctionReaddirPlus2, vFEpp)
//GOM(smbc_setFunctionRemoveCachedServer, vFEpp)
//GOM(smbc_setFunctionRemoveUnusedServer, vFEpp)
//GOM(smbc_setFunctionRemovexattr, vFEpp)
//GOM(smbc_setFunctionRename, vFEpp)
//GOM(smbc_setFunctionRmdir, vFEpp)
//GOM(smbc_setFunctionSetxattr, vFEpp)
//GOM(smbc_setFunctionSplice, vFEpp)
//GOM(smbc_setFunctionStat, vFEpp)
//GOM(smbc_setFunctionStatVFS, vFEpp)
//GOM(smbc_setFunctionTelldir, vFEpp)
//GOM(smbc_setFunctionUnlink, vFEpp)
//GOM(smbc_setFunctionUnlinkPrintJob, vFEpp)
//GOM(smbc_setFunctionUtimes, vFEpp)
//GOM(smbc_setFunctionWrite, vFEpp)
//GOM(smbc_setLogCallback, vFEppp)
//GOM(smbc_setNetbiosName, vFEpp)
//GOM(smbc_setOptionBrowseMaxLmbCount, vFEpi)
//GOM(smbc_setOptionCaseSensitive, vFEpi)
//GOM(smbc_setOptionDebugToStderr, vFEpi)
//GOM(smbc_setOptionFallbackAfterKerberos, vFEpi)
//GOM(smbc_setOptionFullTimeNames, vFEpi)
//GOM(smbc_setOptionNoAutoAnonymousLogin, vFEpi)
//GOM(smbc_setOptionOneSharePerServer, vFEpi)
//GOM(smbc_setOptionOpenShareMode, vFEpu)
//GOM(smbc_setOptionPosixExtensions, vFEpi)
//GOM(smbc_setOptionProtocols, iFEppp)
//GOM(smbc_setOptionSmbEncryptionLevel, vFEpi)
//GOM(smbc_setOptionUrlEncodeReaddirEntries, vFEpi)
//GOM(smbc_setOptionUseCCache, vFEpi)
//GOM(smbc_setOptionUseKerberos, vFEpi)
//GOM(smbc_setOptionUseNTHash, vFEpi)
GO(smbc_setOptionUserData, vFpp)
//GOM(smbc_setPort, vFEpW)
//GOM(smbc_setServerCacheData, vFEpp)
GO(smbc_setTimeout, vFpi)
//GOM(smbc_setUser, vFEpp)
GO(smbc_setWorkgroup, vFpp)
GO(smbc_setxattr, iFpppLi)
GO(smbc_stat, iFpp)
GO(smbc_statvfs, iFpp)
GO(smbc_telldir, lFi)
GO(smbc_unlink, iFp)
GO(smbc_unlink_print_job, iFpi)
GO(smbc_urldecode, iFppL)
GO(smbc_urlencode, iFppi)
//GO(smbc_utime, 
GO(smbc_utimes, iFpp)
GO(smbc_version, pFv)
GO(smbc_write, lFipL)
