#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ucal_add_73, vFpiip)
GO(ucal_clear_73, vFp)
GO(ucal_clearField_73, vFpi)
GO(ucal_clone_73, pFpp)
GO(ucal_close_73, vFp)
GO(ucal_countAvailable_73, iFv)
GO(ucal_equivalentTo_73, cFpp)
GO(ucal_get_73, iFpip)
GO(ucal_getAttribute_73, iFpi)
GO(ucal_getAvailable_73, pFi)
GO(ucal_getCanonicalTimeZoneID_73, iFpipipp)
GO(ucal_getDayOfWeekType_73, iFpip)
GO(ucal_getDefaultTimeZone_73, iFpip)
GO(ucal_getDSTSavings_73, iFpp)
GO(ucal_getFieldDifference_73, iFpdip)
GO(ucal_getGregorianChange_73, dFpp)
GO(ucal_getHostTimeZone_73, iFpip)
GO(ucal_getKeywordValuesForLocale_73, pFppCp)
GO(ucal_getLimit_73, iFpiip)
GO(ucal_getLocaleByType_73, pFpip)
GO(ucal_getMillis_73, dFpp)
GO(ucal_getNow_73, dFv)
GO(ucal_getTimeZoneDisplayName_73, iFpippip)
GO(ucal_getTimeZoneID_73, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_73, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_73, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_73, cFpipp)
GO(ucal_getType_73, pFpp)
GO(ucal_getTZDataVersion_73, pFp)
GO(ucal_getWeekendTransition_73, iFpip)
GO(ucal_getWindowsTimeZoneID_73, iFpipip)
GO(ucal_inDaylightTime_73, cFpp)
GO(ucal_isSet_73, cFpi)
GO(ucal_isWeekend_73, cFpdp)
GO(ucal_open_73, pFpipip)
GO(ucal_openCountryTimeZones_73, pFpp)
GO(ucal_openTimeZoneIDEnumeration_73, pFippp)
GO(ucal_openTimeZones_73, pFp)
GO(ucal_roll_73, vFpiip)
GO(ucal_set_73, vFpii)
GO(ucal_setAttribute_73, vFpii)
GO(ucal_setDate_73, vFpiiip)
GO(ucal_setDateTime_73, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_73, vFpp)
GO(ucal_setGregorianChange_73, vFpdp)
GO(ucal_setMillis_73, vFpdp)
GO(ucal_setTimeZone_73, vFppip)

GO(ucol_clone_73, pFpp)
GO(ucol_close_73, vFp)
GO(ucol_closeElements_73, vFp)
GO(ucol_setMaxVariable_73, vFpip)
GO(ucol_getOffset_73, iFp)
GO(ucol_getRules_73, pFpp)
GO(ucol_getSortKey_73, iFppipi)
GO(ucol_getStrength_73, iFp)
GO(ucol_getVersion_73, vFpp)
GO(ucol_next_73, iFpp)
GO(ucol_previous_73, iFpp)
GO(ucol_open_73, pFpp)
GO(ucol_openElements_73, pFppip)
GO(ucol_openRules_73, pFpiiipp)
GO(ucol_safeClone_73, pFpppp)
GO(ucol_setAttribute_73, vFpiip)
GO(ucol_setVariableTop_73, uFppip)
GO(ucol_strcoll_73, iFppipi)

GO(udat_close_73, vFp)
GO(udat_countSymbols_73, iFpi)
GO(udat_getSymbols_73, iFpiipip)
GO(udat_format_73, iFpdpipp)
GO(udat_open_73, pFiippipip)
GO(udat_setCalendar_73, vFpp)
GO(udat_toPattern_73, iFpCpip)

GO(unum_close_73, vFp)
GO(unum_getAttribute_73, iFpi)
GO(unum_getSymbol_73, iFpipip)
GO(unum_open_73, pFipippp)
GO(unum_toPattern_73, iFpCpip)

GO(udatpg_close_73, vFp)
GO(udatpg_getBestPattern_73, iFppipip)
GO(udatpg_open_73, pFpp)

GO(ulocdata_getCLDRVersion_73, vFpp)
GO(ulocdata_getMeasurementSystem_73, iFpp)

GO(usearch_close_73, vFp)
GO(usearch_first_73, iFpp)
GO(usearch_getBreakIterator_73, pFp)
GO(usearch_getMatchedLength_73, iFp)
GO(usearch_last_73, iFpp)
GO(usearch_openFromCollator_73, pFpipippp)
GO(usearch_setPattern_73, vFppip)
GO(usearch_setText_73, vFppip)
