/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappedlibxcbTYPES32_H_
#define __wrappedlibxcbTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void (*vFp_t)(void*);
typedef void* (*pFpp_t)(void*, void*);
typedef my_xcb_XXX_iterator_t (*ZFpp_t)(void*, void*);
typedef my_xcb_cookie_t (*zFnuu_t)(void*, uint32_t, uint32_t);
typedef my_xcb_cookie_t (*zFnCWp_t)(void*, uint8_t, uint16_t, void*);
typedef my_xcb_cookie_t (*zFnCuuuuu_t)(void*, uint8_t, uint32_t, uint32_t, uint32_t, uint32_t, uint32_t);
typedef my_xcb_cookie_t (*zFnCuuuCup_t)(void*, uint8_t, uint32_t, uint32_t, uint32_t, uint8_t, uint32_t, void*);
typedef my_xcb_cookie_t (*zFpnCuuwwWWWWuup_t)(void*, void*, uint8_t, uint32_t, uint32_t, int16_t, int16_t, uint16_t, uint16_t, uint16_t, uint16_t, uint32_t, uint32_t, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(xcb_disconnect, vFp_t) \
	GO(xcb_connect, pFpp_t)

#endif // __wrappedlibxcbTYPES32_H_
