#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(pixman_transform_init_identity, vFp)
GO(pixman_transform_point_3d, iFpp)
GO(pixman_transform_point, iFpp)
GO(pixman_transform_multiply, iFppp)
GO(pixman_transform_init_scale, vFpii)
GO(pixman_transform_scale, iFppii)
GO(pixman_transform_init_rotate, vFpii)
GO(pixman_transform_rotate, iFppii)
GO(pixman_transform_init_translate, vFpii)
GO(pixman_transform_translate, iFppii)
GO(pixman_transform_bounds, iFpp)
GO(pixman_transform_invert, iFpp)
GO(pixman_transform_is_identity, iFp)
GO(pixman_transform_is_scale, iFp)
GO(pixman_transform_is_int_translate, iFp)
GO(pixman_transform_is_inverse, iFpp)

GO(pixman_transform_from_pixman_f_transform, iFpp)
GO(pixman_f_transform_from_pixman_transform, vFpp)
GO(pixman_f_transform_invert, iFpp)
GO(pixman_f_transform_point, iFpp)
GO(pixman_f_transform_point_3d, vFpp)
GO(pixman_f_transform_multiply, vFppp)
GO(pixman_f_transform_init_scale, vFpdd)
GO(pixman_f_transform_scale, iFppdd)
GO(pixman_f_transform_init_rotate, vFpdd)
GO(pixman_f_transform_rotate, iFppdd)
GO(pixman_f_transform_init_translate, vFpdd)
GO(pixman_f_transform_translate, iFppdd)
GO(pixman_f_transform_bounds, iFpp)
GO(pixman_f_transform_init_identity, vFp)

GO(pixman_region_set_static_pointers, vFppp)
GO(pixman_region_init, vFp)
GO(pixman_region_init_rect, vFpiiuu)
GO(pixman_region_init_rects, iFppi)
GO(pixman_region_init_with_extents, vFpp)
GO(pixman_region_init_from_image, vFpp)
GO(pixman_region_fini, vFp)
GO(pixman_region_translate, vFpii)
GO(pixman_region_copy, iFpp)
GO(pixman_region_intersect, iFppp)
GO(pixman_region_union, iFppp)
GO(pixman_region_union_rect, iFppiiuu)
GO(pixman_region_intersect_rect, iFppiiuu)
GO(pixman_region_subtract, iFppp)
GO(pixman_region_inverse, iFppp)
GO(pixman_region_contains_point, iFpiip)
GO(pixman_region_contains_rectangle, uFpp)
GO(pixman_region_not_empty, iFp)
GO(pixman_region_extents, pFp)
GO(pixman_region_n_rects, iFp)
GO(pixman_region_rectangles, pFpp)
GO(pixman_region_equal, iFpp)
GO(pixman_region_selfcheck, iFp)
GO(pixman_region_reset, vFpp)
GO(pixman_region_clear, vFp)

GO(pixman_region32_init, vFp)
GO(pixman_region32_init_rect, vFpiiuu)
GO(pixman_region32_init_rects, iFppi)
GO(pixman_region32_init_with_extents, vFpp)
GO(pixman_region32_init_from_image, vFpp)
GO(pixman_region32_fini, vFp)
GO(pixman_region32_translate, vFpii)
GO(pixman_region32_copy, iFpp)
GO(pixman_region32_intersect, iFppp)
GO(pixman_region32_union, iFppp)
GO(pixman_region32_intersect_rect, iFppiiuu)
GO(pixman_region32_union_rect, iFppiiuu)
GO(pixman_region32_subtract, iFppp)
GO(pixman_region32_inverse, iFppp)
GO(pixman_region32_contains_point, iFpiip)
GO(pixman_region32_contains_rectangle, uFpp)
GO(pixman_region32_not_empty, iFp)
GO(pixman_region32_extents, pFp)
GO(pixman_region32_n_rects, iFp)
GO(pixman_region32_rectangles, pFpp)
GO(pixman_region32_equal, iFpp)
GO(pixman_region32_selfcheck, iFp)
GO(pixman_region32_reset, vFpp)
GO(pixman_region32_clear, vFp)

GO(pixman_blt, iFppiiiiiiiiii)
GO(pixman_fill, iFpiiiiiiu)

GO(pixman_version, iFv)
GO(pixman_version_string, pFv)

GO(pixman_format_supported_destination, iFu)
GO(pixman_format_supported_source, iFu)

GO(pixman_image_create_solid_fill, pFp)
GO(pixman_image_create_linear_gradient, pFpppi)
GO(pixman_image_create_radial_gradient, pFppiipi)
GO(pixman_image_create_conical_gradient, pFpipi)
GO(pixman_image_create_bits, pFuiipi)
GO(pixman_image_create_bits_no_clear, pFuiipi)

GO(pixman_image_ref, pFp)
GO(pixman_image_unref, iFp)
//GOM(pixman_image_set_destroy_function, vFEppp)
GO(pixman_image_get_destroy_data, pFp)

GO(pixman_image_set_clip_region, iFpp)
GO(pixman_image_set_clip_region32, iFpp)
GO(pixman_image_set_has_client_clip, vFpi)
GO(pixman_image_set_transform, iFpp)
GO(pixman_image_set_repeat, vFpu)
GO(pixman_image_set_dither, vFpu)
GO(pixman_image_set_dither_offset, vFpii)
GO(pixman_image_set_filter, iFpupi)
GO(pixman_image_set_source_clipping, vFpi)
GO(pixman_image_set_alpha_map, vFppww)
GO(pixman_image_set_component_alpha, vFpi)
GO(pixman_image_get_component_alpha, iFp)
//GOM(pixman_image_set_accessors, vFEppp)
GO(pixman_image_set_indexed, vFpp)
GO(pixman_image_get_data, pFp)
GO(pixman_image_get_width, iFp)
GO(pixman_image_get_height, iFp)
GO(pixman_image_get_stride, iFp)
GO(pixman_image_get_depth, iFp)
GO(pixman_image_get_format, uFp)
GO(pixman_filter_create_separable_convolution, pFpiiuuuuii)
GO(pixman_image_fill_rectangles, iFuppip)
GO(pixman_image_fill_boxes, iFuppip)
GO(pixman_compute_composite_region, iFppppwwwwwwWW)
GO(pixman_image_composite, vFupppwwwwwwWW)
GO(pixman_image_composite32, vFupppiiiiiiii)

GO(pixman_disable_out_of_bounds_workaround, vFv)

GO(pixman_glyph_cache_create, pFv)
GO(pixman_glyph_cache_destroy, vFp)
GO(pixman_glyph_cache_freeze, vFp)
GO(pixman_glyph_cache_thaw, vFp)
GO(pixman_glyph_cache_lookup, pFppp)
GO(pixman_glyph_cache_insert, pFpppiip)
GO(pixman_glyph_cache_remove, vFppp)
GO(pixman_glyph_get_extents, vFpipp)
GO(pixman_glyph_get_mask_format, uFpip)
GO(pixman_composite_glyphs, vFuppuiiiiiiiipip)
GO(pixman_composite_glyphs_no_mask, vFuppiiiipip)

GO(pixman_sample_ceil_y, iFii)
GO(pixman_sample_floor_y, iFii)
GO(pixman_edge_step, vFpi)
GO(pixman_edge_init, vFpiiiiii)
GO(pixman_line_fixed_edge_init, vFpiipii)
GO(pixman_rasterize_edges, vFpppii)
GO(pixman_add_traps, vFpwwip)
GO(pixman_add_trapezoids, vFpwiip)
GO(pixman_rasterize_trapezoid, vFppii)
GO(pixman_composite_trapezoids, vFuppuiiiiip)
GO(pixman_composite_triangles, vFuppuiiiiip)
GO(pixman_add_triangles, vFpiiip)
