#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(VA_DRI2Authenticate, 
//GO(VA_DRI2Connect, 
//GO(VA_DRI2CopyRegion, 
//GO(VA_DRI2CreateDrawable, 
//GO(VA_DRI2DestroyDrawable, 
//GO(VA_DRI2GetBuffers, 
//GO(VA_DRI2QueryExtension, 
//GO(VA_DRI2QueryVersion, 
//GO(VA_DRI2SwapBuffers, 
//GO(va_dri_free_drawable, 
//GO(va_dri_free_drawable_hashtable, 
//GO(va_dri_get_drawable, 
//GO(va_dri_get_rendering_buffer, 
//GO(va_dri_swap_buffer, 
GO(vaGetDisplay, pFp)
//GO(va_isDRI2Connected, 
//GO(va_isRenderNodeFd, 
GO(vaPutSurface, iFpuLwwWWwwWWpuu)
