#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(xcb_shm_attach, UFbuuC)
GO(xcb_shm_attach_checked, UFbuuC)
GO(xcb_shm_attach_fd, UFbuiC)
GO(xcb_shm_attach_fd_checked, UFbuiC)
GO(xcb_shm_create_pixmap, UFbuuWWCuu)
GO(xcb_shm_create_pixmap_checked, UFbuuWWCuu)
GO(xcb_shm_create_segment, UFbuuC)
GO(xcb_shm_create_segment_reply, pFbup)
GO(xcb_shm_create_segment_reply_fds, pFbp)
GO(xcb_shm_create_segment_unchecked, UFbuuC)
GO(xcb_shm_detach, UFbu)
GO(xcb_shm_detach_checked, UFbu)
GO(xcb_shm_get_image, UFbuwwWWuCuu)
GO(xcb_shm_get_image_reply, pFbup)
GO(xcb_shm_get_image_unchecked, UFbuwwWWuCuu)
DATA(xcb_shm_id, 8)
GO(xcb_shm_put_image, UFbuuWWWWWWwwCCCuu)
GO(xcb_shm_put_image_checked, UFbuuWWWWWWwwCCCuu)
GO(xcb_shm_query_version, UFb)
GO(xcb_shm_query_version_reply, pFbup)
GO(xcb_shm_query_version_unchecked, UFb)
GO(xcb_shm_seg_end, pFpii)
GO(xcb_shm_seg_next, vFp)
