#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(XRRQueryVersion, iFppp)
GO(XRRFreeScreenConfigInfo, vFp)
GO(XRRGetCrtcInfo, pFppL)
GO(XRRListOutputProperties, pFpLp)
GO(XRRQueryExtension, iFppp)
GO(XRRAllocModeInfo, pFpi)
GO(XRRGetProviderProperty, iFpLLlliiLppppp)
GO(XRRListProviderProperties, pFpLp)
GO(XRRRotations, WFpip)
GO(XRRSetCrtcTransform, vFpLpppi)
GO(XRRGetCrtcGammaSize, iFpL)
GO(XRRConfigRotations, WFpp)
GO(XRRGetProviderInfo, pFppL)
GO(XRRConfigureOutputProperty, vFpLLiiip)
GO(XRRSetOutputPrimary, vFpLL)
GO(XRRFreeOutputInfo, vFp)
GO(XRRUpdateConfiguration, iFp)
GO(XRRGetScreenResources, pFpL)
GO(XRRConfigSizes, pFpp)
GO(XRRQueryProviderProperty, pFpLL)
GO(XRRDeleteOutputProperty, vFpLL)
GO(XRRDeleteProviderProperty, vFpLL)
GO(XRRSetScreenConfig, iFppLiWL)
GO(XRRAllocGamma, pFi)
GO(XRRSetScreenSize, vFpLiiii)
GO(XRRSetScreenConfigAndRate, iFppLiWwL)
GO(XRRFreeScreenResources, vFp)
GO(XRRGetOutputPrimary, LFpL)
GO(XRRCreateMode, LFpLp)
GO(XRRConfigCurrentRate, wFp)
GO(XRRDestroyMode, vFpL)
GO(XRRSetCrtcConfig, iFppLLiiLWpi)
GO(XRRConfigCurrentConfiguration, WFpp)
GO(XRRSizes, pFpip)
GO(XRRAddOutputMode, vFpLL)
GO(XRRFreePanning, vFp)
GO(XRRFreeProviderResources, vFp)
GO(XRRChangeProviderProperty, vFpLLLiipi)
GO(XRRGetPanning, pFppL)
GO(XRRSetProviderOffloadSink, iFpLL)
GO(XRRGetScreenResourcesCurrent, pFpL)
GO(XRRConfigTimes, LFpp)
GO(XRRSetCrtcGamma, vFpLp)
GO(XRRSetProviderOutputSource, iFpLL)
GO(XRRGetScreenSizeRange, iFpLpppp)
GO(XRRRates, pFpiip)
GO(XRRFreeProviderInfo, vFp)
GO(XRRConfigRates, pFpip)
GO(XRRQueryOutputProperty, pFpLL)
GO(XRRGetOutputProperty, iFpLLlliiLppppp)
GO(XRRFreeGamma, vFp)
GO(XRRRootToScreen, iFpL)
GO(XRRGetScreenInfo, pFpL)
GO(XRRFreeCrtcInfo, vFp)
GO(XRRGetProviderResources, pFpL)
GO(XRRFreeModeInfo, vFp)
GO(XRRChangeOutputProperty, vFpLLLiipi)
GO(XRRGetCrtcGamma, pFpL)
GO(XRRSetPanning, iFppLp)
GO(XRRSelectInput, vFpLi)
GO(XRRGetCrtcTransform, iFpLp)
GO(XRRTimes, LFpip)
GO(XRRDeleteOutputMode, vFpLL)
GO(XRRGetOutputInfo, pFppL)
GO(XRRConfigureProviderProperty, vFpLLiiip)
GO(XRRAllocateMonitor, pFpi)
GO(XRRGetMonitors, pFpLip)
GO(XRRSetMonitor, vFpLp)
GO(XRRFreeMonitors, vFp)
