#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_64, pFpp)
GO(ubrk_close_64, vFp)
GO(ubrk_countAvailable_64, iFv)
GO(ubrk_current_64, iFp)
GO(ubrk_first_64, iFp)
GO(ubrk_following_64, iFpi)
GO(ubrk_getAvailable_64, pFi)
GO(ubrk_getBinaryRules_64, iFppip)
GO(ubrk_getLocaleByType_64, pFpip)
GO(ubrk_getRuleStatus_64, iFp)
GO(ubrk_getRuleStatusVec_64, iFppip)
GO(ubrk_isBoundary_64, cFpi)
GO(ubrk_last_64, iFp)
GO(ubrk_next_64, iFp)
GO(ubrk_open_64, pFppip)
GO(ubrk_openBinaryRules_64, pFpipip)
GO(ubrk_openRules_64, pFpipipp)
GO(ubrk_preceding_64, iFpi)
GO(ubrk_previous_64, iFp)
GO(ubrk_refreshUText_64, vFppp)
GO(ubrk_safeClone_64, pFpppp)
GO(ubrk_setText_64, vFppip)
GO(ubrk_setUText_64, vFppp)
GO(ubrk_swap_64, iFppipp)

GO(ucnv_getDefaultName_64, pFv)
GO(ucnv_open_64, pFpp)
GO(ucnv_setDefaultName_64, vFp)

GO(ucurr_forLocale_64, iFppip)
GO(ucurr_getName_64, pFppippp)

GO(uloc_canonicalize_64, iFppip)
GO(uloc_countAvailable_64, iFv)
GO(uloc_getAvailable_64, pFi)
GO(uloc_getBaseName_64, iFppip)
GO(uloc_getCharacterOrientation_64, iFpp)
GO(uloc_getCountry_64, iFppip)
GO(uloc_getDefault_64, pFv)
GO(uloc_getDisplayCountry_64, iFpppip)
GO(uloc_getDisplayLanguage_64, iFpppip)
GO(uloc_getDisplayName_64, iFpppip)
GO(uloc_getISO3Country_64, pFp)
GO(uloc_getISO3Language_64, pFp)
GO(uloc_getKeywordValue_64, iFpppip)
GO(uloc_getLanguage_64, iFppip)
GO(uloc_getLCID_64, uFp)
GO(uloc_getName_64, iFppip)
GO(uloc_getParent_64, iFppip)
GO(uloc_setKeywordValue_64, iFpppip)

GO(ures_close_64, vFp)
GO(ures_getByKey_64, pFpppp)
GO(ures_getSize_64, iFp)
GO(ures_getStringByIndex_64, pFpipp)
GO(ures_open_64, pFppp)

GO(uldn_close_64, vFp)
GO(uldn_keyValueDisplayName_64, iFppppip)
GO(uldn_open_64, pFpip)

GO(uenum_close_64, vFp)
GO(uenum_count_64, iFpp)
GO(uenum_next_64, pFppp)

GO(uidna_close_64, vFp)
GO(uidna_nameToASCII_64, iFppipipp)
GO(uidna_nameToUnicode_64, iFppipipp)
GO(uidna_openUTS46_64, pFpp)

GO(unorm2_getNFCInstance_64, pFp)
GO(unorm2_getNFDInstance_64, pFp)
GO(unorm2_getNFKCInstance_64, pFp)
GO(unorm2_getNFKDInstance_64, pFp)
GO(unorm2_isNormalized_64, CFppip)
GO(unorm2_normalize_64, iFppipip)

GO(u_charsToUChars_64, vFppi)
GO(u_getVersion_64, vFp)
GO(u_strcmp_64, iFpp)
GO(u_strcpy_64, pFpp)
GO(u_strlen_64, iFp)
GO(u_strncpy_64, pFppi)
GO(u_strToLower_64, iFpipipp)
GO(u_strToUpper_64, iFpipipp)
GO(u_tolower_64, uFu)
GO(u_toupper_64, uFu)
GO(u_uastrncpy_64, pFppi)
