/*********************************************************************
 * File automatically generated by rebuild_wrappers_32.py (v0.0.2.2) *
 *********************************************************************/
#ifndef __wrappeddbusTYPES32_H_
#define __wrappeddbusTYPES32_H_

#ifndef LIBNAME
#error You should only #include this file inside a wrapped*.c file
#endif
#ifndef ADDED_FUNCTIONS
#define ADDED_FUNCTIONS() 
#endif

typedef void (*vFp_t)(void*);
typedef void (*vFppp_t)(void*, void*, void*);
typedef uint32_t (*uFpip_t)(void*, int32_t, void*);
typedef uint32_t (*uFpiV_t)(void*, int32_t, ...);
typedef uint32_t (*uFbppuiiiiiiiiipp_ip_t)(struct_ppuiiiiiiiiipp_t*, int32_t, void*);
typedef void (*vFpppp_t)(void*, void*, void*, void*);
typedef uint32_t (*uFpipp_t)(void*, int32_t, void*, void*);
typedef uint32_t (*uFpppp_t)(void*, void*, void*, void*);
typedef uint32_t (*uFpbppuuuuup_ip_t)(void*, struct_ppuuuuup_t*, int32_t, void*);
typedef uint32_t (*uFpbppuuuuup_iV_t)(void*, struct_ppuuuuup_t*, int32_t, ...);
typedef uint32_t (*uFppppbppuuuuup__t)(void*, void*, void*, void*, struct_ppuuuuup_t*);
typedef uint32_t (*uFpppppp_t)(void*, void*, void*, void*, void*, void*);

#define SUPER() ADDED_FUNCTIONS() \
	GO(dbus_free_string_array, vFp_t) \
	GO(dbus_connection_remove_filter, vFppp_t) \
	GO(dbus_timeout_set_data, vFppp_t) \
	GO(dbus_watch_set_data, vFppp_t) \
	GO(dbus_message_append_args, uFpiV_t) \
	GO(dbus_message_iter_append_basic, uFbppuiiiiiiiiipp_ip_t) \
	GO(dbus_connection_set_dispatch_status_function, vFpppp_t) \
	GO(dbus_connection_set_wakeup_main_function, vFpppp_t) \
	GO(dbus_connection_set_data, uFpipp_t) \
	GO(dbus_connection_add_filter, uFpppp_t) \
	GO(dbus_pending_call_set_notify, uFpppp_t) \
	GO(dbus_message_get_args, uFpbppuuuuup_iV_t) \
	GO(dbus_connection_try_register_object_path, uFppppbppuuuuup__t) \
	GO(dbus_connection_set_timeout_functions, uFpppppp_t) \
	GO(dbus_connection_set_watch_functions, uFpppppp_t)

#endif // __wrappeddbusTYPES32_H_
