#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GO(FREEBL_GetVector, 
//GO(NSSLOWHASH_Begin, 
//GO(NSSLOWHASH_Destroy, 
//GO(NSSLOWHASH_End, 
//GO(NSSLOWHASH_Length, 
//GO(NSSLOWHASH_NewContext, 
//GO(NSSLOWHASH_Update, 
//GO(NSSLOW_Init, 
//GO(NSSLOW_Shutdown, 
