#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(bpf_dump, vFpi)
GO(bpf_filter, uFppuu)
GO(bpf_image, pFpi)
GO(bpf_validate, iFpi)
//DATA(eproto_db, 
GO(pcap_activate, iFp)
GO(pcap_breakloop, vFp)
GO(pcap_bufsize, iFp)
GO(pcap_can_set_rfmon, iFp)
GO(pcap_close, vFp)
GO(pcap_compile, iFpppiu)
GO(pcap_compile_nopcap, iFiippiu)
GO(pcap_create, pFpp)
GO(pcap_datalink, iFp)
GO(pcap_datalink_ext, iFp)
GO(pcap_datalink_name_to_val, iFp)
GO(pcap_datalink_val_to_description, pFi)
GO(pcap_datalink_val_to_description_or_dlt, pFi)
GO(pcap_datalink_val_to_name, pFi)
//GOM(pcap_dispatch, iFEpipp)
GO(pcap_dump, vFppp)
GO(pcap_dump_close, vFp)
GO(pcap_dump_file, SFp)
GO(pcap_dump_flush, iFp)
GO(pcap_dump_fopen, pFpS)
GO(pcap_dump_ftell, lFp)
GO(pcap_dump_ftell64, IFp)
GO(pcap_dump_open, pFpp)
GO(pcap_dump_open_append, pFpp)
GO(pcap_ether_aton, pFp)
GO(pcap_ether_hostton, pFp)
GO(pcap_file, SFp)
GO(pcap_fileno, iFp)
GO(pcap_findalldevs, iFpp)
GO(pcap_fopen_offline, pFSp)
GO(pcap_fopen_offline_with_tstamp_precision, pFSup)
GO(pcap_freealldevs, vFp)
GO(pcap_freecode, vFp)
GO(pcap_free_datalinks, vFp)
GO(pcap_free_tstamp_types, vFp)
GO(pcap_geterr, pFp)
GO(pcap_getnonblock, iFpp)
GO(pcap_get_required_select_timeout, pFp)
GO(pcap_get_selectable_fd, iFp)
GO(pcap_get_tstamp_precision, iFp)
GO(pcap_init, iFup)
GO(pcap_inject, iFppL)
GO(pcap_is_swapped, iFp)
GO(pcap_lib_version, pFv)
GO(pcap_list_datalinks, iFpp)
GO(pcap_list_tstamp_types, iFpp)
GO(pcap_lookupdev, pFp)
GO(pcap_lookupnet, iFpppp)
//GOM(pcap_loop, iFEpipp)
GO(pcap_major_version, iFp)
GO(pcap_minor_version, iFp)
GO(pcap_nametoaddr, pFp)
GO(pcap_nametoaddrinfo, pFp)
GO(pcap_nametoeproto, iFp)
GO(pcap_nametollc, iFp)
GO(pcap_nametonetaddr, uFp)
GO(pcap_nametoport, iFppp)
GO(pcap_nametoportrange, iFpppp)
GO(pcap_nametoproto, iFp)
GO(pcap_next, pFpp)
GO(pcap_next_etherent, pFS)
GO(pcap_next_ex, iFppp)
GO(pcap_offline_filter, iFppp)
GO(pcap_open_dead, pFii)
GO(pcap_open_dead_with_tstamp_precision, pFiiu)
GO(pcap_open_live, pFpiiip)
GO(pcap_open_offline, pFpp)
GO(pcap_open_offline_with_tstamp_precision, pFpup)
GO(pcap_perror, vFpp)
GO(pcap_sendpacket, iFppi)
GO(pcap_set_buffer_size, iFpi)
GO(pcap_set_datalink, iFpi)
GO(pcap_setdirection, iFpu)
GO(pcap_setfilter, iFpp)
GO(pcap_set_immediate_mode, iFpi)
GO(pcap_setnonblock, iFpip)
GO(pcap_set_promisc, iFpi)
GO(pcap_set_protocol_linux, iFpi)
GO(pcap_set_rfmon, iFpi)
GO(pcap_set_snaplen, iFpi)
GO(pcap_set_timeout, iFpi)
GO(pcap_set_tstamp_precision, iFpi)
GO(pcap_set_tstamp_type, iFpi)
GO(pcap_snapshot, iFp)
GO(pcap_stats, iFpp)
GO(pcap_statustostr, pFi)
GO(pcap_strerror, pFi)
GO(pcap_tstamp_type_name_to_val, iFp)
GO(pcap_tstamp_type_val_to_description, pFi)
GO(pcap_tstamp_type_val_to_name, pFi)
//DATA(pcap_version, 
