#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GO(PL_ArenaAllocate, 
//GO(PL_ArenaFinish, 
//GO(PL_ArenaGrow, 
//GO(PL_ArenaRelease, 
//GO(PL_ClearArenaPool, 
//GO(PL_CompactArenaPool, 
//GO(PL_CompareStrings, 
//GO(PL_CompareValues, 
//GO(PL_FinishArenaPool, 
//GO(PL_FreeArenaPool, 
//GO(PL_HashString, 
//GO(PL_HashTableAdd, 
//GO(PL_HashTableDestroy, 
//GO(PL_HashTableDump, 
//GO(PL_HashTableEnumerateEntries, 
//GO(PL_HashTableLookup, 
//GO(PL_HashTableLookupConst, 
//GO(PL_HashTableRawAdd, 
//GO(PL_HashTableRawLookup, 
//GO(PL_HashTableRawLookupConst, 
//GO(PL_HashTableRawRemove, 
//GO(PL_HashTableRemove, 
//GO(PL_InitArenaPool, 
//GO(PL_NewHashTable, 
//GO(PL_SizeOfArenaPoolExcludingPool, 