#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ubrk_clone_73, pFpp)
GO(ubrk_close_73, vFp)
GO(ubrk_countAvailable_73, iFv)
GO(ubrk_current_73, iFp)
GO(ubrk_first_73, iFp)
GO(ubrk_following_73, iFpi)
GO(ubrk_getAvailable_73, pFi)
GO(ubrk_getBinaryRules_73, iFppip)
GO(ubrk_getLocaleByType_73, pFpip)
GO(ubrk_getRuleStatus_73, iFp)
GO(ubrk_getRuleStatusVec_73, iFppip)
GO(ubrk_isBoundary_73, cFpi)
GO(ubrk_last_73, iFp)
GO(ubrk_next_73, iFp)
GO(ubrk_open_73, pFppip)
GO(ubrk_openBinaryRules_73, pFpipip)
GO(ubrk_openRules_73, pFpipipp)
GO(ubrk_preceding_73, iFpi)
GO(ubrk_previous_73, iFp)
GO(ubrk_refreshUText_73, vFppp)
GO(ubrk_safeClone_73, pFpppp)
GO(ubrk_setText_73, vFppip)
GO(ubrk_setUText_73, vFppp)
GO(ubrk_swap_73, iFppipp)

GO(ucnv_getDefaultName_73, pFv)
GO(ucnv_open_73, pFpp)
GO(ucnv_setDefaultName_73, vFp)

GO(ucurr_forLocale_73, iFppip)
GO(ucurr_getName_73, pFppippp)

GO(uloc_canonicalize_73, iFppip)
GO(uloc_countAvailable_73, iFv)
GO(uloc_getAvailable_73, pFi)
GO(uloc_getBaseName_73, iFppip)
GO(uloc_getCharacterOrientation_73, iFpp)
GO(uloc_getCountry_73, iFppip)
GO(uloc_getDefault_73, pFv)
GO(uloc_getDisplayCountry_73, iFpppip)
GO(uloc_getDisplayLanguage_73, iFpppip)
GO(uloc_getDisplayName_73, iFpppip)
GO(uloc_getISO3Country_73, pFp)
GO(uloc_getISO3Language_73, pFp)
GO(uloc_getKeywordValue_73, iFpppip)
GO(uloc_getLanguage_73, iFppip)
GO(uloc_getLCID_73, uFp)
GO(uloc_getName_73, iFppip)
GO(uloc_getParent_73, iFppip)
GO(uloc_setKeywordValue_73, iFpppip)

GO(ures_close_73, vFp)
GO(ures_getByKey_73, pFpppp)
GO(ures_getSize_73, iFp)
GO(ures_getStringByIndex_73, pFpipp)
GO(ures_open_73, pFppp)

GO(uldn_close_73, vFp)
GO(uldn_keyValueDisplayName_73, iFppppip)
GO(uldn_open_73, pFpip)

GO(uenum_close_73, vFp)
GO(uenum_count_73, iFpp)
GO(uenum_next_73, pFppp)

GO(uidna_close_73, vFp)
GO(uidna_nameToASCII_73, iFppipipp)
GO(uidna_nameToUnicode_73, iFppipipp)
GO(uidna_openUTS46_73, pFpp)

GO(unorm2_getNFCInstance_73, pFp)
GO(unorm2_getNFDInstance_73, pFp)
GO(unorm2_getNFKCInstance_73, pFp)
GO(unorm2_getNFKDInstance_73, pFp)
GO(unorm2_isNormalized_73, CFppip)
GO(unorm2_normalize_73, iFppipip)

GO(u_charsToUChars_73, vFppi)
GO(u_getVersion_73, vFp)
GO(u_strcmp_73, iFpp)
GO(u_strcpy_73, pFpp)
GO(u_strlen_73, iFp)
GO(u_strncpy_73, pFppi)
GO(u_strToLower_73, iFpipipp)
GO(u_strToUpper_73, iFpipipp)
GO(u_tolower_73, uFu)
GO(u_toupper_73, uFu)
GO(u_uastrncpy_73, pFppi)
