#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

GOM(_ULx86_64_step, iFEp)
GOM(_ULx86_64_get_reg, iFEpip)
GOM(_ULx86_64_get_proc_info, iFEpp)
GOM(_ULx86_64_init_local, iFEpp)
