#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(gst_adapter_available, LFp)
GO(gst_adapter_available_fast, LFp)
GO(gst_adapter_clear, vFp)
GO(gst_adapter_copy, vFppLL)
GO(gst_adapter_copy_bytes, pFpLL)
GO(gst_adapter_distance_from_discont, LFp)
GO(gst_adapter_dts_at_discont, LFp)
GO(gst_adapter_flush, vFpL)
GO(gst_adapter_get_buffer, pFpL)
GO(gst_adapter_get_buffer_fast, pFpL)
GO(gst_adapter_get_buffer_list, pFpL)
GO(gst_adapter_get_list, pFpL)
GO(gst_adapter_get_type, LFv)
GO(gst_adapter_map, pFpL)
GO(gst_adapter_masked_scan_uint32, lFpuuLL)
GO(gst_adapter_masked_scan_uint32_peek, lFpuuLLp)
GO(gst_adapter_new, pFv)
GO(gst_adapter_offset_at_discont, LFp)
GO(gst_adapter_prev_dts, LFpp)
GO(gst_adapter_prev_dts_at_offset, LFpLp)
GO(gst_adapter_prev_offset, LFpp)
GO(gst_adapter_prev_pts, LFpp)
GO(gst_adapter_prev_pts_at_offset, LFpLp)
GO(gst_adapter_pts_at_discont, LFp)
GO(gst_adapter_push, vFpp)
GO(gst_adapter_take, pFpL)
GO(gst_adapter_take_buffer, pFpL)
GO(gst_adapter_take_buffer_fast, pFpL)
GO(gst_adapter_take_buffer_list, pFpL)
GO(gst_adapter_take_list, pFpL)
GO(gst_adapter_unmap, vFp)
GO(gst_aggregator_finish_buffer, iFpp)
GO(gst_aggregator_finish_buffer_list, iFpp)
GO(gst_aggregator_get_allocator, vFppp)
GO(gst_aggregator_get_buffer_pool, pFp)
GO(gst_aggregator_get_latency, LFp)
GO(gst_aggregator_get_type, LFv)
GO(gst_aggregator_negotiate, iFp)
GO(gst_aggregator_pad_drop_buffer, iFp)
GO(gst_aggregator_pad_get_type, LFv)
GO(gst_aggregator_pad_has_buffer, iFp)
GO(gst_aggregator_pad_is_eos, iFp)
GO(gst_aggregator_pad_peek_buffer, pFp)
GO(gst_aggregator_pad_pop_buffer, pFp)
GO(gst_aggregator_peek_next_sample, pFpp)
GO(gst_aggregator_selected_samples, vFpLLLp)
GO(gst_aggregator_set_latency, vFpLL)
GO(gst_aggregator_set_src_caps, vFpp)
GO(gst_aggregator_simple_get_next_time, LFp)
GO(gst_aggregator_start_time_selection_get_type, LFv)
GO(gst_aggregator_update_segment, vFpp)
GO(gst_base_parse_add_index_entry, iFpLLii)
GO(gst_base_parse_convert_default, iFpulup)
GO(gst_base_parse_drain, vFp)
GO(gst_base_parse_finish_frame, iFppi)
GO(gst_base_parse_frame_copy, pFp)
GO(gst_base_parse_frame_free, vFp)
GO(gst_base_parse_frame_get_type, LFv)
GO(gst_base_parse_frame_init, vFp)
GO(gst_base_parse_frame_new, pFpui)
GO(gst_base_parse_get_type, LFv)
GO(gst_base_parse_merge_tags, vFppu)
GO(gst_base_parse_push_frame, iFpp)
GO(gst_base_parse_set_average_bitrate, vFpu)
GO(gst_base_parse_set_duration, vFpuli)
GO(gst_base_parse_set_frame_rate, vFpuuuu)
GO(gst_base_parse_set_has_timing_info, vFpi)
GO(gst_base_parse_set_infer_ts, vFpi)
GO(gst_base_parse_set_latency, vFpLL)
GO(gst_base_parse_set_min_frame_size, vFpu)
GO(gst_base_parse_set_passthrough, vFpi)
GO(gst_base_parse_set_pts_interpolation, vFpi)
GO(gst_base_parse_set_syncable, vFpi)
GO(gst_base_parse_set_ts_at_offset, vFpL)
GO(gst_base_sink_do_preroll, iFpp)
GO(gst_base_sink_get_blocksize, uFp)
GO(gst_base_sink_get_drop_out_of_segment, iFp)
GO(gst_base_sink_get_last_sample, pFp)
GO(gst_base_sink_get_latency, LFp)
GO(gst_base_sink_get_max_bitrate, LFp)
GO(gst_base_sink_get_max_lateness, lFp)
GO(gst_base_sink_get_processing_deadline, LFp)
GO(gst_base_sink_get_render_delay, LFp)
GO(gst_base_sink_get_stats, pFp)
GO(gst_base_sink_get_sync, iFp)
GO(gst_base_sink_get_throttle_time, LFp)
GO(gst_base_sink_get_ts_offset, lFp)
GO(gst_base_sink_get_type, LFv)
GO(gst_base_sink_is_async_enabled, iFp)
GO(gst_base_sink_is_last_sample_enabled, iFp)
GO(gst_base_sink_is_qos_enabled, iFp)
GO(gst_base_sink_query_latency, iFppppp)
GO(gst_base_sink_set_async_enabled, vFpi)
GO(gst_base_sink_set_blocksize, vFpu)
GO(gst_base_sink_set_drop_out_of_segment, vFpi)
GO(gst_base_sink_set_last_sample_enabled, vFpi)
GO(gst_base_sink_set_max_bitrate, vFpL)
GO(gst_base_sink_set_max_lateness, vFpl)
GO(gst_base_sink_set_processing_deadline, vFpL)
GO(gst_base_sink_set_qos_enabled, vFpi)
GO(gst_base_sink_set_render_delay, vFpL)
GO(gst_base_sink_set_sync, vFpi)
GO(gst_base_sink_set_throttle_time, vFpL)
GO(gst_base_sink_set_ts_offset, vFpl)
GO(gst_base_sink_wait, iFpLp)
GO(gst_base_sink_wait_clock, uFpLp)
GO(gst_base_sink_wait_preroll, iFp)
GO(gst_base_src_get_allocator, vFppp)
GO(gst_base_src_get_blocksize, uFp)
GO(gst_base_src_get_buffer_pool, pFp)
GO(gst_base_src_get_do_timestamp, iFp)
GO(gst_base_src_get_type, LFv)
GO(gst_base_src_is_async, iFp)
GO(gst_base_src_is_live, iFp)
GO(gst_base_src_negotiate, iFp)
GO(gst_base_src_new_seamless_segment, iFplll)
GO(gst_base_src_new_segment, iFpp)
GO(gst_base_src_query_latency, iFpppp)
GO(gst_base_src_set_async, vFpi)
GO(gst_base_src_set_automatic_eos, vFpi)
GO(gst_base_src_set_blocksize, vFpu)
GO(gst_base_src_set_caps, iFpp)
GO(gst_base_src_set_do_timestamp, vFpi)
GO(gst_base_src_set_dynamic_size, vFpi)
GO(gst_base_src_set_format, vFpu)
GO(gst_base_src_set_live, vFpi)
GO(gst_base_src_start_complete, vFpi)
GO(gst_base_src_start_wait, iFp)
GO(gst_base_src_submit_buffer_list, vFpp)
GO(gst_base_src_wait_playing, iFp)
GO(gst_base_transform_get_allocator, vFppp)
GO(gst_base_transform_get_buffer_pool, pFp)
GO(gst_base_transform_get_type, LFv)
GO(gst_base_transform_is_in_place, iFp)
GO(gst_base_transform_is_passthrough, iFp)
GO(gst_base_transform_is_qos_enabled, iFp)
GO(gst_base_transform_reconfigure, iFp)
GO(gst_base_transform_reconfigure_sink, vFp)
GO(gst_base_transform_reconfigure_src, vFp)
GO(gst_base_transform_set_gap_aware, vFpi)
GO(gst_base_transform_set_in_place, vFpi)
GO(gst_base_transform_set_passthrough, vFpi)
GO(gst_base_transform_set_prefer_passthrough, vFpi)
GO(gst_base_transform_set_qos_enabled, vFpi)
GO(gst_base_transform_update_qos, vFpdlL)
GO(gst_base_transform_update_src_caps, iFpp)
GO(gst_bit_reader_free, vFp)
GO(gst_bit_reader_get_bits_uint16, iFppu)
GO(gst_bit_reader_get_bits_uint32, iFppu)
GO(gst_bit_reader_get_bits_uint64, iFppu)
GO(gst_bit_reader_get_bits_uint8, iFppu)
GO(gst_bit_reader_get_pos, uFp)
GO(gst_bit_reader_get_remaining, uFp)
GO(gst_bit_reader_get_size, uFp)
GO(gst_bit_reader_init, vFppu)
GO(gst_bit_reader_new, pFpu)
GO(gst_bit_reader_peek_bits_uint16, iFppu)
GO(gst_bit_reader_peek_bits_uint32, iFppu)
GO(gst_bit_reader_peek_bits_uint64, iFppu)
GO(gst_bit_reader_peek_bits_uint8, iFppu)
GO(gst_bit_reader_set_pos, iFpu)
GO(gst_bit_reader_skip, iFpu)
GO(gst_bit_reader_skip_to_byte, iFp)
GO(gst_bit_writer_align_bytes, iFpC)
GO(gst_bit_writer_free, vFp)
GO(gst_bit_writer_free_and_get_buffer, pFp)
GO(gst_bit_writer_free_and_get_data, pFp)
GO(gst_bit_writer_get_data, pFp)
GO(gst_bit_writer_get_size, uFp)
GO(gst_bit_writer_init, vFp)
GO(gst_bit_writer_init_with_data, vFppui)
GO(gst_bit_writer_init_with_size, vFpui)
GO(gst_bit_writer_new, pFv)
GO(gst_bit_writer_new_with_data, pFpui)
GO(gst_bit_writer_new_with_size, pFui)
GO(gst_bit_writer_put_bits_uint16, iFpWu)
GO(gst_bit_writer_put_bits_uint32, iFpuu)
GO(gst_bit_writer_put_bits_uint64, iFpLu)
GO(gst_bit_writer_put_bits_uint8, iFpCu)
GO(gst_bit_writer_put_bytes, iFppu)
GO(gst_bit_writer_reset, vFp)
GO(gst_bit_writer_reset_and_get_buffer, pFp)
GO(gst_bit_writer_reset_and_get_data, pFp)
GO(gst_bit_writer_set_pos, iFpu)
GO(gst_byte_reader_dup_data, iFpup)
GO(gst_byte_reader_dup_string_utf16, iFpp)
GO(gst_byte_reader_dup_string_utf32, iFpp)
GO(gst_byte_reader_dup_string_utf8, iFpp)
GO(gst_byte_reader_free, vFp)
GO(gst_byte_reader_get_data, iFpup)
GO(gst_byte_reader_get_float32_be, iFpp)
GO(gst_byte_reader_get_float32_le, iFpp)
GO(gst_byte_reader_get_float64_be, iFpp)
GO(gst_byte_reader_get_float64_le, iFpp)
GO(gst_byte_reader_get_int16_be, iFpp)
GO(gst_byte_reader_get_int16_le, iFpp)
GO(gst_byte_reader_get_int24_be, iFpp)
GO(gst_byte_reader_get_int24_le, iFpp)
GO(gst_byte_reader_get_int32_be, iFpp)
GO(gst_byte_reader_get_int32_le, iFpp)
GO(gst_byte_reader_get_int64_be, iFpp)
GO(gst_byte_reader_get_int64_le, iFpp)
GO(gst_byte_reader_get_int8, iFpp)
GO(gst_byte_reader_get_pos, uFp)
GO(gst_byte_reader_get_remaining, uFp)
GO(gst_byte_reader_get_size, uFp)
GO(gst_byte_reader_get_string_utf8, iFpp)
GO(gst_byte_reader_get_sub_reader, iFppu)
GO(gst_byte_reader_get_uint16_be, iFpp)
GO(gst_byte_reader_get_uint16_le, iFpp)
GO(gst_byte_reader_get_uint24_be, iFpp)
GO(gst_byte_reader_get_uint24_le, iFpp)
GO(gst_byte_reader_get_uint32_be, iFpp)
GO(gst_byte_reader_get_uint32_le, iFpp)
GO(gst_byte_reader_get_uint64_be, iFpp)
GO(gst_byte_reader_get_uint64_le, iFpp)
GO(gst_byte_reader_get_uint8, iFpp)
GO(gst_byte_reader_init, vFppu)
GO(gst_byte_reader_masked_scan_uint32, uFpuuuu)
GO(gst_byte_reader_masked_scan_uint32_peek, uFpuuuup)
GO(gst_byte_reader_new, pFpu)
GO(gst_byte_reader_peek_data, iFpup)
GO(gst_byte_reader_peek_float32_be, iFpp)
GO(gst_byte_reader_peek_float32_le, iFpp)
GO(gst_byte_reader_peek_float64_be, iFpp)
GO(gst_byte_reader_peek_float64_le, iFpp)
GO(gst_byte_reader_peek_int16_be, iFpp)
GO(gst_byte_reader_peek_int16_le, iFpp)
GO(gst_byte_reader_peek_int24_be, iFpp)
GO(gst_byte_reader_peek_int24_le, iFpp)
GO(gst_byte_reader_peek_int32_be, iFpp)
GO(gst_byte_reader_peek_int32_le, iFpp)
GO(gst_byte_reader_peek_int64_be, iFpp)
GO(gst_byte_reader_peek_int64_le, iFpp)
GO(gst_byte_reader_peek_int8, iFpp)
GO(gst_byte_reader_peek_string_utf8, iFpp)
GO(gst_byte_reader_peek_sub_reader, iFppu)
GO(gst_byte_reader_peek_uint16_be, iFpp)
GO(gst_byte_reader_peek_uint16_le, iFpp)
GO(gst_byte_reader_peek_uint24_be, iFpp)
GO(gst_byte_reader_peek_uint24_le, iFpp)
GO(gst_byte_reader_peek_uint32_be, iFpp)
GO(gst_byte_reader_peek_uint32_le, iFpp)
GO(gst_byte_reader_peek_uint64_be, iFpp)
GO(gst_byte_reader_peek_uint64_le, iFpp)
GO(gst_byte_reader_peek_uint8, iFpp)
GO(gst_byte_reader_set_pos, iFpu)
GO(gst_byte_reader_skip, iFpu)
GO(gst_byte_reader_skip_string_utf16, iFp)
GO(gst_byte_reader_skip_string_utf32, iFp)
GO(gst_byte_reader_skip_string_utf8, iFp)
GO(gst_byte_writer_ensure_free_space, iFpu)
GO(gst_byte_writer_fill, iFpCu)
GO(gst_byte_writer_free, vFp)
GO(gst_byte_writer_free_and_get_buffer, pFp)
GO(gst_byte_writer_free_and_get_data, pFp)
GO(gst_byte_writer_get_remaining, uFp)
GO(gst_byte_writer_init, vFp)
GO(gst_byte_writer_init_with_data, vFppui)
GO(gst_byte_writer_init_with_size, vFpui)
GO(gst_byte_writer_new, pFv)
GO(gst_byte_writer_new_with_data, pFpui)
GO(gst_byte_writer_new_with_size, pFui)
GO(gst_byte_writer_put_data, iFppu)
GO(gst_byte_writer_put_float32_be, iFpf)
GO(gst_byte_writer_put_float32_le, iFpf)
GO(gst_byte_writer_put_float64_be, iFpd)
GO(gst_byte_writer_put_float64_le, iFpd)
GO(gst_byte_writer_put_int16_be, iFpw)
GO(gst_byte_writer_put_int16_le, iFpw)
GO(gst_byte_writer_put_int24_be, iFpi)
GO(gst_byte_writer_put_int24_le, iFpi)
GO(gst_byte_writer_put_int32_be, iFpi)
GO(gst_byte_writer_put_int32_le, iFpi)
GO(gst_byte_writer_put_int64_be, iFpl)
GO(gst_byte_writer_put_int64_le, iFpl)
GO(gst_byte_writer_put_int8, iFpc)
GO(gst_byte_writer_put_string_utf16, iFpp)
GO(gst_byte_writer_put_string_utf32, iFpp)
GO(gst_byte_writer_put_string_utf8, iFpp)
GO(gst_byte_writer_put_uint16_be, iFpW)
GO(gst_byte_writer_put_uint16_le, iFpW)
GO(gst_byte_writer_put_uint24_be, iFpu)
GO(gst_byte_writer_put_uint24_le, iFpu)
GO(gst_byte_writer_put_uint32_be, iFpu)
GO(gst_byte_writer_put_uint32_le, iFpu)
GO(gst_byte_writer_put_uint64_be, iFpL)
GO(gst_byte_writer_put_uint64_le, iFpL)
GO(gst_byte_writer_put_uint8, iFpC)
GO(gst_byte_writer_reset, vFp)
GO(gst_byte_writer_reset_and_get_buffer, pFp)
GO(gst_byte_writer_reset_and_get_data, pFp)
GOM(gst_collect_pads_add_pad, pFEppupi)
GO(gst_collect_pads_available, uFp)
GO(gst_collect_pads_clip_running_time, iFppppp)
GO(gst_collect_pads_event_default, iFpppi)
GO(gst_collect_pads_flush, uFppu)
GO(gst_collect_pads_get_type, LFv)
GO(gst_collect_pads_new, pFv)
GO(gst_collect_pads_peek, pFpp)
GO(gst_collect_pads_pop, pFpp)
GO(gst_collect_pads_query_default, iFpppi)
GO(gst_collect_pads_read_buffer, pFppu)
GO(gst_collect_pads_remove_pad, iFpp)
GOM(gst_collect_pads_set_buffer_function, vFEppp)
GOM(gst_collect_pads_set_clip_function, vFEppp)
//GOM(gst_collect_pads_set_compare_function, vFEppp)
GOM(gst_collect_pads_set_event_function, vFEppp)
//GOM(gst_collect_pads_set_flush_function, vFEppp)
GO(gst_collect_pads_set_flushing, vFpi)
GOM(gst_collect_pads_set_function, vFEppp)
GOM(gst_collect_pads_set_query_function, vFEppp)
GO(gst_collect_pads_set_waiting, vFppi)
GO(gst_collect_pads_src_event_default, iFppp)
GO(gst_collect_pads_start, vFp)
GO(gst_collect_pads_stop, vFp)
GO(gst_collect_pads_take_buffer, pFppu)
GO(gst_data_queue_drop_head, iFpL)
GO(gst_data_queue_flush, vFp)
GO(gst_data_queue_get_level, vFpp)
GO(gst_data_queue_get_type, LFv)
GO(gst_data_queue_is_empty, iFp)
GO(gst_data_queue_is_full, iFp)
GO(gst_data_queue_limits_changed, vFp)
GOM(gst_data_queue_new, pFEpppp)
GO(gst_data_queue_peek, iFpp)
GO(gst_data_queue_pop, iFpp)
GO(gst_data_queue_push, iFpp)
GO(gst_data_queue_push_force, iFpp)
GO(gst_data_queue_set_flushing, vFpi)
GO(gst_flow_combiner_add_pad, vFpp)
GO(gst_flow_combiner_clear, vFp)
GO(gst_flow_combiner_free, vFp)
GO(gst_flow_combiner_get_type, LFv)
GO(gst_flow_combiner_new, pFv)
GO(gst_flow_combiner_ref, pFp)
GO(gst_flow_combiner_remove_pad, vFpp)
GO(gst_flow_combiner_reset, vFp)
GO(gst_flow_combiner_unref, vFp)
GO(gst_flow_combiner_update_flow, iFpi)
GO(gst_flow_combiner_update_pad_flow, iFppi)
GO(gst_push_src_get_type, LFv)
GO(gst_queue_array_clear, vFp)
GO(gst_queue_array_drop_element, pFpu)
GO(gst_queue_array_drop_struct, iFpup)
//GOM(gst_queue_array_find, uFEppp)
GO(gst_queue_array_free, vFp)
GO(gst_queue_array_get_length, uFp)
GO(gst_queue_array_is_empty, iFp)
GO(gst_queue_array_new, pFu)
GO(gst_queue_array_new_for_struct, pFLu)
GO(gst_queue_array_peek_head, pFp)
GO(gst_queue_array_peek_head_struct, pFp)
GO(gst_queue_array_peek_nth, pFpu)
GO(gst_queue_array_peek_nth_struct, pFpu)
GO(gst_queue_array_peek_tail, pFp)
GO(gst_queue_array_peek_tail_struct, pFp)
GO(gst_queue_array_pop_head, pFp)
GO(gst_queue_array_pop_head_struct, pFp)
GO(gst_queue_array_pop_tail, pFp)
GO(gst_queue_array_pop_tail_struct, pFp)
GO(gst_queue_array_push_tail, vFpp)
GO(gst_queue_array_push_tail_struct, vFpp)
//GOM(gst_queue_array_set_clear_func, vFEpp)
GO(gst_type_find_helper, pFpL)
GO(gst_type_find_helper_for_buffer, pFppp)
GO(gst_type_find_helper_for_buffer_with_extension, pFpppp)
GO(gst_type_find_helper_for_data, pFppLp)
GO(gst_type_find_helper_for_data_with_extension, pFppLpp)
GO(gst_type_find_helper_for_extension, pFpp)
//GOM(gst_type_find_helper_get_range, pFEpppLpp)
GOM(gst_type_find_helper_get_range_full, iFEpppLppp)
