#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//GO(xcb_test_compare_cursor, 
//GO(xcb_test_compare_cursor_reply, 
//GO(xcb_test_compare_cursor_unchecked, 
GO(xcb_test_fake_input, UFbCCuuwwC)
GO(xcb_test_fake_input_checked, UFbCCuuwwC)
//GO(xcb_test_get_version, 
//GO(xcb_test_get_version_reply, 
//GO(xcb_test_get_version_unchecked, 
//GO(xcb_test_grab_control, 
//GO(xcb_test_grab_control_checked, 
DATA(xcb_test_id, 8)
