#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(pw_buffers_clear, vFp)
GO(pw_buffers_negotiate, iFpupupup)
GO(pw_check_library_version, iFiii)
GO(pw_check_option, iFpp)
GO(pw_client_add_listener, iFpppp)
GO(pw_client_error, pFpuip)
GO(pw_client_get_permissions, pFpuu)
GO(pw_client_info_free, vFp)
GO(pw_client_info_merge, pFppi)
GO(pw_client_info_update, pFpp)
GO(pw_client_update_permissions, pFpup)
GO(pw_client_update_properties, pFpp)
GO(pw_conf_find_match, iFppi)
GO(pw_conf_load_conf, iFppp)
GO(pw_conf_load_conf_for_context, pFpp)
GO(pw_conf_load_state, iFppp)
GO(pw_conf_match_rules, iFpLpppp)
GO(pw_conf_save_state, iFppp)
GO(pw_conf_section_for_each, iFpppp)
GO(pw_conf_section_match_rules, iFppppp)
GO(pw_conf_section_update_props, iFppp)
GO(pw_conf_section_update_props_rules, iFpppp)
GO(pw_context_acquire_loop, pFpp)
GO(pw_context_add_listener, vFpppp)
GO(pw_context_add_spa_lib, iFppp)
GO(pw_context_conf_section_for_each, iFpppp)
GO(pw_context_conf_section_match_rules, iFppppp)
GO(pw_context_conf_update_props, iFppp)
GO(pw_context_connect, pFppL)
GO(pw_context_connect_fd, pFpipL)
GO(pw_context_connect_self, pFppL)
GO(pw_context_create_client, pFpppL)
GO(pw_context_create_core, pFppL)
GO(pw_context_create_device, pFppL)
GO(pw_context_create_factory, pFpppupL)
GO(pw_context_create_link, pFpppppL)
GO(pw_context_create_metadata, pFpppL)
GO(pw_context_create_node, pFppL)
GO(pw_context_create_port, pFpiupL)
GO(pw_context_destroy, vFp)
GO(pw_context_find_export_type, pFpp)
GO(pw_context_find_factory, pFpp)
GO(pw_context_find_global, pFpu)
GO(pw_context_find_protocol, pFpp)
GO(pw_context_find_spa_lib, pFpp)
GO(pw_context_for_each_global, iFppp)
GO(pw_context_get_conf_section, pFpp)
GO(pw_context_get_data_loop, pFp)
GO(pw_context_get_default_core, pFp)
GO(pw_context_get_main_loop, pFp)
GO(pw_context_get_mempool, pFp)
GO(pw_context_get_object, pFpp)
GO(pw_context_get_properties, pFp)
GO(pw_context_get_support, pFpp)
GO(pw_context_get_timer_queue, pFp)
GO(pw_context_get_user_data, pFp)
GO(pw_context_get_work_queue, pFp)
GO(pw_context_load_module, pFpppp)
GO(pw_context_load_spa_handle, pFppp)
GO(pw_context_new, pFppL)
GO(pw_context_parse_conf_section, iFppp)
GO(pw_context_register_export_type, iFpp)
GO(pw_context_release_loop, vFpp)
GO(pw_context_set_object, iFppp)
GO(pw_context_update_properties, iFpp)
GO(pw_control_add_link, iFpp)
GO(pw_control_add_listener, vFpppp)
GO(pw_control_get_port, pFp)
GO(pw_control_remove_link, iFpp)
GO(pw_core_add_listener, iFpppp)
GO(pw_core_add_proxy_listener, vFpppp)
GO(pw_core_create_object, pFpppupL)
GO(pw_core_destroy, pFpp)
GO(pw_core_disconnect, iFp)
GO(pw_core_error, pFpuiip)
//GO(pw_core_errorf, pFp)  // variadic
GO(pw_core_errorv, pFpuiipp)
GO(pw_core_export, pFppppL)
GO(pw_core_find_proxy, pFpu)
GO(pw_core_get_client, pFp)
GO(pw_core_get_context, pFp)
GO(pw_core_get_mempool, pFp)
GO(pw_core_get_properties, pFp)
GO(pw_core_get_registry, pFpuL)
GO(pw_core_get_user_data, pFp)
GO(pw_core_hello, pFpu)
GO(pw_core_info_free, vFp)
GO(pw_core_info_merge, pFppi)
GO(pw_core_info_update, pFpp)
GO(pw_core_pong, pFpui)
GO(pw_core_set_paused, iFpi)
GO(pw_core_steal_fd, iFp)
GO(pw_core_sync, pFpui)
GO(pw_core_update_properties, iFpp)
GO(pw_data_loop_add_listener, vFpppp)
GO(pw_data_loop_destroy, vFp)
GO(pw_data_loop_exit, vFp)
GO(pw_data_loop_get_class, pFp)
GO(pw_data_loop_get_loop, pFp)
GO(pw_data_loop_get_name, pFp)
GO(pw_data_loop_get_thread, pFp)
GO(pw_data_loop_in_thread, iFp)
GO(pw_data_loop_invoke, iFppupLip)
GO(pw_data_loop_new, pFp)
GO(pw_data_loop_set_thread_utils, vFpp)
GO(pw_data_loop_start, iFp)
GO(pw_data_loop_stop, iFp)
GO(pw_data_loop_wait, iFpi)
GO(pw_debug_is_category_enabled, iFp)
GO(pw_deinit, vFv)
GO(pw_device_add_listener, iFpppp)
GO(pw_device_enum_params, pFpiuuup)
GO(pw_device_info_free, vFp)
GO(pw_device_info_merge, pFppi)
GO(pw_device_info_update, pFpp)
GO(pw_device_set_param, pFpuup)
GO(pw_device_subscribe_params, pFppu)
GO(pw_direction_as_string, pFi)
GO(pw_direction_reverse, iFi)
GO(pw_factory_add_listener, iFpppp)
GO(pw_factory_info_free, vFp)
GO(pw_factory_info_merge, pFppi)
GO(pw_factory_info_update, pFpp)
GO(pw_filter_add_listener, vFpppp)
GO(pw_filter_add_port, pFpiiLppu)
GO(pw_filter_connect, iFpipu)
GO(pw_filter_dequeue_buffer, pFp)
GO(pw_filter_destroy, vFp)
GO(pw_filter_disconnect, iFp)
GO(pw_filter_emit_event, iFpp)
GO(pw_filter_flush, iFpi)
GO(pw_filter_get_core, pFp)
GO(pw_filter_get_data_loop, pFp)
GO(pw_filter_get_dsp_buffer, pFpu)
GO(pw_filter_get_name, pFp)
GO(pw_filter_get_node_id, uFp)
GO(pw_filter_get_nsec, UFp)
GO(pw_filter_get_properties, pFpp)
GO(pw_filter_get_state, iFpp)
GO(pw_filter_get_time, iFpp)
GO(pw_filter_is_driving, iFp)
GO(pw_filter_is_lazy, iFp)
GO(pw_filter_new, pFppp)
GO(pw_filter_new_simple, pFppppp)
GO(pw_filter_queue_buffer, iFpp)
GO(pw_filter_remove_port, iFp)
GO(pw_filter_set_active, iFpi)
//GO(pw_filter_set_error, pFp)  // variadic
GO(pw_filter_state_as_string, pFi)
GO(pw_filter_trigger_process, iFp)
GO(pw_filter_update_params, iFpppu)
GO(pw_filter_update_properties, iFppp)
GO(pw_free_strv, vFp)
GO(pw_get_application_name, pFv)
GO(pw_get_client_name, pFv)
GO(pw_get_domain, pFv)
GO(pw_get_host_name, pFv)
GO(pw_get_library_version, pFv)
GO(pw_get_prgname, pFv)
GO(pw_get_support, uFpu)
GO(pw_get_user_name, pFv)
GO(pw_getrandom, LFpLu)
GO(pw_gettext, pFp)
GO(pw_global_add_listener, vFpppp)
GO(pw_global_add_resource, iFpp)
GO(pw_global_bind, iFppuuu)
GO(pw_global_destroy, vFp)
GO(pw_global_for_each_resource, iFppp)
GO(pw_global_get_context, pFp)
GO(pw_global_get_id, uFp)
GO(pw_global_get_object, pFp)
GO(pw_global_get_permissions, uFpp)
GO(pw_global_get_properties, pFp)
GO(pw_global_get_serial, UFp)
GO(pw_global_get_type, pFp)
GO(pw_global_get_version, uFp)
GO(pw_global_is_type, iFpp)
GO(pw_global_new, pFppuuppp)
GO(pw_global_register, iFp)
GO(pw_global_update_keys, iFppp)
GO(pw_global_update_permissions, iFppuu)
GO(pw_impl_client_add_listener, vFpppp)
GO(pw_impl_client_check_permissions, iFpuu)
GO(pw_impl_client_destroy, vFp)
GO(pw_impl_client_find_resource, pFpu)
GO(pw_impl_client_get_context, pFp)
GO(pw_impl_client_get_core_resource, pFp)
GO(pw_impl_client_get_global, pFp)
GO(pw_impl_client_get_info, pFp)
GO(pw_impl_client_get_mempool, pFp)
GO(pw_impl_client_get_properties, pFp)
GO(pw_impl_client_get_protocol, pFp)
GO(pw_impl_client_get_user_data, pFp)
GO(pw_impl_client_register, iFpp)
GO(pw_impl_client_set_busy, vFpi)
GO(pw_impl_client_unref, vFp)
GO(pw_impl_client_update_permissions, iFpup)
GO(pw_impl_client_update_properties, iFpp)
GO(pw_impl_core_add_listener, vFpppp)
GO(pw_impl_core_destroy, vFp)
GO(pw_impl_core_get_global, pFp)
GO(pw_impl_core_get_properties, pFp)
GO(pw_impl_core_get_user_data, pFp)
GO(pw_impl_core_register, iFpp)
GO(pw_impl_core_update_properties, iFpp)
GO(pw_impl_device_add_listener, vFpppp)
GO(pw_impl_device_destroy, vFp)
GO(pw_impl_device_for_each_param, iFpiuuuppp)
GO(pw_impl_device_get_global, pFp)
GO(pw_impl_device_get_implementation, pFp)
GO(pw_impl_device_get_properties, pFp)
GO(pw_impl_device_get_user_data, pFp)
GO(pw_impl_device_register, iFpp)
GO(pw_impl_device_set_implementation, iFpp)
GO(pw_impl_device_set_param, iFpuup)
GO(pw_impl_device_update_properties, iFpp)
GO(pw_impl_factory_add_listener, vFpppp)
GO(pw_impl_factory_create_object, pFpppupu)
GO(pw_impl_factory_destroy, vFp)
GO(pw_impl_factory_get_global, pFp)
GO(pw_impl_factory_get_info, pFp)
GO(pw_impl_factory_get_properties, pFp)
GO(pw_impl_factory_get_user_data, pFp)
GO(pw_impl_factory_register, iFpp)
GO(pw_impl_factory_set_implementation, vFppp)
GO(pw_impl_factory_update_properties, iFpp)
GO(pw_impl_link_add_listener, vFpppp)
GO(pw_impl_link_destroy, vFp)
GO(pw_impl_link_get_context, pFp)
GO(pw_impl_link_get_global, pFp)
GO(pw_impl_link_get_info, pFp)
GO(pw_impl_link_get_input, pFp)
GO(pw_impl_link_get_output, pFp)
GO(pw_impl_link_get_user_data, pFp)
GO(pw_impl_link_register, iFpp)
GO(pw_impl_metadata_add_listener, vFpppp)
GO(pw_impl_metadata_destroy, vFp)
GO(pw_impl_metadata_get_global, pFp)
GO(pw_impl_metadata_get_implementation, pFp)
GO(pw_impl_metadata_get_properties, pFp)
GO(pw_impl_metadata_get_user_data, pFp)
GO(pw_impl_metadata_register, iFpp)
GO(pw_impl_metadata_set_implementation, iFpp)
GO(pw_impl_metadata_set_property, iFpuppp)
//GO(pw_impl_metadata_set_propertyf, pFp)  // variadic
GO(pw_impl_module_add_listener, vFpppp)
GO(pw_impl_module_destroy, vFp)
GO(pw_impl_module_get_context, pFp)
GO(pw_impl_module_get_global, pFp)
GO(pw_impl_module_get_info, pFp)
GO(pw_impl_module_get_properties, pFp)
GO(pw_impl_module_schedule_destroy, vFp)
GO(pw_impl_module_update_properties, iFpp)
GO(pw_impl_node_add_listener, vFpppp)
GO(pw_impl_node_add_rt_listener, vFpppp)
GO(pw_impl_node_add_target, iFpp)
GO(pw_impl_node_destroy, vFp)
GO(pw_impl_node_find_port, pFpiu)
GO(pw_impl_node_for_each_param, iFpiuuuppp)
GO(pw_impl_node_for_each_port, iFpipp)
GO(pw_impl_node_get_context, pFp)
GO(pw_impl_node_get_free_port, pFpi)
GO(pw_impl_node_get_free_port_id, uFpi)
GO(pw_impl_node_get_global, pFp)
GO(pw_impl_node_get_implementation, pFp)
GO(pw_impl_node_get_info, pFp)
GO(pw_impl_node_get_properties, pFp)
GO(pw_impl_node_get_user_data, pFp)
GO(pw_impl_node_initialized, iFp)
GO(pw_impl_node_is_active, iFp)
GO(pw_impl_node_register, iFpp)
GO(pw_impl_node_remove_rt_listener, vFpp)
GO(pw_impl_node_remove_target, iFpp)
GO(pw_impl_node_send_command, iFpp)
GO(pw_impl_node_set_active, iFpi)
GO(pw_impl_node_set_driver, iFpp)
GO(pw_impl_node_set_implementation, iFpp)
GO(pw_impl_node_set_io, iFpupL)
GO(pw_impl_node_set_param, iFpuup)
GO(pw_impl_node_set_state, iFpi)
GO(pw_impl_node_update_properties, iFpp)
GO(pw_impl_port_add, iFpp)
GO(pw_impl_port_add_listener, vFpppp)
GO(pw_impl_port_get_direction, iFp)
GO(pw_impl_port_get_id, uFp)
GO(pw_impl_port_get_info, pFp)
GO(pw_impl_port_get_node, pFp)
GO(pw_impl_port_get_properties, pFp)
GO(pw_impl_port_get_user_data, pFp)
GO(pw_impl_port_init_mix, iFpp)
GO(pw_impl_port_is_linked, iFp)
GO(pw_impl_port_release_mix, iFpp)
GO(pw_impl_port_set_mix, iFppuu)
GO(pw_impl_port_set_param, iFpuup)
GO(pw_impl_port_update_properties, iFpp)
GO(pw_impl_port_use_buffers, iFppupu)
GO(pw_init, vFpp)
GO(pw_link_add_listener, iFpppp)
GO(pw_link_info_free, vFp)
GO(pw_link_info_merge, pFppi)
GO(pw_link_info_update, pFpp)
GO(pw_link_state_as_string, pFi)
GO(pw_load_spa_handle, pFpppup)
GO(pw_log_get, pFv)
//GO(pw_log_log, pFp)  // variadic
GO(pw_log_logt, pFpppppp)
GO(pw_log_logtv, vFippippp)
GO(pw_log_logv, vFipippp)
GO(pw_log_set, vFp)
GO(pw_log_set_level, vFi)
GO(pw_log_set_level_string, iFp)
GO(pw_log_topic_register, vFp)
GO(pw_log_topic_unregister, vFp)
GO(pw_loop_accept, pFp)
GO(pw_loop_add_event, pFppp)
GO(pw_loop_add_hook, pFpppp)
GO(pw_loop_add_idle, pFpipp)
GO(pw_loop_add_io, pFpiuipp)
GO(pw_loop_add_signal, pFpipp)
GO(pw_loop_add_source, iFpp)
GO(pw_loop_add_timer, pFppp)
GO(pw_loop_check, pFp)
GO(pw_loop_destroy, vFp)
GO(pw_loop_destroy_source, pFpp)
GO(pw_loop_enable_idle, pFppi)
GO(pw_loop_enter, pFp)
GO(pw_loop_get_fd, pFp)
GO(pw_loop_get_time, pFppI)
GO(pw_loop_invoke, pFppupLip)
GO(pw_loop_iterate, pFpi)
GO(pw_loop_leave, pFp)
GO(pw_loop_lock, pFp)
GO(pw_loop_locked, pFppupLp)
GO(pw_loop_new, pFp)
GO(pw_loop_remove_source, pFpp)
GO(pw_loop_set_name, iFpp)
GO(pw_loop_signal, pFpi)
GO(pw_loop_signal_event, pFpp)
GO(pw_loop_unlock, pFp)
GO(pw_loop_update_io, pFppu)
GO(pw_loop_update_source, pFpp)
GO(pw_loop_update_timer, pFppppi)
GO(pw_loop_wait, pFpp)
GO(pw_main_loop_add_listener, vFpppp)
GO(pw_main_loop_destroy, vFp)
GO(pw_main_loop_get_loop, pFp)
GO(pw_main_loop_new, pFp)
GO(pw_main_loop_quit, iFp)
GO(pw_main_loop_run, iFp)
GO(pw_map_range_init, vFpuuu)
GO(pw_memblock_free, vFp)
GO(pw_memblock_map, pFpiuuu)
GO(pw_memblock_unref, vFp)
GO(pw_memmap_free, iFp)
GO(pw_mempool_add_listener, vFpppp)
GO(pw_mempool_alloc, pFpiuL)
GO(pw_mempool_clear, vFp)
GO(pw_mempool_destroy, vFp)
GO(pw_mempool_find_fd, pFpi)
GO(pw_mempool_find_id, pFpu)
GO(pw_mempool_find_ptr, pFpp)
GO(pw_mempool_find_tag, pFpuL)
GO(pw_mempool_import, pFpiui)
GO(pw_mempool_import_block, pFpp)
GO(pw_mempool_import_map, pFpppuu)
GO(pw_mempool_map_id, pFpuiuuu)
GO(pw_mempool_new, pFp)
GO(pw_mempool_remove_id, pFpu)
GO(pw_module_add_listener, iFpppp)
GO(pw_module_info_free, vFp)
GO(pw_module_info_merge, pFppi)
GO(pw_module_info_update, pFpp)
GO(pw_ngettext, pFppL)
GO(pw_node_add_listener, iFpppp)
GO(pw_node_enum_params, pFpiuuup)
GO(pw_node_info_free, vFp)
GO(pw_node_info_merge, pFppi)
GO(pw_node_info_update, pFpp)
GO(pw_node_peer_ref, pFpp)
GO(pw_node_peer_unref, vFp)
GO(pw_node_send_command, pFpp)
GO(pw_node_set_param, pFpuup)
GO(pw_node_state_as_string, pFi)
GO(pw_node_subscribe_params, pFppu)
GO(pw_port_add_listener, iFpppp)
GO(pw_port_enum_params, pFpiuuup)
GO(pw_port_info_free, vFp)
GO(pw_port_info_merge, pFppi)
GO(pw_port_info_update, pFpp)
GO(pw_port_subscribe_params, pFppu)
GO(pw_properties_add, iFpp)
GO(pw_properties_add_keys, iFppp)
GO(pw_properties_clear, vFp)
GO(pw_properties_copy, pFp)
GO(pw_properties_fetch_bool, iFppp)
GO(pw_properties_fetch_int32, iFppp)
GO(pw_properties_fetch_int64, iFppp)
GO(pw_properties_fetch_uint32, iFppp)
GO(pw_properties_fetch_uint64, iFppp)
GO(pw_properties_free, vFp)
GO(pw_properties_get, pFpp)
GO(pw_properties_get_bool, pFppi)
GO(pw_properties_get_int32, pFppi)
GO(pw_properties_get_int64, pFppI)
GO(pw_properties_get_uint32, uFppu)
GO(pw_properties_get_uint64, pFppU)
GO(pw_properties_iterate, pFpp)
GOM(pw_properties_new, pFEpV)
GO(pw_properties_new_dict, pFp)
GO(pw_properties_new_string, pFp)
GO(pw_properties_new_string_checked, pFpLp)
GO(pw_properties_parse_bool, iFp)
GO(pw_properties_parse_double, pFp)
GO(pw_properties_parse_float, pFp)
GO(pw_properties_parse_int, pFp)
GO(pw_properties_parse_int64, pFp)
GO(pw_properties_parse_uint64, pFp)
GO(pw_properties_serialize_dict, iFppu)
GO(pw_properties_set, iFppp)
GOM(pw_properties_setf, iFEpppV)
GO(pw_properties_setva, iFpppp)
GO(pw_properties_update, iFpp)
GO(pw_properties_update_ignore, iFppp)
GO(pw_properties_update_keys, iFppp)
GO(pw_properties_update_string, iFppL)
GO(pw_properties_update_string_checked, iFppLp)
GO(pw_protocol_add_listener, vFpppp)
GO(pw_protocol_add_marshal, iFpp)
GO(pw_protocol_destroy, vFp)
GO(pw_protocol_get_context, pFp)
GO(pw_protocol_get_extension, pFp)
GO(pw_protocol_get_implementation, pFp)
GO(pw_protocol_get_marshal, pFppuu)
GO(pw_protocol_get_user_data, pFp)
GO(pw_protocol_new, pFppL)
GO(pw_proxy_add_listener, vFpppp)
GO(pw_proxy_add_object_listener, vFpppp)
GO(pw_proxy_destroy, vFp)
GO(pw_proxy_error, iFpip)
//GO(pw_proxy_errorf, pFp)  // variadic
GO(pw_proxy_get_bound_id, uFp)
GO(pw_proxy_get_id, uFp)
GO(pw_proxy_get_marshal, pFp)
GO(pw_proxy_get_object_listeners, pFp)
GO(pw_proxy_get_protocol, pFp)
GO(pw_proxy_get_type, pFpp)
GO(pw_proxy_get_user_data, pFp)
GO(pw_proxy_install_marshal, iFpi)
GO(pw_proxy_new, pFppuL)
GO(pw_proxy_ref, vFp)
GO(pw_proxy_set_bound_id, iFpu)
GO(pw_proxy_sync, iFpi)
GO(pw_proxy_unref, vFp)
GO(pw_random, vFpL)
GO(pw_reallocarray, pFpLL)
GO(pw_registry_add_listener, iFpppp)
GO(pw_registry_bind, pFpupuL)
GO(pw_registry_destroy, pFpu)
GO(pw_resource_add_listener, vFpppp)
GO(pw_resource_add_object_listener, vFpppp)
GO(pw_resource_destroy, vFp)
GO(pw_resource_error, vFpip)
//GO(pw_resource_errorf, pFp)  // variadic
//GO(pw_resource_errorf_id, pFp)  // variadic
GO(pw_resource_get_bound_id, uFp)
GO(pw_resource_get_client, pFp)
GO(pw_resource_get_id, uFp)
GO(pw_resource_get_marshal, pFp)
GO(pw_resource_get_object_listeners, pFp)
GO(pw_resource_get_permissions, uFp)
GO(pw_resource_get_protocol, pFp)
GO(pw_resource_get_type, pFpp)
GO(pw_resource_get_user_data, pFp)
GO(pw_resource_install_marshal, iFpi)
GO(pw_resource_new, pFpuupuL)
GO(pw_resource_ping, iFpi)
GO(pw_resource_ref, vFp)
GO(pw_resource_remove, vFp)
GO(pw_resource_set_bound_id, iFpu)
GO(pw_resource_unref, vFp)
GO(pw_set_domain, iFp)
GO(pw_split_ip, iFppip)
GO(pw_split_strv, pFppip)
GO(pw_split_walk, pFpppp)
GO(pw_stream_add_listener, vFpppp)
GO(pw_stream_connect, iFpiuipu)
GO(pw_stream_dequeue_buffer, pFp)
GO(pw_stream_destroy, vFp)
GO(pw_stream_disconnect, iFp)
GO(pw_stream_emit_event, iFpp)
GO(pw_stream_flush, iFpi)
GO(pw_stream_get_control, pFpu)
GO(pw_stream_get_core, pFp)
GO(pw_stream_get_data_loop, pFp)
GO(pw_stream_get_name, pFp)
GO(pw_stream_get_node_id, uFp)
GO(pw_stream_get_nsec, UFp)
GO(pw_stream_get_properties, pFp)
GO(pw_stream_get_state, iFpp)
GO(pw_stream_get_time, iFpp)
GO(pw_stream_get_time_n, iFppL)
GO(pw_stream_is_driving, iFp)
GO(pw_stream_is_lazy, iFp)
GO(pw_stream_new, pFppp)
GO(pw_stream_new_simple, pFppppp)
GO(pw_stream_queue_buffer, iFpp)
GO(pw_stream_return_buffer, iFpp)
GO(pw_stream_set_active, iFpi)
//GO(pw_stream_set_control, pFp)  // variadic
//GO(pw_stream_set_error, pFp)  // variadic
GO(pw_stream_set_param, iFpup)
GO(pw_stream_set_rate, iFpd)
GO(pw_stream_state_as_string, pFi)
GO(pw_stream_trigger_process, iFp)
GO(pw_stream_update_params, iFppu)
GO(pw_stream_update_properties, iFpp)
GO(pw_strip, pFpp)
GO(pw_strv_find, iFpp)
GO(pw_strv_find_common, iFpp)
GO(pw_strv_parse, pFpLip)
GO(pw_thread_fill_attr, pFpp)
GO(pw_thread_loop_accept, vFp)
GO(pw_thread_loop_add_listener, vFpppp)
GO(pw_thread_loop_destroy, vFp)
GO(pw_thread_loop_get_loop, pFp)
GO(pw_thread_loop_get_time, iFppI)
GO(pw_thread_loop_in_thread, iFp)
GO(pw_thread_loop_lock, vFp)
GO(pw_thread_loop_new, pFpp)
GO(pw_thread_loop_new_full, pFppp)
GO(pw_thread_loop_signal, vFpi)
GO(pw_thread_loop_start, iFp)
GO(pw_thread_loop_stop, vFp)
GO(pw_thread_loop_timed_wait, iFpi)
GO(pw_thread_loop_timed_wait_full, iFpp)
GO(pw_thread_loop_unlock, vFp)
GO(pw_thread_loop_wait, vFp)
GO(pw_thread_utils_acquire_rt, pFpi)
GO(pw_thread_utils_create, pFppp)
GO(pw_thread_utils_drop_rt, pFp)
GO(pw_thread_utils_get, pFv)
GO(pw_thread_utils_get_rt_range, pFppp)
GO(pw_thread_utils_join, pFpp)
GO(pw_thread_utils_set, vFp)
GO(pw_timer_queue_add, iFpppIpp)
GO(pw_timer_queue_cancel, iFp)
GO(pw_timer_queue_destroy, vFp)
GO(pw_timer_queue_new, pFp)
GO(pw_type_info, pFv)
GO(pw_unload_spa_handle, iFp)
GO(pw_work_queue_add, uFppipp)
GO(pw_work_queue_cancel, iFppu)
GO(pw_work_queue_complete, iFppui)

DATA(PW_LOG_TOPIC_DEFAULT, 8)
DATA(pw_log_level, 8)
