#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(alutInit,cFpp)
GO(alutInitWithoutContext,cFpp)
GO(alutExit,cFv)
GO(alutGetError,iFv)
GO(alutGetErrorString,pFi)
GO(alutCreateBufferFromFile,uFp)
GO(alutCreateBufferFromFileImage,uFpi)
GO(alutCreateBufferHelloWorld,uFv)
GO(alutCreateBufferWaveform,uFifff)
GO(alutLoadMemoryFromFile,pFpppp)
GO(alutLoadMemoryFromFileImage,pFpippp)
GO(alutLoadMemoryHelloWorld,pFppp)
GO(alutLoadMemoryWaveform,pFifffppp)
GO(alutGetMIMETypes,pFi)
GO(alutGetMajorVersion,iFv)
GO(alutGetMinorVersion,iFv)
GO(alutSleep,cFf)
GO(alutLoadWAVFile,vFpppppp)
GO(alutLoadWAVMemory,vFpppppp)
GO(alutUnloadWAV,vFipii)