#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

//DATAB(buffers, 8)
//DATAB(check_cond, 16)
//DATAB(check_debug, 8)
GO(check_fork, iFv)
//DATAB(check_major_version, 4)
//DATA(check_micro_version, 4)
//DATA(check_minor_version, 4)
//DATAB(check_mutex, 8)
GO(check_waitpid_and_exit, vFi)
//GOM(_ck_assert_failed, vFEpipV)
//GO(_fail_unless, 
GO(gst_buffer_straw_get_buffer, pFpp)
GO(gst_buffer_straw_start_pipeline, vFpp)
GO(gst_buffer_straw_stop_pipeline, vFpp)
GO(gst_check_abi_list, vFpi)
//GOM(gst_check_add_log_filter, pFEpipppp)
GO(gst_check_buffer_data, vFppL)
GO(gst_check_caps_equal, vFpp)
GO(gst_check_chain_func, iFppp)
GO(gst_check_clear_log_filter, vFv)
GO(gst_check_drop_buffers, vFv)
GO(gst_check_element_push_buffer, vFppppp)
GO(gst_check_element_push_buffer_list, vFpppppi)
//DATAB(_gst_check_expecting_log, 4)
GO(gst_check_init, vFpp)
//DATAB(_gst_check_list_tests, 4)
GO(gst_check_message_error, vFpiui)
GO(gst_check_object_destroyed_on_unref, vFp)
//GOM(gst_check_objects_destroyed_on_unref, vFEppV)
//DATAB(_gst_check_raised_critical, 4)
//DATAB(_gst_check_raised_warning, 4)
GO(gst_check_remove_log_filter, vFp)
GO(gst_check_run_suite, iFppp)
GO(_gst_check_run_test_func, iFp)
GO(gst_check_setup_element, pFp)
GO(gst_check_setup_events, vFpppu)
GO(gst_check_setup_events_with_stream_id, vFpppup)
GO(gst_check_setup_sink_pad, pFpp)
GO(gst_check_setup_sink_pad_by_name, pFppp)
GO(gst_check_setup_sink_pad_by_name_from_template, pFppp)
GO(gst_check_setup_sink_pad_from_template, pFpp)
GO(gst_check_setup_src_pad, pFpp)
GO(gst_check_setup_src_pad_by_name, pFppp)
GO(gst_check_setup_src_pad_by_name_from_template, pFppp)
GO(gst_check_setup_src_pad_from_template, pFpp)
GO(gst_check_teardown_element, vFp)
GO(gst_check_teardown_pad_by_name, vFpp)
GO(gst_check_teardown_sink_pad, vFp)
GO(gst_check_teardown_src_pad, vFp)
//DATAB(_gst_check_threads_running, 4)
GO(gst_consistency_checker_add_pad, iFpp)
GO(gst_consistency_checker_free, vFp)
GO(gst_consistency_checker_new, pFp)
GO(gst_consistency_checker_reset, vFp)
GO(gst_harness_add_element_full, vFpppppp)
GO(gst_harness_add_element_sink_pad, vFpp)
GO(gst_harness_add_element_src_pad, vFpp)
GO(gst_harness_add_parse, vFpp)
//GOM(gst_harness_add_probe, vFEpppuppp)
GO(gst_harness_add_propose_allocation_meta, vFpLp)
GO(gst_harness_add_sink, vFpp)
GO(gst_harness_add_sink_harness, vFpp)
GO(gst_harness_add_sink_parse, vFpp)
GO(gst_harness_add_src, vFppi)
GO(gst_harness_add_src_harness, vFppi)
GO(gst_harness_add_src_parse, vFppi)
GO(gst_harness_buffers_in_queue, uFp)
GO(gst_harness_buffers_received, uFp)
GO(gst_harness_crank_multiple_clock_waits, iFpu)
GO(gst_harness_crank_single_clock_wait, iFp)
GO(gst_harness_create_buffer, pFpL)
GO(gst_harness_dump_to_file, vFpp)
GO(gst_harness_events_in_queue, uFp)
GO(gst_harness_events_received, uFp)
GO(gst_harness_find_element, pFpp)
//GOM(gst_harness_get, vFEpppV)
GO(gst_harness_get_allocator, vFppp)
GO(gst_harness_get_last_pushed_timestamp, LFp)
GO(gst_harness_get_testclock, pFp)
GO(gst_harness_new, pFp)
GO(gst_harness_new_empty, pFv)
GO(gst_harness_new_full, pFppppp)
GO(gst_harness_new_parse, pFp)
GO(gst_harness_new_with_element, pFppp)
GO(gst_harness_new_with_padnames, pFppp)
GO(gst_harness_new_with_templates, pFppp)
GO(gst_harness_play, vFp)
GO(gst_harness_pull, pFp)
GO(gst_harness_pull_event, pFp)
GO(gst_harness_pull_until_eos, iFpp)
GO(gst_harness_pull_upstream_event, pFp)
GO(gst_harness_push, iFpp)
GO(gst_harness_push_and_pull, pFpp)
GO(gst_harness_push_event, iFpp)
GO(gst_harness_push_from_src, iFp)
GO(gst_harness_push_to_sink, iFp)
GO(gst_harness_push_upstream_event, iFpp)
GO(gst_harness_query_latency, LFp)
//GOM(gst_harness_set, vFEpppV)
GO(gst_harness_set_blocking_push_mode, vFp)
GO(gst_harness_set_caps, vFppp)
GO(gst_harness_set_caps_str, vFppp)
GO(gst_harness_set_drop_buffers, vFpi)
GO(gst_harness_set_forwarding, vFpi)
GO(gst_harness_set_live, vFpi)
GO(gst_harness_set_propose_allocator, vFppp)
GO(gst_harness_set_sink_caps, vFpp)
GO(gst_harness_set_sink_caps_str, vFpp)
GO(gst_harness_set_src_caps, vFpp)
GO(gst_harness_set_src_caps_str, vFpp)
GO(gst_harness_set_time, iFpL)
GO(gst_harness_set_upstream_latency, vFpL)
GO(gst_harness_sink_push_many, iFpi)
GO(gst_harness_src_crank_and_push_many, iFpii)
GO(gst_harness_src_push_event, iFp)
//GOM(gst_harness_stress_custom_start, pFEppppL)
GO(gst_harness_stress_property_start_full, pFpppL)
GO(gst_harness_stress_push_buffer_start_full, pFppppL)
//GOM(gst_harness_stress_push_buffer_with_cb_start_full, pFEppppppL)
GO(gst_harness_stress_push_event_start_full, pFppL)
//GOM(gst_harness_stress_push_event_with_cb_start_full, pFEppppL)
GO(gst_harness_stress_push_upstream_event_start_full, pFppL)
//GOM(gst_harness_stress_push_upstream_event_with_cb_start_full, pFEppppL)
GO(gst_harness_stress_requestpad_start_full, pFppppiL)
GO(gst_harness_stress_statechange_start_full, pFpL)
GO(gst_harness_stress_thread_stop, uFp)
GO(gst_harness_take_all_data, pFpp)
GO(gst_harness_take_all_data_as_buffer, pFp)
GO(gst_harness_take_all_data_as_bytes, pFp)
GO(gst_harness_teardown, vFp)
GO(gst_harness_try_pull, pFp)
GO(gst_harness_try_pull_event, pFp)
GO(gst_harness_try_pull_upstream_event, pFp)
GO(gst_harness_upstream_events_in_queue, uFp)
GO(gst_harness_upstream_events_received, uFp)
GO(gst_harness_use_systemclock, vFp)
GO(gst_harness_use_testclock, vFp)
GO(gst_harness_wait_for_clock_id_waits, iFpuu)
GO(gst_test_clock_advance_time, vFpl)
GO(gst_test_clock_crank, iFp)
GO(gst_test_clock_get_next_entry_time, LFp)
GO(gst_test_clock_get_type, LFv)
GO(gst_test_clock_has_id, iFpp)
GO(gst_test_clock_id_list_get_latest_time, LFp)
GO(gst_test_clock_new, pFv)
GO(gst_test_clock_new_with_start_time, pFL)
GO(gst_test_clock_peek_id_count, uFp)
GO(gst_test_clock_peek_next_pending_id, iFpp)
GO(gst_test_clock_process_id, iFpp)
GO(gst_test_clock_process_id_list, uFpp)
GO(gst_test_clock_process_next_clock_id, pFp)
GO(gst_test_clock_set_time, vFpL)
GO(gst_test_clock_timed_wait_for_multiple_pending_ids, iFpuup)
GO(gst_test_clock_wait_for_multiple_pending_ids, vFpup)
GO(gst_test_clock_wait_for_next_pending_id, vFpp)
GO(gst_test_clock_wait_for_pending_id_count, vFpu)
GO(_mark_point, vFpi)
//DATAB(mutex, 8)
GO(srunner_add_suite, vFpp)
GO(srunner_create, pFp)
GO(srunner_failures, pFp)
GO(srunner_fork_status, uFp)
GO(srunner_free, vFp)
GO(srunner_has_log, iFp)
GO(srunner_has_tap, iFp)
GO(srunner_has_xml, iFp)
GO(srunner_log_fname, pFp)
GO(srunner_ntests_failed, iFp)
GO(srunner_ntests_run, iFp)
GO(srunner_print, vFpu)
GO(srunner_results, pFp)
GO(srunner_run, vFpppu)
GO(srunner_run_all, vFpu)
GO(srunner_run_tagged, vFpppppu)
GO(srunner_set_fork_status, vFpu)
GO(srunner_set_log, vFpp)
GO(srunner_set_tap, vFpp)
GO(srunner_set_xml, vFpp)
GO(srunner_tap_fname, pFp)
GO(srunner_xml_fname, pFp)
//DATAB(start_cond, 16)
GO(suite_add_tcase, vFpp)
GO(suite_create, pFp)
GO(suite_tcase, iFpp)
//DATAB(sync_cond, 16)
//GOM(tcase_add_checked_fixture, vFEppp)
//GOM(_tcase_add_test, vFEpppiiii)
//GOM(tcase_add_unchecked_fixture, vFEppp)
GO(tcase_create, pFp)
GO(tcase_fn_start, vFppi)
GO(tcase_set_tags, vFpp)
GO(tcase_set_timeout, vFpd)
//DATAB(thread_list, 8)
GO(tr_ctx, uFp)
GO(tr_lfile, pFp)
GO(tr_lno, iFp)
GO(tr_msg, pFp)
GO(tr_rtype, iFp)
GO(tr_tcname, pFp)
