#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif


GO(nvmlComputeInstanceDestroy, uFp)
//GO(nvmlComputeInstanceGetInfo, 
//GO(nvmlComputeInstanceGetInfo_v2, uFpp) // Warning: failed to confirm
GO(nvmlDeviceClearAccountingPids, uFp)
GO(nvmlDeviceClearCpuAffinity, uFp)
GO(nvmlDeviceClearEccErrorCounts, uFpu)
//GO(nvmlDeviceClearFieldValues, uFpip) // Warning: failed to confirm
GO(nvmlDeviceCreateGpuInstance, uFpubp_)
GO(nvmlDeviceCreateGpuInstanceWithPlacement, uFpupbp_)
GO(nvmlDeviceDiscoverGpus, uFp)
GO(nvmlDeviceFreezeNvLinkUtilizationCounter, uFpuuu)
GO(nvmlDeviceGetAccountingBufferSize, uFpp)
GO(nvmlDeviceGetAccountingMode, uFpp)
GO(nvmlDeviceGetAccountingPids, uFppp)
GO(nvmlDeviceGetAccountingStats, uFpup)
GO(nvmlDeviceGetActiveVgpus, uFppp)
GO(nvmlDeviceGetAdaptiveClockInfoStatus, uFpp)
GO(nvmlDeviceGetAddressingMode, uFpp)
GO(nvmlDeviceGetAPIRestriction, uFpup)
GO(nvmlDeviceGetApplicationsClock, uFpup)
GO(nvmlDeviceGetArchitecture, uFpp)
//GO(nvmlDeviceGetAttributes, 
GO(nvmlDeviceGetAttributes_v2, uFpp)
GO(nvmlDeviceGetAutoBoostedClocksEnabled, uFppp)
GO(nvmlDeviceGetBAR1MemoryInfo, uFpp)
GO(nvmlDeviceGetBoardId, uFpp)
GO(nvmlDeviceGetBoardPartNumber, uFppu)
GO(nvmlDeviceGetBrand, uFpp)
GO(nvmlDeviceGetBridgeChipInfo, uFpp)
GO(nvmlDeviceGetBusType, uFpp)
GO(nvmlDeviceGetC2cModeInfoV, uFpp)
GO(nvmlDeviceGetCapabilities, uFpp)
GO(nvmlDeviceGetClkMonStatus, uFpp)
GO(nvmlDeviceGetClock, uFpuup)
GO(nvmlDeviceGetClockInfo, uFpup)
GO(nvmlDeviceGetClockOffsets, uFpp)
GO(nvmlDeviceGetComputeInstanceId, uFpp)
GO(nvmlDeviceGetComputeMode, uFpp)
//GO(nvmlDeviceGetComputeRunningProcesses, 
//GO(nvmlDeviceGetComputeRunningProcesses_v2, 
GO(nvmlDeviceGetComputeRunningProcesses_v3, uFppbuUuu_)
GO(nvmlDeviceGetConfComputeGpuAttestationReport, uFpp)
GO(nvmlDeviceGetConfComputeGpuCertificate, uFpp)
GO(nvmlDeviceGetConfComputeMemSizeInfo, uFpp)
GO(nvmlDeviceGetConfComputeProtectedMemoryUsage, uFpp)
GO(nvmlDeviceGetCoolerInfo, uFpp)
//GO(nvmlDeviceGetCount, 
GO(nvmlDeviceGetCount_v2, uFp)
GO(nvmlDeviceGetCpuAffinity, uFpubL_)
GO(nvmlDeviceGetCpuAffinityWithinScope, uFpubL_u)
GO(nvmlDeviceGetCreatableVgpus, uFppp)
GO(nvmlDeviceGetCudaComputeCapability, uFppp)
GO(nvmlDeviceGetCurrentClockFreqs, uFpp)
GO(nvmlDeviceGetCurrentClocksEventReasons, uFpp)
GO(nvmlDeviceGetCurrentClocksThrottleReasons, uFpp)
GO(nvmlDeviceGetCurrPcieLinkGeneration, uFpp)
GO(nvmlDeviceGetCurrPcieLinkWidth, uFpp)
GO(nvmlDeviceGetDecoderUtilization, uFppp)
GO(nvmlDeviceGetDefaultApplicationsClock, uFpup)
GO(nvmlDeviceGetDefaultEccMode, uFpp)
GO(nvmlDeviceGetDetailedEccErrors, uFpuup)
GO(nvmlDeviceGetDeviceHandleFromMigDeviceHandle, uFpbp_)
GO(nvmlDeviceGetDisplayActive, uFpp)
GO(nvmlDeviceGetDisplayMode, uFpp)
GO(nvmlDeviceGetDramEncryptionMode, uFppp)
//GO(nvmlDeviceGetDriverModel, 
GO(nvmlDeviceGetDriverModel_v2, uFppp)
GO(nvmlDeviceGetDynamicPstatesInfo, uFpp)
GO(nvmlDeviceGetEccMode, uFppp)
GO(nvmlDeviceGetEncoderCapacity, uFpup)
GO(nvmlDeviceGetEncoderSessions, uFppp)
GO(nvmlDeviceGetEncoderStats, uFpppp)
GO(nvmlDeviceGetEncoderUtilization, uFppp)
GO(nvmlDeviceGetEnforcedPowerLimit, uFpp)
GO(nvmlDeviceGetFanControlPolicy_v2, uFpup)
GO(nvmlDeviceGetFanSpeed, uFpp)
GO(nvmlDeviceGetFanSpeedRPM, uFpp)
GO(nvmlDeviceGetFanSpeed_v2, uFpup)
GO(nvmlDeviceGetFBCSessions, uFppp)
GO(nvmlDeviceGetFBCStats, uFpp)
//GO(nvmlDeviceGetFieldValues, uFpip) // Warning: failed to confirm
GO(nvmlDeviceGetGpcClkMinMaxVfOffset, uFppp)
GO(nvmlDeviceGetGpcClkVfOffset, uFpp)
GO(nvmlDeviceGetGpuFabricInfo, uFpp)
GO(nvmlDeviceGetGpuFabricInfoV, uFpp)
GO(nvmlDeviceGetGpuInstanceById, uFpubp_)
GO(nvmlDeviceGetGpuInstanceId, uFpp)
//GO(nvmlDeviceGetGpuInstancePossiblePlacements, 
GO(nvmlDeviceGetGpuInstancePossiblePlacements_v2, uFpupp)
GO(nvmlDeviceGetGpuInstanceProfileInfo, uFpup)
//GO(nvmlDeviceGetGpuInstanceProfileInfoByIdV, uFpubuuuuuuuuuuuUcccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc_)
//GO(nvmlDeviceGetGpuInstanceProfileInfoV, uFpubuuuuuuuuuuuUcccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc_)
GO(nvmlDeviceGetGpuInstanceRemainingCapacity, uFpup)
GO(nvmlDeviceGetGpuInstances, uFpubp_p)
GO(nvmlDeviceGetGpuMaxPcieLinkGeneration, uFpp)
GO(nvmlDeviceGetGpuOperationMode, uFppp)
//GO(nvmlDeviceGetGraphicsRunningProcesses, 
//GO(nvmlDeviceGetGraphicsRunningProcesses_v2, 
GO(nvmlDeviceGetGraphicsRunningProcesses_v3, uFppbuUuu_)
//GO(nvmlDeviceGetGridLicensableFeatures, 
//GO(nvmlDeviceGetGridLicensableFeatures_v2, 
//GO(nvmlDeviceGetGridLicensableFeatures_v3, 
GO(nvmlDeviceGetGridLicensableFeatures_v4, uFpp)
GO(nvmlDeviceGetGspFirmwareMode, uFppp)
GO(nvmlDeviceGetGspFirmwareVersion, uFpp)
//GO(nvmlDeviceGetHandleByIndex, 
GO(nvmlDeviceGetHandleByIndex_v2, uFubp_)
//GO(nvmlDeviceGetHandleByPciBusId, 
GO(nvmlDeviceGetHandleByPciBusId_v2, uFpbp_)
GO(nvmlDeviceGetHandleBySerial, uFpbp_)
GO(nvmlDeviceGetHandleByUUID, uFpbp_)
GO(nvmlDeviceGetHandleByUUIDV, uFpbp_)
GO(nvmlDeviceGetHostname_v1, uFpp)
GO(nvmlDeviceGetHostVgpuMode, uFpp)
GO(nvmlDeviceGetIndex, uFpp)
GO(nvmlDeviceGetInforomConfigurationChecksum, uFpp)
GO(nvmlDeviceGetInforomImageVersion, uFppu)
GO(nvmlDeviceGetInforomVersion, uFpupu)
GO(nvmlDeviceGetIrqNum, uFpp)
GO(nvmlDeviceGetJpgUtilization, uFppp)
GO(nvmlDeviceGetLastBBXFlushTime, uFppbL_)
GO(nvmlDeviceGetMarginTemperature, uFpp)
GO(nvmlDeviceGetMaxClockInfo, uFpup)
GO(nvmlDeviceGetMaxCustomerBoostClock, uFpup)
GO(nvmlDeviceGetMaxMigDeviceCount, uFpp)
GO(nvmlDeviceGetMaxPcieLinkGeneration, uFpp)
GO(nvmlDeviceGetMaxPcieLinkWidth, uFpp)
GO(nvmlDeviceGetMemClkMinMaxVfOffset, uFppp)
GO(nvmlDeviceGetMemClkVfOffset, uFpp)
GO(nvmlDeviceGetMemoryAffinity, uFpubL_u)
GO(nvmlDeviceGetMemoryBusWidth, uFpp)
GO(nvmlDeviceGetMemoryErrorCounter, uFpuuup)
GO(nvmlDeviceGetMemoryInfo, uFpp)
GO(nvmlDeviceGetMemoryInfo_v2, uFpbuUUUU_)
GO(nvmlDeviceGetMigDeviceHandleByIndex, uFpubp_)
GO(nvmlDeviceGetMigMode, uFppp)
GO(nvmlDeviceGetMinMaxClockOfPState, uFpuupp)
GO(nvmlDeviceGetMinMaxFanSpeed, uFppp)
GO(nvmlDeviceGetMinorNumber, uFpp)
GO(nvmlDeviceGetModuleId, uFpp)
//GO(nvmlDeviceGetMPSComputeRunningProcesses, 
//GO(nvmlDeviceGetMPSComputeRunningProcesses_v2, 
GO(nvmlDeviceGetMPSComputeRunningProcesses_v3, uFppbuUuu_)
GO(nvmlDeviceGetMultiGpuBoard, uFpp)
GO(nvmlDeviceGetName, uFppu)
GO(nvmlDeviceGetNumaNodeId, uFpp)
GO(nvmlDeviceGetNumFans, uFpp)
GO(nvmlDeviceGetNumGpuCores, uFpp)
GO(nvmlDeviceGetNvlinkBwMode, uFpp)
GO(nvmlDeviceGetNvLinkCapability, uFpuup)
GO(nvmlDeviceGetNvLinkErrorCounter, uFpuup)
GO(nvmlDeviceGetNvLinkInfo, uFpp)
GO(nvmlDeviceGetNvLinkRemoteDeviceType, uFpup)
//GO(nvmlDeviceGetNvLinkRemotePciInfo, 
GO(nvmlDeviceGetNvLinkRemotePciInfo_v2, uFpup)
GO(nvmlDeviceGetNvLinkState, uFpup)
GO(nvmlDeviceGetNvlinkSupportedBwModes, uFpp)
GO(nvmlDeviceGetNvLinkUtilizationControl, uFpuup)
GO(nvmlDeviceGetNvLinkUtilizationCounter, uFpuupp)
GO(nvmlDeviceGetNvLinkVersion, uFpup)
GO(nvmlDeviceGetOfaUtilization, uFppp)
GO(nvmlDeviceGetP2PStatus, uFppup)
GO(nvmlDeviceGetPcieLinkMaxSpeed, uFpp)
GO(nvmlDeviceGetPcieReplayCounter, uFpp)
GO(nvmlDeviceGetPcieSpeed, uFpp)
GO(nvmlDeviceGetPcieThroughput, uFpup)
//GO(nvmlDeviceGetPciInfo, 
GO(nvmlDeviceGetPciInfoExt, uFpp)
//GO(nvmlDeviceGetPciInfo_v2, 
GO(nvmlDeviceGetPciInfo_v3, uFpp)
GO(nvmlDeviceGetPdi, uFpp)
GO(nvmlDeviceGetPerformanceModes, uFpp)
GO(nvmlDeviceGetPerformanceState, uFpp)
GO(nvmlDeviceGetPersistenceMode, uFpp)
GO(nvmlDeviceGetPgpuMetadataString, uFppp)
GO(nvmlDeviceGetPlatformInfo, uFpp)
GO(nvmlDeviceGetPowerManagementDefaultLimit, uFpp)
GO(nvmlDeviceGetPowerManagementLimit, uFpp)
GO(nvmlDeviceGetPowerManagementLimitConstraints, uFppp)
GO(nvmlDeviceGetPowerManagementMode, uFpp)
GO(nvmlDeviceGetPowerMizerMode_v1, uFpp)
GO(nvmlDeviceGetPowerSource, uFpp)
GO(nvmlDeviceGetPowerState, uFpp)
GO(nvmlDeviceGetPowerUsage, uFpp)
GO(nvmlDeviceGetProcessesUtilizationInfo, uFpbuuUp_)
GO(nvmlDeviceGetProcessUtilization, uFpbuUuuuu_pU)
GO(nvmlDeviceGetRemappedRows, uFppppp)
GO(nvmlDeviceGetRepairStatus, uFpp)
GO(nvmlDeviceGetRetiredPages, uFpupp)
GO(nvmlDeviceGetRetiredPagesPendingStatus, uFpp)
GO(nvmlDeviceGetRetiredPages_v2, uFpuppp)
GO(nvmlDeviceGetRowRemapperHistogram, uFpp)
//GOM(nvmlDeviceGetRunningProcessDetailList, uFEpbuuup_)
//GO(nvmlDeviceGetSamples, uFpuUppp) // Warning: failed to confirm
GO(nvmlDeviceGetSerial, uFppu)
//GO(nvmlDeviceGetSramEccErrorStatus, uFpbuUUUUUUUUUUUu_)
GO(nvmlDeviceGetSramUniqueUncorrectedEccErrorCounts, uFpbuup_)
GO(nvmlDeviceGetSupportedClocksEventReasons, uFpp)
GO(nvmlDeviceGetSupportedClocksThrottleReasons, uFpp)
GO(nvmlDeviceGetSupportedEventTypes, uFpp)
GO(nvmlDeviceGetSupportedGraphicsClocks, uFpupp)
GO(nvmlDeviceGetSupportedMemoryClocks, uFppp)
GO(nvmlDeviceGetSupportedPerformanceStates, uFppu)
GO(nvmlDeviceGetSupportedVgpus, uFppp)
GO(nvmlDeviceGetTargetFanSpeed, uFpup)
GO(nvmlDeviceGetTemperature, uFpup)
GO(nvmlDeviceGetTemperatureThreshold, uFpup)
GO(nvmlDeviceGetTemperatureV, uFpp)
GO(nvmlDeviceGetThermalSettings, uFpup)
GO(nvmlDeviceGetTopologyCommonAncestor, uFppp)
GO(nvmlDeviceGetTopologyNearestGpus, uFpupbp_)
GO(nvmlDeviceGetTotalEccErrors, uFpuup)
GO(nvmlDeviceGetTotalEnergyConsumption, uFpp)
GO(nvmlDeviceGetUtilizationRates, uFpp)
GO(nvmlDeviceGetUUID, uFppu)
GO(nvmlDeviceGetVbiosVersion, uFppu)
GO(nvmlDeviceGetVgpuCapabilities, uFpup)
GO(nvmlDeviceGetVgpuHeterogeneousMode, uFpp)
//GOM(nvmlDeviceGetVgpuInstancesUtilizationInfo, uFEpbuuuUp_)
GO(nvmlDeviceGetVgpuMetadata, uFppp)
GO(nvmlDeviceGetVgpuProcessesUtilizationInfo, uFpbuuUp_)
GO(nvmlDeviceGetVgpuProcessUtilization, uFpUpp)
GO(nvmlDeviceGetVgpuSchedulerCapabilities, uFpp)
//GO(nvmlDeviceGetVgpuSchedulerLog, uFpp) // Warning: failed to confirm
GO(nvmlDeviceGetVgpuSchedulerState, uFpp)
GO(nvmlDeviceGetVgpuTypeCreatablePlacements, uFpubuuupu_)
GO(nvmlDeviceGetVgpuTypeSupportedPlacements, uFpubuuupu_)
//GO(nvmlDeviceGetVgpuUtilization, uFpUppp) // Warning: failed to confirm
GO(nvmlDeviceGetViolationStatus, uFpup)
GO(nvmlDeviceGetVirtualizationMode, uFpp)
GO(nvmlDeviceIsMigDeviceHandle, uFpp)
GO(nvmlDeviceModifyDrainState, uFpu)
GO(nvmlDeviceOnSameBoard, uFppp)
GO(nvmlDevicePowerSmoothingActivatePresetProfile, uFpp)
GO(nvmlDevicePowerSmoothingSetState, uFpp)
GO(nvmlDevicePowerSmoothingUpdatePresetProfileParam, uFpp)
GO(nvmlDeviceQueryDrainState, uFpp)
//GOM(nvmlDeviceReadPRMCounters_v1, uFEpbup_)
GO(nvmlDeviceReadWritePRM_v1, uFpp)
GO(nvmlDeviceRegisterEvents, uFpUp)
//GO(nvmlDeviceRemoveGpu, 
GO(nvmlDeviceRemoveGpu_v2, uFpuu)
GO(nvmlDeviceResetApplicationsClocks, uFp)
GO(nvmlDeviceResetGpuLockedClocks, uFp)
GO(nvmlDeviceResetMemoryLockedClocks, uFp)
GO(nvmlDeviceResetNvLinkErrorCounters, uFpu)
GO(nvmlDeviceResetNvLinkUtilizationCounter, uFpuu)
GO(nvmlDeviceSetAccountingMode, uFpu)
GO(nvmlDeviceSetAPIRestriction, uFpuu)
GO(nvmlDeviceSetApplicationsClocks, uFpuu)
GO(nvmlDeviceSetAutoBoostedClocksEnabled, uFpu)
GO(nvmlDeviceSetClockOffsets, uFpp)
GO(nvmlDeviceSetComputeMode, uFpu)
GO(nvmlDeviceSetConfComputeUnprotectedMemSize, uFpU)
GO(nvmlDeviceSetCpuAffinity, uFp)
GO(nvmlDeviceSetDefaultAutoBoostedClocksEnabled, uFpuu)
GO(nvmlDeviceSetDefaultFanSpeed_v2, uFpu)
GO(nvmlDeviceSetDramEncryptionMode, uFpp)
GO(nvmlDeviceSetDriverModel, uFpuu)
GO(nvmlDeviceSetEccMode, uFpu)
GO(nvmlDeviceSetFanControlPolicy, uFpuu)
GO(nvmlDeviceSetFanSpeed_v2, uFpuu)
GO(nvmlDeviceSetGpcClkVfOffset, uFpi)
GO(nvmlDeviceSetGpuLockedClocks, uFpuu)
GO(nvmlDeviceSetGpuOperationMode, uFpu)
GO(nvmlDeviceSetHostname_v1, uFpp)
GO(nvmlDeviceSetMemClkVfOffset, uFpi)
GO(nvmlDeviceSetMemoryLockedClocks, uFpuu)
GO(nvmlDeviceSetMigMode, uFpup)
GO(nvmlDeviceSetNvlinkBwMode, uFpp)
GO(nvmlDeviceSetNvLinkDeviceLowPowerThreshold, uFpp)
GO(nvmlDeviceSetNvLinkUtilizationControl, uFpuupu)
GO(nvmlDeviceSetPersistenceMode, uFpu)
GO(nvmlDeviceSetPowerManagementLimit, uFpu)
GO(nvmlDeviceSetPowerManagementLimit_v2, uFpp)
GO(nvmlDeviceSetPowerMizerMode_v1, uFpp)
GO(nvmlDeviceSetTemperatureThreshold, uFpup)
GO(nvmlDeviceSetVgpuCapabilities, uFpuu)
GO(nvmlDeviceSetVgpuHeterogeneousMode, uFpp)
GO(nvmlDeviceSetVgpuSchedulerState, uFpp)
GO(nvmlDeviceSetVirtualizationMode, uFpu)
GO(nvmlDeviceValidateInforom, uFp)
GO(nvmlDeviceWorkloadPowerProfileClearRequestedProfiles, uFpp)
GO(nvmlDeviceWorkloadPowerProfileGetCurrentProfiles, uFpp)
GO(nvmlDeviceWorkloadPowerProfileGetProfilesInfo, uFpp)
GO(nvmlDeviceWorkloadPowerProfileSetRequestedProfiles, uFpp)
GO(nvmlErrorString, pFu)
GO(nvmlEventSetCreate, uFbp_)
GO(nvmlEventSetFree, uFp)
//GO(nvmlEventSetWait, 
GO(nvmlEventSetWait_v2, uFpbpUUuu_u)
//GO(nvmlGetBlacklistDeviceCount, 
//GO(nvmlGetBlacklistDeviceInfoByIndex, 
GO(nvmlGetExcludedDeviceCount, uFp)
GO(nvmlGetExcludedDeviceInfoByIndex, uFup)
GO(nvmlGetVgpuCompatibility, uFppp)
GO(nvmlGetVgpuDriverCapabilities, uFup)
GO(nvmlGetVgpuVersion, uFpp)
//GO(nvmlGpmMetricsGet, uFp) // Warning: failed to confirm
GO(nvmlGpmMigSampleGet, uFpup)
GO(nvmlGpmQueryDeviceSupport, uFpp)
GO(nvmlGpmQueryIfStreamingEnabled, uFpp)
GO(nvmlGpmSampleAlloc, uFbp_)
GO(nvmlGpmSampleFree, uFp)
GO(nvmlGpmSampleGet, uFpp)
GO(nvmlGpmSetStreamingEnabled, uFpu)
GO(nvmlGpuInstanceCreateComputeInstance, uFpubp_)
GO(nvmlGpuInstanceCreateComputeInstanceWithPlacement, uFpupbp_)
GO(nvmlGpuInstanceDestroy, uFp)
GO(nvmlGpuInstanceGetActiveVgpus, uFpbuup_)
GO(nvmlGpuInstanceGetComputeInstanceById, uFpubp_)
GO(nvmlGpuInstanceGetComputeInstancePossiblePlacements, uFpupp)
GO(nvmlGpuInstanceGetComputeInstanceProfileInfo, uFpuup)
GO(nvmlGpuInstanceGetComputeInstanceProfileInfoV, uFpuup)
GO(nvmlGpuInstanceGetComputeInstanceRemainingCapacity, uFpup)
GO(nvmlGpuInstanceGetComputeInstances, uFpubp_p)
GO(nvmlGpuInstanceGetCreatableVgpus, uFpbuup_)
//GO(nvmlGpuInstanceGetInfo, uFpp) // Warning: failed to confirm
GO(nvmlGpuInstanceGetVgpuHeterogeneousMode, uFpp)
//GO(nvmlGpuInstanceGetVgpuSchedulerLog, uFpp) // Warning: failed to confirm
GO(nvmlGpuInstanceGetVgpuSchedulerState, uFpp)
GO(nvmlGpuInstanceGetVgpuTypeCreatablePlacements, uFpbuuupu_)
GO(nvmlGpuInstanceSetVgpuHeterogeneousMode, uFpp)
GO(nvmlGpuInstanceSetVgpuSchedulerState, uFpp)
//GO(nvmlInit, 
GO(nvmlInit_v2, uFv)
GO(nvmlInitWithFlags, uFu)
//GO(nvmlInternalGetExportTable, 
GO(nvmlSetVgpuVersion, uFp)
GO(nvmlShutdown, uFv)
GO(nvmlSystemEventSetCreate, uFbup_)
GO(nvmlSystemEventSetFree, uFbup_)
GO(nvmlSystemEventSetWait, uFbuuppuu_)
GO(nvmlSystemGetConfComputeCapabilities, uFp)
GO(nvmlSystemGetConfComputeGpusReadyState, uFp)
GO(nvmlSystemGetConfComputeKeyRotationThresholdInfo, uFp)
GO(nvmlSystemGetConfComputeSettings, uFp)
GO(nvmlSystemGetConfComputeState, uFp)
GO(nvmlSystemGetCudaDriverVersion, uFp)
GO(nvmlSystemGetCudaDriverVersion_v2, uFp)
GO(nvmlSystemGetDriverBranch, uFpu)
GO(nvmlSystemGetDriverVersion, uFpu)
GO(nvmlSystemGetHicVersion, uFpp)
GO(nvmlSystemGetNvlinkBwMode, uFp)
GO(nvmlSystemGetNVMLVersion, uFpu)
GO(nvmlSystemGetProcessName, uFupu)
GO(nvmlSystemGetTopologyGpuSet, uFupbp_)
GO(nvmlSystemRegisterEvents, uFbuUp_)
GO(nvmlSystemSetConfComputeGpusReadyState, uFu)
GO(nvmlSystemSetConfComputeKeyRotationThresholdInfo, uFp)
GO(nvmlSystemSetNvlinkBwMode, uFu)
GO(nvmlUnitGetCount, uFp)
GO(nvmlUnitGetDevices, uFppbp_)
GO(nvmlUnitGetFanSpeedInfo, uFpp)
GO(nvmlUnitGetHandleByIndex, uFubp_)
GO(nvmlUnitGetLedState, uFpp)
GO(nvmlUnitGetPsuInfo, uFpp)
GO(nvmlUnitGetTemperature, uFpup)
GO(nvmlUnitGetUnitInfo, uFpp)
GO(nvmlUnitSetLedState, uFpu)
GO(nvmlVgpuInstanceClearAccountingPids, uFu)
GO(nvmlVgpuInstanceGetAccountingMode, uFup)
GO(nvmlVgpuInstanceGetAccountingPids, uFupp)
GO(nvmlVgpuInstanceGetAccountingStats, uFuup)
GO(nvmlVgpuInstanceGetEccMode, uFup)
GO(nvmlVgpuInstanceGetEncoderCapacity, uFup)
GO(nvmlVgpuInstanceGetEncoderSessions, uFupp)
GO(nvmlVgpuInstanceGetEncoderStats, uFuppp)
GO(nvmlVgpuInstanceGetFBCSessions, uFupp)
GO(nvmlVgpuInstanceGetFBCStats, uFup)
GO(nvmlVgpuInstanceGetFbUsage, uFup)
GO(nvmlVgpuInstanceGetFrameRateLimit, uFup)
GO(nvmlVgpuInstanceGetGpuInstanceId, uFup)
GO(nvmlVgpuInstanceGetGpuPciId, uFupp)
//GO(nvmlVgpuInstanceGetLicenseInfo, 
GO(nvmlVgpuInstanceGetLicenseInfo_v2, uFup)
GO(nvmlVgpuInstanceGetLicenseStatus, uFup)
GO(nvmlVgpuInstanceGetMdevUUID, uFupu)
GO(nvmlVgpuInstanceGetMetadata, uFupp)
GO(nvmlVgpuInstanceGetPlacementId, uFup)
GO(nvmlVgpuInstanceGetRuntimeStateSize, uFup)
GO(nvmlVgpuInstanceGetType, uFup)
GO(nvmlVgpuInstanceGetUUID, uFupu)
GO(nvmlVgpuInstanceGetVmDriverVersion, uFupu)
GO(nvmlVgpuInstanceGetVmID, uFupup)
GO(nvmlVgpuInstanceSetEncoderCapacity, uFuu)
GO(nvmlVgpuTypeGetBAR1Info, uFup)
GO(nvmlVgpuTypeGetCapabilities, uFuup)
GO(nvmlVgpuTypeGetClass, uFupp)
GO(nvmlVgpuTypeGetDeviceID, uFupp)
GO(nvmlVgpuTypeGetFbReservation, uFup)
GO(nvmlVgpuTypeGetFramebufferSize, uFup)
GO(nvmlVgpuTypeGetFrameRateLimit, uFup)
GO(nvmlVgpuTypeGetGpuInstanceProfileId, uFup)
GO(nvmlVgpuTypeGetGspHeapSize, uFup)
GO(nvmlVgpuTypeGetLicense, uFupu)
GO(nvmlVgpuTypeGetMaxInstances, uFpup)
GO(nvmlVgpuTypeGetMaxInstancesPerGpuInstance, uFp)
GO(nvmlVgpuTypeGetMaxInstancesPerVm, uFup)
GO(nvmlVgpuTypeGetName, uFupp)
GO(nvmlVgpuTypeGetNumDisplayHeads, uFup)
GO(nvmlVgpuTypeGetResolution, uFuupp)
