#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(gst_tag_check_language_code, iFp)
GO(gst_tag_demux_get_type, LFv)
GO(gst_tag_demux_result_get_type, LFv)
GO(gst_tag_freeform_string_to_utf8, pFpip)
GO(gst_tag_from_id3_tag, pFp)
GO(gst_tag_from_id3_user_tag, pFpp)
GO(gst_tag_from_vorbis_tag, pFp)
GO(gst_tag_get_id3v2_tag_size, uFp)
GO(gst_tag_get_language_code_iso_639_1, pFp)
GO(gst_tag_get_language_code_iso_639_2B, pFp)
GO(gst_tag_get_language_code_iso_639_2T, pFp)
GO(gst_tag_get_language_codes, pFv)
GO(gst_tag_get_language_name, pFp)
GO(gst_tag_get_license_description, pFp)
GO(gst_tag_get_license_flags, uFp)
GO(gst_tag_get_license_jurisdiction, pFp)
GO(gst_tag_get_license_nick, pFp)
GO(gst_tag_get_licenses, pFv)
GO(gst_tag_get_license_title, pFp)
GO(gst_tag_get_license_version, pFp)
GO(gst_tag_id3_genre_count, uFv)
GO(gst_tag_id3_genre_get, pFu)
GO(gst_tag_image_data_to_image_sample, pFpui)
GO(gst_tag_image_type_get_type, LFv)
GO(gst_tag_license_flags_get_type, LFv)
GO(gst_tag_list_add_id3_image, iFppuu)
GO(gst_tag_list_from_exif_buffer, pFpiu)
GO(gst_tag_list_from_exif_buffer_with_tiff_header, pFp)
GO(gst_tag_list_from_id3v2_tag, pFp)
GO(gst_tag_list_from_vorbiscomment, pFpLpup)
GO(gst_tag_list_from_vorbiscomment_buffer, pFppup)
GO(gst_tag_list_from_xmp_buffer, pFp)
GO(gst_tag_list_new_from_id3v1, pFp)
GO(gst_tag_list_to_exif_buffer, pFpiu)
GO(gst_tag_list_to_exif_buffer_with_tiff_header, pFp)
GO(gst_tag_list_to_vorbiscomment_buffer, pFppup)
GO(gst_tag_list_to_xmp_buffer, pFpip)
GO(gst_tag_mux_get_type, LFv)
GO(gst_tag_parse_extended_comment, iFppppi)
GO(gst_tag_register_musicbrainz_tags, vFv)
GO(gst_tag_to_id3_tag, pFp)
GO(gst_tag_to_vorbis_comments, pFpp)
GO(gst_tag_to_vorbis_tag, pFp)
GO(gst_tag_xmp_list_schemas, pFv)
GO(gst_tag_xmp_writer_add_all_schemas, vFp)
GO(gst_tag_xmp_writer_add_schema, vFpp)
GO(gst_tag_xmp_writer_get_type, LFv)
GO(gst_tag_xmp_writer_has_schema, iFpp)
GO(gst_tag_xmp_writer_remove_all_schemas, vFp)
GO(gst_tag_xmp_writer_remove_schema, vFpp)
GO(gst_tag_xmp_writer_tag_list_to_xmp_buffer, pFppi)
GO(gst_vorbis_tag_add, vFppp)
