#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif


GO(XDamageAdd, vFXLL)
GO(XDamageCreate, LFXLi)
GO(XDamageDestroy, vFXL)
GO(XDamageFindDisplay, pFX)
GO(XDamageQueryExtension, iFXpp)
GO(XDamageQueryVersion, iFXpp)
GO(XDamageSubtract, vFXLLL)
