#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif
//typedef uint32_t xkb_keycode_t

GO(xkb_compose_state_feed, uFpu)
GO(xkb_compose_state_get_compose_table, pFp)
GO(xkb_compose_state_get_one_sym, uFp)
GO(xkb_compose_state_get_status, uFp)
GO(xkb_compose_state_get_utf8, iFppL)
GO(xkb_compose_state_new, pFpu)
GO(xkb_compose_state_ref, pFp)
GO(xkb_compose_state_reset, vFp)
GO(xkb_compose_state_unref, vFp)
GO(xkb_compose_table_entry_utf8, pFp)
GO(xkb_compose_table_entry_keysym, uFp)
GO(xkb_compose_table_iterator_free, vFp)
GO(xkb_compose_table_iterator_new, pFp)
GO(xkb_compose_table_iterator_next, pFp)
GO(xkb_compose_table_entry_sequence, pFpp)
GO(xkb_compose_table_new_from_buffer, pFppLpuu)
GO(xkb_compose_table_new_from_file, pFpSpuu)
GO(xkb_compose_table_new_from_locale, pFppu)
GO(xkb_compose_table_ref, pFp)
GO(xkb_compose_table_unref, vFp)
GO(xkb_context_get_log_level, uFp)
GO(xkb_context_get_log_verbosity, iFp)
GO(xkb_context_get_user_data, pFp)
GO(xkb_context_include_path_append, iFpp)
GO(xkb_context_include_path_append_default, iFp)
GO(xkb_context_include_path_clear, vFp)
GO(xkb_context_include_path_get, pFpu)
GO(xkb_context_include_path_reset_defaults, iFp)
GO(xkb_context_new, pFu)
GO(xkb_context_num_include_paths, uFp)
GO(xkb_context_ref, pFp)
//GOM(xkb_context_set_log_fn, vFEpp)
GO(xkb_context_set_log_level, vFpu)
GO(xkb_context_set_log_verbosity, vFpi)
GO(xkb_context_set_user_data, vFpp)
GO(xkb_context_unref, vFp)
GO(xkb_keymap_get_as_string, pFpu)
GO(xkb_keymap_key_by_name, uFpp)
GOM(xkb_keymap_key_for_each, vFEppp)
GO(xkb_keymap_key_get_mods_for_level, LFpuuupL)
GO(xkb_keymap_key_get_name, pFpu)
GO(xkb_keymap_key_get_syms_by_level, iFpuuup)
GO(xkb_keymap_key_repeats, iFpu)
GO(xkb_keymap_layout_get_index, uFpp)
GO(xkb_keymap_layout_get_name, pFpu)
GO(xkb_keymap_led_get_index, uFpp)
GO(xkb_keymap_led_get_name, pFpu)
GO(xkb_keymap_max_keycode, uFp)
GO(xkb_keymap_min_keycode, uFp)
GO(xkb_keymap_mod_get_index, uFpp)
GO(xkb_keymap_mod_get_name, pFpu)
GO(xkb_keymap_new_from_buffer, pFppLuu)
GO(xkb_keymap_new_from_file, pFpSuu)
GO(xkb_keymap_new_from_names, pFppu)
GO(xkb_keymap_new_from_string, pFppuu)
GO(xkb_keymap_num_layouts, uFp)
GO(xkb_keymap_num_layouts_for_key, uFpu)
GO(xkb_keymap_num_leds, uFp)
GO(xkb_keymap_num_levels_for_key, uFpuu)
GO(xkb_keymap_num_mods, uFp)
GO(xkb_keymap_ref, pFp)
GO(xkb_keymap_unref, vFp)
GO(xkb_keysym_from_name, uFpu)
GO(xkb_keysym_get_name, iFupL)
GO(xkb_keysym_to_lower, uFu)
GO(xkb_keysym_to_upper, uFu)
GO(xkb_keysym_to_utf32, uFu)
GO(xkb_keysym_to_utf8, iFupL)
GO(xkb_state_get_keymap, pFp)
GO(xkb_state_key_get_consumed_mods, uFpu)
GO(xkb_state_key_get_consumed_mods2, uFpuu)
GO(xkb_state_key_get_layout, uFpu)
GO(xkb_state_key_get_level, uFpuu)
GO(xkb_state_key_get_one_sym, uFpu)
GO(xkb_state_key_get_syms, iFpup)
GO(xkb_state_key_get_utf32, uFpu)
GO(xkb_state_key_get_utf8, iFpupL)
GO(xkb_state_layout_index_is_active, iFpuu)
GO(xkb_state_layout_name_is_active, iFppu)
GO(xkb_state_led_index_is_active, iFpu)
GO(xkb_state_led_name_is_active, iFpp)
GO(xkb_state_mod_index_is_active, iFpuu)
GO(xkb_state_mod_index_is_consumed, iFpuu)
GO(xkb_state_mod_index_is_consumed2, iFpuuu)
//GOM(xkb_state_mod_indices_are_active, iFEpuuV)
GO(xkb_state_mod_mask_remove_consumed, uFpuu)
GO(xkb_state_mod_name_is_active, iFppu)
//GOM(xkb_state_mod_names_are_active, iFEpuuV)
GO(xkb_state_new, pFp)
GO(xkb_state_ref, pFp)
GO(xkb_state_serialize_layout, uFpu)
GO(xkb_state_serialize_mods, uFpu)
GO(xkb_state_unref, vFp)
GO(xkb_state_update_key, uFpuu)
GO(xkb_state_update_mask, uFpuuuuuu)
