#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(amdgpu_bo_alloc, iFppp)
GO(amdgpu_bo_cpu_map, iFpp)
GO(amdgpu_bo_cpu_unmap, iFp)
GO(amdgpu_bo_export, iFpup)
GO(amdgpu_bo_free, iFp)
GO(amdgpu_bo_import, iFpuup)
GO(amdgpu_bo_inc_ref, vFp)
GO(amdgpu_bo_list_create, iFpuppp)
GO(amdgpu_bo_list_create_raw, iFpupp)
GO(amdgpu_bo_list_destroy, iFp)
GO(amdgpu_bo_list_destroy_raw, iFpu)
GO(amdgpu_bo_list_update, iFpupp)
GO(amdgpu_bo_query_info, iFpp)
GO(amdgpu_bo_set_metadata, iFpp)
GO(amdgpu_bo_va_op, iFpUUUUu)
GO(amdgpu_bo_va_op_raw, iFppUUUUu)
GO(amdgpu_bo_wait_for_idle, iFpUp)
GO(amdgpu_create_bo_from_user_mem, iFppUp)
GO(amdgpu_cs_chunk_fence_info_to_data, vFpp)
GO(amdgpu_cs_chunk_fence_to_dep, vFpp)
GO(amdgpu_cs_create_semaphore, iFp)
GO(amdgpu_cs_create_syncobj, iFpp)
GO(amdgpu_cs_create_syncobj2, iFpup)
GO(amdgpu_cs_ctx_create, iFpp)
GO(amdgpu_cs_ctx_create2, iFpup)
GO(amdgpu_cs_ctx_free, iFp)
GO(amdgpu_cs_ctx_override_priority, iFppiu)
GO(amdgpu_cs_ctx_stable_pstate, iFpuup)
GO(amdgpu_cs_destroy_semaphore, iFp)
GO(amdgpu_cs_destroy_syncobj, iFpu)
GO(amdgpu_cs_export_syncobj, iFpup)
GO(amdgpu_cs_fence_to_handle, iFppup)
GO(amdgpu_cs_import_syncobj, iFpip)
GO(amdgpu_cs_query_fence_status, iFpUUp)
GO(amdgpu_cs_query_reset_state, iFppp)
GO(amdgpu_cs_query_reset_state2, iFpp)
GO(amdgpu_cs_signal_semaphore, iFpuuup)
GO(amdgpu_cs_submit, iFpUpu)
GO(amdgpu_cs_submit_raw, iFpppipp)
GO(amdgpu_cs_submit_raw2, iFppuipp)
GO(amdgpu_cs_syncobj_export_sync_file, iFpup)
GO(amdgpu_cs_syncobj_export_sync_file2, iFpuUup)
GO(amdgpu_cs_syncobj_import_sync_file, iFpui)
GO(amdgpu_cs_syncobj_import_sync_file2, iFpuUi)
GO(amdgpu_cs_syncobj_query, iFpppu)
GO(amdgpu_cs_syncobj_query2, iFpppuu)
GO(amdgpu_cs_syncobj_reset, iFppu)
GO(amdgpu_cs_syncobj_signal, iFppu)
GO(amdgpu_cs_syncobj_timeline_signal, iFpppu)
GO(amdgpu_cs_syncobj_timeline_wait, iFpppuIup)
GO(amdgpu_cs_syncobj_transfer, iFpuUuUu)
GO(amdgpu_cs_syncobj_wait, iFppuIup)
GO(amdgpu_cs_wait_fences, iFpuiUpp)
GO(amdgpu_cs_wait_semaphore, iFpuuup)
GO(amdgpu_device_deinitialize, iFp)
GO(amdgpu_device_get_fd, iFp)
GO(amdgpu_device_initialize, iFippp)
GO(amdgpu_device_initialize2, iFiippp)
GO(amdgpu_find_bo_by_cpu_mapping, iFppUpp)
GO(amdgpu_get_marketing_name, pFp)
GO(amdgpu_query_buffer_size_alignment, iFpp)
GO(amdgpu_query_crtc_from_id, iFpup)
GO(amdgpu_query_firmware_version, iFpuuupp)
GO(amdgpu_query_gds_info, iFpp)
GO(amdgpu_query_gpu_info, iFpp)
GO(amdgpu_query_gpuvm_fault_info, iFpup)
GO(amdgpu_query_heap_info, iFpuup)
GO(amdgpu_query_hw_ip_count, iFpup)
GO(amdgpu_query_hw_ip_info, iFpuup)
GO(amdgpu_query_info, iFpuup)
GO(amdgpu_query_sensor_info, iFpuup)
GO(amdgpu_query_sw_info, iFpup)
GO(amdgpu_query_video_caps_info, iFpuup)
GO(amdgpu_read_mm_registers, iFpuuuup)
GO(amdgpu_va_get_start_addr, UFp)
GO(amdgpu_va_manager_alloc, pFv)
GO(amdgpu_va_manager_deinit, vFp)
GO(amdgpu_va_manager_init, vFpUUUUu)
GO(amdgpu_va_range_alloc, iFpuUUUppU)
GO(amdgpu_va_range_alloc2, iFpuUUUppU)
GO(amdgpu_va_range_free, iFp)
GO(amdgpu_va_range_query, iFpupp)
GO(amdgpu_vm_reserve_vmid, iFpu)
GO(amdgpu_vm_unreserve_vmid, iFpu)
