#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh....
#endif

//GO(XctCreate, 
//GO(XctFree, 
//GO(XctNextItem, 
//GO(XctReset, 
//GO(_XEditResCheckMessages, 
//GO(_XEditResGet16, 
//GO(_XEditResGet32, 
//GO(_XEditResGet8, 
//GO(_XEditResGetSigned16, 
//GO(_XEditResGetString8, 
//GO(_XEditResGetWidgetInfo, 
//GO(_XEditResPut16, 
//GO(_XEditResPut32, 
//GO(_XEditResPut8, 
//GO(_XEditResPutString8, 
//GO(_XEditResPutWidgetInfo, 
//GO(_XEditResResetStream, 
//GO(XmuAddCloseDisplayHook, 
//GO(XmuAddInitializer, 
//GO(XmuAllStandardColormaps, 
//GO(XmuAppendSegment, 
//GO(XmuAreaAnd, 
//GO(XmuAreaCopy, 
//GO(XmuAreaDup, 
//GO(XmuAreaNot, 
//GO(XmuAreaOrXor, 
//GO(XmuCallInitializers, 
//GO(_XmuCCLookupDisplay, 
GO(XmuClientWindow, pFpp)
//GO(XmuCompareISOLatin1, 
//GO(XmuConvertStandardSelection, 
//GO(XmuCopyISOLatin1Lowered, 
//GO(XmuCopyISOLatin1Uppered, 
//GO(XmuCreateColormap, 
//GO(XmuCreatePixmapFromBitmap, 
//GO(XmuCreateStippledPixmap, 
//GO(XmuCursorNameToIndex, 
//GO(XmuCvtBackingStoreToString, 
//GO(XmuCvtFunctionToCallback, 
//GO(XmuCvtGravityToString, 
//GO(XmuCvtJustifyToString, 
//GO(XmuCvtLongToString, 
//GO(XmuCvtOrientationToString, 
//GO(XmuCvtShapeStyleToString, 
//GO(XmuCvtStringToBackingStore, 
//GO(XmuCvtStringToBitmap, 
//GO(XmuCvtStringToColorCursor, 
//GO(XmuCvtStringToCursor, 
//GO(XmuCvtStringToGravity, 
//GO(XmuCvtStringToJustify, 
//GO(XmuCvtStringToLong, 
//GO(XmuCvtStringToOrientation, 
//GO(XmuCvtStringToShapeStyle, 
//GO(XmuCvtStringToWidget, 
//GO(XmuCvtWidgetToString, 
//GO(XmuDeleteStandardColormap, 
//GO(XmuDestroyScanlineList, 
//GO(XmuDestroySegmentList, 
//GO(XmuDistinguishableColors, 
//GO(XmuDistinguishablePixels, 
//GO(XmuDQAddDisplay, 
//GO(XmuDQCreate, 
//GO(XmuDQDestroy, 
//GO(XmuDQLookupDisplay, 
//GO(XmuDQRemoveDisplay, 
//GO(XmuDrawLogo, 
//GO(XmuDrawRoundedRectangle, 
//GO(XmuFillRoundedRectangle, 
//GO(XmuGetAtomName, 
//GO(XmuGetColormapAllocation, 
//GO(XmuGetHostname, 
//GO(XmuInternAtom, 
//GO(XmuInternStrings, 
//GO(XmuLocateBitmapFile, 
//GO(XmuLocatePixmapFile, 
//GO(XmuLookupAPL, 
//GO(XmuLookupArabic, 
//GO(XmuLookupCloseDisplayHook, 
//GO(XmuLookupCyrillic, 
//GO(XmuLookupGreek, 
//GO(XmuLookupHebrew, 
//GO(XmuLookupJISX0201, 
//GO(XmuLookupKana, 
//GO(XmuLookupLatin1, 
//GO(XmuLookupLatin2, 
//GO(XmuLookupLatin3, 
//GO(XmuLookupLatin4, 
//GO(XmuLookupStandardColormap, 
//GO(XmuLookupString, 
//GO(XmuMakeAtom, 
//GO(XmuNameOfAtom, 
//GO(XmuNCopyISOLatin1Lowered, 
//GO(XmuNCopyISOLatin1Uppered, 
//GO(XmuNewArea, 
//GO(XmuNewCvtStringToWidget, 
//GO(XmuNewScanline, 
//GO(XmuNewSegment, 
//GO(XmuOptimizeArea, 
//GO(XmuOptimizeScanline, 
//GO(XmuPrintDefaultErrorMessage, 
//GO(XmuReadBitmapData, 
//GO(XmuReadBitmapDataFromFile, 
//GO(XmuRegisterExternalAgent, 
//GO(XmuReleaseStippledPixmap, 
//GO(XmuRemoveCloseDisplayHook, 
//GO(XmuReshapeWidget, 
//GO(XmuScanlineAnd, 
//GO(XmuScanlineAndSegment, 
//GO(XmuScanlineCopy, 
//GO(XmuScanlineEqu, 
//GO(XmuScanlineNot, 
//GO(XmuScanlineOr, 
//GO(XmuScanlineOrSegment, 
//GO(XmuScanlineXor, 
//GO(XmuScanlineXorSegment, 
GO(XmuScreenOfWindow, pFpp)
//GO(XmuSimpleErrorHandler, 
//GO(XmuSnprintf, 
//GO(XmuStandardColormap, 
//GO(_XmuStringToBitmapFreeCache, 
//GO(_XmuStringToBitmapInitCache, 
//GO(XmuUpdateMapHints, 
//GO(XmuValidArea, 
//GO(XmuValidScanline, 
//GO(XmuVisualStandardColormaps, 
//GO(XmuWnCountOwnedResources, 
//GO(XmuWnFetchResources, 
//GO(XmuWnInitializeNodes, 
//GO(XmuWnNameToNode, 
