#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(mnt_cache_device_has_tag, iFpppp)
GO(mnt_cache_find_tag_value, pFppp)
GO(mnt_cache_read_tags, iFpp)
GO(mnt_cache_set_sbprobe, iFpi)
GO(mnt_cache_set_targets, iFpp)
GO(mnt_context_append_options, iFpp)
GO(mnt_context_apply_fstab, iFp)
GO(mnt_context_disable_canonicalize, iFpi)
GO(mnt_context_disable_helpers, iFpi)
GO(mnt_context_disable_mtab, iFpi)
GO(mnt_context_disable_swapmatch, iFpi)
GO(mnt_context_do_mount, iFp)
GO(mnt_context_do_umount, iFp)
GO(mnt_context_enable_fake, iFpi)
GO(mnt_context_enable_force, iFpi)
GO(mnt_context_enable_fork, iFpi)
GO(mnt_context_enable_lazy, iFpi)
GO(mnt_context_enable_loopdel, iFpi)
GO(mnt_context_enable_noautofs, iFpi)
GO(mnt_context_enable_onlyonce, iFpi)
GO(mnt_context_enable_rdonly_umount, iFpi)
GO(mnt_context_enable_rwonly_mount, iFpi)
GO(mnt_context_enable_sloppy, iFpi)
GO(mnt_context_enable_verbose, iFpi)
GO(mnt_context_finalize_mount, iFp)
GO(mnt_context_finalize_umount, iFp)
GO(mnt_context_find_umount_fs, iFppbp_)
GO(mnt_context_forced_rdonly, iFp)
GO(mnt_context_force_unrestricted, iFp)
GO(mnt_context_get_cache, pFp)
GO(mnt_context_get_excode, iFpipL)
GO(mnt_context_get_fs, pFp)
GO(mnt_context_get_fstab, iFpbp_)
GO(mnt_context_get_fstab_userdata, pFp)
GO(mnt_context_get_fstype, pFp)
GO(mnt_context_get_fs_userdata, pFp)
GO(mnt_context_get_helper_status, iFp)
GO(mnt_context_get_lock, pFp)
//GOM(mnt_context_get_mesgs, pFEp)
GO(mnt_context_get_mflags, iFpbL_)
GO(mnt_context_get_mtab, iFpbp_)
GO(mnt_context_get_mtab_userdata, pFp)
GO(mnt_context_get_nmesgs, LFpc)
GO(mnt_context_get_options, pFp)
GO(mnt_context_get_optsmode, iFp)
GO(mnt_context_get_origin_ns, pFp)
GO(mnt_context_get_source, pFp)
GO(mnt_context_get_status, iFp)
GO(mnt_context_get_syscall_errno, iFp)
GO(mnt_context_get_table, iFppbp_)
GO(mnt_context_get_target, pFp)
GO(mnt_context_get_target_ns, pFp)
GO(mnt_context_get_target_prefix, pFp)
GO(mnt_context_get_user_mflags, iFpbL_)
GO(mnt_context_helper_executed, iFp)
GO(mnt_context_helper_setopt, iFpip)
GO(mnt_context_init_helper, iFpii)
GO(mnt_context_is_child, iFp)
GO(mnt_context_is_fake, iFp)
GO(mnt_context_is_force, iFp)
GO(mnt_context_is_fork, iFp)
GO(mnt_context_is_fs_mounted, iFppp)
GO(mnt_context_is_lazy, iFp)
GO(mnt_context_is_loopdel, iFp)
GO(mnt_context_is_nocanonicalize, iFp)
GO(mnt_context_is_nohelpers, iFp)
GO(mnt_context_is_nomtab, iFp)
GO(mnt_context_is_onlyonce, iFp)
GO(mnt_context_is_parent, iFp)
GO(mnt_context_is_rdonly_umount, iFp)
GO(mnt_context_is_restricted, iFp)
GO(mnt_context_is_rwonly_mount, iFp)
GO(mnt_context_is_sloppy, iFp)
GO(mnt_context_is_swapmatch, iFp)
GO(mnt_context_is_verbose, iFp)
GO(mnt_context_mount, iFp)
GO(mnt_context_next_mount, iFppbp_pp)
GO(mnt_context_next_remount, iFppbp_pp)
GO(mnt_context_next_umount, iFppbp_pp)
GO(mnt_context_prepare_mount, iFp)
GO(mnt_context_prepare_umount, iFp)
GO(mnt_context_reset_status, iFp)
GO(mnt_context_set_cache, iFpp)
GO(mnt_context_set_fs, iFpp)
GO(mnt_context_set_fstab, iFpp)
GO(mnt_context_set_fstype, iFpp)
GO(mnt_context_set_fstype_pattern, iFpp)
GO(mnt_context_set_mflags, iFpL)
GO(mnt_context_set_mountdata, iFpp)
GO(mnt_context_set_options, iFpp)
GO(mnt_context_set_options_pattern, iFpp)
GO(mnt_context_set_optsmode, iFpi)
//GOM(mnt_context_set_passwd_cb, iFEppp)
GO(mnt_context_set_source, iFpp)
GO(mnt_context_set_syscall_status, iFpi)
//GOM(mnt_context_set_tables_errcb, iFEpp)
GO(mnt_context_set_target, iFpp)
GO(mnt_context_set_target_ns, iFpp)
GO(mnt_context_set_target_prefix, iFpp)
GO(mnt_context_set_user_mflags, iFpL)
GO(mnt_context_strerror, iFppL)
GO(mnt_context_switch_ns, pFpp)
GO(mnt_context_switch_origin_ns, pFp)
GO(mnt_context_switch_target_ns, pFp)
GO(mnt_context_syscall_called, iFp)
GO(mnt_context_tab_applied, iFp)
GO(mnt_context_umount, iFp)
GO(mnt_context_wait_for_children, iFppp)
GO(mnt_copy_fs, pFpp)
GO(mnt_diff_tables, iFppp)
GO(mnt_free_cache, vFp)
GO(mnt_free_context, vFp)
GO(mnt_free_fs, vFp)
GO(mnt_free_iter, vFp)
GO(mnt_free_lock, vFp)
//GO(mnt_free_mntent, vFbppppii_)
GO(mnt_free_tabdiff, vFp)
GO(mnt_free_table, vFp)
GO(mnt_free_update, vFp)
GO(mnt_fs_append_attributes, iFpp)
GO(mnt_fs_append_comment, iFpp)
GO(mnt_fs_append_options, iFpp)
GO(mnt_fs_fetch_statmount, iFpU)
GO(mnt_fs_get_attribute, iFppbp_bL_)
GO(mnt_fs_get_attributes, pFp)
GO(mnt_fs_get_bindsrc, pFp)
GO(mnt_fs_get_comment, pFp)
GO(mnt_fs_get_devno, UFp)
GO(mnt_fs_get_freq, iFp)
GO(mnt_fs_get_fs_options, pFp)
GO(mnt_fs_get_fstype, pFp)
GO(mnt_fs_get_id, iFp)
GO(mnt_fs_get_ns, UFp)
GO(mnt_fs_get_option, iFppbp_bL_)
GO(mnt_fs_get_optional_fields, pFp)
GO(mnt_fs_get_options, pFp)
GO(mnt_fs_get_parent_id, iFp)
GO(mnt_fs_get_parent_uniq_id, UFp)
GO(mnt_fs_get_passno, iFp)
GO(mnt_fs_get_priority, iFp)
GO(mnt_fs_get_propagation, iFpbL_)
GO(mnt_fs_get_root, pFp)
GO(mnt_fs_get_size, IFp)
GO(mnt_fs_get_source, pFp)
GO(mnt_fs_get_srcpath, pFp)
GO(mnt_fs_get_statmnt, pFp)
GO(mnt_fs_get_swaptype, pFp)
GO(mnt_fs_get_table, iFpbp_)
GO(mnt_fs_get_tag, iFpbp_bp_)
GO(mnt_fs_get_target, pFp)
GO(mnt_fs_get_tid, iFp)
GO(mnt_fs_get_uniq_id, UFp)
GO(mnt_fs_get_usedsize, IFp)
GO(mnt_fs_get_userdata, pFp)
GO(mnt_fs_get_user_options, pFp)
GO(mnt_fs_get_vfs_options, pFp)
GO(mnt_fs_get_vfs_options_all, pFp)
GO(mnt_fs_is_kernel, iFp)
GO(mnt_fs_is_netfs, iFp)
GO(mnt_fs_is_pseudofs, iFp)
GO(mnt_fs_is_regularfs, iFp)
GO(mnt_fs_is_swaparea, iFp)
GO(mnt_fs_match_fstype, iFpp)
GO(mnt_fs_match_options, iFpp)
GO(mnt_fs_match_source, iFppp)
GO(mnt_fs_match_target, iFppp)
GO(mnt_fs_prepend_attributes, iFpp)
GO(mnt_fs_prepend_options, iFpp)
GO(mnt_fs_print_debug, iFpS)
GO(mnt_fs_refer_statmnt, iFpp)
GO(mnt_fs_set_attributes, iFpp)
GO(mnt_fs_set_bindsrc, iFpp)
GO(mnt_fs_set_comment, iFpp)
GO(mnt_fs_set_freq, iFpi)
GO(mnt_fs_set_fstype, iFpp)
GO(mnt_fs_set_ns, iFpU)
GO(mnt_fs_set_options, iFpp)
GO(mnt_fs_set_passno, iFpi)
GO(mnt_fs_set_priority, iFpi)
GO(mnt_fs_set_root, iFpp)
GO(mnt_fs_set_source, iFpp)
GO(mnt_fs_set_target, iFpp)
GO(mnt_fs_set_uniq_id, iFpU)
GO(mnt_fs_set_userdata, iFpp)
GO(mnt_fs_strdup_options, pFp)
GO(mnt_fs_streq_srcpath, iFpp)
GO(mnt_fs_streq_target, iFpp)
//GOM(mnt_fs_to_mntent, iFEpbp_)
GO(mnt_fstype_is_netfs, iFp)
GO(mnt_fstype_is_pseudofs, iFp)
GOM(mnt_get_builtin_optmap, pFEi)
GO(mnt_get_fstab_path, pFv)
GO(mnt_get_fstype, pFppp)
//GOM(mnt_get_library_features, iFEbp_)
GO(mnt_get_library_version, iFbp_)
GO(mnt_get_mountpoint, pFp)
GO(mnt_get_mtab_path, pFv)
GO(mnt_get_swaps_path, pFv)
GO(mnt_guess_system_root, iFUpbp_)
GO(mnt_has_regular_mtab, iFbp_p)
GO(mnt_id_from_path, iFppp)
GO(mnt_init_debug, vFi)
GO(mnt_iter_get_direction, iFp)
GO(mnt_lock_block_signals, iFpi)
GO(mnt_lock_file, iFp)
GO(mnt_mangle, pFp)
GO(mnt_match_fstype, iFpp)
GO(mnt_match_options, iFpp)
GO(mnt_monitor_close_fd, iFp)
GO(mnt_monitor_enable_kernel, iFpi)
GO(mnt_monitor_enable_userspace, iFpip)
GO(mnt_monitor_event_cleanup, iFp)
GO(mnt_monitor_get_fd, iFp)
GO(mnt_monitor_next_change, iFpbp_p)
GO(mnt_monitor_veil_kernel, iFpi)
GO(mnt_monitor_wait, iFpi)
GO(mnt_new_cache, pFv)
GO(mnt_new_context, pFv)
GO(mnt_new_fs, pFv)
GO(mnt_new_iter, pFi)
GO(mnt_new_lock, pFpi)
GO(mnt_new_monitor, pFv)
GO(mnt_new_statmnt, pFv)
GO(mnt_new_tabdiff, pFv)
GO(mnt_new_table, pFv)
GO(mnt_new_table_from_dir, pFp)
GO(mnt_new_table_from_file, pFp)
GO(mnt_new_update, pFv)
GO(mnt_optstr_append_option, iFbp_pp)
//GO(mnt_optstr_apply_flags, iFbp_Lbpii_)
GO(mnt_optstr_deduplicate_option, iFbp_p)
GOM(mnt_optstr_get_flags, iFEpbL_p)
GO(mnt_optstr_get_option, iFppbp_bL_)
//GO(mnt_optstr_get_options, iFpbp_bpii_i)
GO(mnt_optstr_next_option, iFbp_bp_bL_bp_bL_)
GO(mnt_optstr_prepend_option, iFbp_pp)
GO(mnt_optstr_remove_option, iFbp_p)
GO(mnt_optstr_set_option, iFbp_pp)
GO(mnt_parse_version_string, iFp)
GO(mnt_pretty_path, pFpp)
GO(mnt_ref_cache, vFp)
GO(mnt_ref_fs, vFp)
GO(mnt_ref_lock, vFp)
GO(mnt_ref_monitor, vFp)
GO(mnt_ref_statmnt, vFp)
GO(mnt_ref_table, vFp)
GO(mnt_reset_context, iFp)
GO(mnt_reset_fs, vFp)
GO(mnt_reset_iter, vFpi)
GO(mnt_reset_table, iFp)
GO(mnt_resolve_path, pFpp)
GO(mnt_resolve_spec, pFpp)
GO(mnt_resolve_tag, pFppp)
GO(mnt_resolve_target, pFpp)
GO(mnt_split_optstr, iFpbp_bp_bp_ii)
GO(mnt_statmnt_disable_fetching, iFpi)
GO(mnt_statmnt_set_mask, iFpU)
GO(mnt_tabdiff_next_change, iFppbp_bp_p)
GO(mnt_table_add_fs, iFpp)
GO(mnt_table_append_intro_comment, iFpp)
GO(mnt_table_append_trailing_comment, iFpp)
GO(mnt_table_enable_comments, vFpi)
GO(mnt_table_enable_listmount, iFpi)
//GO(mnt_table_enable_noautofs, 
GO(mnt_table_fetch_listmount, iFp)
GO(mnt_table_find_devno, pFpUi)
GO(mnt_table_find_fs, iFpp)
GO(mnt_table_find_id, pFpi)
GO(mnt_table_find_mountpoint, pFppi)
//GOM(mnt_table_find_next_fs, iFEppppbp_)
GO(mnt_table_find_pair, pFpppi)
GO(mnt_table_find_source, pFppi)
GO(mnt_table_find_srcpath, pFppi)
GO(mnt_table_find_tag, pFpppi)
GO(mnt_table_find_target, pFppi)
GO(mnt_table_find_target_with_option, pFppppi)
GO(mnt_table_find_uniq_id, pFpU)
GO(mnt_table_first_fs, iFpbp_)
GO(mnt_table_get_cache, pFp)
GO(mnt_table_get_intro_comment, pFp)
GO(mnt_table_get_nents, iFp)
GO(mnt_table_get_root_fs, iFpbp_)
GO(mnt_table_get_trailing_comment, pFp)
GO(mnt_table_get_userdata, pFp)
GO(mnt_table_insert_fs, iFpipp)
GO(mnt_table_is_empty, iFp)
GO(mnt_table_is_fs_mounted, iFpp)
//GO(mnt_table_is_noautofs, 
GO(mnt_table_last_fs, iFpbp_)
GO(mnt_table_listmount_set_id, iFpU)
GO(mnt_table_listmount_set_ns, iFpU)
GO(mnt_table_listmount_set_stepsiz, iFpL)
GO(mnt_table_move_fs, iFppipp)
GO(mnt_table_next_child_fs, iFpppbp_)
GO(mnt_table_next_fs, iFppbp_)
GO(mnt_table_over_fs, iFppbp_)
GO(mnt_table_parse_dir, iFpp)
GO(mnt_table_parse_file, iFpp)
GO(mnt_table_parse_fstab, iFpp)
GO(mnt_table_parse_mtab, iFpp)
GO(mnt_table_parse_stream, iFpSp)
GO(mnt_table_parse_swaps, iFpp)
GO(mnt_table_refer_statmnt, iFpp)
GO(mnt_table_remove_fs, iFpp)
GO(mnt_table_replace_file, iFpp)
GO(mnt_table_set_cache, iFpp)
GO(mnt_table_set_intro_comment, iFpp)
GO(mnt_table_set_iter, iFppp)
//GOM(mnt_table_set_parser_errcb, iFEpp)
GO(mnt_table_set_trailing_comment, iFpp)
GO(mnt_table_set_userdata, iFpp)
GOM(mnt_table_uniq_fs, iFEpip)
GO(mnt_table_with_comments, iFp)
GO(mnt_table_write_file, iFpS)
GO(mnt_tag_is_valid, iFp)
GO(mnt_unlock_file, vFp)
GO(mnt_unmangle, pFp)
GO(mnt_unref_cache, vFp)
GO(mnt_unref_fs, vFp)
GO(mnt_unref_lock, vFp)
GO(mnt_unref_monitor, vFp)
GO(mnt_unref_statmnt, vFp)
GO(mnt_unref_table, vFp)
GO(mnt_update_force_rdonly, iFpi)
GO(mnt_update_get_filename, pFp)
GO(mnt_update_get_fs, pFp)
GO(mnt_update_get_mflags, LFp)
GO(mnt_update_is_ready, iFp)
GO(mnt_update_set_fs, iFpLpp)
GO(mnt_update_table, iFpp)
