#if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
#error Meh...
#endif

GO(ucal_add_68, vFpiip)
GO(ucal_clear_68, vFp)
GO(ucal_clearField_68, vFpi)
GO(ucal_clone_68, pFpp)
GO(ucal_close_68, vFp)
GO(ucal_countAvailable_68, iFv)
GO(ucal_equivalentTo_68, cFpp)
GO(ucal_get_68, iFpip)
GO(ucal_getAttribute_68, iFpi)
GO(ucal_getAvailable_68, pFi)
GO(ucal_getCanonicalTimeZoneID_68, iFpipipp)
GO(ucal_getDayOfWeekType_68, iFpip)
GO(ucal_getDefaultTimeZone_68, iFpip)
GO(ucal_getDSTSavings_68, iFpp)
GO(ucal_getFieldDifference_68, iFpdip)
GO(ucal_getGregorianChange_68, dFpp)
GO(ucal_getHostTimeZone_68, iFpip)
GO(ucal_getKeywordValuesForLocale_68, pFppCp)
GO(ucal_getLimit_68, iFpiip)
GO(ucal_getLocaleByType_68, pFpip)
GO(ucal_getMillis_68, dFpp)
GO(ucal_getNow_68, dFv)
GO(ucal_getTimeZoneDisplayName_68, iFpippip)
GO(ucal_getTimeZoneID_68, iFppip)
GO(ucal_getTimeZoneIDForWindowsID_68, iFpippip)
GO(ucal_getTimeZoneOffsetFromLocal_68, vFpiippp)
GO(ucal_getTimeZoneTransitionDate_68, cFpipp)
GO(ucal_getType_68, pFpp)
GO(ucal_getTZDataVersion_68, pFp)
GO(ucal_getWeekendTransition_68, iFpip)
GO(ucal_getWindowsTimeZoneID_68, iFpipip)
GO(ucal_inDaylightTime_68, cFpp)
GO(ucal_isSet_68, cFpi)
GO(ucal_isWeekend_68, cFpdp)
GO(ucal_open_68, pFpipip)
GO(ucal_openCountryTimeZones_68, pFpp)
GO(ucal_openTimeZoneIDEnumeration_68, pFippp)
GO(ucal_openTimeZones_68, pFp)
GO(ucal_roll_68, vFpiip)
GO(ucal_set_68, vFpii)
GO(ucal_setAttribute_68, vFpii)
GO(ucal_setDate_68, vFpiiip)
GO(ucal_setDateTime_68, vFpiiiiiip)
GO(ucal_setDefaultTimeZone_68, vFpp)
GO(ucal_setGregorianChange_68, vFpdp)
GO(ucal_setMillis_68, vFpdp)
GO(ucal_setTimeZone_68, vFppip)

GO(ucol_clone_68, pFpp)
GO(ucol_close_68, vFp)
GO(ucol_closeElements_68, vFp)
GO(ucol_setMaxVariable_68, vFpip)
GO(ucol_getOffset_68, iFp)
GO(ucol_getRules_68, pFpp)
GO(ucol_getSortKey_68, iFppipi)
GO(ucol_getStrength_68, iFp)
GO(ucol_getVersion_68, vFpp)
GO(ucol_next_68, iFpp)
GO(ucol_previous_68, iFpp)
GO(ucol_open_68, pFpp)
GO(ucol_openElements_68, pFppip)
GO(ucol_openRules_68, pFpiiipp)
GO(ucol_safeClone_68, pFpppp)
GO(ucol_setAttribute_68, vFpiip)
GO(ucol_setVariableTop_68, uFppip)
GO(ucol_strcoll_68, iFppipi)

GO(udat_close_68, vFp)
GO(udat_countSymbols_68, iFpi)
GO(udat_getSymbols_68, iFpiipip)
GO(udat_format_68, iFpdpipp)
GO(udat_open_68, pFiippipip)
GO(udat_setCalendar_68, vFpp)
GO(udat_toPattern_68, iFpCpip)

GO(unum_close_68, vFp)
GO(unum_getAttribute_68, iFpi)
GO(unum_getSymbol_68, iFpipip)
GO(unum_open_68, pFipippp)
GO(unum_toPattern_68, iFpCpip)

GO(udatpg_close_68, vFp)
GO(udatpg_getBestPattern_68, iFppipip)
GO(udatpg_open_68, pFpp)

GO(ulocdata_getCLDRVersion_68, vFpp)
GO(ulocdata_getMeasurementSystem_68, iFpp)

GO(usearch_close_68, vFp)
GO(usearch_first_68, iFpp)
GO(usearch_getBreakIterator_68, pFp)
GO(usearch_getMatchedLength_68, iFp)
GO(usearch_last_68, iFpp)
GO(usearch_openFromCollator_68, pFpipippp)
GO(usearch_setPattern_68, vFppip)
GO(usearch_setText_68, vFppip)
